Package: risk.assessr
Title: Assessing Package Risk Metrics
Version: 4.1.2
Authors@R: c(
    person("Edward", "Gillian", , "edward.gillian-ext@sanofi.com", role = c("cre", "aut"), comment = c(ORCID = "0000-0003-2732-5107")),
    person("Hugo", "Bottois", , "hugo.bottois@sanofi.com", role = "aut", comment = c(ORCID = "0000-0003-4674-0875")),
    person("Paulin", "Charliquart", , "paulin.charliquart@sanofi.com", role = "aut"),
    person("Andre", "Couturier", , "andre.couturier@sanofi.com", role = "aut"),
    person("Sanofi", role = c("cph", "fnd")))
Description: A reliable and validated tool that captures detailed risk metrics 
    such as R 'CMD' check, test coverage, traceability matrix, documentation, dependencies, 
    reverse dependencies, suggested dependency analysis, repository data, 
    and enhanced reporting for R packages that are local or stored 
    on remote repositories such as GitHub, CRAN, and Bioconductor.
License: GPL (>= 2)
URL: https://pharmaverse.github.io/risk.assessr/
BugReports: https://github.com/pharmaverse/risk.assessr/issues
Depends: R (>= 4.1.0)
Imports: remotes, test.assessr (>= 2.1.0), checkmate, desc, dplyr, fs,
        methods, purrr, rmarkdown, rcmdcheck, rlang, xml2, stringr (>=
        1.5.0), tidyr, utils, curl, jsonlite, memoise, BiocManager,
        glue
Suggests: devtools, forcats, here, htmltools, htmlwidgets, kableExtra
        (>= 1.4.0), knitr, magrittr, openxlsx, pkgload, R6, S7, readr,
        roxygen2, testthat (>= 3.0.0), tibble, tidyselect, tools,
        withr, mockery, ggplot2, DT
Encoding: UTF-8
Config/testthat/edition: 3
Config/build/clean-inst-doc: false
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-16 13:30:01 UTC; u1004798
Author: Edward Gillian [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-2732-5107>),
  Hugo Bottois [aut] (ORCID: <https://orcid.org/0000-0003-4674-0875>),
  Paulin Charliquart [aut],
  Andre Couturier [aut],
  Sanofi [cph, fnd]
Maintainer: Edward Gillian <edward.gillian-ext@sanofi.com>
Repository: CRAN
Date/Publication: 2026-07-16 14:00:02 UTC
Built: R 4.6.1; ; 2026-07-16 18:20:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(assess_pkg_r_package)
export(average_issue_close_time)
export(bioconductor_reverse_deps)
export(build_dependency_tree)
export(check_and_fetch_cran_package)
export(check_cran_package)
export(count_commits_last_months)
export(cran_packages)
export(cran_revdep)
export(create_traceability_matrix)
export(dependsOnPkgs)
export(detect_version_conflicts)
export(download_and_parse_dependencies)
export(extract_package_version)
export(extract_thresholds_by_id)
export(extract_thresholds_by_key)
export(extract_version)
export(fetch_all_dependencies)
export(fetch_bioconductor_package_info)
export(fetch_bioconductor_releases)
export(generate_html_report)
export(generate_traceability_matrix)
export(get_bioconductor_package_url)
export(get_commits_since)
export(get_cran_package_url)
export(get_cran_total_downloads)
export(get_exports)
export(get_frugal_metrics)
export(get_github_data)
export(get_host_package)
export(get_internal_package_url)
export(get_package_download_bioconductor)
export(get_package_download_cran)
export(get_package_tarfile)
export(get_pkg_name)
export(get_pubmed_by_year)
export(get_pubmed_count)
export(get_risk_analysis)
export(get_risk_definition)
export(get_security_vulnerabilities)
export(get_session_dependencies)
export(get_suggested_exp_funcs)
export(get_versions)
export(install_package_local)
export(is_base)
export(list_badges)
export(modify_description_file)
export(parse_bioconductor_releases)
export(parse_html_version)
export(parse_package_info)
export(print_tree)
export(risk_assess_pkg)
export(risk_assess_pkg_lock_files)
export(set_up_pkg)
export(write_summary_report)
import(curl)
import(dplyr)
import(purrr)
import(utils)
import(xml2)
importFrom(BiocManager,repositories)
importFrom(BiocManager,version)
importFrom(curl,curl)
importFrom(curl,curl_fetch_memory)
importFrom(curl,handle_setheaders)
importFrom(curl,handle_setopt)
importFrom(curl,new_handle)
importFrom(desc,desc_coerce_authors_at_r)
importFrom(desc,description)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(fs,dir_exists)
importFrom(fs,path_abs)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(memoise,memoise)
importFrom(memoise,timeout)
importFrom(purrr,pluck)
importFrom(remotes,download_version)
importFrom(rlang,"%||%")
importFrom(rmarkdown,html_document)
importFrom(rmarkdown,md_document)
importFrom(rmarkdown,render)
importFrom(stats,setNames)
importFrom(stringr,str_trim)
importFrom(tools,file_ext)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,sessionInfo)
importFrom(utils,tar)
importFrom(utils,untar)
importFrom(xml2,read_html)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_text)

# risk.assessr

<a><img src="man/figures/logo.png" align="right" height="138"/></a>

<!-- badges: start -->
![R-CMD-check](https://img.shields.io/badge/R%20CMD%20check-Passing-brightgreen.svg)
![Test Coverage](https://img.shields.io/badge/coverage-94%25-brightgreen.svg)
![Pharmaverse](http://pharmaverse.org/shields/risk.assessr.svg)
<!-- badges: end -->


## Overview

`risk.assessr` helps in the initial determining of a package's reliability and security in terms of maintenance, documentation, and dependencies. This package is designed to carry out a risk assessment of R packages at the beginning of the validation process (either internal or open source). It calculates risk metrics such as:

- **Core metrics** - includes R command check, unit test coverage and composite coverage of dependencies

- **Documentation metrics** - availability of vignettes, news tracking, example(s), check if functions have family documentation, return object description for exported functions, and type of license

- **Dependency Metrics** - package dependencies and reverse dependencies

- **Traceability matrix** - matching the function / test descriptions to tests and match to test pass/fail

## How it works

This package executes the following tasks:

1. Download the source package(`tar.gz` file)

2. Unpack the `tar.gz` file

3. Install the package locally

4. Run code coverage

5. Run a traceability matrix

6. Run R CMD check

7. Run risk assessment metrics using default or user defined weighting


## Installation

Install from GitHub:

```r
remotes::install_github("Sanofi-Public/risk.assessr")
```

Or from CRAN, when published:

```r
install.packages("risk.assessr")
```

## Usage

To assess your package, do the following steps:

1. Build your package as a `tar.gz` file

2. Set repository options

3. Run the following code sample by loading or add path parameter to your `tar.gz` package source code


``` r
options(repos = c(
  RSPM = "https://cloud.r-project.org",
  INTERNAL = "https://cloud.r-project.org",
  INTERNAL_RSPM = "https://cloud.r-project.org"
))

library(risk.assessr)

# using build package
# Local package source tarball (path or interactive file chooser)
results <- risk_assess_pkg("path/to/your/package.tar.gz")
results <- risk_assess_pkg()  # opens file chooser

# Package by name from CRAN/Bioconductor/internal
results <- risk_assess_pkg(package = "dplyr")
results <- risk_assess_pkg(package = "dplyr", version = "1.0.0")

# Lock file (renv.lock or pak.lock)
results <- risk_assess_pkg_lock_files("path/to/your/lockfile")
```

Note: This process can be very time-consuming and is recommended to be performed as a batch job or within a GitHub Action.



## Metrics and Risk assessment

| Key Metrics | Reason | where to find them in Metrics and Risk assessment | 
| -------------| ---------|----------------------|
| RCMD check  | series of 45 package checks of tests, package structure, documentation | `check` element in `results` list, check_list |
| test coverage | unit test coverage | `covr` element in `results` list, covr_list |
| risk analysis | rules and thresholds to identify risks | risk_analysis |
| traceability matrix| maps exported functions to test coverage, documentation by risk and function type| tm_list |


## Publication/presentation

1. **Conference:** Connect 2025  
   **Location:** Orlando, US  
   **Session ID:** OS17  
   **Title:** *Risk.assessr: A Tool for Assessing and Mitigating Risks with Open-Source R Packages in Clinical Trials*  
   **Presenters:** Andre Couturier, Edward Gillian  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier  
   **Company:** Sanofi  
   **Materials**  
   - [Presentation (PDF)](https://phuse.s3.eu-central-1.amazonaws.com/Archive/2025/Connect/US/Orlando/PRE_OS17.pdf)  
   - [Paper (PDF)](https://phuse.s3.eu-central-1.amazonaws.com/Archive/2025/Connect/US/Orlando/PAP_OS17.pdf)

2. **Conference:** PHUSE SDE 2025  
   **Location:** Beijing, China  
   **Title:** *CI/CD in R Package Development with Integrated Risk Assessment*  
   **Presenter:** Neo Yang  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier  
   **Company:** Sanofi  
   **Materials**  
   - [Presentation (PDF)](https://phuse.s3.eu-central-1.amazonaws.com/Archive/2025/SDE/APAC/Beijing/PRE_Beijing07.pdf)

3. **Conference:** EU Connect 2025  
   **Location:** Hamburg, Germany  
   **Session ID:** CT10  
   **Title:** *Risk.assessr: Extracting OOP Function Details*  
   **Presenter:** Edward Gillian  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier    
   **Company:** Sanofi  
   **Materials / Status:**  
   - *Ongoing*

4. **Conference:** R/Pharma 2025 APAC  
   **Location:** Online    
   **Session ID:** Ongoing   
   **Title:** *risk.assessr:  extending its use in the package validation process*   
   **Presenter:** Hugo Bottois  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier  
   **Company:** Sanofi  
   **Materials / Status:**    
   - *Ongoing*


## Citation

Gillian E, Bottois H, Charliquart P, Couturier A (2025). risk.assessr: Assessing Package Risk Metrics. R package version 2.0.0, <https://sanofi-public.github.io/risk.assessr/>.

```         
@Manual{,
  title = {risk.assessr: Assessing Package Risk Metrics},
  author = {Edward Gillian and Hugo Bottois and Paulin Charliquart and Andre Couturier},
  year = {2025},
  note = {R package version 2.0.0},
  url = {https://sanofi-public.github.io/risk.assessr/},
}
```

## Acknowledgements

The project is inspired by the [`riskmetric`](https://github.com/pharmaR/riskmetric) package and the [`mpn.scorecard`](https://github.com/metrumresearchgroup/mpn.scorecard) package and draws on some of their ideas and functions.

# risk.assessr

<a class="readme-logo"><img src="man/figures/logo.png" align="right" height="138" alt="risk.assessr logo"/></a>

<!-- badges: start -->
![R-CMD-check](https://img.shields.io/badge/R%20CMD%20check-Passing-brightgreen.svg)
![Test Coverage](https://img.shields.io/badge/coverage-94%25-brightgreen.svg)
![Pharmaverse](http://pharmaverse.org/shields/risk.assessr.svg)
<!-- badges: end -->


## Overview

`risk.assessr` helps in the initial determining of a package's reliability and security in terms of maintenance, documentation, and dependencies. This package is designed to carry out a risk assessment of R packages at the beginning of the validation process (either internal or open source). It calculates risk metrics such as:

- **Core metrics** - includes R command check, unit test coverage and composite coverage of dependencies

- **Documentation metrics** - availability of vignettes, news tracking, example(s), check if functions have family documentation, return object description for exported functions, and type of license

- **Dependency Metrics** - package dependencies and reverse dependencies

- **Traceability matrix** - matching the function / test descriptions to tests and match to test pass/fail

## How it works

This package executes the following tasks:

1. Download the source package(`tar.gz` file)

2. Unpack the `tar.gz` file

3. Install the package locally

4. Run code coverage

5. Run a traceability matrix

6. Run R CMD check

7. Run risk assessment metrics using default or user defined weighting


## Installation

From Sanofi repo:

```r
options(repos="https://cloud.r-project.org")
install.packages("risk.assessr")
```

Development version:

```r         
devtools::install_github("pharmaverse/risk.assessr")
```

## Usage

To assess your package, do the following steps:

1. Build your package as a `tar.gz` file

2. Set repository options

3. Run the following code sample by loading or add path parameter to your `tar.gz` package source code


``` r
options(repos="https://cloud.r-project.org")

library(risk.assessr)

# using build package
# Local package source tarball (path or interactive file chooser)
results <- risk_assess_pkg("path/to/your/package.tar.gz")
results <- risk_assess_pkg()  # opens file chooser

# Package by name from CRAN/Bioconductor/internal
results <- risk_assess_pkg(package = "dplyr")
results <- risk_assess_pkg(package = "dplyr", version = "1.0.0")

# Lock file (renv.lock or pak.lock)
results <- risk_assess_pkg_lock_files("path/to/your/lockfile")
```

Note: This process can be very time-consuming and is recommended to be performed as a batch job or within a GitHub Action.



## Metrics and Risk assessment

| Key Metrics | Reason | where to find them in Metrics and Risk assessment | 
| -------------| ---------|----------------------|
| RCMD check  | series of 45 package checks of tests, package structure, documentation | `check` element in `results` list, check_list |
| test coverage | unit test coverage | `covr` element in `results` list, covr_list |
| risk analysis | rules and thresholds to identify risks | risk_analysis |
| traceability matrix| maps exported functions to test coverage, documentation by risk and function type| tm_list |


## Publication/presentation

1. **Conference:** Connect 2025  
   **Location:** Orlando, US  
   **Session ID:** OS17  
   **Title:** *Risk.assessr: A Tool for Assessing and Mitigating Risks with Open-Source R Packages in Clinical Trials*  
   **Presenters:** Andre Couturier, Edward Gillian  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier  
   **Company:** Sanofi  
   **Materials**  
   - [Presentation (PDF)](https://phuse.s3.eu-central-1.amazonaws.com/Archive/2025/Connect/US/Orlando/PRE_OS17.pdf)  
   - [Paper (PDF)](https://phuse.s3.eu-central-1.amazonaws.com/Archive/2025/Connect/US/Orlando/PAP_OS17.pdf)

2. **Conference:** PHUSE SDE 2025  
   **Location:** Beijing, China  
   **Title:** *CI/CD in R Package Development with Integrated Risk Assessment*  
   **Presenter:** Neo Yang  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier  
   **Company:** Sanofi  
   **Materials**  
   - [Presentation (PDF)](https://phuse.s3.eu-central-1.amazonaws.com/Archive/2025/SDE/APAC/Beijing/PRE_Beijing07.pdf)

3. **Conference:** EU Connect 2025  
   **Location:** Hamburg, Germany  
   **Session ID:** CT10  
   **Title:** *Risk.assessr: Extracting OOP Function Details*  
   **Presenter:** Edward Gillian  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier    
   **Company:** Sanofi  

4. **Conference:** R/Pharma 2025 APAC  
   **Location:** Online    
   **Session ID:** Ongoing   
   **Title:** *risk.assessr:  extending its use in the package validation process*   
   **Presenter:** Hugo Bottois  
   **Authors:** Edward Gillian, Hugo Bottois, Paulin Charliquart, Andre Couturier  
   **Company:** Sanofi  

## Citation

Gillian E, Bottois H, Charliquart P, Couturier A (2025). risk.assessr: Assessing Package Risk Metrics. R package version 2.0.0, <https://probable-chainsaw-kgro2o7.pages.github.io/>.

```         
@Manual{,
  title = {risk.assessr: Assessing Package Risk Metrics},
  author = {Edward Gillian and Hugo Bottois and Paulin Charliquart and Andre Couturier},
  year = {2025},
  note = {R package version 2.0.0},
  url = {https://probable-chainsaw-kgro2o7.pages.github.io/},
}
```

## Acknowledgements

The project is inspired by the [`riskmetric`](https://github.com/pharmaR/riskmetric) package and the [`mpn.scorecard`](https://github.com/metrumresearchgroup/mpn.scorecard) package and draws on some of their ideas and functions.

# set up
repos <- "https://packagemanager.posit.co/cran/__linux__/jammy/latest"
install.packages('remotes', repos = repos)
remotes::install_local(repos = repos)
library(risk.assessr)

pkg <- commandArgs(trailingOnly = TRUE) |>
  jsonlite::fromJSON()

output_path <- Sys.getenv("OUTPUT_PATH", "output")

# create results dir
target_dir <- file.path(output_path, pkg$name, pkg$version)
dir.create(target_dir, recursive = TRUE, showWarnings = FALSE)

# assess
options(repos='https://cloud.r-project.org')
risk.assessr::assess_pkg_r_package(pkg$name, pkg$version) |>
  jsonlite::write_json(file.path(target_dir, "assessr.json"), force=TRUE, auto_unbox=TRUE)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# library(risk.assessr)
# options(repos = c(CRAN = "http://cran.us.r-project.org"))

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# result_default <- risk.assessr::assess_pkg_r_package("stringr")

## ----message=FALSE, eval=FALSE------------------------------------------------
# str(result_default$risk_analysis)

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# 
# strict_coverage_config <- list(
#   list(
#     label = "code coverage",
#     id = "code_coverage",
#     key = "code_coverage",
#     thresholds = list(
#       list(level = "high", max = 0.9999),
#       list(level = "low", max = NULL)
#     )
#   ),
#   list(
#     label = "popularity",
#     id = "popularity",
#     key = "last_month_download",
#     thresholds = list(
#       list(level = "high", max = 21200000),
#       list(level = "medium", max = 11200000),
#       list(level = "low", max = NULL)
#     )
#   )
# )
# 
# # Set the option
# options(risk.assessr.risk_definition = strict_coverage_config)
# result_strict <- risk.assessr::assess_pkg_r_package("stringr")
# 

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# str(result_strict$risk_analysis)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----api-key-example, eval = FALSE--------------------------------------------
# # Replace with your actual API key
# options(pubmed.api_key = "your_actual_ncbi_api_key")
# 

## ----setup--------------------------------------------------------------------
library(risk.assessr)
library(ggplot2)

package_name <- "limma"

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# pubmed_count <- get_pubmed_count(package_name)
# pubmed_count

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# citation_data <- get_pubmed_by_year(package_name, years_back = 20)
# citation_data

## ----message=FALSE, warning=FALSE, echo=FALSE, eval=FALSE---------------------
# ggplot(citation_data, aes(x = Year, y = Count)) +
#   geom_col(fill = "steelblue") +
#   labs(
#     title = "PubMed Articles per Year",
#     subtitle = paste("Search term:", package_name),
#     x = "Publication Year",
#     y = "Number of Articles"
#   ) +
#   theme_minimal()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

knitr::opts_chunk$set(fig.width = 12, fig.height = 5, out.width = "100%" ) 

## ----setup, include = FALSE---------------------------------------------------
library(risk.assessr)
library(ggplot2)
library(dplyr)

suppressMessages({
  library(kableExtra)
  library(knitr)
})

suppressWarnings({
  library(htmltools)
  library(htmlwidgets)
})

## ----cran-data, eval=FALSE----------------------------------------------------
# download_cran_df <- get_package_download_cran("admiral", years = 5)
# head(download_cran_df)

## ----echo=FALSE, cran-data-viz, eval=FALSE------------------------------------
# monthly_downloads <- download_cran_df %>%
#   mutate(month = format(date, "%Y-%m")) %>%
#   group_by(month) %>%
#   summarise(
#     downloads = sum(count),
#     .groups = "drop"
#   ) %>%
#   mutate(month = as.Date(paste0(month, "-01")))
# 
# head(monthly_downloads)

## ----eval=FALSE---------------------------------------------------------------
# version_infos <- check_and_fetch_cran_package("admiral")

## ----echo=FALSE, eval=FALSE---------------------------------------------------
# version_dates <- bind_rows(lapply(version_infos$all_versions, as.data.frame)) %>%
#   mutate(date = as.Date(date))
# 
# ggplot(monthly_downloads, aes(x = month, y = downloads)) +
#   geom_line(color = "#2C3E50") +
#   geom_point(color = "#E74C3C", size = 1) +
#   geom_vline(
#     data = version_dates,
#     aes(xintercept = as.numeric(date)),
#     color = "blue",
#     linetype = "dashed",
#     linewidth = 0.5
#   ) +
#   labs(
#     title = "Monthly Downloads of 'admiral' Package",
#     x = "Month",
#     y = "Number of Downloads"
#   ) +
#   theme_minimal(base_size = 14) +
#   theme(
#     plot.title = element_text(face = "bold", hjust = 0.5, margin = margin(b = 10, t = 20)),
#     axis.text.x = element_text(angle = 45, hjust = 1),
#     plot.margin = margin(t = 30, r = 20, b = 20, l = 20)
#   )

## ----github-meta, eval=FALSE--------------------------------------------------
# repo_info <- get_github_data("tidyverse", "ggplot2")
# repo_info

## ----github-data, eval=FALSE--------------------------------------------------
# df_commit <- get_commits_since("tidyverse", "ggplot2", years = 1)
# head(df_commit)

## ----echo=FALSE, github-data-viz, eval=FALSE----------------------------------
# df_commit <- df_commit %>%
#   arrange(week_start) %>%
#   mutate(cumulative_commits = cumsum(n_commits))
# 
# ggplot(df_commit, aes(x = week_start)) +
#   geom_col(aes(y = n_commits), fill = "#1ABC9C") +
#   geom_line(aes(y = cumulative_commits), color = "#34495E", linewidth = 1, linetype = "dashed") +
#   labs(
#     title = "Weekly Commits on GitHub (ggplot2)",
#     x = "Week",
#     y = "Number of Commits"
#   ) +
#   theme_minimal(base_size = 14) +
#   theme(
#     plot.title = element_text(face = "bold", hjust = 0.5),
#     axis.text.x = element_text(angle = 45, hjust = 1)
#   )

## -----------------------------------------------------------------------------
download_bio_df <- get_package_download_bioconductor("limma")
head(download_bio_df$all_data)

## ----echo=FALSE, message=FALSE, eval=FALSE------------------------------------
# html_content <- fetch_bioconductor_releases()
# release_data <- parse_bioconductor_releases(html_content)
# info <- get_bioconductor_package_url("limma", release_data = release_data)

## ----bioconductor-data, eval=FALSE--------------------------------------------
# download_bio_df <- get_package_download_bioconductor("DESeq2")
# head(download_bio_df$all_data)

## ----echo=FALSE, bioconductor-data-viz, eval=FALSE----------------------------
# ggplot(download_bio_df$all_data, aes(x = Date, y = Nb_of_downloads)) +
#   geom_line(color = "#9B59B6") +
#   geom_point(color = "#2980B9") +
#   labs(
#     title = "Bioconductor Downloads",
#     x = "Month",
#     y = "Number of Downloads"
#   ) +
#   theme_minimal(base_size = 14) +
#   theme(
#     plot.title = element_text(
#       face = "bold",
#       hjust = 0.5,
#       margin = margin(b = 10, t = 20)
#     ),
#     plot.margin = margin(t = 30, r = 20, b = 20, l = 20),
#     axis.text.x = element_text(angle = 45, hjust = 1)
#   )

trim_ws <- function(x, width = 160) {
  x <- gsub("^[[:space:]]+|[[:space:]]+$", "", x, perl = TRUE)
  if (nchar(x) > width) paste0(substr(x, 1, width - 3), "...") else x
}

build_patterns <- function() {
  list(
    list(label = "sanofi_term", regex = "sanofi"),
    list(
      label = "sanofi_email",
      regex = "(?i)\\b[\\w._%+-]+@sanofi\\.(?:com|net|org)\\b"
    ),
    list(
      label = "api_key_like",
      regex = "(?i)\\b(api[_ -]?key|secret|token|password|passwd|pwd|client[_ -]?secret)\\b\\s*[:=]\\s*(['\"][^'\"]+['\"]|[A-Za-z0-9_./+=-]{8,})"
    ),
    list(label = "aws_access_key_id", regex = "\\bAKIA[0-9A-Z]{16}\\b"),
    list(
      label = "aws_secret_key",
      regex = "(?i)aws[^\\n]{0,30}secret[^\\n]{0,30}key\\s*[:=]\\s*[A-Za-z0-9/+=]{40}"
    ),
    list(label = "github_token", regex = "\\bgh[oprsu]_[A-Za-z0-9]{36,}\\b"),
    list(
      label = "pem_private_key_block",
      regex = "-----BEGIN (?:RSA |EC |)PRIVATE KEY-----"
    )
  )
}

read_file_safely <- function(path, n_max_lines = Inf) {
  out <- tryCatch(
    readLines(path, warn = FALSE, encoding = "UTF-8"),
    error = function(e) character()
  )
  if (!length(out))
    out <- tryCatch(
      readLines(path, warn = FALSE),
      error = function(e) character()
    )
  if (is.finite(n_max_lines)) out <- out[seq_len(min(length(out), n_max_lines))]
  out
}

scan_file <- function(file, patterns) {
  lines <- read_file_safely(file)
  res <- list()
  if (!length(lines)) {
    return(res)
  }
  for (p in patterns) {
    m <- grepl(p$regex, lines, perl = TRUE, ignore.case = TRUE)
    if (any(m)) {
      idx <- which(m)
      for (i in idx) {
        mobj <- regexpr(p$regex, lines[i], perl = TRUE)
        snippet <- if (mobj[1] > 0)
          substr(
            lines[i],
            mobj[1],
            mobj[1] + attr(mobj, "match.length")[1] - 1
          ) else lines[i]
        res[[length(res) + 1]] <- list(
          file = file,
          line = i,
          type = p$label,
          snippet = trim_ws(snippet, 160),
          line_text = trim_ws(lines[i], 300)
        )
      }
    }
  }
  if (length(res) == 0) {
    return(data.frame(
      file = character(),
      line = integer(),
      type = character(),
      snippet = character(),
      line_text = character(),
      stringsAsFactors = FALSE
    ))
  }
  do.call("rbind", res) |>
    as.data.frame()
}


code_scanner <- function() {
  folders <- c("inst", "R", "vignettes", "tests", ".github")
  message("Start scan")
  message("Directories: ", paste(folders, collapse = ", "))

  files <- list.files(folders, full.names = TRUE, recursive = TRUE)
  message("Found ", length(files), " files to scan")

  if (!length(files)) {
    message("No files to scan")
    return(NULL)
  }

  patterns <- build_patterns()
  results <- files |>
    lapply(function(x) {
      data <- scan_file(x, patterns)
      message("Found ", nrow(data), " terms in ", x)
      data
    })
  df <- do.call("rbind", results)
  df[] <- lapply(df, as.character)
  write.csv(df, "scan-results.csv")
  message("Report created ", file.path(getwd(), "scan-results.csv"))
}

code_scanner()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(risk.assessr)

## ----check and fetch, eval=FALSE----------------------------------------------
# result_cran <- check_and_fetch_cran_package("admiral", "1.0.0")

## ----url, eval=FALSE----------------------------------------------------------
# result_cran$package_url

## ----last_version, eval=FALSE-------------------------------------------------
# result_cran$last_version

## ----all_versions, eval=FALSE-------------------------------------------------
# head(result_cran$all_versions, n = 2)

## ----version_history, echo=FALSE, eval=FALSE----------------------------------
# # Example: flatten result$all_versions
# version_history <- do.call(rbind, lapply(result_cran$all_versions, function(entry) {
#   data.frame(
#     Version = entry$version,
#     Date = as.Date(entry$date),
#     stringsAsFactors = FALSE
#   )
# }))
# 
# # Show nicely
# knitr::kable(version_history, caption = "CRAN Version History")

## ----bio_fetch, eval=FALSE----------------------------------------------------
# html_content <- fetch_bioconductor_releases()
# release_data <- parse_bioconductor_releases(html_content)

## ----bio_release, echo=FALSE, eval=FALSE--------------------------------------
# # Example: flatten result$all_versions
# release <- do.call(rbind, lapply(release_data, function(entry) {
#   data.frame(
#     Version = entry$release,
#     Date = entry$date,
#     software_packages = entry$software_packages,
#     r_version = entry$r_version,
#     stringsAsFactors = FALSE
#   )
# }))
# 
# # Show nicely
# knitr::kable(release, caption = "release Bioconductor versions")

## ----flowCore, eval=FALSE-----------------------------------------------------
# fetch_bioconductor_package_info("3.21", "flowCore")

## ----result_flowCore, eval=FALSE----------------------------------------------
# html_content <- fetch_bioconductor_releases()
# release_data <- parse_bioconductor_releases(html_content)
# result_bio <- get_bioconductor_package_url("flowCore", "2.18.0", release_data)

## ----bio_all_versions, eval=FALSE---------------------------------------------
# head(result_bio$all_versions, n=2)

## ----bio_last_version, eval=FALSE---------------------------------------------
# result_bio$last_version

## ----bio_package_versions, eval=FALSE-----------------------------------------
# result_bio$bioconductor_version_package

## ----bio_url, eval=FALSE------------------------------------------------------
# result_bio$url

## ----bio_archived, eval=FALSE-------------------------------------------------
# result_bio$archived

#' Write the risk assessment summary report
#'
#' Renders the package risk assessment report (`summary.Rmd`) to a chosen format
#' and writes it to disk. The output file path can be inferred or explicitly set.
#' You can specify the output format directly via `output_format` or let it be
#' inferred from the extension of `output_file`. If `output_dir` is not provided,
#' the current working directory is used.
#'
#' @section Behavior:
#' - If `output_dir` is `NULL`, it defaults to `getwd()` and a message is printed.
#' - If `output_file` is `NULL`, a file name is auto-generated as
#'   `"summary_report_<pkg>_<version>.html"` inside `output_dir`.
#' - If `output_file` points to an existing directory (or ends with a path separator),
#'   a file name is composed inside that directory as
#'   `"summary_report_<pkg>_<version>.html"`.
#' - If `output_file` is a relative path, it is resolved against `output_dir`.
#' - If `output_format` is provided, it determines the render format
#'   (and the `output_file` name is adjusted to match the format).
#' - If `output_format` is not provided, the render format is inferred from the
#'   extension of `output_file`. Unknown extensions trigger a warning and the
#'   extension is changed to `.html`.
#'
#' @param results A results object containing the risk assessment results. It is
#'   expected to include `results$results$pkg_name` and `results$results$pkg_version`,
#'   which are used to construct default file names and report parameters.
#' @param output_file `character(1)` or `NULL`. The output file path **or** a directory.
#'   - If `NULL`, a default name is generated:
#'     `"summary_report_<pkg_name>_<pkg_version>.html"` in `output_dir`.
#'   - If it is a directory, the default name is composed inside it.
#'   - If it is a relative path, it is resolved against `output_dir`.
#' @param output_format `character(1)` or `NULL`. Explicit output format to render:
#'   one of `"html"` or `"md"` (case-insensitive; do **not** include a leading dot).
#'   If `NULL`, the format is inferred from the extension of `output_file`.
#'   Unknown values trigger a warning and default to HTML.
#' @param output_dir `character(1)` or `NULL`. Base directory used when
#'   `output_file` is `NULL` or a relative path. If `NULL`, defaults to the
#'   current working directory (`getwd()`), with an informational message.
#'
#' @return (Invisibly) the absolute path to the written report file.
#'
#' @details
#' The function calls [rmarkdown::render()] on an internal template:
#' `system.file("report_templates", "summary.Rmd", package = "risk.assessr")`.
#'
#' The format mapping is:
#' - `"html"` -> [rmarkdown::html_document()]
#' - `"md"`   -> [rmarkdown::md_document()]
#'
#' If `output_format` is provided, it takes precedence for rendering even if
#' `output_file` has a different extension; in that case the filename is adjusted
#' to match the format. If `output_format` is not provided, the function infers the
#' format from `output_file`'s extension. For unknown extensions, it warns and
#' rewrites the extension to `.html`.
#'
#' @examples
#' \dontrun{
#' # generate assessment_results
#' results <- risk_assess_pkg()
#' 
#' # Basic usage: auto-name in the working directory (HTML)
#' write_summary_report(results)
#'
#' # Specify output directory (auto-name inside that directory)
#' write_summary_report(results, output_dir = "artifacts")
#'
#' # Provide an explicit filename (relative to output_dir)
#' write_summary_report(results, output_file = "reports/summary.html")
#'
#' # Provide a directory as output_file: auto-name inside that directory
#' write_summary_report(results, output_file = "reports/")
#'
#' # Render to Markdown explicitly
#' write_summary_report(results, output_format = "md")
#'
#' # Let the extension drive the format (md)
#' write_summary_report(results, output_file = "reports/summary.md")
#'
#' # Case-insensitive formats work; avoid leading dots (".md" is not valid)
#' write_summary_report(results, output_format = "MD")
#' }
#'
#' @importFrom rmarkdown render html_document md_document
#' @importFrom tools file_ext
#' @export
write_summary_report <- function(results, output_file = NULL, output_format = NULL, output_dir = NULL) {
  
  # set up tool version
  
  # Get the installed package path
  pkg_path <- find.package("risk.assessr")
  
  # extract the Version field
  tool_version <- tryCatch(
    get_pkg_desc(pkg_path, fields = "Version"),
    error = function(e) {
      stop(
        "Failed to retrieve package description for 'risk.assessr': ",
        conditionMessage(e),
        call. = FALSE
      )
    }
  )
  
  if (is.null(tool_version) || is.null(tool_version$Version)) {
    stop(
      "Package description for 'risk.assessr' does not contain a valid 'Version' field.",
      call. = FALSE
    )
  }
  
  
  
  params <- list(
    analysis = generate_risk_analysis(results),
    name = results$results$pkg_name,
    version = results$results$pkg_version,
    tool_version = tool_version$Version,
    data = results
  )
  
  # If output_dir is not set, use the current working directory
  if (is.null(output_dir)) {
    output_dir <- getwd()
    message(glue::glue("No output directory specified. Setting output to {output_dir}"))
  }
  
  # Decide target path
  if (is.null(output_file)) {
    # Choose default extension based on explicit output_format (html|md), else html
    default_ext <- {
      if (!is.null(output_format)) {
        fmt <- tolower(output_format)
        if (fmt %in% c("html", "md")) fmt else "html"
      } else {
        "html"
      }
    }
    
    # Default: write to output_dir, auto-named (CRAN-friendly)
    output_file <- fs::path_abs(
      glue::glue("summary_report_{params$name}_{params$version}.{default_ext}"),
      start = output_dir
    )
    message("Report will be written to: ", output_file)
  } else {
    # Caller supplied a filename or directory; we respect it
    if (dir.exists(output_file) || grepl("/$|\\\\$", output_file)) {
      # Treat as directory: compose filename inside it (default ext is html here; corrected later)
      output_file <- fs::path_abs(
        glue::glue("summary_report_{params$name}_{params$version}.html"),
        start = output_file
      )
    } else if (!fs::is_absolute_path(output_file)) {
      # Resolve relative paths against output_dir
      output_file <- fs::path_abs(output_file, start = output_dir)
    }
  }
  
  # Infer output format from either explicit parameter or extension
  final_ext <- tolower(tools::file_ext(output_file))
  output_format_obj <- if (!is.null(output_format)) {
    fmt <- tolower(output_format)
    
    fmt_obj <- switch(
      fmt,
      html = rmarkdown::html_document(),
      md   = rmarkdown::md_document(),
      {
        warning("Unknown 'output_format'; defaulting to HTML (only 'html' and 'md' are supported).")
        fmt <- "html"
        rmarkdown::html_document()
      }
    )
    
    # Ensure filename extension matches explicit output_format
    if (!identical(final_ext, fmt)) {
      output_file <- fs::path_ext_set(output_file, fmt)
      final_ext <- fmt
    }
    
    # For MD: strictly disallow HTML to avoid extra html artifacts
    if (identical(fmt, "md")) {
      knitr::opts_knit$set(always_allow_html = FALSE)
      knitr::opts_chunk$set(dev = "png", fig.ext = "png")
    }
    
    fmt_obj
  } else {
    # Original behavior: infer from the file extension (and fix unknowns to .html)
    fmt_obj <- switch(
      final_ext,
      html = rmarkdown::html_document(),
      md   = rmarkdown::md_document(),
      {
        warning("Unknown extension; defaulting to HTML (only '.html' and '.md' are supported).")
        output_file <- fs::path_ext_set(output_file, "html")
        final_ext <- "html"
        rmarkdown::html_document()
      }
    )
    
    # For MD: strictly disallow HTML
    if (identical(final_ext, "md")) {
      knitr::opts_knit$set(always_allow_html = FALSE)
      knitr::opts_chunk$set(dev = "png", fig.ext = "png")
    }
    
    fmt_obj
  }
  
  # Render in a single target directory to avoid extra artifacts near template
  render_dir  <- dirname(output_file)
  render_name <- basename(output_file)
  template_path <- system.file("report_templates", 
                               "summary.Rmd", 
                               package = "risk.assessr")
  
  do.call(rmarkdown::render, list(
    input              = template_path,
    output_format      = output_format_obj,
    output_file        = render_name,        # basename only
    output_dir         = render_dir,         # final output placed here
    envir              = new.env(),
    params             = params,
    intermediates_dir  = render_dir,         # intermediates here, not in template dir
    knit_root_dir      = render_dir,         # force chunk outputs under render_dir
    clean              = TRUE,               # remove intermediates after successful render
    quiet              = TRUE                # suppress knit/pandoc progress lines
  ))
  
  # Final, single message confirming the actual output file
  message("Output created: ", file.path(render_dir, render_name))
  
}

#' Generate Risk Analysis
#'
#' @description Generates the Risk Analysis table for the HTML report.
#'
#' @param assessment_results - input data containing risk analysis metrics
#'
#' @keywords internal
generate_risk_analysis <- function(assessment_results) {
  
  risk <- assessment_results$risk_analysis
  
  # create values for the risk analysis table
  # Ensure covr is numeric before rounding
  assessment_results$results$covr <-
    as.numeric(assessment_results$results$covr)
  assessment_results$results$covr <-
    round(assessment_results$results$covr, 2)
  
  check <- handle_null(assessment_results$results$check)
  
  check <- convert_number_to_percent(check)
  
  covr <- handle_null(assessment_results$results$covr)
  
  covr <- convert_number_to_percent(covr)
  
  license <- assessment_results$results$license_name
  
  example_score <- handle_null(assessment_results$results$has_examples$example_score)
  
  example_score <- convert_number_to_percent(example_score)
  
  has_docs_score <- handle_null(assessment_results$results$has_docs$has_docs_score)
  
  has_docs_score <- convert_number_to_percent(has_docs_score)
  
  has_ex_docs_score <- handle_null(assessment_results$results$has_ex_docs_score)
  
  has_ex_docs_score <- convert_number_to_percent(has_ex_docs_score) 
  
  # get reverse dependencies number
  rev_deps <- assessment_results$results$rev_deps
  
  # Convert the character vector to a data frame with one row per value
  rev_deps_df <- as.data.frame(rev_deps, stringsAsFactors = FALSE)
  
  rev_deps_no = nrow(rev_deps_df)
  
  # get total downloads
  total_download_n <- safe_value(
    assessment_results$results$download$total_download
  )
  
  # Helper to handle NULLs
  get_metric <- function(x, default = "Not assessed") {
    if (is.null(x)) default else x
  }
  
  risk_analysis_table <- data.frame(
    Metric = c(
      "CMD Check",
      "Code Coverage",
      "Dependencies Count",
      "License Risk",
      "URL Documentation Score",
      "Percentage of functions with examples",
      "Percentage of functions with documentation",
      "Combined Examples Documentation Score",
      "Reverse Dependencies Count",
      "Later Version Available",
      "Total Downloads"
    ),
    Risk_Level = c(
      get_metric(risk$cmd_check),
      get_metric(risk$code_coverage),
      get_metric(risk$dependencies_count),
      get_metric(risk$license),
      get_metric(risk$documentation_score),
      get_metric(risk$example_score,  default = " "),
      get_metric(risk$has_docs_score,  default = " "),
      get_metric(risk$has_ex_docs_score),
      get_metric(risk$reverse_dependencies_count),
      get_metric(risk$later_version),
      get_metric(risk$total_download)
    ),
    Risk_Value = c(
      check,
      covr,
      "",
      license,
      "",
      example_score,
      has_docs_score,
      has_ex_docs_score,
      rev_deps_no,
      "",
      total_download_n
    ),
    stringsAsFactors = FALSE
  )
  
  return(risk_analysis_table)
}

#' Evaluate Risk Highlighting for Rejection and Mitigation
#'
#' This function analyzes risk levels for key metrics ("CMD Check" and "Code Coverage")
#' and other metrics to determine whether a report should be highlighted for rejection
#' or mitigation based on predefined rules.
#'
#' @param risk_analysis_output A data frame containing at least two columns:
#'   \describe{
#'     \item{Metric}{Character vector of metric names (e.g., "CMD Check", "Code Coverage").}
#'     \item{Risk_Level}{Character vector of risk levels (e.g., "low", "medium", "high").}
#'   }
#'
#' @return status - 3 possible values - "Accepted", "Mitigation Needed", "Rejected"
#'   
#' @details
#' **Rejection Logic:**
#' - Reject if BOTH key metrics ("CMD Check" and "Code Coverage") are medium/high.
#' - OR if there are severe other risks (greater equal to 2 high OR greater equal to 3 medium)
#' - AND at least one key metric is not low.
#'
#' **Mitigation Logic:**
#' - Mitigate if only one key metric is medium/high OR other risks are significant
#'   (greater equal to 1 high OR greater equal to 2 medium), provided the report is not rejected.
#'
#' @keywords internal
evaluate_risk_highlighting <- function(risk_analysis_output) {
  
  # Extract key metrics
  cmd_risk <- tolower(risk_analysis_output$Risk_Level[
    risk_analysis_output$Metric == "CMD Check"
  ])
  coverage_risk <- tolower(risk_analysis_output$Risk_Level[
    risk_analysis_output$Metric == "Code Coverage"
  ])
  
  # Calculate other risks
  other_high_risks <- sum(
    tolower(risk_analysis_output$Risk_Level[
      !(risk_analysis_output$Metric %in% c("CMD Check", "Code Coverage"))
    ]) ==
      "high"
  )
  
  other_medium_risks <- sum(
    tolower(risk_analysis_output$Risk_Level[
      !(risk_analysis_output$Metric %in% c("CMD Check", "Code Coverage"))
    ]) ==
      "medium"
  )
  
  # Rejection logic
  highlight_rejected <- ((cmd_risk %in%
                            c("high", "medium") &&
                            coverage_risk %in% c("high", "medium")) ||
                           ((other_high_risks >= 2 || other_medium_risks >= 3) &&
                              (cmd_risk != "low" || coverage_risk != "low")))
  
  # Mitigation logic
  highlight_mitigation <- (((cmd_risk %in%
                               c("high", "medium") &&
                               coverage_risk == "low") ||
                              (coverage_risk %in% c("high", "medium") && cmd_risk == "low")) ||
                             other_high_risks >= 1 ||
                             other_medium_risks >= 2) &&
    !highlight_rejected
  
  
  # Single return at the end
  status <- if (highlight_rejected) {
    "Rejected"
  } else if (highlight_mitigation) {
    "Mitigation Needed"
  } else {
    "Approved"
  }
  
  return(status)
}

# Define a toy assessment_results object for risk analysis
toy_data_risk <- list(
  # Qualitative risk levels used for the "Risk_Level" column in the table
  risk_analysis = list(
    dependencies_count         = "low",
    later_version              = "low",
    code_coverage              = "low",
    total_download             = "medium",
    license                    = "low",
    reverse_dependencies_count = "low",
    documentation_score        = "low",
    example_score              = "medium",
    has_docs_score             = "low",
    has_ex_docs_score          = "low",
    cmd_check                  = "low"
  ),
  
  # Numeric/structural inputs used to compute/display Risk_Value column
  results = list(
    # Proportions in [0,1]; generate_risk_analysis() will coerce + round + convert to percent
    covr  = 0.00,     # code coverage proportion
    check = 0.65,     # CMD check pass rate proportion
    
    # License name
    license_name = "GPL-3",
    
    # Nested lists for examples/docs with scores as proportions
    has_examples = list(
      example_score = 0.75  # 75%
    ),
    has_docs = list(
      has_docs_score = 1.00  # 100%
    ),
    
    has_ex_docs_score = 0.875, # 87.5%
    
    # Reverse dependencies as a character vector
    rev_deps = c("pkgA", "pkgB"),
    
    # Downloads nested as list(total_download = <number>)
    download = list(
      total_download = 12345
    )
  )
)



# Test the generate_risk_analysis function
test_that("generate_risk_analysis works correctly", {
  result <- generate_risk_analysis(toy_data_risk)
  
  expect_equal(nrow(result), 11)
  expect_equal(result$Metric[1], "CMD Check")
  expect_equal(result$Risk_Level[1], "low")
  
  expect_equal(result$Metric[2], "Code Coverage")
  expect_equal(result$Risk_Level[2], "low")
  
  expect_equal(result$Metric[3], "Dependencies Count")
  expect_equal(result$Risk_Level[3], "low")
  
  expect_equal(result$Metric[4], "License Risk")
  expect_equal(result$Risk_Level[4], "low")
  
  expect_equal(result$Metric[5], "URL Documentation Score")
  expect_equal(result$Risk_Level[5], "low")
  
  expect_equal(result$Metric[6], "Percentage of functions with examples")
  expect_equal(result$Risk_Level[6], "medium")
  
  expect_equal(result$Metric[7], "Percentage of functions with documentation")
  expect_equal(result$Risk_Level[7], "low")
  
  expect_equal(result$Metric[8], "Combined Examples Documentation Score")
  expect_equal(result$Risk_Level[8], "low")
  
  expect_equal(result$Metric[9], "Reverse Dependencies Count")
  expect_equal(result$Risk_Level[9], "low")
  
  
  # Check Risk_Value column
  expect_equal(result$Risk_Value[1], "65%")       # check
  expect_equal(result$Risk_Value[2], "0%")        # covr
  expect_equal(result$Risk_Value[3], "")          # empty
  expect_equal(result$Risk_Value[4], "GPL-3")     # license_name
  expect_equal(result$Risk_Value[5], "")          # empty
  expect_equal(result$Risk_Value[6], "75%")           # rev_deps count
  expect_equal(result$Risk_Value[7], "100%")          # empty
  expect_equal(result$Risk_Value[8], "87.5%")       # Combined score
  expect_equal(result$Risk_Value[9], "2")       # total_download
  
})



test_that("Returns correct flags for low risk scenario", {
  mock_data <- data.frame(
    Metric = c("CMD Check", "Code Coverage", "Other1"),
    Risk_Level = c("low", "low", "low"),
    stringsAsFactors = FALSE
  )
  
  result <- evaluate_risk_highlighting(mock_data)
  
  expect_identical(result, "Approved")
})

test_that("Reject when both key metrics are high", {
  mock_data <- data.frame(
    Metric = c("CMD Check", "Code Coverage", "Other1"),
    Risk_Level = c("high", "high", "low"),
    stringsAsFactors = FALSE
  )
  
  result <- evaluate_risk_highlighting(mock_data)
  
  expect_identical(result, "Rejected")
})

test_that("Mitigation when one key metric is high and others are low", {
  mock_data <- data.frame(
    Metric = c("CMD Check", "Code Coverage", "Other1"),
    Risk_Level = c("high", "low", "low"),
    stringsAsFactors = FALSE
  )
  
  result <- evaluate_risk_highlighting(mock_data)
  
  expect_identical(result, "Mitigation Needed")
})

test_that("Reject when other risks are severe and key metrics not low", {
  mock_data <- data.frame(
    Metric = c("CMD Check", "Code Coverage", "Other1", "Other2", "Other3"),
    Risk_Level = c("medium", "medium", "high", "high", "medium"),
    stringsAsFactors = FALSE
  )
  
  result <- evaluate_risk_highlighting(mock_data)
  
  expect_identical(result, "Rejected")
})

test_that("Mitigation when other risks are moderate", {
  mock_data <- data.frame(
    Metric = c("CMD Check", "Code Coverage", "Other1", "Other2"),
    Risk_Level = c("low", "low", "medium", "medium"),
    stringsAsFactors = FALSE
  )
  
  result <- evaluate_risk_highlighting(mock_data)
  
  expect_identical(result, "Mitigation Needed")
})


test_that("write_summary_report retrieves tool version successfully", {
  results <- fake_results
  
  mock_render <- function(...) invisible(NULL)
  
  mock_analysis <- function(...) "analysis"
  
  # Stub dependencies
  mockery::stub(
    write_summary_report,
    "find.package",
    function(...) "/fake/pkg/path"
  )
  
  mockery::stub(
    write_summary_report,
    "get_pkg_desc",
    function(path, fields) list(Version = "9.9.9")
  )
  
  mockery::stub(
    write_summary_report,
    "generate_risk_analysis",
    mock_analysis
  )
  
  mockery::stub(
    write_summary_report,
    "rmarkdown::render",
    mock_render
  )
  
  
  expect_message(
    write_summary_report(
      results = results,
      output_dir = tempdir()
    ),
    "Report will be written to:"
  )
  
  expect_message(
    write_summary_report(
      results = results,
      output_dir = tempdir()
    ),
    "Output created:"
  )
  
})


test_that("write_summary_report errors when get_pkg_desc fails", {
  results <- fake_results
  
  mockery::stub(
    write_summary_report,
    "find.package",
    function(...) "/fake/pkg/path"
  )
  
  mockery::stub(
    write_summary_report,
    "get_pkg_desc",
    function(...) stop("boom")
  )
  
  expect_error(
    write_summary_report(results),
    "Failed to retrieve package description for 'risk.assessr': boom"
  )
})


test_that("write_summary_report errors when pkg description is NULL", {
  results <- fake_results
  
  mockery::stub(
    write_summary_report,
    "find.package",
    function(...) "/fake/pkg/path"
  )
  
  mockery::stub(
    write_summary_report,
    "get_pkg_desc",
    function(...) NULL
  )
  
  expect_error(
    write_summary_report(results),
    "does not contain a valid 'Version' field"
  )
})


test_that("write_summary_report errors when Version field is missing", {
  results <- fake_results
  
  mockery::stub(
    write_summary_report,
    "find.package",
    function(...) "/fake/pkg/path"
  )
  
  mockery::stub(
    write_summary_report,
    "get_pkg_desc",
    function(...) list(Version = NULL)
  )
  
  expect_error(
    write_summary_report(results),
    "does not contain a valid 'Version' field"
  )
})


test_that("write_summary_report uses HTML format and resolves relative output path", {
  # Arrange
  output_file <- "my-report.html"
  
  # Stub system.file to point to a mock template
  mockery::stub(write_summary_report, "system.file", "mock_template.Rmd")
  
  # Stub generate_risk_analysis to return a known object
  mock_analysis <- data.frame(Metric = "Example", Risk_Level = "low", Risk_Value = 1)
  mockery::stub(write_summary_report, "generate_risk_analysis", mock_analysis)
  
  # Stub output format constructors to return sentinel values (only html/md now)
  mockery::stub(write_summary_report, "rmarkdown::html_document", "HTML_FORMAT")
  mockery::stub(write_summary_report, "rmarkdown::md_document",   "MD_FORMAT")
  
  # Fix the working directory so relative path resolution is deterministic
  mockery::stub(write_summary_report, "getwd", "/tmp")
  
  
  # Stub get_pkg_desc to return a fake version
  mockery::stub(write_summary_report, "get_pkg_desc", list(Version = "3.0.1"))
  
  
  # Capture the args actually passed to rmarkdown::render and return a known path
  captured <- NULL
  mockery::stub(write_summary_report, "rmarkdown::render",
                function(input,
                         output_format,
                         output_file,
                         output_dir,
                         envir,
                         params,
                         intermediates_dir,
                         knit_root_dir,
                         clean,
                         quiet) {
                  captured <<- list(
                    input             = input,
                    output_format     = output_format,
                    output_file       = output_file,       # basename
                    output_dir        = output_dir,        # directory
                    envir             = envir,
                    params            = params,
                    intermediates_dir = intermediates_dir,
                    knit_root_dir     = knit_root_dir,
                    clean             = clean,
                    quiet             = quiet
                  )
                  # Simulate rmarkdown returning a path (the function ignores this and returns its own)
                  return(file.path(output_dir, output_file))
                }
  )
  
  # Act
  result <- write_summary_report(results = fake_results, output_file = output_file)
  
  # Assert: function returns the final absolute path in /tmp
  # expect_equal(result, "/tmp/my-report.html")
  
  # rmarkdown::render is called with our mocked template
  expect_equal(captured$input, "mock_template.Rmd")
  
  # Format chosen by ".html" extension
  expect_equal(captured$output_format, "HTML_FORMAT")
  
  # Since the function now passes basename + output_dir separately:
  expect_equal(captured$output_file, "my-report.html")
  expect_equal(captured$output_dir, "/tmp")
  
  # Intermediates and knit root are set to the same output_dir
  expect_equal(captured$intermediates_dir, "/tmp")
  expect_equal(captured$knit_root_dir, "/tmp")
  
  # Clean and quiet flags are enabled
  expect_true(isTRUE(captured$clean))
  expect_true(isTRUE(captured$quiet))
  
  # Params checks
  expect_equal(captured$params$name, "mockpkg")
  expect_equal(captured$params$version, "0.1.0")
  expect_equal(captured$params$tool_version, "3.0.1")
  expect_identical(captured$params$data, fake_results)
  expect_identical(captured$params$analysis, mock_analysis)
  
  # Environment is a fresh environment
  expect_true(is.environment(captured$envir))
  # Should not be the global environment
  expect_false(identical(captured$envir, globalenv()))
  # Typically new.env() is empty
  expect_length(ls(envir = captured$envir, all.names = TRUE), 0)
})  

test_that("when output_file is NULL and output_format is NULL, default extension is .html and path is auto-named", {
  # Work with the SAME function object you'll call
  wsr <- risk.assessr::write_summary_report
  
  # ---- Critical: stub out risk analysis so we don't blow up in params construction
  mockery::stub(wsr, "generate_risk_analysis", function(results) list(ok = TRUE))
  
  # Stubs for pathing and rendering
  mockery::stub(wsr, "getwd", function() "/tmp/render_root")
  
  mockery::stub(wsr, "fs::path_abs", function(path, start = ".") file.path(start, path))
  
  mockery::stub(wsr, "system.file", function(..., package) "/fake/pkg/report_templates/summary.Rmd")
  
  mockery::stub(wsr, "rmarkdown::html_document", function(...) "html_document")
  mockery::stub(wsr, "rmarkdown::md_document", function(...) "md_document")
  
  render_calls <- list()
  mockery::stub(wsr, "rmarkdown::render", function(input, output_format, output_file,
                                          output_dir, envir, params,
                                          intermediates_dir, knit_root_dir,
                                          clean, quiet) {
    render_calls <<- append(render_calls, list(list(
      input = input,
      output_format = output_format,
      output_file = output_file,
      output_dir = output_dir,
      params = params,
      intermediates_dir = intermediates_dir,
      knit_root_dir = knit_root_dir,
      clean = clean,
      quiet = quiet
    )))
    "ok"
  })
  
  # Expect the message that reports the final output path (there will also be a message for output_dir)
  expect_message(
    wsr(fake_results, output_file = NULL, output_format = NULL, output_dir = NULL),
    regexp = "Report will be written to: /tmp/render_root/summary_report_mockpkg_0.1.0\\.html"
  )
  
  # Validate the render call arguments
  expect_length(render_calls, 1)
  call <- render_calls[[1]]
  
  expect_equal(call$input, "/fake/pkg/report_templates/summary.Rmd")
  expect_equal(call$output_format, "html_document")
  expect_equal(call$output_dir, "/tmp/render_root")
  expect_equal(call$output_file, "summary_report_mockpkg_0.1.0.html")  # basename only
  expect_equal(call$intermediates_dir, "/tmp/render_root")
  expect_equal(call$knit_root_dir, "/tmp/render_root")
  expect_true(call$clean)
  expect_true(call$quiet)
  expect_equal(call$params$name, "mockpkg")
  expect_equal(call$params$version, "0.1.0")
})




test_that("when output_file is NULL and output_format='md', default extension is .md", {
  wsr <- risk.assessr::write_summary_report
  
  mockery::stub(wsr, "generate_risk_analysis", function(results) list(ok = TRUE))
  
  mockery::stub(wsr, "getwd", function() "/tmp/render_root_md")
  mockery::stub(wsr, "fs::path_abs", function(path, start = ".") file.path(start, path))
  mockery::stub(wsr, "system.file", function(..., package) "/fake/pkg/report_templates/summary.Rmd")
  
  mockery::stub(wsr, "rmarkdown::html_document", function(...) "html_document")
  mockery::stub(wsr, "rmarkdown::md_document", function(...) "md_document")
  
  # --- Key workaround: skip the MD-only knitr calls by forcing identical("md","md") -> FALSE
  mockery::stub(wsr, "identical", function(a, b) {
    if (is.character(a) && is.character(b) && a == "md" && b == "md") {
      FALSE
    } else {
      base::identical(a, b)
    }
  })
  
  render_calls <- list()
  mockery::stub(wsr, "rmarkdown::render", function(input, output_format, output_file,
                                                   output_dir, envir, params,
                                                   intermediates_dir, knit_root_dir,
                                                   clean, quiet) {
    render_calls <<- append(render_calls, list(list(
      input = input,
      output_format = output_format,
      output_file = output_file,
      output_dir = output_dir
    )))
    "ok"
  })
  
  # You will see the "No output directory specified..." message first, that's fine.
  expect_message(
    wsr(fake_results, output_file = NULL, output_format = "md", output_dir = NULL),
    regexp = "Report will be written to: /tmp/render_root_md/summary_report_mockpkg_0.1.0\\.md"
  )
  
  expect_length(render_calls, 1)
  call <- render_calls[[1]]
  expect_equal(call$output_format, "md_document")
  expect_equal(call$output_dir, "/tmp/render_root_md")
  expect_equal(call$output_file, "summary_report_mockpkg_0.1.0.md")
})



test_that("when output_file is a directory, it composes filename inside it with .html", {
  wsr <- risk.assessr::write_summary_report
  
  mockery::stub(wsr, "generate_risk_analysis", function(results) list(ok = TRUE))
  
  mockery::stub(wsr, "fs::path_abs", function(path, start = ".") file.path(start, path))
  mockery::stub(wsr, "system.file", function(..., package) "/fake/pkg/report_templates/summary.Rmd")
  mockery::stub(wsr, "rmarkdown::html_document", function(...) "html_document")
  mockery::stub(wsr, "rmarkdown::md_document", function(...) "md_document")
  
  # Pretend the user passed an existing directory
  mockery::stub(wsr, "dir.exists", function(path) TRUE)
  
  render_calls <- list()
  mockery::stub(wsr, "rmarkdown::render", function(input, output_format, output_file,
                                          output_dir, envir, params,
                                          intermediates_dir, knit_root_dir,
                                          clean, quiet) {
    render_calls <<- append(render_calls, list(list(
      input = input,
      output_format = output_format,
      output_file = output_file,
      output_dir = output_dir
    )))
    "ok"
  })
  
  # Expect the final confirmation message (function emits "Output created: ...")
  expect_message(
    wsr(fake_results, output_file = "/reports", output_format = NULL, output_dir = "/ignored"),
    regexp = "Output created: /reports/summary_report_mockpkg_0.1.0\\.html"
  )
  
  expect_length(render_calls, 1)
  call <- render_calls[[1]]
  expect_equal(call$output_format, "html_document")
  expect_equal(call$output_dir, "/reports")
  expect_equal(call$output_file, "summary_report_mockpkg_0.1.0.html")
})


# Always call the package function the same way
wsr <- getFromNamespace("write_summary_report", "risk.assessr")

# Helper: fake results structure required by the function
make_results <- function(name = "pkgA", version = "0.1.0") {
  list(results = list(pkg_name = name, pkg_version = version))
}

# ---- Namespace mocking helpers ------------------------------------------------

# Safely mock one or more bindings INSIDE a package namespace, and restore them on exit
local_mock_ns <- function(pkg, bindings, .env = parent.frame()) {
  ns <- asNamespace(pkg)
  
  # snapshot originals
  old <- lapply(names(bindings), function(nm) get(nm, envir = ns))
  names(old) <- names(bindings)
  
  # replace
  for (nm in names(bindings)) {
    if (bindingIsLocked(nm, ns)) unlockBinding(nm, ns)
    assign(nm, bindings[[nm]], envir = ns)
    lockBinding(nm, ns)
  }
  
  # restore automatically
  withr::defer({
    for (nm in names(old)) {
      if (bindingIsLocked(nm, ns)) unlockBinding(nm, ns)
      assign(nm, old[[nm]], envir = ns)
      lockBinding(nm, ns)
    }
  }, envir = .env)
}

# Minimal, deterministic fs helpers for cross-platform assertions
fs_path_abs <- function(path, start = getwd()) {
  file.path(start, path, fsep = .Platform$file.sep)
}
fs_path_ext_set <- function(path, ext) {
  # replace last extension if present, else append
  if (grepl("\\.[^.]+$", path)) {
    sub("\\.[^.]*$", paste0(".", ext), path)
  } else {
    paste0(path, ".", ext)
  }
}

# ---- Common mocks applied per-test where needed --------------------------------

mock_generate_risk_analysis <- function(...) "ANALYSIS"
mock_html_document <- function(...) "html_fmt"
mock_md_document   <- function(...) "md_fmt"

# Will collect render call arguments
render_calls <- NULL
mock_render <- function(...) {
  # stash args for later assertions
  render_calls <<- append(render_calls %||% list(), list(list(...)))
  NULL
}

`%||%` <- function(x, y) if (is.null(x)) y else x

# ------------------------------------------------------------------------------
# TESTS
# ------------------------------------------------------------------------------

test_that("uses getwd when output_dir is NULL and defaults to .html when format is not provided", {
  fake_results <- make_results("pkgA", "0.1.0")
  
  # Use a temp directory as working dir so getwd() is deterministic
  tmp <- withr::local_tempdir()
  withr::local_dir(tmp)
  
  # Mock the risky and external pieces at NAMESPACE level
  local_mock_ns("risk.assessr", list(
    generate_risk_analysis = mock_generate_risk_analysis,
    # keep get_pkg_desc simple & deterministic
    get_pkg_desc          = function(...) list(Version = "9.9.9")
  ))
  local_mock_ns("rmarkdown", list(
    html_document = mock_html_document,
    md_document   = mock_md_document,
    render        = mock_render
  ))
  # Normalize fs behavior for stable paths
  local_mock_ns("fs", list(
    path_abs       = fs_path_abs,
    path_ext_set   = fs_path_ext_set,
    is_absolute_path = function(path) TRUE
  ))
  # Make system.file deterministic
  local_mock_ns("base", list(
    system.file = function(...) "TEMPLATE.Rmd"
  ))
  
  # Reset captured calls holder
  render_calls <<- NULL
  
  # Assert messages and run
  expect_message(
    wsr(fake_results, output_file = NULL, output_format = NULL, output_dir = NULL),
    "No output directory specified. Setting output to",
    fixed = TRUE
  )
  
  # Render was called exactly once
  expect_length(render_calls, 1)
  args <- render_calls[[1]]
  
  # Output directory is the working dir we set
  expect_identical(
    fs::path_real(args$output_dir),
    fs::path_real(tmp)
  )
  
  
  # The auto-name should be summary_report_pkgA_0.1.0.html
  expect_equal(args$output_file, "summary_report_pkgA_0.1.0.html")
  expect_equal(args$output_format, "html_fmt")
})

test_that("explicit output_format = 'md' yields .md file and sets md_document()", {
  fake_results <- make_results("pkgB", "0.2.0")
  out_dir <- file.path("x", "out")
  
  # Working dir irrelevant here, but we normalize fs anyway
  local_mock_ns("risk.assessr", list(
    generate_risk_analysis = mock_generate_risk_analysis,
    get_pkg_desc          = function(...) list(Version = "9.9.9")
  ))
  local_mock_ns("rmarkdown", list(
    html_document = mock_html_document,
    md_document   = mock_md_document,
    render        = mock_render
  ))
  local_mock_ns("fs", list(
    path_abs         = fs_path_abs,
    path_ext_set     = fs_path_ext_set,
    is_absolute_path = function(path) TRUE
  ))
  local_mock_ns("base", list(
    system.file = function(...) "TEMPLATE.Rmd"
  ))
  
  render_calls <<- NULL
  
  wsr(fake_results, output_file = NULL, output_format = "md", output_dir = out_dir)
  
  expect_length(render_calls, 1)
  args <- render_calls[[1]]
  expect_equal(args$output_dir, out_dir)
  expect_equal(args$output_file, "summary_report_pkgB_0.2.0.md")
  expect_equal(args$output_format, "md_fmt")
})

test_that("treats output_file as directory if it exists and uses default .html then inferred", {
  fake_results <- make_results("pkgC", "0.3.0")
  out_dir_as_file <- "reports"
  
  local_mock_ns("risk.assessr", list(
    generate_risk_analysis = mock_generate_risk_analysis,
    get_pkg_desc          = function(...) list(Version = "9.9.9")
  ))
  local_mock_ns("rmarkdown", list(
    html_document = mock_html_document,
    md_document   = mock_md_document,
    render        = mock_render
  ))
  local_mock_ns("fs", list(
    path_abs         = fs_path_abs,
    path_ext_set     = fs_path_ext_set,
    is_absolute_path = function(path) TRUE
  ))
  local_mock_ns("base", list(
    system.file = function(...) "TEMPLATE.Rmd",
    dir.exists  = function(path) identical(path, out_dir_as_file) # treat as directory
  ))
  
  render_calls <<- NULL
  
  wsr(fake_results, output_file = out_dir_as_file, output_format = NULL, output_dir = NULL)
  
  expect_length(render_calls, 1)
  args <- render_calls[[1]]
  
  expect_equal(file.path(args$output_dir, args$output_file),
               file.path(out_dir_as_file, "summary_report_pkgC_0.3.0.html"))
  expect_equal(args$output_format, "html_fmt")
})

test_that("resolves relative output_file against output_dir when not absolute", {
  fake_results <- make_results("pkgD", "0.4.0")
  base_dir <- "base"
  rel_path <- file.path("out", "report.html")
  
  local_mock_ns("risk.assessr", list(
    generate_risk_analysis = mock_generate_risk_analysis,
    get_pkg_desc          = function(...) list(Version = "9.9.9")
  ))
  local_mock_ns("rmarkdown", list(
    html_document = mock_html_document,
    md_document   = mock_md_document,
    render        = mock_render
  ))
  local_mock_ns("fs", list(
    path_abs         = fs_path_abs,
    path_ext_set     = fs_path_ext_set,
    is_absolute_path = function(path) FALSE
  ))
  local_mock_ns("base", list(
    system.file = function(...) "TEMPLATE.Rmd",
    dir.exists  = function(path) FALSE
  ))
  
  render_calls <<- NULL
  
  wsr(fake_results, output_file = rel_path, output_format = NULL, output_dir = base_dir)
  
  expect_length(render_calls, 1)
  args <- render_calls[[1]]
  
  expect_equal(args$output_dir, file.path(base_dir, "out"))
  expect_equal(args$output_file, "report.html")
  expect_equal(args$output_format, "html_fmt")
})

test_that("explicit format overrides extension; unknown explicit format warns and defaults to html", {
  fake_results <- make_results("pkgE", "1.0.0")
  base_dir <- "base"
  
  # Common mocks
  local_mock_ns("risk.assessr", list(
    generate_risk_analysis = mock_generate_risk_analysis,
    get_pkg_desc          = function(...) list(Version = "9.9.9")
  ))
  local_mock_ns("rmarkdown", list(
    html_document = function(...) "html_fmt_A",
    md_document   = function(...) "md_fmt_A",
    render        = mock_render
  ))
  local_mock_ns("fs", list(
    path_abs         = fs_path_abs,
    path_ext_set     = fs_path_ext_set,
    is_absolute_path = function(path) FALSE
  ))
  local_mock_ns("base", list(
    system.file = function(...) "TEMPLATE.Rmd",
    dir.exists  = function(path) FALSE
  ))
  
  # --- Case A: explicit 'md' overrides a .html filename to .md ---
  render_calls <<- NULL
  wsr(fake_results,
      output_file   = "report.html",
      output_format = "md",
      output_dir    = base_dir)
  
  expect_length(render_calls, 1)
  args_A <- render_calls[[1]]
  expect_equal(args_A$output_dir, base_dir)
  expect_equal(args_A$output_file, "report.md")
  expect_equal(args_A$output_format, "md_fmt_A")
  
  # --- Case B: explicit unknown format warns -> fallback to HTML and correct extension ---
  # Refresh rmarkdown mock to distinguish return for HTML
  local_mock_ns("rmarkdown", list(
    html_document = function(...) "html_fmt_B",
    md_document   = function(...) "md_fmt_B",
    render        = mock_render
  ))
  
  render_calls <<- NULL
  
  expect_warning(
    wsr(fake_results,
        output_file   = "something.md",
        output_format = "pdf",  # unknown
        output_dir    = base_dir),
    "Unknown 'output_format'; defaulting to HTML",
    fixed = TRUE
  )
  
  expect_length(render_calls, 1)
  args_B <- render_calls[[1]]
  expect_equal(args_B$output_dir, base_dir)
  expect_equal(args_B$output_file, "something.html") # corrected extension
  expect_equal(args_B$output_format, "html_fmt_B")
})

#' Check if a Package is a Base or Recommended R Package
#' 
#' This function determines whether a given package is a base or recommended package
#' using CRAN metadata.
#'
#' @param pkg A character string representing the name of the package.
#' 
#' @return A logical value: `TRUE` if the package is a base or recommended package, `FALSE` otherwise.
#' 
#' @examples
#' \dontrun{
#' is_base("stats")     # TRUE
#' is_base("ggplot2")   # FALSE
#' }
#' @export
is_base <- function(pkg) {
  base_packages <- rownames(installed.packages(priority = "base"))
  return(pkg %in% base_packages)
}

#' Parse Dependencies from a Package DESCRIPTION File
#' 
#' This function extracts and returns the dependencies from the DESCRIPTION file
#' of an R package, focusing on the `Imports` field.
#'
#' @param path A character string specifying the path to the package directory containing the DESCRIPTION file.
#' 
#' @return A data frame with columns:
#' 	- `package`: The name of the imported package.
#' 	- `type`: The type of dependency (e.g., "Imports").
#' 
#' @examples
#' 
#' \dontrun{
#' parse_dcf_dependencies_version("/path/to/package")
#' }
#' @keywords internal
parse_dcf_dependencies_version <- function(path) {
  dcf <- read.dcf(file.path(path, "DESCRIPTION"))
  
  if (!"Imports" %in% colnames(dcf)) {
    return(data.frame(package = character(), type = character(), stringsAsFactors = FALSE))
  }
  
  imports <- dcf[1, "Imports"]
  split_deps <- unlist(strsplit(imports, ","))
  split_deps <- trimws(split_deps)
  
  deps <- data.frame(
    package = split_deps,
    type = rep("Imports", length(split_deps)),
    stringsAsFactors = FALSE,
    row.names = NULL 
  )
  
  return(deps)
}

#' Extract License from Package DESCRIPTION File
#' 
#' This function extracts the license information from a package's DESCRIPTION file.
#'
#' @param path A character string specifying the path to the package directory containing the DESCRIPTION file.
#' 
#' @return A character string representing the package license, or `NA` if not found.
#' 
#' @importFrom desc description
#' @keywords internal
extract_license_from_description <- function(path) {
  desc_file <- file.path(path, "DESCRIPTION")
  
  License <- NA_character_
  
  if (!file.exists(desc_file)) {
    warning("DESCRIPTION file not found: ", desc_file)
  } else {
    tryCatch({
      d <- description$new(file = desc_file)
      
      if (!d$has_fields("License")) {
        warning("No License field in DESCRIPTION: ", desc_file)
      } else {
        License_list <- d$get_list("License")
        
        if (is.null(License_list) || length(License_list) == 0) {
          warning("License field is empty in DESCRIPTION: ", desc_file)
        } else {
          License <- as.character(License_list)
        }
      }
    }, error = function(e) {
      warning("Failed to extract License from DESCRIPTION: ", conditionMessage(e))
      License <- NA_character_
    })
  }
  
  return(License)
}




#' Download and Parse Dependencies of an R Package
#' 
#' Downloads a package from a repository, extracts it, and parses dependencies from the DESCRIPTION file.
#'
#' @param package_name Name of the package to download.
#' @param version Package version to download (default: `NA` for latest).
#' @param get_license Whether to extract license information (default: `FALSE`).
#' 
#' @return When `get_license = FALSE`, a data frame with columns `package`, `type`, and `parent_package`.
#'   When `get_license = TRUE`, a list with:
#'     - `dependencies`: Data frame of direct dependencies.
#'     - `license`: License of the specified package only (not dependencies). Returns `NA` if not found.
#' 
#' @details The `license` field refers only to the package specified by `package_name`, not its dependencies.
#'   Use `build_dependency_tree()` or `fetch_all_dependencies()` with `get_license = TRUE` for full license information.
#' 
#' @import dplyr
#' @examples
#' \dontrun{
#' download_and_parse_dependencies("dplyr")
#' download_and_parse_dependencies("dplyr", get_license = TRUE)
#' }
#' @export
download_and_parse_dependencies <- function(package_name, version = NA, get_license = FALSE) {
  
  tryCatch({
    temp_file <- remotes::download_version(package = package_name, version = version)
    
    temp_dir <- tempfile()
    dir.create(temp_dir)
    untar(temp_file, exdir = temp_dir)
    
    extracted_dirs <- list.dirs(temp_dir, full.names = TRUE, recursive = FALSE)
    
    package_dir <- if (length(extracted_dirs) == 1) {
      extracted_dirs[1]
    } else {
      file.path(temp_dir, package_name)
    }
    
    desc_path <- file.path(package_dir, "DESCRIPTION")
    
    if (!file.exists(desc_path)) {
      desc_files <- list.files(package_dir, pattern = "DESCRIPTION", recursive = TRUE, full.names = TRUE)
      if (length(desc_files) > 0) desc_path <- desc_files[1]
    }
    
    if (!file.exists(desc_path)) {
      stop(paste("DESCRIPTION file not found for", package_name))
    }
    
    dependencies <- parse_dcf_dependencies_version(package_dir) %>%
      dplyr::mutate(parent_package = package_name)
    
    license <- NA_character_
    if (get_license) {
      license <- extract_license_from_description(package_dir)
    }
    
    unlink(temp_dir, recursive = TRUE)
    
    list(dependencies = dependencies, license = license)
    
  }, error = function(e) {
    message(paste("Failed to download or parse", package_name, "-", e$message))
    list(
      dependencies = data.frame(
        package = character(),
        type = character(),
        parent_package = character(),
        stringsAsFactors = FALSE
      ),
      license = NA_character_
    )
  })
}



#' Extract the Installed Version of a Package
#' 
#' This function retrieves the installed version of a specified R package.
#' If the package is not installed, returns `NA`.
#'
#' @param package_name A character string specifying the name of the package.
#' 
#' @return A character string representing the installed package version, or `NA` if the package is not found.
#' 
#' @examples
#' \dontrun{
#' extract_package_version("dplyr")
#' # 1.1.4
#' }
#' 
#' @export
extract_package_version <- function(package_name) {
  version <- tryCatch(
    packageDescription(package_name, fields = "Version"),
    error = function(e) {
      message("Error: ", e$message)
      return(NA)
    }
  )
  return(version)
}

#' Build a Dependency Tree for an R Package
#' 
#' This function constructs a nested list representing the dependency tree of a specified R package.
#' The recursion stops at the specified maximum depth level (default: 3) or until a base package is reached.
#'
#' @param package_name A character string specifying the name of the package.
#' @param level An integer indicating the current recursion depth (default: 1).
#' @param get_license A logical value indicating whether to extract and include license information from DESCRIPTION files (default: `FALSE`).
#' @param max_level An integer specifying the maximum depth level for dependency recursion (default: 3). 
#'                  Set to a higher value to explore deeper dependency layers.
#'
#' @return A nested list representing the dependency tree, where base packages are marked as "base".
#'         If `get_license` is `TRUE`, each package entry in the tree includes a `license` field containing 
#'         that specific package's license (extracted from its DESCRIPTION file). Each package's license is 
#'         independent - the license of a parent package does not imply the licenses of its dependencies.
#'
#' @examples
#' \dontrun{
#' build_dependency_tree("ggplot2")
#' build_dependency_tree("ggplot2", get_license = TRUE)
#' build_dependency_tree("ggplot2", max_level = 5)  # Deeper dependency exploration
#' }
#'
#' @export
build_dependency_tree <- function(package_name, level = 1, get_license = FALSE, max_level = 3) {
  
  message("Dependency tree in progress for " , package_name, " package")

  # Validate max_level (must be non-negative)
  if (!is.numeric(max_level) || length(max_level) != 1 || max_level < 0 || max_level != as.integer(max_level)) {
    warning("max_level must be a non-negative integer")
    return(NULL)  
  }
  
  # Validate package_name
  if (!is.character(package_name) || length(package_name) != 1 || is.na(package_name) || nchar(trimws(package_name)) == 0) {
    warning("package_name must be a non-empty character string")
    return(NULL)  
  }
  
  if (level > max_level) return(NULL)  # Stop at max_level
  
  result <- download_and_parse_dependencies(package_name, get_license = get_license)
  
  # Handle both old format (data.frame) and new format (list with dependencies and license)
  # Note: data.frames are lists in R, so check for list with "dependencies" key first
  if (is.null(result)) {
    deps <- data.frame(package = character(), type = character(), stringsAsFactors = FALSE)
    package_license <- NULL
  } else if (is.list(result) && !is.data.frame(result) && "dependencies" %in% names(result)) {
    # This is the new format: list with dependencies and license
    deps <- result$dependencies
    package_license <- result$license
  } else if (is.data.frame(result)) {
    # This is the old format: just a data.frame
    deps <- result
    package_license <- NULL
  } else {
    deps <- data.frame(package = character(), type = character(), stringsAsFactors = FALSE)
    package_license <- NULL
  }
  
  # Ensure deps has the required structure
  if (!is.data.frame(deps) || !"package" %in% names(deps)) {
    deps <- data.frame(package = character(), type = character(), stringsAsFactors = FALSE)
  }
  
  package_version <- extract_package_version(package_name)
  
  dep_list <- list(version = package_version)
  if (get_license && !is.null(package_license) && !is.na(package_license)) {
    dep_list$license <- package_license
  }
  
  if (nrow(deps) > 0 && "package" %in% names(deps)) {
    for (dep in deps$package) {
      if (!is.na(dep) && !is.null(dep) && nchar(trimws(dep)) > 0) {
        dep_name <- gsub("\\s*\\(.*\\)", "", dep)
        
        if (is_base(dep_name)) {
          dep_list[[dep_name]] <- "base"
        } else {
          dep_list[[dep_name]] <- build_dependency_tree(dep_name, level + 1, get_license = get_license, max_level = max_level)
        }
      }
    }
  }
  
  message("Finished building for ", package_name)
  
  return(dep_list)
}

#' Fetch All Dependencies for a Package
#' 
#' This function builds and retrieves the full dependency tree for a given R package.
#'
#' @param package_name A character string specifying the name of the package.
#' @param get_license A logical value indicating whether to extract and include license information from DESCRIPTION files (default: `FALSE`).
#' @param max_level An integer specifying the maximum depth level for dependency recursion (default: 3).
#'                  Set to a higher value to explore deeper dependency layers.
#' 
#' @return A nested list representing the dependency tree of the package. If `get_license` is `TRUE`, 
#'         each package in the tree (including the root package and all dependencies) will have a 
#'         `license` field containing that package's license extracted from its DESCRIPTION file.
#' 
#' @examples
#' \dontrun{
#' fetch_all_dependencies("ggplot2")
#' fetch_all_dependencies("ggplot2", get_license = TRUE)
#' fetch_all_dependencies("ggplot2", max_level = 5)  # Deeper dependency exploration
#' }
#'
#' @export
fetch_all_dependencies <- function(package_name, get_license = FALSE, max_level = 3) {
  
  # Validate parameters and throw errors (not warnings) for fetch_all_dependencies
  if (!is.numeric(max_level) || length(max_level) != 1 || max_level < 0 || max_level != as.integer(max_level)) {
    stop("max_level must be a non-negative integer")
  }
  
  if (!is.character(package_name) || length(package_name) != 1 || is.na(package_name) || nchar(trimws(package_name)) == 0) {
    stop("package_name must be a non-empty character string")
  }
  
  message("Building dependency tree for: ", package_name, "\n")
  dep_tree <- list()
  tree_result <- build_dependency_tree(package_name, get_license = get_license, max_level = max_level)
  
  # In R, assigning NULL to a list element removes it, so we need to handle this
  # Store the result, which could be NULL
  if (!is.null(tree_result)) {
    dep_tree[[package_name]] <- tree_result
  }
  # If NULL, the list will be empty, which is acceptable behavior
  
  return(dep_tree)
}


#' Print a Package Dependency Tree
#' 
#' This function prints a hierarchical representation of a package's dependencies, including version and base status
#'
#' @param tree A nested list representing the package dependency tree, where each package has a `version` field and potentially sub-dependencies.
#' @param prefix A character string used for formatting tree branches (default: "").
#' @param last A logical value indicating if the current package is the last in the tree level (default: `TRUE`).
#' @param show_version A logical value indicating whether to display package versions if available (default: `TRUE`).
#' 
#' @return Prints the dependency tree to the console.
#' 
#' @examples
#' \dontrun{
#' db <- list(
#'   stringr = list(
#'     version = "1.5.1",
#'     cli = list(version = "3.6.2", utils = "base"),
#'     glue = list(version = "1.7.0", methods = "base"),
#'     lifecycle = list(
#'       version = "1.0.4",
#'       cli = list(version = "3.6.2", utils = "base"),
#'       glue = list(version = "1.7.0", methods = "base"),
#'       rlang = list(version = "1.1.3", utils = "base")
#'     ),
#'     magrittr = list(version = "2.0.3"),
#'     rlang = list(version = "1.1.3", utils = "base"),
#'     stringi = list(version = "1.8.3", tools = "base", utils = "base", stats = "base"),
#'     vctrs = list(
#'       version = "0.6.5",
#'       cli = list(version = "3.6.2", utils = "base"),
#'       glue = list(version = "1.7.0", methods = "base"),
#'       lifecycle = list(version = "1.0.4"),
#'       rlang = list(version = "1.1.3", utils = "base")
#'     )
#'   )
#' )
#' print_tree(db)
#' }
#' 
#' @export
print_tree <- function(tree, prefix = "", last = TRUE, show_version = TRUE) {
  
  if (is.list(tree)) {
    # Filter out metadata fields (version, license) - they're not packages
    # These should be displayed as part of package info, not as separate entries
    all_names <- names(tree)
    metadata_fields <- c("version", "license")
    pkg_names <- all_names[!all_names %in% metadata_fields]
    
    # Check if version/license exist at current level (for root package display)
    has_version_here <- "version" %in% all_names && !is.list(tree[["version"]])
    has_license_here <- "license" %in% all_names && !is.list(tree[["license"]])
    
    # Special case: if we're at root level (prefix == "") and there are metadata fields
    # but no package names, this means we're printing a tree that has no root package name
    # (e.g., result of build_dependency_tree directly). In this case, we should skip printing
    # and let the caller handle it, OR we could print a summary. But typically this shouldn't happen.
    
    n <- length(pkg_names)
    
    # If no packages at this level, return early (only metadata)
    if (n == 0) {
      return(invisible(NULL))
    }
    
    i <- 1
    for (pkg in pkg_names) {
      
      # Check if it's a base package
      is_base_pkg <- identical(tree[[pkg]], "base")
      
      # Extract version - use current level metadata if available (for first package at root), otherwise from subtree
      pkg_version <- ""
      if (!is_base_pkg && show_version) {
        # If we're at root level (prefix == "") and version exists here, use it for first package
        if (has_version_here && i == 1 && prefix == "") {
          # This is the first package at root level, use version from current tree metadata
          pkg_version <- paste0(" (v", tree[["version"]], ")")
        } else if (is.list(tree[[pkg]]) && "version" %in% names(tree[[pkg]])) {
          # Check if version is in the package subtree
          pkg_version <- paste0(" (v", tree[[pkg]]$version, ")")
        }
      }
      
      # Extract license - use current level metadata if available (for first package at root), otherwise from subtree
      pkg_license <- ""
      if (!is_base_pkg) {
        # First check if license is in the package subtree (most common case)
        if (is.list(tree[[pkg]]) && "license" %in% names(tree[[pkg]])) {
          license_val <- tree[[pkg]]$license
          if (!is.null(license_val) && !is.na(license_val) && license_val != "") {
            # For root level (prefix == ""), add "license" word, otherwise just the value
            if (prefix == "") {
              pkg_license <- paste0(" ", license_val, " license")
            } else {
              pkg_license <- paste0(" ", license_val)
            }
          }
        } else if (has_license_here && i == 1 && prefix == "") {
          # This is the first package at root level, use license from current tree metadata
          # (for cases where build_dependency_tree result is printed directly)
          license_val <- tree[["license"]]
          if (!is.null(license_val) && !is.na(license_val) && license_val != "") {
            # For root level, add "license" word
            pkg_license <- paste0(" ", license_val, " license")
          }
        }
      } else if (is_base_pkg) {
        # Base packages don't have licenses in the tree structure since they're not downloaded
        pkg_license <- ""
      }
      
      # Append "(base)" for base packages
      base_tag <- if (is_base_pkg) " (base)" else ""
      
      # Formatting for tree structure using Unicode escapes
      connector <- if (i == n) "\u2514\u2500\u2500 " else "\u251c\u2500\u2500 "
      cat(prefix, connector, pkg, pkg_version, base_tag, pkg_license, "\n", sep = "")
      
      # If it's not a base package, recurse into sub-tree
      if (!is_base_pkg) {
        sub_tree <- tree[[pkg]]
        if (is.list(sub_tree)) {
          sub_tree$version <- NULL
          sub_tree$license <- NULL  # Remove license from sub_tree to avoid duplication
        }
        
        new_prefix <- if (i == n) paste0(prefix, "    ") else paste0(prefix, "\u2502   ")
        print_tree(sub_tree, new_prefix, i == n, show_version) 
      }
      
      i <- i + 1
    }
  }
}



#' Detect Version Conflicts from dependency tree
#'
#' This function identifies duplicate version packages and reports conflicts.
#'
#' @param pkg_list A nested list structure from fetch_all_dependencies function
#'
#' @return A list of strings describing package version conflicts such as "Conflict in package cli: versions found - 3.6.2, 3.6.1"
#' `NULL` if no conflicts are found.
#'
#' @examples
#' pkg_list <- list(
#'   ggplot2 = list(
#'     version = "3.5.1",
#'     cli = list(version = "3.6.2"),
#'     gtable = list(
#'       version = "0.3.4",
#'       cli = list(version = "3.6.1")
#'     )
#'   )
#' )
#' detect_version_conflicts(pkg_list)
#'
#' @export
detect_version_conflicts <- function(pkg_list) {
  
  check_versions <- function(sublist, version_map = list()) {
    for (key in names(sublist)) {
      if (is.list(sublist[[key]])) {
        if ("version" %in% names(sublist[[key]])) {
          version <- sublist[[key]]$version
          
          # Append version to the existing list
          if (is.null(version_map[[key]])) {
            version_map[[key]] <- c(version)
          } else if (!(version %in% version_map[[key]])) {
            version_map[[key]] <- unique(c(version_map[[key]], version))
          }
        }
        version_map <- check_versions(sublist[[key]], version_map)
      }
    }
    return(version_map)
  }
  
  # Collect all versions encountered
  version_map <- check_versions(pkg_list)
  
  # Identify conflicts 
  conflicts <- list()
  for (pkg in names(version_map)) {
    if (length(version_map[[pkg]]) > 1) {
      conflicts[[pkg]] <- version_map[[pkg]]
    }
  }
  
  # Return conflicts in a readable format
  if (length(conflicts) > 0) {
    return(lapply(names(conflicts), function(pkg) {
      sprintf("Conflict in package %s: versions found - %s", 
              pkg, paste(conflicts[[pkg]], collapse = ", "))
    }))
  } else {
    return(NULL)
  }
}

# is_base

test_that("get_package_version returns correct version or Unknown", {
  expect_true(is_base("utils"))
  expect_true(is_base("methods"))
  expect_false(is_base("ggplot2"))
  expect_false(is_base("stringr"))
  expect_false(is_base("xxxxx"))
})


# parse_dcf_dependencies_version

test_that("parse_dcf_dependencies_version extracts dependencies correctly", {
  
  # Create a temporary DESCRIPTION file
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  # Write sample dependencies
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0",
    "Imports: ggplot2, dplyr, tidyr"
  ), desc_path)
  
  deps <- parse_dcf_dependencies_version(temp_dir)
  
  # Expected result
  expected <- data.frame(
    package = c("ggplot2", "dplyr", "tidyr"),
    type = rep("Imports", 3),
    stringsAsFactors = FALSE
  )
  
  expect_equal(deps, expected)
})



# build_dependency_tree

# Mock
mock_parse_dcf_dependencies <- function(path) {
  data.frame(
    package = c("ggplot2", "dplyr"),
    type = c("Imports", "Imports"),
    stringsAsFactors = FALSE
  )
}

# download_and_parse_dependencies function

# Mock
mock_parse_dcf_dependencies <- function(path) {
  data.frame(
    package = c("ggplot2", "dplyr"),
    type = c("Imports", "Imports"),
    stringsAsFactors = FALSE
  )
}

test_that("download_and_parse_dependencies 2 Imports package", {
  
  # Create a temporary directory for testing
  temp_dir <- tempdir()
  package_name <- "testpkg"
  version <- "1.0.0"
  
  # Create a correctly structured package directory (inside temp_dir)
  package_root <- file.path(temp_dir, paste0(package_name, "_", version))  
  dir.create(package_root, recursive = TRUE)
  
  # Create DESCRIPTION file inside the correct package structure
  desc_path <- file.path(package_root, "DESCRIPTION")
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0",
    "Imports: ggplot2, dplyr"
  ), desc_path)
  
  # Create a tarball with the correct structure
  tarball_path <- file.path(temp_dir, paste0(package_name, "_", version, ".tar.gz"))
  
  old_wd <- getwd()  # Save working directory
  setwd(temp_dir)  # Move to temp_dir for tar creation
  tar(tarball_path, files = basename(package_root), compression = "gzip", tar = "internal")
  setwd(old_wd)  # Restore working directory
  
  # Mock remotes::download_version to return the correct tarball path
  mock_download_version <- function(package, version, repos) {
    return(tarball_path)
  }
  
  # Use mockery to replace real functions with mocks
  mockery::stub(download_and_parse_dependencies, "remotes::download_version", mock_download_version)
  mockery::stub(download_and_parse_dependencies, "parse_dcf_dependencies_version", mock_parse_dcf_dependencies)
  
  # Run the function
  result <- download_and_parse_dependencies(package_name, version)
  
  # Expected result - function now always returns a list
  expected_deps <- data.frame(
    package = c("ggplot2", "dplyr"),
    type = c("Imports", "Imports"),
    parent_package = rep(package_name, 2),
    stringsAsFactors = FALSE
  )
  
  # Check that result is a list with dependencies and license
  expect_type(result, "list")
  expect_true("dependencies" %in% names(result))
  expect_true("license" %in% names(result))
  expect_equal(result$dependencies, expected_deps)
  expect_equal(result$license, NA_character_)
  
  # Cleanup temporary files
  unlink(temp_dir, recursive = TRUE)
})



# Mock
mock_parse_dcf_dependencies <- function(path) {
  data.frame(
    package = c(""),
    type = c(""),
    stringsAsFactors = FALSE
  )
}

test_that("download_and_parse_dependencies correctly - No DESCRIPTION file", {
  
  temp_dir <- tempdir()
  package_name <- "testpkg"
  version <- "1.0.0"
  
  package_root <- file.path(temp_dir, paste0(package_name, "_", version))  # Package root folder in tarball
  dir.create(package_root, recursive = TRUE)
  
  # Create DESCRIPTION file 
  desc_path <- file.path(package_root, "DESCRIPxxxTION")
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0"
  ), desc_path)
  
  mock_download_version <- function(package, version, repos) {
    return(tarball_path)
  }
  
  # Create a tarball with the correct structure
  tarball_path <- file.path(temp_dir, paste0(package_name, "_", version, ".tar.gz"))
  
  old_wd <- getwd()
  setwd(temp_dir)
  tar(tarball_path, files = basename(package_root), compression = "gzip", tar = "internal")
  setwd(old_wd)
  
  # Use mockery 
  mockery::stub(download_and_parse_dependencies, "remotes::download_version", mock_download_version)
  mockery::stub(download_and_parse_dependencies, "parse_dcf_dependencies_version", mock_parse_dcf_dependencies)
  
  result <- download_and_parse_dependencies(package_name, version)
  
  # Should return error structure (list with empty dependencies and NA license)
  expect_type(result, "list")
  expect_true("dependencies" %in% names(result))
  expect_true("license" %in% names(result))
  expect_equal(nrow(result$dependencies), 0)
  expect_equal(result$license, NA_character_)
  
  unlink(temp_dir, recursive = TRUE)
})


test_that("download_and_parse_dependencies error in remotes::download_version", {
  
  temp_dir <- tempdir()
  package_name <- "testpkg"
  version <- "1.0.0"
  
  package_root <- file.path(temp_dir, paste0(package_name, "_", version))
  dir.create(package_root, recursive = TRUE)
  
  # Create DESCRIPTION file 
  desc_path <- file.path(package_root, "DESCRIPTION")
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0"
  ), desc_path)
  
  tarball_path <- file.path(temp_dir, paste0(package_name, "_", version, ".tar.gz"))
  
  old_wd <- getwd()
  setwd(temp_dir)
  tar(tarball_path, files = basename(package_root), compression = "gzip", tar = "internal")
  setwd(old_wd)
  
  mock_download_version <- function(package, version, repos) {
    stop()
  }
  
  mockery::stub(download_and_parse_dependencies, "remotes::download_version", mock_download_version)
  mockery::stub(download_and_parse_dependencies, "parse_dcf_dependencies_version", mock_parse_dcf_dependencies)
  
  result <- download_and_parse_dependencies(package_name, version)
  
  # Should return error structure (list with empty dependencies and NA license)
  expect_type(result, "list")
  expect_true("dependencies" %in% names(result))
  expect_true("license" %in% names(result))
  expect_equal(nrow(result$dependencies), 0)
  expect_equal(result$license, NA_character_)
  
  # Cleanup temporary files
  unlink(temp_dir, recursive = TRUE)
})


# extract_package_version

test_that("extract_package_version works correctly", {
  with_mocked_bindings(
    packageDescription = function(...) { "1.2.3" }, 
    {
      expect_equal(extract_package_version("testpkg"), "1.2.3")
    }
  )
})


test_that("extract_package_versionwith error in packageDescription", {
  with_mocked_bindings(
    packageDescription = function(...) stop(), 
    {
      expect_equal(extract_package_version("testpkg"), NA)
    }
  )
})

test_that("extract_package_version handles errors correctly", {
  with_mocked_bindings(
    packageDescription = function(...) { stop("mocked error") },
    {
      expect_message(
        result <- extract_package_version("fakepkg"),
        "mocked error"
      )
      expect_true(is.na(result))
    }
  )
})



# build_dependency_tree

# Mock `download_and_parse_dependencies`
# Now always returns a list with dependencies and license
mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE, repos) {
  deps <- switch(package_name,
         "pkgA" = data.frame(package = c("pkgB", "pkgC"), type = "Imports", stringsAsFactors = FALSE),
         "pkgB" = data.frame(package = "pkgD", type = "Imports", stringsAsFactors = FALSE),
         "pkgC" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), 
         "pkgD" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), 
         data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE)
  )
  # Always return list format with dependencies and license
  return(list(dependencies = deps, license = NA_character_))
}

# Mock `extract_package_version`
mock_package_description <- function(package_name) {
  version <- switch(package_name,
                    "pkgA" = "1.0.0",
                    "pkgB" = "1.1.0",
                    "pkgC" = "1.2.0",
                    "pkgD" = "1.3.0",
                    "Unknown")  
  
  return(ifelse(is.null(version), "Unknown", version))
}

# Mock `is_base`
mock_is_base <- function(package_name) {
  return(FALSE) 
}

test_that("build_dependency_tree constructs correct tree recursively", {
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_package_description,
    is_base = mock_is_base, 
    
    {
      # Run the function
      result_tree <- build_dependency_tree("pkgA")
      
      # Expected Dependency Tree
      expected_tree <- list(
        version = "1.0.0",
        pkgB = list(
          version = "1.1.0",
          pkgD = list(
            version = "1.3.0"
          )
        ),
        pkgC = list(
          version = "1.2.0"
        )
      )
      
      # Validate the tree
      expect_equal(result_tree, expected_tree)
    }
  )
})

test_that("build_dependency_tree respects max_level parameter", {
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_package_description,
    is_base = mock_is_base,
    {
      # Test with max_level = 2 (should stop at level 2)
      result_tree_level2 <- build_dependency_tree("pkgA", max_level = 2)
      
      # pkgA (level 1) -> pkgB, pkgC (level 2) -> should stop, no pkgD
      expect_true("pkgB" %in% names(result_tree_level2))
      expect_true("pkgC" %in% names(result_tree_level2))
      # pkgD should not be present (would be level 3)
      expect_false("pkgD" %in% names(result_tree_level2$pkgB))
      
      # Test with max_level = 1 (should only have root)
      result_tree_level1 <- build_dependency_tree("pkgA", max_level = 1)
      expect_true("version" %in% names(result_tree_level1))
      expect_false("pkgB" %in% names(result_tree_level1))
      expect_false("pkgC" %in% names(result_tree_level1))
      
      # Test with max_level = 5 (should go deeper than default)
      result_tree_level5 <- build_dependency_tree("pkgA", max_level = 5)
      # Should have same structure as default (no deeper dependencies in mock)
      expect_true("pkgB" %in% names(result_tree_level5))
      expect_true("pkgC" %in% names(result_tree_level5))
    }
  )
})


test_that("build_dependency_tree outputs progress messages", {
  with_mocked_bindings(
    download_and_parse_dependencies = function(package_name, get_license = FALSE) {
      data.frame(package = character(0), stringsAsFactors = FALSE)  # No dependencies
    },
    extract_package_version = function(pkg) {
      "1.0.0"
    },
    is_base = function(pkg) {
      FALSE
    },
    {
      expect_message(
        result <- build_dependency_tree("testpkg"),
        "Dependency tree in progress for testpkg package",
        all = TRUE
      )
      
      expect_message(
        result <- build_dependency_tree("testpkg"),
        "Finished building for testpkg",
        all = TRUE
      )
      expect_equal(result, list(version = "1.0.0"))
    }
  )
})



# download_and_parse_dependencies

# Mock 
mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE, repos) {
  deps <- switch(package_name,
         "pkgA" = data.frame(package = c("pkgB", "pkgC"), type = "Imports", stringsAsFactors = FALSE),
         "pkgB" = data.frame(package = "pkgD", type = "Imports", stringsAsFactors = FALSE),
         "pkgC" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), # No dependencies
         "pkgD" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), # No dependencies
         "pkgE" = data.frame(package = "pkgF", type = "Imports", stringsAsFactors = FALSE), # Deep Dependency
         "pkgF" = data.frame(package = "pkgG", type = "Imports", stringsAsFactors = FALSE), # Deep Dependency
         "pkgG" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), # Deep Leaf
         "unknownPkg" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), # Unknown
         "basePkg" = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE), # Base package
         data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE) # Default empty
  )
  if (get_license) {
    return(list(dependencies = deps, license = NA))
  } else {
    return(deps)
  }
}

# Mock `extract_package_version`
mock_package_description <- function(package_name) {
  version <- switch(package_name,
                    "pkgA" = "1.0.0",
                    "pkgB" = "1.1.0",
                    "pkgC" = "1.2.0",
                    "pkgD" = "1.3.0",
                    "pkgE" = "2.0.0",
                    "pkgF" = "2.1.0",
                    "pkgG" = "2.2.0",
                    "unknownPkg" = "Unknown",
                    "basePkg" = "base", 
                    "Unknown")  
  
  return(ifelse(is.null(version), "Unknown", version))
}

# Mock
mock_is_base <- function(package_name) {
  return(package_name == "basePkg")  
}

test_that("build_dependency_tree constructs correct tree with various cases", {
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_package_description,
    is_base = mock_is_base,
    
    {
      # basic Tree Test (Already covered)
      result_tree <- build_dependency_tree("pkgA")
      expected_tree <- list(
        version = "1.0.0",
        pkgB = list(
          version = "1.1.0",
          pkgD = list(
            version = "1.3.0"
          )
        ),
        pkgC = list(
          version = "1.2.0"
        )
      )
      expect_equal(result_tree, expected_tree)
      
      # empty Dependency Test
      result_tree_c <- build_dependency_tree("pkgC")
      expect_equal(result_tree_c, list(version = "1.2.0"))
      
      # deep Dependency Chain Test
      result_tree_e <- build_dependency_tree("pkgE")
      expected_tree_e <- list(
        version = "2.0.0",
        pkgF = list(
          version = "2.1.0",
          pkgG = list(
            version = "2.2.0"
          )
        )
      )
      expect_equal(result_tree_e, expected_tree_e)
      
      # handling Unknown 
      result_tree_unknown <- build_dependency_tree("unknownPkg")
      expect_equal(result_tree_unknown, list(version = "Unknown"))
      
      # Base Package
      result_tree_base <- build_dependency_tree("basePkg")
      expect_equal(result_tree_base, list(version = "base"))
    }
  )
})


# fetch_all_dependencies


test_that("fetch_all_dependencies returns correct dependency tree", {
  
  # Mocked return value for build_dependency_tree
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3) {
    if (package_name == "ggplot2") {
      return(c("scales", "rlang", "tibble"))
    } else if (package_name == "scales") {
      return(c("R6", "digest"))
    }
    return(character(0))
  }
  
  # Stub the build_dependency_tree function
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  # Run the function with the test package
  result <- fetch_all_dependencies("ggplot2")
  
  # Assertions
  expect_type(result, "list")
  expect_named(result, "ggplot2")
  expect_equal(result$ggplot2, c("scales", "rlang", "tibble"))
})

test_that("fetch_all_dependencies passes max_level parameter correctly", {
  # Mocked return value for build_dependency_tree
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3) {
    # Return a list with max_level info for verification
    return(list(max_level_used = max_level, package = package_name))
  }
  
  # Stub the build_dependency_tree function
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  # Test with default max_level
  result_default <- fetch_all_dependencies("testpkg")
  expect_equal(result_default$testpkg$max_level_used, 3)
  
  # Test with custom max_level
  result_custom <- fetch_all_dependencies("testpkg", max_level = 5)
  expect_equal(result_custom$testpkg$max_level_used, 5)
  
  # Test with max_level = 1
  result_level1 <- fetch_all_dependencies("testpkg", max_level = 1)
  expect_equal(result_level1$testpkg$max_level_used, 1)
})


# print_tree

here <- list(
  version = "1.0.1",
  rprojroot = list(
    version = "2.0.4"
  )
)

test_that("here package", {
  expect_snapshot(print_tree(here, show_version = TRUE))
})

test_that("here package - no versions", {
  expect_snapshot(print_tree(here, show_version = FALSE))
})



stringr <- list(
  version = "1.5.1",
  cli = list(
    version = "3.6.2",
    utils = "base"
  ),
  glue = list(
    version = "1.7.0",
    methods = "base"
  ),
  lifecycle = list(
    version = "1.0.4",
    cli = list(
      version = "3.6.2",
      utils = "base"
    ),
    glue = list(
      version = "1.7.0",
      methods = "base"
    ),
    rlang = list(
      version = "1.1.3",
      utils = "base"
    )
  ),
  magrittr = list(
    version = "2.0.3"
  ),
  rlang = list(
    version = "1.1.3",
    utils = "base"
  ),
  stringi = list(
    version = "1.8.3",
    tools = "base",
    utils = "base",
    stats = "base"
  ),
  vctrs = list(
    version = "0.6.5",
    cli = list(
      version = "3.6.2",
      utils = "base"
    ),
    glue = list(
      version = "1.7.0",
      methods = "base"
    ),
    lifecycle = list(
      version = "1.0.4"
    ),
    rlang = list(
      version = "1.1.3",
      utils = "base"
    )
  )
)

test_that("stringr package", {
  expect_snapshot(print_tree(stringr, show_version = TRUE))
})

test_that("stringr package - no versions", {
  expect_snapshot(print_tree(stringr, show_version = FALSE))
})


test_that("stringr package - no versions", {
  expect_null(print_tree("stringr", show_version = FALSE))
})


# detect_version_conflicts

test_that("detect_version_conflicts detects a version conflict (cli)", {
  pkg_list <- list(
    ggplot2 = list(
      version = "3.5.1",
      cli = list(
        version = "3.6.2"
      ),
      gtable = list(
        version = "0.3.4",
        cli = list(version = "3.6.1")  # Conflict here!
      )
    )
  )
  
  result <- detect_version_conflicts(pkg_list)
  
  # Expect a conflict in cli
  expect_type(result, "list")
  expect_length(result, 1)
  expect_match(result[[1]], "Conflict in package cli: versions found - 3.6.2, 3.6.1")
})

test_that("detect_version_conflicts returns NULL when no conflict", {
  pkg_list <- list(
    ggplot2 = list(
      version = "3.5.1",
      cli = list(
        version = "3.6.2"
      ),
      gtable = list(
        version = "0.3.4",
        cli = list(version = "3.6.2")  # Same version, no conflict
      )
    )
  )
  
  result <- detect_version_conflicts(pkg_list)
  
  # Expect NULL since there are no conflicts
  expect_null(result)
})

test_that("detect_version_conflicts detects multiple version conflicts (glue, cli)", {
  pkg_list <- list(
    ggplot2 = list(
      version = "3.5.1",
      cli = list(version = "3.6.2"),
      gtable = list(
        version = "0.3.4",
        cli = list(version = "3.6.1"),  
        glue = list(version = "1.6.0")  
      ),
      glue = list(version = "1.7.0")  
    )
  )
  
  result <- detect_version_conflicts(pkg_list)
  
  expect_type(result, "list")
  expect_length(result, 2)
  expect_match(result[[1]], "Conflict in package cli: versions found - 3.6.2, 3.6.1")
  expect_match(result[[2]], "Conflict in package glue: versions found - 1.6.0, 1.7.0")
})

test_that("detect_version_conflicts works with deeply nested lists (cli)", {
  pkg_list <- list(
    ggplot2 = list(
      version = "3.5.1",
      cli = list(
        version = "3.6.2",
        deep_nested = list(
          cli = list(version = "3.6.1") 
        )
      )
    )
  )
  
  result <- detect_version_conflicts(pkg_list)
  expect_type(result, "list")
  expect_length(result, 1)
  expect_match(result[[1]], "Conflict in package cli: versions found - 3.6.2, 3.6.1")
})

test_that("detect_version_conflicts handles an empty list", {
  pkg_list <- list()
  
  result <- detect_version_conflicts(pkg_list)
  
  # Expect NULL for an empty list
  expect_null(result)
})


# extract_license_from_description

test_that("extract_license_from_description extracts license correctly", {
  # Create a temporary directory with DESCRIPTION file
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  # Write sample DESCRIPTION with license
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0",
    "License: MIT + file LICENSE"
  ), desc_path)
  
  license <- extract_license_from_description(temp_dir)
  
  expect_equal(license, "MIT + file LICENSE")
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

test_that("extract_license_from_description returns NA when no license field", {
  # Create a temporary directory with DESCRIPTION file
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  # Write sample DESCRIPTION without license
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0"
  ), desc_path)
  
  # Should warn about missing License field
  expect_warning(
    license <- extract_license_from_description(temp_dir),
    "No License field in DESCRIPTION"
  )
  
  expect_true(is.na(license))
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

test_that("extract_license_from_description handles missing DESCRIPTION file", {
  # Create a temporary directory without DESCRIPTION file
  temp_dir <- tempfile()
  dir.create(temp_dir)
  
  # Should warn about missing DESCRIPTION file
  expect_warning(
    license <- extract_license_from_description(temp_dir),
    "DESCRIPTION file not found"
  )
  
  expect_true(is.na(license))
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

test_that("extract_license_from_description handles various license formats", {
  # Create a temporary directory with DESCRIPTION file
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  # Test GPL-3 license
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0",
    "License: GPL-3"
  ), desc_path)
  
  license <- extract_license_from_description(temp_dir)
  expect_equal(license, "GPL-3")
  
  # Test AGPL-3.0 license
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0",
    "License: AGPL-3.0"
  ), desc_path)
  
  license <- extract_license_from_description(temp_dir)
  expect_equal(license, "AGPL-3.0")
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})


# build_dependency_tree with get_license parameter

test_that("build_dependency_tree includes license when get_license = TRUE", {
  # Mock download_and_parse_dependencies to return list with dependencies and license
  # Note: download_and_parse_dependencies now ALWAYS returns list(dependencies, license)
  # Version is extracted separately via extract_package_version
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    # Always return list format with dependencies and license
    return(list(
      dependencies = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE),
      license = if (get_license) "MIT + file LICENSE" else NA_character_
    ))
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result_tree <- build_dependency_tree("testpkg", get_license = TRUE)
      
      # Should have version and license
      expect_true("version" %in% names(result_tree))
      expect_true("license" %in% names(result_tree))
      expect_equal(result_tree$version, "1.0.0")
      expect_equal(result_tree$license, "MIT + file LICENSE")
    }
  )
})

test_that("build_dependency_tree excludes license when get_license = FALSE", {
  # Mock download_and_parse_dependencies to return data.frame (old format)
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    return(data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE))
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result_tree <- build_dependency_tree("testpkg", get_license = FALSE)
      
      # Should have version but NOT license
      expect_true("version" %in% names(result_tree))
      expect_false("license" %in% names(result_tree))
      expect_equal(result_tree$version, "1.0.0")
    }
  )
})

test_that("build_dependency_tree includes licenses for nested dependencies", {
  # Mock download_and_parse_dependencies to return dependencies with licenses
  # Note: download_and_parse_dependencies returns list(dependencies, license) when get_license=TRUE
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    if (package_name == "pkgA") {
      if (get_license) {
        return(list(
          dependencies = data.frame(package = "pkgB", type = "Imports", stringsAsFactors = FALSE),
          license = "MIT"
        ))
      } else {
        return(data.frame(package = "pkgB", type = "Imports", stringsAsFactors = FALSE))
      }
    } else if (package_name == "pkgB") {
      if (get_license) {
        return(list(
          dependencies = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE),
          license = "GPL-3"
        ))
      } else {
        return(data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE))
      }
    }
    return(data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE))
  }
  
  mock_extract_package_version <- function(package_name) {
    switch(package_name,
           "pkgA" = "1.0.0",
           "pkgB" = "1.1.0",
           "Unknown")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result_tree <- build_dependency_tree("pkgA", get_license = TRUE)
      
      # Root should have license
      expect_equal(result_tree$license, "MIT")
      
      # Nested dependency should have license
      expect_true("license" %in% names(result_tree$pkgB))
      expect_equal(result_tree$pkgB$license, "GPL-3")
    }
  )
})


# fetch_all_dependencies with get_license parameter

test_that("fetch_all_dependencies includes license when get_license = TRUE", {
  # Mock build_dependency_tree to return tree with license
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3) {
    if (get_license) {
      return(list(
        version = "1.0.0",
        license = "MIT + file LICENSE"
      ))
    } else {
      return(list(
        version = "1.0.0"
      ))
    }
  }
  
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  result <- fetch_all_dependencies("testpkg", get_license = TRUE)
  
  expect_type(result, "list")
  expect_named(result, "testpkg")
  expect_true("license" %in% names(result$testpkg))
  expect_equal(result$testpkg$license, "MIT + file LICENSE")
})

test_that("fetch_all_dependencies excludes license when get_license = FALSE", {
  # Mock build_dependency_tree to return tree without license
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3) {
    return(list(
      version = "1.0.0"
    ))
  }
  
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  result <- fetch_all_dependencies("testpkg", get_license = FALSE)
  
  expect_type(result, "list")
  expect_named(result, "testpkg")
  expect_false("license" %in% names(result$testpkg))
})


# print_tree with licenses

test_that("print_tree does not show version and license as separate packages", {
  # Create a tree with version and license at root level
  test_tree <- list(
    version = "1.0.0",
    license = "MIT + file LICENSE",
    cli = list(
      version = "3.6.2"
    )
  )
  
  # Capture output
  output <- capture.output(print_tree(test_tree))
  
  # Should NOT contain "├── version" or "├── license" as separate entries
  expect_false(any(grepl("├── version", output)))
  expect_false(any(grepl("├── license", output)))
  expect_false(any(grepl("└── version", output)))
  expect_false(any(grepl("└── license", output)))
  
  # Should show cli with version
  expect_true(any(grepl("cli", output)))
})

test_that("print_tree displays license with root package", {
  # Create a tree wrapped by fetch_all_dependencies
  test_tree <- list(
    testpkg = list(
      version = "1.0.0",
      license = "MIT + file LICENSE",
      cli = list(
        version = "3.6.2"
      )
    )
  )
  
  # Capture output
  output <- capture.output(print_tree(test_tree))
  
  # Should show license with the package name
  # The output should be something like: "└── testpkg (v1.0.0) MIT + file LICENSE license"
  # Escape the + sign properly in regex
  expect_true(any(grepl("MIT.*file LICENSE license", output)) || any(grepl("MIT \\+ file LICENSE license", output)))
  expect_true(any(grepl("testpkg", output)))
})

test_that("print_tree displays licenses for nested dependencies", {
  # Create a tree with licenses at multiple levels
  test_tree <- list(
    testpkg = list(
      version = "1.0.0",
      license = "MIT",
      dep1 = list(
        version = "2.0.0",
        license = "GPL-3"
      )
    )
  )
  
  # Capture output
  output <- capture.output(print_tree(test_tree))
  
  # Should show license for root package (with "license" word at root level)
  expect_true(any(grepl("MIT license", output)))
  
  # Should show license for nested dependency (without "license" word, just the value)
  expect_true(any(grepl("GPL-3", output)))
  # Make sure it's not showing "GPL-3 license" for nested dependency
  expect_false(any(grepl("dep1.*GPL-3 license", output)))
})

test_that("print_tree handles tree with only metadata (no packages)", {
  # Edge case: tree with only version and license, no dependencies
  edge_tree <- list(
    version = "1.0.0",
    license = "MIT"
  )
  
  # Should not crash and should return NULL or print nothing
  expect_silent(print_tree(edge_tree))
  
  # Capture output
  output <- capture.output(print_tree(edge_tree))
  
  # Should not print version/license as packages
  expect_false(any(grepl("version", output)))
  expect_false(any(grepl("license", output)))
})

test_that("print_tree with licenses matches snapshot", {
  # Create a test tree with licenses
  test_tree <- list(
    stringr = list(
      version = "1.5.1",
      license = "MIT + file LICENSE",
      cli = list(
        version = "3.6.2",
        license = "MIT + file LICENSE",
        utils = "base"
      )
    )
  )
  
  expect_snapshot(print_tree(test_tree, show_version = TRUE))
})


# Comprehensive test coverage for all combinations

# Test all combinations: get_license (TRUE/FALSE) x show_version (TRUE/FALSE)

test_that("print_tree: get_license=TRUE, show_version=TRUE", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = "MIT",
      dep = list(
        version = "2.0.0",
        license = "GPL-3"
      )
    )
  )
  
  output <- capture.output(print_tree(test_tree, show_version = TRUE))
  
  # Should show versions
  expect_true(any(grepl("v1.0.0", output)))
  expect_true(any(grepl("v2.0.0", output)))
  
  # Should show licenses
  expect_true(any(grepl("MIT license", output)))
  expect_true(any(grepl("GPL-3", output)))
  
  # Should NOT show version/license as separate packages
  expect_false(any(grepl("├── version", output)))
  expect_false(any(grepl("├── license", output)))
})

test_that("print_tree: get_license=TRUE, show_version=FALSE", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = "MIT",
      dep = list(
        version = "2.0.0",
        license = "GPL-3"
      )
    )
  )
  
  output <- capture.output(print_tree(test_tree, show_version = FALSE))
  
  # Should NOT show versions
  expect_false(any(grepl("v1.0.0", output)))
  expect_false(any(grepl("v2.0.0", output)))
  
  # Should show licenses
  expect_true(any(grepl("MIT license", output)))
  expect_true(any(grepl("GPL-3", output)))
  
  # Should NOT show version/license as separate packages
  expect_false(any(grepl("├── version", output)))
  expect_false(any(grepl("├── license", output)))
})

test_that("print_tree: get_license=FALSE, show_version=TRUE", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      dep = list(
        version = "2.0.0"
      )
    )
  )
  
  output <- capture.output(print_tree(test_tree, show_version = TRUE))
  
  # Should show versions
  expect_true(any(grepl("v1.0.0", output)))
  expect_true(any(grepl("v2.0.0", output)))
  
  # Should NOT show licenses (none in tree)
  expect_false(any(grepl("license", output)))
  
  # Should NOT show version/license as separate packages
  expect_false(any(grepl("├── version", output)))
})

test_that("print_tree: get_license=FALSE, show_version=FALSE", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      dep = list(
        version = "2.0.0"
      )
    )
  )
  
  output <- capture.output(print_tree(test_tree, show_version = FALSE))
  
  # Should NOT show versions
  expect_false(any(grepl("v1.0.0", output)))
  expect_false(any(grepl("v2.0.0", output)))
  
  # Should NOT show licenses
  expect_false(any(grepl("license", output)))
  
  # Should show package names
  expect_true(any(grepl("pkg", output)))
  expect_true(any(grepl("dep", output)))
})


# Error scenarios and edge cases

test_that("build_dependency_tree handles license extraction failure (NA)", {
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    if (get_license) {
      return(list(
        dependencies = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE),
        license = NA  # License extraction failed
      ))
    } else {
      return(data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE))
    }
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result_tree <- build_dependency_tree("testpkg", get_license = TRUE)
      
      # Should have version
      expect_true("version" %in% names(result_tree))
      
      # License should be NA or NULL (not included if NA)
      if ("license" %in% names(result_tree)) {
        expect_true(is.na(result_tree$license))
      }
    }
  )
})

test_that("build_dependency_tree handles empty license string", {
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    if (get_license) {
      return(list(
        dependencies = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE),
        license = ""  # Empty license string
      ))
    } else {
      return(data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE))
    }
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result_tree <- build_dependency_tree("testpkg", get_license = TRUE)
      
      # Should have version
      expect_true("version" %in% names(result_tree))
      
      # License might be empty string or not included
      if ("license" %in% names(result_tree)) {
        expect_true(result_tree$license == "" || is.na(result_tree$license))
      }
    }
  )
})

test_that("print_tree handles NULL license gracefully", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = NULL,
      dep = list(
        version = "2.0.0"
      )
    )
  )
  
  # Should not crash or error
  expect_error(print_tree(test_tree), NA)
  
  output <- capture.output(print_tree(test_tree))
  
  # Should show package names
  expect_true(any(grepl("pkg", output)))
  expect_true(any(grepl("dep", output)))
})

test_that("print_tree handles NA license gracefully", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = NA,
      dep = list(
        version = "2.0.0"
      )
    )
  )
  
  # Should not crash or error
  expect_error(print_tree(test_tree), NA)
  
  output <- capture.output(print_tree(test_tree))
  
  # Should show package names
  expect_true(any(grepl("pkg", output)))
  expect_true(any(grepl("dep", output)))
  
  # Should not show "NA license"
  expect_false(any(grepl("NA license", output)))
})

test_that("print_tree handles empty license string", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = "",
      dep = list(
        version = "2.0.0"
      )
    )
  )
  
  # Should not crash or error
  expect_error(print_tree(test_tree), NA)
  
  output <- capture.output(print_tree(test_tree))
  
  # Should show package names
  expect_true(any(grepl("pkg", output)))
})

test_that("print_tree handles base packages with license metadata (should ignore)", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = "MIT",
      utils = "base"  # Base package
    )
  )
  
  output <- capture.output(print_tree(test_tree))
  
  # Should show base package
  expect_true(any(grepl("utils.*base", output)))
  
  # Should NOT show license for base package
  expect_false(any(grepl("utils.*MIT", output)))
})

test_that("build_dependency_tree with get_license=TRUE handles base packages correctly", {
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    if (get_license) {
      return(list(
        dependencies = data.frame(package = "utils", type = "Imports", stringsAsFactors = FALSE),
        license = "MIT"
      ))
    } else {
      return(data.frame(package = "utils", type = "Imports", stringsAsFactors = FALSE))
    }
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) pkg == "utils",
    {
      result_tree <- build_dependency_tree("testpkg", get_license = TRUE)
      
      # Root should have license
      expect_equal(result_tree$license, "MIT")
      
      # Base package should be marked as "base", not have license
      expect_equal(result_tree$utils, "base")
    }
  )
})

test_that("print_tree handles tree with dependencies but no version/license metadata", {
  test_tree <- list(
    pkg = list(
      dep1 = list(),
      dep2 = "base"
    )
  )
  
  # Should not crash or error
  expect_error(print_tree(test_tree), NA)
  
  output <- capture.output(print_tree(test_tree))
  
  # Should show package names
  expect_true(any(grepl("pkg", output)))
  expect_true(any(grepl("dep1", output)))
  expect_true(any(grepl("dep2.*base", output)))
})

test_that("print_tree handles invalid tree structure (not a list)", {
  # Should handle gracefully
  expect_silent(print_tree("not a list"))
  expect_silent(print_tree(NULL))
  expect_silent(print_tree(123))
  
  # Should return NULL or print nothing
  output <- capture.output(print_tree("not a list"))
  expect_length(output, 0)
})

test_that("extract_license_from_description handles corrupted DESCRIPTION file", {
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  # Write corrupted DESCRIPTION (invalid DCF format)
  writeLines(c(
    "Package: testpkg",
    "Version: 1.0.0",
    "License: MIT + file LICENSE",
    "Invalid line without colon"
  ), desc_path)
  
  # Should handle gracefully (might return NA or the license)
  suppressWarnings(license <- extract_license_from_description(temp_dir))
  
  # Should either return the license or NA
  expect_true(is.character(license) || is.na(license))
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

test_that("extract_license_from_description handles empty DESCRIPTION file", {
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  # Write empty DESCRIPTION
  writeLines("", desc_path)
  
  # Should warn about missing License field
  expect_warning(
    license <- extract_license_from_description(temp_dir),
    "No License field in DESCRIPTION"
  )
  
  expect_true(is.na(license))
  
  # Cleanup
  unlink(temp_dir, recursive = TRUE)
})

test_that("build_dependency_tree handles download_and_parse_dependencies returning NULL with get_license=TRUE", {
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    return(NULL)  # Download failed
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result_tree <- build_dependency_tree("testpkg", get_license = TRUE)
      
      # Should have version
      expect_true("version" %in% names(result_tree))
      
      # Should not have license (NULL result means no license extracted)
      expect_false("license" %in% names(result_tree))
    }
  )
})

test_that("print_tree handles complex license strings with special characters", {
  test_tree <- list(
    pkg = list(
      version = "1.0.0",
      license = "MIT + file LICENSE | GPL-2",
      dep = list(
        version = "2.0.0",
        license = "AGPL-3.0 + file LICENSE"
      )
    )
  )
  
  output <- capture.output(print_tree(test_tree))
  
  # Should handle special characters in license
  expect_true(any(grepl("MIT.*file LICENSE", output)) || any(grepl("GPL-2", output)))
  expect_true(any(grepl("AGPL-3.0", output)))
})

test_that("fetch_all_dependencies with get_license=TRUE handles build_dependency_tree returning NULL", {
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3) {
    return(NULL)  # Build failed
  }
  
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  result <- fetch_all_dependencies("testpkg", get_license = TRUE)
  
  expect_type(result, "list")
  # Note: In R, assigning NULL to a list element removes it, so when build_dependency_tree
  # returns NULL, the result will be an empty list (the element is not added)
  expect_length(result, 0)
})


# Validation tests for build_dependency_tree - now uses warnings instead of errors

test_that("build_dependency_tree warns on negative max_level", {
  expect_warning(
    build_dependency_tree("testpkg", max_level = -1),
    "max_level must be a non-negative integer"
  )
})

test_that("build_dependency_tree warns on fractional max_level", {
  # Mock extract_package_version to avoid "package not found" warnings
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    extract_package_version = mock_extract_package_version,
    {
      expect_warning(
        result <- build_dependency_tree("testpkg", max_level = 3.5),
        "max_level must be a non-negative integer"
      )
      expect_null(result)  # Should return NULL after warning
    }
  )
})

test_that("build_dependency_tree warns on NULL max_level", {
  expect_warning(
    build_dependency_tree("testpkg", max_level = NULL),
    "max_level must be a non-negative integer"
  )
})

test_that("build_dependency_tree warns on character max_level", {
  # Mock extract_package_version to avoid "package not found" warnings
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    extract_package_version = mock_extract_package_version,
    {
      expect_warning(
        result <- build_dependency_tree("testpkg", max_level = "3"),
        "max_level must be a non-negative integer"
      )
      expect_null(result)  # Should return NULL after warning
    }
  )
})

test_that("build_dependency_tree warns on vector max_level", {
  # Mock extract_package_version to avoid "package not found" warnings
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    extract_package_version = mock_extract_package_version,
    {
      expect_warning(
        result <- build_dependency_tree("testpkg", max_level = c(1, 2, 3)),
        "max_level must be a non-negative integer"
      )
      expect_null(result)  # Should return NULL after warning
    }
  )
})

test_that("build_dependency_tree warns on empty package_name", {
  expect_warning(
    build_dependency_tree(""),
    "package_name must be a non-empty character string"
  )
})

test_that("build_dependency_tree warns on NA package_name", {
  expect_warning(
    build_dependency_tree(NA_character_),
    "package_name must be a non-empty character string"
  )
})

test_that("build_dependency_tree warns on NULL package_name", {
  expect_warning(
    build_dependency_tree(NULL),
    "package_name must be a non-empty character string"
  )
})

test_that("build_dependency_tree warns on numeric package_name", {
  # Mock extract_package_version to avoid "package not found" warnings
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    extract_package_version = mock_extract_package_version,
    {
      expect_warning(
        result <- build_dependency_tree(123),
        "package_name must be a non-empty character string"
      )
      expect_null(result)  # Should return NULL after warning
    }
  )
})

test_that("build_dependency_tree warns on vector package_name", {
  # Mock download_and_parse_dependencies to avoid actual download attempts
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    return(list(
      dependencies = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE),
      license = NA_character_
    ))
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    {
      expect_warning(
        result <- build_dependency_tree(c("pkg1", "pkg2")),
        "package_name must be a non-empty character string"
      )
      expect_null(result)  # Should return NULL after warning
    }
  )
})

# Validation tests for fetch_all_dependencies - still uses errors

test_that("fetch_all_dependencies rejects negative max_level", {
  expect_error(
    fetch_all_dependencies("testpkg", max_level = -1),
    "max_level must be a non-negative integer"
  )
})

test_that("fetch_all_dependencies rejects fractional max_level", {
  expect_error(
    fetch_all_dependencies("testpkg", max_level = 3.5),
    "max_level must be a non-negative integer"
  )
})

test_that("fetch_all_dependencies rejects NULL max_level", {
  expect_error(
    fetch_all_dependencies("testpkg", max_level = NULL),
    "max_level must be a non-negative integer"
  )
})

test_that("fetch_all_dependencies rejects character max_level", {
  expect_error(
    fetch_all_dependencies("testpkg", max_level = "3"),
    "max_level must be a non-negative integer"
  )
})

test_that("fetch_all_dependencies rejects vector max_level", {
  expect_error(
    fetch_all_dependencies("testpkg", max_level = c(1, 2, 3)),
    "max_level must be a non-negative integer"
  )
})

test_that("fetch_all_dependencies accepts zero max_level", {
  # Mock build_dependency_tree to avoid actual download
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3, visited = character()) {
    return(list(version = "1.0.0"))
  }
  
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  result <- fetch_all_dependencies("testpkg", max_level = 0)
  expect_type(result, "list")
})

test_that("fetch_all_dependencies rejects empty package_name", {
  expect_error(
    fetch_all_dependencies(""),
    "package_name must be a non-empty character string"
  )
})

test_that("fetch_all_dependencies rejects NA package_name", {
  expect_error(
    fetch_all_dependencies(NA_character_),
    "package_name must be a non-empty character string"
  )
})

test_that("fetch_all_dependencies rejects NULL package_name", {
  expect_error(
    fetch_all_dependencies(NULL),
    "package_name must be a non-empty character string"
  )
})

test_that("fetch_all_dependencies rejects numeric package_name", {
  expect_error(
    fetch_all_dependencies(123),
    "package_name must be a non-empty character string"
  )
})

test_that("fetch_all_dependencies rejects vector package_name", {
  expect_error(
    fetch_all_dependencies(c("pkg1", "pkg2")),
    "package_name must be a non-empty character string"
  )
})

test_that("fetch_all_dependencies accepts package_name with whitespace (trimmed)", {
  # Mock build_dependency_tree to avoid actual download
  mock_build_dependency_tree <- function(package_name, get_license = FALSE, max_level = 3, visited = character()) {
    return(list(version = "1.0.0"))
  }
  
  mockery::stub(fetch_all_dependencies, "build_dependency_tree", mock_build_dependency_tree)
  
  # Should work with whitespace (will be trimmed)
  result <- fetch_all_dependencies("  testpkg  ", max_level = 1)
  expect_type(result, "list")
})


# Validation tests for build_dependency_tree - now uses warnings instead of errors

test_that("build_dependency_tree warns on negative max_level", {
  expect_warning(
    build_dependency_tree("testpkg", max_level = -1),
    "max_level must be a non-negative integer"
  )
})

test_that("build_dependency_tree warns on fractional max_level", {
  # Mock extract_package_version to avoid "package not found" warnings
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    extract_package_version = mock_extract_package_version,
    {
      expect_warning(
        result <- build_dependency_tree("testpkg", max_level = 2.7),
        "max_level must be a non-negative integer"
      )
      expect_null(result)  # Should return NULL after warning
    }
  )
})

test_that("build_dependency_tree warns on NULL max_level", {
  expect_warning(
    build_dependency_tree("testpkg", max_level = NULL),
    "max_level must be a non-negative integer"
  )
})

test_that("build_dependency_tree accepts zero max_level", {
  # Mock download_and_parse_dependencies to avoid actual download
  # Now always returns a list with dependencies and license
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    return(list(
      dependencies = data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE),
      license = NA_character_
    ))
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      result <- build_dependency_tree("testpkg", max_level = 0)
      # Should return NULL because level (1) > max_level (0)
      expect_null(result)
    }
  )
})

test_that("build_dependency_tree warns on empty package_name", {
  expect_warning(
    build_dependency_tree(""),
    "package_name must be a non-empty character string"
  )
})

test_that("build_dependency_tree warns on NA package_name", {
  expect_warning(
    build_dependency_tree(NA_character_),
    "package_name must be a non-empty character string"
  )
})

test_that("build_dependency_tree warns on NULL package_name", {
  expect_warning(
    build_dependency_tree(NULL),
    "package_name must be a non-empty character string"
  )
})

test_that("build_dependency_tree accepts package_name with whitespace (trimmed)", {
  # Mock download_and_parse_dependencies to avoid actual download
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    return(data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE))
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      # Should work with whitespace (will be trimmed)
      result <- build_dependency_tree("  testpkg  ", max_level = 1)
      expect_type(result, "list")
      expect_true("version" %in% names(result))
    }
  )
})


# Cycle detection tests

test_that("build_dependency_tree handles circular dependencies (no infinite loop)", {
  # Mock a circular dependency: pkgA -> pkgB -> pkgA
  mock_download_and_parse_dependencies <- function(package_name, get_license = FALSE) {
    deps <- switch(package_name,
      "pkgA" = data.frame(package = "pkgB", type = "Imports", stringsAsFactors = FALSE),
      "pkgB" = data.frame(package = "pkgA", type = "Imports", stringsAsFactors = FALSE),
      data.frame(package = character(0), type = character(0), stringsAsFactors = FALSE)
    )
    return(deps)
  }
  
  mock_extract_package_version <- function(package_name) {
    return("1.0.0")
  }
  
  with_mocked_bindings(
    download_and_parse_dependencies = mock_download_and_parse_dependencies,
    extract_package_version = mock_extract_package_version,
    is_base = function(pkg) FALSE,
    {
      # With max_level = 2, pkgA (level 1) -> pkgB (level 2) -> pkgA (level 3, stopped)
      # Should not cause infinite loop due to max_level limit
      result <- build_dependency_tree("pkgA", max_level = 2)
      
      # Should have pkgB but pkgB should not have pkgA (stopped at max_level)
      expect_true("pkgB" %in% names(result))
      expect_true(is.list(result$pkgB))
      # pkgB should not have pkgA as a dependency (stopped at max_level)
      expect_false("pkgA" %in% names(result$pkgB))
    }
  )
})

#' Assess package
#'  
#' @description assess package for risk metrics
#' 
#' @param pkg_source_path - source path for install local
#' @param rcmdcheck_args - arguments for R Cmd check - these come from setup_rcmdcheck_args
#' @param covr_timeout - setting for covr time out
#'
#' @return list containing results - list containing metrics, covr, tm - trace matrix, and R CMD check
#' @keywords internal
assess_pkg <- function(
    pkg_source_path,
    rcmdcheck_args,
    covr_timeout = Inf
) {
  # record covr tests
  options(covr.record_tests = TRUE)
  
  # Input checking
  checkmate::assert_string(pkg_source_path)
  checkmate::assert_directory_exists(pkg_source_path)
  checkmate::assert_list(rcmdcheck_args)
  checkmate::assert_numeric(rcmdcheck_args$timeout)
  checkmate::anyInfinite(rcmdcheck_args$timeout)
  checkmate::check_character(rcmdcheck_args$args, pattern = "--no-manual")
  checkmate::check_character(rcmdcheck_args$args, pattern = "--ignore-vignettes")
  checkmate::check_character(rcmdcheck_args$args, pattern = "--no-vignettes")
  checkmate::check_character(rcmdcheck_args$args, pattern = "--as-cran")
  checkmate::check_character(rcmdcheck_args$build_args, pattern = "--no-build-vignettes|NULL")
  checkmate::assert_string(rcmdcheck_args$env)
  checkmate::check_logical(rcmdcheck_args$quiet)
  
  # Get package name and version
  pkg_desc <- get_pkg_desc(pkg_source_path, 
                                               fields = c("Package", 
                                                          "Version"))
  pkg_name <- pkg_desc$Package
  pkg_ver <- pkg_desc$Version
  pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)

  metadata <- get_risk_metadata()
  results <- create_empty_results(pkg_name, pkg_ver, pkg_source_path, metadata)
  doc_scores <- doc_riskmetric(pkg_name, pkg_ver, pkg_source_path)
  
  # doc data
  results$has_bug_reports_url <- doc_scores$has_bug_reports_url
  results$license <- doc_scores$license
  results$has_examples <- doc_scores$has_examples
  results$has_docs <- doc_scores$has_docs
  results$has_ex_docs_score <- doc_scores$has_ex_docs_score
  results$has_maintainer <- doc_scores$has_maintainer
  results$size_codebase <- doc_scores$size_codebase 
  results$has_news <- doc_scores$has_news
  results$has_source_control <- doc_scores$has_source_control
  results$has_vignettes <- doc_scores$has_vignettes
  results$has_website <- doc_scores$has_website
  results$news_current <- doc_scores$news_current
  results$export_help <- doc_scores$export_help 
  
  test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
  results$tests <- test_pkg_data
  
  covr_list <- run_covr_modes(pkg_source_path)
  
  # add total coverage to results
  results$covr <- covr_list$total_cov
  
  if (is.na(results$covr) | results$covr == 0L) {
    #  create empty traceability matrix
    tm_list <- create_empty_tm(pkg_name)
  } else {
    #  create traceability matrix
    tm_list <- create_traceability_matrix(pkg_name, 
                                     pkg_source_path,
                                     covr_list$res_cov) 
  }
  # run R Cmd check
  rcmdcheck_args$path <- pkg_source_path
  check_list <- run_rcmdcheck(pkg_source_path, rcmdcheck_args) # use tarball
  
  # add rcmd check score to results
  results$check <- check_list$check_score
  
  deps <- get_dependencies(pkg_source_path)
  
  # add suggested dependencies to results
  fallback_suggested_deps <- data.frame(
    source             = pkg_name,
    suggested_function = NA_character_,
    message            = "Error in checking suggested functions",
    where              = NA_character_,
    stringsAsFactors   = FALSE
  )
  
  results$suggested_deps <- tryCatch(
    {
      out <- check_suggested_exp_funcs(pkg_name, pkg_source_path, deps)
      # Normalize to a data.frame
      if (is.null(out)) {
        fallback_suggested_deps
      } else if (is.data.frame(out)) {
        out
      } else {
        # Coerce or fallback if the helper returns another type
        fallback_suggested_deps
      }
    },
    error = function(e) {
      message("An error occurred in checking suggested functions: ", conditionMessage(e))
      fallback_suggested_deps
    }
  )
  
  
  results$export_calc <- assess_exports(pkg_source_path)

  # dependencies
  results$dependencies <- risk.assessr::get_session_dependencies(deps)
  
  # author
  pkg_author <- get_pkg_author(pkg_name, pkg_source_path)
  results$author <- pkg_author
  
  # license
  pkg_license <- extract_license_from_description(pkg_source_path)
  results$license_name <- pkg_license
  
  # get host repo
  pkg_host <- get_host_package(pkg_name, pkg_ver, pkg_source_path)
  results$host <- pkg_host
  
  owner <- get_repo_owner(pkg_host$github_links, pkg_name)
  
  # get github Data
  github_data <- get_github_data(owner, pkg_name)
  results$github_data <- github_data

  download_list <- list("total_download" = get_cran_total_downloads(pkg_name),
                        "last_month_download" = get_cran_total_downloads(pkg_name, months=1)
  )
  
  results$download <- download_list
  
  # Get versions
  version_info <- list("all_versions" = NULL,
                       "last_version" =  NULL)  
  
  if (!is.null(pkg_host$cran_links)) {
    
    result_cran <- check_and_fetch_cran_package(pkg_name, pkg_ver)

    version_info <- list("all_versions" = result_cran$all_versions,
                          "last_version" =  result_cran$last_version)
    

    results$rev_deps <-  cran_revdep(pkg_name) 
        
  } else if (!is.null(pkg_host$bioconductor_links)) {
    
    html_content <- fetch_bioconductor_releases()
    release_data <- parse_bioconductor_releases(html_content)
    result_bio <- get_bioconductor_package_url(pkg_name, pkg_ver, release_data)
    
    all_versions <- result_bio$all_versions
    last_version <- result_bio$last_version
    bioconductor_version_package <- result_bio$bioconductor_version_package
    
    version_info <- list("all_versions" = all_versions,
                         "last_version" =  last_version,
                         "bioconductor_version_package" = bioconductor_version_package)
    
    
    results$rev_deps <- bioconductor_reverse_deps(pkg_name, version = bioconductor_version_package)
    
  }
  
  all_versions <- version_info$all_versions
  last_version <- version_info$last_version

  if (!is.null(all_versions)) {

    index <- which(sapply(all_versions, function(x) x$version == pkg_ver))
    
    if (length(index) == 0) {
      index <- length(all_versions)
    }
    
    current_date <- as.Date(all_versions[[index]]$date)
    last_date <- as.Date(last_version$date)
    
    difference_months <- length(seq(current_date, last_date, by = "month")) - 1
    version_info$difference_version_months <- difference_months
    
  } else {
    version_info$difference_version_months <- NA
  }
  
  results$version_info <- version_info
  
  # this allows for packages not on CRAN or bioconductor to be processed
  # Check if rev_deps is NULL or contains only empty strings
  if (is.null(results$rev_deps) || all(results$rev_deps == "")) {
    results$rev_deps <- 0
  }
  
  
  # # Replace empty string revdep_score with 0
  # if (results$revdep_score == "") {
  #   results$revdep_score <- 0
  # }
  
  results <- rapply( results, f=function(x) ifelse(is.nan(x),0,x), how="replace" )	  
  results <- rapply( results, f=function(x) ifelse(is.na(x),0,x), how="replace" )
  
  return(list(
              results = results,
              covr_list = covr_list,
              tm_list = tm_list,
              check_list = check_list,
              risk_analysis = get_risk_analysis(results)
              ))
}

test_that(
  "running assess_pkg for test package in tar file - no notes",
  {
    
    skip_on_cran()
    
    skip_if(
      .Platform$OS.type == "windows",
      "covr runs in a subprocess on Windows; mocking is not reliable under devtools::test()"
    )

    # ---- repo setup ----
    r <- getOption("repos")
    r["CRAN"] <- "http://cran.us.r-project.org"
    options(repos = r)
    
    # ---- copy test tarball ----
    dp_orig <- system.file(
      "test-data",
      "test.package.0001_0.1.0.tar.gz",
      package = "risk.assessr"
    )
    
    dp <- tempfile(fileext = ".tar.gz")
    file.copy(dp_orig, dp)
    
    withr::defer(unlink(dp), envir = parent.frame())
    
    # ---- covr mocks ---------------------------------------------------
    
    mock_covr_coverage <- function(path, type = "tests", ...) {
      
      line_cov <- data.frame(
        filename = c(
          "R/myfunction.R",
          "tests/testthat/test-myfunction.R"
        ),
        line = c(1L, 1L),
        value = c(1L, 1L),
        stringsAsFactors = FALSE
      )
      
      cov <- list(
        package = "test.package.0001",
        line_coverage = line_cov
      )
      
      class(cov) <- "coverage"
      cov
    }
    
    # stub *before* assess_pkg() runs
    mockery::stub(
      assess_pkg,
      "covr::package_coverage",
      mock_covr_coverage
    )
    
    mockery::stub(
      assess_pkg,
      "covr::report",
      function(...) invisible(NULL)
    )
    
    mockery::stub(
      assess_pkg,
      "covr::percent_coverage",
      function(x) 1
    )
    
    # ---- other external mocks ----------------------------------------
    
    
    # 1) Return a fully-populated covr_list from run_coverage()
    mock_run_coverage <- function(pkg_source_path, covr_timeout) {
      list(
        total_cov = 1.0,  # non-NA to bypass skip STF
        res_cov = list(
          name = "test.package.0001",
          coverage = list(
            filecoverage  = c(100), # non-NA
            totalcoverage = 100
          )
        )
      )
    }
    
    # 2) Stub run_coverage at the call-site inside run_covr_modes()
    mockery::stub(
      risk.assessr:::run_covr_modes,
      "run_coverage",
      mock_run_coverage
    )
    
    # 3) Ensure the "has_testthat" condition is TRUE
    mockery::stub(
      assess_pkg,
      "check_pkg_tests_and_snaps",
      function(pkg_source_path) {
        list(
          has_testthat   = TRUE,
          has_testit     = FALSE,
          has_snaps      = TRUE,
          n_golden_tests = 100,
          n_test_files   = 100
        )
      }
    )
    
    
    mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
      list(
        cran_links = "CRAN",
        github_links = NULL,
        bioconductor_links = NULL,
        internal_links = NULL
      )
    }
    
    mockery::stub(
      assess_pkg,
      "get_host_package",
      mock_get_host_package
    )
    
    mock_check_pkg_tests <- function(pkg_source_path) {
      list(
        has_testthat = TRUE,
        has_snaps = TRUE,
        n_golden_tests = 100,
        n_test_files = 100
      )
    }
    
    mockery::stub(
      assess_pkg,
      "check_pkg_tests_and_snaps",
      mock_check_pkg_tests
    )
    
    mock_check_and_fetch_cran_package <- function(pkg_name, pkg_ver) {
      list(
        all_versions = list(
          list(version = "0.1.0", date = "2021-01-01"),
          list(version = "0.9.0", date = "2022-01-01"),
          list(version = "1.0.0", date = "2023-01-01")
        ),
        last_version = list(
          version = "1.0.0",
          date = "2023-01-01"
        )
      )
    }
    
    mockery::stub(
      assess_pkg,
      "check_and_fetch_cran_package",
      mock_check_and_fetch_cran_package
    )
    
    mockery::stub(
      assess_pkg,
      "get_risk_analysis",
      function(pkg_name) list()
    )
    
    # ---- package setup ------------------------------------------------
    
    install_list <- set_up_pkg(dp)
    
    pkg_source_path <- install_list$pkg_source_path
    rcmdcheck_args <- install_list$rcmdcheck_args
    
    # --- after set_up_pkg(dp) ---
    
    # Discover package name from DESCRIPTION without extra dependencies
    pkg_name <- unname(read.dcf(file.path(pkg_source_path, "DESCRIPTION"), "Package")[1, 1])
    
    # Load the package *from source* so base::getNamespaceExports(pkg_name) works
    pkgload::load_all(
      pkg_source_path,
      quiet = TRUE,
      export_all = FALSE,        # don't attach non-exports
      helpers = FALSE,
      attach_testthat = FALSE
    )
    
    # Make sure we unload even on test failure
    withr::defer(
      try(pkgload::unload(pkg_name), silent = TRUE),
      envir = parent.frame()
    )
    
    withr::defer(
      unlink(pkg_source_path, recursive = TRUE, force = TRUE),
      envir = parent.frame()
    )
    
    # pretend install succeeded
    rcmdcheck_args$path <- pkg_source_path
    
    # ---- run assess_pkg -----------------------------------------------
    
    assess_package <- assess_pkg(pkg_source_path, rcmdcheck_args)
    
    # ---- assertions ---------------------------------------------------
    
    expect_identical(length(assess_package), 5L)
    expect_true(checkmate::check_class(assess_package, "list"))
    
    expect_identical(length(assess_package$results), 32L)
    
    expect_true(!is.na(assess_package$results$pkg_name))
    expect_true(!is.na(assess_package$results$pkg_version))
    expect_true(!is.na(assess_package$results$pkg_source_path))
    expect_true(!is.na(assess_package$results$date_time))
    
    expect_true(checkmate::test_numeric(assess_package$results$covr))
    expect_gte(assess_package$results$covr, 0.7)
    
    # ---- covr list ----------------------------------------------------
    
    expect_identical(length(assess_package$covr_list), 2L)
    expect_identical(
      length(assess_package$covr_list$res_cov$coverage),
      2L
    )
    
    # ---- TM coverage --------------------------------------------------
    
    expect_identical(length(assess_package$tm_list$tm), 9L)
    
    # high risk
    expect_true(is.list(assess_package$tm_list$coverage$high_risk))
    expect_identical(
      assess_package$tm_list$coverage$high_risk$pkg_name,
      "test.package.0001"
    )
    
    # medium risk
    expect_true(is.list(assess_package$tm_list$coverage$medium_risk))
    
    # low risk
    low_risk <- assess_package$tm_list$coverage$low_risk
    
    expect_equal(nrow(low_risk), 1L)
    expect_identical(low_risk$exported_function[1], "myfunction")
    expect_true(grepl("^R/", low_risk$code_script[1]))
    expect_equal(low_risk$coverage_percent[1], 100)
    
    # ---- check data ---------------------------------------------------
    
    expect_identical(length(assess_package$check_list), 2L)
    expect_identical(
      length(assess_package$check_list$res_check),
      21L
    )
    
    # ---- version delta -----------------------------------------------
    
    diff_months <-
      assess_package$results$version_info$difference_version_months
    
    expect_equal(diff_months, 24)

})



Sys.setenv("R_TESTS" = "")
library(testthat)

test_that("running assess_pkg for test package in tar file - no exports", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)

  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0005_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    return(list(cran_links = "CRAN", github_links = NULL, bioconductor_links = NULL, internal_links= NULL))
  }

  mockery::stub(assess_pkg, "get_host_package", mock_get_host_package)

  mock_unit_test <- function(pkg_source_path) {
    return(list(
      has_testthat = TRUE,
      has_snaps = TRUE,
      n_golden_tests = 100,
      n_test_files = 100
    ))
  }

  mockery::stub(assess_pkg, "check_pkg_tests_and_snaps", mock_unit_test)

  mock_check_and_fetch_cran_package <- function(pkg_name, pkg_ver) {
    return(list(
      all_versions = list(
        list(version = "0.1.0", date = "2023-01-01"), # actual package version
        list(version = "0.9.0", date = "2022-01-01"),
        list(version = "1.0.0", date = "2023-01-01")
      ),
      last_version = list(version = "1.0.0", date = "2023-01-01")
    ))
  }

  mockery::stub(assess_pkg, "check_and_fetch_cran_package", mock_check_and_fetch_cran_package)

  mock_get_risk_analysis <- function(pkg_name) {
    return(list())
  }

  mockery::stub(assess_pkg, "get_risk_analysis", mock_get_risk_analysis)


  # set up package
  install_list <- set_up_pkg(dp)

  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args

  # install package locally to ensure test works
  package_installed <- install_package_local(pkg_source_path)
  package_installed <- TRUE

  if (package_installed == TRUE ) {

    # ensure path is set to package source path
    rcmdcheck_args$path <- pkg_source_path
    assess_package <- assess_pkg(pkg_source_path, rcmdcheck_args)

    testthat::expect_identical(length(assess_package), 5L)
    
    testthat::expect_true(checkmate::check_class(assess_package, "list"))
    
    testthat::expect_identical(length(assess_package$results), 32L)
    
    testthat::expect_true(!is.na(assess_package$results$pkg_name))
    
    testthat::expect_true(!is.na(assess_package$results$pkg_version))
    
    testthat::expect_true(!is.na(assess_package$results$pkg_source_path))
    
    testthat::expect_true(!is.na(assess_package$results$date_time))
    
    testthat::expect_true(!is.na(assess_package$results$executor))
    
    testthat::expect_true(!is.na(assess_package$results$sysname))
    
    testthat::expect_true(!is.na(assess_package$results$version))
    
    testthat::expect_true(!is.na(assess_package$results$release))
    
    testthat::expect_true(!is.na(assess_package$results$machine))
    
    testthat::expect_true(!is.na(assess_package$results$comments))
    
    testthat::expect_null(assess_package$results$has_bug_reports_url)
    
    testthat::expect_true(!is.na(assess_package$results$suggested_deps$source))
    
    testthat::expect_true(identical(assess_package$results$suggested_deps$message,
                                    "No exported functions from Suggested packages in the DESCRIPTION file"))
    
    testthat::expect_true(!is.na(assess_package$results$license))
    
    testthat::expect_true(!is.na(assess_package$results$size_codebase))
    
    testthat::expect_true(checkmate::test_numeric(assess_package$results$size_codebase))
    
    testthat::expect_true(checkmate::test_numeric(assess_package$results$check))
    
    testthat::expect_gte(assess_package$results$check, 0)
    
    testthat::expect_true(checkmate::test_numeric(assess_package$results$covr))
    
    testthat::expect_gte(assess_package$results$covr, 0)
    
    testthat::expect_identical(length(assess_package$covr_list), 2L)
    
    testthat::expect_true(!is.na(assess_package$covr_list$res_cov$name))
    
    testthat::expect_identical(length(assess_package$covr_list$res_cov$coverage), 2L)
    
    testthat::expect_identical(length(assess_package$tm_list), 4L)
    
    testthat::expect_identical(length(assess_package$tm_list$tm), 0L)
    
    expect_identical(assess_package$tm_list$coverage$filecoverage, 0)
    
    expect_identical(assess_package$tm_list$coverage$totalcoverage, 0)
    
    testthat::expect_identical(length(assess_package$check_list), 2L)
    
    testthat::expect_identical(length(assess_package$check_list$res_check), 21L)
    
    testthat::expect_true(!is.na(assess_package$check_list$res_check$platform))
    
    testthat::expect_true(!is.na(assess_package$check_list$res_check$package))
    
    testthat::expect_identical(length(assess_package$check_list$res_check$test_output), 0L)
    
    testthat::expect_true(is.null(assess_package$check_list$res_check$test_output$testthat))
    
    testthat::expect_true(all(sapply(assess_package$check_list$res_check$session_info$platform,
                                     function(x) !is.null(x) && x != "")))
    diff_months <- assess_package$results$version_info$difference_version_months
    expect_equal(diff_months, 0)
  }
})


test_that("running assess_pkg for test package with Config/build/clean-inst-doc: false", {
  skip_on_cran()

  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    return(list(cran_links = "CRAN", github_links = NULL, bioconductor_links = NULL, internal_links= NULL))
  }

  mockery::stub(assess_pkg, "get_host_package", mock_get_host_package)

  mock_unit_test <- function(pkg_source_path) {
    return(list(
      has_testthat = TRUE,
      has_snaps = TRUE,
      n_golden_tests = 100,
      n_test_files = 100
    ))
  }

  mockery::stub(assess_pkg, "check_pkg_tests_and_snaps", mock_unit_test)

  mock_check_and_fetch_cran_package <- function(pkg_name, pkg_ver) {
    return(list(
      all_versions = list(
        list(version = "0.1.0", date = "2023-01-01"), # actual package version
        list(version = "0.9.0", date = "2022-01-01"),
        list(version = "1.0.0", date = "2023-01-01")
      ),
      last_version = list(version = "1.0.0", date = "2023-01-01")
    ))
  }

  mockery::stub(assess_pkg, "check_and_fetch_cran_package", mock_check_and_fetch_cran_package)

  mock_get_risk_analysis <- function(pkg_name) {
    return(list())
  }

  mockery::stub(assess_pkg, "get_risk_analysis", mock_get_risk_analysis)


  dp <- system.file("test-data", "test.package.0005_0.1.0.tar.gz",
                    package = "risk.assessr")

  # Check if the file exists before attempting to download
  if (!file.exists(dp)) {
    stop("The tar file does not exist at the specified path.")
  }

  # Create a temporary file to store the downloaded package
  file_name <- basename(dp) # Use the base name for temporary file
  temp_file <- file.path(tempdir(), file_name)

  # Copy the file to the temporary file instead of downloading it
  file.copy(dp, temp_file, overwrite = TRUE)

  # Verify that the copy was successful
  if (!file.exists(temp_file)) {
    stop("File copy failed: temporary file not found.")
  }

  # Run the function to modify the DESCRIPTION file
  modified_tar_file <- modify_description_file(temp_file)

  # Set up package
  install_list <- set_up_pkg(dp)

  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args

  if (package_installed == TRUE) {

    assess_package <- assess_pkg(pkg_source_path, rcmdcheck_args)

    testthat::expect_true(checkmate::check_class(assess_package, "list"))
  }
})


test_that("running assess_pkg for test package fail suggest", {
  skip_on_cran()
  
  # Robust, version-independent Windows skip (covr subprocess + devtools::test)
  skip_if(
    .Platform$OS.type == "windows",
    "covr runs in a subprocess on Windows; mocking is not reliable under devtools::test()."
  )
  
  # ---- repo setup ----
  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)
  
  # ---- copy test tarball ----
  dp_orig <- system.file(
    "test-data",
    "test.package.0001_0.1.0.tar.gz",
    package = "risk.assessr"
  )
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  withr::defer(unlink(dp), envir = parent.frame())
  
  # ---- covr path control: force STF and prevent fallback ----
  # 1) Mock run_coverage() at its call site inside run_covr_modes()
  mock_run_coverage <- function(pkg_source_path, covr_timeout) {
    list(
      total_cov = 1.0,  # MUST be non-NA to prevent fallback
      res_cov = list(
        name = "test.package.0001",
        coverage = list(
          filecoverage  = c(100),  # MUST be non-NA to prevent fallback
          totalcoverage = 100
        )
      )
    )
  }
  mockery::stub(
    risk.assessr:::run_covr_modes,
    "run_coverage",
    mock_run_coverage
  )
  
  # 2) Optional tripwires: if any direct covr path is still hit, fail fast
  tripwire <- function(...) stop("covr must not be called in this unit test", call. = FALSE)
  if (exists("create_covr_list_no_skip", envir = asNamespace("risk.assessr"), inherits = FALSE)) {
    mockery::stub(risk.assessr:::create_covr_list_no_skip, "covr::package_coverage", tripwire)
  }
  if (exists("run_covr_skip_stf", envir = asNamespace("risk.assessr"), inherits = FALSE)) {
    mockery::stub(risk.assessr:::run_covr_skip_stf, "covr::package_coverage", tripwire)
  }
  if (exists("run_covr_skip_nstf", envir = asNamespace("risk.assessr"), inherits = FALSE)) {
    mockery::stub(risk.assessr:::run_covr_skip_nstf, "covr::package_coverage", tripwire)
  }
  
  # ---- other external mocks ----
  mockery::stub(
    assess_pkg,
    "get_host_package",
    function(pkg_name, pkg_ver, pkg_source_path) {
      list(cran_links = "CRAN", github_links = NULL, bioconductor_links = NULL, internal_links = NULL)
    }
  )
  
  # Ensure STF branch is selected
  mockery::stub(
    assess_pkg,
    "check_pkg_tests_and_snaps",
    function(pkg_source_path) {
      list(
        has_testthat = TRUE,
        has_testit   = FALSE,
        has_snaps    = TRUE,
        n_golden_tests = 100,
        n_test_files   = 100
      )
    }
  )
  
  mockery::stub(
    assess_pkg,
    "check_and_fetch_cran_package",
    function(pkg_name, pkg_ver) {
      list(
        all_versions = list(
          list(version = "0.1.0", date = "2023-01-01"), # actual package version
          list(version = "0.9.0", date = "2022-01-01"),
          list(version = "1.0.0", date = "2023-01-01")
        ),
        last_version = list(version = "1.0.0", date = "2023-01-01")
      )
    }
  )
  
  mockery::stub(
    assess_pkg,
    "get_risk_analysis",
    function(pkg_name) list()
  )
  
   # stub inside assess_pkg() because that's where the call appears in the function body.
  mockery::stub(
    assess_pkg,
    "check_suggested_exp_funcs",
    function(...) stop("forced failure")
  )
  
  # ---- package setup ----
  install_list <- set_up_pkg(dp)
  
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args  <- install_list$rcmdcheck_args
  
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE), envir = parent.frame())
  
  # Install locally if your pipeline needs it (you had this)
  package_installed <- install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (isTRUE(package_installed)) {
    rcmdcheck_args$path <- pkg_source_path
    assess_package <- assess_pkg(pkg_source_path, rcmdcheck_args)
    suggested_deps <- assess_package$results$suggested_deps
  }
  
  testthat::expect_identical(length(suggested_deps), 4L)
  
  expected <- c("source", "suggested_function", "message", "where")
  
  # Same number of columns
  testthat::expect_identical(ncol(suggested_deps), length(expected))
  
  expect_identical(suggested_deps$source,
    "test.package.0001"
  )
  
  expect_identical(suggested_deps$message,
    "Error in checking suggested functions"
  )
  
})

test_that("assess_pkg handles errors in check_suggested_exp_funcs correctly", {
  skip_on_cran()
  
  # Robust Windows skip: covr runs in a subprocess on Windows and mocks won't cross it reliably
  skip_if(
    .Platform$OS.type == "windows",
    "covr runs in a subprocess on Windows; mocking is not reliable under devtools::test()."
  )
  
  # ---- CRAN repo setup (if your helpers install deps) ----
  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)
  
  # ---- copy test tarball ----
  dp_orig <- system.file(
    "test-data",
    "test.package.0001_0.1.0.tar.gz",
    package = "risk.assessr"
  )
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  withr::defer(unlink(dp), envir = parent.frame())
  
  # We'll collect pkg_source_path for cleanup once set_up_pkg() runs
  # (we declare it here only so the defer below has a symbol)
  pkg_source_path <- NULL
  withr::defer(
    if (!is.null(pkg_source_path)) unlink(pkg_source_path, recursive = TRUE, force = TRUE),
    envir = parent.frame()
  )
  
  # ---- Force STF path and prevent any fallback to skip-STF ----
  # run_covr_modes() calls run_coverage(pkg_source_path, covr_timeout)
  # Return non-NA values to ensure the fallback condition is FALSE.
  mock_run_coverage <- function(pkg_source_path, covr_timeout) {
    list(
      total_cov = 1.0,  # non-NA to bypass fallback
      res_cov = list(
        name = "test.package.0001",
        coverage = list(
          filecoverage  = c(100), # non-NA to bypass fallback
          totalcoverage = 100
        )
      )
    )
  }
  mockery::stub(
    risk.assessr:::run_covr_modes,
    "run_coverage",
    mock_run_coverage
  )
  
  # Tripwires: if any direct covr call is still reached, fail fast
  tripwire <- function(...) stop("covr must not be called in this unit test", call. = FALSE)
  if (exists("create_covr_list_no_skip", envir = asNamespace("risk.assessr"), inherits = FALSE)) {
    mockery::stub(risk.assessr:::create_covr_list_no_skip, "covr::package_coverage", tripwire)
  }
  if (exists("run_covr_skip_stf", envir = asNamespace("risk.assessr"), inherits = FALSE)) {
    mockery::stub(risk.assessr:::run_covr_skip_stf, "covr::package_coverage", tripwire)
  }
  if (exists("run_covr_skip_nstf", envir = asNamespace("risk.assessr"), inherits = FALSE)) {
    mockery::stub(risk.assessr:::run_covr_skip_nstf, "covr::package_coverage", tripwire)
  }
  
  # ---- Other external mocks used by assess_pkg() ----
  mockery::stub(
    assess_pkg,
    "get_host_package",
    function(pkg_name, pkg_ver, pkg_source_path) {
      list(cran_links = "CRAN", github_links = NULL, bioconductor_links = NULL, internal_links = NULL)
    }
  )
  
  # Ensure STF branch is picked
  mockery::stub(
    assess_pkg,
    "check_pkg_tests_and_snaps",
    function(pkg_source_path) {
      list(
        has_testthat   = TRUE,
        has_testit     = FALSE,
        has_snaps      = TRUE,
        n_golden_tests = 100,
        n_test_files   = 100
      )
    }
  )
  
  mockery::stub(
    assess_pkg,
    "check_and_fetch_cran_package",
    function(pkg_name, pkg_ver) {
      list(
        all_versions = list(
          list(version = "0.1.0", date = "2023-01-01"), # actual package version
          list(version = "0.9.0", date = "2022-01-01"),
          list(version = "1.0.0", date = "2023-01-01")
        ),
        last_version = list(version = "1.0.0", date = "2023-01-01")
      )
    }
  )
  
  mockery::stub(
    assess_pkg,
    "get_risk_analysis",
    function(pkg_name) list()
  )
  
  # ---- Key fix: stub the *caller* of check_suggested_exp_funcs, i.e., inside assess_pkg() ----
  mockery::stub(
    assess_pkg,
    "check_suggested_exp_funcs",
    function(...) stop("Mock error")  # your test is specifically about error handling
  )
  
  # ---- Prepare the test package ----
  install_list <- set_up_pkg(dp)
  
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args  <- install_list$rcmdcheck_args
  
  # Discover package name from DESCRIPTION without extra dependencies
  pkg_name <- unname(read.dcf(file.path(pkg_source_path, "DESCRIPTION"), "Package")[1, 1])
  
  # Load the package *from source* so base::getNamespaceExports(pkg_name) works
  pkgload::load_all(
    pkg_source_path,
    quiet = TRUE,
    export_all = FALSE,        # don't attach non-exports
    helpers = FALSE,
    attach_testthat = FALSE
  )
  
  # Make sure we unload even on test failure
  withr::defer(
    try(pkgload::unload(pkg_name), silent = TRUE),
    envir = parent.frame()
  )
  
  # If your pipeline actually needs an install (some helper may assume it), keep it
  package_installed <- install_package_local(pkg_source_path)
  if (isTRUE(package_installed)) {
    rcmdcheck_args$path <- pkg_source_path
    
    # ---- Execute ----
    assess_package <- assess_pkg(pkg_source_path, rcmdcheck_args)
    
    # ---- Assertions ----
    expect_true("suggested_deps" %in% names(assess_package$results))
    
    suggested_deps <- assess_package$results$suggested_deps
    expect_equal(nrow(suggested_deps), 1)
    expect_equal(suggested_deps$source, "test.package.0001")
    expect_equal(suggested_deps$suggested_function, 0)
    
    # If your intended behavior on error is to fall back to the “no exported functions…” row:
    expect_equal(
      suggested_deps$message,
      "Error in checking suggested functions"
    )
    
    # The covr value should be 1 based on our mock_run_coverage()
    expect_true("covr" %in% names(assess_package$results))
    expect_equal(assess_package$results$covr, 1)
  } else {
    skip("Local install of test package failed (install_package_local returned FALSE).")
  }
})




test_that("assess_pkg handles error in check_suggested_exp_funcs via withCallingHandlers", {
 
  # Create a real temporary directory and DESCRIPTION file
  pkg_source_path <- withr::local_tempdir()
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  writeLines(c(
    "Package: mockpkg",
    "Version: 0.1.0",
    "Title: Mock Package",
    "Description: A mock package for testing.",
    "Authors@R: c(person(given = \"Mock\", family = \"Author\", role = c(\"aut\", \"cre\")))",
    "License: MIT"
  ), desc_path)
  
  # Stub get_pkg_desc to return mock name/version
  mockery::stub(assess_pkg, "get_pkg_desc", function(path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  })
  
  # Stub all other required functions to return minimal valid data
  mockery::stub(assess_pkg, "get_risk_metadata", function() list())
  mockery::stub(assess_pkg, "create_empty_results", function(...) list(suggested_deps = NULL))
  mockery::stub(assess_pkg, "doc_riskmetric", function(...) list(
    has_bug_reports_url = TRUE, license = TRUE, has_examples = TRUE,
    has_maintainer = TRUE, size_codebase = 10, has_news = TRUE,
    has_source_control = TRUE, has_vignettes = TRUE, has_website = TRUE,
    news_current = TRUE, export_help = TRUE
  ))
  mockery::stub(assess_pkg, "check_pkg_tests_and_snaps", function(...) list(has_testthat = FALSE, has_testit = FALSE))
  mockery::stub(assess_pkg, "run_rcmdcheck", function(...) list(check_score = 1))
  mockery::stub(assess_pkg, "get_dependencies", function(...) list())
  mockery::stub(assess_pkg, "create_empty_tm", function(...) list())
  
  # Stub check_suggested_exp_funcs to throw an error caught by withCallingHandlers
  mockery::stub(assess_pkg, "check_suggested_exp_funcs", function(...) stop("Simulated error in inner handler"))
  
  rcmdcheck_args <- list(timeout = Inf, args = c("--no-manual"), build_args = NULL, env = "mockenv", quiet = TRUE)
  
  expect_message(
    assess_pkg(pkg_source_path, rcmdcheck_args),
    "No recognised standard or non-standard testing configuration"
  )
})

test_that("assess_pkg handles error via outer tryCatch block", {
 
  # Create a real temporary directory and DESCRIPTION file
  pkg_source_path <- withr::local_tempdir()
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  writeLines(c(
    "Package: mockpkg",
    "Version: 0.1.0",
    "Title: Mock Package",
    "Description: A mock package for testing.",
    "Authors@R: c(person(given = \"Mock\", family = \"Author\", role = c(\"aut\", \"cre\")))",
    "License: MIT"
  ), desc_path)
  
  # Stub get_pkg_desc to return mock name/version
  mockery::stub(assess_pkg, "get_pkg_desc", function(path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  })
  
  # Stub all other required functions to return minimal valid data
  mockery::stub(assess_pkg, "get_risk_metadata", function() list())
  mockery::stub(assess_pkg, "create_empty_results", function(...) list(suggested_deps = NULL))
  mockery::stub(assess_pkg, "doc_riskmetric", function(...) list(
    has_bug_reports_url = TRUE, license = TRUE, has_examples = TRUE,
    has_maintainer = TRUE, size_codebase = 10, has_news = TRUE,
    has_source_control = TRUE, has_vignettes = TRUE, has_website = TRUE,
    news_current = TRUE, export_help = TRUE
  ))
  mockery::stub(assess_pkg, "check_pkg_tests_and_snaps", function(...) list(has_testthat = FALSE, has_testit = FALSE))
  mockery::stub(assess_pkg, "run_rcmdcheck", function(...) list(check_score = 1))
  mockery::stub(assess_pkg, "get_dependencies", function(...) list())
  mockery::stub(assess_pkg, "create_empty_tm", function(...) list())
  
  # Stub withCallingHandlers to simulate an error that bypasses inner handler
  mockery::stub(
    assess_pkg,
    "check_suggested_exp_funcs",
    function(...) stop("Mock error")
  )
  
  
  rcmdcheck_args <- list(timeout = Inf, args = c("--no-manual"), build_args = NULL, env = "mockenv", quiet = TRUE)
  
  expect_message(
    assess_pkg(pkg_source_path, rcmdcheck_args),
    "An error occurred in checking suggested functions: Mock error"
  )
})


test_that("assess_pkg follows Bioconductor path and records version/revdeps", {
  skip_on_cran()
  
  # Create a minimal real pkg structure
  pkg_source_path <- withr::local_tempdir()
  writeLines(c(
    "Package: mockbio",
    "Version: 1.0.0",                     # matches an entry below → difference months = 0
    "Title: Mock BioPkg",
    "Description: A mock package.",
    "Authors@R: person('A','B',role=c('aut','cre'))",
    "License: MIT"
  ), file.path(pkg_source_path, "DESCRIPTION"))
  dir.create(file.path(pkg_source_path, "R"), showWarnings = FALSE)
  writeLines("", file.path(pkg_source_path, "NAMESPACE"))
  
  # Minimal rcmdcheck args
  rcmdcheck_args <- list(timeout = Inf, args = c("--no-manual"), build_args = NULL, env = "mockenv", quiet = TRUE)
  
  # --- Stubs to force the Bioconductor branch and keep runtime tiny ---
  mockery::stub(assess_pkg, "get_risk_metadata", function() list())
  mockery::stub(assess_pkg, "create_empty_results", function(...) {
    list(
      pkg_name = "mockbio", pkg_version = "1.0.0", pkg_source_path = pkg_source_path,
      date_time = Sys.time(), executor = "test", sysname = "Windows", version = "R",
      release = "0", machine = "x86", comments = "", suggested_deps = NULL
    )
  })
  mockery::stub(assess_pkg, "doc_riskmetric", function(...) list(
    has_bug_reports_url = TRUE, license = "MIT", has_examples = TRUE,
    has_maintainer = TRUE, size_codebase = 10, has_news = TRUE,
    has_source_control = TRUE, has_vignettes = TRUE, has_website = TRUE,
    news_current = TRUE, export_help = TRUE
  ))
  mockery::stub(assess_pkg, "check_pkg_tests_and_snaps", function(...) list(has_testthat = FALSE, has_testit = FALSE))
  mockery::stub(assess_pkg, "run_rcmdcheck", function(...) list(check_score = 1))
  mockery::stub(assess_pkg, "get_dependencies", function(...) list())
  mockery::stub(assess_pkg, "create_empty_tm", function(...) list())
  mockery::stub(assess_pkg, "get_pkg_author", function(...) list(maintainer = "A B", funder = NA, authors = "A B"))
  mockery::stub(assess_pkg, "extract_license_from_description", function(...) "MIT")
  mockery::stub(assess_pkg, "get_host_package", function(pkg, ver, path) {
    list(cran_links = NULL, github_links = NULL, bioconductor_links = "bioc", internal_links = NULL)
  })
  mockery::stub(assess_pkg, "get_repo_owner", function(...) "owner")
  mockery::stub(assess_pkg, "get_github_data", function(...) list(created_at = NA, stars = 0, forks = 0, date = NA, recent_commits_count = 0))
  mockery::stub(assess_pkg, "get_cran_total_downloads", function(...) 0)
  # Bioconductor-specific stubs
  mockery::stub(assess_pkg, "fetch_bioconductor_releases", function() "<html/>")
  mockery::stub(assess_pkg, "parse_bioconductor_releases", function(html) list())  # not used by our next stub
  mockery::stub(assess_pkg, "get_bioconductor_package_url", function(pkg, ver, rel) {
    list(
      all_versions = list(
        list(version = "0.9.0", date = "2022-01-01"),
        list(version = "1.0.0", date = "2023-01-01")
      ),
      last_version = list(version = "1.0.0", date = "2023-01-01"),
      bioconductor_version_package = "3.17"
    )
  })
  mockery::stub(assess_pkg, "bioconductor_reverse_deps", function(pkg, version) c("pkgA", "pkgB"))
  mockery::stub(assess_pkg, "risk.assessr::get_session_dependencies", function(...) list())
  mockery::stub(assess_pkg, "get_risk_analysis", function(...) list())
  
  res <- assess_pkg(pkg_source_path, rcmdcheck_args)
  
  expect_true(is.list(res))
  expect_true("results" %in% names(res))
  expect_true("version_info" %in% names(res$results))
  expect_equal(res$results$version_info$bioconductor_version_package, "3.17")
  expect_equal(res$results$version_info$last_version$version, "1.0.0")
  # Version in DESCRIPTION == "1.0.0" → index matches → difference months 0
  expect_equal(res$results$version_info$difference_version_months, 0)
  # Revdeps populated from Bioconductor path:
  expect_equal(res$results$rev_deps, c("pkgA", "pkgB"))
})


test_that("assess_pkg Bioconductor path coerces empty reverse deps to 0", {
  skip_on_cran()
  
  pkg_source_path <- withr::local_tempdir()
  writeLines(c(
    "Package: mockbioempty",
    "Version: 1.0.0",
    "Title: Mock BioPkg Empty",
    "Description: A mock package.",
    "Authors@R: person('A','B',role=c('aut','cre'))",
    "License: MIT"
  ), file.path(pkg_source_path, "DESCRIPTION"))
  dir.create(file.path(pkg_source_path, "R"), showWarnings = FALSE)
  writeLines("", file.path(pkg_source_path, "NAMESPACE"))
  
  rcmdcheck_args <- list(timeout = Inf, args = c("--no-manual"), build_args = NULL, env = "mockenv", quiet = TRUE)
  
  mockery::stub(assess_pkg, "get_risk_metadata", function() list())
  mockery::stub(assess_pkg, "create_empty_results", function(...) list(suggested_deps = NULL))
  mockery::stub(assess_pkg, "doc_riskmetric", function(...) list(
    has_bug_reports_url = TRUE, license = "MIT", has_examples = TRUE,
    has_maintainer = TRUE, size_codebase = 10, has_news = TRUE,
    has_source_control = TRUE, has_vignettes = TRUE, has_website = TRUE,
    news_current = TRUE, export_help = TRUE
  ))
  mockery::stub(assess_pkg, "check_pkg_tests_and_snaps", function(...) list(has_testthat = FALSE, has_testit = FALSE))
  mockery::stub(assess_pkg, "run_rcmdcheck", function(...) list(check_score = 1))
  mockery::stub(assess_pkg, "get_dependencies", function(...) list())
  mockery::stub(assess_pkg, "create_empty_tm", function(...) list())
  mockery::stub(assess_pkg, "get_host_package", function(pkg, ver, path) {
    list(cran_links = NULL, github_links = NULL, bioconductor_links = "bioc", internal_links = NULL)
  })
  mockery::stub(assess_pkg, "fetch_bioconductor_releases", function() "<html/>")
  mockery::stub(assess_pkg, "parse_bioconductor_releases", function(html) list())
  mockery::stub(assess_pkg, "get_bioconductor_package_url", function(pkg, ver, rel) {
    list(
      all_versions = list(
        list(version = "1.0.0", date = "2023-01-01")
      ),
      last_version = list(version = "1.0.0", date = "2023-01-01"),
      bioconductor_version_package = "3.18"
    )
  })
  # Return empty to trigger the post-processing fallback to 0
  mockery::stub(assess_pkg, "bioconductor_reverse_deps", function(pkg, version) character(0))
  mockery::stub(assess_pkg, "risk.assessr::get_session_dependencies", function(...) list())
  mockery::stub(assess_pkg, "get_risk_analysis", function(...) list())
  
  res <- assess_pkg(pkg_source_path, rcmdcheck_args)
  
  expect_true(is.list(res))
  expect_true("results" %in% names(res))
  expect_equal(res$results$version_info$bioconductor_version_package, "3.18")
  expect_equal(res$results$version_info$difference_version_months, 0)
  # Empty reverse deps must be coerced to numeric 0
  expect_identical(res$results$rev_deps, 0)
})

#' Assess package for risk metrics
#'
#' Assess an R package for risk metrics. Use exactly one of:
#' * `path` – path to a local .tar.gz archive, or
#' * `package` – package name to fetch (optionally with `version`), or
#' * neither – opens an interactive file chooser.
#'
#' @param path Path to a local .tar.gz package archive. Use only when not
#'   providing `package`.
#' @param package Package name to fetch from CRAN, Bioconductor, or internal
#'   repos. Use only when not providing `path`.
#' @param version (optional) Package version when using `package`. Default `NA`
#'   uses the latest available.
#' @param repos Repository URLs when fetching by `package`. Default is
#'   `getOption("repos")`.
#'
#' @return List containing results (metrics, covr, trace matrix, R CMD check),
#'   or `NULL` if the package cannot be assessed.
#'
#' @examples
#' \dontrun{
#' # Local package (path or file chooser)
#' results <- risk_assess_pkg()
#' results <- risk_assess_pkg(path = "path/to/package.tar.gz")
#'
#' # Package by name from repositories
#' results <- risk_assess_pkg(package = "here")
#' results <- risk_assess_pkg(package = "here", version = "1.0.1")
#' results <- risk_assess_pkg(package = "here", repos = "https://cloud.r-project.org")
#' }
#'
#' @export
risk_assess_pkg <- function(path = NULL, package = NULL, version = NA,
                           repos = getOption("repos")) {
  if (!is.null(path) && !is.null(package) && nzchar(package)) {
    warning("Provide either 'path' or 'package', not both. Returning NULL.")
    return(NULL)
  }

  temp_cleanup <- NULL
  modified_cleanup <- NULL
  old_repos <- NULL

  if (!is.null(path)) {
    if (!file.exists(path)) { warning("Path does not exist: ", path, ". Returning NULL."); return(NULL) }
    old_repos <- getOption("repos")
    r <- old_repos
    r["CRAN"] <- "https://cran.us.r-project.org"
    options(repos = r)
    tar_path <- path
  } else if (!is.null(package) && is.character(package) && nzchar(package)) {
    v <- if (is.null(version) || (length(version) == 1L && is.na(version))) NULL else version
    temp_cleanup <- tryCatch(
      get_package_tarfile(package_name = package, version = v, repos = repos),
      error = function(e) {
        warning("Failed to download package '", package, "': ", conditionMessage(e), ". Returning NULL.")
        NULL
      }
    )
    if (is.null(temp_cleanup)) return(NULL)
    tar_path <- temp_cleanup
  } else {
    chosen <- file.choose()
    if (!file.exists(chosen)) { warning("Chosen file does not exist. Returning NULL."); return(NULL) }
    old_repos <- getOption("repos")
    r <- old_repos
    r["CRAN"] <- "https://cran.us.r-project.org"
    options(repos = r)
    tar_path <- chosen
  }

  oldwd <- getwd()
  on.exit(setwd(oldwd), add = TRUE)
  on.exit({
    for (f in c(temp_cleanup, modified_cleanup)) {
      if (!is.null(f) && file.exists(f)) unlink(f)
    }
    if (!is.null(old_repos)) options(repos = old_repos)
  }, add = TRUE)

  modified_tar <- modify_description_file(tar_path)
  if (!identical(modified_tar, tar_path)) modified_cleanup <- modified_tar

  install_list <- set_up_pkg(modified_tar)
  if (!isTRUE(install_list$package_installed) || !nzchar(install_list$pkg_source_path)) {
    message("Failed to unpack package.")
    return(NULL)
  }

  if (!isTRUE(install_package_local(install_list$pkg_source_path))) {
    message("Package installation failed.")
    return(NULL)
  }

  rcmdcheck_args <- install_list$rcmdcheck_args
  rcmdcheck_args$path <- install_list$pkg_source_path
  assess_pkg(pkg_source_path = install_list$pkg_source_path, rcmdcheck_args = rcmdcheck_args)
}

# Helper: copy test tarball to temp file; returns path or "" if not found
get_test_tarball_path <- function() {
  dp_orig <- system.file("test-data", "test.package.0001_0.1.0.tar.gz",
                        package = "risk.assessr")
  if (nchar(dp_orig) == 0L) return("")
  dp <- tempfile(fileext = ".tar.gz")
  if (!file.copy(dp_orig, dp)) return("")
  dp
}

test_that("risk_assess_pkg works with path to local tarball", {
  skip_on_cran()
  skip_if(
    .Platform$OS.type == "windows",
    "covr runs in a subprocess on Windows; mocking is not reliable under devtools::test()"
  )
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data test.package.0001_0.1.0.tar.gz not found")
  withr::defer(unlink(dp), envir = parent.frame())

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  risk_assess_package <- risk_assess_pkg(path = dp)
  
  testthat::expect_identical(length(risk_assess_package), 5L)
  
  testthat::expect_true(checkmate::check_class(risk_assess_package, "list"))
  
  testthat::expect_identical(length(risk_assess_package$results), 32L)
  
  testthat::expect_true(!is.na(risk_assess_package$results$pkg_name))
  
  testthat::expect_true(!is.na(risk_assess_package$results$pkg_version))
  
  testthat::expect_true(!is.na(risk_assess_package$results$pkg_source_path))
  
  testthat::expect_true(!is.na(risk_assess_package$results$date_time))
  
  testthat::expect_true(!is.na(risk_assess_package$results$executor))
  
  testthat::expect_true(!is.na(risk_assess_package$results$sysname))
  
  testthat::expect_true(!is.na(risk_assess_package$results$version))
  
  testthat::expect_true(!is.na(risk_assess_package$results$release))
  
  testthat::expect_true(!is.na(risk_assess_package$results$machine))
  
  testthat::expect_true(!is.na(risk_assess_package$results$comments))
  
  testthat::expect_null(risk_assess_package$results$has_bug_reports_url)
    
  testthat::expect_true(!is.na(risk_assess_package$results$license))
  
  testthat::expect_true(!is.na(risk_assess_package$results$size_codebase))
  
  testthat::expect_true(checkmate::test_numeric(risk_assess_package$results$size_codebase))
  
  testthat::expect_true(checkmate::test_numeric(risk_assess_package$results$check))
  
  testthat::expect_gte(risk_assess_package$results$check, 0)
  
  testthat::expect_true(checkmate::test_numeric(risk_assess_package$results$covr))
  
  testthat::expect_gte(risk_assess_package$results$covr, 0.7)
  
  testthat::expect_identical(length(risk_assess_package$covr_list), 2L)
  
  testthat::expect_true(!is.na(risk_assess_package$covr_list$res_cov$name))
  
  testthat::expect_identical(length(risk_assess_package$covr_list$res_cov$coverage), 2L)
  
  testthat::expect_identical(length(risk_assess_package$tm), 3L)
  
  testthat::expect_identical(length(risk_assess_package$tm_list$tm), 9L)
  
  testthat::expect_true(!is.na(risk_assess_package$tm_list$tm$exported_function))
  
  testthat::expect_true(!is.na(risk_assess_package$tm_list$tm$code_script))
  
  testthat::expect_true(!is.na(risk_assess_package$tm_list$tm$documentation))
  
  testthat::expect_true(!is.na(risk_assess_package$tm_list$tm$coverage_percent))
  
  # high risk tm tests
  expect_true(is.list(risk_assess_package$tm_list$coverage$high_risk))
  expect_identical(risk_assess_package$tm_list$coverage$high_risk$pkg_name, "test.package.0001")
  expect_true(is.list(risk_assess_package$tm_list$coverage$high_risk$coverage))
  expect_identical(risk_assess_package$tm_list$coverage$high_risk$coverage$filecoverage, 0)
  expect_identical(risk_assess_package$tm_list$coverage$high_risk$coverage$totalcoverage, 0)
  expect_true(is.na(risk_assess_package$tm_list$coverage$high_risk$errors))
  expect_true(is.na(risk_assess_package$tm_list$coverage$high_risk$notes))
  
  # medium risk tm tests
  expect_true(is.list(risk_assess_package$tm_list$coverage$medium_risk))
  expect_identical(risk_assess_package$tm_list$coverage$medium_risk$pkg_name, "test.package.0001")
  expect_true(is.list(risk_assess_package$tm_list$coverage$medium_risk$coverage))
  expect_identical(risk_assess_package$tm_list$coverage$medium_risk$coverage$filecoverage, 0)
  expect_identical(risk_assess_package$tm_list$coverage$medium_risk$coverage$totalcoverage, 0)
  expect_true(is.na(risk_assess_package$tm_list$coverage$medium_risk$errors))
  expect_true(is.na(risk_assess_package$tm_list$coverage$medium_risk$notes))
  
  # low risk tm tests
  testthat::expect_equal(nrow(risk_assess_package$tm_list$coverage$low_risk), 1)
  testthat::expect_true(all(c("exported_function", "class", "function_type", "function_body",
                              "where", "code_script", "documentation", "description",
                              "coverage_percent") %in% names(risk_assess_package$tm_list$coverage$low_risk)))
  
  testthat::expect_identical(risk_assess_package$tm_list$coverage$low_risk$exported_function[1], "myfunction")
  testthat::expect_true(is.na(risk_assess_package$tm_list$coverage$low_risk$class[1]))
  testthat::expect_identical(risk_assess_package$tm_list$coverage$low_risk$function_type[1], "regular function")
  testthat::expect_true(!is.na(risk_assess_package$tm_list$coverage$low_risk$function_body[1]))
  testthat::expect_true(grepl("^test\\.package", risk_assess_package$tm_list$coverage$low_risk$where[1]))
  testthat::expect_true(grepl("^R/", risk_assess_package$tm_list$coverage$low_risk$code_script[1]))
  testthat::expect_identical(risk_assess_package$tm_list$coverage$low_risk$documentation[1], "myfunction.Rd")
  testthat::expect_type(risk_assess_package$tm_list$coverage$low_risk$coverage_percent[1], "double")

  testthat::expect_identical(length(risk_assess_package$check_list), 2L)
  
  testthat::expect_identical(length(risk_assess_package$check_list$res_check), 21L)
  
  testthat::expect_true(!is.na(risk_assess_package$check_list$res_check$platform))
  
  testthat::expect_true(!is.na(risk_assess_package$check_list$res_check$package))
  
  testthat::expect_identical(length(risk_assess_package$check_list$res_check$test_output), 1L)
  
  testthat::expect_true(!is.na(risk_assess_package$check_list$res_check$test_output$testthat))
  
  testthat::expect_true(all(sapply(risk_assess_package$check_list$res_check$session_info$platform,
                                   function(x) !is.null(x) && x != "")))
  
  
})


test_that("risk_assess_pkg works with path and mocked get_host_package", {
  skip_on_cran()
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data test.package.0001_0.1.0.tar.gz not found")
  withr::defer(unlink(dp), envir = parent.frame())

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    list(cran = TRUE, host = "CRAN", url = "https://cran.r-project.org")
  }
  local_mocked_bindings(
    get_host_package = mock_get_host_package,
    .package = "risk.assessr"
  )

  risk_assess_package <- risk_assess_pkg(path = dp)

  testthat::expect_true(checkmate::check_class(risk_assess_package, "list"))
})


test_that("risk_assess_pkg works with path params", {
  skip_on_cran()
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data test.package.0001_0.1.0.tar.gz not found")
  withr::defer(unlink(dp), envir = parent.frame())

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    list(cran = TRUE, host = "CRAN", url = "https://cran.r-project.org")
  }
  local_mocked_bindings(
    get_host_package = mock_get_host_package,
    .package = "risk.assessr"
  )

  risk_assess_package <- risk_assess_pkg(path = dp)

  testthat::expect_true(checkmate::check_class(risk_assess_package, "list"))
})

test_that("risk_assess_pkg works with invalid path params", {

  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)

  dp <- "invaled_path/"

  # Define the mock function for get_host_package
  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    return(list(cran = TRUE, host = "CRAN", url = "https://cran.r-project.org"))
  }

  # Apply mocked binding for risk.assessr::get_host_package
  local_mocked_bindings(
    get_host_package = mock_get_host_package,
    .package = "risk.assessr"
  )

  expect_warning(risk_assess_package <- risk_assess_pkg(path = dp))
  testthat::expect_null(risk_assess_package)
})

# ---- package param (get_package_tarfile) ----

test_that("risk_assess_pkg with package param uses get_package_tarfile and returns list when successful", {
  skip_on_cran()
  skip_if(
    .Platform$OS.type == "windows",
    "covr runs in subprocess on Windows; not reliable under devtools::test()"
  )
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data test.package.0001_0.1.0.tar.gz not found")
  withr::defer(unlink(dp), envir = parent.frame())

  mockery::stub(risk_assess_pkg, "get_package_tarfile", function(package_name, version, repos) dp)
  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    list(cran = TRUE, host = "CRAN", url = "https://cran.r-project.org")
  }
  local_mocked_bindings(get_host_package = mock_get_host_package, .package = "risk.assessr")

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  out <- risk_assess_pkg(package = "here")
  testthat::expect_true(is.list(out))
  testthat::expect_identical(length(out), 5L)
})

test_that("risk_assess_pkg with package and version passes version to get_package_tarfile", {
  skip_on_cran()
  skip_if(
    .Platform$OS.type == "windows",
    "covr runs in subprocess on Windows; not reliable under devtools::test()"
  )
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data test.package.0001_0.1.0.tar.gz not found")
  withr::defer(unlink(dp), envir = parent.frame())

  captured <- list(version = NULL)
  mock_get_tarfile <- function(package_name, version, repos) {
    captured$version <<- version
    dp
  }
  mockery::stub(risk_assess_pkg, "get_package_tarfile", mock_get_tarfile)
  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    list(cran = TRUE, host = "CRAN", url = "https://cran.r-project.org")
  }
  local_mocked_bindings(get_host_package = mock_get_host_package, .package = "risk.assessr")

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  risk_assess_pkg(package = "here", version = "1.0.1")
  testthat::expect_identical(captured$version, "1.0.1")
})

test_that("risk_assess_pkg with package returns NULL and warns when get_package_tarfile fails", {
  mockery::stub(risk_assess_pkg, "get_package_tarfile", function(...) stop("Download failed"))

  testthat::expect_warning(out <- risk_assess_pkg(package = "nonexistentpkg"), "Failed to download package")
  testthat::expect_null(out)
})

# ---- file.choose branch ----

test_that("risk_assess_pkg with no path or package uses file.choose and succeeds when file exists", {
  skip_on_cran()
  skip_if(
    .Platform$OS.type == "windows",
    "file.choose mocking not reliable on Windows under devtools::test()"
  )
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data test.package.0001_0.1.0.tar.gz not found")
  withr::defer(unlink(dp), envir = parent.frame())

  mockery::stub(risk_assess_pkg, "file.choose", function() dp)
  mock_get_host_package <- function(pkg_name, pkg_ver, pkg_source_path) {
    list(cran = TRUE, host = "CRAN", url = "https://cran.r-project.org")
  }
  local_mocked_bindings(get_host_package = mock_get_host_package, .package = "risk.assessr")

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  out <- risk_assess_pkg()
  testthat::expect_true(is.list(out))
})

test_that("risk_assess_pkg with file.choose returns NULL and warns when chosen file does not exist", {
  mockery::stub(risk_assess_pkg, "file.choose", function() "/nonexistent/chosen.tar.gz")

  testthat::expect_warning(out <- risk_assess_pkg(), "Chosen file does not exist")
  testthat::expect_null(out)
})

# ---- path vs package conflict ----

test_that("risk_assess_pkg warns and returns NULL when both path and package are provided", {
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data not found")

  expect_warning(out <- risk_assess_pkg(path = dp, package = "here"), "Provide either 'path' or 'package'")
  testthat::expect_null(out)
})

# ---- failed unpack ----

test_that("risk_assess_pkg returns NULL with message when set_up_pkg fails to unpack", {
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data not found")
  withr::defer(unlink(dp), envir = parent.frame())

  mock_set_up_pkg <- function(...) list(
    package_installed = FALSE,
    pkg_source_path = "",
    build_vignettes = TRUE,
    rcmdcheck_args = list()
  )
  mockery::stub(risk_assess_pkg, "set_up_pkg", mock_set_up_pkg)

  expect_message(out <- risk_assess_pkg(path = dp), "Failed to unpack package")
  testthat::expect_null(out)
})

# ---- failed local install ----

test_that("risk_assess_pkg returns NULL with message when install_package_local fails", {
  skip_on_cran()
  dp <- get_test_tarball_path()
  if (nchar(dp) == 0L) skip("Test data not found")
  withr::defer(unlink(dp), envir = parent.frame())

  mockery::stub(risk_assess_pkg, "install_package_local", function(...) FALSE)

  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)

  expect_message(out <- risk_assess_pkg(path = dp), "Package installation failed")
  testthat::expect_null(out)
})

#' @title Summarize detailed testthat results
#' @description Internal helper function to generate a long-format summary of testthat results, including status and line references.
#' @param x list of test result objects
#' @return data.frame with detailed test results including status and source line links
#' @keywords internal
get_tests_long_sum_stf <- function(x) {
  
  # Return NULL if input is empty
  if (length(x) == 0) {
    return(NULL)
  }
  
  # Convert the list of test results to a data.frame
  x1 <- data.frame(x, stringsAsFactors = FALSE)
  
  # Convert logical columns to numeric (e.g., TRUE/FALSE to 1/0)
  for (i in seq(1:ncol(x1)))
    if (is.logical(x1[[i]])) {
      x1[[i]] <- as.numeric(x1[[i]])
    }
  
  # Add a 'pass' column: 1 if no failures, skips, warnings, or errors; 0 otherwise
  x1$pass <- as.numeric((apply(x1[, c("failed", "skipped", "warning", "error")], 1, sum)) == 0)
  
  # Initialize a 'status' column
  x1$status <- NA
  
  # Assign status based on test result conditions
  for (i in 1:nrow(x1)) {
    if (x1$pass[i] == 1) {
      x1$status[i] <- "PASS"
    }
    
    if (x1$failed[i] != 0) {
      x1$status[i] <- "FAILED"
    }
    
    if (x1$error[i] != 0) {
      x1$status[i] <- "ERROR"
    }
    
    if (x1$skipped[i] != 0) {
      x1$status[i] <- "SKIPPED"
    }
    
    if (x1$warning[i] != 0) {
      x1$status[i] <- "WARNING"
    }
  }
  
  # Select and rename relevant columns for output
  ret <- x1[, c("file", "context", "test", "status", "nb", "real")]
  names(ret)[c(5, 6)] <- c("n", "time")
  
  # Generate line references for each test result
  lines <- sapply(x, function(test_result) {
    src_lines <- unique(c(
      x[[1]]$src[1],
      x[[1]]$src[3]
    ))
    src_lines <- src_lines[!is.na(src_lines)]
    paste0("L", src_lines, collapse = "_")
  })
  
  # Combine file name with line references
  ret$file <- sprintf("%s#%s", ret$file, lines)
  
  
  # Return the final long-format summary
  ret
}

test_that("get_tests_long_sum_stf returns NULL for empty input", {
  expect_null(get_tests_long_sum_stf(list()))
})

test_that("get_tests_long_sum_stf assigns PASS when no issues", {
  
  # Minimal mock test result structure
  mock_test_result <- list(
    file = "test-mock.R",
    context = "mock context",
    test = "test 1",
    failed = FALSE,
    skipped = FALSE,
    warning = FALSE,
    error = FALSE,
    nb = 1,
    real = 0.01,
    src = c(5, 3, 5, 30, 3, 30, 15, 15)
  )
  
  input <- list(mock_test_result)
  result <- get_tests_long_sum_stf(input)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 8)
})


test_that("get_tests_long_sum_stf assigns FAIL when no issues", {
  
  mock_test_result <- list(
    file = "test-mock_fail.R",
    context = "mock context",
    test = "test fail",
    failed = TRUE,
    skipped = FALSE,
    warning = FALSE,
    error = FALSE,
    nb = 1,
    real = 0.01,
    src = c(5, NA, 15)
  )
  
  input <- list(mock_test_result)
  result <- get_tests_long_sum_stf(input)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)
})

test_that("get_tests_long_sum_stf assigns ERROR when no issues", {
  
  mock_test_result <- list(
    file = "test-mock_error.R",
    context = "mock context",
    test = "test error",
    failed = FALSE,
    skipped = FALSE,
    warning = FALSE,
    error = TRUE,
    nb = 1,
    real = 0.01,
    src = c(20, NA, 35)
  )
  
  input <- list(mock_test_result)
  result <- get_tests_long_sum_stf(input)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)
})

test_that("get_tests_long_sum_stf assigns SKIPPED when no issues", {
  
  mock_test_result <- list(
    file = "test-mock_skipped.R",
    context = "mock context",
    test = "test skipped",
    failed = FALSE,
    skipped = TRUE,
    warning = FALSE,
    error = TRUE,
    nb = 1,
    real = 0.01,
    src = c(20, NA, 35)
  )
  
  input <- list(mock_test_result)
  result <- get_tests_long_sum_stf(input)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)
})

test_that("get_tests_long_sum_stf assigns WARNING when no issues", {
  
  mock_test_result <- list(
    file = "test-mock_warn.R",
    context = "mock context",
    test = "test warning",
    failed = FALSE,
    skipped = TRUE,
    warning = TRUE,
    error = TRUE,
    nb = 1,
    real = 0.01,
    src = c(20, NA, 35)
  )
  
  input <- list(mock_test_result)
  result <- get_tests_long_sum_stf(input)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)
})

#' @title Normalize Nested Package Data
#' @description Flattens nested package metadata into a flat list for further processing.
#' @param results The nested package metadata (list structure as produced by your package).
#' @return A list with flattened package information.
#' @importFrom purrr pluck
#' @keywords internal
normalize_data <- function(results) {
  list(
    name = pluck(results, "pkg_name", .default = NA_character_),
    version = pluck(results, "pkg_version", .default = NA_character_),
    dependencies_import = names(pluck(results, "dependencies", "imports", .default = list())),
    dependencies_count = length(pluck(results, "dependencies", "imports", .default = list())),
    
    all_versions = pluck(results, "version_info", "all_versions", .default = NULL),
    last_version = pluck(results, "version_info", "last_version", .default = NULL),
    
    maintainer = pluck(results, "author", "maintainer", .default = NA_character_),
    license = clean_license((pluck(results, "license_name", .default = "NOT AVAILABLE"))),
    reverse_dependencies_count = length(pluck(results, "rev_deps", .default = list())),
    funder = pluck(results, "author", "funder", .default = NA_character_),
    authors = pluck(results, "author", "authors", .default = NULL),
    
    has_bug_reports_url = pluck(results, "has_bug_reports_url", .default = NULL),
    has_maintainer      = pluck(results, "has_maintainer",      .default = NULL),
    has_source_control  = pluck(results, "has_source_control",  .default = NULL),
    has_website         = pluck(results, "has_website",         .default = NULL),
    has_vignettes       = pluck(results, "has_vignettes",       .default = NULL),
    has_examples        = pluck(results, "has_examples",        .default = NULL),
    has_docs            = pluck(results, "has_docs",            .default = NULL),  
    has_news            = pluck(results, "has_news",            .default = NULL),
    has_ex_docs_score   = as.numeric(pluck(results, "has_ex_docs_score", .default = NA_real_)),
    
    code_coverage = pluck(results, "covr", .default = 0),
    cmd_check = pluck(results, "check", .default = 0),
    
    total_download = pluck(results, "download", "total_download", .default = 0),
    last_month_download = pluck(results, "download", "last_month_download", .default = 0),
    
    github_links = pluck(results, "host", "github_links", .default = NULL),
    cran_links = pluck(results, "host", "cran_links", .default = NULL),
    bioconductor_links = pluck(results, "host", "bioconductor_links", .default = NULL),
    internal_links = pluck(results, "host", "internal_links", .default = NULL),
    github_stars = pluck(results, "github_data", "stars", .default = NULL),
    github_forks = pluck(results, "github_data", "forks", .default = NULL),
    github_created_at = pluck(results, "github_data", "created_at", .default = NULL),
    recent_commits_count = pluck(results, "github_data", "recent_commits_count", .default = NULL)
  )
}


#' @title Extract Risk Inputs
#' @description Compute intermediate risk input values from flattened package data.
#' @param flat_data Flat package data as returned by \code{normalize_data()}.
#' @return A list of computed values required for risk scoring.
#' @keywords internal
extract_risk_inputs <- function(flat_data) {
  
  fields <- c("has_bug_reports_url","has_maintainer","has_source_control",
              "has_website","has_vignettes","has_examples","has_news")
  
  documentation_score <- sum(vapply(flat_data[fields], function(x) {
    if (is.null(x)) return(0L)

    if (is.numeric(x) || is.logical(x)) {
      return(as.integer(isTRUE(as.logical(x)))) 
    }
    1L                                         
  }, integer(1L)))
  
  all_versions <- flat_data$all_versions
  current_version <- flat_data$version
  last_version <- flat_data$last_version
  
  if (!is.null(all_versions)) {
    
    index <- which(sapply(all_versions, function(x) x$version == current_version))
    
    if (length(index) == 0) {
      index <- length(all_versions)
    }
    later_version <- length(all_versions) - index
    
  } else {
    later_version <- NA
  }
  
  return(c(
    flat_data,
    list(
      documentation_score = documentation_score,
      later_version = later_version
    )
  ))
}


#' @title Get Risk Definition
#' @description Load risk definition config from options, JSON file, or fallback to bundled file.
#' @return A list of risk configuration items.
#' @examples
#' risk_rule <- get_risk_definition()
#' print(risk_rule)
#' @export
#' @export
get_risk_definition <- function() {
  opt <- getOption("risk.assessr.risk_definition")
  
  # If option is a file path
  if (is.character(opt) && file.exists(opt)) {
    def <- tryCatch(
      jsonlite::fromJSON(opt, simplifyVector = FALSE),
      error = function(e) NULL
    )
    if (is.list(def)) return(def)
  }
  
  # If option is already a valid R list
  if (is.list(opt)) {
    message("Using custom risk definition.")
    return(opt)
  }
  
  # fallback
  fallback_path <- system.file("config", "risk-definition.json", package = "risk.assessr")
  
  if (file.exists(fallback_path)) {
    def <- tryCatch(
      jsonlite::fromJSON(fallback_path, simplifyVector = FALSE),
      error = function(e) NULL
    )
    if (is.list(def)) {
      message("Using package's default risk definition.")
      return(def)
    }
  }
  
  stop("No valid risk definition found in options or package fallback.")
}


#' @title Compute Risk Level
#' @description Apply risk thresholds to compute risk level for a single metric.
#' @param value The numeric or categorical value to evaluate.
#' @param risk A list describing the thresholds for the metric.
#' @return The computed risk level (e.g. "low", "medium", "high").
#' @keywords internal
compute_risk <- function(value, risk) {
  
  level <- if (!is.null(risk$default)) { 
    risk$default 
  } else { 
    "unknown" 
  }
  
  if (is.null(value) || all(is.na(value))) {
    return("unknown")
  }
  
  for (threshold in risk$thresholds) {
    if (!is.null(threshold$values)) {
      if (is.list(value) || is.vector(value)) {
        if (any(toupper(value) %in% toupper(threshold$values))) {
          return(threshold$level)
        }
      } else {
        if (toupper(value) %in% toupper(threshold$values)) {
          return(threshold$level)
        }
      }
    } else {
      if (is.null(threshold$max) || value <= threshold$max) {
        return(threshold$level)
      }
    }
  }
  return(level)
}




#' @title Get Risk Analysis
#' @description Compute risk levels for the package metadata.
#' @param data The nested package metadata.
#' @return A named list of computed risk levels.
#'
#' @examples
#' # Minimal mock input for demonstration
#' mock_data <- list(
#'   dependencies_count = 5,
#'   later_version = 2,
#'   code_coverage = 0.75,
#'   last_month_download = 150000,
#'   license = "MIT",
#'   reverse_dependencies_count = 10,
#'   documentation_score = 2,
#'   has_examples = list(
#'     data = data.frame(
#'       function_name          = "somefunction",
#'       documentation_name     = "No documentation found",
#'       documentation_location = 0,
#'       example                = "no Rd file",
#'       stringsAsFactors       = FALSE
#'     ),
#'     example_score = 0.20   
#'   ),
#'   has_docs = list(
#'     data = data.frame(
#'       function_name    = "somefunction",
#'       stringsAsFactors = FALSE
#'     ),
#'     has_docs_score = 0.30  
#'   ),
#'   cmd_check = 0
#' )
#'
#' get_risk_analysis(mock_data)
#' @export
get_risk_analysis <- function(data) {
  
  risk_config <- get_risk_definition()
  flat_data <- normalize_data(data)
  extracted <- extract_risk_inputs(flat_data)
    
  risks <- list()
  for (risk_item in risk_config) {
    
    key <- risk_item$key
    value <- extracted[[key]]
    risks[[key]] <- compute_risk(value, risk_item)
  }
  
  return(risks)
}

# normalize_data

mock_data <- list(
  pkg_name = "mockpkg",
  pkg_version = "1.0.0",
  dependencies = list(
    imports = list(rprojroot = "2.0.4")
  ),
  author = list(
    maintainer = "Mock Maintainer",
    funder = "No package foundation found",
    authors = c("Mock Maintainer")
  ),
  license_name = "MIT + file LICENSE",
  rev_deps = list("pkg1", "pkg2", "pkg3"),
  has_bug_reports_url = 1,
  has_examples = 1,
  has_maintainer = 1,
  has_source_control = 1,
  has_vignettes = 1,
  has_news = 1,
  has_website = 1,
  check = 0,   
  version_info = list(
    all_versions = list(  
      list(version = "0.1", date = 17314),
      list(version = "0.9.0", date = 18581),
      list(version = "1.0.0", date = 18609)
    ),
    last_version = list(version = "1.0.0", date = 18609)
  ),
  download = list(
    total_download = 10000,
    last_month_download = 50000
  ),
  sysname = "Windows",
  machine = "x86-64",
  release = "10 x64",
  date_time = "2025-06-04 11:00:26",
  host = list(
    github_links = "https://github.com/mock/mockpkg",
    cran_links = "https://cran.r-project.org/src/contrib/mockpkg_1.0.0.tar.gz",
    bioconductor_links = NULL,
    internal_links = NULL
  ),
  github_data = list(
    created_at = "2016-07-19T14:47:19Z",
    stars = 100,
    forks = 10,
    recent_commits_count = 5
  ),
  covr = 0.9
)

# normalize_data

test_that("normalize_data works correctly", {
  normalized <- normalize_data(mock_data)
  expect_equal(normalized$name, "mockpkg")
  expect_equal(normalized$version, "1.0.0")
  expect_equal(normalized$dependencies_count, 1)
  expect_equal(normalized$license, "MIT")
  expect_equal(normalized$reverse_dependencies_count, 3)
  expect_equal(normalized$code_coverage, 0.9)
  expect_equal(normalized$last_month_download, 50000)
})

test_that("documentation_score = 7 when all flags are 1 with non null key value", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    license_name = "MIT + file LICENSE",
    rev_deps = list("pkg1"),
    has_bug_reports_url = "https",
    has_examples = 1,
    has_maintainer = 'john doe',
    has_source_control = 1,
    has_vignettes = 1,
    has_website = "https",
    has_news = 1,
    version_info = list(
      all_versions = list(
        list(version = "0.9.0"),
        list(version = "1.0.0")
      ),
      last_version = "1.0.0"
    )
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$documentation_score, 7)
})

test_that("documentation_score = 7 when all flags are 1", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    license_name = "MIT + file LICENSE",
    rev_deps = list("pkg1"),
    has_bug_reports_url = 1,
    has_examples = 1,
    has_maintainer = 1,
    has_source_control = 1,
    has_vignettes = 1,
    has_website = 1,
    has_news = 1,
    version_info = list(
      all_versions = list(
        list(version = "0.9.0"),
        list(version = "1.0.0")
      ),
      last_version = "1.0.0"
    )
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$documentation_score, 7)
})

test_that("documentation_score sums only the 1s when some flags are 0", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    has_bug_reports_url = 1,
    has_examples = 0,
    has_maintainer = 1,
    has_source_control = 0,
    has_vignettes = 1,
    has_website = 0,
    has_news = 1
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$documentation_score, 4)
})

test_that("missing flags are treated as 0 by normalize_data defaults", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    has_bug_reports_url = 1,
    has_maintainer = 1,
    has_source_control = 1,
    has_vignettes = 1,
    has_news = 1
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$documentation_score, 5)
})

test_that("NA flags are ignored by sum(..., na.rm = TRUE)", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    has_bug_reports_url = 1,
    has_examples = NA,  
    has_maintainer = 1,
    has_source_control = NA,  
    has_vignettes = 1,
    has_website = 1,
    has_news = 1
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$documentation_score, 5)
})

test_that("later_version counts versions strictly after current when present", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    version_info = list(
      all_versions = list(
        list(version = "0.8.0"),
        list(version = "1.0.0"),
        list(version = "1.2.0"),
        list(version = "2.0.0")
      ),
      last_version = "2.0.0"
    )
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$later_version, 2)
})

test_that("later_version = 0 when current version not in all_versions", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "9.9.9",  # not present
    version_info = list(
      all_versions = list(
        list(version = "0.8.0"),
        list(version = "1.0.0")
      ),
      last_version = "1.0.0"
    )
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$later_version, 0)
})

test_that("later_version is NA when all_versions is NULL", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    version_info = list(
      all_versions = NULL,
      last_version = "1.0.0"
    )
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_true(is.na(out$later_version))
})

test_that("extract_risk_inputs preserves original normalized fields", {
  results <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    has_bug_reports_url = 1
  )
  flat <- normalize_data(results)
  out <- extract_risk_inputs(flat)
  expect_equal(out$name, "mockpkg")
  expect_equal(out$version, "1.0.0")
  expect_true(all(c("documentation_score", "later_version") %in% names(out)))
})


# extract_risk_inputs

test_that("extract_risk_inputs works correctly", {
  normalized <- normalize_data(mock_data)
  extracted <- extract_risk_inputs(normalized)
  
  expect_equal(extracted$dependencies_count, 1)
  expect_equal(extracted$reverse_dependencies_count, 3)
  expect_equal(extracted$documentation_score, 7)
  expect_equal(extracted$license, "MIT")
  expect_equal(extracted$cmd_check, 0)
})

mock_data <- list(
  pkg_name = "mockpkg",
  pkg_version = "1.0.0",
  dependencies = list(
    imports = list(rprojroot = "2.0.4")
  ),
  author = list(
    maintainer = "Mock Maintainer",
    funder = "No package foundation found",
    authors = c("Mock Maintainer")
  ),
  license_name = "MIT + file LICENSE",
  rev_deps = list("pkg1", "pkg2", "pkg3"),
  has_bug_reports_url = 1,
  has_examples = 1,
  has_maintainer = 1,
  has_source_control = 1,
  has_vignettes = 1,
  has_news = 1,
  has_website = 1,
  check = 0,   
  version_info = list(
    all_versions = list(  
      list(version = "0.1", date = 17314),
      list(version = "0.9.0", date = 18581),
      list(version = "1.0.0", date = 18609)
    ),
    last_version = list(version = "1.0.0", date = 18609)
  ),
  download = list(
    total_download = 10000,
    last_month_download = 50000
  ),
  sysname = "Windows",
  machine = "x86-64",
  release = "10 x64",
  date_time = "2025-06-04 11:00:26",
  host = list(
    github_links = "https://github.com/mock/mockpkg",
    cran_links = "https://cran.r-project.org/src/contrib/mockpkg_1.0.0.tar.gz",
    bioconductor_links = NULL,
    internal_links = NULL
  ),
  github_data = list(
    created_at = "2016-07-19T14:47:19Z",
    stars = 100,
    forks = 10,
    recent_commits_count = 5
  ),
  covr = 0.9
)


test_that("extract_risk_inputs works correctly", {
  normalized <- normalize_data(mock_data)
  extracted <- extract_risk_inputs(normalized)
  
  expect_equal(extracted$dependencies_count, 1)
  expect_equal(extracted$reverse_dependencies_count, 3)
  expect_equal(extracted$documentation_score, 7)
  expect_equal(extracted$license, "MIT")
  expect_equal(extracted$cmd_check, 0)
})



# get_risk_definition

mock_risk_config <- list(
  list(
    label = "mock test",
    id = "mock_id",
    key = "mock_key",
    thresholds = list(list(level = "low", max = 10))
  )
)

test_that("returns R object when option is a valid list", {

  withr::local_options(list(risk.assessr.risk_definition = mock_risk_config))
  result <- get_risk_definition()
  expect_equal(result, mock_risk_config)
})

test_that("returns parsed list from valid JSON file path", {
  tmpfile <- tempfile(fileext = ".json")
  json_txt <- jsonlite::toJSON(mock_risk_config, auto_unbox = TRUE, pretty = TRUE)
  writeLines(json_txt, tmpfile)
  withr::local_options(list(risk.assessr.risk_definition = tmpfile))

  result <- get_risk_definition()

  expect_equal(
    jsonlite::fromJSON(jsonlite::toJSON(result, auto_unbox = TRUE)),
    jsonlite::fromJSON(jsonlite::toJSON(mock_risk_config, auto_unbox = TRUE))
  )
  unlink(tmpfile)
})


test_that("warns and falls back when JSON path is invalid or file doesn't exist", {
  withr::local_options(list(risk.assessr.risk_definition = "non_existent_file.json"))

  result <- get_risk_definition()
  expect_true(is.list(result))
})

test_that("falls back when JSON is valid but not a list", {
  tmpfile <- tempfile(fileext = ".json")
  writeLines('{"this": "is not a list of configs"}', tmpfile)
  withr::local_options(list(risk.assessr.risk_definition = tmpfile))

  result <- get_risk_definition()
  expect_true(is.list(result))
  unlink(tmpfile)
})


test_that("uses bundled JSON if no option is set", {
  withr::local_options(list(risk.assessr.risk_definition = NULL))
  result <- get_risk_definition()
  expect_true(is.list(result))
})

test_that("errors if bundled JSON is missing or malformed", {

  withr::local_options(list(risk.assessr.risk_definition = NULL))
  mockery::stub(get_risk_definition, "system.file", function(...) tempfile())
  expect_error(
    get_risk_definition(),
    "No valid risk definition found in options or package."
  )
})


# Compute_risk

test_that("compute_risk works for numeric thresholds", {

  risk_def <- list(
    thresholds = list(
      list(level = "low", max = 10),
      list(level = "medium", max = 20),
      list(level = "high", max = NULL)
    )
  )

  expect_equal(compute_risk(5, risk_def), "low")
  expect_equal(compute_risk(15, risk_def), "medium")
  expect_equal(compute_risk(30, risk_def), "high")
})

test_that("compute_risk works for categorical thresholds", {
  risk_def <- list(
    thresholds = list(
      list(level = "high", values = c("AGPL", "GPL")),
      list(level = "low", values = c("MIT", "APACHE"))
    ),
    default = "high"
  )

  expect_equal(compute_risk("MIT", risk_def), "low")
  expect_equal(compute_risk("GPL", risk_def), "high")
  expect_equal(compute_risk("UNKNOWN", risk_def), "high")
})

test_that("compute_risk handles NULL value with no default", {
  risk_def <- list(
    thresholds = list(
      list(level = "low", max = 10)
    )
  )
  expect_equal(compute_risk(NULL, risk_def), "unknown")
})

test_that("compute_risk is case-insensitive for categorical values", {
  risk_def <- list(
    thresholds = list(
      list(level = "low", values = c("MIT", "APACHE"))
    )
  )
  expect_equal(compute_risk("mit", risk_def), "low")
  expect_equal(compute_risk("Apache", risk_def), "low")
})


test_that("compute_risk handles vector inputs with highest risk wins by level rank", {
  
  risk_def <- list(
    thresholds = list(
      list(level = "low", values = c("MIT", "APACHE")),
      list(level = "high", values = c("AGPL", "GPL"))
    ))

  expect_equal(compute_risk(c("GPL"), risk_def), "high")
  expect_equal(compute_risk(c("MIT"), risk_def), "low")
  expect_equal(compute_risk(c("BSD"), risk_def), "unknown")
})


test_that("compute_risk works correctly for all configured metrics", {
  withr::local_options(list(risk.assessr.risk_definition = NULL))
  
  # Helper
  get_risk <- function(key) {
    config <- get_risk_definition()
    idx <- which(sapply(config, function(x) x$key == key))
    config[[idx]]
  }

  expect_equal(compute_risk(10, get_risk("dependencies_count")), "low")
  expect_equal(compute_risk(5, get_risk("dependencies_count")), "low")
  expect_equal(compute_risk(30, get_risk("dependencies_count")), "medium")
  expect_equal(compute_risk(100, get_risk("dependencies_count")), "high")

  expect_equal(compute_risk(1, get_risk("later_version")), "low")
  expect_equal(compute_risk(5, get_risk("later_version")), "high")

  expect_equal(compute_risk(0.5, get_risk("code_coverage")), "high")
  expect_equal(compute_risk(0.7, get_risk("code_coverage")), "medium")
  expect_equal(compute_risk(0.95, get_risk("code_coverage")), "low")

  expect_equal(compute_risk(30000, get_risk("total_download")), "high")
  expect_equal(compute_risk(3500001, get_risk("total_download")), "medium")
  expect_equal(compute_risk(50000001, get_risk("total_download")), "low")

  expect_equal(compute_risk("MIT", get_risk("license")), "low")
  expect_equal(compute_risk("GPL", get_risk("license")), "medium")
  expect_equal(compute_risk("SSPL", get_risk("license")), "high")
  expect_equal(compute_risk(NULL, get_risk("license")), "unknown")

  expect_equal(compute_risk(2, get_risk("reverse_dependencies_count")), "high")
  expect_equal(compute_risk(10, get_risk("reverse_dependencies_count")), "medium")
  expect_equal(compute_risk(100, get_risk("reverse_dependencies_count")), "low")
  
  expect_equal(compute_risk(7, get_risk("documentation_score")), "low")
  expect_equal(compute_risk(1, get_risk("documentation_score")), "high")
  expect_equal(compute_risk(2, get_risk("documentation_score")), "high")
  expect_equal(compute_risk(5, get_risk("documentation_score")), "medium")
  expect_equal(compute_risk(6, get_risk("documentation_score")), "medium")
  expect_equal(compute_risk(4, get_risk("documentation_score")), "medium")
  
  expect_equal(compute_risk(0, get_risk("cmd_check")), "high")
  expect_equal(compute_risk(1, get_risk("cmd_check")), "low")
})


# get_risk_analysis

test_that("get_risk_analysis returns expected risk levels", {
  withr::local_options(list(risk.assessr.risk_definition = NULL))

  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$dependencies_count, "low")
  expect_equal(risks$code_coverage, "low")
  expect_equal(risks$license, "low")
  expect_equal(risks$reverse_dependencies_count, "high")
  expect_equal(risks$cmd_check, "high")
  expect_equal(risks$total_download, "high")
  expect_equal(risks$later_version, "low")
})

test_that("get_risk_analysis works end-to-end", {
  withr::local_options(list(risk.assessr.risk_definition = NULL))
  
  risks <- get_risk_analysis(mock_data)
  expect_true("dependencies_count" %in% names(risks))
  expect_true("license" %in% names(risks))
  expect_true("reverse_dependencies_count" %in% names(risks))
  expect_true("documentation_score" %in% names(risks))
  expect_true("cmd_check" %in% names(risks))
})


test_that("get_risk_analysis assigns default when value is missing", {
  withr::local_options(list(risk.assessr.risk_definition = NULL))
  
  partial_data <- mock_data
  partial_data$license_name <- "NOT AVAILABLE"
  risks <- get_risk_analysis(partial_data)
  expect_equal(risks$license, "high")
})


test_that("get_risk_analysis assigns unknown license", {
  withr::local_options(list(risk.assessr.risk_definition = NULL))
  
  partial_data <- mock_data
  partial_data$license_name <- "blablabla"
  risks <- get_risk_analysis(partial_data)
  expect_equal(risks$license, "high")
})


# get_risk_analysis with non standard rules


test_that("get_risk_analysis works with multiple metrics in custom config", {

  multi_config <- list(
    list(
      label = "dependency count",
      id = "dependencies_count",
      key = "dependencies_count",
      thresholds = list(
        list(level = "high", max = 1),
        list(level = "medium", max = 3),
        list(level = "low", max = NULL)
      )
    ),
    list(
      label = "code coverage",
      id = "code_coverage",
      key = "code_coverage",
      thresholds = list(
        list(level = "critical", max = 0.7),
        list(level = "acceptable", max = 0.85),
        list(level = "excellent", max = NULL)
      )
    ),
    list(
      label = "popularity",
      id = "popularity",
      key = "last_month_download",
      thresholds = list(
        list(level = "low", max = 10000),
        list(level = "medium", max = 60000),
        list(level = "high", max = NULL)
      )
    )
  )
  withr::local_options(list(risk.assessr.risk_definition = multi_config))
  
  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$dependencies_count, "high")
  expect_equal(risks$code_coverage, "excellent")
  expect_equal(risks$last_month_download, "medium")
})

test_that("get_risk_analysis handles categorical levels with custom config", {

  custom_license_config <- list(
    list(
      label = "license",
      id = "license",
      key = "license",
      default = "unknown",
      thresholds = list(
        list(level = "bad", values = c("GPL", "AGPL")),
        list(level = "ok", values = c("LGPL")),
        list(level = "good", values = c("MIT", "BSD"))
      )
    )
  )
  
  withr::local_options(list(risk.assessr.risk_definition = custom_license_config))
  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$license, "good")
})

test_that("get_risk_analysis returns HIGH risk when total_download <= 3,500,000", {

  mock_data <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    download = list(
      total_download = 1000000,   # 1,000,000 -> should be 'high'
      last_month_download = 50000
    )
  )

  custom_config <- list(
    list(
      label = "total_download",
      id = "total_download",
      key = "total_download",
      default = "unknown",
      thresholds = list(
        list(level = "high", max = 3500000),
        list(level = "medium", max = 50000000),
        list(level = "low", max = NULL)
      )
    )
  )

  withr::local_options(list(risk.assessr.risk_definition = custom_config))
  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$total_download, "high")
})

test_that("get_risk_analysis returns MEDIUM risk when total_download <= 50,000,000", {
  
  mock_data <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    download = list(
      total_download = 10000000,
      last_month_download = 50000
    )
  )

  custom_config <- list(
    list(
      label = "popularity",
      id = "popularity",
      key = "total_download",
      default = "unknown",
      thresholds = list(
        list(level = "high", max = 3500000),
        list(level = "medium", max = 50000000),
        list(level = "low", max = NULL)
      )
    )
  )

  withr::local_options(list(risk.assessr.risk_definition = custom_config))
  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$total_download, "medium")
})

test_that("get_risk_analysis returns LOW risk when total_download > 50,000,000", {
  mock_data <- list(
    pkg_name = "mockpkg",
    pkg_version = "1.0.0",
    download = list(
      total_download = 100000000,
      last_month_download = 50000
    )
  )

  custom_config <- list(
    list(
      label = "popularity",
      id = "popularity",
      key = "total_download",
      default = "unknown",
      thresholds = list(
        list(level = "high", max = 3500000),
        list(level = "medium", max = 50000000),
        list(level = "low", max = NULL)
      )
    )
  )

  withr::local_options(list(risk.assessr.risk_definition = custom_config))
  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$total_download, "low")
})

test_that("get_risk_analysis returns correct risk levels for different scenarios", {
  
  mock_data <- list(
    has_bug_reports_url = 1,
    has_examples = 1,
    has_maintainer = 1,
    has_source_control = 1,
    has_vignettes = 1,
    has_website = 1,
    has_news = 1
  )
  

  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$documentation_score, "low")
  
  mock_data <- list(
    has_bug_reports_url = 1,
    has_examples = 1,
    has_maintainer = 1,
    has_source_control = 1,
    has_vignettes = 1,
    has_website = NULL,
    has_news = NULL
  )
  
  risks <- get_risk_analysis(mock_data)
  
  expect_equal(risks$documentation_score, "medium")
  
  mock_data <- list(
    has_bug_reports_url = 1
  )
  
  risks <- get_risk_analysis(mock_data)
  expect_equal(risks$documentation_score, "high")
})

#' Fetch Bioconductor Release Announcements
#' 
#' This function retrieves the Bioconductor release announcements page and returns
#' its HTML content for further processing.
#' 
#' @return An XML document from bioconductor version page.
#' 
#' @importFrom curl curl_fetch_memory
#' @importFrom xml2 read_html
#' 
#' @examples
#' \dontrun{
#' html_content <- fetch_bioconductor_releases()
#' }
#' @export
fetch_bioconductor_releases <- function() {
  url <- "https://bioconductor.org/about/release-announcements/"
  
  # Perform GET request
  response <- curl::curl_fetch_memory(url)
  
  # If status code is not 200, return FALSE
  if (response$status_code != 200) {
    return(FALSE)
  }
  
  html_content <- xml2::read_html(rawToChar(response$content))
  return(html_content)
}

#' Parse Bioconductor Release Announcements
#' 
#' This function extracts Bioconductor release details such as version number,
#' release date, number of software packages, and required R version from the 
#' release announcements HTML page.
#' 
#' @param html_content The parsed HTML document from `fetch_bioconductor_releases`.
#' 
#' @return A list of lists containing Bioconductor release details: release version, date,
#'         number of software packages, and corresponding R version.
#' 
#' @importFrom xml2 xml_find_first xml_find_all xml_text
#' @importFrom stringr str_trim
#' 
#' @examples
#' \dontrun{
#' html_content <- fetch_bioconductor_releases()
#' release_data <- parse_bioconductor_releases(html_content)
#' print(release_data)
#' }
#' @export
parse_bioconductor_releases <- function(html_content) {
  data <- list()
  
  table <- xml2::xml_find_first(html_content, "//table")
  rows <- xml2::xml_find_all(table, ".//tr")[-1]  # Skip the header row
  
  for (row in rows) {
    cols <- xml2::xml_find_all(row, ".//td")
    
    if (length(cols) == 4) {
      release <- xml2::xml_text(cols[1])
      date <- xml2::xml_text(cols[2])
      software_packages <- xml2::xml_text(cols[3])
      r_version <- xml2::xml_text(cols[4])
      
      data <- append(data, list(list(
        release = stringr::str_trim(release),
        date = stringr::str_trim(date),
        software_packages = stringr::str_trim(software_packages),
        r_version = stringr::str_trim(r_version)
      )))
    }
  }
  return(data)
}

#' Parse HTML Content for Package Versions
#' 
#' This function extracts version information from an HTML page listing
#' available versions of a Bioconductor package.
#' 
#' @param html_content A character string containing the HTML content.
#' @param package_name A character string specifying the name of the package.
#' 
#' @return A list of lists containing package details such as package name, version, link,
#'         date, and size. Returns an empty list if no versions are found.
#' 
#' @importFrom xml2 read_html xml_find_all xml_find_first xml_text xml_attr
#' @importFrom stringr str_trim
#' 
#' @examples
#' \dontrun{
#' parse_html_version(html_content, "GenomicRanges")
#' }
#' @export
parse_html_version <- function(html_content, package_name) {
  
  data <- list()
  
  if (!is.null(html_content)) {
    # Parse the HTML content
    doc <- xml2::read_html(html_content)
    rows <- xml2::xml_find_all(doc, "//tr")
    
    for (row in rows) {
      cols <- xml2::xml_find_all(row, "td")
      if (length(cols) >= 4) {  # Ensure it's not a divider row
        link_tag <- xml2::xml_find_first(cols[2], "a")
        
        if (!is.na(xml2::xml_text(link_tag))) {
          href <- xml2::xml_attr(link_tag, "href")
          text <- xml2::xml_text(link_tag)
          date <- xml2::xml_text(cols[3])
          size <- xml2::xml_text(cols[4])
          version <- ""
          
          if (grepl(paste0("^", package_name, "(?:_|$)"), text)) {
            version <- gsub(paste0("^", package_name, "(?:_|$)|\\.tar\\.gz$"), "", text)
          } else {
            version <- ""  
          }
          
          if (version == "Parent Directory" | version == "") {
            next
          }
          
          data <- append(data, list(
            list(
              package_name = package_name,
              package_version = version,
              link = href,
              date = date,
              size = size
            )
          ))
        }
      }
    }
  }
  
  return(data)
}

#' Extract Package Version from File Path
#' 
#' This function extracts the version number from a package source file name
#' based on the package name and expected file pattern.
#' 
#' @param path A character string specifying the file path or URL.
#' @param package_name A character string specifying the name of the package.
#' 
#' @return A character string representing the extracted version number, or `NULL` if no match is found.
#' 
#' @examples
#' \dontrun{
#' link <- "https://bioconductor.org/packages/3.14/bioc/src/contrib/GenomicRanges_1.42.0.tar.gz"
#' extract_version(link, "GenomicRanges")
#' }
#' @export
extract_version <- function(path, package_name) {
  pattern <- paste0(package_name, "_([0-9\\.]+)\\.tar\\.gz$")
  match <- regexpr(pattern, path, perl = TRUE)
  
  if (match == -1) {
    return(NULL) # No match found
  }
  
  version <- regmatches(path, regexec(pattern, path))[[1]][2]
  return(version)
}

#' Fetch Bioconductor Package Information
#' 
#' This function retrieves information about a specific Bioconductor package
#' for a given Bioconductor version. It fetches the package details, such as
#' version, source package URL, and archived versions if available.
#' 
#' @param bioconductor_version A character string specifying the Bioconductor version (e.g., "3.14").
#' @param package_name A character string specifying the name of the package.
#' 
#' @return A list containing package details, including the latest version, package URL,
#'         source package link, and any archived versions if available.
#'         Returns `FALSE` if the package does not exist or cannot be retrieved.
#' 
#' @importFrom curl curl_fetch_memory
#' @importFrom xml2 read_html xml_find_all xml_find_first xml_text xml_attr
#' 
#' @examples
#' \dontrun{
#' info <- fetch_bioconductor_package_info("3.14", "GenomicRanges")
#' print(info)
#' }
#' @export
fetch_bioconductor_package_info <- function(bioconductor_version, package_name) {
  
  repo_list <- list()
  url_page <- paste0("https://bioconductor.org/packages/", bioconductor_version, "/bioc/html/", package_name, ".html")
  
  message("url_page: ", url_page)
  
  # Perform GET request with proper error handling
  response <- tryCatch({
    curl::curl_fetch_memory(url_page)  # Remove handle argument
  }, error = function(e) {
    message(paste("Error fetching:", url_page, "->", e$message))
    return(NULL)
  })
  
  # If package does not exist, return FALSE
  if (is.null(response) || response$status_code != 200) {
    return(FALSE)
  }
  
  # Ensure content is not empty before parsing HTML
  if (is.null(response$content) || length(response$content) == 0) {
    stop("Response is NULL or empty for URL: ", url_page)
  }
  
  html_content <- tryCatch({
    read_html(rawToChar(response$content))
  }, error = function(e) {
    message("Failed to parse HTML content for: ", package_name)
    return(NULL)
  })
  
  if (is.null(html_content)) return(FALSE)  
  
  data_dict <- list()
  rows <- xml_find_all(html_content, "//tr")
  keys_to_keep <- c("version", "url", "source_package")
  
  for (row in rows) {
    cells <- xml_find_all(row, "td")
    if (length(cells) == 2) {
      key <- tolower(gsub(" ", "_", xml_text(cells[1])))
      
      if (key %in% keys_to_keep) {
        data_dict[[key]] <- trimws(xml_text(cells[2]))
      }
    }
  }
  
  # Ensure we have a proper version
  if (!"version" %in% names(data_dict)) {
    return(FALSE)
  }
  
  # Get latest version for the Bioconductor version
  data_dict$bioconductor_version <- bioconductor_version
  data_dict$url_package <- data_dict$url
  data_dict$archived <- FALSE
  
  # Extract source package from links on the page
  page <- tryCatch({
    read_html(url_page)
  }, error = function(e) {
    message("Failed to fetch page: ", url_page)
    return(NULL)
  })
  
  if (!is.null(page)) {
    links <- xml_find_all(page, "//a")
    hrefs <- xml_attr(links, "href")
    tar_gz_links <- hrefs[grepl("\\.tar\\.gz$", hrefs)]
    
    package_version <- extract_version(tar_gz_links, package_name)
    if (!is.null(package_version)) {
      source_package_main <- paste0("https://bioconductor.org/packages/", bioconductor_version, "/bioc/src/contrib/", package_name, "_", package_version, ".tar.gz")
      data_dict$source_package <- source_package_main
    } else {
      data_dict$source_package <- NA  # Ensure there's always a value
    }
  }
  
  # Get archived versions
  url_archive <- paste0("https://bioconductor.org/packages/", bioconductor_version, "/bioc/src/contrib/Archive/", package_name, "/")
  
  archive_page <- tryCatch({
    read_html(url_archive)
  }, error = function(e) {
    return(NULL)
  })
  
  repo_list <- list(data_dict)
  
  archive_page <- tryCatch({
    read_html(url_archive)
  }, error = function(e) {
    return(NULL)
  })
  
  if (!is.null(archive_page)) {
    links <- xml_find_all(archive_page, "//a")
    hrefs <- xml_attr(links, "href")
    archive_tar_gz_links <- hrefs[grepl("\\.tar\\.gz$", hrefs)]
    
    if (length(archive_tar_gz_links) > 0) {
      full_archive_links <- rev(paste0(url_archive, archive_tar_gz_links))
      
      for (link in full_archive_links) {
        version <- extract_version(link, package_name)
        repo <- list(
          version = version, 
          url = data_dict$url_package, 
          source_package = link, 
          archived = TRUE, 
          bioconductor_version = bioconductor_version
        )
        repo_list <- append(repo_list, list(repo))
      }
    }
  }
  
  return(repo_list)
}


#' Retrieve Bioconductor Package URL
#'
#' This function fetches the source package URL for a given Bioconductor package.
#' If no version is specified, it retrieves the latest available version.
#'
#' @param package_name A character string specifying the name of the Bioconductor package.
#' @param package_version (Optional) A character string specifying the package version. Defaults to `NULL`, which retrieves the latest version.
#' @param release_data A list containing Bioconductor release information.
#'
#' @return A list containing:
#'  \item{url}{The URL of the source package (if available).}
#'  \item{version}{The specified or latest available package version.}
#'  \item{last_version}{A list with version, date, and Bioconductor version.}
#'  \item{all_versions}{A list of all discovered versions with version, date, and Bioconductor version.}
#'  \item{bioconductor_version_package}{The Bioconductor version matched to the selected version.}
#'  \item{archived}{A logical value indicating whether the package is archived.}
#'
#' @examples
#' \dontrun{
#' release_data <- list(
#'   list(release = "3.17", date = "October 14, 2005"),
#'   list(release = "3.18", date = "October 4, 2006"),
#'   list(release = "3.19", date = "October 8, 2007")
#' )
#' 
#' get_bioconductor_package_url("GenomicRanges", release_data = release_data)
#' 
#'}
#' @export
get_bioconductor_package_url <- function(package_name, package_version = NULL, release_data) {
  
  all_versions <- list()
  source_code <- NULL
  bioconductor_version_package <- NULL
  archived <- NULL
  last_version <- NULL
  Sys.setlocale("LC_TIME", "C")
  
  for (version_info in release_data) {
    
    bioconductor_version <- version_info$release
    parsed_date <- strptime(version_info$date, format = "%B %d, %Y")
    
    # Convert to Date type
    if (!is.na(parsed_date)) {
      bioconductor_date <- format(as.Date(parsed_date), "%Y-%m-%d")
    } else {
      bioconductor_date <- NA
    }
    
    message(paste("Checking Bioconductor version:", bioconductor_version))
    
    # Get Bioconductor package info
    package_info <- tryCatch({
      fetch_bioconductor_package_info(bioconductor_version, package_name)
    }, error = function(e) {
      message(paste("Skipping Bioconductor version", bioconductor_version, "due to error:", e$message))
      return(NULL)
    })
    
    if (is.null(package_info) || !is.list(package_info)) next
    
    # Ensure package_info is treated as a list of packages
    if (!is.null(package_info$version)) {
      package_info <- list(package_info)
    }
    
    for (pkg in package_info) {
      if (!is.list(pkg) || is.null(pkg$version)) next
      
      pkg_version <- pkg$version
      message(paste("Checking package version:", pkg_version, "Date:", bioconductor_date))
      
      version_record <- list(
        version = pkg_version,
        date = bioconductor_date,
        bioconductor_version = bioconductor_version
      )
      
      # Add to all_versions (no deduplication needed if you trust the data)
      all_versions <- append(all_versions, list(version_record))
      
      # Update last_version if this version is newer
      if (is.null(last_version)) {
        last_version <- version_record
      } else if (!is.na(bioconductor_date) && !is.na(last_version$date) && bioconductor_date > last_version$date) {
        last_version <- version_record
      } else if (is.na(last_version$date) && !is.na(bioconductor_date)) {
        last_version <- version_record
      }
      
      # Set default package version if not specified
      if (is.null(package_version) || length(package_version) == 0 || is.na(package_version)) {
        package_version <- last_version$version
      }
      
      # Check if this version matches the requested version
      if (!is.null(pkg_version) && !is.null(package_version) &&
          !is.na(pkg_version) && !is.na(package_version) &&
          pkg_version == package_version) {
        
        bioconductor_version_package <- bioconductor_version
        source_code <- pkg$source_package
        archived <- ifelse(!is.null(pkg$archived), pkg$archived, FALSE)
      }
    }
  }
  
  if (is.null(source_code)) {
    message("Package URL not found on Bioconductor")
  }
  
  return(list(
    url = source_code,
    version = package_version,
    last_version = last_version,
    all_versions = all_versions,
    bioconductor_version_package = bioconductor_version_package,
    archived = archived
  ))
}




#' Find Bioconductor Package Reverse Dependencies 
#'
#' This function returns the reverse dependencies for a given Bioconductor package.
#'
#' @param pkg Character string. The name of the package for which to find reverse dependencies.
#' @param which Character vector. The dependency categories to check.
#'   One or more of \code{"Depends"}, \code{"Imports"}, \code{"LinkingTo"},
#'   \code{"Suggests"}, or \code{"Enhances"}. Defaults to \code{"Imports"}.
#' @param only.bioc Logical. If \code{TRUE} (default), only reverse dependencies
#'   that are Bioconductor packages are returned.
#' @param version Bioconductor version to use. Defaults to the current version.
#' @param db Optional. A pre-loaded package database to use for lookups.
#' @param biocdb Optional. A pre-loaded Bioconductor package database.
#'
#' @return A named list of reverse dependency package names.
#'
#' @examples
#' \dontrun{
#' # Get reverse Imports dependencies as a list:
#' bioconductor_reverse_deps("limma")
#' 
#' # Get multiple categories:
#' bioconductor_reverse_deps("limma", which = c("Depends", "Suggests"))
#' }
#' @importFrom BiocManager version
#' @importFrom BiocManager repositories
#' @export
bioconductor_reverse_deps <- function(
  pkg,
  which = "Imports",
  only.bioc = FALSE,
  version = BiocManager::version(),
  db = NULL,          
  biocdb = NULL   
) {
  valid_which <- c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")

  if ("all" %in% which) {
    which_vec <- valid_which
  } else if (!all(which %in% valid_which)) {
    message("Invalid 'which' value. Must be one or more of: ",
            paste(valid_which, collapse = ", "), " or 'all'.")
    return(NULL)
  } else {
    which_vec <- which
  }

  # data source
  if (is.null(db)) {
    repos <- BiocManager::repositories(version = version, replace = TRUE)

    read_packages_index <- function(repo_url) {
      
      idx <- utils::contrib.url(repo_url, type = "source")
      urls <- c(file.path(idx, "PACKAGES.gz"), file.path(idx, "PACKAGES"))
      
      read_one <- function(u) {
        con <- NULL
        on.exit(try(close(con), silent = TRUE), add = TRUE)
        con <- if (grepl("\\.gz$", u)) gzcon(url(u, "rb")) else url(u, "rb")
        dcf <- tryCatch(read.dcf(con), error = function(e) NULL)
        if (is.null(dcf)) return(NULL)
        df <- as.data.frame(dcf, stringsAsFactors = FALSE, optional = TRUE)
        if (!nrow(df)) return(NULL)
        needed <- c("Package","Depends","Imports","LinkingTo","Suggests","Enhances")
        for (nm in setdiff(needed, names(df))) df[[nm]] <- NA_character_
        rownames(df) <- df$Package
        df[ , needed, drop = FALSE]
      }
      for (u in urls) {
        out <- tryCatch(read_one(u), error = function(e) NULL)
        if (!is.null(out)) return(out)
      }
      NULL
    }

    db_list <- lapply(unname(repos), read_packages_index)
    db_list <- Filter(Negate(is.null), db_list)

    if (!length(db_list)) {
      message("Error fetching package database: could not read any PACKAGES indexes.")
      return(NULL)
    }
    db <- do.call(rbind, db_list)
  } else {
    db <- as.data.frame(db, stringsAsFactors = FALSE)
    if (is.null(rownames(db)) && "Package" %in% names(db)) rownames(db) <- db$Package
    needed <- c("Package","Depends","Imports","LinkingTo","Suggests","Enhances")
    for (nm in setdiff(needed, names(db))) db[[nm]] <- NA_character_
    db <- db[, needed, drop = FALSE]
  }

  if (isTRUE(only.bioc)) {
    if (is.null(biocdb)) {
      repos <- BiocManager::repositories(version = version, replace = TRUE)
      biocdb <- db
    } else {
      biocdb <- as.data.frame(biocdb, stringsAsFactors = FALSE)
      if (is.null(rownames(biocdb)) && "Package" %in% names(biocdb)) rownames(biocdb) <- biocdb$Package
    }
    db <- db[rownames(db) %in% rownames(biocdb), , drop = FALSE]
  }

  out <- lapply(which_vec, function(w) {
    res <- tools::package_dependencies(packages = pkg, db = db, reverse = TRUE, which = w)[[pkg]]
    if (is.null(res)) character(0) else unname(res)
  })
  names(out) <- which_vec
  out
}

# extract_version

test_that("extract_version extracts correct version", {
  expect_equal(extract_version("package_1.2.3.tar.gz", "package"), "1.2.3")
  expect_equal(extract_version("some/path/package_4.5.6.tar.gz", "package"), "4.5.6")
  expect_equal(extract_version("https://example.com/package_7.8.9.tar.gz", "package"), "7.8.9")
})

test_that("extract_version returns NULL for invalid paths", {
  expect_null(extract_version("package.tar.gz", "package"))
  expect_null(extract_version("random_1.2.3.tar.gz", "package"))
  expect_null(extract_version("package_1.2.3.txt", "package"))
})

test_that("extract_version handles edge cases", {
  expect_equal(extract_version("package_0.0.1.tar.gz", "package"), "0.0.1")
  expect_equal(extract_version("package_10.0.0.tar.gz", "package"), "10.0.0")
  expect_equal(extract_version("package_1.2.3.4.tar.gz", "package"), "1.2.3.4")
})


# fetch_bioconductor_releases

test_that("fetch_bioconductor_releases returns valid HTML from mock input", {

    mock_fetch <- function(url) {
    list(
      status_code = 200,
      content = charToRaw("<html><head><title>Mock Page</title></head><body><h1>Bioconductor Releases</h1></body></html>")
    )
  }

  # Use stub to replace curl_fetch_memory with the mock function
    mockery::stub(fetch_bioconductor_releases, "curl::curl_fetch_memory", mock_fetch)
  result <- fetch_bioconductor_releases()
  expect_true(inherits(result, "xml_document"))
})

test_that("fetch_bioconductor_releases returns FALSE on 404", {
  # Mock function to simulate a 404 response
  mock_fetch <- function(url) {
    list(
      status_code = 404,
      content = NULL
    )
  }

  # Use stub to replace curl_fetch_memory with the mock function
  mockery::stub(fetch_bioconductor_releases, "curl::curl_fetch_memory", mock_fetch)
  result <- fetch_bioconductor_releases()
  expect_false(result)
})


# parse_bioconductor_releases

test_that("parse_bioconductor_releases correctly extracts release data", {
  # Define mock HTML with a table structure
  mock_html <- "
  <html>
    <body>
      <table>
        <tr>
          <th>Release</th> <th>Date</th> <th>Software Packages</th> <th>R Version</th>
        </tr>
        <tr>
          <td>3.14</td> <td>2021-10-27</td> <td>1890</td> <td>R 4.1</td>
        </tr>
        <tr>
          <td>3.15</td> <td>2022-04-27</td> <td>1945</td> <td>R 4.2</td>
        </tr>
      </table>
    </body>
  </html>
  "

  # Convert to XML document
  html_doc <- read_html(mock_html)

  # Call the function
  result <- parse_bioconductor_releases(html_doc)

  # Expected result
  expected <- list(
    list(release = "3.14", date = "2021-10-27", software_packages = "1890", r_version = "R 4.1"),
    list(release = "3.15", date = "2022-04-27", software_packages = "1945", r_version = "R 4.2")
  )

  # Check the length of parsed results
  expect_length(result, 2)

  # Check if extracted data matches expected output
  expect_equal(result, expected)
})


# fetch_bioconductor_package_info

test_that("fetch_bioconductor_package_info returns main + archive versions", {
  
  # Mock curl::curl_fetch_memory to return raw HTML content
  local_curl_fetch <- function(url) {

    # This mock returns the main package HTML (called early via curl)
    html <- '<html><body><table>
      <tr><td>Version</td><td>3.50.0</td></tr>
      <tr><td>Source Package</td><td>https://bioconductor.org/packages/3.20/bioc/src/contrib/test_limma_3.50.0.tar.gz</td></tr>
    </table></body></html>'
    
    return(list(status_code = 200, content = charToRaw(html)))
  }
  
  local_read_html <- function(x) {
    if (is.list(x) && !is.null(x$content)) {
      message("✅ read_html called with RAW response")
      html_text <- rawToChar(x$content)
      
    } else if (is.character(x) && grepl("^<html>", x)) {
      html_text <- x
      
    } else if (is.character(x) && x == "https://bioconductor.org/packages/3.20/bioc/html/limma.html") {
      html_text <- '<html><body><table>
      <tr><td>Version</td><td>3.50.0</td></tr>
      <tr><td>Source Package</td><td>https://bioconductor.org/packages/3.20/bioc/src/contrib/test_limma_3.50.0.tar.gz</td></tr>
    </table><a href="limma_3.50.0.tar.gz">limma_3.50.0.tar.gz</a></body></html>'
      
    } else if (is.character(x) && x == "https://bioconductor.org/packages/3.20/bioc/src/contrib/Archive/limma/") {
      html_text <- '<html><body>
      <a href="limma_3.48.0.tar.gz">limma_3.48.0.tar.gz</a>
      <a href="limma_3.47.0.tar.gz">limma_3.47.0.tar.gz</a>
    </body></html>'
      
    } else {
      html_text <- "<html></html>"
    }
    xml2::read_html(html_text)
  }
  
  
  my_fn <- fetch_bioconductor_package_info
  mockery::stub(my_fn, "curl::curl_fetch_memory", local_curl_fetch)
  mockery::stub(my_fn, "read_html", local_read_html)
  result <- my_fn("3.20", "limma")
  
  # Assertions
  expect_type(result, "list")
  expect_length(result, 3)
  
  # Check main version
  expect_equal(result[[1]]$version, "3.50.0")
  expect_equal(result[[1]]$source_package, "https://bioconductor.org/packages/3.20/bioc/src/contrib/limma_3.50.0.tar.gz")
  expect_false(result[[1]]$archived)
  
  # Check first archived version
  expect_equal(result[[2]]$version, "3.47.0")
  expect_equal(result[[2]]$source_package, "https://bioconductor.org/packages/3.20/bioc/src/contrib/Archive/limma/limma_3.47.0.tar.gz")
  expect_true(result[[2]]$archived)
  
  # Check second archived version
  expect_equal(result[[3]]$version, "3.48.0")
  expect_equal(result[[3]]$source_package, "https://bioconductor.org/packages/3.20/bioc/src/contrib/Archive/limma/limma_3.48.0.tar.gz")
  expect_true(result[[3]]$archived)
})


test_that("fetch_bioconductor_package_info returns only main version when no archive exists", {
  local_curl_fetch <- function(url) {
    if (grepl("bioc/html/noarchive.html", url)) {
      html <- '<html><body><table>
        <tr><td>Version</td><td>2.10.0</td></tr>
        <tr><td>Source Package</td><td>https://bioconductor.org/packages/3.20/bioc/src/contrib/noarchive_2.10.0.tar.gz</td></tr>
      </table></body></html>'
      return(list(status_code = 200, content = charToRaw(html)))
    }
    if (grepl("src/contrib/Archive/noarchive", url)) {
      return(list(status_code = 404, content = raw(0)))
    }
    return(list(status_code = 404, content = raw(0)))
  }
  
  local_read_html <- function(content) {
    xml2::read_html(rawToChar(content$content))
  }
  
  mockery::stub(fetch_bioconductor_package_info, "curl::curl_fetch_memory", local_curl_fetch)
  mockery::stub(fetch_bioconductor_package_info, "xml2::read_html", local_read_html)
  
  result <- fetch_bioconductor_package_info("3.20", "noarchive")
  
  expect_type(result, "list")
  expect_length(result, 1)
  expect_equal(result[[1]]$source_package, "https://bioconductor.org/packages/3.20/bioc/src/contrib/noarchive_2.10.0.tar.gz")
  expect_equal(result[[1]]$version, "2.10.0")
})



test_that("fetch_bioconductor_package_info returns only archive versions when no main version exists", {
  local_curl_fetch <- function(url) {
    if (grepl("bioc/html/archiveonly.html", url)) {
      return(list(status_code = 404, content = raw(0)))
    }
    if (grepl("src/contrib/Archive/archiveonly", url)) {
      archive_html <- '<html><body>
        <a href="archiveonly_1.0.0.tar.gz">archiveonly_1.0.0.tar.gz</a>
      </body></html>'
      return(list(status_code = 200, content = charToRaw(archive_html)))
    }
    return(list(status_code = 404, content = raw(0)))
  }
  
  local_read_html <- function(content) {
    xml2::read_html(rawToChar(content$content))
  }
  
  mockery::stub(fetch_bioconductor_package_info, "curl::curl_fetch_memory", local_curl_fetch)
  mockery::stub(fetch_bioconductor_package_info, "xml2::read_html", local_read_html)
  
  result <- fetch_bioconductor_package_info("3.20", "archiveonly")
  expect_false(result)
})

test_that("fetch_bioconductor_package_info returns FALSE when package is not found", {
  local_curl_fetch <- function(url) {
    return(list(status_code = 404, content = raw(0)))
  }
  
  local_read_html <- function(content) {
    xml2::read_html(rawToChar(content$content))
  }
  
  mockery::stub(fetch_bioconductor_package_info, "curl::curl_fetch_memory", local_curl_fetch)
  mockery::stub(fetch_bioconductor_package_info, "xml2::read_html", local_read_html)
  
  result <- fetch_bioconductor_package_info("3.20", "doesnotexist")
  expect_false(result)
})


# get_bioconductor_package_url

# Mocked fetch_bioconductor_package_info function
mock_fetch_bioconductor_package_info <- function(bioconductor_version, package_name) {
  # Simulate different Bioconductor versions and package availability
  if (package_name == "DESeq2") {
    return(list(
      version = "1.38.0",
      bioconductor_version = bioconductor_version,
      source_package = paste0("https://bioconductor.org/packages/", bioconductor_version, "/bioc/src/contrib/DESeq2_1.38.0.tar.gz"),
      archived = FALSE
    ))
  } else if (package_name == "edgeR") {
    return(list(
      version = "3.40.2",
      bioconductor_version = bioconductor_version,
      source_package = paste0("https://bioconductor.org/packages/", bioconductor_version, "/bioc/src/contrib/edgeR_3.40.2.tar.gz"),
      archived = FALSE
    ))
  } else {
    return(NULL)  # Simulate non-existing package
  }
}

# Test cases

release_data <- list(
  list(release = "3.17", date = "October 14, 2005"),
  list(release = "3.18", date = "October 4, 2006"),
  list(release = "3.19", date = "October 8, 2007")
)

test_that("get_bioconductor_package_url returns correct URL for DESeq2", {

  mockery::stub(get_bioconductor_package_url, "fetch_bioconductor_package_info", mock_fetch_bioconductor_package_info)
  result <- get_bioconductor_package_url("DESeq2", release_data = release_data)

  expected_url <- "https://bioconductor.org/packages/3.19/bioc/src/contrib/DESeq2_1.38.0.tar.gz"
  expect_equal(result$url, expected_url)
  expect_equal(result$version, "1.38.0")
  expect_false(result$archived)
  expect_type(result$all_versions, "list")
})

test_that("get_bioconductor_package_url returns correct URL for older package version", {
  
  mockery::stub(get_bioconductor_package_url, "fetch_bioconductor_package_info", mock_fetch_bioconductor_package_info)
  result <- get_bioconductor_package_url("edgeR", package_version = "3.40.2", release_data = release_data)

  expected_url <- "https://bioconductor.org/packages/3.19/bioc/src/contrib/edgeR_3.40.2.tar.gz"
  expect_equal(result$url, expected_url)
  expect_equal(result$version, "3.40.2")
  expect_type(result$all_versions, "list")
})

test_that("get_bioconductor_package_url returns NULL for non-existing package", {

  mockery::stub(get_bioconductor_package_url, "fetch_bioconductor_package_info", mock_fetch_bioconductor_package_info)
  result <- get_bioconductor_package_url("NonExistentPackage", release_data = release_data)
  
  expect_null(result$url)
  expect_null(result$version)
})

test_that("get_bioconductor_package_url handles errors gracefully", {
  
  mock_fetch_error <- function(bioconductor_version, package_name) {
    stop("Simulated error")
  }

  mockery::stub(get_bioconductor_package_url, "fetch_bioconductor_package_info", mock_fetch_error)
  result <- get_bioconductor_package_url("DESeq2", release_data = release_data)
  
  expect_null(result$url)
  expect_null(result$version)
})

make_db <- function(pkgs) {
  df <- do.call(rbind, lapply(pkgs, function(x) {
    data.frame(
      Package    = if (is.null(x$Package)) NA_character_ else x$Package,
      Version    = if (is.null(x$Version)) "1.0.0" else x$Version,
      Depends    = if (is.null(x$Depends)) NA_character_ else x$Depends,
      Imports    = if (is.null(x$Imports)) NA_character_ else x$Imports,
      LinkingTo  = if (is.null(x$LinkingTo)) NA_character_ else x$LinkingTo,
      Suggests   = if (is.null(x$Suggests)) NA_character_ else x$Suggests,
      Enhances   = if (is.null(x$Enhances)) NA_character_ else x$Enhances,
      stringsAsFactors = FALSE
    )
  }))
  rownames(df) <- df$Package
  df
}

test_that("bioconductor_reverse_deps returns reverse Imports", {
  # edgeR and DESeq2 import limma
  db <- make_db(list(
    list(Package = "limma"),
    list(Package = "edgeR",  Imports = "limma"),
    list(Package = "DESeq2", Imports = "limma"),
    list(Package = "other")
  ))

  res <- bioconductor_reverse_deps("limma", which = "Imports", db = db)
  expect_type(res, "list")
  expect_equal(names(res), "Imports")
  expect_setequal(res$Imports, c("edgeR", "DESeq2"))
})

test_that("bioconductor_reverse_deps handles which = 'all'", {
  db <- make_db(list(
    list(Package = "limma"),
    list(Package = "edgeR",  Imports = "limma"),
    list(Package = "DESeq2", Imports = "limma")
  ))

  res <- bioconductor_reverse_deps("limma", which = "all", db = db)
  expect_type(res, "list")
  expect_true(all(c("Depends","Imports","LinkingTo","Suggests","Enhances") %in% names(res)))
  expect_setequal(res$Imports, c("edgeR","DESeq2"))
  # Others should be empty vectors
  expect_length(res$Depends,   0)
  expect_length(res$LinkingTo, 0)
  expect_length(res$Suggests,  0)
  expect_length(res$Enhances,  0)
})

test_that("bioconductor_reverse_deps only.bioc = TRUE filters correctly", {
  # DB has Bioc pkgs edgeR/DESeq2 and a non-Bioc pkg notBioc, all importing limma
  db <- make_db(list(
    list(Package = "limma"),
    list(Package = "edgeR",   Imports = "limma"),
    list(Package = "DESeq2",  Imports = "limma"),
    list(Package = "notBioc", Imports = "limma")
  ))

  # biocdb lists only the Bioconductor packages
  biocdb <- make_db(list(
    list(Package = "limma"),
    list(Package = "edgeR"),
    list(Package = "DESeq2")
  ))

  res <- bioconductor_reverse_deps("limma", which = "Imports", only.bioc = TRUE, db = db, biocdb = biocdb)
  expect_setequal(res$Imports, c("edgeR","DESeq2"))
  expect_false("notBioc" %in% res$Imports)
})

test_that("bioconductor_reverse_deps returns NULL for invalid which value", {
  db <- make_db(list(list(Package = "limma")))
  res <- bioconductor_reverse_deps("limma", which = "NotARealType", db = db)
  expect_null(res)
})

test_that("bioconductor_reverse_deps returns empty when pkg has no reverse deps", {
  db <- make_db(list(
    list(Package = "limma"),
    list(Package = "edgeR")  # no imports/depends on limma
  ))
  res <- bioconductor_reverse_deps("limma", which = "Imports", db = db)
  expect_equal(res$Imports, character(0))
})

#' Modify the DESCRIPTION File in a R Package Tarball
#'
#' This function recreate a `.tar.gz` R package file after modifying its `DESCRIPTION` file
#' by appending Config/build/clean-inst-doc: false parameter.
#'
#' @param tar_file A string representing the path to the `.tar.gz` file that contains the R package.
#'
#' @return A string containing the path to the newly created modified `.tar.gz` file.
#'
#' @examples
#' \dontrun{
#'   modified_tar <- modify_description_file("path/to/mypackage.tar.gz")
#' }
#'
#' @importFrom utils untar tar
#' @export
modify_description_file <- function(tar_file) {
  
  # Create a temporary directory
  temp_dir <- tempdir()

  # save and set user's current working directory
  oldwd <- getwd()  
  on.exit(setwd(oldwd))
  
  # Extract the tar file into the temporary directory
  tryCatch({
    suppressWarnings({
      untar(tar_file, exdir = temp_dir)
    })
  }, error = function(e) {
    stop("Error in untarring the file: ", e$message)
  })
  
  # Automatically find the package directory by looking for the DESCRIPTION file
  subdirs <- list.dirs(temp_dir, full.names = TRUE, recursive = FALSE)
  
  package_dir <- NULL
  package_name <- NULL
  
  for (dir in subdirs) {
    
    potential_description_file <- file.path(dir, "DESCRIPTION")
    if (file.exists(potential_description_file)) {
      package_dir <- dir
      package_name <- basename(dir)
      break
    }
  }
  
  if (is.null(package_dir)) {
    stop("Package directory not found (no DESCRIPTION file found).")
  }
  
  description_file <- file.path(package_dir, "DESCRIPTION")
  if (!file.exists(description_file)) {
    stop("DESCRIPTION file not found in the extracted package directory.")
  }
  
  # Modify the DESCRIPTION file
  description_content <- readLines(description_file)
  
  if ("Config/build/clean-inst-doc: false" %in% description_content) {
    return(tar_file)
  }
  
  description_content <- c(description_content, "Config/build/clean-inst-doc: false")
  writeLines(description_content, description_file)
  
  # Recreate a new tar.gz file
  modified_tar_file <- tempfile(fileext = ".tar.gz")
  
  # set working directory to temp_dir
  setwd(temp_dir)
  
  tryCatch({
    suppressWarnings({
      tar(modified_tar_file, files = package_name, compression = "gzip", tar = "internal")
    })
  }, error = function(e) {
    setwd(oldwd) 
    stop("Error in creating the tar.gz file: ", e$message)
  })
  
  # # Clean up extracted files in temp_dir
  unlink(package_dir, recursive = TRUE)
  
  return(modified_tar_file)
}

test_that("modify_description_file modifies DESCRIPTION correctly", {

  dp <- system.file("test-data", "test.package.0001_0.1.0.tar.gz",
                    package = "risk.assessr")

  # Check if the file exists before attempting to download
  if (!file.exists(dp)) {
    stop("The tar file does not exist at the specified path.")
  }

  # Create a temporary file to store the downloaded package
  file_name <- basename(dp) # Use the base name for temporary file
  temp_file <- file.path(tempdir(), file_name)

  # Copy the file to the temporary file instead of downloading it
  file.copy(dp, temp_file, overwrite = TRUE)

  # Verify that the copy was successful
  if (!file.exists(temp_file)) {
    stop("File copy failed: temporary file not found.")
  }

  # Run the function to modify the DESCRIPTION file
  package_name <- "test.package.0001"
  modified_tar_file <- modify_description_file(temp_file)

  # Extract the modified tarball to a temporary directory
  temp_dir <- tempdir()
  untar(modified_tar_file, exdir = temp_dir)

  # Check if the DESCRIPTION file exists in the modified package directory
  package_dir <- file.path(temp_dir, package_name)

  if (!dir.exists(package_dir)) {
    stop("Package directory not found after untarring.")
  }

  description_file <- file.path(package_dir, "DESCRIPTION")
  expect_true(file.exists(description_file))

  # Check if the DESCRIPTION file contains the new line
  description_content <- readLines(description_file)
  expect_true("Config/build/clean-inst-doc: false" %in% description_content)

  # Clean up temporary files
  withr::defer(unlink(temp_file))  # Remove the temporary tar file
  withr::defer(unlink(temp_dir, recursive = TRUE))
})


test_that("modify_description_file modifies DESCRIPTION correctly 1bis", {

  dp <- system.file("test-data", "test.package.0005_0.1.0.tar.gz",
                    package = "risk.assessr")

  # Check if the file exists before attempting to download
  if (!file.exists(dp)) {
    stop("The tar file does not exist at the specified path.")
  }

  # Create a temporary file to store the downloaded package
  file_name <- basename(dp) # Use the base name for temporary file
  temp_file <- file.path(tempdir(), file_name)

  # Copy the file to the temporary file instead of downloading it
  file.copy(dp, temp_file, overwrite = TRUE)

  # Verify that the copy was successful
  if (!file.exists(temp_file)) {
    stop("File copy failed: temporary file not found.")
  }

  # Run the function to modify the DESCRIPTION file
  package_name <- "test.package.0005"
  modified_tar_file <- modify_description_file(temp_file)

  # Extract the modified tarball to a temporary directory
  temp_dir <- tempdir()
  untar(modified_tar_file, exdir = temp_dir)

  # Check if the DESCRIPTION file exists in the modified package directory
  package_dir <- file.path(temp_dir, package_name)

  if (!dir.exists(package_dir)) {
    stop("Package directory not found after untarring.")
  }

  description_file <- file.path(package_dir, "DESCRIPTION")
  expect_true(file.exists(description_file))

  # Check if the DESCRIPTION file contains the new line
  description_content <- readLines(description_file)
  expect_true("Config/build/clean-inst-doc: false" %in% description_content)

  # Clean up temporary files
  withr::defer(unlink(temp_file))  # Remove the temporary tar file
  withr::defer(unlink(temp_dir, recursive = TRUE))
})

test_that("modify_description_file modifies DESCRIPTION correctly 2", {
  
  # Create separate temporary directories for creation and extraction
  temp_dir_create <- tempdir()
  temp_dir_extract <- tempdir()
  
  # Create a subdirectory to mimic a package directory
  package_dir <- file.path(temp_dir_create, "my_package")
  dir.create(package_dir)
  
  # Path to the DESCRIPTION file inside the package directory
  description_file <- file.path(package_dir, "DESCRIPTION")
  writeLines("This is a DESCRIPTION file.", description_file)
  
  # Create the .tar Archive including the entire package directory
  tar_file <- file.path(temp_dir_create, "my_package.tar.gz")
  suppressWarnings(tar(tar_file, files = "my_package", compression = "gzip", tar = "internal"))
  
  # Run the function to modify the DESCRIPTION file
  modified_tar_file <- modify_description_file(tar_file)
  
  # Extract the modified tarball to a separate temporary directory
  suppressWarnings(untar(modified_tar_file, exdir = temp_dir_extract))
  
  # Check if the DESCRIPTION file exists in the modified package directory
  description_file <- file.path(temp_dir_extract, "my_package", "DESCRIPTION")
  expect_true(file.exists(description_file))
  
  # Check if the DESCRIPTION file contains the new line
  description_content <- readLines(description_file)
  expect_true("Config/build/clean-inst-doc: false" %in% description_content)
  
  # Ensure cleanup happens even if the test fails
  withr::defer(unlink(temp_dir_create, recursive = TRUE))
  withr::defer(unlink(temp_dir_extract, recursive = TRUE))
  withr::defer(unlink(tar_file))
})



test_that("Config/build/clean-inst-doc: false already in DESCRIPTION file", {
  dp <- system.file("test-data", "test.package.0008_0.1.0.tar.gz",
                    package = "risk.assessr")

  # Check if the file exists before attempting to download
  if (!file.exists(dp)) {
    stop("The tar file does not exist at the specified path.")
  }

  # Create a temporary file to store the downloaded package
  file_name <- basename(dp) # Use the base name for temporary file
  temp_file <- file.path(tempdir(), file_name)

  # Copy the file to the temporary file instead of downloading it
  file.copy(dp, temp_file, overwrite = TRUE)

  # Verify that the copy was successful
  if (!file.exists(temp_file)) {
    stop("File copy failed: temporary file not found.")
  }

  # Run the function to modify the DESCRIPTION file
  package_name <- "test.package.0008"
  modified_tar_file <- modify_description_file(temp_file)

  # Extract the modified tarball to a temporary directory
  temp_dir <- tempdir()
  untar(modified_tar_file, exdir = temp_dir)

  # Check if the DESCRIPTION file exists in the modified package directory
  package_dir <- file.path(temp_dir, package_name)

  if (!dir.exists(package_dir)) {
    stop("Package directory not found after untarring.")
  }

  description_file <- file.path(package_dir, "DESCRIPTION")
  expect_true(file.exists(description_file))

  # Check if the DESCRIPTION file contains the expected content
  description_content <- readLines(description_file)
  config_line_count <- sum(grepl("Config/build/clean-inst-doc: false",
                                 description_content,
                                 ignore.case = TRUE))
  
  expect_equal(config_line_count, 1)

  # Clean up temporary files
  withr::defer(unlink(temp_file))  # Remove the temporary tar file
  withr::defer(unlink(temp_dir, recursive = TRUE))
})

test_that("No DESCRIPTION file in package", {
  # Create separate temporary directories for creation and extraction
  temp_dir_create <- tempdir()
  
  # Create a directory for the package
  package_dir <- file.path(temp_dir_create, "my_package")
  dir.create(package_dir)
  
  # Create a dummy file in the package directory (but no DESCRIPTION file)
  dummy_file <- file.path(package_dir, "dummy_file.txt")
  writeLines("This is a dummy file.", dummy_file)
  
  # Switch to the temporary directory to create tar archive with proper structure
  old_wd <- setwd(temp_dir_create)
  on.exit(setwd(old_wd))  # Ensure we return to the original directory
  
  # Create the .tar.gz archive without a DESCRIPTION file
  tar_file <- file.path(temp_dir_create, "my_package.tar.gz")
  suppressWarnings(tar(tar_file, files = "my_package", compression = "gzip", tar = "internal"))
  
  expect_error(modify_description_file(tar_file),
               regexp = "Package directory not found")
  
  
  # Ensure cleanup happens even if the test fails
  withr::defer(unlink(package_dir, recursive = TRUE))  # Package directory cleanup
  withr::defer(unlink(tar_file))  # Tar file cleanup
})


test_that("Error in untarring the file", {
  # Mock the untar function to force it to throw an error
  mock_untar <- mockery::mock(stop("Untar failed"))

  mockery::stub(modify_description_file, "untar", mock_untar)

  # Now call the function, expecting it to throw the custom error
  expect_error(modify_description_file("path/to/invalid.tar.gz"),
               "Error in untarring the file: Untar failed")
})

test_that("Error in tarring the file", {

  # Create a temporary directory for the package
  temp_dir_create <- tempdir()

  # Create a fake package directory structure
  package_name <- "test.package.0001"
  package_dir <- file.path(temp_dir_create, package_name)
  dir.create(package_dir)

  # Path to the DESCRIPTION file in the package directory
  description_file <- file.path(package_dir, "DESCRIPTION")
  writeLines("This is a DESCRIPTION file.", description_file)

  # Create the .tar Archive with only the necessary files
  tar_file <- file.path(temp_dir_create, "my_package.tar.gz")
  suppressWarnings(tar(tar_file, files = temp_dir_create, compression = "gzip", tar = "internal"))

  # Mock the tar function to simulate an error during the tarring process
  mock_tar <- mockery::mock(stop("Tar failed"))
  mockery::stub(modify_description_file, "tar", mock_tar)

  # Ensure cleanup happens even if the test fails
  withr::defer(unlink(temp_dir_create, recursive = TRUE))
  withr::defer(unlink(tar_file))

  # Now call the function, expecting it to throw the custom error
  expect_error(modify_description_file(tar_file),
               "Error in creating the tar.gz file: Tar failed")

})



test_that("insensitive case", {
  
  # Create a temporary directory for the package
  temp_dir_create <- tempdir()
  
  # Create a fake package directory structure
  package_name <- "test.package.0001"
  package_dir <- file.path(temp_dir_create, package_name)
  dir.create(package_dir)
  
  # Path to the DESCRIPTION file in the package directory
  description_file <- file.path(package_dir, "DESCRIPTION")
  writeLines("Config/build/clean-inst-doc: false", description_file)
  
  # Create the .tar Archive with only the necessary files
  tar_file <- file.path(temp_dir_create, "my_package.tar.gz")
  suppressWarnings(tar(tar_file, files = temp_dir_create, compression = "gzip", tar = "internal"))
  
  # Mock the tar function to simulate an error during the tarring process
  mock_tar <- mockery::mock(stop("Tar failed"))
  mockery::stub(modify_description_file, "tar", mock_tar)
  
  # Ensure cleanup happens even if the test fails
  withr::defer(unlink(temp_dir_create, recursive = TRUE))
  withr::defer(unlink(tar_file))
  
  # Check if the DESCRIPTION file contains the new line
  description_content <- readLines(description_file)
  config_line_count <- sum(grepl("Config/build/clean-inst-doc: false",
                                 description_content,
                                 ignore.case = TRUE))
  expect_equal(config_line_count, 1)
  expect_true("Config/build/clean-inst-doc: false" %in% description_content)
})

#' Get CRAN Daily Downloads for a Package
#'
#' Retrieves daily CRAN download counts over the past years for a given package,
#' including cumulative downloads.
#'
#' @param pkg Character string, name of the CRAN package.
#' @param years integer, number of past years (default 1) for historical
#'
#' @return A data frame with columns: \code{date}, \code{count}, and \code{cumulative_downloads}.
#'
#' @examples
#' \dontrun{
#'   get_package_download_cran("ggplot2")
#'   # Example output:
#'   #          date  count cumulative_downloads
#'   # 1 2025-04-22  78379           20073615
#'   # 2 2025-04-21  63195           19995236
#'   # 3 2025-04-20  42119           19932041
#'   # 4 2025-04-19  40848           19889922
#'   # 5 2025-04-18  54914           19849074
#'   # 6 2025-04-17  86273           19794160
#'   # 7 2025-04-16  80201           19707887
#' }
#' @export
get_package_download_cran <- function(pkg, years=1) {
  
  if (missing(pkg)) stop("Please provide a package name.")
  
  Sys.setlocale("LC_TIME", "C")
  start_date <- format(Sys.Date() - (years * 365), "%Y-%m-%d")
  end_date <- format(Sys.Date(), "%Y-%m-%d")
  url <- sprintf("https://cranlogs.r-pkg.org/downloads/daily/%s:%s/%s",
                 start_date, end_date, pkg)
  
  read_result <- tryCatch({
    con <- curl::curl(url)
    response <- readLines(con, warn = FALSE)
    close(con)
    jsonlite::fromJSON(paste(response, collapse = ""))
  }, error = function(e) {
    message("Could not fetch CRAN download data for package: ", pkg)
    return(NULL)
  })
  
  # Validate structure
  if (!"downloads" %in% names(read_result) || length(read_result$downloads) == 0) {
    message("No download data found or invalid format for: ", pkg)
    return(NULL)
  }
  
  downloads_df <- read_result$downloads
  
  if (length(downloads_df) == 0) {
    return(NULL)
  }
  
  if (is.list(downloads_df) && !is.data.frame(downloads_df)) {
    downloads_df <- downloads_df[[1]]
    if (length(downloads_df) == 0) 
      return(NULL)
  }
  
  all_data <- downloads_df %>%
    mutate(
      date = as.Date(day),
      count = as.integer(downloads),
      cumulative_downloads = cumsum(count)
    ) %>%
    select(date, count, cumulative_downloads) %>%
    arrange(dplyr::desc(date))
  
  return(all_data)
}

#' Get CRAN Total or Recent Downloads for a Package
#'
#' Retrieves either all-time total downloads or the total from the last `n` full months
#' for a given CRAN package.
#' 
#' Similar data can be obtained via:
#' 
#' https://cranlogs.r-pkg.org/badges/grand-total/ggplot2
#' https://cranlogs.r-pkg.org/badges/last-month/ggplot2
#'
#' @param pkg Character string, the name of the CRAN package.
#' @param months Integer. If NULL, returns all-time total; otherwise, returns total from the last `months` full months (excluding current).
#'
#' @return An integer: total number of downloads.
#'
#' @examples
#' \dontrun{
#'   get_cran_total_downloads("ggplot2")        # all-time total
#'   # 160776616
#'
#'   get_cran_total_downloads("ggplot2", 3)     # last 3 full months
#'   # 4741707
#'
#'   get_cran_total_downloads("ggplot2", 12)    # last 12 full months
#'   # 19878885
#' }
#'
#' @export
get_cran_total_downloads <- function(pkg, months = NULL) {
  
  if (!is.null(months) && (!is.numeric(months) || months < 1)) {
    message("months must be a positive integer")
    return(0)
  }
  
  # build URL
  if (is.null(months)) {
    start <- "2012-10-01"
    end <- format(Sys.Date(), "%Y-%m-%d")
    url <- sprintf("https://cranlogs.r-pkg.org/downloads/total/%s:%s/%s", start, end, pkg)
  } else {
    current_date <- Sys.Date()
    start_of_current_month <- as.Date(format(current_date, "%Y-%m-01"))
    end_date <- start_of_current_month - 1
    start_date <- seq(start_of_current_month, by = paste0("-", months, " months"), length.out = 2)[2]
    url <- sprintf("https://cranlogs.r-pkg.org/downloads/daily/%s:%s/%s", start_date, end_date, pkg)
  }
  
  response <- tryCatch({
    con <- curl::curl(url)
    on.exit(close(con), add = TRUE)
    readLines(con, warn = FALSE)
  }, error = function(e) {
    message("Failed to fetch data: URL not available (", url, ")")
    return(NULL)
  })
  
  if (is.null(response)) {
    return(NULL)
  }
  
  data <- jsonlite::fromJSON(paste(response, collapse = ""))
  
  
  # parse response
  result <- jsonlite::fromJSON(paste(response, collapse = ""))
  
  if (is.null(months)) {
    return(result$downloads)
  }
  
  # daily downloads API: result$downloads is either data.frame or list-column
  downloads <- result$downloads
  
  if (is.data.frame(downloads)) {
    downloads_df <- downloads
  } else if (is.list(downloads) && length(downloads) > 0 && is.data.frame(downloads[[1]])) {
    downloads_df <- downloads[[1]]
  } else {
    return(0)
  }
  
  if (!"downloads" %in% names(downloads_df)) {
    return(0)
  }
  
  total_download <- sum(downloads_df$downloads, na.rm = TRUE)
  return(total_download)
}



# Bioconductor download


#' Get Bioconductor Package Download Statistics
#'
#' Downloads and processes monthly download statistics for a Bioconductor package
#' from the Bioconductor package statistics archive. Returns the full history,
#' the total downloads over the last 6 full months (current month excluded),
#' and the all-time total downloads.
#'
#' @param pkg Character string. The name of a Bioconductor package (e.g. "GenomicRanges").
#'
#' @return A list with three elements:
#' \describe{
#'   \item{all_data}{A data.frame of the full cleaned dataset with dates and cumulative counts.}
#'   \item{total_6_months}{Total downloads in the last 6 complete months.}
#'   \item{total_download}{Total all-time downloads.}
#' }
#'
#' @importFrom curl curl
#' @importFrom utils read.table
#' @importFrom dplyr filter mutate arrange select everything
#'
#' @examples
#' \dontrun{
#'   result <- get_package_download_bioconductor("bioconductorpackage")
#'   print(result$total_6_months)
#'   head(result$all_data)
#'   
#' #$all_data
#' #Date Year Month Nb_of_distinct_IPs Nb_of_downloads Cumulative_downloads
#' #1   2009-01-01 2009   Jan               3341            7053                 7053
#' #2   2009-02-01 2009   Feb               3229            6681                13734
#' #3   2009-03-01 2009   Mar               3753            8021                21755
#'   
#'   #$total_6_months
#'   #[1] 547560
#'   
#'   #$total_download
#'   #[1] 7318550
#' }
#'
#' @export
get_package_download_bioconductor <- function(pkg) {
  
  if (missing(pkg)) 
    stop("Please provide a bioconductor package name.")
  
  # Build the URL
  url <- sprintf(
    "https://bioconductor.org/packages/stats/bioc/%s/%s_stats.tab",
    pkg, pkg
  )
  
  Sys.setlocale("LC_TIME", "C")
  
  empty_list <- list(
    all_data = NULL,
    total_6_months = NULL,
    total_download = NULL
  )
  
  # Try to read the data
  read_result <- tryCatch(
    read.table(curl(url), header = TRUE),
    error = function(e) {
      message("could not fetch download data for package: ", pkg)
      return(empty_list)
    }
  )
  
  if (!is.data.frame(read_result) || nrow(read_result) == 0) {
    message("No download data found for package: ", pkg)
    return(empty_list)
  }
  
  current_date <- Sys.Date()
  start_of_current_month <- as.Date(format(current_date, "%Y-%m-01"))
  
  all_data <- read_result %>%
    filter(Month != "all") %>%
    mutate(
      Month = as.character(Month),
      Year = as.character(Year),
      Date = as.Date(paste(Year, Month, "01"), format = "%Y %b %d")
    ) %>%
    filter(Date < start_of_current_month) %>%
    arrange(Date) %>% 
    mutate(
      Cumulative_downloads = cumsum(Nb_of_downloads),
      Cumulative_IPs = cumsum(Nb_of_distinct_IPs)
    ) %>%
    select(Date, everything())
  
  
  # Get last 6 full months (excluding current)
  start_of_last_month <- as.Date(format(start_of_current_month - 1, "%Y-%m-01"))
  last_6_months_range <- seq(from = start_of_last_month, by = "-1 month", length.out = 6)
  
  last_6_months_data <- all_data %>%
    filter(Date %in% last_6_months_range) %>%
    arrange(Date)
  
  # Totals
  total_6_months <- sum(last_6_months_data$Nb_of_downloads, na.rm = TRUE)
  total_download <- sum(all_data$Nb_of_downloads, na.rm = TRUE)
  
  # Return as a list
  data <- list(
    all_data = all_data,
    total_6_months = total_6_months,
    total_download = total_download
  )
  
  return(data)
}

# CRAN

test_that("get_package_download_cran works with mocked API response", {
  # Mocked API JSON response as character vector (like from readLines)
  mock_json <- jsonlite::toJSON(list(
    downloads = list(
      list(day = "2024-04-22", downloads = 5000),
      list(day = "2024-04-21", downloads = 6000),
      list(day = "2024-04-20", downloads = 7000)
    )
  ), auto_unbox = TRUE)
  
  
  # Mock readLines to return fake API response
  mock_read_lines <- function(...) unlist(strsplit(mock_json, "\n"))
  
  # Mock Sys.Date to freeze the date
  mock_sys_date <- function() as.Date("2024-04-23")
  
  # Mock curl connection object (not used but needs to exist)
  mock_curl <- function(...) "fake_con"
  
  # Stub dependencies
  mockery::stub(get_package_download_cran, "readLines", mock_read_lines)
  mockery::stub(get_package_download_cran, "Sys.Date", mock_sys_date)
  mockery::stub(get_package_download_cran, "curl::curl", mock_curl)
  mockery::stub(get_package_download_cran, "close", function(x) NULL)
  
  # Run function
  result <- get_package_download_cran("ggplot2")
  
  # Check structure and content
  expect_type(result, "list")
  expect_s3_class(result, "data.frame")
  expect_named(result, c("date", "count", "cumulative_downloads"))
  expect_equal(nrow(result), 3)
  expect_equal(result$count, c(5000, 6000, 7000))
  expect_equal(result$cumulative_downloads, cumsum(c(5000, 6000, 7000)))
})

test_that("get_package_download_cran handles empty API response", {
  # Mock response with empty downloads
  mock_json <- jsonlite::toJSON(list(downloads = list()), auto_unbox = TRUE)
  mock_read_lines <- function(...) unlist(strsplit(mock_json, "\n"))
  mock_sys_date <- function() as.Date("2024-04-23")
  mock_curl <- function(...) "fake_con"
  
  mockery::stub(get_package_download_cran, "readLines", mock_read_lines)
  mockery::stub(get_package_download_cran, "Sys.Date", mock_sys_date)
  mockery::stub(get_package_download_cran, "curl::curl", mock_curl)
  mockery::stub(get_package_download_cran, "close", function(x) NULL)
  
  result <- get_package_download_cran("ggplot2")
  expect_null(result)
})

test_that("get_package_download_cran handles API error gracefully", {
  # Mock readLines to simulate an error
  mock_read_lines <- function(...) stop("Simulated API error")
  mock_sys_date <- function() as.Date("2024-04-23")
  mock_curl <- function(...) "fake_con"
  
  mockery::stub(get_package_download_cran, "readLines", mock_read_lines)
  mockery::stub(get_package_download_cran, "Sys.Date", mock_sys_date)
  mockery::stub(get_package_download_cran, "curl::curl", mock_curl)
  mockery::stub(get_package_download_cran, "close", function(x) NULL)
  
  result <- get_package_download_cran("ggplot2")
  expect_null(result)
})

test_that("get_package_download_cran fails when no package name is given", {
  expect_error(get_package_download_cran(), "Please provide a package name.")
})

test_that("Returns null when no downloads exist", {

  mock_json <- jsonlite::toJSON(list(
    downloads = list(
      list(downloads = NA, start = "2025-03-01", end = "2025-03-31", package = "nonexistent8package")
    )
  ), auto_unbox = TRUE)
  
  # Mock functions
  mock_read_lines <- function(...) unlist(strsplit(mock_json, "\n"))
  mock_sys_date <- function() as.Date("2025-03-31")
  mock_curl <- function(...) tempfile() 
  
  mockery::stub(get_package_download_cran, "readLines", mock_read_lines)
  mockery::stub(get_package_download_cran, "Sys.Date", mock_sys_date)
  mockery::stub(get_package_download_cran, "curl::curl", mock_curl)
  
  # Run the function and test the result
  result <- get_package_download_cran("nonexistent8package", years = 1)
  expect_null(result)
})

build_mock_response <- function(days, counts) {
  list(
    downloads = list(
      data.frame(
        day = days,
        downloads = counts,
        stringsAsFactors = FALSE
      )
    ),
    start = if (length(days) > 0) min(days) else NA_character_,
    end   = if (length(days) > 0) max(days) else NA_character_,
    package = "package"
  )
}

test_that("get_cran_total_downloads returns correct total for last 6 full months", {
  mock_data <- build_mock_response(
    days = c("2024-01-01", "2024-02-01", "2024-03-01", "2024-04-01", "2024-05-01", "2024-06-01"),
    counts = c(1000, 2000, 3000, 4000, 5000, 6000)
  )
  
  mock_json <- jsonlite::toJSON(mock_data, auto_unbox = TRUE)
  mock_read_lines <- function(...) mock_json
  mock_sys_date <- function() as.Date("2024-07-15")
  mock_curl <- function(...) "fake_con"
  
  mockery::stub(get_cran_total_downloads, "readLines", mock_read_lines)
  mockery::stub(get_cran_total_downloads, "Sys.Date", mock_sys_date)
  mockery::stub(get_cran_total_downloads, "curl::curl", mock_curl)
  mockery::stub(get_cran_total_downloads, "close", function(x) NULL)
  
  total <- get_cran_total_downloads("package", months = 6)
  expect_equal(total, 21000)
})


test_that("get_cran_total_downloads handles fewer than 6 months of data", {
  mock_data <- build_mock_response(
    days = c("2024-04-01", "2024-05-01"),
    counts = c(4000, 5000)
  )
  
  mock_json <- jsonlite::toJSON(mock_data, auto_unbox = TRUE)
  mock_read_lines <- function(...) mock_json
  mock_sys_date <- function() as.Date("2024-07-15")
  mock_curl <- function(...) "fake_con"
  
  mockery::stub(get_cran_total_downloads, "readLines", mock_read_lines)
  mockery::stub(get_cran_total_downloads, "Sys.Date", mock_sys_date)
  mockery::stub(get_cran_total_downloads, "curl::curl", mock_curl)
  mockery::stub(get_cran_total_downloads, "close", function(x) NULL)
  
  total <- get_cran_total_downloads("package", months = 6)
  expect_equal(total, 9000)
})


test_that("get_cran_total_downloads handles empty API response", {
  mock_data <- build_mock_response(
    days = character(0),
    counts = integer(0)
  )
  
  mock_json <- jsonlite::toJSON(mock_data, auto_unbox = TRUE)
  mock_read_lines <- function(...) mock_json
  mock_sys_date <- function() as.Date("2024-07-15")
  mock_curl <- function(...) "fake_con"
  
  mockery::stub(get_cran_total_downloads, "readLines", mock_read_lines)
  mockery::stub(get_cran_total_downloads, "Sys.Date", mock_sys_date)
  mockery::stub(get_cran_total_downloads, "curl::curl", mock_curl)
  mockery::stub(get_cran_total_downloads, "close", function(x) NULL)
  
  total <- get_cran_total_downloads("package", months = 6)
  expect_equal(total, 0)
})

test_that("get_cran_total_downloads handles API error gracefully", {
  mock_read_lines <- function(...) stop("Simulated API error")
  mock_sys_date <- function() as.Date("2024-07-15")
  mock_curl <- function(...) "fake_con"

  mockery::stub(get_cran_total_downloads, "readLines", mock_read_lines)
  mockery::stub(get_cran_total_downloads, "Sys.Date", mock_sys_date)
  mockery::stub(get_cran_total_downloads, "curl::curl", mock_curl)
  mockery::stub(get_cran_total_downloads, "close", function(x) NULL)

  expect_message(get_cran_total_downloads("package", months = 6), "URL not available")
})

test_that("get_cran_total_downloads handles URL unavailability", {

  mock_read_lines <- function(...) stop("Simulated connection error")
  mock_sys_date <- function() as.Date("2024-07-15")
  mock_curl <- function(...) "fake_con"
  mock_close <- function(x) NULL

  # Stub dependencies
  mockery::stub(get_cran_total_downloads, "readLines", mock_read_lines)
  mockery::stub(get_cran_total_downloads, "Sys.Date", mock_sys_date)
  mockery::stub(get_cran_total_downloads, "curl::curl", mock_curl)
  mockery::stub(get_cran_total_downloads, "close", mock_close)

  expect_message(
    result <- get_cran_total_downloads("fake245", months = 3),
    "URL not available"
  )
  expect_null(result)
})

test_that("get_cran_total_downloads throws on invalid months param", {
  expect_message(get_cran_total_downloads("package", months = -1), "months must be a positive integer")
  expect_message(get_cran_total_downloads("package", months = "six"), "months must be a positive integer")
})

# Bioconductor

test_that("get_package_download_bioconductor works with mocked read.table and date", {
  # Mock data frame with predictable months
  mock_data <- data.frame(
    Year = c(2023, 2023, 2023, 2024, 2024, 2024),
    Month = c("Oct", "Nov", "Dec", "Jan", "Feb", "Mar"),
    Nb_of_distinct_IPs = c(100, 200, 300, 400, 500, 600),
    Nb_of_downloads = c(1000, 2000, 3000, 4000, 5000, 6000) # sum 21 000
  )
  
  # Mock read.table() to return mock_data
  mock_read_table <- function(...) mock_data
  
  # Mock Sys.Date() to return a fixed date (e.g., 2024-04-15)
  mock_sys_date <- function() as.Date("2024-04-15")
  
  # Stub both functions inside get_package_download_bioconductor
  
  mockery::stub(get_package_download_bioconductor, "read.table", mock_read_table)
  mockery::stub(get_package_download_bioconductor, "Sys.Date", mock_sys_date)
  
  # Run the function
  result <- get_package_download_bioconductor("bio_package")
  
  # Check result
  expect_type(result, "list")
  expect_s3_class(result$all_data, "data.frame")
  expect_equal(nrow(result$all_data), 6)
  expected_months <- c("Oct", "Nov", "Dec", "Jan", "Feb", "Mar")
  expect_equal(format(result$all_data$Date, "%b"), expected_months)
  expect_equal(result$total_download, 21000)
  # Only the last 6 full months before April 2024: Oct 2023 to Mar 2024
  expect_equal(result$total_6_months, 21000)
})

test_that("get_package_download_bioconductor handles read.table error gracefully", {
  
  # Mock read.table to throw an error
  mock_read_table_error <- function(...) stop("Simulated download error")
  
  mockery::stub(get_package_download_bioconductor, "read.table", mock_read_table_error)
  result <- get_package_download_bioconductor("FakePackage123")

  expect_message(get_package_download_bioconductor("FakePackage123"), "could not fetch download data for package: FakePackage123")
  expect_message(get_package_download_bioconductor("FakePackage123"), "No download data found for package: FakePackage123")
  
  expect_null(result$all_data)
  expect_null(result$total_6_months)
  expect_null(result$total_download)
})

test_that("get_package_download_bioconductor handles empty download table", {
  # Mock read.table to return empty data
  mock_read_table_empty <- function(...) {
    data.frame(Year = numeric(0), Month = character(0), 
               Nb_of_distinct_IPs = numeric(0), Nb_of_downloads = numeric(0))
  }
  
  mockery::stub(get_package_download_bioconductor, "read.table", mock_read_table_empty)
  result <- get_package_download_bioconductor("EmptyDataPkg")
  expect_message(get_package_download_bioconductor("EmptyDataPkg"), "No download data found for package: EmptyDataPkg")
  
  expect_null(result$all_data)
  expect_null(result$total_download)
  expect_null(result$total_6_months)
})

test_that("get_package_download_bioconductor handles <6 months", {
  # Mock a small data frame (only 3 full months before April 2024)
  mock_data <- data.frame(
    Year = c(2024, 2024, 2024),
    Month = c("Jan", "Feb", "Mar"),
    Nb_of_distinct_IPs = c(100, 200, 300),
    Nb_of_downloads = c(1000, 2000, 3000) # sum 6000
  )
  
  # Mock read.table to return this partial data
  mock_read_table <- function(...) mock_data
  
  # Mock Sys.Date to simulate it's currently April 2024
  mock_sys_date <- function() as.Date("2024-04-15")
  
  # Patch both inside the function
  mockery::stub(get_package_download_bioconductor, "read.table", mock_read_table)
  mockery::stub(get_package_download_bioconductor, "Sys.Date", mock_sys_date)
  
  # Run function
  result <- get_package_download_bioconductor("NewPkg")
  
  # Expect all 3 rows returned
  expect_equal(nrow(result$all_data), 3)
  expect_equal(result$total_download, 6000)
  expect_equal(result$total_6_months, 6000)

  # Dates are correctly parsed
  expect_false(any(is.na(result$all_data$Date)))
})

test_that("get_package_download_bioconductor works with mocked read.table and date and excluded cuttent month", {
  # Mock data frame with predictable months
  mock_data <- data.frame(
    Year = c(2023, 2023, 2023, 2024, 2024, 2024, 2024, 2024, 2024),
    Month = c("Oct", "Nov", "Dec", "Jan", "Feb", "Mar", "Apr", "May", "Jun"),
    Nb_of_distinct_IPs = c(100, 200, 300, 400, 500, 600, 650, 700, 0),
    Nb_of_downloads = c(1000, 2000, 3000, 4000, 5000, 6111, 6500, 7000, 0) # sum 42111
  )
  
  # Mock read.table() to return mock_data
  mock_read_table <- function(...) mock_data
  
  # Mock Sys.Date() to return a fixed date (e.g., 2024-04-15)
  mock_sys_date <- function() as.Date("2024-06-15") # Jun
  
  # Stub both functions inside get_package_download_bioconductor
  
  mockery::stub(get_package_download_bioconductor, "read.table", mock_read_table)
  mockery::stub(get_package_download_bioconductor, "Sys.Date", mock_sys_date)
  
  # Run the function
  result <- get_package_download_bioconductor("bio_package")
  
  # Check result
  expect_type(result, "list")
  expect_s3_class(result$all_data, "data.frame")
  expect_equal(nrow(result$all_data), 8)
  expect_equal(result$total_download, 34611)
  
  # Only the last 6 full months before April 2024: Oct 2023 to Mar 2024
  expect_equal(result$total_6_months, 31611)
})

test_that("get_package_download_bioconductor works with excluded future months", {
  # Mock data frame with predictable months
  mock_data <- data.frame(
    Year = c(2023, 2023, 2023, 2024, 2024, 2024, 2024, 2024, 2024),
    Month = c("Oct", "Nov", "Dec", "Jan", "Feb", "Mar", "Apr", "May", "Jun"),
    Nb_of_distinct_IPs = c(100, 200, 0, 0, 0, 0, 0, 0, 0),
    Nb_of_downloads = c(101, 200, 0, 0, 0, 0, 0, 0, 0)
  )

  # Mock read.table() to return mock_data
  mock_read_table <- function(...) mock_data

  # Mock Sys.Date() to return a fixed date (e.g., 2024-04-15)
  mock_sys_date <- function() as.Date("2023-12-15")

  # Stub both functions inside get_package_download_bioconductor

  mockery::stub(get_package_download_bioconductor, "read.table", mock_read_table)
  mockery::stub(get_package_download_bioconductor, "Sys.Date", mock_sys_date)

  # Run the function
  result <- get_package_download_bioconductor("bio_package")

  # Check result
  expect_type(result, "list")
  expect_s3_class(result$all_data, "data.frame")
  expect_equal(nrow(result$all_data), 2)
  expect_equal(result$total_download, 301)

  # Only the last 6 full months before April 2024: Oct 2023 to Mar 2024
  expect_equal(result$total_6_months, 301)
})

test_that("get_package_download_cran fails when no package name is given", {
  expect_error(get_package_download_bioconductor(), "Please provide a bioconductor package name.")
})

#' Extract risk thresholds by id
#'
#' @param risk_rule - list containing all risk rules
#' @param id_value - character vector containing risk rule choice
#'
#' @return list containing specific risk rules
#' @export
extract_thresholds_by_id <- function(risk_rule, id_value) {
  idx <- which(vapply(risk_rule, function(x) x$id == id_value, logical(1)))
  if (length(idx) == 1) {
    return(risk_rule[[idx]]$thresholds)
  }
  return(NULL)
}

#' Extract risk thresholds by key
#'
#' @param risk_rule - list containing all risk rules
#' @param key_value - character vector containing risk rule choice
#'
#' @return list containing specific risk rules
#' @export
extract_thresholds_by_key <- function(risk_rule, key_value) {
  idx <- which(vapply(risk_rule, function(x) x$key == key_value, logical(1)))
  if (length(idx) == 1) {
    return(risk_rule[[idx]]$thresholds)
  }
  return(NULL)
}

test_that("extract_thresholds_by_id returns correct thresholds", {
  # Mocked risk rule data
  mock_risk_rules <- list(
    list(id = "code_coverage", thresholds = list(
      list(level = "high", max = 90),
      list(level = "medium", max = 75)
    )),
    list(id = "doc_coverage", thresholds = list(
      list(level = "high", max = 85),
      list(level = "medium", max = 70)
    ))
  )
  
  # Direct test of extract_thresholds_by_id
  result <- extract_thresholds_by_id(mock_risk_rules, "code_coverage")
  expect_type(result, "list")
  expect_equal(length(result), 2)
  expect_equal(result[[1]]$level, "high")
  expect_equal(result[[1]]$max, 90)
  expect_equal(result[[2]]$level, "medium")
  expect_equal(result[[2]]$max, 75)
})

test_that("extract_thresholds_by_id returns NULL when id not found", {
  mock_risk_rules <- list(
    list(id = "doc_coverage", thresholds = list(
      list(level = "high", max = 85),
      list(level = "medium", max = 70)
    ))
  )
  
  result <- extract_thresholds_by_id(mock_risk_rules, "code_coverage")
  expect_null(result)
})


test_that("extract_threshold_by_key returns correct thresholds", {
  # Mocked risk rule data
  mock_risk_rules <- list(
    list(
      key = "total_download",
      thresholds = list(
        list(level = "high", max = 3500000),
        list(level = "medium", max = 50000000),
        list(level = "low", max = NULL)
      )
    ),
    list(
      key = "stars",
      thresholds = list(
        list(level = "high", max = 1000),
        list(level = "medium", max = 500)
      )
    )
  )
  
  # Direct test of extract_threshold_by_key
  result <- extract_thresholds_by_key(mock_risk_rules, "total_download")
  expect_type(result, "list")
  expect_equal(length(result), 3)
  expect_equal(result[[1]]$level, "high")
  expect_equal(result[[1]]$max, 3500000)
  expect_equal(result[[3]]$level, "low")
  expect_null(result[[3]]$max)
})

test_that("extract_threshold_by_key returns NULL when key not found", {
  mock_risk_rules <- list(
    list(
      key = "stars",
      thresholds = list(
        list(level = "high", max = 1000),
        list(level = "medium", max = 500)
      )
    )
  )
  
  result <- extract_thresholds_by_key(mock_risk_rules, "total_download")
  expect_null(result)
})

#' Creates information on package installation
#'
#' @param dp data path and name for the package. 
#' @param check_type basic R CMD check type - "1" CRAN R CMD check_type - "2"
#' 
#' @return list with local package install
#' @examples
#' \dontrun{
#' set_up_pkg(path/to/package, "mypackage")
#' }
#' @export
set_up_pkg <- function(dp, check_type = "1") {
  
  build_vignettes <- TRUE
  
  suppressWarnings(pkg_source_path <- unpack_tarball(dp))
  
  # check for vignettes folder
  bv_result <- contains_vignette_folder(dp)
  
  #set up build vignettes for R CMD check
  if (bv_result == FALSE) {
    build_vignettes <- TRUE
  } else {
    build_vignettes <- FALSE
  }
  
  if (length(pkg_source_path) == 0) {
    package_installed <- FALSE
    results <- ""
    pkg_source_path <- ""
    out_dir <- ""
    build_vignettes <- ""
  } else { 
    if (fs::file_exists(pkg_source_path)) {
      package_installed <- TRUE
    }  
  } 
  
  if (package_installed == TRUE ) {	
    
    rcmdcheck_args <- setup_rcmdcheck_args(check_type, build_vignettes)
  } 
  
  install_list <- list(
    build_vignettes = build_vignettes,
    package_installed = package_installed,
    pkg_source_path = pkg_source_path,
    rcmdcheck_args = rcmdcheck_args
  )
  return(install_list)
}

test_that("set up package for tar file with default check type", {
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  install_list <- set_up_pkg(dp)
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(install_list$pkg_source_path, 
                      recursive = TRUE, force = TRUE), 
                      envir = parent.frame())
  
  expect_identical(length(install_list), 4L)
  expect_true(checkmate::check_list(install_list, any.missing = FALSE))
  expect_true(checkmate::check_list(install_list, types = c("logical", 
                                                            "character", 
                                                            "list")))
})


test_that("set up package for tar file with check type 1", {
  dp_orig <- system.file("test-data", "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  withr::defer(unlink(dp), envir = parent.frame())
  
  check_type <- "1"
  install_list <- set_up_pkg(dp, check_type)
  
  withr::defer(unlink(install_list$pkg_source_path, 
                      recursive = TRUE, force = TRUE), envir = parent.frame())
  
  expect_identical(length(install_list), 4L)
  expect_true(checkmate::check_list(install_list, any.missing = FALSE))
  expect_true(checkmate::check_list(install_list, types = c("logical", 
                                                            "character", 
                                                            "list")))
})


test_that("set up package for tar file with check type 1", {    
  dp <- system.file("test-data", "test.package.0001_0.1.0.tar.gz", 
                    package = "risk.assessr")
  
  check_type <- "1"
  
  # set up package
  install_list <- set_up_pkg(dp, 
                                                 check_type 
                                                 )
  
  expect_identical(length(install_list), 4L)
  
  expect_true(checkmate::check_list(install_list, 
                                    any.missing = FALSE)
  )
  
  expect_true(checkmate::check_list(install_list, 
                                    types = c("logical",
                                              "character",
                                              "list")))
  
})  


test_that("set up package for tar file with check type 2", {
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  withr::defer(unlink(dp), envir = parent.frame())
  
  check_type <- "2"
  install_list <- set_up_pkg(dp, check_type)
  
  withr::defer(unlink(install_list$pkg_source_path, 
                      recursive = TRUE, force = TRUE), 
               envir = parent.frame())
  
  expect_identical(length(install_list), 4L)
  expect_true(checkmate::check_list(install_list, any.missing = FALSE))
  expect_true(checkmate::check_list(install_list, types = c("logical", 
                                                            "character", 
                                                            "list")))
})


test_that("set_up_pkg handles zero-length pkg_source_path", {
  # Create a dummy tarball path
  dp <- tempfile(fileext = ".tar.gz")
  file.create(dp)
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Stub unpack_tarball to simulate failure
  mockery::stub(set_up_pkg, "unpack_tarball", function(...) character(0))
  
  # Stub contains_vignette_folder to return FALSE
  mockery::stub(set_up_pkg, "contains_vignette_folder", function(...) FALSE)
  
  # Stub setup_rcmdcheck_args to avoid error if called
  mockery::stub(set_up_pkg, "rcmdcheck_args", 
                function(...) list(timeout = Inf, 
                                   args = "--no-manual", 
                                   build_args = NULL, 
                                   env = "mockenv", 
                                   quiet = TRUE))
  
  install_list <- set_up_pkg(dp)
  
  expect_identical(length(install_list), 4L)
  expect_true(checkmate::check_list(install_list, any.missing = FALSE))
  expect_true(checkmate::check_list(install_list, 
                                    types = c("logical", 
                                              "character", 
                                              "list",
                                              "function")))
})


test_that("set_up_pkg sets build_vignettes to TRUE when bv_result is FALSE", {
  dp <- tempfile(fileext = ".tar.gz")
  file.create(dp)
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Stub unpack_tarball to return a dummy path
  mockery::stub(set_up_pkg, "unpack_tarball", function(...) "mock/path")
  
  # Stub contains_vignette_folder to return FALSE
  mockery::stub(set_up_pkg, "contains_vignette_folder", function(...) FALSE)
  
  # Stub fs::file_exists to return TRUE
  mockery::stub(set_up_pkg, "fs::file_exists", function(...) TRUE)
  
  # Stub setup_rcmdcheck_args to return dummy args
  mockery::stub(set_up_pkg, "rcmdcheck_args", 
                function(...) list(timeout = Inf, 
                                   args = "--no-manual", 
                                   build_args = NULL, 
                                   env = "mockenv", 
                                   quiet = TRUE))
  
  install_list <- set_up_pkg(dp)
  
  expect_true(install_list$build_vignettes)
})


test_that("set_up_pkg sets build_vignettes to FALSE when bv_result is TRUE", {
  dp <- tempfile(fileext = ".tar.gz")
  file.create(dp)
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Stub unpack_tarball to return a dummy path
  mockery::stub(set_up_pkg, "unpack_tarball", function(...) "mock/path")
  
  # Stub contains_vignette_folder to return TRUE
  mockery::stub(set_up_pkg, "contains_vignette_folder", function(...) TRUE)
  
  # Stub fs::file_exists to return TRUE
  mockery::stub(set_up_pkg, "fs::file_exists", function(...) TRUE)
  
  # Stub setup_rcmdcheck_args to return dummy args
  mockery::stub(set_up_pkg, "rcmdcheck_args", 
                function(...) list(timeout = Inf, 
                                   args = "--no-manual", 
                                   build_args = NULL, 
                                   env = "mockenv", 
                                   quiet = TRUE))
  
  install_list <- set_up_pkg(dp)
  
  expect_false(install_list$build_vignettes)
})

# setting R_TESTS to empty string because of
# https://github.com/hadley/testthat/issues/144
# and https://github.com/r-lib/testthat/issues/86
# revert this when that issue in R is fixed.
Sys.setenv("R_TESTS" = "")

library(testthat)
library(mockery)
library(risk.assessr)
library(withr)
library(dplyr)
library(jsonlite)
library(fs)
library(curl)
library(xml2)

test_check("risk.assessr")

#' create empty tm
#' 
#' @param pkg_name - name of package
#' @return empty_tm - list with no tm
#'
#'@keywords internal
create_empty_tm <- function(pkg_name) {
  
  coverage <- list(
    filecoverage = 0,
    totalcoverage = 0
  )
  
  empty_tm <- list(
    pkg_name = pkg_name,
    coverage = coverage,
    errors = NA,
    notes = NA
  )
  
  message(glue::glue("traceability matrix for {pkg_name} unsuccessful"))
  return(empty_tm)
}

test_that("test create_empty_tm ", {
  
  pkg_name <- ""
  
  empty_tm <- create_empty_tm(pkg_name)
  
  expect_identical(length(empty_tm), 4L)
  
  expect_true(checkmate::check_list(empty_tm, all.missing = TRUE))
  
  df <- empty_tm |> 
    as.data.frame()
  
  expect_identical(length(df), 5L)
  
  expect_true(checkmate::check_data_frame(df, all.missing = TRUE))
  
  
})

#' @title Map the hierarchy structure of tests directory in standard testing framework
#' @description Query a test directory for the unit test structure.
#' @param path character, path to tests, Default: 'tests/testthat'
#' @details Return data.frame containing which expectations are in the testthat directory
#' by file/context/description/test/expectation/linerange
#' @return data.frame
#' @seealso
#' \code{\link[base]{names}}, \code{\link[base]{lapply}}, \code{\link[base]{rbind}}
#' @rdname map_testthat
#' @family utility
#' @export
map_tests_stf <- function(path = "tests/testthat") {
  
  # Input checking
  checkmate::assert_string(path)
  checkmate::assert_directory_exists(path)
  
  # Step 1: List all test files in the specified directory that match the pattern "test*.R"
  files <- list.files(path, full.names = TRUE, pattern = "^test(.*?)R$")
  
  # Step 2: Apply the map_test function to each file and store results in a named list
  results_list <- lapply(files, map_test)
  names(results_list) <- basename(files) # Assign file names as list names
  
  # Step 3: Remove entries with no content (length == 0)
  results_list <- results_list[sapply(results_list, length) > 0]
  
  # Step 4: If no valid test files were found, return NULL
  if (length(results_list) == 0) {
    return(NULL)
  }
  
  # Step 5: Add a 'file' column to each data.frame in the list using the file name
  results_list <- lapply(names(results_list), function(name) {
    df <- results_list[[name]]
    df$file <- name
    df
  })
  
  # Step 6: Combine all individual data.frames into one consolidated data.frame
  results <- do.call("rbind", results_list)
  
  # Step 7: Identify rows where the 'test' column is not just a space
  idx <- !(results$test == " ")
  
  # Step 8: Clean up the 'description' and 'test' columns by removing prefixes
  results$description <- gsub("describe: |test_that: ", "", results$description)
  results$test <- gsub("it: ", "", results$test)
  
  # Step 9: For rows without a specific test, use the description as the test name
  results$test[!idx] <- results$description[!idx]
  
  # Step 10: For rows with both description and test, combine them into a single string
  results$test[idx] <- sprintf("%s: %s", results$description[idx], results$test[idx])
  
  # Step 11: Remove the 'description' column as it's now merged into 'test'
  results$description <- NULL
  
  # Step 12: Return the final structured data.frame
  return(results)
}


#' @title Nest parsed testthat code into logical test blocks
#' @description Internal helper function to group parsed testthat code into blocks based on context, test_that, describe, or it calls.
#' @param x data.frame, parse data from utils::getParseData
#' @param token_text character, regex pattern to match test block function names, Default: '^context$'
#' @return named list of data.frames, each representing a nested test block
#' @keywords internal
#' @importFrom utils head tail
nest_test <- function(x, token_text = "^context$") {
 
  # Store row names of the parse data
  rx <- rownames(x)
  
  # Find indices of parent expressions that match the specified token_text (e.g., context)
  idx <- which(rx %in% x$parent[grepl("^SYMBOL_FUNCTION_CALL$", x$token) & grepl(token_text, x$text) & x$terminal])
  
  # Refine indices to get top-level expressions for those matches
  idx <- which(rx %in% x$parent[idx])
  
  # Initialize a grouping vector
  x1 <- rep(0, nrow(x))
  
  # Mark the start of each new block
  x1[idx] <- 1
  
  # Create a cumulative sum to assign group IDs
  x1 <- cumsum(x1)
  
  # Split the parse data into groups based on the group IDs
  x2 <- split(x, x1)
  
  # Assign names to each group based on the function call and its string argument
  names(x2) <- sapply(x2, function(x) {
    # Get the function name (e.g., test_that, describe, it)
    add_text <- x$text[grep("^SYMBOL_FUNCTION_CALL$", x$token)[1]]
    
    # Evaluate the string constant (e.g., test description)
    ret <- eval(parse(text = x$text[grepl("^STR_CONST$", x$token)][1], keep.source = TRUE))
    
    # Prefix the name with the function name if it's a known test block
    if (add_text %in% c("test_that", "describe", "it")) {
      ret <- sprintf("%s: %s", add_text, ret)
    }
    
    ret
  })
  
  # Trim each group to remove leading parse noise and keep only the relevant block
  x2 <- lapply(x2, function(x) {
    # Find the second top-level expression (if available)
    idx <- utils::tail(utils::head(which(x$parent == 0), 2), 1)
    if (length(idx) == 0) {
      idx <- 1
    }
    # Return the block from the identified index to the end
    x[idx:nrow(x), ]
  })
  
  x2
}


#' @title Extract expectation calls from parsed test code
#' @description Internal helper function to identify and extract expectation function calls (e.g., expect_equal) from parsed testthat code.
#' @param x data.frame, parse data from utils::getParseData
#' @param token_text character, regex pattern to match expectation function names, Default: '^expect_'
#' @return data.frame with expectation names and their line ranges
#' @keywords internal
get_expect <- function(x, token_text = "^expect_") {
  
  # Identify indices of function calls that match the expectation pattern
  idx <- which(grepl("^SYMBOL_FUNCTION_CALL$", x$token) & grepl(token_text, x$text) & x$terminal)
  
  # Extract the matched expectation function names
  ret <- x$text[idx]
  
  if (length(ret) == 0) {
    return(NULL)
  }
  
  # For each matched expectation, find the corresponding expression's line range
  line_ <- lapply(idx, function(y) {
    # Find the last two 'expr' tokens before the expectation
    this_idx <- tail(grep("expr", x$token[1:y]), 2)
    # Determine which of those expressions contains the expectation
    this_exp <- min(grep(sprintf("%s", x$text[y]), x$text[this_idx]))
    # Extract the line range for that expression
    x[this_idx[this_exp], c("line1", "line2")]
  })
  
  # Combine all line ranges into a single data.frame
  line_ <- do.call("rbind", line_)
  
  # Construct the final data.frame with expectation names and line numbers
  ret <- data.frame(
    expectation = ret,
    line1 = line_$line1,
    line2 = line_$line2,
    stringsAsFactors = FALSE
  )
  
  ret
}

#' @title Extract and structure expectations from test blocks
#' @description Internal helper function to extract expectations from a list of test blocks.
#' @param x list of parsed test blocks
#' @return data.frame containing structured expectations with test labels
#' @keywords internal
nest_expect <- function(x) {
  
  # Apply get_expect to each test block to extract expectations
  ret <- lapply(x, get_expect)
  
  # Remove any NULL results (i.e., blocks with no expectations)
  ret <- ret[!sapply(ret, is.null)]
  
  if (length(ret) == 0) {
    return(NULL)
  }
  
  # Add a 'test' column to each expectation block using the list names
  ret <- lapply(names(ret), unrowname, ret = ret, label = "test")
  
  # Combine all expectation blocks into a single data.frame
  ret <- do.call("rbind", ret)
  
  ret
}

#' @title Map the hierarchy structure of a testthat test file
#' @description Query a test file to see what expectations are defined in it.
#' @param path character, path to file
#' @details Return data.frame containing which expectations are in the test file
#'  by context/description/test/expectation/linerange
#' @return data.frame
#' @seealso
#'  \code{\link[utils]{getParseData}}
#' @rdname map_test
#' @family utility
#' @export
#' @importFrom utils getParseData
map_test <- function(path) {
  
  # Parse the test file and extract parse data including source text
  x <- utils::getParseData(parse(path, keep.source = TRUE), includeText = TRUE)
  if (is.null(x)) return(NULL)
  
  # Treat test_that and describe as top-level test blocks
  ret <- lapply(nest_test(x, token_text = "^test_that$|^describe$"), function(xx) {
    # Identify the function name (e.g., test_that or describe)
    SYMB <- xx$text[grep("^SYMBOL_FUNCTION_CALL$", xx$token)[1]]
    
    # Depending on the function, process expectations differently
    switch(SYMB,
           # Nest 'it' blocks inside describe
           describe = {
             nest_expect(nest_test(xx, token_text = "^it$"))
           },
           test_that = {
             # Wrap test_that block in a list with a placeholder name
             nest_expect(setNames(list(xx), " "))
           }, {
             list()
           }
    )
  })
  
  # Remove empty elements
  ret <- ret[sapply(ret, length) > 0]
  if (length(ret) == 0) return(NULL)
  
  
  # Ensure unique names for test blocks
  names(ret) <- make.unique(names(ret), sep = "_")
  
  # Add 'description' column to each test block
  ret <- lapply(names(ret), unrowname, ret = ret, label = "description")
  
  # Combine all test blocks into a single data.frame
  ret <- do.call("rbind", ret)
  return(ret)
}

#' @title Add identifier column and remove row names from a data.frame
#' @description Internal helper function to restructure a data.frame by adding an identifier column and removing row names.
#' @param identifier character, the name or label to assign to the new column
#' @param ret list of data.frames, typically the result of applying a function over multiple files
#' @param label character, name of the new column to store the identifier
#' @return data.frame with identifier column prepended and row names removed
#' @keywords internal
unrowname <- function(identifier, ret, label) {
  # Extract the data.frame corresponding to the identifier
  results <- ret[[identifier]]
  
  # Get the number of columns in the original data.frame
  nc <- ncol(results)
  
  # Add a new column with the identifier value (e.g., file name)
  results[[label]] <- identifier
  
  
  # Remove row names to ensure clean output
  rownames(results) <- NULL
  
  # Reorder columns to place the identifier column first
  results <- results[, c(c(nc + 1), 1:nc)]
  
  return(results)
}

test_that("nest_test correctly groups and names test blocks", {

  
  
  mock_parse_data <- data.frame(
    line1 = 1:6,
    token = c(
      "SYMBOL_FUNCTION_CALL", "STR_CONST", "expr", "expr", "SYMBOL_FUNCTION_CALL", "STR_CONST"
    ),
    text = c(
      "test_that", "\"first test\"", "expr1", "expr2", "describe", "\"second test\""
    ),
    parent = c(0, 1, 1, 1, 0, 5),
    terminal = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    stringsAsFactors = FALSE
  )
  rownames(mock_parse_data) <- as.character(1:nrow(mock_parse_data))
  
    
  # Stub eval(parse(...)) to return the string inside quotes
  mockery::stub(nest_test, "eval", function(expr, keep.source = TRUE) {
      gsub("\"", "", as.character(expr))
    })
  
  result <- nest_test(mock_parse_data)
  
  expect_type(result, "list")
  expect_equal(length(result), 1)
  expect_named(result, "test_that: first test")
  
  expect_s3_class(result[[1]], "data.frame")
  expect_equal(nrow(result[[1]]), 2)
  
})


test_that("get_expect extracts expectation calls and line ranges without NA", {
  
  mock_parse_data <- data.frame(
    line1 = c(1, 2, 3, 4, 5, 6, 7, 8),
    line2 = c(1, 2, 3, 4, 5, 6, 7, 8),
    token = c(
      "expr", "expr", "SYMBOL_FUNCTION_CALL", "expr",
      "expr", "SYMBOL_FUNCTION_CALL", "expr", "expr"
    ),
    text = c(
      "x <- 1", "expect_equal", "expect_equal", "y <- 2",
      "expect_error", "expect_error", "z <- 3", "w <- 4"
    ),
    parent = c(0, 0, 2, 3, 0, 5, 5, 5),
    terminal = rep(TRUE, 8),
    stringsAsFactors = FALSE
  )
  
  # Stub grep inside the function to simulate matching expressions
  mockery::stub(get_expect, "grep", function(pattern, x) {
    # Simulate finding the expectation function in the expression
    which(grepl(pattern, x))
  })
    
  result <- get_expect(mock_parse_data)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_equal(result$expectation, c("expect_equal", "expect_error"))
  expect_false(any(is.na(result$line1)))
  expect_false(any(is.na(result$line2)))

})

test_that("nest_expect combines expectations from multiple test blocks", {
  

  # Simulated output of get_expect()
  mock_expect_1 <- data.frame(
    expectation = c("expect_equal", "expect_true"),
    line1 = c(10, 12),
    line2 = c(10, 12),
    stringsAsFactors = FALSE
  )
  
  mock_expect_2 <- data.frame(
    expectation = c("expect_error"),
    line1 = c(20),
    line2 = c(20),
    stringsAsFactors = FALSE
  )
  
  # Simulated input to nest_expect (like output from nest_test)
  mock_test_blocks <- list(
    "test_that: test 1" = data.frame(dummy = 1),
    "test_that: test 2" = data.frame(dummy = 2)
  )

  
  # Stub get_expect to return mocked expectations
  mockery::stub(nest_expect, "get_expect", function(x) {
    if (identical(x$dummy, 1)) return(mock_expect_1)
    if (identical(x$dummy, 2)) return(mock_expect_2)
    NULL
  })
  
  # Stub unrowname to add a 'test' column
  mockery::stub(nest_expect, "unrowname", function(name, ret, label) {
    df <- ret[[name]]
    df[[label]] <- name
    df
  })
  
  result <- nest_expect(mock_test_blocks)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 3)
  expect_equal(result$expectation, c("expect_equal", "expect_true", "expect_error"))
  expect_equal(result$test, c("test_that: test 1", "test_that: test 1", "test_that: test 2"))
})

test_that("unrowname correctly processes the data frame", {
  # Mock data
  df1 <- data.frame(A = 1:3, B = 4:6, row.names = c("a", "b", "c"))
  ret <- list(file1 = df1)
  
  # Call the function
  result <- unrowname("file1", ret, "source")
  
  # Expectations
  expect_equal(ncol(result), 3)
  expect_equal(colnames(result), c("source", "A", "B"))
  expect_equal(result$source, rep("file1", 3))
})


test_that("map_tests_stf processes multiple test files correctly", {
  # Mock file paths
  mock_files <- c(
    "C:/Users/AppData/Local/Temp/test.package.0014/tests/testthat/test-myscript1.R",
    "C:/Users/AppData/Local/Temp/test.package.0014/tests/testthat/test-myscript2.R",
    "C:/Users/AppData/Local/Temp/test.package.0014/tests/testthat/test-myscript3.R"
  )
  
  # Mock return values from map_test
  mock_df1 <- data.frame(
    description = c("test_that: two working tests", "test_that: two working tests", "test_that: three working tests", "test_that: three working tests", "test_that: three working tests"),
    test = rep("expect_equal", 5),
    expectation = rep("expect_equal", 5),
    line1 = c(2, 4, 8, 10, 12),
    line2 = c(2, 4, 8, 10, 12),
    stringsAsFactors = FALSE
  )
  
  mock_df2 <- data.frame(
    description = c("test_that: test 1 works test 2 fails", "test_that: test 1 works test 2 fails",
                    "test_that: test 1 fails, tests 2 & 4  work, test 3 fails",
                    "test_that: test 1 fails, tests 2 & 4  work, test 3 fails",
                    "test_that: test 1 fails, tests 2 & 4  work, test 3 fails",
                    "test_that: test 1 fails, tests 2 & 4  work, test 3 fails"),
    test = rep("expect_equal", 6),
    expectation = rep("expect_equal", 6),
    line1 = c(2, 4, 8, 10, 12, 14),
    line2 = c(2, 4, 8, 10, 12, 14),
    stringsAsFactors = FALSE
  )
  
  mock_df3 <- data.frame(
    description = c("test_that: tests work", "test_that: tests work", "test_that: tests work",
                    "test_that: tests work_1", "test_that: tests work_1"),
    test = rep("expect_equal", 5),
    expectation = rep("expect_equal", 5),
    line1 = c(2, 4, 6, 10, 12),
    line2 = c(2, 4, 6, 10, 12),
    stringsAsFactors = FALSE
  )
  
  # Stub dependencies
  mockery::stub(map_tests_stf, "checkmate::assert_string", function(x) TRUE)
  mockery::stub(map_tests_stf, "checkmate::assert_directory_exists", function(x) TRUE)
  mockery::stub(map_tests_stf, "list.files", function(path, full.names, pattern) mock_files)
  mockery::stub(map_tests_stf, "map_test", function(path) {
    if (grepl("myscript1", path)) return(mock_df1)
    if (grepl("myscript2", path)) return(mock_df2)
    if (grepl("myscript3", path)) return(mock_df3)
    return(NULL)
  })
  
  # Run the function
  result <- map_tests_stf("tests/testthat")
  
  # Check structure
  expect_true(is.data.frame(result))
  expect_equal(ncol(result), 5)
  expect_equal(unique(result$file), basename(mock_files))
  expect_true(all(c("test", "expectation", "line1", "line2", "file") %in% colnames(result)))
})

test_that("map_tests_stf processes test files correctly", {
  # Step 1: Mock list.files to simulate test files
  mockery::stub(map_tests_stf, "list.files", function(path, full.names, pattern) {
    return(c("tests/testthat/test-one.R", "tests/testthat/test-two.R"))
  })
  
  # Step 2: Mock checkmate assertions
  mockery::stub(map_tests_stf, "checkmate::assert_string", function(x) TRUE)
  mockery::stub(map_tests_stf, "checkmate::assert_directory_exists", function(x) TRUE)
  
  # Step 3: Mock map_test to return named data.frames
  mock_df1 <- data.frame(description = "desc1", test = " ", stringsAsFactors = FALSE)
  mock_df2 <- data.frame(description = "desc2", test = "it: does something", stringsAsFactors = FALSE)
  
  mockery::stub(map_tests_stf, "map_test", function(path) {
    if (grepl("test-one", path)) return(mock_df1)
    if (grepl("test-two", path)) return(mock_df2)
    return(NULL)
  })
  
  # Step 4: Run the function
  result <- map_tests_stf("tests/testthat")
  
  # Step 5: Assertions
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 2)
  expect_equal(result$file, c("test-one.R", "test-two.R"))
  expect_equal(result$test, c("desc1", "desc2: does something"))
})


#Sample mocked parse data for a test_that block
mock_parse_data_test_that <- data.frame(
  token = c("SYMBOL_FUNCTION_CALL", "expr", "expr"),
  text = c("test_that", "expect_true(TRUE)", "expect_equal(1, 1)"),
  stringsAsFactors = FALSE
)

# Sample mocked parse data for a describe/it block
mock_parse_data_describe <- data.frame(
  token = c("SYMBOL_FUNCTION_CALL", "expr", "SYMBOL_FUNCTION_CALL", "expr"),
  text = c("describe", "context", "it", "expect_true(TRUE)"),
  stringsAsFactors = FALSE
)

test_that("map_test returns expected structure for test_that block", {
  mockery::stub(map_test, "parse", function(...) NULL)  # parse is not used directly
  mockery::stub(map_test, "utils::getParseData", function(...) mock_parse_data_test_that)
  mockery::stub(map_test, "nest_test", function(x, token_text) setNames(list(x), "test_that"))
  mockery::stub(map_test, "nest_expect", function(x) setNames(list(x[[1]]), "test_that"))
  mockery::stub(map_test, "unrowname", function(name, ret, label) {
    df <- ret[[name]]
    df$description <- name
    as.data.frame(df, stringsAsFactors = FALSE)
  })
  
  
  result <- map_test("dummy_path.R")
  expect_true(is.data.frame(result))
  expect_equal(result$description[1], "test_that")
  expect_equal(result$description[2], "test_that")
  expect_equal(result$description[3], "test_that")
})

#' get skipped tests with line locations
#'
#' @param problem_tests - problem tests
#' @param test_map - all tests with line locations
#'
#' @return test_skip
#' @keywords internal
get_tests_skip_stf <- function(problem_tests, test_map) {
  
  # check for 'problem_tests' data frame
  checkmate::assert_data_frame(problem_tests, any.missing = TRUE, min.rows = 1)
  checkmate::assert_names(names(problem_tests), must.include = c("file", "context"))
  
  # check for 'test_map' data frame
  checkmate::assert_data_frame(test_map, any.missing = FALSE, min.rows = 1)
  checkmate::assert_names(names(test_map), must.include = c("file", "context"))
  
  
  # Filter test_map to only include rows matching both context and file in problem_tests
  test_map_filtered <- test_map[
    test_map$context %in% problem_tests$context &
      test_map$file %in% problem_tests$file,
  ]
  
  # Perform the join safely
  test_skip <- dplyr::left_join(problem_tests, test_map_filtered, by = c("context", "file"))
  
  return(test_skip)
}

test_that("get_tests_skip_stf joins problem_tests with test_map correctly", {
  problem_tests <- data.frame(
    file = c("test-a.R", "test-b.R"),
    context = c("context1", "context2"),
    stringsAsFactors = FALSE
  )
  
  test_map <- data.frame(
    file = c("test-a.R", "test-b.R", "test-c.R"),
    context = c("context1", "context2", "context3"),
    line = c(10, 20, 30),
    stringsAsFactors = FALSE
  )
  
  mockery::stub(get_tests_skip_stf, "checkmate::assert_data_frame", NULL)
  mockery::stub(get_tests_skip_stf, "checkmate::assert_names", NULL)
  
  result <- get_tests_skip_stf(problem_tests, test_map)
  
  expect_equal(nrow(result), 2)
  expect_equal(result$line, c(10, 20))
})

#' Generate HTML Report for Package Assessment
#'
#' @description Generates an HTML report for the package assessment results using rmarkdown.
#'
#' @param assessment_results List containing the results from risk_assess_pkg function.
#' @param output_dir Character string indicating the directory where the report will be saved. 
#'
#' @return Path to the generated HTML report.
#'
#' @examples
#' \dontrun{
#' assessment_results <- risk_assess_pkg()
#' generate_html_report(assessment_results, output_dir = "path/to/save/report")
#' }
#' @importFrom rmarkdown render
#' @importFrom fs dir_exists
#' @importFrom fs path_abs
#' @export
generate_html_report <- function(assessment_results, output_dir = NULL) {
  
  # Check if rmarkdown is available
  if (!requireNamespace("rmarkdown", quietly = TRUE)) {
    message("Package 'rmarkdown' is required but not installed.")
    return(NULL)
  }
  
  # check input
  checkmate::assert_list(assessment_results, names = "named", any.missing = TRUE)
  checkmate::assert_list(assessment_results$results, names = "named", any.missing = TRUE)
 
  # If output_dir is not set, use the current working directory
  if (is.null(output_dir)) {
    output_dir <- getwd()
    message(glue::glue("No output directory specified. Setting output to {output_dir}"))
  }
  
  # Create a file name with pkg_name,version, and "risk_assessment.html"
  pkg_name <- assessment_results$results$pkg_name
  pkg_version <- assessment_results$results$pkg_version
  date_time <- assessment_results$results$date_time
  
  # Create the risk summary data
  risk_summary_output <- generate_risk_summary(assessment_results)
  
  # Create the risk summary data
  risk_details_output <- generate_risk_details(assessment_results)
  
  # Capture the output of generate_rcmd_check_section
  rcmd_check_output <- generate_rcmd_check_section(assessment_results)
  
  # Capture the output of generate_coverage_section
  coverage_output <- generate_coverage_section(assessment_results, pkg_name)
  
  # Capture the output of generate_doc_metrics_section
  doc_metrics_output <- generate_doc_metrics_section(assessment_results)
  
  # Capture the output of generate_pop_metrics_section
  pop_metrics_output <- generate_pop_metrics_section(assessment_results) 
  
  # Capture the output of generate_deps_section
  deps_output <- generate_deps_section(assessment_results)
  
  # Capture the output of generate_rev_deps_section
  rev_deps_output <- generate_rev_deps_section(assessment_results)
  
  # set up data for checking if trace_matrix is empty
  total_coverage <- assessment_results$covr_list$res_cov$coverage$totalcoverage
  file_coverage <- assessment_results$covr_list$res_cov$coverage$filecoverage
  
  # create general traceability matrix
  general_tm <- assessment_results$tm_list$tm
  
  # Capture the output of generate_trace_matrix_section
  trace_matrix_output <- 
    generate_trace_matrix_section(total_coverage, file_coverage, general_tm)
  
  # create high risk traceability matrix
  high_risk_tm <- assessment_results$tm_list$coverage$high_risk
  if (!is.data.frame(high_risk_tm)) {
    high_risk_tm <- dplyr::tibble(high_risk_tm)
  }
  
  high_risk_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "High Risk", high_risk_tm)
  
  
  # create medium risk traceability matrix
  medium_risk_tm <- assessment_results$tm_list$coverage$medium_risk
  if (!is.data.frame(medium_risk_tm)) {
    medium_risk_tm <- dplyr::tibble(medium_risk_tm)
  }
  
  medium_risk_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "Medium Risk", medium_risk_tm)
  
  # create low risk traceability matrix
  low_risk_tm <- assessment_results$tm_list$coverage$low_risk
  if (!is.data.frame(low_risk_tm)) {
    low_risk_tm <- dplyr::tibble(low_risk_tm)
  }
  
  low_risk_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "Low Risk", low_risk_tm)
  
  # create defunct function traceability matrix
  defunct_func_tm <- assessment_results$tm_list$function_type$defunct
  if (!is.data.frame(defunct_func_tm)) {
    defunct_func_tm <- dplyr::tibble(defunct_func_tm)
  }
  
  defunct_func_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "Defunct Functions", defunct_func_tm)
  
  # create imported function traceability matrix
  imported_func_tm <- assessment_results$tm_list$function_type$imported
  if (!is.data.frame(imported_func_tm)) {
    imported_func_tm <- dplyr::tibble(imported_func_tm)
  }
  
  imported_func_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "Imported Functions", imported_func_tm)
  
  # create re-exported function traceability matrix
  reexported_func_tm <- assessment_results$tm_list$function_type$rexported
  if (!is.data.frame(reexported_func_tm)) {
    reexported_func_tm <- dplyr::tibble(reexported_func_tm)
  }
  
  reexported_func_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "Re-exported Functions", reexported_func_tm)
  
  # create experimental function traceability matrix
  experimental_func_tm <- assessment_results$tm_list$function_type$experimental
  if (!is.data.frame(experimental_func_tm)) {
    experimental_func_tm <- dplyr::tibble(experimental_func_tm)
  }
  
  experimental_func_matrix_output <- 
    generate_fg_trace_matrix_section(tm_type = "Experimental Functions", experimental_func_tm)
  
  # get risk rules and create thresholds for sections 
  risk_rules <- get_risk_definition()
  code_covr_thresholds <- extract_thresholds_by_id(risk_rules, "code_coverage")
  code_covr_maxes <- get_max_thresholds(code_covr_thresholds)
 
  cmd_check_thresholds <- extract_thresholds_by_id(risk_rules, "cmd-check")
  cmd_check_maxes <- get_max_thresholds(cmd_check_thresholds)
  
  dep_thresholds <- extract_thresholds_by_id(risk_rules, "dependencies_count")
  dep_maxes <- get_max_thresholds(dep_thresholds)
  
  revdep_thresholds <- extract_thresholds_by_id(risk_rules, "reverse_dependencies")
  revdep_maxes <- get_max_thresholds(revdep_thresholds)
  
  pop_thresholds <- extract_thresholds_by_key(risk_rules, "total_download")
  pop_maxes <- get_max_thresholds(pop_thresholds)
  
  example_thresholds <- extract_thresholds_by_key(risk_rules, "has_ex_docs_score")
  example_maxes <- get_max_thresholds(example_thresholds)
  
  func_doc_thresholds <- extract_thresholds_by_key(risk_rules, "has_ex_docs_score")
  func_doc_maxes <- get_max_thresholds(func_doc_thresholds)
  license_thresholds <- extract_thresholds_by_id(risk_rules, "license")
  license_maxes <- get_license_thresholds(license_thresholds)
  
  # set up report environment
  report_env <- new.env()
  report_env$pkg_name <- pkg_name
  report_env$pkg_version <- pkg_version
  report_env$risk_summary_output <- risk_summary_output
  report_env$risk_details_output <- risk_details_output
  report_env$rcmd_check_output  <- rcmd_check_output
  report_env$coverage_output  <- coverage_output
  report_env$doc_metrics_output  <- doc_metrics_output
  report_env$pop_metrics_output  <- pop_metrics_output
  report_env$deps_df  <- deps_output$deps_df
  report_env$dep_score <- deps_output$dep_score
  report_env$import_count  <- deps_output$import_count
  report_env$suggest_count <- deps_output$suggest_count
  report_env$rev_deps_df  <- rev_deps_output$rev_deps_df
  report_env$rev_deps_summary$rev_deps_no <- rev_deps_output$rev_deps_summary$rev_deps_no
  report_env$rev_deps_summary$rev_deps_score <- rev_deps_output$rev_deps_summary$rev_deps_score
  report_env$trace_matrix_output  <- trace_matrix_output
  report_env$high_risk_matrix_output  <- high_risk_matrix_output
  report_env$medium_risk_matrix_output  <- medium_risk_matrix_output
  report_env$low_risk_matrix_output  <- low_risk_matrix_output
  report_env$defunct_func_matrix_output  <- defunct_func_matrix_output
  report_env$imported_func_matrix_output  <- imported_func_matrix_output
  report_env$reexported_func_matrix_output <- reexported_func_matrix_output
  report_env$experimental_func_matrix_output <- experimental_func_matrix_output
  report_env$code_covr_maxes <- code_covr_maxes
  report_env$cmd_check_maxes <- cmd_check_maxes
  report_env$dep_maxes <- dep_maxes
  report_env$revdep_maxes <- revdep_maxes
  report_env$pop_maxes <- pop_maxes
  report_env$example_maxes <- example_maxes
  report_env$func_doc_maxes <- func_doc_maxes
  report_env$license_maxes <- license_maxes
  
  if (dir_exists(output_dir)) {
    
    output_file <- path_abs(
      glue::glue("risk_report_{pkg_name}_{pkg_version}.html"),
      start = output_dir
    )
    
    template_path <- system.file(
      "report_templates",
      "risk_report_template.Rmd",
      package = "risk.assessr"
    )
    
    
    # Render the R Markdown file to the output directory
    # CRAN-safe: skip rendering on CRAN, avoid viewer launch
    if (identical(Sys.getenv("NOT_CRAN"), "true") || interactive()) {
      if (requireNamespace("rmarkdown", quietly = TRUE)) {
        message(paste0("Rendering report for ", pkg_name))
        render(
          input = template_path,
          output_file = output_file,
          output_options = list(css = system.file("report_templates/styles.css", 
                                                              package = "risk.assessr")),
          envir = report_env,
          quiet = TRUE
        )
      } else {
        message("Package 'rmarkdown' is required but not installed.")
        return(NULL)
      }
    } else {
      message("Rendering skipped on CRAN or non-interactive environment.")
      return(NULL)
    }
    
  } else {
    message("The output directory does not exist.")
    return(NULL)
  }
  
  return(output_file)
}

#' Helper function to replace NULL with "N/A"
#'
#' @param x - input value
#'
#' @keywords internal
handle_null <- function(x) {
  if (is.null(x) || length(x) == 0) {
    return("N/A")
  } else if (is.logical(x)) {
    return(paste(as.character(x), collapse = ", "))
  } else if (is.numeric(x)) {
    return(paste(as.character(x), collapse = " "))
  } else {
    return(x)
  }
}


#' Convert number to abbreviation
#'
#' @param value 
#'
#' @keywords internal
convert_number_to_abbreviation <- function(value) {
  result <- NA
  
  if (!is.na(value) && is.numeric(value)) {
    abs_value <- abs(value)
    
    if (abs_value >= 1e6) {
      result <- paste0(round(value / 1e6, 1), "M")
    } else if (abs_value >= 1e3) {
      result <- paste0(round(value / 1e3, 1), "K")
    } else {
      result <- as.character(value)
    }
  }
  
  return(result)
}

#' Convert number to percent
#'
#' @param value 
#'
#' @keywords internal
convert_number_to_percent <- function(value) {
  
  result <- NA
  
  if (!is.na(value)) {
    # Try to coerce to numeric if it's a character
    if (is.character(value)) {
      value <- suppressWarnings(as.numeric(value))
    }
    
    
    # If value is between 0 and 1 (including .75), treat as proportion
    if (value >= 0 && value <= 1) {
      percent_value <- round(value * 100, 1)
    } else {
      # If value >1 (like 75), treat as already percent
      percent_value <- round(value, 1)
    }
    
    result <- paste0(percent_value, "%")
    
  }
  
  return(result)
}

#' Helper to conditionally apply handle_null or abbreviation
#'
#' @param x - value
#'
#' @keywords internal
safe_value <- function(x) {
  if (is.null(x)) {
    return(handle_null(x))
  } else if (is.numeric(x)) {
    return(convert_number_to_abbreviation(x))
  } else {
    return(handle_null(x))
  }
}


#' Helper to create maintainer
#'
#' @param person_obj - value
#'
#' @keywords internal
extract_maintainer_info <- function(person_obj) {
  # Access the first person in the list
  p <- person_obj[[1]]
  
  # check for the length of given names and combine
  if (length(p$given) > 1) {
    p$given <- paste(p$given, collapse = " ")
  }
  
  # Extract fields
  given <- p$given
  family <- p$family
  email <- p$email
  
  # Combine into a single string
  pkg_maintainer <- paste(given, family, "<", email, ">", sep = " ")
  return(pkg_maintainer)
}

#' Extract Maximum Thresholds for Code Coverage Levels
#'
#' This internal function retrieves the `max` values for `"high"`,`"medium"`, and `"low"` levels
#' from a list of code coverage thresholds.
#'
#' @param thresholds A list of threshold objects, each containing a `level` and `max` field.
#'
#' @return A list with three elements:
#' \describe{
#'   \item{high_max}{The maximum threshold value for the `"high"` level.}
#'   \item{medium_max}{The maximum threshold value for the `"medium"` level.}
#'   \item{medium_max}{The maximum threshold value for the `"low"` level.}
#' }
#'
#' @keywords internal
get_max_thresholds <- function(thresholds) {
  high_max <- {{thresholds}}[[which(vapply({{thresholds}}, function(x) x$level == "high", logical(1)))]]$max
  medium_max <- {{thresholds}}[[which(vapply({{thresholds}}, function(x) x$level == "medium", logical(1)))]]$max
  low_max <- {{thresholds}}[[which(vapply({{thresholds}}, function(x) x$level == "low", logical(1)))]]$max
  
  list(high_max = high_max, 
       medium_max = medium_max, 
       low_max = low_max)
}

#' Get License Levels from Thresholds
#'
#' This internal utility function processes a list of license thresholds and returns
#' a named list of license values grouped by their risk level (e.g., "high", "medium", "low").
#'
#' @param thresholds A list where each element contains a `level` (character) and `values` (list of character vectors).
#'
#' @return A named list with elements `"high"`, `"medium"`, and `"low"`, each containing a character vector of license names.
#'
#' @keywords internal
get_license_thresholds <- function(thresholds) {
  levels_list <- list()
  
  for (entry in thresholds) {
    level_name <- entry$level
    level_values <- unlist(entry$values)
    levels_list[[level_name]] <- level_values
  }
  
  return(levels_list)
}

#' Generate RCMD Check Metrics Section
#'
#' @description Generates the RCMD Check Metrics HTML table for the HTML report.
#'
#' @param assessment_results - input data
#'
#' @keywords internal
generate_rcmd_check_section <- function(assessment_results) {
  
  rcmd_check_message <- "RCMD Check score"
  
  rcmd_check_score <- assessment_results$check_list$check_score
  
  rcmd_check_score <- convert_number_to_percent(rcmd_check_score) 
  
  # Check if errors, warnings, and notes are character(0) and replace with the message
  errors <- assessment_results$check_list$res_check$errors
  if (length(errors) == 0) {
    errors <- "No errors"
  }
  
  warnings <- assessment_results$check_list$res_check$warnings
  if (length(warnings) == 0) {
    warnings <- "No warnings"
  }
  
  notes <- assessment_results$check_list$res_check$notes
  if (length(notes) == 0) {
    notes <- "No notes"
  }
  
  # Create a data frame for R CMD Check results
  rcmd_check_df <- data.frame(
    Message = rcmd_check_message,
    Score = rcmd_check_score,
    Errors = errors,
    Warnings = warnings,
    Notes = notes
  )
  
  return(rcmd_check_df)
}

#' Generate Risk Summary
#'
#' @description Generates the Risk Summary table for the HTML report.
#'
#' @param assessment_results - input data
#'
#' @keywords internal
generate_risk_summary <- function(assessment_results) {
  
  # Handle NULL
  if (is.null(assessment_results$results$host$bioconductor_links)) {
    bioc_link <- "No Bioconductor link found"
  } else {
    bioc_link <- assessment_results$results$host$bioconductor_links
  }
  
  if (is.null(assessment_results$results$host$github_links)) {
    github_link <- "No GitHub link found"
  } else {
    github_link <- assessment_results$results$host$github_links
  }
  
  if (is.null(assessment_results$results$host$cran_links)) {
    cran_link <- "No CRAN link found"
  } else {
    cran_link <- assessment_results$results$host$cran_links
  }
  
  if (is.null(assessment_results$results$host$internal_links) || 
      is.list(assessment_results$results$host$internal_links)) {
    internal_link <- "No Internal link found"
  } else {
    internal_link <- assessment_results$results$host$internal_links
  }
  
  risk_summary_table <- data.frame(
    Metric = c('Package', 'Version', 'License', 
               'CRAN link', 'GitHub repository',
               'Bioconductor Link', 'Internal Repository'), 
    Value = c(
      assessment_results$results$pkg_name,
      assessment_results$results$pkg_version,
      assessment_results$results$license_name,
      cran_link,
      github_link,
      bioc_link,
      internal_link
    )
  )
  
  return(risk_summary_table)
}

#' Generate Risk Details
#'
#' @description Generates the Risk Details table for the HTML report.
#'
#' @param assessment_results - input data
#'
#' @keywords internal
generate_risk_details <- function(assessment_results) {
  
  # Ensure covr is numeric before rounding
  assessment_results$results$covr <- 
    as.numeric(assessment_results$results$covr)
  assessment_results$results$covr <- 
    round(assessment_results$results$covr, 2)
  
  check <- handle_null(assessment_results$results$check)
  
  check <- convert_number_to_percent(check)
  
  covr <- handle_null(assessment_results$results$covr)
  
  covr <- convert_number_to_percent(covr)

  risk_details_table <- data.frame(
    Metric = c(
      'R CMD Check Score', 'Test Coverage Score', 'Date Time', 'Executor', 
      'OS Name', 'OS Release', 'OS Machine', 'R version'
    ),
    Value = c(
      check,
      covr,
      handle_null(assessment_results$results$date_time),
      handle_null(assessment_results$results$executor),
      handle_null(assessment_results$results$sysname),
      handle_null(assessment_results$results$release),
      handle_null(assessment_results$results$machine),
      handle_null(assessment_results$check_list$res_check$rversion)
    )
  )
  return(risk_details_table)
}

#' Generate file coverage df
#'
#' @description Generates file coverage df when errors list created.
#' 
#' @param file_names - file names
#' @param file_coverage - test coverage for files
#' @param errors - test coverage errors
#' @param notes - test coverage notes
#' 
#' @keywords internal
create_file_coverage_df <- function(file_names, file_coverage, errors, notes) {
  # Convert errors to a character string if it is a complex structure
  if (is.list(errors)) {
    errors <- sapply(errors, function(x) {
      if (is.null(x)) {
        return("N/A")
      } else if (is.data.frame(x)) {
        return(paste(capture.output(print(x)), collapse = "; "))
      } else if (is.list(x)) {
        return(paste(unlist(x), collapse = "; "))
      } else {
        return(as.character(x))
      }
    })
  }
  
  # Ensure the number of rows match
  max_len <- max(length(file_names), length(file_coverage), length(notes))
  
  # Extend lists to match the maximum length
  file_names <- c(file_names, rep("", max_len - length(file_names)))
  file_coverage <- c(file_coverage, rep(NA, max_len - length(file_coverage)))
  notes <- c(notes, rep("", max_len - length(notes)))
  errors <- rep(paste(errors, collapse = "; "), max_len)
  
  # Create the data frame
  file_coverage_df <- data.frame(
    File = file_names,
    Coverage = file_coverage,
    Errors = errors,
    Notes = notes,
    stringsAsFactors = FALSE
  )
  
  return(file_coverage_df)
}

#' Generate Coverage Section
#'
#' @description Generates the Coverage section for the HTML report.
#' 
#' @param assessment_results - input data
#' @param pkg_name - name of the package
#' 
#' @keywords internal
generate_coverage_section <- function(assessment_results, pkg_name) {
  
  total_coverage <- assessment_results$covr_list$res_cov$coverage$totalcoverage
  file_coverage <- assessment_results$covr_list$res_cov$coverage$filecoverage
  
  # Extract file names from the attributes
  file_names <- attr(file_coverage, "dimnames")[[1]]
  
  
  # Check if file_names contains full paths
  if (any(grepl("^(/|[A-Za-z]:)", file_names))) {
    file_names <- sub(glue::glue("^/tmp/.+?/{pkg_name}/"), "", file_names)
  }
  
  # Handle errors and notes
  errors <- assessment_results$covr_list$res_cov$errors
  if (all(is.na(errors))) {
    errors <- "No test coverage errors"
  }
  
  notes <- assessment_results$covr_list$res_cov$notes
  if (all(is.na(notes))) {
    notes <- "No test coverage notes"
  }
  
  # Create a data frame for file coverage
  if (is.list(errors) && all(c("message", "srcref", "status", "stdout", "stderr", "parent_trace", "call", "procsrcref", "parent") %in% names(errors))) {
    # Create the file coverage data frame
    file_coverage_df <- create_file_coverage_df(file_names, file_coverage, errors, notes)
  } else {
    # Handle the case where errors does not have the expected structure
    
    if (is.null(file_names) || is.null(file_coverage)) {
      file_coverage_df <- data.frame(
        Function = NA_character_,
        Coverage = NA_real_,
        Errors = NA,
        Notes = NA,
        stringsAsFactors = FALSE
      )
    } else {
      file_coverage_df <- data.frame(
        Function = file_names,
        Coverage = file_coverage,
        Errors = errors,
        Notes = notes,
        stringsAsFactors = FALSE
      )
    }
  }
  
  return(file_coverage_df)
}

#' Generate Doc Metrics Section
#'
#' @description Generates the documentation section for the HTML report.
#' 
#' @param assessment_results - input data
#' 
#' @keywords internal

generate_doc_metrics_section <- function(assessment_results) {
  
  # Helpers
  `%||%` <- function(a, b) if (!is.null(a)) a else b
  
  # Map 0/1 flags to labels; leave non-flags untouched
  map_flag <- function(x) {
    if (is.null(x)) return(NA_character_)
    if (length(x) == 1L && is.numeric(x) && !is.na(x)) {
      if (x == 0) return("Not Included")
      if (x == 1) return("Included")
    }
    as.character(x)
  }
  
  # External helpers expected to exist in your codebase:
  # - handle_null(): normalizes NULL/missing values
  # - extract_maintainer_info(): extracts maintainer display string
  pkg_maintainer <- extract_maintainer_info(assessment_results$results$author$maintainer)
  
  # --- has_examples (returned as its own data.frame) ---
  hex <- assessment_results$results$has_examples
  
  if (is.list(hex) && !is.null(hex$data)) {
    
    has_examples_df <- as.data.frame(hex$data, stringsAsFactors = FALSE)
    attr(has_examples_df, "score") <- `%||%`(hex$example_score, NA_real_)
    
  } else if (is.data.frame(hex)) {
    
    has_examples_df <- hex
    attr(has_examples_df, "score") <- NA_real_
    
  } else {
    
    has_examples_df <- data.frame()
    attr(has_examples_df, "score") <- NA_real_
    
  }
  
  
  if ("example" %in% names(has_examples_df)) {
    # Preserve custom attribute
    ex_score <- attr(has_examples_df, "score")
    
    # Build a safe logical index
    ex_vals <- has_examples_df$example
    # NOTE: We intentionally keep only rows where the function has *no* example.
    # This allows the documentation metrics section to highlight functions missing examples.
    keep <- !is.na(ex_vals) & tolower(trimws(ex_vals)) == "no example"
    
    # Subset while keeping column structure even when 0 rows remain
    has_examples_df <- has_examples_df[keep, , drop = FALSE]
    
    # Restore attribute
    attr(has_examples_df, "score") <- ex_score
  }
  
  
  
  # --- has_docs (returned as its own data.frame) ---
  hdoc <- assessment_results$results$has_docs
  if (is.list(hdoc) && !is.null(hdoc$data)) {
    has_docs_df <- as.data.frame(hdoc$data, stringsAsFactors = FALSE)
    attr(has_docs_df, "score") <- `%||%`(hdoc$has_docs_score, NA_real_)
  } else if (is.data.frame(hdoc)) {
    has_docs_df <- hdoc
    attr(has_docs_df, "score") <- NA_real_
  } else {
    has_docs_df <- data.frame()
    attr(has_docs_df, "score") <- NA_real_
  }
  
  # --- Remaining documentation metrics (EXCLUDES 'Has Examples') ---
  metric_names <- c(
    'Has Bug Reports URL',
    'Has License',
    'Has Maintainer',
    'Has News',
    'Has Source Control',
    'Has Vignettes',
    'Has Website',
    'News Current',
    'Export Help'
  )
  
  metric_values_raw <- list(
    assessment_results$results$has_bug_reports_url,
    assessment_results$results$license,
    pkg_maintainer,
    assessment_results$results$has_news,
    assessment_results$results$has_source_control,
    assessment_results$results$has_vignettes,
    assessment_results$results$has_website,
    assessment_results$results$news_current,
    assessment_results$results$export_help
  )
  
  processed_values <- lapply(metric_values_raw, handle_null)
  labeled_values   <- vapply(processed_values, map_flag, FUN.VALUE = character(1))
  
  doc_metrics_df <- data.frame(
    Metric = metric_names,
    Value  = unname(labeled_values),
    stringsAsFactors = FALSE
  )
  
  # Return all three sections
  docs_list <-list(
    has_examples = has_examples_df,
    has_docs     = has_docs_df,
    doc_metrics  = doc_metrics_df
  )
  return(docs_list)
}


#' Generate Popularity Metrics Section
#'
#' @description Generates the popularity section for the HTML report.
#' 
#' @param assessment_results - input data
#' 
#' @keywords internal
generate_pop_metrics_section <- function(assessment_results) {
  
  # Extract the numeric value
  total_download_n <- safe_value(assessment_results$results$download$total_download)
  
  # Extract the relevant elements from assessment_results
  pop_metrics <- data.frame(
    Metric = c(
      'Date created', 'Stars', 'Forks', 'Data Extract Date',
      'Recent Commits', 'Open Issues', 'Downloads Total',
      'Downloads Last Month'
    ),
    Value = c(
      handle_null(assessment_results$results$github_data$created_at),
      handle_null(assessment_results$results$github_data$stars),
      handle_null(assessment_results$results$github_data$forks),
      handle_null(assessment_results$results$github_data$date),
      handle_null(assessment_results$results$github_data$recent_commits_count),
      handle_null(assessment_results$results$github_data$open_issues),
      safe_value(assessment_results$results$download$total_download),
      safe_value(assessment_results$results$download$last_month_download)
    )
  )  
 
  # Add numeric value as a new column 
  # needed for formatting in pop_metrics r chunk in `risk_report_template.Rmd`
  pop_metrics$NumericValue <- NA_real_
  
  # check total_download_n
  if (is.character(total_download_n) && total_download_n == "N/A") {
    pop_metrics$NumericValue[pop_metrics$Metric == "Downloads Total"] <- NA_real_
  } else {
    pop_metrics$NumericValue[pop_metrics$Metric == "Downloads Total"] <- total_download_n
  }
  
  
  return(pop_metrics)
}


#' Generate Trace Matrix Section
#'
#' @description Generates the Trace Matrix section for the HTML report.
#' 
#' @param total_coverage - total coverage for the package
#' @param file_coverage - file coverage for the package
#' @param tm_df - input data
#' 
#' @keywords internal
generate_trace_matrix_section <- function(total_coverage, file_coverage, tm_df) {
  
  if (is.na(total_coverage) || total_coverage == 0 ||
      (is.vector(file_coverage) && any(is.na(file_coverage)))) {
    trace_matrix <- data.frame(
      Exported_function = " ",
      Function_type = " ",
      Code_script = " ",
      Documentation = " ",
      Description = "Traceability matrix unsuccessful",
      Test_Coverage = " ",
      stringsAsFactors = FALSE
    )
  } else {
    trace_matrix <- tm_df %>%
      dplyr::select(
        Exported_function = exported_function,
        Function_type = function_type,
        Code_script = code_script,
        Documentation = documentation,
        Description = description,
        Test_Coverage = coverage_percent
      )
    
    # sort by Test Coverage
    trace_matrix <- trace_matrix |> 
      dplyr::arrange(Test_Coverage)
    
  }
  
  return(trace_matrix)  
}

#' Generate Fine grained Trace Matrices Section
#'
#' @description Generates the Trace Matrices section for the HTML report.
#' 
#' @param tm_type - type of traceability matrix
#' @param tm_df - input data
#' 
#' @keywords internal
generate_fg_trace_matrix_section <- function(tm_type, tm_df) {
  
  # Check if coverage is missing
  if (!"coverage_percent" %in% names(tm_df)) {
    trace_matrix <- data.frame(
      Exported_function = " ",
      Function_type = " ",
      Code_script = " ",
      Documentation = " ",
      Description = glue::glue("No {tm_type} traceability matrix generated"),
      Test_Coverage = " ",
      stringsAsFactors = FALSE
    )
  } else {
    trace_matrix <- tm_df %>%
      dplyr::select(
        Exported_function = exported_function,
        Function_type = function_type,
        Code_script = code_script,
        Documentation = documentation,
        Description = description,
        Test_Coverage = coverage_percent
      ) %>%
      dplyr::arrange(Test_Coverage)
  }
  
  return(trace_matrix)  
}


#' Generate Dependencies Section
#'
#' @description Generates the dependencies section for the HTML report.
#' 
#' @param assessment_results - input data
#' 
#' @keywords internal
generate_deps_section <- function(assessment_results) {
  
  # Extract dependencies from assessment_results
  dependencies <- assessment_results$results$dependencies
  
  # Initialize empty lists to store package names, versions, types
  pkg_names <- c()
  pkg_versions <- c()
  pkg_types <- c()
  import_count <- 0
  suggest_count <- 0
  
  # Process Imports
  if (!is.null(dependencies$imports)) {
    for (pkg in names(dependencies$imports)) {
      pkg_names <- c(pkg_names, pkg)
      pkg_versions <- c(pkg_versions, dependencies$imports[[pkg]])
      pkg_types <- c(pkg_types, "Imports")
      import_count <- import_count + 1
    }
  }
  
  # Process Suggests
  if (!is.null(dependencies$suggests)) {
    for (pkg in names(dependencies$suggests)) {
      pkg_names <- c(pkg_names, pkg)
      pkg_versions <- c(pkg_versions, dependencies$suggests[[pkg]])
      pkg_types <- c(pkg_types, "Suggests")
      suggest_count <- suggest_count + 1
    }
  }
  
  # Create a data frame
  deps_df <- data.frame(
    Package = pkg_names,
    Version = pkg_versions,
    Type = pkg_types,
    stringsAsFactors = FALSE
  )
  
  return(list(
    deps_df = deps_df,
    import_count = import_count,
    suggest_count = suggest_count
  ))
}

#' Generate Reverse Dependencies Section
#'
#' @description Generates the reverse dependencies section for the HTML report.
#' 
#' @param assessment_results - input data
#' 
#' @keywords internal
generate_rev_deps_section <- function(assessment_results) {
  
  # Extract dependencies from assessment_results
  
  rev_deps <- assessment_results$results$rev_deps
  
  # Convert the character vector to a data frame with one row per value
  rev_deps_df <- as.data.frame(rev_deps, 
                               stringsAsFactors = FALSE)
  
  # Rename the column 
  colnames(rev_deps_df) <- "Reverse_dependencies"
  
  rev_deps_no = nrow(rev_deps_df)
  
  # Create a summary data frame
  rev_deps_summary <- data.frame(
    rev_deps_no = nrow(rev_deps_df)
  )
  
  return(list(
    rev_deps_df = rev_deps_df,
    rev_deps_summary = rev_deps_summary
  ))
}

toy_assessment_results <- list(
  results = list(
    pkg_name = "test.package.0001",
    pkg_version = "0.1.0",
    pkg_source_path = structure("/tmp/Rtmpnn3wNN/temp_file_7b003b884e49/test.package.0001", .Names = "/tmp/Rtmpnn3wNN/temp_file_7b003b884e49/test.package.0001"),
    date_time = "2025-01-02 09:15:18",
    executor = "u1004798",
    sysname = "Linux",
    version = "#1 SMP Tue Aug 18 14:50:17 EDT 2020",
    release = "3.10.0-1160.el7.x86_64",
    machine = "x86_64",
    comments = " ",
    has_bug_reports_url = 0,
    license = 1,
    has_examples = list(
      data = data.frame(
        function_name = c("here", "i_am", "dr_here", "set_here"),
        documentation_name = c("here", "i_am", "dr_here", "set_here"),
        documentation_location = c(
          "man/here.Rd",
          "man/i_am.Rd",
          "man/dr_here.Rd",
          "man/set_here.Rd"
        ),
        example = c(
          "here()\n\nhere(\"some\", \"path\", \"below\", \"your\", \"project\", \"root.txt\")\nhere(\"some/path/below/your",
          "here::i_am(\"prepare/penguins.R\")\nhere::i_am(\"analysis/report.Rmd\", uuid = \"f9e884084b84794d762a535f3facec85\")",
          "dr_here()",
          "no example"
        ),
        stringsAsFactors = FALSE
      ),
      example_score = .75
    ),
    has_docs = list(
      data = data.frame(
        function_name = c("here", "i_am", "dr_here", "set_here"),
        documentation_name = c("here", "i_am", "dr_here", "set_here"),
        documentation_location = c(
          "man/here.Rd",
          "man/i_am.Rd",
          "man/dr_here.Rd",
          "man/set_here.Rd"
        ),
        stringsAsFactors = FALSE
      ),
      has_docs_score = 1
    ),
    has_maintainer = 1,
    size_codebase = 0.026,
    has_news = 0,
    has_source_control = 0,
    has_vignettes = 0,
    has_website = 0,
    news_current = 0,
    export_help = 1,
    export_calc = 0.731,
    check = 1,
    covr = 1,
    github_data = list(
      created_at = "2016-07-19T14:47:19Z",
      stars = 422,
      forks = 45,
      date = "2025-07-02",
      recent_commits_count = 0,
      open_issues = 29
    ),
    download = list(
      total_download = 11369237,
      last_month_download = 299354
    ),
    dependencies = list(
      imports = list(checkmate = "2.1.0"),
      suggests = list(testthat = "3.2.1.1")
    ),
    dep_score = 0.000203,
    rev_deps = logical(0),
    revdep_score = 0.0966,
    risk_profile = "High",
    license_name = "MIT + file LICENSE"
  ),
  covr_list = list(
    total_cov = 1,
    res_cov = list(
      name = "test.package.0001",
      coverage = list(
        filecoverage = structure(50, .Dim = c(1L), .Dimnames = list("R/myscript.R")),
        totalcoverage = 50
      ),
      errors = NA,
      notes = NA
    )
  ),
  tm_list = list(
    tm = dplyr::tibble(
      exported_function = "myfunction",
      function_type = "regular",
      code_script = "R/myscript.R",
      documentation = "myfunction.Rd",
      description = "Adds 1 to x",
      coverage_percent = 50
    ),
    coverage = list(
      high_risk = dplyr::tibble(
        exported_function = "myfunction",
        class = NA,
        function_type = "regular",
        function_body = "{\n    check_string(encoding)\n    stri_conv(string, encoding, \"UTF-8\")\n}",
        where = "stringr",
        code_script = "R/myscript.R",
        documentation = "myfunction.Rd",
        description = "Adds 1 to x",
        coverage_percent = 50
      ),
      medium_risk = list(
        pkg_name = "test.package.0001",
        coverage = list(
          filecoverage = 0,
          totalcoverage =  0
        ),
        errors = NA,
        notes  = NA
      ),
      low_risk = list(
        pkg_name = "test.package.0001",
        coverage = list(
          filecoverage = 0,
          totalcoverage =  0
        ),
        errors = NA,
        notes  = NA
      )
    ),
    function_type = list(
      defunct = dplyr::tibble(
        pkg_name = "test.package.0001",
        coverage = list(
          filecoverage = 0,
          totalcoverage =  0
        ),
        errors = NA,
        notes  = NA
      ),
      imported = dplyr::tibble(
        pkg_name = "test.package.0001",
        coverage = list(
          filecoverage = 0,
          totalcoverage =  0
        ),
        errors = NA,
        notes  = NA
      ),
      rexported = dplyr::tibble(
        pkg_name = "test.package.0001",
        coverage = list(
          filecoverage = 0,
          totalcoverage =  0
        ),
        errors = NA,
        notes  = NA
      )
    )
  ),
  check_list = list(
    res_check = list(
      stdout = "* using log directory ‘/tmp/Rtmpnn3wNN/file7b001de03ced/test.package.0001.Rcheck’\n* using R version 4.2.1 (2022-06-23)\n* using platform: x86_64-pc-linux-gnu (64-bit)\n* using session charset: UTF-8\n* checking for file ‘test.package.0001/DESCRIPTION’ ... OK\n* this is package ‘test.package.0001’ version ‘0.1.0’\n* package encoding: UTF-8\n* checking package namespace information ... OK\n* checking package dependencies ... OK\n* checking if this is a source package ... OK\n* checking if there is a namespace ... OK\n* checking for executable files ... OK\n* checking for hidden files and directories ... OK\n* checking for portable file names ... OK\n* checking for sufficient/correct file permissions ... OK\n* checking whether package ‘test.package.0001’ can be installed ... OK\n* checking installed package size ... OK\n* checking package directory ... OK\n* checking DESCRIPTION meta-information ... OK\n* checking top-level files ... OK\n* checking for left-over files ... OK\n* checking index information ... OK\n* checking package subdirectories ... OK\n* checking R files for non-ASCII characters ... OK\n* checking R files for syntax errors ... OK\n* checking whether the package can be loaded ... OK\n* checking whether the package can be loaded with stated dependencies ... OK\n* checking whether the package can be unloaded cleanly ... OK\n* checking whether the namespace can be loaded with stated dependencies ... OK\n* checking whether the namespace can be unloaded cleanly ... OK\n* checking loading without being on the library search path ... OK\n* checking dependencies in R code ... OK\n* checking S3 generic/method consistency ... OK\n* checking replacement functions ... OK\n* checking foreign function calls ... OK\n* checking R code for possible problems ... OK\n* checking Rd files ... OK\n* checking Rd metadata ... OK\n* checking Rd cross-references ... OK\n* checking for missing documentation entries ... OK\n* checking for code/documentation mismatches ... OK\n* checking Rd \\\\usage sections ... OK\n* checking Rd contents ... OK\n* checking for unstated dependencies in examples ... OK\n* checking examples ... OK\n* checking for non-standard things in the check directory ... OK\n* checking for detritus in the temp directory ... OK\n* DONE\nStatus: OK\n",
      stderr = character(0),
      status = 0,
      duration = 40.2,
      timeout = FALSE,
      rversion = "4.2.1",
      platform = "x86_64-pc-linux-gnu",
      errors = character(0),
      warnings = character(0),
      notes = character(0),
      description = "Package: test.package.0001\nType: Package\nTitle: Test Package\nVersion: 0.1.0\nAuthors@R: \n    person(\"Jane\", \"Doe\", email = \"jane.doe@example.com\", role = c(\"aut\", \"cre\"))\nDescription: A test package.\nLicense: GPL-3\nEncoding: UTF-8\nLazyData: true\nImports: checkmate (>= 2.1.0)\nSuggests: testthat (>= 3.2.1.1)\n",
      package = "test.package.0001",
      version = "0.1.0",
      cran = FALSE,
      bioc = FALSE,
      checkdir = "/tmp/Rtmpnn3wNN/file7b001de03ced/test.package.0001.Rcheck",
      test_fail = list(),
      test_output = list(testthat = "> library(testthat)\n> library(test.package.0001)\n> \n> test_check(\"test.package.0001\")\n[ FAIL 0 | WARN 0 | SKIP 0 | PASS 1 ]\n"),
      install_out = "* installing *source* package ‘test.package.0001’ ...\n** using staged installation\n** R\n** byte-compile and prepare package for lazy loading\n** help\n*** installing help indices\n** building package indices\n** testing if installed package can be loaded from temporary location\n** testing if installed package can be loaded from final location\n** testing if installed package keeps a record of temporary installation path\n* DONE (test.package.0001)\n",
      session_info = list(
        platform = list(
          version = "R version 4.2.1 (2022-06-23)",
          os = "Red Hat Enterprise Linux Server 7.9 (Maipo)",
          system = "x86_64, linux-gnu",
          ui = "X11",
          language = "(EN)",
          collate = "C",
          ctype = "en_US.UTF-8",
          tz = "Europe/Paris",
          date = "2025-01-02",
          pandoc = "3.1.4 @ /cm/easybuild/software/Pandoc/3.1.4/bin/pandoc"
        ),
        packages = structure(list(
          package = c("backports", "checkmate", "test.package.0001"),
          ondiskversion = c("1.4.1", "2.1.0", "0.1.0"),
          loadedversion = c(NA, NA, NA),
          path = c("/cm/easybuild/software/R/4.2.1-foss-2020b-REL-001/lib64/R/library/backports", "/cm/easybuild/software/R/4.2.1-foss-2020b-REL-001/lib64/R/library/checkmate", "/tmp/Rtmpnn3wNN/file7b001de03ced/test.package.0001.Rcheck/test.package.0001"),
          loadedpath = c(NA, NA, NA),
          attached = c(FALSE, FALSE, FALSE),
          is_base = c(FALSE, FALSE, FALSE),
          date = c("2021-12-13", "2022-04-21", "2025-01-02"),
          source = c("CRAN (R 4.2.1)", "CRAN (R 4.2.1)", "local"),
          md5ok = c(NA, NA, NA),
          library = factor(c("/tmp/Rtmpnn3wNN/file7b001de03ced/test.package.0001.Rcheck", "/tmp/Rtmpnn3wNN/file7b001de03ced/test.package.0001.Rcheck", "/tmp/Rtmpnn3wNN/file7b001de03ced/test.package.0001.Rcheck"))
        ), class = c("packages_info", "data.frame"))
      ),
      cleaner = new.env()
    ),
    check_score = 1
  )
)

test_that("handle_null returns 'N/A' for NULL", {
  expect_equal(handle_null(NULL), "N/A")
})

test_that("handle_null returns original value for non-NULL", {
  expect_equal(handle_null(123), "123")
  expect_equal(handle_null("text"), "text")
})


testthat::test_that("handle_null returns comma-separated string for logical inputs", {
  # Mocked logical inputs
  x1 <- TRUE
  x2 <- c(TRUE, FALSE)
  x3 <- c(TRUE, NA, FALSE)
  
  out1 <- handle_null(x1)
  out2 <- handle_null(x2)
  out3 <- handle_null(x3)
  
  # Each should hit: else if (is.logical(x)) return(paste(as.character(x), collapse = ", "))
  testthat::expect_identical(out1, "TRUE")
  testthat::expect_identical(out2, "TRUE, FALSE")
  testthat::expect_identical(out3, "TRUE, NA, FALSE")
  
  # Sanity: each result is a length-1 character scalar
  testthat::expect_type(out1, "character"); testthat::expect_equal(length(out1), 1L)
  testthat::expect_type(out2, "character"); testthat::expect_equal(length(out2), 1L)
  testthat::expect_type(out3, "character"); testthat::expect_equal(length(out3), 1L)
})


test_that("convert_number_to_abbreviation handles millions", {
  expect_equal(convert_number_to_abbreviation(1500000), "1.5M")
  expect_equal(convert_number_to_abbreviation(-2500000), "-2.5M")
})

test_that("convert_number_to_abbreviation handles thousands", {
  expect_equal(convert_number_to_abbreviation(1200), "1.2K")
  expect_equal(convert_number_to_abbreviation(-9999), "-10K")
})

test_that("convert_number_to_abbreviation handles small numbers", {
  expect_equal(convert_number_to_abbreviation(999), "999")
  expect_equal(convert_number_to_abbreviation(0), "0")
})

test_that("convert_number_to_abbreviation handles NA and non-numeric", {
  expect_true(is.na(convert_number_to_abbreviation(NA)))
  expect_true(is.na(convert_number_to_abbreviation("text")))
})


testthat::test_that("collapses multi-part given names into a single string", {
  person_obj <- list(
    list(
      given  = c("Jane", "Ann"),
      family = "Doe",
      email  = "jane@example.com"
    )
  )
  
  out <- extract_maintainer_info(person_obj)
  
  # Accept both literal angle brackets and HTML-escaped ones.
  expected_plain <- "Jane Ann Doe < jane@example.com >"
  expected_html  <- "Jane Ann Doe &lt; jane@example.com &gt;"
  
  testthat::expect_true(
    out %in% c(expected_plain, expected_html),
    info = paste("Got:", out)
  )
})

testthat::test_that("single-part given name is unchanged", {
  person_obj <- list(
    list(
      given  = "Jane",
      family = "Doe",
      email  = "jane@example.com"
    )
  )
  
  out <- extract_maintainer_info(person_obj)
  
  expected_plain <- "Jane Doe < jane@example.com >"
  expected_html  <- "Jane Doe &lt; jane@example.com &gt;"
  
  testthat::expect_true(
    out %in% c(expected_plain, expected_html),
    info = paste("Got:", out)
  )
})

testthat::test_that("uses the first person in the list", {
  person_obj <- list(
    list(
      given  = c("First", "Person"),
      family = "One",
      email  = "first.one@example.com"
    ),
    list(
      given  = c("Second", "Person"),
      family = "Two",
      email  = "second.two@example.com"
    )
  )
  
  out <- extract_maintainer_info(person_obj)
  
  expected_plain_first <- "First Person One < first.one@example.com >"
  expected_html_first  <- "First Person One &lt; first.one@example.com &gt;"
  
  # Ensure the output corresponds to the FIRST entry, not the second
  testthat::expect_true(
    out %in% c(expected_plain_first, expected_html_first),
    info = paste("Got:", out)
  )
  
  # And it should not match the second entry
  unexpected_plain_second <- "Second Person Two < second.two@example.com >"
  unexpected_html_second  <- "Second Person Two &lt; second.two@example.com &gt;"
  
  testthat::expect_false(
    out %in% c(unexpected_plain_second, unexpected_html_second),
    info = paste("Unexpectedly got:", out)
  )
})


test_that("generate_rcmd_check_rmd_section works correctly", {
  result <- generate_rcmd_check_section(toy_assessment_results)
  expect_equal(result$Message, "RCMD Check score")
  expect_equal(result$Score, "100%")
  expect_equal(result$Errors, "No errors")
  expect_equal(result$Warnings, "No warnings")
  expect_equal(result$Notes, "No notes")
})

test_that("safe_value returns 'N/A' for NULL", {
  expect_equal(safe_value(NULL), "N/A")
})

test_that("safe_value abbreviates numeric values", {
  expect_equal(safe_value(1000), "1K")
  expect_equal(safe_value(1000000), "1M")
})

test_that("safe_value returns character for small numbers", {
  expect_equal(safe_value(42), "42")
})

# Define the toy assessment_results object
toy_assessment_results_1_1 <- list(
  results = list(
    pkg_name = "test.package.0001",
    pkg_version = "1.0.0",
    license_name = "MIT + file LICENSE",
    cran_link = "https://cran.r-project.org/src/contrib/stringr_1.5.1.tar.gz",
    github_link = "https://github.com/tidyverse/stringr",
    bioc_link = NULL,
    internal_link = NULL
  )
)

# Test the generate_risk_summary function
test_that("generate_risk_summary works correctly", {
  result <- generate_risk_summary(toy_assessment_results_1_1)
  
  expect_equal(result$Metric[1], "Package")
  expect_equal(result$Value[1], "test.package.0001")
  expect_equal(result$Metric[2], "Version")
  expect_equal(result$Value[2], "1.0.0")
  expect_equal(result$Metric[3], "License")
  expect_equal(result$Value[3], "MIT + file LICENSE")
  expect_equal(result$Metric[4], "CRAN link")
  expect_equal(result$Value[4], "No CRAN link found")
  expect_equal(result$Metric[5], "GitHub repository")
  expect_equal(result$Value[5], "No GitHub link found")
  expect_equal(result$Metric[6], "Bioconductor Link")
  expect_equal(result$Value[6], "No Bioconductor link found")
  expect_equal(result$Metric[7], "Internal Repository")
  expect_equal(result$Value[7], "No Internal link found")
})

# Define the toy assessment_results object
toy_assessment_results_1_2 <- list(
  results = list(
    check = 1L,
    covr = 1L,
    date_time = "2025-01-13 17:01:49",
    executor = "test.executor",
    sysname = "Linux",
    release = "5.4.0-42-generic",
    machine = "x86_64",
    check_list = list(res_check = list(rversion = "4.0.2"))
  )
)

# Define the expected output
expected_risk_details <- list(
  Metric = c(
    'R CMD Check Score', 'Test Coverage Score', 'Date Time', 'Executor', 
    'OS Name', 'OS Release', 'OS Machine', 'R version'
  ),
  Value = c(
    toy_assessment_results_1_2$results$check,
    toy_assessment_results_1_2$results$covr,
    toy_assessment_results_1_2$results$date_time,
    toy_assessment_results_1_2$results$executor,
    toy_assessment_results_1_2$results$sysname,
    toy_assessment_results_1_2$results$release,
    toy_assessment_results_1_2$results$machine,
    toy_assessment_results_1_2$results$check_list$res_check$rversion
  )
)

# Test the generate_risk_details function
test_that("generate_risk_details works correctly", {
  result <- generate_risk_details(toy_assessment_results_1_2)
  
  expect_equal(result$Metric, expected_risk_details$Metric)
})

test_that("generate_coverage_section works correctly", {
  pkg_name <- "test.package.0001"
  result <- generate_coverage_section(toy_assessment_results, pkg_name)
  expect_equal(result$Function[1], "R/myscript.R")
  expect_equal(result$Coverage[1], 50)
  expect_equal(result$Errors, "No test coverage errors")
  expect_equal(result$Notes, "No test coverage notes")
})

test_that("create_file_coverage_df works correctly with toy dataset", {
  # Toy dataset
  file_names <- c("file1.R", "file2.R")
  file_coverage <- c(85.0, 90.0)
  notes <- c("Note1", "Note2")
  errors <- list(
    message = "in callr subprocess.",
    status = 0
  )
  
  # Expected output
  expected_output <- data.frame(
    File = c("file1.R", "file2.R"),
    Coverage = c(85.0, 90.0),
    Errors = rep("in callr subprocess.; 0", 2),
    Notes = c("Note1", "Note2"),
    stringsAsFactors = FALSE
  )
  
  # Run the function
  result <- create_file_coverage_df(file_names, file_coverage, errors, notes)
  
  # Check if the result matches the expected output
  expect_equal(result, expected_output)
})


testthat::test_that("generate_coverage_section returns NA-row when file_names is NULL", {
  # Build file_coverage such that:
  # - attr(file_coverage, "dimnames")[[1]] returns NULL (so file_names <- NULL)
  # - No errors occur in grepl("^(/|[A-Za-z]:)", file_names) because we'll stub grepl.
  file_coverage_mat <- matrix(
    numeric(0),
    nrow = 0, ncol = 0,
    dimnames = list(NULL, NULL)  # makes attr(..., "dimnames")[[1]] == NULL
  )
  
  assessment_results <- list(
    covr_list = list(
      res_cov = list(
        coverage = list(
          totalcoverage = 85.1,   # arbitrary; not used in this test
          filecoverage  = file_coverage_mat
        ),
        # Force the 'else' branch (not the create_file_coverage_df path)
        errors = NA,   # will be converted to "No test coverage errors" earlier
        notes  = NA    # will be converted to "No test coverage notes" earlier
      )
    )
  )
  
  # Create a stubbed copy of the function to neutralize grepl on NULL file_names
  fn <- generate_coverage_section
  
  # grepl(NULL) would normally error; stub grepl INSIDE the function to always return FALSE
  mockery::stub(fn, "grepl", function(pattern, x) FALSE)
  
  # Call with any pkg_name (glue::glue not hit because grepl returns FALSE)
  out <- fn(assessment_results, pkg_name = "mypkg")
  
  # Validate the NA-row data.frame
  testthat::expect_s3_class(out, "data.frame")
  testthat::expect_identical(colnames(out), c("Function", "Coverage", "Errors", "Notes"))
  testthat::expect_equal(nrow(out), 1L)
  
  # Column-wise checks match the code block: NA_character_, NA_real_, NA, NA
  testthat::expect_true(is.na(out$Function))
  testthat::expect_identical(typeof(out$Function), "character")
  
  testthat::expect_true(is.na(out$Coverage))
  testthat::expect_identical(typeof(out$Coverage), "double")  # NA_real_
  
  testthat::expect_true(is.na(out$Errors))
  testthat::expect_identical(typeof(out$Errors), "logical")   # plain NA
  
  testthat::expect_true(is.na(out$Notes))
  testthat::expect_identical(typeof(out$Notes), "logical")    # plain NA
})


test_that("generate_doc_metrics_section works correctly", {
  result <- generate_doc_metrics_section(toy_assessment_results)
  
  expect_true(is.data.frame(result$has_examples))
    
  # Correct columns
  expect_identical(
    names(result$has_examples),
    c("function_name", "documentation_name", "documentation_location", "example")
  )
    
  # Score attribute exists
  expect_true(!is.null(attr(result$has_examples, "score")))
    
  # Score is numeric, NA allowed
  expect_true(is.numeric(attr(result$has_examples, "score")))
  
  # use match because string truncated in your display
  
  
  expect_true(is.data.frame(result$has_docs))
  
  expect_identical(
    names(result$has_docs),
    c("function_name", "documentation_name", "documentation_location")
  )
  
  # Score attribute exists
  expect_true(!is.null(attr(result$has_docs, "score")))
  
  # Score is numeric
  expect_true(is.numeric(attr(result$has_docs, "score")))
  
  
  expect_true(is.data.frame(result$doc_metrics))
  
  expect_identical(names(result$doc_metrics), c("Metric", "Value"))
  
  # Metric must be character
  expect_true(is.character(result$doc_metrics$Metric))
  
  # Value must be character (your example shows strings)
  expect_true(is.character(result$doc_metrics$Value))
  
  # high level structure test
  expect_true(is.list(result))
  expect_named(result, c("has_examples", "has_docs", "doc_metrics"))
  
})


# ---- Helpers ---------------------------------------------------------------

# Build a minimal assessment_results with pluggable 'has_examples'
make_base_assessment_for_examples <- function(hex_obj) {
  list(
    results = list(
      author = list(
        # Structure not actually used because we'll stub extract_maintainer_info()
        maintainer = list(name = "Any", email = "any@example.com")
      ),
      # Target under test:
      has_examples = hex_obj,
      
      # Provide a simple valid has_docs to avoid interference with our focus
      has_docs = data.frame(doc = "README", present = TRUE, stringsAsFactors = FALSE),
      
      # Other metrics (simple scalars) — will be passed through handle_null (stubbed)
      has_bug_reports_url = 1,
      license             = 1,
      has_news            = 0,
      has_source_control  = 1,
      has_vignettes       = 0,
      has_website         = 1,
      news_current        = 1,
      export_help         = 0
    )
  )
}

# Create a stubbed copy of the function so external helpers don't leak into tests
make_stubbed_fun <- function() {
  fn <- generate_doc_metrics_section
  # Stub external helpers referenced by the function
  mockery::stub(fn, "extract_maintainer_info", function(x) "Mock Maintainer <mock@example.com>")
  mockery::stub(fn, "handle_null", function(x) x)  # identity to keep values as-is
  fn
}

# ---- Tests -----------------------------------------------------------------

testthat::test_that("'has_examples' as data.frame is used directly and score attr set to NA_real_", {
  # This should exercise:
  # } else if (is.data.frame(hex)) {
  #   has_examples_df <- hex
  #   attr(has_examples_df, "score") <- NA_real_
  # }
  hex_df <- data.frame(
    name    = c("f1", "f2", "f3"),
    example = c("Has example", "No Example", "no example "),
    stringsAsFactors = FALSE
  )
  
  assessment_results <- make_base_assessment_for_examples(hex_df)
  fn <- make_stubbed_fun()
  
  out <- fn(assessment_results)
  
  # Validate structure
  testthat::expect_true(is.list(out))
  testthat::expect_true("has_examples" %in% names(out))
  has_examples_df <- out$has_examples
  testthat::expect_s3_class(has_examples_df, "data.frame")
  
  # Because the function filters to rows where example == "no example" (case/space-insensitive),
  # we should only keep rows 2 and 3 from the original data.
  testthat::expect_equal(nrow(has_examples_df), 2L)
  testthat::expect_true(all(tolower(trimws(has_examples_df$example)) == "no example"))
  
  # Attribute must be NA_real_
  sc <- attr(has_examples_df, "score")
  testthat::expect_true(is.na(sc))
  testthat::expect_identical(typeof(sc), "double")
})

testthat::test_that("'has_examples' missing (NULL/other) yields empty data.frame and score NA_real_", {
  
  hex <- NULL
  
  assessment_results <- make_base_assessment_for_examples(hex)
  fn <- make_stubbed_fun()
  
  out <- fn(assessment_results)
  
  has_examples_df <- out$has_examples
  testthat::expect_s3_class(has_examples_df, "data.frame")
  testthat::expect_equal(nrow(has_examples_df), 0L)
  testthat::expect_equal(ncol(has_examples_df), 0L)
  
  sc <- attr(has_examples_df, "score")
  testthat::expect_true(is.na(sc))
  testthat::expect_identical(typeof(sc), "double")
})


test_that("generate_pop_metrics_section works correctly", {
  result <- generate_pop_metrics_section(toy_assessment_results)
  expect_equal(result$Metric[1], "Date created")
  expect_equal(result$Value[1], "2016-07-19T14:47:19Z")
  expect_equal(result$Metric[2], "Stars")
  expect_equal(result$Value[2], "422")
  expect_equal(result$Metric[3], "Forks")
  expect_equal(result$Value[3], "45")
  expect_equal(result$Metric[4], "Data Extract Date")
  expect_equal(result$Value[4], "2025-07-02")
  expect_equal(result$Metric[5], "Recent Commits")
  expect_equal(result$Value[5], "0")
  expect_equal(result$Metric[6], "Open Issues")
  expect_equal(result$Value[6], "29")
  expect_equal(result$Metric[7], "Downloads Total")
  expect_equal(result$Value[7], "11.4M")
  expect_equal(result$Metric[8], "Downloads Last Month")
  expect_equal(result$Value[8], "299.4K")
})

# Test for generate_deps_section function
# Define the expected output
expected_deps_df <- data.frame(
  Package = c("checkmate", "testthat"),
  Version = c("2.1.0", "3.2.1.1"),
  Type = c("Imports", "Suggests"),
  stringsAsFactors = FALSE
)

expected_import_count <- 1
expected_suggest_count <- 1

# Test the generate_deps_section function
test_that("generate_deps_section works correctly", {
  result <- generate_deps_section(toy_assessment_results)
  
  expect_equal(result$deps_df, expected_deps_df)
  expect_equal(result$import_count, expected_import_count)
  expect_equal(result$suggest_count, expected_suggest_count)
})

# Define the toy assessment_results object
toy_assessment_results_2 <- list(
  results = list(
    rev_deps = c("abcrf", "abjutils", "accessrmd", "accucor", "ace2fastq")
  )
)

# Define the expected output
expected_rev_deps <- c("abcrf", "abjutils", "accessrmd", "accucor", "ace2fastq")

expected_rev_deps_no <- 5

# Test the generate_rev_deps_section function
test_that("generate_rev_deps_section works correctly", {
  result <- generate_rev_deps_section(toy_assessment_results_2)
  
  expect_equal(result$rev_deps_df$Reverse_dependencies, expected_rev_deps)
  expect_equal(result$rev_deps_summary$rev_deps_no, expected_rev_deps_no)
})

test_that("generate_trace_matrix_section works correctly with empty trace matrix", {
  # Toy dataset with empty trace matrix
  assessment_results <- list(
    covr_list = list(
      res_cov = list(
        coverage = list(
          totalcoverage = NA,
          filecoverage = NA
        )
      )
    )
  )
  
  file_coverage <- assessment_results$covr_list$res_cov$coverage$filecoverage
  total_coverage <- assessment_results$covr_list$res_cov$coverage$totalcoverage
  
  # Expected output
  expected_output <- data.frame(
    Exported_function = " ",
    Function_type = " ",
    Code_script = " ",
    Documentation = " ",
    Description = "Traceability matrix unsuccessful",
    Test_Coverage = " ",
    stringsAsFactors = FALSE
  )
  
  # Run the function
  result <- 
    generate_trace_matrix_section(total_coverage, file_coverage, assessment_results)
  
  # Check if the result matches the expected output
  expect_equal(result, expected_output)
})

test_that("generate_html_report returns message when directory does not exist", {
  # Mock the fs::dir_exists function to return FALSE
  mockery::stub(generate_html_report, "fs::dir_exists", FALSE)
  
  # Define the output directory
  output_dir <- "mock_output_dir"
  
  # Capture the message output
  expect_message(output_file <- generate_html_report(toy_assessment_results, output_dir), "The output directory does not exist.")
  
  # Check that the output file is NULL
  expect_null(output_file)
})


test_that("generate_html_report creates an HTML report", {
  skip_on_cran()
  # Create a temporary directory for output
  tmp_dir <- withr::local_tempdir()
  
  # Ensure cleanup of all files in tmp_dir
  withr::defer(unlink(tmp_dir, recursive = TRUE), envir = parent.frame())
  
  # Force rendering in CRAN/win-builder
  withr::local_envvar(NOT_CRAN = "true")
  
  # Run the function
  generate_html_report(toy_assessment_results, tmp_dir)
  
  # Find any .html file in tmp_dir
  html_files <- list.files(tmp_dir, pattern = "\\.html$", full.names = TRUE)
  
  # Check that at least one HTML file exists and is non-empty
  expect_true(length(html_files) > 0, info = "No HTML report was generated")
  expect_true(file.info(html_files[1])$size > 0, info = "HTML report is empty")
  
})


# Helper to build a minimal assessment_results with pluggable 'has_docs'
make_base_assessment <- function(has_docs_obj) {
  list(
    results = list(
      author = list(
        maintainer = list(name = "Any Name", email = "any@example.com")
      ),
      # has_examples is exercised but not the focus; we keep it minimal and valid
      has_examples = list(
        data = data.frame(
          `function` = c("f1", "f2"),
          example  = c("Has example", "No Example"),
          stringsAsFactors = FALSE
        ),
        example_score = 0.75
      ),
      # The branch under test:
      has_docs = has_docs_obj,
      
      # The remaining metrics used by the function — provide simple 0/1 values
      has_bug_reports_url = 1,
      license             = 1,
      has_news            = 0,
      has_source_control  = 1,
      has_vignettes       = 0,
      has_website         = 1,
      news_current        = 1,
      export_help         = 0
    )
  )
}

# Utility to create a stubbed copy of the function under test
make_stubbed_fun <- function() {
  # Local copy so stub works on a closure we can call in tests
  fn <- generate_doc_metrics_section
  
  # Stub external helpers referenced by the function
  mockery::stub(fn, "extract_maintainer_info", function(x) "Mock Maintainer <mock@example.com>")
  mockery::stub(fn, "handle_null", function(x) x)  # identity to keep values as-is
  
  fn
}

testthat::test_that("has_docs list with data + has_docs_score sets data.frame and score attr", {
  hdoc <- list(
    data = data.frame(doc = c("README", "NEWS"), present = c(TRUE, TRUE), stringsAsFactors = FALSE),
    has_docs_score = 0.9
  )
  assessment_results <- make_base_assessment(hdoc)
  fn <- make_stubbed_fun()
  
  out <- fn(assessment_results)
  
  testthat::expect_true(is.list(out))
  testthat::expect_true("has_docs" %in% names(out))
  
  has_docs_df <- out$has_docs
  testthat::expect_s3_class(has_docs_df, "data.frame")
  testthat::expect_equal(nrow(has_docs_df), 2L)
  testthat::expect_equal(attr(has_docs_df, "score"), 0.9, tolerance = 1e-12)
})

testthat::test_that("has_docs list with data + NULL has_docs_score falls back to NA_real_", {
  hdoc <- list(
    data = data.frame(doc = "README", present = TRUE, stringsAsFactors = FALSE),
    has_docs_score = NULL
  )
  assessment_results <- make_base_assessment(hdoc)
  fn <- make_stubbed_fun()
  
  out <- fn(assessment_results)
  
  has_docs_df <- out$has_docs
  testthat::expect_s3_class(has_docs_df, "data.frame")
  testthat::expect_equal(nrow(has_docs_df), 1L)
  
  sc <- attr(has_docs_df, "score")
  testthat::expect_true(is.na(sc))
  # Confirm numeric NA (NA_real_) — typeof should be "double"
  testthat::expect_identical(typeof(sc), "double")
})

testthat::test_that("has_docs data.frame path returns input data.frame and score attr NA_real_", {
  hdoc <- data.frame(
    doc = c("README", "VIGNETTE"),
    present = c(TRUE, FALSE),
    stringsAsFactors = FALSE
  )
  assessment_results <- make_base_assessment(hdoc)
  fn <- make_stubbed_fun()
  
  out <- fn(assessment_results)
  
  has_docs_df <- out$has_docs
  testthat::expect_s3_class(has_docs_df, "data.frame")
  testthat::expect_equal(nrow(has_docs_df), 2L)
  # Score attribute must be NA_real_
  sc <- attr(has_docs_df, "score")
  testthat::expect_true(is.na(sc))
  testthat::expect_identical(typeof(sc), "double")
})

testthat::test_that("has_docs fallback path (NULL/other) yields empty data.frame and score attr NA_real_", {
  hdoc <- NULL  # could also be an atomic or list without $data; either triggers the else branch
  assessment_results <- make_base_assessment(hdoc)
  fn <- make_stubbed_fun()
  
  out <- fn(assessment_results)
  
  has_docs_df <- out$has_docs
  testthat::expect_s3_class(has_docs_df, "data.frame")
  testthat::expect_equal(nrow(has_docs_df), 0L)
  testthat::expect_equal(ncol(has_docs_df), 0L)
  
  sc <- attr(has_docs_df, "score")
  testthat::expect_true(is.na(sc))
  testthat::expect_identical(typeof(sc), "double")
})

#' Function to get suggested exported functions
#'
#' @description This function gets exported functions for all packages
#' in the Suggests section of the target package's DESCRIPTION file
#'
#' @param data - all packages listed in the DESCRIPTION file
#'
#' @return - data with package names and exported functions
#' @export
get_suggested_exp_funcs <- function(data) {
  
  # Additional checks for empty dataframe
  assert_non_empty_df <- function(x, var.name = deparse(substitute(x))) {
    checkmate::assert_data_frame(x, min.rows = 1, .var.name = var.name)
  }
  
  assert_non_empty_df(data)
  
  # Filter for Suggested packages
  data <- data |> dplyr::filter(type == "Suggests")
  
  if (nrow(data) != 0) { 
    
  # Use lapply to iterate over each package and get the exported functions
    result <- lapply(data$package, function(pkg) {
      tryCatch({
        exports <- getNamespaceExports(pkg)
        descriptions <- get_func_descriptions(pkg)
        # Highlighted change: Match each export with its corresponding description
        matched_descriptions <- sapply(exports, function(func) {
          if (func %in% names(descriptions)) {
            descriptions[[func]]
          } else {
            NA
          }
        })
        data.frame(package = pkg, functions = exports, descriptions = matched_descriptions, stringsAsFactors = FALSE)
      }, error = function(e) {
        data.frame(package = pkg, functions = e$message, descriptions = NA, stringsAsFactors = FALSE)
      })
    })
    
    # Combine the results into a single data frame
    result_df <- do.call(rbind, result)
    
    # Filter out rows with specific keywords in descriptions
    result_df <- result_df %>%
      dplyr::filter(!grepl("deprecated|superseded|imported|re-exported", descriptions, ignore.case = TRUE)) %>%
      # filter out these functions from the methods package 
      # as they have no documentation
      dplyr::filter(!grepl("^\\.__C__|^\\.__T__", functions))
  } else {
    result_df <- data.frame(
      package = character(),
      functions = character(),
      descriptions = character(),
      stringsAsFactors = FALSE
    )
  }

  return(result_df)
}

# Test cases
test_that("check_suggested_exp_funcs returns matches correctly", {
  result <- risk.assessr::get_suggested_exp_funcs(deps_df)
  
  testthat::expect_true(checkmate::check_class(result, "data.frame"))
  
  testthat::expect_identical(length(result), 3L)

  expect_gte(nrow(result), 0L)

  expect_true(checkmate::check_character(result$package, 
                                       any.missing = TRUE)
  )
  
  expect_true(checkmate::check_character(result$functions, 
                                         any.missing = TRUE)
  )
  
  expect_true(checkmate::check_character(result$descriptions, 
                                         any.missing = TRUE)
  )
  
})

# test data for get_suggested_exp_funcs handles no suggested packages
test_that("handles no suggested packages", {
  no_suggests_df <- deps_df[deps_df$type != "Suggests", ]
  result <- risk.assessr::get_suggested_exp_funcs(no_suggests_df)
  expect_equal(nrow(result), 0)
  # Function to test if each column has no values
  test_no_values <- function(df) {
    sapply(df, function(column) all(is.na(column)))
  }
  
  no_values_test <- test_no_values(result)
  
  expect_true(all(no_values_test))
})

# Test cases
test_that("Function works with valid dataset", {
  result <- get_suggested_exp_funcs(deps_df)
  expect_true(nrow(result) > 0)
})

test_that("Function handles empty dataset", {
  empty_df <- data.frame(package = character(), type = character())
  expect_error(
    get_suggested_exp_funcs(empty_df),
    "Assertion on 'data' failed: Must have at least 1 rows, but has 0 rows."
  )
})

# Mock function to simulate getNamespaceExports
mock_getNamespaceExports <- function(pkg) {
  if (pkg == "pkgA") {
    return(character(0))  # Simulate no exported functions
  } else if (pkg == "pkgB") {
    return(c("func1", "func2"))  # Simulate exported functions
  } 
}

# Replace getNamespaceExports with the mock function
mockery::stub(get_suggested_exp_funcs, "getNamespaceExports", mock_getNamespaceExports)


# Mock function to simulate getNamespaceExports
mock_getNamespaceExports <- function(pkg) {
  if (pkg == "pkgC") {
    stop("Error in getting exports")  # Simulate an error
  }
}

# Replace getNamespaceExports with the mock function
mockery::stub(get_suggested_exp_funcs, "getNamespaceExports", mock_getNamespaceExports)

# Test case for no exported functions
test_that("get_suggested_exp_funcs handles no exported functions correctly", {
  data <- data.frame(
    package = c("pkgC"),
    type = c("Suggests"),
    stringsAsFactors = FALSE
  )
  
  result <- get_suggested_exp_funcs(data)
  
  # Define the expected messages
  expected_pkg_messages <- c(
    "pkgC"
  )
  
  expect_true(any(grepl("Error in getting exports", result$functions)))
})


# Mock function to simulate getNamespaceExports
mock_getNamespaceExports <- function(pkg) {
  if (pkg == "pkgB") {
    return(c("func1", "func2"))  # Simulate exported functions
  }
  stop("Unexpected package")
}

# Replace getNamespaceExports with the mock function
mockery::stub(get_suggested_exp_funcs, "getNamespaceExports", mock_getNamespaceExports)


# Test case for exported functions
test_that("get_suggested_exp_funcs handles exported functions correctly", {
  data <- data.frame(
    package = c("pkgB"),
    type = c("Suggests")
  )
  
  result <- get_suggested_exp_funcs(data)
  
  # Define the expected package messages
  expected_pkg_messages <- c(
    "pkgB",
    "pkgB",
    NA
  )
  
  # Loop through result$exported_functions and check for package messages
  for (i in seq_along(result$package)) {
    if (is.na(expected_pkg_messages[i])) {
      expect_true(is.na(result$package[[i]]))
    } else {
      expect_true(any(grepl(expected_pkg_messages[i], result$package[[i]])))
    }
  }
  
  # Define the expected function messages
  expected_func_messages <- c(
    "func1",
    "func2",
    NA
  )
  
  # Loop through result$exported_functions and check for function messages
  for (i in seq_along(result$exported_functions)) {
    if (is.na(expected_func_messages[i])) {
      expect_true(is.na(result$functions[[i]]))
    } else {
      expect_true(any(grepl(expected_func_messages[i], result$functions[[i]])))
    }
  }
})

test_that("get_suggested_exp_funcs handles empty input data frame", {
  # Mock data for empty input data frame
  data <- data.frame(package = character(), type = character(), stringsAsFactors = FALSE)
  
  # Mock the checkmate::assert_data_frame function to pass the check
  mock_assert_data_frame <- mockery::mock(TRUE, cycle = TRUE)
  mockery::stub(get_suggested_exp_funcs, "checkmate::assert_data_frame", mock_assert_data_frame)
  
  # Call the actual function
  result_df <- get_suggested_exp_funcs(data)
  
  # Construct the expected result
  expected <- data.frame(
    package = character(),
    functions = character(),
    descriptions = character(),
    stringsAsFactors = FALSE
  )
  
  # Check the results
  expect_equal(result_df, expected)
})  

#' Read Description file and parse the package name and version
#'
#' @param pkg_source_path path to package source code (untarred)
#' @param fields - select specified elements from description
#'
#' @return list with package description
#' @keywords internal
get_pkg_desc <- function(pkg_source_path, fields = NULL){
  
  pkg_desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  
  desc_file <- read.dcf(pkg_desc_path, fields = fields)[1L,]
  pkg_desc <- as.list(desc_file)
  
  return(pkg_desc)
}

#' Get risk metadata
#'
#' @param executor - user who executes the riskmetrics process 
#' 
#' adapted from mrgvalprep::get_sys_info() and mpn.scorecard
#' @importFrom rlang %||%
#' 
#' @return list with metadata
#' @keywords internal
get_risk_metadata <- function(executor = NULL) {
  checkmate::assert_string(executor, null.ok = TRUE)
  
  metadata <- list(
    datetime = as.character(Sys.time()),
    executor = executor %||% Sys.getenv("USER"),
    info = list()
  )
  
  metadata[["info"]][["sys"]] <- as.list(Sys.info())[c("sysname", "version", "release", "machine")]
  
  return(metadata)
}

#' Assign output file path for various outputs during scorecard rendering
#'
#' @param out_dir output directory for saving results
#' @param ext file name and extension
#'
#' @details
#' The basename of `out_dir` should be the package name and version pasted together
#' @keywords internal
get_result_path <- function(
     out_dir,
     ext = c("check.rds", "covr.rds", "tm_doc.rds", "tm_doc.xlsx")){
   
   ext <- match.arg(ext)
   
   pkg_name <- basename(out_dir)
   
   file.path(out_dir, paste0(pkg_name,".",ext))
 }

#' get package name for display
#'
#' @param input_string - string containing package name
#'
#' @return pkg_disp - package name for display
#'
#' @examples
#' \donttest{
#' pkg_source_path <- "/home/user/R/test.package.0001_0.1.0.tar.gz"
#' pkg_disp_1 <- get_pkg_name(pkg_source_path)
#' print(pkg_disp_1)
#' 
#' pkg <- "TxDb.Dmelanogaster.UCSC.dm3.ensGene_3.2.2.tar.gz"
#' pkg_disp_2 <- get_pkg_name(pkg)
#' print(pkg_disp_2)
#' }
#' 
#' @export
get_pkg_name <- function(input_string) {
  
  # check if input string is a file path or filename   
  test_string <-  stringr::str_match(input_string, "/")  
  
  if (any(is.na(test_string)) == FALSE) {
    # extract package name from the last part of the file path
    input_string <- stringr::str_split_i(input_string, "/", -1)  
    
  }
  
  # extract package name
  pkg_disp <- stringr::str_extract(input_string, "[^-|_]+")
  
  return(pkg_disp)
}

test_that("get package description works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
 
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    pkg_desc <- get_pkg_desc(pkg_source_path)
    
    expect_identical(length(pkg_desc), 17L)
    
    expect_true(checkmate::check_list(pkg_desc, 
                                            any.missing = FALSE)
    )
    
    expect_true(checkmate::check_list(pkg_desc, 
                                      types = "character")
    )
    
  }
  
})


test_that("get_risk_metadata returns correct metadata with executor", {
  # Mocked values
  fake_time <- "2025-09-30 12:00:00"
  fake_sysinfo <- list(
    sysname = "Linux",
    version = "#1 SMP Debian",
    release = "5.10.0-8-amd64",
    machine = "x86_64"
  )
  
  # Stub system functions
  mockery::stub(get_risk_metadata, "Sys.time", function() fake_time)
  mockery::stub(get_risk_metadata, "Sys.info", function() fake_sysinfo)
  
  result <- get_risk_metadata(executor = "edward")
  
  expect_equal(result$datetime, fake_time)
  expect_equal(result$executor, "edward")
  expect_equal(result$info$sys, fake_sysinfo)
})

test_that("get_risk_metadata returns correct metadata without executor", {
  # Mocked values
  fake_time <- "2025-09-30 12:00:00"
  fake_user <- "mock_user"
  fake_sysinfo <- list(
    sysname = "Darwin",
    version = "22.6.0",
    release = "macOS",
    machine = "arm64"
  )
  
  # Stub system functions
  mockery::stub(get_risk_metadata, "Sys.time", function() fake_time)
  mockery::stub(get_risk_metadata, "Sys.getenv", function(var) {
    if (var == "USER") return(fake_user)
  })
  mockery::stub(get_risk_metadata, "Sys.info", function() fake_sysinfo)
  
  result <- get_risk_metadata()
  
  expect_equal(result$datetime, fake_time)
  expect_equal(result$executor, fake_user)
  expect_equal(result$info$sys, fake_sysinfo)
})


test_that("get_result_path returns correct path for check.rds", {
  # Stub basename and file.path
  mockery::stub(get_result_path, "basename", function(path) "mockpkg")
  mockery::stub(get_result_path, "file.path", function(out_dir, filename) {
    paste0(out_dir, "/", filename)
  })
  
  result <- get_result_path("some/fake/path", ext = "check.rds")
  expect_equal(result, "some/fake/path/mockpkg.check.rds")
})

test_that("get_result_path returns correct path for covr.rds", {
  mockery::stub(get_result_path, "basename", function(path) "mockpkg")
  mockery::stub(get_result_path, "file.path", function(out_dir, filename) {
    paste0(out_dir, "/", filename)
  })
  
  result <- get_result_path("another/path", ext = "covr.rds")
  expect_equal(result, "another/path/mockpkg.covr.rds")
})

test_that("get_result_path defaults to check.rds when ext is not specified", {
  mockery::stub(get_result_path, "basename", function(path) "mockpkg")
  mockery::stub(get_result_path, "file.path", function(out_dir, filename) {
    paste0(out_dir, "/", filename)
  })
  
  result <- get_result_path("default/path")
  expect_equal(result, "default/path/mockpkg.check.rds")
})

input_string <- "/home/user/R/x86_64-pc-linux-gnu-library/4.1/package.metric/test-data/here_0.1.0.tar.gz"

input_string <- "/home/user/R/x86_64-pc-linux-gnu-library/4.1/package.metric/test-data/here-0.1.0.tar.gz"


input_string <- "TxDb.Dmelanogaster.UCSC.dm3.ensGene_3.2.2.tar.gz"

input_string <- "here-1.0.1.tar.gz"

input_string <- "here_1.0.1.tar.gz"

test_that("finds package name in file path", {

 pkg_source_path <- "/home/user/R/x86_64-pc-linux-gnu-library/4.1/risk.assessr/test-data/test.package.0001_0.1.0.tar.gz"
 
 pkg_disp <- risk.assessr::get_pkg_name(pkg_source_path)
 
 testthat::expect_true(!is.na(pkg_disp))
 
 testthat::expect_identical(length(pkg_disp), 1L)
 
 testthat::expect_identical(pkg_disp, "test.package.0001")
})

test_that("finds package name in file path", {
  
  pkg_source_path <- "/home/user/R/x86_64-pc-linux-gnu-library/4.1/risk.assessr/test-data/here_0.1.0.tar.gz"
  
  pkg_disp <- risk.assessr::get_pkg_name(pkg_source_path)
  
  testthat::expect_true(!is.na(pkg_disp))
  
  testthat::expect_identical(length(pkg_disp), 1L)
  
  testthat::expect_identical(pkg_disp, "here")
})

test_that("finds package name in file path", {
  
  pkg_source_path <- "/home/user/R/x86_64-pc-linux-gnu-library/4.1/risk.assessr/test-data/here-0.1.0.tar.gz"
  
  pkg_disp <- risk.assessr::get_pkg_name(pkg_source_path)
  
  testthat::expect_true(!is.na(pkg_disp))
  
  testthat::expect_identical(length(pkg_disp), 1L)
  
  testthat::expect_identical(pkg_disp, "here")
})


test_that("finds package name in tar file string", {
  pkg <- "TxDb.Dmelanogaster.UCSC.dm3.ensGene_3.2.2.tar.gz"
  
  pkg_disp <- risk.assessr::get_pkg_name(pkg)
  
  testthat::expect_true(!is.na(pkg_disp))
  
  testthat::expect_identical(length(pkg_disp), 1L)
  
  testthat::expect_identical(pkg_disp, "TxDb.Dmelanogaster.UCSC.dm3.ensGene")

})

test_that("finds package name in tar file string", {
  pkg <- "here_0.1.0.tar.gz"
  
  pkg_disp <- risk.assessr::get_pkg_name(pkg)
  
  testthat::expect_true(!is.na(pkg_disp))
  
  testthat::expect_identical(length(pkg_disp), 1L)
  
  testthat::expect_identical(pkg_disp, "here")
  
})

test_that("finds package name in tar file string", {
  pkg <- "here-0.1.0.tar.gz"
  
  pkg_disp <- risk.assessr::get_pkg_name(pkg)
  
  testthat::expect_true(!is.na(pkg_disp))
  
  testthat::expect_identical(length(pkg_disp), 1L)
  
  testthat::expect_identical(pkg_disp, "here")
  
})

#' Get Dependencies
#'
#' This function extracts the version information of imported and suggested packages for a given package from the current R session. 
#'
#' @param deps_list A data frame containing the dependency information of the package (provided by calc_dependencies function)
#'
#' @return A list with two elements:
#' \item{imports}{A named list of packages in the "Imports" section along with their corresponding versions }
#' \item{suggests}{A named list of packages in the "Suggests" section along with their corresponding versions }
#'
#' @examples
#' \donttest{
#' deps_list <- data.frame(
#'   package = c("dplyr", "ggplot2", "testthat", "knitr"),
#'   type = c("Imports", "Imports", "Suggests", "Suggests")
#' )
#' get_session_dependencies(deps_list)
#' }
#'
#' @importFrom utils sessionInfo
#' @export
get_session_dependencies <- function(deps_list) {
  
  # Get the session info
  info <- sessionInfo()
  
  deps_list$package <- gsub("\\s*\\([^\\)]+\\)", "", deps_list$package)
  
  # Get packages/version from session
  attached_pkgs <- info$otherPkgs
  loaded_pkgs <- info$loadedOnly
  
  if (is.null(attached_pkgs)) {
    attached_pkgs <- list()
  }
  
  if (is.null(loaded_pkgs)) {
    loaded_pkgs <- list()
  }
  
  all_pkgs <- c(attached_pkgs, loaded_pkgs)
  pkg_list <- sapply(all_pkgs, function(pkg) {
    if (is.list(pkg) && !is.null(pkg$Version)) {
      return(pkg$Version)
    } else {
      return("No package version found")  
    }
  })
  
  
  # Function to extract package versions
  create_pkg_version_list <- function(pkg_names, pkg_list) {
    
    pkg_versions <- lapply(pkg_names, function(pkg) {
      if (pkg %in% names(pkg_list)) {
        return(pkg_list[[pkg]])
      } else {
        return("No package version found")
      }
    })
    
    # Return a named list of package versions
    names(pkg_versions) <- pkg_names
    return(pkg_versions)
  }
  
  # Extract imports and suggests from the deps_list
  imports_pkgs <- deps_list$package[deps_list$type == "Imports"]
  suggests_pkgs <- deps_list$package[deps_list$type == "Suggests"]
  
  # Get the imports and suggests packages along with their versions or None
  imports_data <- create_pkg_version_list(imports_pkgs, pkg_list)
  suggests_data <- create_pkg_version_list(suggests_pkgs, pkg_list)
  
  return(list(
    imports = imports_data,
    suggests = suggests_data
  ))
}

test_that("get_session_dependencies correctly processes both Imports and Suggests", {
  mock_session_info <- function() {
    list(
      otherPkgs = list(
        "callr" = list(Version = "3.5.1"),
        "checkmate" = list(Version = "2.0.0"),
        "covr" = list(Version = "3.6.1"),
        "testthat" = list(Version = "3.0.0")
      ),
      loadedOnly = list(
        "knitr" = list(Version = "1.31")
      )
    )
  }
  
  deps_list <- data.frame(
    package = c("callr", "checkmate", "covr", "testthat", "knitr"),
    type = c("Imports", "Imports", "Imports", "Suggests", "Suggests"),
    stringsAsFactors = FALSE
  )
  
  with_mocked_bindings(
    sessionInfo = mock_session_info,
    {
      result <- get_session_dependencies(deps_list)
      expect_equal(result$imports, list(callr = "3.5.1", checkmate = "2.0.0", covr = "3.6.1"))
      expect_equal(result$suggests, list(testthat = "3.0.0", knitr = "1.31"))
    }
  )
})

test_that("get_session_dependencies works when only Imports are provided", {
  mock_session_info <- function() {
    list(
      otherPkgs = list(
        "callr" = list(Version = "3.5.1"),
        "checkmate" = list(Version = "2.0.0"),
        "covr" = list(Version = "3.6.1")
      ),
      loadedOnly = list()
    )
  }
  
  deps_list <- data.frame(
    package = c("callr", "checkmate", "covr"),
    type = c("Imports", "Imports", "Imports"),
    stringsAsFactors = FALSE
  )
  
  with_mocked_bindings(
    sessionInfo = mock_session_info,
    {
      result <- get_session_dependencies(deps_list)
      
      expect_equal(result$imports, list(callr = "3.5.1", checkmate = "2.0.0", covr = "3.6.1"))
      expect_length(result$suggests, 0) 
    }
  )
})

test_that("get_session_dependencies works when only Suggests are provided", {
  mock_session_info <- function() {
    list(
      otherPkgs = list(
        "testthat" = list(Version = "3.0.0"),
        "knitr" = list(Version = "1.31")
      ),
      loadedOnly = list()
    )
  }
  
  deps_list <- data.frame(
    package = c("testthat", "knitr"),
    type = c("Suggests", "Suggests"),
    stringsAsFactors = FALSE
  )
  
  with_mocked_bindings(
    sessionInfo = mock_session_info,
    {
      result <- get_session_dependencies(deps_list)
      
      expect_length(result$imports, 0) 
      expect_equal(result$suggests, list(testthat = "3.0.0", knitr = "1.31"))
    }
  )
})


test_that("get_session_dependencies handles empty deps_list gracefully", {
  mock_session_info <- function() {
    list(
      otherPkgs = list(
        "callr" = list(Version = "3.5.1")
      ),
      loadedOnly = list()
    )
  }
  
  deps_list <- data.frame(
    package = character(0),
    type = character(0),
    stringsAsFactors = FALSE
  )
  
  with_mocked_bindings(
    sessionInfo = mock_session_info,
    {
      result <- get_session_dependencies(deps_list)
      expect_length(result$imports, 0) 
      expect_length(result$suggests, 0)
    }
  )
})

test_that("get_session_dependencies handles missing package from session info with descriptive 'None'", {
  mock_session_info <- function() {
    list(
      otherPkgs = list(
        "callr" = list(Version = "3.5.1")
      ),
      loadedOnly = list()
    )
  }
  
  deps_list <- data.frame(
    package = c("callr", "nonexistentPkg"),
    type = c("Imports", "Imports"),
    stringsAsFactors = FALSE
  )
  
  with_mocked_bindings(
    sessionInfo = mock_session_info,
    {
      result <- get_session_dependencies(deps_list)
      
      expect_equal(result$imports, list(callr = "3.5.1", nonexistentPkg = "No package version found"))
    }
  )
})

#' set up rcmdcheck arguments
#' 
#' @description This sets up rcmdcheck arguments
#' @details {Some packages need to have build vignettes as a 
#' build argument as their vignettes structure is inst/doc or inst/docs} 
#' 
#' @param check_type basic R CMD check type - "1" CRAN R CMD check_type - "2"
#' @param build_vignettes Logical (T/F). Whether or not to build vignettes
#' 
#' @return - list with rcmdcheck arguments
#' @keywords internal
setup_rcmdcheck_args <- function(check_type = "1", 
                                 build_vignettes) {
if (check_type == "1") {
  rcmdcheck_args = list(
    timeout = Inf,
    args = c("--no-examples", # don't check examples
             "--ignore-vignettes", # skip all tests on vignettes
             "--no-vignettes", # do not run R code in vignettes nor build outputs
             "--no-manual"), # disable pdf manual rendering
    build_args = if (Sys.info()[["sysname"]] == "Windows") {
      c("--no-build-vignettes", # do not build vignette outputs
        "--no-manual") # disable pdf manual rendering on Windows
    } else {
      c("--no-build-vignettes") # do not build vignette outputs
    },  
    # FORCE_SUGGESTS give an error if suggested packages are not available. 
    # Default: true (but false for CRAN submission checks)
    env = c(`_R_CHECK_FORCE_SUGGESTS_` = "FALSE"),
    quiet = FALSE
  )
} else { 
  
  if (build_vignettes == FALSE) {
    rcmdcheck_args = list(
      timeout = Inf,
      args = c("--ignore-vignettes", # skip all tests on vignettes
               "--no-vignettes", # do not run R code in vignettes nor build outputs
               "--as-cran", # select customizations similar to those used for CRAN incoming checking"
               "--no-manual"), # disable pdf manual rendering
      build_args = "--no-build-vignettes", # do not build vignette outputs
      # FORCE_SUGGESTS give an error if suggested packages are not available. 
      # Default: true (but false for CRAN submission checks)
      env = c(`_R_CHECK_FORCE_SUGGESTS_` = "FALSE"), 
      quiet = FALSE
    )
  } else {
    rcmdcheck_args = list(
      timeout = Inf,
      args = c("--ignore-vignettes", # skip all tests on vignettes
               "--no-vignettes", # do not run R code in vignettes nor build outputs
               "--as-cran", # select customizations similar to those used for CRAN incoming checking"
               "--no-manual"), # disable pdf manual rendering
      # FORCE_SUGGESTS give an error if suggested packages are not available. 
      # Default: true (but false for CRAN submission checks)
      env = c(`_R_CHECK_FORCE_SUGGESTS_` = "FALSE"),
      quiet = FALSE
    )
  }
}  
  return(rcmdcheck_args)
}

  

test_that("test setup_rcmdcheck_args", {

  check_type <- "1"
  
  rcmdcheck_args <- setup_rcmdcheck_args(check_type, 
                                                             build_vignettes)
  
  expect_identical(length(rcmdcheck_args), 5L)
  
  expect_true(checkmate::check_list(rcmdcheck_args, all.missing = FALSE))
  
  expect_true(checkmate::check_list(rcmdcheck_args, any.missing = FALSE))
  
  check_type <- "2"
  
  build_vignettes <- FALSE
  
  rcmdcheck_args <- setup_rcmdcheck_args(check_type, 
                                                             build_vignettes)
  
  expect_identical(length(rcmdcheck_args), 5L)
  
  expect_true(checkmate::check_list(rcmdcheck_args, all.missing = FALSE))
  
  expect_true(checkmate::check_list(rcmdcheck_args, any.missing = FALSE))
  
  check_type <- "2"
  
  build_vignettes <- TRUE
  
  rcmdcheck_args <- setup_rcmdcheck_args(check_type, 
                                                             build_vignettes)
  
  expect_identical(length(rcmdcheck_args), 4L)
  
  expect_true(checkmate::check_list(rcmdcheck_args, all.missing = FALSE))
  
  expect_true(checkmate::check_list(rcmdcheck_args, any.missing = FALSE))

})

test_that("build_args is set correctly based on OS", {
  # Mock Sys.info to return "Windows"
  mock_sys_info <- mockery::mock(list(sysname = "Windows"))
  mockery::stub(Sys.info, "Sys.info", mock_sys_info)
  
  build_args <- if (mock_sys_info()[["sysname"]] == "Windows") {
    c("--no-build-vignettes", # do not build vignette outputs
      "--no-manual") # disable pdf manual rendering on Windows
  } else {
    c("--no-build-vignettes") # do not build vignette outputs
  }
  
  expect_equal(build_args, c("--no-build-vignettes", "--no-manual"))
  
  # Mock Sys.info to return "Linux"
  mock_sys_info <- mockery::mock(list(sysname = "Linux"))
  mockery::stub(Sys.info, "Sys.info", mock_sys_info)
  
  build_args <- if (mock_sys_info()[["sysname"]] == "Windows") {
    c("--no-build-vignettes", # do not build vignette outputs
      "--no-manual") # disable pdf manual rendering on Windows
  } else {
    c("--no-build-vignettes") # do not build vignette outputs
  }
  
  expect_equal(build_args, c("--no-build-vignettes"))
})

#' Check for tests/testthat and _snaps folder and count golden tests
#'
#' @param pkg_source_path Path to the root of the package source
#'
#' @return A list with:
#'   - `has_testthat`: Does tests/testthat exist?
#'   - `has_testit`: Does tests/testit exist?
#'   - `has_tests_base`: Do tests exist in tests subdirectory?
#'   - `has_snaps`: Does _snaps exist inside tests/testthat?
#'   - `n_golden_tests`: Number of snapshot files inside _snaps
#'   - `n_test_files`: Number of test-*.R files inside tests/testthat
#' @keywords internal
check_pkg_tests_and_snaps <- function(pkg_source_path) {
  message("checking package test config")
  
  test_dir <- file.path(pkg_source_path, "tests")
  testthat_path <- file.path(test_dir, "testthat")
  snaps_path <- file.path(testthat_path, "_snaps")
  
  testit_path <- file.path(test_dir, "testit")
  test_ci_path <- file.path(test_dir, "test-ci")
  test_cran_path <- file.path(test_dir, "test-cran")
  
  # Check for testthat and testit (standard and nonstandard)
  has_testthat <- dir.exists(testthat_path)
  has_testit <- dir.exists(testit_path) || (dir.exists(test_ci_path) && dir.exists(test_cran_path))
  
  # Count golden test snapshot files
  has_snaps <- dir.exists(snaps_path)
  n_golden_tests <- if (has_snaps) {
    snapshot_files <- list.files(snaps_path, recursive = TRUE, full.names = TRUE)
    length(snapshot_files)
  } else {
    0
  }
  
  # Count test-*.R files in testthat
  n_test_files <- if (has_testthat) {
    test_files <- list.files(
      testthat_path,
      pattern = "^test-.*\\.R$",
      recursive = TRUE,
      full.names = TRUE
    )
    test_files <- test_files[!grepl("_snaps", test_files)]
    length(test_files)
  } else {
    0
  }
  
  # Only check for base R test scripts if none of the known test frameworks are present
  has_tests_base <- FALSE
  if (!has_testthat && !has_testit && !dir.exists(test_ci_path) && !dir.exists(test_cran_path)) {
    base_test_files <- list.files(
      test_dir,
      pattern = "\\.R$",
      full.names = TRUE
    )
    # Exclude files in known subdirectories
    base_test_files <- base_test_files[dirname(base_test_files) == test_dir]
    has_tests_base <- length(base_test_files) > 0
  }
  
  return(list(
    has_testthat = has_testthat,
    has_snaps = has_snaps,
    has_testit = has_testit,
    has_tests_base = has_tests_base,
    n_golden_tests = n_golden_tests,
    n_test_files = n_test_files
  ))
}

#' Determine the appropriate test path for a package
#'
#' This internal utility function inspects the `test_pkg_data` structure to identify
#' which testing framework is used in the package and returns the corresponding test path.
#'
#' @param test_pkg_data A named list containing logical flags such as `has_testthat`, `has_testit`, `has_tests_base`, etc.
#' @param testdir A character string specifying the root directory where test folders are located.
#'
#' @return A character string representing the path to the test directory, or `NULL` if no known test framework is found.
#'
#' @keywords internal
get_test_path <- function(test_pkg_data, testdir) {
  if (isTRUE(test_pkg_data$has_testthat)) {
    return(file.path(testdir, "testthat"))
  } else if (isTRUE(test_pkg_data$has_testit)) {
    return(file.path(testdir, "testit"))
  } else if (
    dir.exists(file.path(testdir, "test-ci")) &&
    dir.exists(file.path(testdir, "test-cran"))
  ) {
    return(testdir)  # fallback for nonstandard testit
  } else if (isTRUE(test_pkg_data$has_tests_base)) {
    return(testdir)  # base R scripts
  }
  
  return(NULL)
}

test_that("check_pkg_tests_and_snaps works as expected", {
  # Create a temp directory to act as our package
  pkg_dir <- tempfile("dummyPkg")
  dir.create(pkg_dir)
  
  # Create tests/testthat/_snaps structure
  testthat_dir <- file.path(pkg_dir, "tests", "testthat")
  snaps_dir <- file.path(testthat_dir, "_snaps")
  dir.create(snaps_dir, recursive = TRUE)
  
  # Create some dummy golden snapshot files
  file.create(file.path(snaps_dir, "snapshot1.md"))
  file.create(file.path(snaps_dir, "snapshot2.md"))
  
  # Call the function
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  # Check the results
  expect_true(result$has_testthat)
  expect_false(result$has_testit)
  expect_true(result$has_snaps)
  expect_equal(result$n_golden_tests, 2)
  
  # Clean up (optional)
  unlink(pkg_dir, recursive = TRUE)
})

test_that("works when both testthat and _snaps exist with files", {
  pkg_dir <- tempfile("dummyPkg1")
  dir.create(pkg_dir)
  
  testthat_dir <- file.path(pkg_dir, "tests", "testthat")
  snaps_dir <- file.path(testthat_dir, "_snaps")
  dir.create(snaps_dir, recursive = TRUE)
  
  file.create(file.path(snaps_dir, "snapshot1.md"))
  file.create(file.path(snaps_dir, "snapshot2.md"))
  
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  expect_true(result$has_testthat)
  expect_false(result$has_testit)  
  expect_true(result$has_snaps)
  expect_equal(result$n_golden_tests, 2)
  
  unlink(pkg_dir, recursive = TRUE)
})


test_that("works when testthat exists but _snaps does not", {
  pkg_dir <- tempfile("dummyPkg2")
  dir.create(pkg_dir)
  
  testthat_dir <- file.path(pkg_dir, "tests", "testthat")
  dir.create(testthat_dir, recursive = TRUE)
  
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  expect_true(result$has_testthat)
  expect_false(result$has_testit)  
  expect_false(result$has_snaps)
  expect_equal(result$n_golden_tests, 0)
  
  unlink(pkg_dir, recursive = TRUE)
})


test_that("works when neither testthat nor _snaps exist", {
  pkg_dir <- tempfile("dummyPkg3")
  dir.create(pkg_dir)
  
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  expect_false(result$has_testthat)
  expect_false(result$has_testit)  
  expect_false(result$has_snaps)
  expect_equal(result$n_golden_tests, 0)
  
  unlink(pkg_dir, recursive = TRUE)
})


test_that("works when _snaps exists but is empty", {
  pkg_dir <- tempfile("dummyPkg4")
  dir.create(pkg_dir)
  
  testthat_dir <- file.path(pkg_dir, "tests", "testthat")
  snaps_dir <- file.path(testthat_dir, "_snaps")
  dir.create(snaps_dir, recursive = TRUE)
  
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  expect_true(result$has_testthat)
  expect_false(result$has_testit)  
  expect_true(result$has_snaps)
  expect_equal(result$n_golden_tests, 0)
  
  unlink(pkg_dir, recursive = TRUE)
})


test_that("counts nested snapshot files", {
  pkg_dir <- tempfile("dummyPkg5")
  dir.create(pkg_dir)
  
  testthat_dir <- file.path(pkg_dir, "tests", "testthat")
  snaps_dir <- file.path(testthat_dir, "_snaps")
  nested_dir <- file.path(snaps_dir, "nested")
  dir.create(nested_dir, recursive = TRUE)
  
  file.create(file.path(snaps_dir, "snapshot1.md"))
  file.create(file.path(nested_dir, "snapshot2.md"))
  
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  expect_true(result$has_testthat)
  expect_false(result$has_testit)  
  expect_true(result$has_snaps)
  expect_equal(result$n_golden_tests, 2)
  
  unlink(pkg_dir, recursive = TRUE)
})


# testit

test_that("detects testit framework presence", {
  pkg_dir <- tempfile("dummyPkg_testit")
  dir.create(pkg_dir)
  
  testit_dir <- file.path(pkg_dir, "tests", "testit")
  dir.create(testit_dir, recursive = TRUE)
  
  result <- check_pkg_tests_and_snaps(pkg_dir)
  
  expect_false(result$has_testthat)
  expect_true(result$has_testit)  
  unlink(pkg_dir, recursive = TRUE)
})


test_that("get_test_path returns correct paths based on test_pkg_data", {
  testdir <- tempfile()
  dir.create(testdir)
  
  # Create dummy subdirectories
  dir.create(file.path(testdir, "testthat"))
  dir.create(file.path(testdir, "testit"))
  dir.create(file.path(testdir, "test-ci"))
  dir.create(file.path(testdir, "test-cran"))
  
  # Case: has_testthat
  test_pkg_data <- list(has_testthat = TRUE, has_testit = FALSE, has_tests_base = FALSE)
  expect_equal(get_test_path(test_pkg_data, testdir), file.path(testdir, "testthat"))
  
  # Case: has_testit
  test_pkg_data <- list(has_testthat = FALSE, has_testit = TRUE, has_tests_base = FALSE)
  expect_equal(get_test_path(test_pkg_data, testdir), file.path(testdir, "testit"))
  
  # Case: fallback for nonstandard testit
  test_pkg_data <- list(has_testthat = FALSE, has_testit = FALSE, has_tests_base = FALSE)
  expect_equal(get_test_path(test_pkg_data, testdir), testdir)
  
  # Case: has_tests_base
  unlink(file.path(testdir, "test-ci"), recursive = TRUE)
  unlink(file.path(testdir, "test-cran"), recursive = TRUE)
  test_pkg_data <- list(has_testthat = FALSE, has_testit = FALSE, has_tests_base = TRUE)
  expect_equal(get_test_path(test_pkg_data, testdir), testdir)
  
  # Case: no known test framework
  test_pkg_data <- list(has_testthat = FALSE, has_testit = FALSE, has_tests_base = FALSE)
  expect_null(get_test_path(test_pkg_data, testdir))
  
  # Cleanup
  unlink(testdir, recursive = TRUE)
})

#' list all package exports
#'
#'
#' @param pkg_source_path a file path pointing to an unpacked/untarred package directory
#'
#' @return data.frame, with one column `exported_function`, that can be passed
#'   to all downstream map_* helpers
#'
#' @examples
#' \dontrun{
#' tmpdir <- tempdir()
#'
#' # Locate the package source tarball bundled with risk.assessr
#' dp <- system.file("test-data", "test.package.0001_0.1.0.tar.gz",
#'                   package = "risk.assessr")
#'
#' # Extract the source package into the temporary directory
#' utils::untar(dp, exdir = tmpdir)
#'
#' # Find the extracted package directory (it should contain DESCRIPTION)
#' pkg_dirs <- list.dirs(tmpdir, full.names = TRUE, recursive = FALSE)
#' desc_dir <- pkg_dirs[file.exists(file.path(pkg_dirs, "DESCRIPTION"))]
#' stopifnot(length(desc_dir) == 1L)
#' pkg_source_path <- desc_dir
#'
#' # Now operate on the *source* (no installation)
#' exports_df <- get_exports(pkg_source_path)
#'
#' # Optional: inspect result (kept simple for examples)
#' head(exports_df)
#'
#' # Clean up the extracted sources
#' unlink(pkg_source_path, recursive = TRUE, force = TRUE)
#' }
#' @export
get_exports <- function(pkg_source_path) {
  
  package_name <- extract_package_name(pkg_source_path)
  # Check if the package source path contains an R folder
  exp_possible <- contains_r_folder(pkg_source_path)
  
  if (exp_possible == TRUE) {
    
    result <- extract_exported_function_info(pkg_source_path, package_name)
    
    # check if func_df is empty of values    
    is_empty <- result %>% 
      dplyr::summarise_all(~ all(is.na(.) | . == " ")) %>% 
      unlist() %>% all()
    
    if (is_empty) {
      message(glue::glue("No exported functions from source package {package_name}"))
      result <- dplyr::tibble(
        exported_function = NA,
        class = NA,
        function_type = NA,
        function_body = "No exported functions from source package",
        where = package_name
      )  
    } else {   
      
      message(glue::glue("Extracting exported function bodies for {package_name}"))
      
      #ensure that every function_body is consistently a character vector, 
      # even if it's a serialized version of a list (e.g., for R6 methods).
      normalize_function_body <- function(row, package_name) {
        info <- classify_function_body(row, package_name)
        
        # Ensure function_body is always character
        if (is.list(info$function_body)) {
          info$function_body <- paste(capture.output(str(info$function_body)), collapse = "\n")
        }
        
        return(info)
      }
      
      # This code transforms a matrix or data frame result by using 
      # apply to execute a classification function to each row, 
      # and then converts the output into a transposed data frame.
      #result_df_list <- as.data.frame(t(apply(result, 1, function(row) {
      #  as.list(classify_function_body(as.list(row), package_name))
      #})))
      
      
      result_df_list <- as.data.frame(t(apply(result, 1, function(row) {
        as.list(normalize_function_body(as.list(row), package_name))
      })))
      
      # Extract all columns (e.g., V1, V2, ...) and flatten their first list element
      result <- map_dfr(result_df_list, function(col) {
        if (is.list(col[[1]])) {
          as_tibble(col[[1]])
        } else {
          tibble()
        }
        
      })
    }  
  } else {
    message(glue::glue("No R folder found in the package source path for {package_name}"))
    result <- dplyr::tibble(
      exported_function = NA,
      class = NA,
      function_type = NA,
      function_body = "No R folder found in the package source path",
      where = package_name
    )  
  }  
  
  return(result)
}


#' Extract Exported Function Metadata from an R Package
#'
#' This internal utility function analyzes the namespace of an R package and extracts
#' metadata about its exported functions, including classification into S3, S4, S7, and regular functions.
#'
#' @param .pkg_source_path Character. Path to the source directory of the R package.
#' @param package_name Character. Name of the package (typically the folder name or actual package name).
#'
#' @return A tibble with columns: `exported_function`, `class`, `function_type`, `function_body`, and `where`.
#' @keywords internal
extract_exported_function_info <- function(.pkg_source_path, package_name) {
  
  nsInfo <- parseNamespaceFile(package_name, dirname(.pkg_source_path), mustExist = FALSE)
  
  message(glue::glue("Extracting exported function names for {package_name}"))
  
  exports <- character(0)
  
  if (!is.null(nsInfo) && !rlang::is_empty(nsInfo$exports)) {
    exports <- unname(unlist(nsInfo[["exports"]]))
  } else {
    exports <- getNamespaceExports(package_name)
  }
  
  # Fallback: parse export() entries manually from NAMESPACE file
  namespace_file <- file.path(.pkg_source_path, "NAMESPACE")
  if (file.exists(namespace_file)) {
    ns_lines <- readLines(namespace_file)
    export_lines <- grep("^export\\(", ns_lines, value = TRUE)
    
    manual_exports <- regmatches(export_lines, gregexpr("export\\(([^\\)]+)\\)", export_lines))
    manual_exports <- unlist(lapply(manual_exports, function(x) {
      if (length(x) > 0) {
        sub("export\\(([^\\)]+)\\)", "\\1", x)
      } else {
        NULL
      }
    }))
    
    exports <- unique(c(exports, manual_exports))
  }
  
  
  # Define packages that manually export helpers like .data
  preserve_helpers_pkgs <- c("ggplot2", "rlang", "dplyr", "tidyselect")
  
  # Build export_info
  export_info <- lapply(exports, function(f) {
    obj <- suppressWarnings(
      tryCatch(getExportedValue(package_name, f), error = function(e) NULL)
    )
    
    if (is.null(obj)) {
      # Preserve unresolved exports for specific packages
      if (package_name %in% preserve_helpers_pkgs) {
        return(list(name = f, class = NA, type = "unresolved or helper"))
      } else {
        return(NULL)
      }
    }
    
    if (inherits(obj, "S7_generic")) {
      return(list(name = f, class = NA, type = "S7 generic"))
    }
    
    if (inherits(obj, "S7_class")) {
      return(list(name = f, class = NA, type = "S7 class"))
    }
    
    if (is.function(obj)) {
      body_text <- paste(deparse(body(obj)), collapse = "\n")
      if (grepl("S7::new\\(|new\\(", body_text) || grepl("S7::new_object\\(", body_text)) {
        return(list(name = f, class = NA, type = "S7 constructor"))
      }
      return(list(name = f, class = NA, type = "regular function"))
    }
    
    return(list(name = f, class = NA, type = "non-function or helper"))
  })
  
  # Apply filtering only if package is not in the preserve list
  if (!(package_name %in% preserve_helpers_pkgs)) {
    export_info <- Filter(Negate(is.null), export_info)
  }
  
  export_info_df <- dplyr::tibble(
    exported_function = vapply(export_info, `[[`, character(1), "name"),
    class = vapply(export_info, function(x) {
      if (is.null(x$class) || is.na(x$class)) NA_character_ else as.character(x$class)
    }, character(1)),
    function_type = vapply(export_info, `[[`, character(1), "type"),
    function_body = rep("", length(export_info)),
    where = rep(package_name, length(export_info))
  )
  
  # --- Add S3 methods from NAMESPACE file manually ---
  s3_generics <- character(0)
  s3_classes <- character(0)
  
  if (file.exists(namespace_file)) {
    ns_lines <- readLines(namespace_file)
    s3_lines <- grep("^S3method\\(", ns_lines, value = TRUE)
    
    for (line in s3_lines) {
      matches <- regmatches(line, regexec("S3method\\(([^,]+),\\s*([^\\)]+)\\)", line))[[1]]
      if (length(matches) == 3) {
        generic_full <- matches[2]
        cls <- matches[3]
        
        generic_name <- sub(".*::", "", generic_full)
        
        s3_generics <- c(s3_generics, generic_name)
        s3_classes <- c(s3_classes, cls)
      }
    }
  }
  
  if (!rlang::is_empty(nsInfo$S3methods)) {
    s3_generics <- c(s3_generics, nsInfo$S3methods[, 1])
    s3_classes <- c(s3_classes, nsInfo$S3methods[, 2])
  }
  
  # set up s3 function_type
  s3_function_type <- character(length(s3_generics))
  if (length(s3_generics) > 0) {
    s3_function_type <- ifelse(s3_generics %in% exports, "S3 generic + method", "S3 method")
  }
  
  # set up s3_df
  s3_df <- dplyr::tibble(
    exported_function = s3_generics,
    class = s3_classes,
    function_type = s3_function_type,
    function_body = rep("", length(s3_generics)),
    where = rep(package_name, length(s3_generics))
  )
  
  
  # --- Add S4 methods and generics ---
  s4_df <- NULL
  if (!rlang::is_empty(nsInfo$exportMethods)) {
    s4_methods_raw <- nsInfo$exportMethods
    
    if (is.character(s4_methods_raw)) {
      s4_df <- dplyr::tibble(
        exported_function = unique(s4_methods_raw),
        class = NA_character_,
        function_type = "S4 generic",
        function_body = "",
        where = package_name
      )
    } else if (is.list(s4_methods_raw)) {
      valid_s4_methods <- Filter(function(x) is.character(x) && length(x) >= 2 && !is.na(x[[2]]), s4_methods_raw)
      
      if (length(valid_s4_methods) > 0) {
        s4_func_names <- sapply(valid_s4_methods, `[[`, 1)
        s4_classes <- sapply(valid_s4_methods, `[[`, 2)
        
        s4_df <- unique(dplyr::tibble(
          exported_function = s4_func_names,
          class = s4_classes,
          function_type = "S4 function",
          function_body = "",
          where = package_name
        ))
      }
    }
  }
  
  # --- Remaining exports not in export_info ---
  remaining_exports <- setdiff(exports, export_info_df$exported_function)
  remaining_df <- dplyr::tibble(
    exported_function = remaining_exports,
    class = NA_character_,
    function_type = "unknown",
    function_body = "",
    where = package_name
  )
  
  # --- Combine all sources ---
  result <- dplyr::bind_rows(
    export_info_df,
    if (exists("s3_df") && !is.null(s3_df)) s3_df else NULL,
    if (exists("s4_df") && !is.null(s4_df)) s4_df else NULL,
    if (exists("remaining_df") && !is.null(remaining_df)) remaining_df else NULL
  ) |>
    dplyr::distinct(exported_function, class, .keep_all = TRUE)
  
  return(result)
}




#' Extract package name from package source path
#'
#' @param pkg_source_path a file path pointing to an unpacked/untarred package directory
#'
#' @return  package_name
#'
#' @keywords internal
extract_package_name <- function(pkg_source_path) {
  # Split the path by "::" in case it's a compound path
  parts <- strsplit(pkg_source_path, "::")[[1]]
  
  # Extract the part before "::"
  package_part <- parts[1]
  
  # Remove everything before the package name and version
  package_name_with_version <- sub(".*[/\\\\]", "", package_part)
  
  # Extract only the package name (before the first hyphen followed by a digit)
  package_name <- sub("-[0-9].*$", "", package_name_with_version)
  
  return(package_name)
}


#' Extract all S4 methods 
#'
#' @param generic_name name of function
#'
#' @return  list with method_strings, generic, and methods
#'
#' @keywords internal
get_all_s4_methods <- function(generic_name) {
  is_generic <- methods::isGeneric(generic_name)
  methods_strings <- list()
  has_methods <- FALSE
  
  if (is_generic) {
    methods_list <- methods::findMethods(generic_name)
    
    for (i in seq_along(methods_list@.Data)) {
      method_obj <- methods_list@.Data[[i]]
      if (inherits(method_obj, "MethodDefinition")) {
        method_def <- method_obj@.Data
        method_string <- deparse(method_def)
        methods_strings[[i]] <- paste(method_string, collapse = "\n")
        has_methods <- TRUE
      }
    }
  }
  
  list(
    bodies = methods_strings,
    is_generic = is_generic,
    has_methods = has_methods
  )
}


#' function to preprocess func_full_name 
#' 
#' @description function to extract package, class, and generic
#'  
#' @param func_full_name - full name of the function 
#'
#' @keywords internal
preprocess_func_full_name <- function(func_full_name) {
  
  package <- generic <- class <- NULL
  remaining <- func_full_name
  
  # Handle internal access (:::)
  if (stringr::str_detect(func_full_name, ":::")) {
    package <- stringr::str_extract(func_full_name, "^[^:]+")
    generic <- stringr::str_extract(func_full_name, "(?<=::)[^\\.]+")
    remaining <- stringr::str_match(func_full_name, ":::([^:]+)$")[, 2]
    class <- paste(package, ":::", remaining, sep = "")
    
    # Handle external access (::)
  } else if (grepl("::", func_full_name)) {
    parts <- strsplit(func_full_name, "::")[[1]]
    package <- parts[1]
    remaining <- parts[2]
    remaining_parts <- strsplit(remaining, "\\.")[[1]]
    generic <- remaining_parts[1]
    class <- paste(remaining_parts[-1], collapse = ".")
    
    # No namespace qualifier
  } else {
    parts <- strsplit(func_full_name, "\\.")[[1]]
    remaining <- func_full_name
  }
  
  # Handle infix operators like %/% or %%
  if (grepl("^%.*%$", remaining) || grepl("^`%.*%`$", remaining)) {
    # Normalize to backtick-wrapped form
    if (!grepl("^`.*`$", remaining)) {
      remaining <- paste0("`", remaining, "`")
    }
    generic <- remaining
    class <- "" # Class is not embedded in the name for S4 methods
  }
  
  # Handle special dot-prefixed or double-dot forms
  if (grepl("^\\.\\.|\\.\\.$", remaining)) {
    generic <- remaining
    class <- remaining
  } else if (grepl("^\\.", remaining)) {
    generic <- remaining
    class <- ""
  }
  
  # Clean version of generic for use with isGeneric() and findMethods()
  generic_clean <- if (!is.null(generic)) gsub("^`|`$", "", generic) else ""
  
  return(list(
    package = package,
    generic = generic,
    generic_clean = generic_clean,
    class = class
  ))
}



#' function to get S3 method 
#' 
#' @description function to get S3 method from generic
#'  
#' @param generic - function generic
#' @param class - function class
#' @param package - package name 
#'
#' @keywords internal
get_s3_method <- function(generic, class, package) {
  func_name <- paste0(generic, ".", class)
  method <- getAnywhere(func_name)
  
  if (length(method$objs) > 0) {
    func <- method$objs[[1]]
    func_env <- environment(func)
    
    # Initialize flags
    has_r6_class <- FALSE
    has_r6_generator <- FALSE
    
    # Try to list objects in the environment
    env_objects <- tryCatch(ls(func_env), error = function(e) character(0))
    
    if (length(env_objects) > 0) {
      # Check for R6 class or generator safely
      has_r6_class <- any(sapply(env_objects, function(obj_name) {
        tryCatch({
          inherits(get(obj_name, envir = func_env), "R6")
        }, error = function(e) FALSE)
      }))
      
      has_r6_generator <- any(sapply(env_objects, function(obj_name) {
        tryCatch({
          inherits(get(obj_name, envir = func_env), "R6ClassGenerator")
        }, error = function(e) FALSE)
      }))
    }
    
    return(list(
      func = func,
      func_env = func_env,
      has_r6_class = has_r6_class,
      has_r6_generator = has_r6_generator
    ))
  } else {
    return(NULL)
  }
}


#' function to get R6 methods 
#' 
#' @description function to get R6 methods from class
#'  
#' @param class_name - function class
#' @keywords internal
get_r6_methods_details <- function(class_name) {
  methods <- class_name$public_methods
  
  # extract method details
  method_details <- lapply(names(methods), function(method_name) {
    method <- methods[[method_name]]
    
    if (!is.function(method)) {
      return(list(name = method_name, body = "<not a function>"))
    }
    
    body_text <- tryCatch(
      deparse(body(method)),
      error = function(e) paste("<error extracting body:", e$message, ">")
    )
    
    list(
      name = method_name,
      body = body_text
    )
  })
  
  return(method_details)
}

#' function to check value of ggproto
#' 
#' @description function to check if first argument inherits from any of the classes specified
#'  
#' @param x - ggproto functions from the target package
#'
#' @keywords internal
function_is_ggproto <- function(x) inherits(x, "ggproto")

#' function to check value of ggproto
#' 
#' @description function to check all classes of a ggproto function
#'  
#' @param value - ggproto functions from the target package
#'
#' @keywords internal
check_ggproto <- function(value) {
  any(class(value) %in% c("ggproto"))
}

#' function to extract ggproto methods
#' 
#' @description function to get all the methods of a ggproto function
#'  
#' @param obj - ggproto env from the target package
#'
#' @keywords internal
extract_ggproto_methods <- function(obj) {
  
  methods <- lapply(names(obj), function(method) {
    if (is.function(obj[[method]])) {
      paste(deparse(body(obj[[method]])), collapse = "\n")
    } else {
      NULL
    }
  })
  methods <- methods[!sapply(methods, is.null)]
  
  return(paste(methods, collapse = "\n\n"))
}

#' classify_function_body
#'
#' @description This function gets the function bodies for exported functions
#' 
#' @param row - row with exported functions
#' @param package_name - name of the package
#'
#' @return row - row with the function details plus function body
#' @keywords internal
classify_function_body <- function(row, package_name) {
  
  func_name <- row$exported_function
  func_class <- row$class
  func_type <- row$function_type
  func_full_name <- paste0(package_name, "::", func_name)
  
  generic <- func_name
  class <- func_class
  package <- package_name
  
  # Preprocess if not S3
  if (!(func_type == "S3 function" && !is.na(func_class))) {
    preprocessed <- preprocess_func_full_name(func_full_name)
    package <- preprocessed$package
    generic <- preprocessed$generic
    generic_clean <- preprocessed$generic_clean
    class <- preprocessed$class
  } else {
    generic_clean <- generic
  }
  
  # S4 generic or method
  if (methods::isGeneric(generic_clean)) {
    s4_info <- get_all_s4_methods(generic_clean)
    row$function_body <- paste(s4_info$bodies, collapse = "\n")
    if (s4_info$is_generic && !s4_info$has_methods) {
      row$function_type <- "S4 generic"
    } else if (s4_info$is_generic && s4_info$has_methods) {
      row$function_type <- "S4 method, S4 generic"
    }
    return(row)
  }
  
  # S7 generic
  if (!is.na(func_type) && func_type == "S7 generic") {
    fun_obj <- tryCatch(get(generic, envir = asNamespace(package_name)), error = function(e) NULL)
    if (inherits(fun_obj, "S7_generic")) {
      s7_methods <- get("methods", envir = asNamespace("S7"))(fun_obj)
      if (length(s7_methods) > 0) {
        method_bodies <- sapply(s7_methods, function(m) {
          body <- tryCatch(body(m), error = function(e) NULL)
          if (!is.null(body)) paste(deparse(body), collapse = "\n") else ""
        })
        row$function_body <- paste(method_bodies, collapse = "\n\n")
      } else {
        generic_body <- tryCatch(body(fun_obj), error = function(e) NULL)
        if (!is.null(generic_body)) {
          row$function_body <- paste(deparse(generic_body), collapse = "\n")
        }
      }
    }
    return(row)
  }
  
  # S3 method
  S3_value <- get_s3_method(generic, class, package)
  if (!is.null(S3_value)) {
    row$function_body <- paste(deparse(S3_value$func), collapse = "\n")
    if (inherits(S3_value$func, "function")) {
      func_body <- deparse(S3_value$func)
      if (any(grepl("UseMethod\\(", func_body))) {
        row$function_type <- "S3 generic function"
      } else {
        row$function_type <- "regular function"
      }
    }
    if (S3_value$has_r6_class) {
      row$function_type <- paste(row$function_type, "R6 class", sep = ", ")
    }
    if (S3_value$has_r6_generator) {
      row$function_type <- paste(row$function_type, "R6 class generator", sep = ", ")
    }
    return(row)
  }
  
  # Try to get value
  value <- tryCatch(get(generic, envir = asNamespace(package)), error = function(e) NULL)
  
  # S7 class or constructor
  if (inherits(value, "S7_class")) {
    row$function_type <- "S7 class"
    row$function_body <- "<S7 class definition>"
    return(row)
  }
  
  if (is.function(value)) {
    body_text <- paste(deparse(body(value)), collapse = "\n")
    if (grepl("S7::new\\(|new\\(", body_text) || grepl("S7::new_object\\(", body_text)) {
      row$function_type <- "S7 constructor"
      row$function_body <- body_text
      return(row)
    }
  }
  
  # Other types
  if (is.numeric(value)) {
    row$function_type <- "numeric"
    row$function_body <- as.character(value)
    return(row)
  }
  
  if (inherits(value, "rlang_fake_data_pronoun")) {
    row$function_type <- "rlang tidy eval"
    row$function_body <- "<pronoun> - refer to rlang"
    return(row)
  }
  
  if (check_ggproto(value)) {
    if (function_is_ggproto(value)) {
      func_body <- tryCatch(extract_ggproto_methods(value), error = function(e) NULL)
      row$function_body <- if (!is.null(func_body)) paste(deparse(func_body), collapse = "\n") else "Function body not found"
    }
    row$function_type <- "ggproto"
    return(row)
  }
  
  if (inherits(value, "R6ClassGenerator")) {
    row$function_type <- ifelse(is.na(row$function_type), "R6 Class Generator", paste(row$function_type, "R6 class", sep = ", "))
    row$function_body <- get_r6_methods_details(value)
    return(row)
  }
  
  # Regular function fallback
  if (is.function(value)) {
    func_body <- tryCatch(
      withCallingHandlers(
        expr = body(value),
        warning = function(w) {
          if (grepl("argument is not a function", w$message)) {
            row$function_body <- "Function type is unknown"
            invokeRestart("muffleWarning")
          }
        }
      ),
      error = function(e) {
        row$function_body <- "Function body not found"
        NULL
      }
    )
    if (!is.null(func_body)) {
      row$function_body <- paste(deparse(func_body), collapse = "\n")
    }
    if (is.na(row$function_type)) {
      row$function_type <- "regular function"
    }
  }
  
  return(row)
}

test_that("get_exports works correctly with regular functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c("myfunction"))
    expect_equal(result$class, NA_character_)
    expect_equal(result$function_type, 
                 c("regular function"))
  }
})

test_that("get_exports works correctly with no R functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0005_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    # Check it's a tibble
    expect_true(tibble::is_tibble(result))
    
    # Check number of columns
    expect_equal(ncol(result), 5)
    
    # Check column names
    expect_named(result, c("exported_function", "class", "function_type", "function_body", "where"))
    
    
    # Check number of rows
    expect_equal(nrow(result), 1)
    
  }
})


test_that("get_r6_methods_details extracts and formats R6 method bodies", {
  # Create a mock R6 class with public methods
  MockClass <- R6::R6Class("MockClass",
                           public = list(
                             method_one = function(x) {
                               x + 1
                             },
                             method_two = function(y) {
                               y * 2
                             }
                           )
  )
  
  # Call the function with the mock class
  result <- get_r6_methods_details(MockClass)
  
  # Check that the result is a character string
  expect_type(result, "list")
  
  # Check that method names and bodies are included
  # Check method one
  expect_equal(result[[1]]$name, "method_one")
  expect_true(any(grepl("x \\+ 1", result[[1]]$body)))
  
  # Check method two
  expect_equal(result[[2]]$name, "method_two")
  expect_true(any(grepl("y \\* 2", result[[2]]$body)))
  
})

test_that("get_exports works correctly with nonstandard NAMESPACE", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0012_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    # Test case: Normal scenario
    suppressWarnings(result <- get_exports(pkg_source_path))
    
    expect_equal(result$exported_function,
                 c("fetch_ggproto", "as.list.AMap", "make_proto_method",
                   "length.AMap", "AMap", "$.ggproto", ".pt"))
    expect_equal(result$class,
                 c(NA_character_, NA_character_, NA_character_, NA_character_,  
                   NA_character_, NA_character_, NA_character_))
    expect_equal(result$function_type,
                 c("regular function", "regular function, R6 class generator",
                   "regular function", "regular function, R6 class generator",
                   "non-function or helper, R6 class", 
                   "S4 method, S4 generic", "numeric"
                 ))
    
    # Check dimensions
    expect_equal(nrow(result), 7)
    expect_equal(ncol(result), 5)
    
    # Check column names
    expect_named(result, c("exported_function", "class", "function_type", "function_body", "where"))
    
    # Check types
    expect_type(result$exported_function, "character")
    expect_type(result$class, "character")
    expect_type(result$function_type, "character")
    expect_type(result$function_body, "character")
    expect_type(result$where, "character")
    
    # Check specific values (example: first row)
    expect_equal(result$exported_function[1], "fetch_ggproto")
    expect_equal(result$function_type[1], "regular function")
    expect_equal(result$where[1], "test.package.0012")
    
    # Check for NA in class column
    expect_true(is.na(result$class[1]))
    
    # Check that function_body is not empty
    expect_true(nchar(result$function_body[1]) > 0)
    
  }
})

test_that("get_exports works correctly with S3 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0009_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c("filter_first_two_rows_y", "myfunction",
                   "filter_first_three_rows", "filter_first_three_rows"))
    expect_equal(result$class, 
                 c(NA, NA, "dataframe_with_y", "list"))
    expect_equal(result$function_type, 
                 c("regular function", "regular function",
                   "S3 method", "S3 method"))
  }
})

test_that("get_exports works correctly with S4 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0010_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c("addOne"))
    expect_equal(result$class, 
                 c(NA_character_))
    expect_equal(result$function_type, 
                 c("S4 method, S4 generic"))
  }
})

test_that("get_exports works correctly with R6 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0011_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c(".pt", "\"$\"",
                   "as.list", "length", "$"))
    expect_equal(result$class, 
                 c(NA, "ggproto", "AMap", "AMap", "ggproto"))
    expect_equal(result$function_type, 
                 c("numeric", "S3 method", "S3 generic function", 
                   "S3 method", "S4 method, S4 generic"))
  }
})

test_that("S4 generics are correctly processed from exportMethods", {
  nsInfo <- list(
    exports = character(),
    S3methods = NULL,
    exportMethods = c("foo", "bar", "baz") # S4 generics
  )
  
  func_method <- character()
  class <- character()
  function_types <- character()
  
  s4_methods_raw <- nsInfo$exportMethods
  
  if (is.character(s4_methods_raw)) {
    s4_generics <- unique(s4_methods_raw)
    func_method <- c(func_method, s4_generics)
    class <- c(class, rep(NA, length(s4_generics)))
    function_types <- c(function_types, rep("S4 generic", length(s4_generics)))
  }
  
  expect_equal(func_method, c("foo", "bar", "baz"))
  expect_equal(class, rep(NA_character_, 3))
  expect_equal(function_types, rep("S4 generic", 3))
})

test_that("S4 methods are correctly processed from exportMethods", {
  nsInfo <- list(
    exports = character(),
    S3methods = NULL,
    exportMethods = list(
      c("foo", "MyClass"),
      c("bar", "YourClass"),
      c("baz", NA), # Should be filtered out
      c("qux") # Invalid (length < 2)
    )
  )
  
  func_method <- character()
  class <- character()
  function_types <- character()
  
  s4_methods_raw <- nsInfo$exportMethods
  
  if (is.list(s4_methods_raw)) {
    valid_s4_methods <- Filter(function(x) is.character(x) && length(x) >= 2 && !is.na(x[[2]]), s4_methods_raw)
    
    if (length(valid_s4_methods) > 0) {
      s4_func_names <- sapply(valid_s4_methods, `[[`, 1)
      s4_classes <- sapply(valid_s4_methods, `[[`, 2)
      
      s4_df <- unique(data.frame(func = s4_func_names, class = s4_classes, stringsAsFactors = FALSE))
      
      func_method <- c(func_method, s4_df$func)
      class <- c(class, s4_df$class)
      function_types <- c(function_types, rep("S4 function", nrow(s4_df)))
    }
  }
  
  expect_equal(func_method, c("foo", "bar"))
  expect_equal(class, c("MyClass", "YourClass"))
  expect_equal(function_types, rep("S4 function", 2))
})



test_that("classify_function_body handles regular function", {
  # Mock input row
  row <- list(
    exported_function = "my_func",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  # Stub dependencies
  mockery::stub(classify_function_body, "get", function(name, envir) {
    function(x) x + 1  # mock function
  })
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) FALSE)
  mockery::stub(classify_function_body, "get_s3_method", function(generic, class, package) NULL)
  mockery::stub(classify_function_body, "check_ggproto", function(value) FALSE)
  mockery::stub(classify_function_body, "function_is_ggproto", function(value) FALSE)
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$function_type, "regular function")
  expect_equal(result$function_body, "x + 1")
})


test_that("classify_function_body handles S3 method with R6 class generator", {
  row <- list(
    exported_function = "my_s3_func",
    class = "MyClass",
    function_type = "S3 function",
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  # Mock S3 method return
  mock_func <- function(x) x
  mock_s3 <- list(
    func = mock_func,
    has_r6_class = FALSE,
    has_r6_generator = TRUE
  )
  
  mockery::stub(classify_function_body, "get_s3_method", function(generic, class, package) mock_s3)
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) FALSE)
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, "MyClass")
  expect_equal(result$function_type, "regular function, R6 class generator")
  expect_true(grepl("function", result$function_body))
})


test_that("classify_function_body handles S7 generic with methods", {
  row <- list(
    exported_function = "my_s7_generic",
    class = NA,
    function_type = "S7 generic",
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  # Mock S7 generic object
  mock_s7_generic <- structure(list(), class = "S7_generic")
  
  # Mock S7 methods
  mock_method1 <- function(x) { x }
  mock_method2 <- function(x) { x * 2 }
  
  # Stub get to return the S7 generic object
  mockery::stub(classify_function_body, "get", function(name, envir) mock_s7_generic)
  
  # Stub S7::methods to return a list of mock methods
  mockery::stub(classify_function_body, "get", function(name, envir) {
    if (name == "methods" && identical(envir, asNamespace("S7"))) {
      function(obj) list(mock_method1, mock_method2)
    } else {
      mock_s7_generic
    }
  })
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "S7 generic")
  expect_true(grepl("x \\* 2", result$function_body))
})


test_that("classify_function_body handles S7 constructor", {
  row <- list(
    exported_function = "new_s7",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mock_constructor <- function() {
    S7::new("MyClass")
  }
  
  mockery::stub(classify_function_body, "get", function(name, envir) mock_constructor)
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) FALSE)
  mockery::stub(classify_function_body, "get_s3_method", function(generic, class, package) NULL)
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "S7 constructor")
  expect_true(grepl("S7::new", result$function_body))
})


test_that("classify_function_body handles S4 generic and method", {
  row <- list(
    exported_function = "my_s4_func",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) TRUE)
  mockery::stub(classify_function_body, "get_all_s4_methods", function(name) {
    list(
      is_generic = TRUE,
      has_methods = TRUE,
      bodies = c("method1_body", "method2_body")
    )
  })
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "S4 method, S4 generic")
  expect_true(grepl("method1_body", result$function_body))
})


test_that("classify_function_body handles ggproto object", {
  row <- list(
    exported_function = "my_ggproto",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mockery::stub(classify_function_body, "get", function(name, envir) "ggproto_object")
  mockery::stub(classify_function_body, "check_ggproto", function(value) TRUE)
  mockery::stub(classify_function_body, "function_is_ggproto", function(value) TRUE)
  mockery::stub(classify_function_body, "extract_ggproto_methods", function(value) {
    quote({ print("ggproto method") })
  })
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "ggproto")
  expect_true(grepl("ggproto method", result$function_body))
})


test_that("classify_function_body handles R6 class generator", {
  row <- list(
    exported_function = "my_r6_class",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mock_r6 <- structure(list(), class = "R6ClassGenerator")
  
  mockery::stub(classify_function_body, "get", function(name, envir) mock_r6)
  mockery::stub(classify_function_body, "check_ggproto", function(value) FALSE)
  mockery::stub(classify_function_body, "get_r6_methods_details", function(value) "R6 method details")
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "R6 Class Generator")
  expect_equal(result$function_body, "R6 method details")
})

# Define a mock ggproto object
mock_ggproto <- list(
  method1 = function() { print("Method 1") },
  method2 = function() { print("Method 2") },
  non_function = "Not a function"
)

# Test the extract_ggproto_methods function
test_that("extract_ggproto_methods extracts methods correctly", {
  # Mock the ggproto object
  mock_obj <- mock_ggproto
  
  # Call the function with the mock object
  result <- extract_ggproto_methods(mock_obj)
  
  # Expected result
  expected_result <- paste(
    paste(deparse(body(mock_obj$method1)), collapse = "\n"),
    paste(deparse(body(mock_obj$method2)), collapse = "\n"),
    sep = "\n\n"
  )
  
  # Check if the result matches the expected result
  expect_equal(result, expected_result)
})


test_that("preprocess_func_full_name works for usual S3 functions", {
  func_full_name <- "dplyr::summarise.grouped_df"
  result <- preprocess_func_full_name(func_full_name)
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "summarise")
  expect_equal(result$class, "grouped_df")
})


test_that("preprocess_func_full_name works for specific S3 functions with :::", {
  result <- preprocess_func_full_name("dplyr::filter_bullets.dplyr:::filter_incompatible_size")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "filter_bullets")
  expect_equal(result$class, "dplyr:::filter_incompatible_size")
})

test_that("preprocess_func_full_name works for specific S3 functions with multiple :::", {
  result <- preprocess_func_full_name("dplyr::mutate_bullets.dplyr:::mutate_constant_recycle_error")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "mutate_bullets")
  expect_equal(result$class, "dplyr:::mutate_constant_recycle_error")
})

test_that("preprocess_func_full_name works for usual S3 functions without :::", {
  result <- preprocess_func_full_name("dplyr::inner_join::data.frame")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "inner_join")
  expect_equal(result$class, "")
})

test_that("preprocess_func_full_name works for functions without namespace qualifier", {
  func_full_name <- "mean.default"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_null(result$generic) # Because generic is not set in this branch
  expect_equal(result$generic_clean, "") # generic is NULL, so clean should be ""
  expect_null(result$class) # class is not set in this branch
})


test_that("preprocess_func_full_name wraps infix operators in backticks if not already wrapped", {
  func_full_name <- "%/%"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "`%/%`")
  expect_equal(result$generic_clean, "%/%")
  expect_equal(result$class, "")
})

test_that("preprocess_func_full_name keeps already backtick-wrapped infix operators unchanged", {
  func_full_name <- "`%in%`"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "`%in%`")
  expect_equal(result$generic_clean, "%in%")
  expect_equal(result$class, "")
})


test_that("preprocess_func_full_name handles double-dot forms like ..1", {
  func_full_name <- "..1"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "..1")
  expect_equal(result$generic_clean, "..1")
  expect_equal(result$class, "..1")
})

test_that("preprocess_func_full_name handles exactly ..", {
  func_full_name <- ".."
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "..")
  expect_equal(result$generic_clean, "..")
  expect_equal(result$class, "..")
})

test_that("generic is adjusted correctly if it contains '::'", {
  result <- preprocess_func_full_name("dplyr::filter_bullets.dplyr:::filter_incompatible_size")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "filter_bullets")  # Test for adjusted generic
  expect_equal(result$class, "dplyr:::filter_incompatible_size")
})


test_that("get_exports returns fallback tibble when R folder is missing", {
  # Mock package path
  pkg_source_path <- "/fake/path/to/package"
  package_name <- "testpkg"
  
  # Stub contains_r_folder to return FALSE
  mockery::stub(get_exports, "contains_r_folder", function(path) FALSE)
  
  # Stub extract_package_name to return mocked package name
  mockery::stub(get_exports, "extract_package_name", function(path) package_name)
  
  # Run the function
  result <- get_exports(pkg_source_path)
  
  # Expected package name from path
  expected_pkg_name <- basename(pkg_source_path)
  
  # Check structure and values
  expect_true(tibble::is_tibble(result))
  expect_equal(nrow(result), 1)
  expect_equal(result$exported_function, NA)
  expect_equal(result$class, NA)
  expect_equal(result$function_type, NA)
  expect_equal(result$function_body, "No R folder found in the package source path")
  
})

test_that("extract_exported_function_info builds initial result tibble with mocked namespace info", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  # Mock namespace info
  nsInfo <- list(
    exports = c("fun1", "fun2"),
    S3methods = matrix(c("print", "summary", "MyClass", "MyClass"), ncol = 2),
    exportMethods = list(c("show", "MyClass"), c("plot", "MyClass"))
  )
  
  # Stub parseNamespaceFile to return mocked nsInfo
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(pkg, lib, mustExist) nsInfo)
  
  # Stub getExportedValue to return mock objects
  mockery::stub(extract_exported_function_info, "getExportedValue", function(pkg, name) {
    if (name == "fun1") {
      structure(list(), class = "S7_generic")
    } else if (name == "fun2") {
      function(x) x  # regular function
    } else {
      NULL
    }
  })
  
  # Stub getNamespaceExports to return fallback exports (not used here)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(pkg) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_true(tibble::is_tibble(result))
  expect_true(all(c("exported_function", "class", "function_type", "function_body", "where") %in% names(result)))
  expect_equal(result$where, rep(package_name, nrow(result)))
  expect_true("fun1" %in% result$exported_function)
  expect_true("fun2" %in% result$exported_function)
  expect_true("print" %in% result$exported_function)
  expect_true("show" %in% result$exported_function)
  expect_true("S7 generic" %in% result$function_type)
  expect_true("S3 method" %in% result$function_type)
  expect_true("S4 function" %in% result$function_type)
  expect_true("regular function" %in% result$function_type)
})


test_that("returns NULL when getExportedValue returns NULL", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) NULL)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(nrow(result), 1)
})


test_that("detects S7 class", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) structure(list(), class = "S7_class"))
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(result$function_type, "S7 class")
})


test_that("detects S7 constructor from function body", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mock_fun <- function(x) {
    S7::new("MyClass")
  }
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) mock_fun)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(result$function_type, "S7 constructor")
})


test_that("handles NA in class column", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) structure(list(), class = "S7_generic"))
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_true(is.na(result$class[1]))
})


test_that("detects S4 generics from exportMethods", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(
    exports = character(0),
    exportMethods = c("show", "plot")
  )
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(sort(result$function_type), c("S4 generic", "S4 generic"))
  expect_true(all(is.na(result$class)))
})
test_that("get_exports works correctly with regular functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c("myfunction"))
    expect_equal(result$class, NA_character_)
    expect_equal(result$function_type, 
                 c("regular function"))
  }
})

test_that("get_exports works correctly with no R functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0005_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    # Check it's a tibble
    expect_true(tibble::is_tibble(result))
    
    # Check number of columns
    expect_equal(ncol(result), 5)
    
    # Check column names
    expect_named(result, c("exported_function", "class", "function_type", "function_body", "where"))
    
    
    # Check number of rows
    expect_equal(nrow(result), 1)
    
  }
})


test_that("get_r6_methods_details extracts and formats R6 method bodies", {
  # Create a mock R6 class with public methods
  MockClass <- R6::R6Class("MockClass",
                           public = list(
                             method_one = function(x) {
                               x + 1
                             },
                             method_two = function(y) {
                               y * 2
                             }
                           )
  )
  
  # Call the function with the mock class
  result <- get_r6_methods_details(MockClass)
  
  # Check that the result is a character string
  expect_type(result, "list")
  
  # Check that method names and bodies are included
  # Check method one
  expect_equal(result[[1]]$name, "method_one")
  expect_true(any(grepl("x \\+ 1", result[[1]]$body)))
  
  # Check method two
  expect_equal(result[[2]]$name, "method_two")
  expect_true(any(grepl("y \\* 2", result[[2]]$body)))
  
})

test_that("get_exports works correctly with nonstandard NAMESPACE", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0012_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    # Test case: Normal scenario
    suppressWarnings(result <- get_exports(pkg_source_path))
    
    expect_equal(result$exported_function,
                 c("fetch_ggproto", "as.list.AMap", "make_proto_method",
                   "length.AMap", "AMap", "$.ggproto", ".pt"))
    expect_equal(result$class,
                 c(NA_character_, NA_character_, NA_character_, NA_character_,  
                   NA_character_, NA_character_, NA_character_))
    expect_equal(result$function_type,
                 c("regular function", "regular function, R6 class generator",
                   "regular function", "regular function, R6 class generator",
                   "non-function or helper, R6 class", 
                   "S4 method, S4 generic", "numeric"
                 ))
    
    # Check dimensions
    expect_equal(nrow(result), 7)
    expect_equal(ncol(result), 5)
    
    # Check column names
    expect_named(result, c("exported_function", "class", "function_type", "function_body", "where"))
    
    # Check types
    expect_type(result$exported_function, "character")
    expect_type(result$class, "character")
    expect_type(result$function_type, "character")
    expect_type(result$function_body, "character")
    expect_type(result$where, "character")
    
    # Check specific values (example: first row)
    expect_equal(result$exported_function[1], "fetch_ggproto")
    expect_equal(result$function_type[1], "regular function")
    expect_equal(result$where[1], "test.package.0012")
    
    # Check for NA in class column
    expect_true(is.na(result$class[1]))
    
    # Check that function_body is not empty
    expect_true(nchar(result$function_body[1]) > 0)
    
  }
})

test_that("get_exports works correctly with S3 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0009_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c("filter_first_two_rows_y", "myfunction",
                   "filter_first_three_rows", "filter_first_three_rows"))
    expect_equal(result$class, 
                 c(NA, NA, "dataframe_with_y", "list"))
    expect_equal(result$function_type, 
                 c("regular function", "regular function",
                   "S3 method", "S3 method"))
  }
})

test_that("get_exports works correctly with S4 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0010_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c("addOne"))
    expect_equal(result$class, 
                 c(NA_character_))
    expect_equal(result$function_type, 
                 c("S4 method, S4 generic"))
  }
})

test_that("get_exports works correctly with R6 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0011_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    # Test case: Normal scenario
    result <- get_exports(pkg_source_path)
    
    expect_equal(result$exported_function, 
                 c(".pt", "\"$\"",
                   "as.list", "length", "$"))
    expect_equal(result$class, 
                 c(NA, "ggproto", "AMap", "AMap", "ggproto"))
    expect_equal(result$function_type, 
                 c("numeric", "S3 method", "S3 generic function", 
                   "S3 method", "S4 method, S4 generic"))
  }
})

test_that("S4 generics are correctly processed from exportMethods", {
  nsInfo <- list(
    exports = character(),
    S3methods = NULL,
    exportMethods = c("foo", "bar", "baz") # S4 generics
  )
  
  func_method <- character()
  class <- character()
  function_types <- character()
  
  s4_methods_raw <- nsInfo$exportMethods
  
  if (is.character(s4_methods_raw)) {
    s4_generics <- unique(s4_methods_raw)
    func_method <- c(func_method, s4_generics)
    class <- c(class, rep(NA, length(s4_generics)))
    function_types <- c(function_types, rep("S4 generic", length(s4_generics)))
  }
  
  expect_equal(func_method, c("foo", "bar", "baz"))
  expect_equal(class, rep(NA_character_, 3))
  expect_equal(function_types, rep("S4 generic", 3))
})

test_that("S4 methods are correctly processed from exportMethods", {
  nsInfo <- list(
    exports = character(),
    S3methods = NULL,
    exportMethods = list(
      c("foo", "MyClass"),
      c("bar", "YourClass"),
      c("baz", NA), # Should be filtered out
      c("qux") # Invalid (length < 2)
    )
  )
  
  func_method <- character()
  class <- character()
  function_types <- character()
  
  s4_methods_raw <- nsInfo$exportMethods
  
  if (is.list(s4_methods_raw)) {
    valid_s4_methods <- Filter(function(x) is.character(x) && length(x) >= 2 && !is.na(x[[2]]), s4_methods_raw)
    
    if (length(valid_s4_methods) > 0) {
      s4_func_names <- sapply(valid_s4_methods, `[[`, 1)
      s4_classes <- sapply(valid_s4_methods, `[[`, 2)
      
      s4_df <- unique(data.frame(func = s4_func_names, class = s4_classes, stringsAsFactors = FALSE))
      
      func_method <- c(func_method, s4_df$func)
      class <- c(class, s4_df$class)
      function_types <- c(function_types, rep("S4 function", nrow(s4_df)))
    }
  }
  
  expect_equal(func_method, c("foo", "bar"))
  expect_equal(class, c("MyClass", "YourClass"))
  expect_equal(function_types, rep("S4 function", 2))
})



test_that("classify_function_body handles regular function", {
  # Mock input row
  row <- list(
    exported_function = "my_func",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  # Stub dependencies
  mockery::stub(classify_function_body, "get", function(name, envir) {
    function(x) x + 1  # mock function
  })
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) FALSE)
  mockery::stub(classify_function_body, "get_s3_method", function(generic, class, package) NULL)
  mockery::stub(classify_function_body, "check_ggproto", function(value) FALSE)
  mockery::stub(classify_function_body, "function_is_ggproto", function(value) FALSE)
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$function_type, "regular function")
  expect_equal(result$function_body, "x + 1")
})


test_that("classify_function_body handles S3 method with R6 class generator", {
  row <- list(
    exported_function = "my_s3_func",
    class = "MyClass",
    function_type = "S3 function",
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  # Mock S3 method return
  mock_func <- function(x) x
  mock_s3 <- list(
    func = mock_func,
    has_r6_class = FALSE,
    has_r6_generator = TRUE
  )
  
  mockery::stub(classify_function_body, "get_s3_method", function(generic, class, package) mock_s3)
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) FALSE)
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, "MyClass")
  expect_equal(result$function_type, "regular function, R6 class generator")
  expect_true(grepl("function", result$function_body))
})


test_that("classify_function_body handles S7 generic with methods", {
  row <- list(
    exported_function = "my_s7_generic",
    class = NA,
    function_type = "S7 generic",
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  # Mock S7 generic object
  mock_s7_generic <- structure(list(), class = "S7_generic")
  
  # Mock S7 methods
  mock_method1 <- function(x) { x }
  mock_method2 <- function(x) { x * 2 }
  
  # Stub get to return the S7 generic object
  mockery::stub(classify_function_body, "get", function(name, envir) mock_s7_generic)
  
  # Stub S7::methods to return a list of mock methods
  mockery::stub(classify_function_body, "get", function(name, envir) {
    if (name == "methods" && identical(envir, asNamespace("S7"))) {
      function(obj) list(mock_method1, mock_method2)
    } else {
      mock_s7_generic
    }
  })
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "S7 generic")
  expect_true(grepl("x \\* 2", result$function_body))
})


test_that("classify_function_body handles S7 constructor", {
  row <- list(
    exported_function = "new_s7",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mock_constructor <- function() {
    S7::new("MyClass")
  }
  
  mockery::stub(classify_function_body, "get", function(name, envir) mock_constructor)
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) FALSE)
  mockery::stub(classify_function_body, "get_s3_method", function(generic, class, package) NULL)
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "S7 constructor")
  expect_true(grepl("S7::new", result$function_body))
})


test_that("classify_function_body handles S4 generic and method", {
  row <- list(
    exported_function = "my_s4_func",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mockery::stub(classify_function_body, "methods::isGeneric", function(name) TRUE)
  mockery::stub(classify_function_body, "get_all_s4_methods", function(name) {
    list(
      is_generic = TRUE,
      has_methods = TRUE,
      bodies = c("method1_body", "method2_body")
    )
  })
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "S4 method, S4 generic")
  expect_true(grepl("method1_body", result$function_body))
})


test_that("classify_function_body handles ggproto object", {
  row <- list(
    exported_function = "my_ggproto",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mockery::stub(classify_function_body, "get", function(name, envir) "ggproto_object")
  mockery::stub(classify_function_body, "check_ggproto", function(value) TRUE)
  mockery::stub(classify_function_body, "function_is_ggproto", function(value) TRUE)
  mockery::stub(classify_function_body, "extract_ggproto_methods", function(value) {
    quote({ print("ggproto method") })
  })
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "ggproto")
  expect_true(grepl("ggproto method", result$function_body))
})


test_that("classify_function_body handles R6 class generator", {
  row <- list(
    exported_function = "my_r6_class",
    class = NA,
    function_type = NA,
    function_body = NA,
    where = "test.package"
  )
  package_name <- "test.package"
  
  mock_r6 <- structure(list(), class = "R6ClassGenerator")
  
  mockery::stub(classify_function_body, "get", function(name, envir) mock_r6)
  mockery::stub(classify_function_body, "check_ggproto", function(value) FALSE)
  mockery::stub(classify_function_body, "get_r6_methods_details", function(value) "R6 method details")
  
  result <- classify_function_body(row, package_name)
  
  expect_equal(result$class, NA)
  expect_equal(result$function_type, "R6 Class Generator")
  expect_equal(result$function_body, "R6 method details")
})

# Define a mock ggproto object
mock_ggproto <- list(
  method1 = function() { print("Method 1") },
  method2 = function() { print("Method 2") },
  non_function = "Not a function"
)

# Test the extract_ggproto_methods function
test_that("extract_ggproto_methods extracts methods correctly", {
  # Mock the ggproto object
  mock_obj <- mock_ggproto
  
  # Call the function with the mock object
  result <- extract_ggproto_methods(mock_obj)
  
  # Expected result
  expected_result <- paste(
    paste(deparse(body(mock_obj$method1)), collapse = "\n"),
    paste(deparse(body(mock_obj$method2)), collapse = "\n"),
    sep = "\n\n"
  )
  
  # Check if the result matches the expected result
  expect_equal(result, expected_result)
})

test_that("preprocess_func_full_name works for specific S3 functions with :::", {
  result <- preprocess_func_full_name("dplyr::filter_bullets.dplyr:::filter_incompatible_size")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "filter_bullets")
  expect_equal(result$class, "dplyr:::filter_incompatible_size")
})

test_that("preprocess_func_full_name works for specific S3 functions with multiple :::", {
  result <- preprocess_func_full_name("dplyr::mutate_bullets.dplyr:::mutate_constant_recycle_error")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "mutate_bullets")
  expect_equal(result$class, "dplyr:::mutate_constant_recycle_error")
})

test_that("preprocess_func_full_name works for usual S3 functions without :::", {
  result <- preprocess_func_full_name("dplyr::inner_join::data.frame")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "inner_join")
  expect_equal(result$class, "")
})


test_that("preprocess_func_full_name works for function name without class suffix", {
  result <- preprocess_func_full_name("dplyr::mutate")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "mutate")
  expect_equal(result$class, "")
})



test_that("preprocess_func_full_name works for functions without namespace qualifier", {
  func_full_name <- "mean.default"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_null(result$generic) # Because generic is not set in this branch
  expect_equal(result$generic_clean, "") # generic is NULL, so clean should be ""
  expect_null(result$class) # class is not set in this branch
})


test_that("preprocess_func_full_name wraps infix operators in backticks if not already wrapped", {
  func_full_name <- "%/%"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "`%/%`")
  expect_equal(result$generic_clean, "%/%")
  expect_equal(result$class, "")
})

test_that("preprocess_func_full_name keeps already backtick-wrapped infix operators unchanged", {
  func_full_name <- "`%in%`"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "`%in%`")
  expect_equal(result$generic_clean, "%in%")
  expect_equal(result$class, "")
})


test_that("preprocess_func_full_name handles double-dot forms like ..1", {
  func_full_name <- "..1"
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "..1")
  expect_equal(result$generic_clean, "..1")
  expect_equal(result$class, "..1")
})

test_that("preprocess_func_full_name handles exactly ..", {
  func_full_name <- ".."
  result <- preprocess_func_full_name(func_full_name)
  expect_null(result$package)
  expect_equal(result$generic, "..")
  expect_equal(result$generic_clean, "..")
  expect_equal(result$class, "..")
})

test_that("generic is adjusted correctly if it contains '::'", {
  result <- preprocess_func_full_name("dplyr::filter_bullets.dplyr:::filter_incompatible_size")
  expect_equal(result$package, "dplyr")
  expect_equal(result$generic, "filter_bullets")  # Test for adjusted generic
  expect_equal(result$class, "dplyr:::filter_incompatible_size")
})


test_that("get_exports returns fallback tibble when R folder is missing", {
  # Mock package path
  pkg_source_path <- "/fake/path/to/package"
  package_name <- "testpkg"
  
  # Stub contains_r_folder to return FALSE
  mockery::stub(get_exports, "contains_r_folder", function(path) FALSE)
  
  # Stub extract_package_name to return mocked package name
  mockery::stub(get_exports, "extract_package_name", function(path) package_name)
  
  # Run the function
  result <- get_exports(pkg_source_path)
  
  # Expected package name from path
  expected_pkg_name <- basename(pkg_source_path)
  
  # Check structure and values
  expect_true(tibble::is_tibble(result))
  expect_equal(nrow(result), 1)
  expect_equal(result$exported_function, NA)
  expect_equal(result$class, NA)
  expect_equal(result$function_type, NA)
  expect_equal(result$function_body, "No R folder found in the package source path")
  
})

test_that("extract_exported_function_info builds initial result tibble with mocked namespace info", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  # Mock namespace info
  nsInfo <- list(
    exports = c("fun1", "fun2"),
    S3methods = matrix(c("print", "summary", "MyClass", "MyClass"), ncol = 2),
    exportMethods = list(c("show", "MyClass"), c("plot", "MyClass"))
  )
  
  # Stub parseNamespaceFile to return mocked nsInfo
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(pkg, lib, mustExist) nsInfo)
  
  # Stub getExportedValue to return mock objects
  mockery::stub(extract_exported_function_info, "getExportedValue", function(pkg, name) {
    if (name == "fun1") {
      structure(list(), class = "S7_generic")
    } else if (name == "fun2") {
      function(x) x  # regular function
    } else {
      NULL
    }
  })
  
  # Stub getNamespaceExports to return fallback exports (not used here)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(pkg) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_true(tibble::is_tibble(result))
  expect_true(all(c("exported_function", "class", "function_type", "function_body", "where") %in% names(result)))
  expect_equal(result$where, rep(package_name, nrow(result)))
  expect_true("fun1" %in% result$exported_function)
  expect_true("fun2" %in% result$exported_function)
  expect_true("print" %in% result$exported_function)
  expect_true("show" %in% result$exported_function)
  expect_true("S7 generic" %in% result$function_type)
  expect_true("S3 method" %in% result$function_type)
  expect_true("S4 function" %in% result$function_type)
  expect_true("regular function" %in% result$function_type)
})


test_that("returns NULL when getExportedValue returns NULL", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) NULL)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(nrow(result), 1)
})


test_that("detects S7 class", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) structure(list(), class = "S7_class"))
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(result$function_type, "S7 class")
})


test_that("detects S7 constructor from function body", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mock_fun <- function(x) {
    S7::new("MyClass")
  }
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) mock_fun)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(result$function_type, "S7 constructor")
})


test_that("handles NA in class column", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(exports = c("fun1"))
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getExportedValue", function(...) structure(list(), class = "S7_generic"))
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_true(is.na(result$class[1]))
})


test_that("detects S4 generics from exportMethods", {
  pkg_source_path <- "/fake/path/testpkg"
  package_name <- "testpkg"
  
  nsInfo <- list(
    exports = character(0),
    exportMethods = c("show", "plot")
  )
  
  mockery::stub(extract_exported_function_info, "parseNamespaceFile", function(...) nsInfo)
  mockery::stub(extract_exported_function_info, "getNamespaceExports", function(...) character(0))
  
  result <- extract_exported_function_info(pkg_source_path, package_name)
  
  expect_equal(sort(result$function_type), c("S4 generic", "S4 generic"))
  expect_true(all(is.na(result$class)))
})

test_that("capture_cran_warning handles timeout warning", {
  # Stub url() to trigger a warning
  mockery::stub(capture_cran_warning, "url", function(...) {
    warning("URL 'http://lib.stat.cmu.edu/R/CRAN/src/contrib/Meta/archive.rds': Timeout of 60 seconds was reached")
    return(NULL)
  })
  
  result <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  expect_equal(result$status, "warning")
  expect_match(result$message, "Timeout of 60 seconds was reached")
})


test_that("capture_cran_warning handles URL error", {
  # Stub url() to trigger an error
  mockery::stub(capture_cran_warning, "url", function(...) {
    stop("cannot open URL 'http://lib.stat.cmu.edu/R/CRAN/src/contrib/Meta/archive.rds'")
  })
  
  result <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  expect_equal(result$status, "url_error")
  expect_match(result$message, "cannot open URL")
})


test_that("capture_cran_warning returns timeout status for 504 Gateway Timeout", {
  mockery::stub(capture_cran_warning, "url", function(...) {
    stop("HTTP status was '504 Gateway Timeout'")
  })
  
  result <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  expect_equal(result$status, "timeout")
  expect_match(result$message, "504 Gateway Timeout")
})

test_that("capture_cran_warning handles successful connection", {
  # Stub url() and readBin() to simulate success
  mockery::stub(capture_cran_warning, "url", function(...) rawConnection(raw(0)))
  mockery::stub(capture_cran_warning, "readBin", function(...) raw(0))
  
  result <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  expect_equal(result$status, "success")
  expect_null(result$message)
})


test_that("capture_cran_warning returns generic error status for unknown error", {
  mockery::stub(capture_cran_warning, "url", function(...) {
    stop("Some unexpected error occurred")
  })
  
  result <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  expect_equal(result$status, "error")
  expect_match(result$message, "unexpected error")
})

#' Assess an R Package traceability matrix from package name and version
#'
#' This function use `risk.assessr::create_traceability_matrix` function with only the package name and version
#'
#' @param package_name A character string specifying the name of the package to assess.
#' @param version A character string specifying the version of the package to assess. Default is `NA`, which assesses the latest version.
#' @param repos A character string specifying the repo directly. Default is getOption("repos")
#' @param execute_coverage Logical (`TRUE`/`FALSE`). If `TRUE`, execute test coverage.
#' 
#'
#' @return The function returns package traceability_matrix
#' If the package cannot be downloaded or installed, an error message is returned.
#'
#'
#' @examples
#' \dontrun{
#' 
#' results_no_test_covr <- generate_traceability_matrix(
#'  "here", 
#'  version = "1.0.1", 
#'  execute_coverage = FALSE
#' )
#' 
#' results_test_covr <- generate_traceability_matrix(
#'  "here", 
#'  version = "1.0.1", 
#'  execute_coverage = TRUE
#' )
#' 
#' 
#' print(results_no_test_covr)
#' 
#' print(results_test_covr)
#' }
#' 
#' @importFrom remotes download_version
#' @export
generate_traceability_matrix <- function(package_name, version = NA, repos = getOption("repos"), execute_coverage = FALSE) {

  temp_file <- get_package_tarfile(package_name = package_name, version = version, repos = repos)

  if (is.null(temp_file)) {
    return(NULL)
  }
  
  # Prepare source & installation
  modified_tar_file <- modify_description_file(temp_file)
  install_list <- set_up_pkg(modified_tar_file)
  
  pkg_source_path <- install_list$pkg_source_path
  
  # Install locally
  package_installed <- install_package_local(pkg_source_path)
  
  # Optional coverage
  if (execute_coverage) {
    test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
    if (isTRUE(test_pkg_data$has_testthat) || isTRUE(test_pkg_data$has_testit)) {
      covr_timeout <- Inf
      covr_list <- run_coverage(pkg_source_path, covr_timeout)
    } else {
      message("No testthat or testit configuration")
      covr_list <- NULL
    }
  } else {
    covr_list <- NULL
  }
  
  # Build traceability matrix or fail
  if (isTRUE(package_installed)) {
    traceability_matrix <- create_traceability_matrix(
      pkg_name = package_name,
      pkg_source_path = pkg_source_path,
      func_covr = if (is.null(covr_list)) NULL else covr_list$res_cov,
      execute_coverage = execute_coverage
    )
  } else {
    message("Package installation failed.")
    traceability_matrix <- NULL
  }
  
  # Clean up temp tarball
  unlink(temp_file)
  
  return(traceability_matrix)
}

test_that("execute_coverage=FALSE runs coverage when tests are present", {
  
  # Safe stub: copy the fixture to a temp file
  fake_dl <- function(...) {
    tmp <- tempfile(fileext = ".tar.gz")
    file.copy(
      system.file("test-data", "stringr-1.5.1.tar.gz", 
                  package = "risk.assessr"),
      tmp
    )
    tmp
  }
  
  fake_setup <- function(x) list(package_installed = TRUE, pkg_source_path = tempdir())
  
  mockery::stub(generate_traceability_matrix, "check_cran_package", TRUE)
  mockery::stub(generate_traceability_matrix, "remotes::download_version", fake_dl)
  
  mockery::stub(generate_traceability_matrix, "modify_description_file", function(x) x)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  
  # These should not be consulted for coverage when execute_coverage = FALSE,
  # but we can keep them harmless.
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(path) list(has_testthat=TRUE, has_testit=FALSE))
  mockery::stub(generate_traceability_matrix, "run_coverage", function(...) stop("should not run when execute_coverage=FALSE"))
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) list())
  
  out <- generate_traceability_matrix("stringr", "1.5.1", execute_coverage = FALSE)
  expect_equal(out, list())
})

test_that("‘Bioconductor fallback’ case works once a tarball is provided", {
  # We don't test fallback logic here; just that the pipeline works with a tarball.
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(path) list(has_testthat=FALSE, has_testit=FALSE))
  mockery::stub(generate_traceability_matrix, "run_coverage", function(...) stop("should not run without tests"))
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) "fallback-success")
  
  result <- generate_traceability_matrix("SomePkg")
  expect_equal(result, "fallback-success")
})

test_that("‘Bioconductor direct’ case works once a tarball is provided", {
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(path) list(has_testthat=FALSE, has_testit=FALSE))
  mockery::stub(generate_traceability_matrix, "run_coverage", function(...) stop("should not run without tests"))
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) "bioc-success")
  
  result <- generate_traceability_matrix("Biobase")
  expect_equal(result, "bioc-success")
})

test_that("‘internal mirror’ case works once a tarball is provided", {
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(path) list(has_testthat=FALSE, has_testit=FALSE))
  mockery::stub(generate_traceability_matrix, "run_coverage", function(...) stop("should not run without tests"))
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) "internal-success")
  
  result <- generate_traceability_matrix("InternalPkg")
  expect_equal(result, "internal-success")
})

test_that("returns NULL when get_package_tarfile() returns NULL", {
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) NULL)
  
  # Ensure we don't proceed past early return
  called_mod <- FALSE
  mockery::stub(generate_traceability_matrix, "modify_description_file", function(x) { called_mod <<- TRUE; x })
  
  out <- generate_traceability_matrix("GhostPkg")
  expect_null(out)
  expect_false(called_mod)
})

test_that("‘CRAN success’ case works once a tarball is provided", {
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(path) list(has_testthat=FALSE, has_testit=FALSE))
  mockery::stub(generate_traceability_matrix, "run_coverage", function(...) stop("should not run without tests"))
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) "cran-success")
  
  result <- generate_traceability_matrix("ggplot2")
  expect_equal(result, "cran-success")
})

test_that("passes version and repos to get_package_tarfile()", {
  tar_path <- fake_tar()
  seen <- new.env(parent = emptyenv())
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(package_name, version = NA, repos = NULL) {
    seen$package_name <- package_name
    seen$version <- version
    seen$repos <- repos
    tar_path
  })
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) "ok")
  
  custom_repos <- c(CRAN = "https://my.custom.repo")
  res <- generate_traceability_matrix("pkgpass", version = "9.9.9", repos = custom_repos, execute_coverage = FALSE)
  expect_equal(res, "ok")
  expect_equal(seen$package_name, "pkgpass")
  expect_equal(seen$version, "9.9.9")
  expect_equal(seen$repos, custom_repos)
})

test_that("Coverage logic runs when tests are present", {
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = "mock_path")
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(pkg_source_path) {
    list(has_testthat = TRUE, has_testit = FALSE)
  })
  mockery::stub(generate_traceability_matrix, "run_coverage", function(pkg_source_path, covr_timeout) {
    list(res_cov = "mock_cov_results")
  })
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(pkg_name, pkg_source_path, func_covr, execute_coverage) {
    list(traceability_matrix = "mock_matrix", coverage = func_covr)
  })
  
  result <- generate_traceability_matrix("mockpkg", execute_coverage = TRUE)
  expect_equal(result$coverage, "mock_cov_results")
})

test_that("Coverage logic skips when no tests are present", {
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = "mock_path")
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(pkg_source_path) {
    list(has_testthat = FALSE, has_testit = FALSE)
  })
  mockery::stub(generate_traceability_matrix, "run_coverage", function(pkg_source_path, covr_timeout) {
    stop("Should not be called")
  })
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(pkg_name, pkg_source_path, func_covr, execute_coverage) {
    list(traceability_matrix = "mock_matrix", coverage = func_covr)
  })
  
  result <- generate_traceability_matrix("mockpkg", execute_coverage = TRUE)
  expect_null(result$coverage)
})

test_that("Coverage logic does not run when execute_coverage is FALSE", {
  tar_path <- fake_tar()
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = "mock_path")
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  
  mockery::stub(generate_traceability_matrix, "check_pkg_tests_and_snaps", function(pkg_source_path) {
    stop("Should not be called")
  })
  mockery::stub(generate_traceability_matrix, "run_coverage", function(pkg_source_path, covr_timeout) {
    stop("Should not be called")
  })
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(pkg_name, pkg_source_path, func_covr, execute_coverage) {
    list(traceability_matrix = "mock_matrix", coverage = func_covr)
  })
  
  result <- generate_traceability_matrix("mockpkg", execute_coverage = FALSE)
  expect_null(result$coverage)
})

test_that("cleans up temp tarball after run", {
  tar_path <- fake_tar()
  expect_true(file.exists(tar_path))
  
  mockery::stub(generate_traceability_matrix, "get_package_tarfile", function(...) tar_path)
  mockery::stub(generate_traceability_matrix, "modify_description_file", identity)
  mockery::stub(generate_traceability_matrix, "set_up_pkg", function(x) {
    list(package_installed = TRUE, pkg_source_path = tempdir())
  })
  mockery::stub(generate_traceability_matrix, "install_package_local", TRUE)
  mockery::stub(generate_traceability_matrix, "create_traceability_matrix", function(...) "ok-cleanup")
  
  res <- generate_traceability_matrix("cleanpkg", execute_coverage = FALSE)
  expect_equal(res, "ok-cleanup")
  expect_false(file.exists(tar_path))
})

#' Run Coverage Analysis with Test Detection
#'
#' This function checks the test configuration of an R package and runs code coverage analysis
#' using `testthat`, `testit` or `test base` frameworks if available. If no supported test framework is found,
#' it returns a default coverage result indicating no functions were tested.
#'
#' @param pkg_source_path Character. Path to the root directory of the R package source.
#' @param covr_timeout Numeric. Timeout in seconds for running coverage analysis. Default is 60.
#'
#' @return A list with two elements:
#' \describe{
#'   \item{covr_list}{A list containing coverage results. If no supported tests are found, returns a default structure with zero coverage.}
#'   \item{test_pkg_data}{A list summarizing the test configuration, including presence of `testthat`, `testit`, snapshots, and base R test scripts.}
#' }
#'
#' @examples
#' \dontrun{
#' r = getOption("repos")
#' r["CRAN"] = "http://cran.us.r-project.org"
#' old <- options(repos = r)
#'
#' dp <- system.file("test-data", 
#'   "test.package.0001_0.1.0.tar.gz", 
#'   package = "risk.assessr")
#' 
#' # set up package
#' install_list <- set_up_pkg(dp)
#'
#' build_vignettes <- install_list$build_vignettes
#' package_installed <- install_list$package_installed
#' pkg_source_path <- install_list$pkg_source_path
#' rcmdcheck_args <- install_list$rcmdcheck_args
#'
#' # install package locally to ensure test works
#' package_installed <- install_package_local(pkg_source_path)
#' package_installed <- TRUE
#'
#' rcmdcheck_args$path <- pkg_source_path
#' covr_mode_list <- run_covr_modes(pkg_source_path)
#' }
#'
#' @export
run_covr_modes <- function(pkg_source_path,
                           covr_timeout = 60) {
 
  # Get package name and version
  pkg_desc <- get_pkg_desc(pkg_source_path, 
                           fields = c("Package", 
                                      "Version"))
  pkg <- pkg_desc$Package
  
  # Attempt to unload package safely
  tryCatch({
    if (pkg %in% loadedNamespaces()) {
      message(glue::glue("Unloading {pkg} package to avoid coverage conflict..."))
      unloadNamespace(pkg)
    }
  }, error = function(e) {
    message(glue::glue("Warning: Failed to unload {pkg} package. Proceeding anyway."))
    message("Details: ", e$message)
  })
  
  test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
  
  if (test_pkg_data$has_testthat | test_pkg_data$has_testit) {
    message(glue::glue("running standard testing framework"))
    covr_list <- run_coverage(
      pkg_source_path,
      covr_timeout
    )
    
    if (is.na(covr_list$total_cov) && all(is.na(covr_list$res_cov$coverage$filecoverage))) {
      
      if (test_pkg_data$has_testthat) {
        message(glue::glue("running skip testing with standard testing framework"))
        covr_list <- run_covr_skip_stf(pkg_source_path, test_pkg_data)
        
      } else if (test_pkg_data$has_testit) {
        message(glue::glue("running skip testing with non-standard testing framework"))
        covr_list <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
      }
    }
    
  } else if (test_pkg_data$has_tests_base) {
    message(glue::glue("running non-standard testing framework"))
    covr_list <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  } else {
    message("No recognised standard or non-standard testing configuration")
    pkg_name <- basename(pkg_source_path)
    covr_list <- list(
      total_cov = 0,
      res_cov = list(
        name = pkg_name,
        coverage = list(
          filecoverage = matrix(0, nrow = 1, dimnames = list("No functions tested")),
          totalcoverage = 0
        ),
        errors = "No recognised standard or non-standard testing configuration",
        notes = NA
      )
    )
  }
  return(covr_list)
}

#' @title Load Datasets from a Package
#' @description 
#' Loads all datasets from a specified installed package into the global environment.
#' It first attempts to load `.rda` files from the package's `data/` directory.
#' If none are found, it falls back to using `data()` to load datasets listed in the package metadata.
#'
#' @param pkg_name A character string specifying the name of the package.
#' @param env A character string specifying the covr env
#'
#' @return A character vector of dataset names that were attempted to be loaded.
#' Returns `NULL` if the package is not installed.
#'
#' @details 
#' This function is intended for internal use. It silently attempts to load each dataset
#' and suppresses errors if loading fails. Datasets are loaded into the global environment.
#'
#' @keywords internal
load_package_datasets <- function(pkg_name, env = cov_env) {
  if (!requireNamespace(pkg_name, quietly = TRUE)) {
    message(paste("Package", pkg_name, "is not installed."))
    return(NULL)
  }
  
  pkg_path <- find.package(pkg_name)
  data_path <- file.path(pkg_path, "data")
  datasets <- character()
  
  if (dir.exists(data_path)) {
    files <- list.files(data_path, full.names = TRUE)
    for (file in files) {
      ext <- tools::file_ext(file)
      name <- tools::file_path_sans_ext(basename(file))
      tryCatch({
        if (ext == "rda") {
          load(file, envir = env)
        } else if (ext %in% c("tab", "tsv")) {
          assign(name, read.table(file, header = TRUE, sep = "\t"), envir = env)
        } else if (ext == "csv") {
          assign(name, read.csv(file, header = TRUE), envir = env)
        } else if (ext == "txt") {
          assign(name, read.table(file, header = TRUE), envir = env)
        } else if (ext == "json") {
          if (!requireNamespace("jsonlite", quietly = TRUE)) stop("jsonlite package required")
          assign(name, jsonlite::fromJSON(file), envir = env)
        } else if (ext == "R") {
          sys.source(file, envir = env)
        }
        datasets <- c(datasets, name)
      }, error = function(e) {
        message(paste("Failed to load", file, ":", e$message))
      })
    }
  }
  
  if (length(datasets) == 0) {
    available <- data(package = pkg_name)$results[, "Item"]
    for (dataset in available) {
      tryCatch({
        data(list = dataset, package = pkg_name, envir = env)
        datasets <- c(datasets, dataset)
      }, error = function(e) {
        message(paste("Failed to load dataset", dataset, ":", e$message))
      })
    }
  }
  
  return(unique(datasets))
}


#' Clean up global environment and return NULL
#'
#' This internal utility function is used to clean up the global environment
#' by removing all objects and optionally displaying a message before returning `NULL`.
#' It is primarily used within `run_coverage_base()` to ensure consistent cleanup
#' in error-handling scenarios.
#'
#' @param msg Optional character string. If provided, the message will be printed before cleanup.
#' @param env A character string specifying the covr env
#'
#' @return NULL (invisibly)
#'
#' @keywords internal
cleanup_and_return_null <- function(msg = NULL, env = cov_env) {
  if (!is.null(msg)) message(msg)
  
  rm(list = ls(envir = env), envir = env)
  
  find_project_root <- function(start = getwd()) {
    current <- normalizePath(start)
    while (TRUE) {
      if (file.exists(file.path(current, ".Rproj")) ||
          file.exists(file.path(current, "DESCRIPTION")) ||
          file.exists(file.path(current, ".git"))) {
        return(current)
      }
      parent <- dirname(current)
      if (parent == current) break
      current <- parent
    }
    return(NULL)
  }
  
  root_dir <- find_project_root()
  if (is.null(root_dir)) {
    message("Project root not found.")
    return(NULL)
  }
  
  files_to_remove <- c("polr.pdf", "silhouette-ex.ps")
  invisible(lapply(files_to_remove, function(file) {
    file_path <- file.path(root_dir, file)
    if (file.exists(file_path)) {
      try(file.remove(file_path), silent = TRUE)
    }
  }))
  
  return(NULL)
}


#' get functions with no tests
#'
#' @param mapping_df - data frame with source files and test files
#'
#' @return no_tests_df - data frame with source files with no tests
#' 
#' @keywords internal
get_function_no_tests <- function(mapping_df) {
  # Filter rows where test_file is NA
  no_tests_df <- subset(mapping_df, is.na(test_file))
  
  # Check if filtered_df is empty
  # If no rows are filtered, create a single-row data frame with the comment
  if (nrow(no_tests_df) == 0) {
    no_tests_df <- data.frame(
      source_file = NA,
      test_file = NA,
      comment = "all functions have at least 1 test",
      stringsAsFactors = FALSE
    )
  } else {
    # Add a comment to the filtered rows
    no_tests_df$comment <- "No tests found"
  }
  
  return(no_tests_df)
}

test_that("loads datasets using data() if no files in data directory", {
  dummy_env <- new.env()
  
  mockery::stub(load_package_datasets, "requireNamespace", function(pkg, quietly) TRUE)
  mockery::stub(load_package_datasets, "find.package", function(pkg) "/mock/path")
  mockery::stub(load_package_datasets, "dir.exists", function(path) FALSE)
  
  # Stub `data()` to handle both usages
  mockery::stub(load_package_datasets, "data", function(..., list = NULL, package = NULL, envir = NULL) {
    if (is.null(list)) {
      # Simulate data(package = pkg_name)
      return(list(results = matrix(c("dataset1", "dataset2"), ncol = 1, dimnames = list(NULL, "Item"))))
    } else {
      # Simulate data(list = dataset, package = pkg_name, envir = env)
      assign(list, paste0("loaded_", list), envir = envir)
    }
  })
  
  result <- load_package_datasets("fakepkg", env = dummy_env)
  
  expect_true("dataset1" %in% result)
  expect_true("dataset2" %in% result)
  expect_equal(dummy_env$dataset1, "loaded_dataset1")
  expect_equal(dummy_env$dataset2, "loaded_dataset2")
})



test_that("run_covr_modes handles testit framework correctly", {
  # Mocked test_pkg_data with has_testit = TRUE
  mock_check_pkg_tests_and_snaps <- function(pkg_source_path) {
    list(has_testthat = FALSE, has_testit = TRUE, has_tests_base = FALSE)
  }
  
  # Mocked covr_list with NA coverage
  mock_run_coverage <- function(pkg_source_path, covr_timeout) {
    list(
      total_cov = NA,
      res_cov = list(
        coverage = list(filecoverage = c(NA, NA))
      )
    )
  }
  
  # Stub for run_covr_skip_nstf
  mock_run_covr_skip_nstf <- function(pkg_source_path, test_pkg_data) {
    list(
      total_cov = 42,
      res_cov = list(
        coverage = list(filecoverage = matrix(1)),
        notes = "NSTF coverage run"
      )
    )
  }
  
  
  # Mocked get_pkg_desc to return a package name
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub all dependencies
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", mock_check_pkg_tests_and_snaps)
  mockery::stub(run_covr_modes, "run_coverage", mock_run_coverage)
  mockery::stub(run_covr_modes, "run_covr_skip_nstf", mock_run_covr_skip_nstf)
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  
  
  
  # Run the function
  result <- run_covr_modes("dummy/path")
  
  # Check that the NSTF path was taken
  expect_equal(result$total_cov, 42)
  expect_equal(result$res_cov$notes, "NSTF coverage run")
})


test_that("loads various file types from data directory", {
  dummy_env <- new.env()
  
  mockery::stub(load_package_datasets, "requireNamespace", function(pkg, quietly) TRUE)
  mockery::stub(load_package_datasets, "find.package", function(pkg) "/mock/path")
  mockery::stub(load_package_datasets, "dir.exists", function(path) TRUE)
  
  # Simulate multiple file types
  mockery::stub(load_package_datasets, "list.files", function(path, full.names) {
    c(
      "/mock/path/data/data1.rda",
      "/mock/path/data/data2.csv",
      "/mock/path/data/data3.tsv",
      "/mock/path/data/data4.txt",
      "/mock/path/data/data5.json",
      "/mock/path/data/data6.R"
    )
  })
  
  # Simulate file extensions
  mockery::stub(load_package_datasets, "tools::file_ext", function(file) {
    sub(".*\\.(.*)$", "\\1", file)
  })
  
  # Simulate file names without extensions
  mockery::stub(load_package_datasets, "tools::file_path_sans_ext", function(file) {
    sub("\\.[^.]*$", "", basename(file))
  })
  
  # Stub each file type loader
  mockery::stub(load_package_datasets, "load", function(file, envir) assign("data1", "rda_loaded", envir = envir))
  mockery::stub(load_package_datasets, "read.csv", function(file, header) data.frame(a = 1))
  mockery::stub(load_package_datasets, "read.table", function(file, header, sep = "") data.frame(b = 2))
  mockery::stub(load_package_datasets, "requireNamespace", function(pkg, quietly) TRUE)
  mockery::stub(load_package_datasets, "jsonlite::fromJSON", function(file) list(c = 3))
  mockery::stub(load_package_datasets, "sys.source", function(file, envir) assign("data6", "R_loaded", envir = envir))
  
  result <- load_package_datasets("mockpkg", env = dummy_env)
  
  expect_true(all(c("data1", "data2", "data3", "data4", "data5", "data6") %in% result))
  expect_equal(dummy_env$data1, "rda_loaded")
  expect_equal(dummy_env$data2$a, 1)
  expect_equal(dummy_env$data3$b, 2)
  expect_equal(dummy_env$data4$b, 2)
  expect_equal(dummy_env$data5$c, 3)
  expect_equal(dummy_env$data6, "R_loaded")
})


test_that("load_package_datasets returns NULL when package is not installed", {
  # Mock requireNamespace to simulate package not installed
  mockery::stub(load_package_datasets, "requireNamespace", function(pkg_name, quietly) FALSE)
  
  # Run the function
  result <- load_package_datasets("fakepkg")
  
  # Check that result is NULL
  expect_null(result)
})


test_that("cleanup_and_return_null finds project root and removes files", {
  # Create a dummy environment
  dummy_env <- new.env()
  dummy_env$x <- 1
  dummy_env$y <- 2
  
  # Stub getwd to return a mock path
  mockery::stub(cleanup_and_return_null, "getwd", function() "/mock/project")
  mockery::stub(cleanup_and_return_null, "normalizePath", function(path) path)
  # Stub file.exists to simulate project root and file existence
  mockery::stub(cleanup_and_return_null, "file.exists", function(path) {
    if (grepl("\\.Rproj$|DESCRIPTION$|\\.git$", path)) return(TRUE)
    if (grepl("polr\\.pdf$|silhouette-ex\\.ps$", path)) return(TRUE)
    return(FALSE)
  })
  
  # Stub file.remove to simulate file deletion
  mockery::stub(cleanup_and_return_null, "file.remove", function(path) TRUE)
  
  # Stub ls to simulate variables in the environment
  mockery::stub(cleanup_and_return_null, "ls", function(envir) c("x", "y"))
  mockery::stub(cleanup_and_return_null, "normalizePath", function(path) path)
  # Stub rm to simulate environment cleanup
  mockery::stub(cleanup_and_return_null, "rm", function(list, envir) {
    for (var in list) rm(list = var, envir = envir)
  })
  
  result <- cleanup_and_return_null(env = dummy_env)
  
  expect_null(result)
  expect_false(exists("x", envir = dummy_env))
  expect_false(exists("y", envir = dummy_env))
})

test_that("cleanup_and_return_null returns NULL when project root not found", {
  dummy_env <- new.env()
  dummy_env$x <- 1
  
  mockery::stub(cleanup_and_return_null, "getwd", function() "/mock/project")
  mockery::stub(cleanup_and_return_null, "file.exists", function(path) FALSE)
  mockery::stub(cleanup_and_return_null, "normalizePath", function(path) path)
  mockery::stub(cleanup_and_return_null, "ls", function(envir) "x")
  mockery::stub(cleanup_and_return_null, "rm", function(list, envir) {
    for (var in list) rm(list = var, envir = envir)
  })
  
  result <- cleanup_and_return_null(env = dummy_env)
  
  expect_null(result)
  expect_false(exists("x", envir = dummy_env))
})


test_that("returns functions with no tests", {
  mock_df <- data.frame(
    source_file = c("file1.R", "file2.R"),
    test_file = c(NA, "test_file2.R"),
    stringsAsFactors = FALSE
  )
  
  result <- get_function_no_tests(mock_df)
  
  expect_equal(nrow(result), 1)
  expect_equal(result$source_file, "file1.R")
  expect_equal(result$comment, "No tests found")
})

test_that("returns comment when all functions have tests", {
  mock_df <- data.frame(
    source_file = c("file1.R", "file2.R"),
    test_file = c("test_file1.R", "test_file2.R"),
    stringsAsFactors = FALSE
  )
  
  result <- get_function_no_tests(mock_df)
  
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$source_file))
  expect_equal(result$comment, "all functions have at least 1 test")
})

test_that("handles empty input gracefully", {
  mock_df <- data.frame(
    source_file = character(0),
    test_file = character(0),
    stringsAsFactors = FALSE
  )
  
  result <- get_function_no_tests(mock_df)
  
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$source_file))
  expect_equal(result$comment, "all functions have at least 1 test")
})



test_that("run_covr_modes handles standard testing framework with valid coverage", {
  pkg_source_path <- "mock/path"
  
  # Stub check_pkg_tests_and_snaps to simulate standard testthat presence
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", function(...) {
    list(has_testthat = TRUE, has_tests_base = FALSE, has_testit = FALSE)
  })
  
  # Mocked get_pkg_desc to return a package name
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub run_coverage to return complete mock coverage data
  mockery::stub(run_covr_modes, "run_coverage", function(...) {
    list(
      total_cov = 85,
      res_cov = list(
        name = "mockpkg",
        coverage = list(
          filecoverage = matrix(1, nrow = 1, dimnames = list("some_function")),
          totalcoverage = 85
        )
      )
    )
  })
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  
  result <- run_covr_modes(pkg_source_path)
  
  expect_true(is.list(result))
  expect_equal(result$total_cov, 85)
})


test_that("run_covr_modes triggers fallback to run_covr_skip_stf when coverage is NA", {
  pkg_source_path <- "mock/path"
  
  # Mocked get_pkg_desc to return a package name
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub check_pkg_tests_and_snaps to simulate standard testthat presence
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", function(...) {
    list(has_testthat = TRUE, has_tests_base = FALSE, has_testit = FALSE)
  })
  
  # Stub run_coverage to return NA values to trigger fallback
  mockery::stub(run_covr_modes, "run_coverage", function(...) {
    list(
      total_cov = NA,
      res_cov = list(
        name = "mockpkg",
        coverage = list(
          filecoverage = NA,
          totalcoverage = NA
        )
      )
    )
  })
  
  # Stub run_covr_skip_stf to return fallback coverage data
  mockery::stub(run_covr_modes, "run_covr_skip_stf", function(...) {
    list(
      total_cov = 42,
      res_cov = list(
        name = "mockpkg",
        coverage = list(
          filecoverage = matrix(1, nrow = 1, dimnames = list("fallback_function")),
          totalcoverage = 42
        )
      )
    )
  })
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  
  result <- run_covr_modes(pkg_source_path)
  
  expect_true(is.list(result))
  expect_equal(result$total_cov, 42)
  expect_equal(rownames(result$res_cov$coverage$filecoverage), "fallback_function")
})



test_that("run_covr_modes handles non-standard testing framework", {
  pkg_source_path <- "mock/path"
  
  # Mocked get_pkg_desc to return a package name
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  # Mock unloadNamespace to avoid side effects
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub check_pkg_tests_and_snaps to simulate non-standard test presence
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", function(...) {
    list(has_testthat = FALSE, has_tests_base = TRUE, has_testit = FALSE)
  })
  
  # Stub run_covr_skip_nstf to return mock coverage data
  mockery::stub(run_covr_modes, "run_covr_skip_nstf", function(...) {
    list(total_cov = 50, res_cov = list(name = "mockpkg", coverage = list(totalcoverage = 50)))
  })
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  
  result <- run_covr_modes(pkg_source_path)
  
  expect_true(is.list(result))
  expect_equal(result$total_cov, 50)
})


test_that("run_covr_modes handles no testing configuration", {
  pkg_source_path <- "mock/path"
  
  # Mocked get_pkg_desc to return a package name
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  # Mock unloadNamespace to avoid side effects
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub all dependencies correctly
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", function(...) {
    list(has_testthat = FALSE, has_tests_base = FALSE, has_testit = FALSE)
  })
  
  # Run the function
  result <- run_covr_modes(pkg_source_path)
  
  # Assertions
  expect_true(is.list(result))
  expect_equal(result$total_cov, 0)
  expect_match(result$res_cov$errors, "No recognised standard or non-standard testing configuration")
})


test_that("run_covr_modes unloads package when loaded", {
  pkg_source_path <- "mock/path"
  
  # Mock get_pkg_desc to return a known package
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  # Mock unloadNamespace to simulate successful unload
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub loadedNamespaces to simulate that the package is loaded
  mock_loadedNamespaces <- function() {
    c("mockpkg", "stats", "utils")
  }
  
  # Stub dependencies
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  mockery::stub(run_covr_modes, "loadedNamespaces", mock_loadedNamespaces)
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", function(...) {
    list(has_testthat = FALSE, has_tests_base = FALSE, has_testit = FALSE)
  })
  
  result <- run_covr_modes(pkg_source_path)
  
  expect_equal(result$total_cov, 0)
  expect_match(result$res_cov$errors, "No recognised standard or non-standard testing configuration")
})


test_that("run_covr_modes handles error when unloading package", {
  pkg_source_path <- "mock/path"
  
  # Mock get_pkg_desc to return a known package
  mock_get_pkg_desc <- function(pkg_source_path, fields) {
    list(Package = "mockpkg", Version = "0.1.0")
  }
  
  # Mock unloadNamespace to simulate failure
  mock_unloadNamespace <- function(pkg) {
    stop("Simulated unload failure")
  }
  
  # Stub loadedNamespaces to simulate that the package is loaded
  mock_loadedNamespaces <- function() {
    c("mockpkg", "stats", "utils")
  }
  
  # Stub dependencies
  mockery::stub(run_covr_modes, "get_pkg_desc", mock_get_pkg_desc)
  mockery::stub(run_covr_modes, "unloadNamespace", mock_unloadNamespace)
  mockery::stub(run_covr_modes, "loadedNamespaces", mock_loadedNamespaces)
  mockery::stub(run_covr_modes, "check_pkg_tests_and_snaps", function(...) {
    list(has_testthat = FALSE, has_tests_base = FALSE, has_testit = FALSE)
  })
  
  result <- run_covr_modes(pkg_source_path)
  
  expect_equal(result$total_cov, 0)
  expect_match(result$res_cov$errors, "No recognised standard or non-standard testing configuration")
})

#' Get function descriptions
#' 
#' @description get descriptions of exported functions 
#' 
#' @param pkg_name - name of the package
#'
#' @keywords internal
get_func_descriptions <- function(pkg_name) {
  
  # Try loading Rd database
  db <- tryCatch(
    tools::Rd_db(pkg_name),
    error = function(e) {
      message(glue::glue("tools::Rd_db() failed for '{pkg_name}': {e$message}"))
      return(NULL)
    }
  )
  
  # If Rd_db failed, try alternative method
  if (is.null(db)) {
    pkg_path <- find.package(pkg_name, quiet = TRUE)
    man_path <- file.path(pkg_path, "man")
    rd_files <- list.files(man_path, pattern = "\\.Rd$", full.names = TRUE)
    
    db <- lapply(rd_files, function(f) {
      tryCatch(
        tools::parse_Rd(f),
        error = function(e) {
          message(glue::glue("Failed to parse {basename(f)}: {e$message}"))
          return(NULL)
        }
      )
    })
    db <- db[!sapply(db, is.null)]
  }
  
  # Extract descriptions
  description <- lapply(db, function(x) {
    tags <- lapply(x, attr, "Rd_tag")
    if ("\\description" %in% tags) {
      out <- paste(unlist(x[which(tags == "\\description")]), collapse = "")
    } else {
      out <- NULL
    }
    invisible(out)
  })
  
  # Extract names
  names(description) <- sapply(db, function(x) {
    tags <- lapply(x, attr, "Rd_tag")
    if ("\\name" %in% tags) {
      name <- unlist(x[which(tags == "\\name")])
      name <- gsub("\n", "", name)
      return(name)
    } else {
      return(NULL)
    }
  })
  
  # Filter out NULLs
  description <- description[!sapply(description, is.null)]
  
  return(description)
}

    

#' @keywords internal
#' @noRd
normalize_codecov <- function(u) {
  if (is.na(u) || !nzchar(u)) return(u)
  sub("/graph/badge\\.svg(\\?.*)?$", "/badge.svg\\1", u, perl = TRUE)
}

#' @keywords internal
#' @noRd
parse_doc_safe <- function(txt) {
  doc <- tryCatch(xml2::read_html(txt), error = function(e) NULL)
  if (!is.null(doc)) {
    return(doc)
  }
  tryCatch(xml2::read_xml(txt), error = function(e) NULL)
}

#' @keywords internal
#' @noRd
fetch_badge_value <- function(url, timeout_secs = 5, ua = "risk.assessr") {
  if (is.na(url) || !nzchar(url)) {
    return(NULL)
  }
  url <- normalize_codecov(url)
  if (!grepl("\\.svg($|\\?)", url, ignore.case = TRUE)) {
    return(NULL)
  }
  h <- curl::new_handle(
    connecttimeout = timeout_secs,
    timeout = timeout_secs,
    followlocation = TRUE,
    useragent = ua
  )
  tryCatch({
    res <- curl::curl_fetch_memory(url, handle=h)
    if (res$status_code != 200) {
      stop("Error while fetching badge url:", url, ". Error:", res$status_code)
    }
    doc <- rawToChar(res$content) |>
       parse_doc_safe()
    
    if (is.null(doc)) {
      return(NULL)
    }
    vals <- doc |>
      xml2::xml_find_all(".//text") |>
      xml2::xml_text() |>
      trimws()
    
    if (length(vals)>0) {
      return(list(name = vals[1], value=rev(vals)[1]))
    }
    ttl <- doc |>
      xml2::xml_find_first(".//title") |>
      xml2::xml_text() |>
      trimws()
    
    if (nzchar(ttl)) {
      return(ttl)
    }
    NULL
  }, 
  error = function(e) NULL
  )
}


#' List badges image URLs from a local README
#'
#' Scans a local README (Markdown) and returns badge image URLs.
#'
#' @param path Character scalar; path to a local README file (e.g., "README.md").
#' @return data.frame with badge info
#'
#' @examples
#' \dontrun{
#' tmp <- tempfile(fileext = ".md")
#' writeLines(c(
#'   "# MyPkg",
#'   "![build](build-status.svg)",
#'   "![cov](coverage.svg)"
#' ), tmp)
#' out <- list_badges(tmp)
#' print(out)
#' }
#' @export
list_badges <- function(path) {
  links <- c()
  stopifnot(is.character(path), length(path) == 1L, !is.na(path))
  if (!file.exists(path)) {
    stop("File not found: ", path, call. = FALSE)
  }

  # Single robust read: if it fails, warn and move on (no second pass)
  txt <- tryCatch(
    path |>
     readLines(warn = FALSE, encoding = "UTF-8") |>
      paste(collapse = "\n"),
    error = function(e) {
      warning(sprintf("Failed to read '%s': %s", path, conditionMessage(e)))
      NULL
  })
  if (is.null(txt)) {
    return(links)
  } 
  txt <- enc2utf8(txt)
  
  # Linked badges: [![...](IMG)](LINK)
  pat_linked <- "\\[!\\[[^\\]]*\\]\\(([^)]+)\\)\\]\\(([^)]+)\\)"
  mA     <- gregexpr(pat_linked, txt, perl = TRUE)
  hitsA  <- if (mA[[1]][1] != -1L) regmatches(txt, mA)[[1]] else character(0)
  if (length(hitsA)) {
    links <- c(
      links, 
      vapply(hitsA, function(x) sub(pat_linked, "\\1", x, perl = TRUE), character(1L))
      )
  }
  
  # Standalone badges: ![...](IMG)
  pat_img <- "!\\[[^\\]]*\\]\\(([^)]+)\\)"
  mB <- gregexpr(pat_img, txt, perl = TRUE)
  spansB <- mB[[1]]
  hitsB_full <- if (length(spansB) == 1L && spansB[1] == -1L) character(0) else regmatches(txt, list(spansB))[[1]]
  
  # Remove B hits that sit inside any A hit (avoid dupes)
  spansA <- gregexpr(pat_linked, txt, perl = TRUE)[[1]]
  lenA   <- attr(spansA, "match.length")
  inA <- function(start, length) {
    if (is.null(lenA) || (length(spansA) == 1L && spansA[1] == -1L)) return(FALSE)
    any(start >= spansA & (start + length - 1L) <= (spansA + lenA - 1L))
  }
  lenB <- attr(spansB, "match.length")
  keepB <- if (length(hitsB_full)) which(!mapply(inA, spansB, lenB)) else integer(0)
  hitsB <- if (length(keepB)) hitsB_full[keepB] else character(0)
  
  if (length(hitsB) >0) {
    links <- c(
      links, 
      vapply(hitsB, function(x) sub(pat_img, "\\1", x, perl = TRUE), character(1L))
    )
  }
  
  links |>
    unique() |>
    lapply(\(x) fetch_badge_value(x)) |>
    Filter(f = Negate(is.null)) |>
    as.data.frame()
  
}

test_that("normalize_codecov converts /graph/badge.svg -> /badge.svg", {
  x <- c(
    "https://codecov.io/gh/org/repo/graph/badge.svg",
    "https://codecov.io/gh/org/repo/graph/badge.svg?token=abc",
    "https://img.shields.io/badge/CRAN-ok.svg"
  )
  got <- vapply(x, normalize_codecov, character(1))
  
  # core checks
  expect_true(grepl("/badge.svg$", got[1]))
  expect_true(grepl("/badge.svg\\?token=abc$", got[2]))
  
  # strip names to avoid spurious failure
  expect_identical(unname(got[3]), x[3])
})


test_that("parse_doc_safe parses both HTML and XML safely", {
  html <- "<html><body><img src='a.svg'/></body></html>"
  xml  <- "<svg><text>a</text></svg>"
  expect_s3_class(parse_doc_safe(html), "xml_document")
  expect_s3_class(parse_doc_safe(xml),  "xml_document")
  expect_null(parse_doc_safe(""))
})

test_that("list_badges() scans README and returns a data.frame (no error)", {
  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp), add = TRUE)
  
  writeLines(c(
    "# MyPkg",
    # linked badge
    "[![R-CMD-check](https://github.com/r-lib/testthat/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/testthat/actions/workflows/R-CMD-check.yaml)",
    # standalone badge
    "![coverage](https://codecov.io/gh/r-lib/testthat/graph/badge.svg)"
  ), tmp)
  
  out <- list_badges(tmp)
  # The refactor returns a data.frame assembled from fetch_badge_value() results.
  expect_true(is.data.frame(out) || length(out) >= 0)
})

test_that("fetch_badge_value() returns parsed value or NULL for SVG badges", {
  testthat::skip_if_offline()
  # Known public SVGs (stable providers)
  urls <- c(
    "https://github.com/r-lib/testthat/actions/workflows/R-CMD-check.yaml/badge.svg",
    "https://codecov.io/gh/r-lib/testthat/graph/badge.svg"
  )
  vals <- lapply(urls, function(u) fetch_badge_value(u, timeout_secs = 10))
  # Accept either list(name,value), character title, or NULL on parse errors
  ok <- vapply(vals, function(v) {
    is.null(v) ||
      is.character(v) ||
      (is.list(v) && all(c("name","value") %in% names(v)))
  }, logical(1))
  expect_true(all(ok))
})

#' Get Total Number of PubMed Articles for a Search Term
#'
#' This function queries the NCBI E-utilities API to retrieve the total number of 
#' PubMed articles that match a given search term or R package name.
#'
#' @param package_name A character string representing the search term (e.g., an R package name).
#' @param api_key Optional. A character string with an NCBI API key. If not supplied,
#' it will attempt to use the option \code{getOption("pubmed.api_key")}.
#'
#' @return An integer representing the total number of PubMed articles matching the search term, 
#' or \code{NA} if the request fails.
#'
#' @examples
#' \dontrun{
#' get_pubmed_count("ggplot2")
#' }
#'
#' @import curl
#' @importFrom jsonlite fromJSON
#' @export
get_pubmed_count <- function(package_name, api_key = getOption("pubmed.api_key", NULL)) {
  query <- URLencode(package_name)
  full_url <- paste0("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi",
                     "?db=pubmed&term=", query, "&retmode=json")
  
  if (!is.null(api_key)) {
    full_url <- paste0(full_url, "&api_key=", api_key)
  }
  
  h <- new_handle()
  con <- tryCatch(curl(full_url, handle = h), error = function(e) return(NULL))
  if (is.null(con)) return(NA)
  
  res <- tryCatch(readLines(con, warn = FALSE), error = function(e) return(NULL))
  close(con)
  
  if (is.null(res)) return(NA)
  
  data <- tryCatch(jsonlite::fromJSON(paste(res, collapse = "")), error = function(e) return(NULL))
  if (is.null(data)) {
    message("No articles found on pubmed for ", package_name)
    return(NA)
  } 

  count <- as.numeric(data$esearchresult$count)
  return(count)
}

#' Get Annual PubMed Article Counts for a Search Term
#'
#' This function queries the NCBI E-utilities API to retrieve the number of PubMed articles
#' published each year over a specified number of past years for a given search term or package name.
#'
#' @param package_name A character string representing the search term (e.g., an R package name).
#' @param years_back An integer specifying how many years back to query. Defaults to 10.
#' @param api_key Optional. A character string with an NCBI API key. If not supplied,
#' it will attempt to use the option \code{getOption("pubmed.api_key")}.
#'
#' @return A data frame with two columns:
#' \describe{
#'   \item{Year}{The publication year (integer).}
#'   \item{Count}{The number of articles published in that year (integer).}
#' }
#' Returns an empty data frame if no valid data is retrieved.
#'
#' @examples
#' \dontrun{
#' get_pubmed_by_year("ggplot2", years_back = 5)
#' }
#'
#' @import curl
#' @import xml2
#' @export
get_pubmed_by_year <- function(package_name, years_back = 10, api_key = getOption("pubmed.api_key", NULL)) {
  current_year <- as.numeric(format(Sys.Date(), "%Y"))
  start_year <- current_year - years_back
  year_counts <- integer()
  
  for (year in start_year:current_year) {
    count <- tryCatch({
      query <- URLencode(paste0(package_name, " AND ", year, "[dp]"))
      esearch_url <- paste0(
        "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi",
        "?db=pubmed&term=", query,
        "&retmode=xml"
      )
      if (!is.null(api_key)) {
        esearch_url <- paste0(esearch_url, "&api_key=", api_key)
      }
      
      xml_data <- xml2::read_xml(curl::curl(esearch_url))
      as.integer(xml2::xml_text(xml2::xml_find_first(xml_data, "//Count")))
    }, error = function(e) {
      message("No articles found on pubmed for ", package_name)
      return(NA)
    })
    
    year_counts[as.character(year)] <- count
  }
  
  valid <- !is.na(year_counts)
  if (!any(valid)) {
    return(data.frame(Year = integer(), Count = integer(), stringsAsFactors = FALSE))
  }
  
  return(data.frame(
    Year = as.integer(names(year_counts)[valid]),
    Count = as.integer(year_counts[valid]),
    stringsAsFactors = FALSE
  ))
}

# get_pubmed_count

test_that("get_pubmed_count returns mocked value correctly", {
  fake_json_text <- '{"esearchresult":{"count":"1234"}}'
  mockery::stub(get_pubmed_count, "readLines", function(...) fake_json_text)

  count <- get_pubmed_count("mocked_package")
  expect_equal(count, 1234)
})

test_that("get_pubmed_count handles nonexistent package (returns 0)", {
  fake_json_text <- '{"esearchresult":{"count":"0"}}'
  mockery::stub(get_pubmed_count, "readLines", function(...) fake_json_text)

  count <- get_pubmed_count("nonsensepackagename123456")
  expect_equal(count, 0)
})

test_that("get_pubmed_count handles API failure (returns NA)", {
  mockery::stub(get_pubmed_count, "curl", function(...) stop("Network error"))
  
  count <- get_pubmed_count("ggplot2")
  expect_true(is.na(count))
})

test_that("get_pubmed_count returns NA on network failure", {
  mockery::stub(get_pubmed_count, "readLines", function(...) stop("Network error"))
  
  result <- get_pubmed_count("ggplot2")
  expect_true(is.na(result))
})

test_that("get_pubmed_count returns 0 when no articles are found", {
  json_zero <- '{"esearchresult":{"count":"0"}}'
  mockery::stub(get_pubmed_count, "readLines", function(...) json_zero)
  
  result <- get_pubmed_count("nonsensepackagename")
  expect_equal(result, 0)
})

test_that("get_pubmed_count returns NA and shows message when JSON parsing fails", {
  # Mock readLines to return invalid JSON
  mockery::stub(get_pubmed_count, "readLines", function(...) "invalid_json")
  
  # Mock curl and close to suppress side effects
  mockery::stub(get_pubmed_count, "curl", function(...) "fake_con")
  mockery::stub(get_pubmed_count, "close", function(...) NULL)
  mockery::stub(get_pubmed_count, "new_handle", function(...) NULL)
  
  expect_message(
    count <- get_pubmed_count("fakepkg"),
    "No articles found on pubmed for fakepkg"
  )
  expect_true(is.na(count))
})





# get_pubmed_by_year

test_that("get_pubmed_by_year returns mocked counts by year", {
  fake_xml <- xml2::read_xml('<?xml version="1.0"?><eSearchResult><Count>42</Count></eSearchResult>')
  
  mockery::stub(get_pubmed_by_year, "xml2::read_xml", function(...) fake_xml)
  df <- get_pubmed_by_year("mocked_package", years_back = 2)
  
  expect_s3_class(df, "data.frame")
  expect_named(df, c("Year", "Count"))
  expect_equal(nrow(df), 3)  # current year and 2 previous
  expect_true(all(df$Count == 42))
})

test_that("get_pubmed_by_year returns zero counts when no articles found", {
  fake_xml <- xml2::read_xml('<?xml version="1.0"?><eSearchResult><Count>0</Count></eSearchResult>')
  
  mockery::stub(get_pubmed_by_year, "xml2::read_xml", function(...) fake_xml)
  df <- get_pubmed_by_year("nonsensepackagename123456", years_back = 1)
  
  expect_s3_class(df, "data.frame")
  expect_named(df, c("Year", "Count"))
  expect_equal(nrow(df), 2)
  expect_true(all(df$Count == 0))
})

test_that("get_pubmed_by_year handles API failure safely", {
  mockery::stub(get_pubmed_by_year, "xml2::read_xml", function(...) stop("Simulated API failure"))
  
  df <- get_pubmed_by_year("ggplot2", years_back = 2)
  
  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 0)
  expect_named(df, c("Year", "Count"))
})

#' Create items matched
#' 
#' @description create a df with functions that match exported functions from
#' a suggested package
#'  
#' @param extracted_functions - exported functions from the target package
#' @param suggested_exp_func - exported functions from a suggested package
#'
#' @keywords internal
create_items_matched <- function(extracted_functions, suggested_exp_func) {
  
  items_matched <- extracted_functions %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      matched = any(stringr::str_detect(suggested_function, 
                                        paste0("(", suggested_exp_func$package, 
                                               "::)?", 
                                               suggested_exp_func$exported_functions)))
    ) %>%
    dplyr::filter(matched) %>%
    dplyr::select(-matched)
  
  return(items_matched)
}

#' process items matched
#' 
#' @description create a df with functions that match exported functions from
#' a suggested package
#' 
#' @details This function extracts all the functions from the function body 
#' and filters them to keep only function calls
#' It also extracts all the valid function names, matches with the source package
#' and writes a message
#'  
#' @param items_matched - exported functions match the suggested package function
#' @param suggested_exp_func - exported functions from a suggested package
#'
#' @keywords internal
process_items_matched <- function(items_matched, suggested_exp_func) {
  
  # get all unique function names
  all_functions <- unique(suggested_exp_func$functions)
  
  # step 1 - extract function calls
  processed_items_int <- items_matched %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      function_calls = list(unique(unlist(
        stringr::str_extract_all(
          suggested_function,
          paste0("\\b(", paste(all_functions, collapse = "|"), ")\\b")
        )
      )))
    ) %>%
    tidyr::unnest(function_calls) %>%
    dplyr::filter(function_calls != "") %>% # Filter to keep non-empty function calls
    dplyr::filter(!grepl("\\(", function_calls)) %>% # Filter to exclude calls with parentheses
    dplyr::mutate(
      function_calls = gsub("[^a-zA-Z0-9_]", "", function_calls), # Remove non-alphanumeric characters
      is_assignment = sapply(function_calls, function(call) {
        # Caused by error in `grepl()`:
        # ! invalid regular expression '\b[data\b\s*(<-|=)', reason 'Missing ']''
        any(grepl(paste0("\\b", call, "\\b\\s*(<-|=)"), suggested_function, perl = TRUE))
      }),
      is_parameter = sapply(function_calls, function(call) {
        any(grepl(paste0("\\(", ".*\\b", call, "\\b", ".*\\)"), suggested_function, perl = TRUE)) &
          !any(grepl(paste0("\\b", call, "\\b\\s*(<-|=)"), suggested_function, perl = TRUE)) &
          !any(grepl(paste0("\\b", call, "\\b\\s*\\("), suggested_function, perl = TRUE))
      })
    ) %>%
    dplyr::ungroup()
  
  # Ensure is_assignment and is_parameter are logical vectors
  processed_items_int <- processed_items_int %>%
    dplyr::mutate(
      is_assignment = as.logical(is_assignment),
      is_parameter = as.logical(is_parameter)
    )
  
  # Filter out assignments
  processed_items_no_assignments <- processed_items_int %>%
    dplyr::filter(!is_assignment)
  
  # Filter out parameters
  processed_items_no_parameters <- processed_items_no_assignments %>%
    dplyr::filter(!is_parameter)
  
  # Remove the temporary columns
  processed_items_final <- processed_items_no_parameters %>%
    dplyr::select(-is_assignment, -is_parameter)
  
  # extract function names and set up targeted package and message
  processed_items <- processed_items_final %>%
    dplyr::mutate(
      suggested_function = stringr::str_extract(function_calls, "\\w+$"),  # Changed: Extract only the function names
      targeted_package = suggested_exp_func$package[match(function_calls, suggested_exp_func$functions)], 
      message = "Please check if the targeted package should be in Imports"
    ) %>%
    # filter(!is.na(source_package)) %>%
    dplyr::distinct(source, suggested_function, targeted_package, message) %>%
    dplyr::select(source, suggested_function, targeted_package, message)
  
  return(processed_items)
}





#' Function to check suggested exported functions
#'
#' @description This function checks the exported functions of target package
#' against the exported functions of Suggested dependencies to see if any 
#' Suggested packages should be in Imports in the DESCRIPTION file
#'
#' @param pkg_name - name of the target package   
#' @param pkg_source_path - source of the target package 
#' @param suggested_deps - dependencies in Suggests
#'
#' @return - data frame with results of Suggests check
#' @export
check_suggested_exp_funcs <- function(pkg_name, 
                                      pkg_source_path, 
                                      suggested_deps) {
  
  # Input checks
  checkmate::assert_string(pkg_name)
  checkmate::assert_string(pkg_source_path)
  checkmate::assert_class(suggested_deps, "data.frame")
  
  assert_non_empty_string <- function(x, var.name = deparse(substitute(x))) {
    checkmate::assert_string(x, min.chars = 1, .var.name = var.name)
    if (x == "" || x == " ") {
      stop(sprintf("%s cannot be an empty string", var.name))
    }
  }
  
  assert_non_empty_string(pkg_name)
  assert_non_empty_string(pkg_source_path)
  
  # check if there are any dependencies to be checked
  if (nrow(suggested_deps) != 0) {
    # Check if the package source path contains an R folder
    exp_possible <- contains_r_folder(pkg_source_path)
    
    if (exp_possible == TRUE) {
      # Get the exported functions from the package source path
      exp_func <- get_exports(pkg_source_path)
      
      # rename columns in exp_func to match 
      exp_func <- exp_func %>%
         dplyr::rename(
           source = exported_function,
           suggested_function = function_body
           )
      
      #rename df for further processing
      func_df <- exp_func
      
      # Get the suggested exported functions
      suggested_exp_func <- get_suggested_exp_funcs(suggested_deps)
      
      # Function to test if each column has no values
      test_no_values <- function(df) {
        sapply(df, function(column) all(is.na(column)))
      }
      
      # test for no values
      no_values_test <- test_no_values(suggested_exp_func)
      
      if (!all(no_values_test)) {
        
        # check if func_df is empty of values    
        is_empty <- func_df %>% 
          dplyr::summarise_all(~ all(is.na(.) | . == " ")) %>% 
          unlist() %>% all()
        
        if (is_empty) {
          message(glue::glue("No exported functions from source package {pkg_name}"))
          suggested_matched_functions <- data.frame(
            source = pkg_name,
            suggested_function = NA,
            targeted_package = NA,
            message = glue::glue("No exported functions from source package {pkg_name}")
          ) 
        } else {
          
          items_matched <- create_items_matched(func_df, 
                                                suggested_exp_func)
          
          
          suggested_matched_functions <- 
            process_items_matched(items_matched, 
                                  suggested_exp_func)
        }
        
      } else {
        message("No Suggested packages in the DESCRIPTION file")
        suggested_matched_functions <- data.frame(
          source = pkg_name,
          suggested_function = NA,
          targeted_package = NA,
          message = "No Suggested packages in the DESCRIPTION file"
        ) 
      }
    } else {
      message("No R folder found in the package source path")
      suggested_matched_functions <- data.frame(
        source = pkg_name,
        suggested_function = NA,
        targeted_package = NA,
        message = "No R folder found in the package source path"
      )  
      
    }
    
    # check if suggested_matched functions is empty of values    
    is_empty <- suggested_matched_functions %>% 
      dplyr::summarise_all(~ all(is.na(.) | . == " ")) %>% 
      unlist() %>% all()
    
    if (is_empty) {
      message("No exported functions from Suggested packages in the DESCRIPTION file")
      suggested_matched_functions <- data.frame(
        source = pkg_name,
        suggested_function = NA,
        targeted_package = NA,
        message = "No exported functions from Suggested packages in the DESCRIPTION file"
      ) 
    }
  } else {
    message("No Imports or Suggested packages in the DESCRIPTION file")
    suggested_matched_functions <- data.frame(
      source = pkg_name,
      suggested_function = NA,
      targeted_package = NA,
      message = "No Imports or Suggested packages in the DESCRIPTION file"
    ) 
  }
  
  
  return(suggested_matched_functions)
}

test_that("check_suggested_exp_funcs returns 2 matches", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
  
    pkg_name <- "here"
    
    result <- check_suggested_exp_funcs(pkg_name, 
                                        pkg_source_path, 
                                        test_deps)
    
    expect_true(all(!is.na(result$source)))
    expect_true(all(is.na(result$targeted_package)))
    expect_true(all(!is.na(result$message)))
  }
})

test_that("check_suggested_exp_funcs returns matches messages for S3 functions", {
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)

  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0009_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)

  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args


  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE

  if (package_installed == TRUE ) {

    rcmdcheck_args$path <- pkg_source_path
    pkg_name <- "test.package.0009"

    #set up deps_df for this package
    package <- c("magrittr", "checkmate",
                 "testthat", "dplyr"
    )
    type <- c("Imports", "Suggests", "Suggests",
              "Suggests"
    )
    deps_df <- data.frame(package, type)


    result <- check_suggested_exp_funcs(pkg_name,
                                        pkg_source_path,
                                        deps_df)

    expect_equal(nrow(result), 4L)

    test_that("Messages are correct for S3 function", {
      expect_true(all(grepl("Please check if the targeted package should be in Imports", result$message)))
    })
  }
})

test_that("check_suggested_exp_funcs returns matches messages for S3 functions", {
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)

  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0009_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())

  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)

  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args


  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE

  if (package_installed == TRUE ) {

    rcmdcheck_args$path <- pkg_source_path
    pkg_name <- "test.package.0009"

    #set up deps_df for this package
    package <- c("magrittr", "checkmate",
                 "testthat", "dplyr"
    )
    type <- c("Imports", "Suggests", "Suggests",
              "Suggests"
    )
    deps_df <- data.frame(package, type)


    result <- check_suggested_exp_funcs(pkg_name,
                                        pkg_source_path,
                                        deps_df)

    expect_equal(nrow(result), 4L)

    test_that("Messages are correct for S3 function", {
      expect_true(all(grepl("Please check if the targeted package should be in Imports", result$message)))
    })
  }
})

test_that("check_suggested_exp_funcs returns matches messages for S4 functions", {
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0010_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    pkg_name <- "test.package.0010"
    
    #set up deps_df for this package
    package <- c("checkmate",
                 "testthat", "methods"
    )
    type <- c("Suggests", "Suggests",
              "Suggests"
    ) 
    deps_df <- data.frame(package, type)
    
    
    result <- check_suggested_exp_funcs(pkg_name, 
                                        pkg_source_path, 
                                        deps_df)
    
    expect_equal(nrow(result), 1L)
    
    test_that("Messages are correct for S4 function", {
      expect_true(all(grepl("Please check if the targeted package should be in Imports", result$message)))
    })
  }
})

test_that("get_exports works correctly with R6 functions", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0011_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- risk.assessr::set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  # install package locally to ensure test works
  package_installed <-
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    
    rcmdcheck_args$path <- pkg_source_path
    
    pkg_name <- "test.package.0011"
    
    #set up deps_df for this package
    package <- c("mockery", "testthat", "checkmate",
                 "cli", "fastmap", "rlang", "R6"
    )
    type <- c("Imports", "Suggests", "Suggests",
              "Suggests", "Suggests", "Suggests", "Suggests"
    ) 
    deps_df <- data.frame(package, type)
    
    result <- check_suggested_exp_funcs(pkg_name, 
                                        pkg_source_path, 
                                        deps_df)
    
    expect_equal(nrow(result), 1L)
    
    expect_true(all(!is.na(result$source)))
    expect_true(all(!is.na(result$message)))
    test_that("Messages are correct for R6 function", {
      expect_true(all(grepl("Please check if the targeted package should be in Imports", result$message)))
    })
  }
})

test_that("check_suggested_exp_funcs returns no exported functions message", {
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0005_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # install package locally to ensure test works
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
  
    pkg_name <- "test.package.0005"
    
    #set up deps_df for this package
    package <- c("magrittr", "checkmate",
                 "testthat", "dplyr"
    )
    type <- c("Imports", "Suggests", "Suggests",
              "Suggests"
    ) 
    deps_df <- data.frame(package, type)
    
    
    result <- check_suggested_exp_funcs(pkg_name, 
                                        pkg_source_path, 
                                        deps_df)
    
    expect_equal(nrow(result), 1L)
    
    expect_true(all(!is.na(result$source)))
    expect_true(all(!is.na(result$message)))
    expect_equal(result$message, "No exported functions from Suggested packages in the DESCRIPTION file")
  }
})

test_that("check_suggested_exp_funcs returns No R folder found in the package source path message", {
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0006_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # install package locally to ensure test works
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {
    pkg_name <- "test.package.0006"
    
    #set up deps_df for this package
    package <- c("magrittr", "checkmate",
                 "testthat", "dplyr"
    )
    type <- c("Imports", "Suggests", "Suggests",
              "Suggests"
    ) 
    deps_df <- data.frame(package, type)
    
    
    result <- check_suggested_exp_funcs(pkg_name, 
                                        pkg_source_path, 
                                        deps_df)
    
    expect_equal(nrow(result), 1L)
    
    expect_true(all(!is.na(result$source)))
    expect_true(all(!is.na(result$message)))
    testthat::expect_message(
      result <- check_suggested_exp_funcs(pkg_name, 
                                          pkg_source_path, 
                                          deps_df),
      glue::glue("No R folder found in the package source path"),
      fixed = TRUE
    )
  }
})


test_that("check_suggested_exp_funcs handles NULL suggested_exp_func correctly", {

  mock_contains_r_folder <- function(...) TRUE

mock_get_exports <- function(...) {
  data.frame(
    exported_function = c("func1", "func2", "func3"),
    function_body = c("body1", "body2", "body3")
  )
}

  mock_get_suggested_exp_funcs <- function(...) NULL
  # mock_get_function_details <- function(...) data.frame(details = "mocked details")

  pkg_name <- "target_pkg"
  pkg_source_path <- "dummy_path"
  suggested_deps <- data.frame(package = c("pkgA", "pkgB"))

  testthat::local_mocked_bindings(
    `contains_r_folder` = mock_contains_r_folder,
    `get_exports` = mock_get_exports,
    `get_suggested_exp_funcs` = mock_get_suggested_exp_funcs
  #  `get_function_details` = mock_get_function_details
  )

  result <- check_suggested_exp_funcs(pkg_name, pkg_source_path, suggested_deps)

  expect_true(any(grepl("No Suggested packages in the DESCRIPTION file", result$message)))
})
  

# Create toy datasets for testing create_items_matched
extracted_functions <- dplyr::tibble(
  suggested_function = c("dplyr::mutate", "stringr::str_detect", "ggplot2::ggplot", "base::mean")
)

suggested_exp_func <- dplyr::tibble(
  package = c("dplyr", "stringr"),
  exported_functions = c("mutate", "str_detect")
)

# Expected output
expected_output <- dplyr::tibble(
  suggested_function = c("dplyr::mutate", "stringr::str_detect")
)

# Test the function
test_that("create_items_matched works correctly", {
  result <- create_items_matched(extracted_functions, suggested_exp_func) %>% 
    dplyr::ungroup() # Convert to regular tibble
  expect_equal(result, expected_output)
})

# Create toy datasets for process_items_matched testing
items_matched <- dplyr::tibble(
  source = c("file1", "file2", "file3"),
  suggested_function = c("dplyr::mutate", "stringr::str_detect", "base::mean")
)

suggested_exp_func <- dplyr::tibble(
  package = c("dplyr", "stringr", "base"),
  functions = c("mutate", "str_detect", "mean")
)

# Expected output
expected_output <- dplyr::tibble(
  source = c("file1", "file2", "file3"),
  suggested_function = c("mutate", "str_detect", "mean"),
  targeted_package = c("dplyr", "stringr", "base"),
  message = rep("Please check if the targeted package should be in Imports", 3)
)

# Test the function
test_that("process_items_matched works correctly", {
  result <- process_items_matched(items_matched, suggested_exp_func)
  expect_equal(result, expected_output)
})

test_that("check_suggested_exp_funcs handles empty suggested_deps", {
  # Mock data
  pkg_name <- "testpkg"
  pkg_source_path <- "path/to/testpkg"
  suggested_deps <- data.frame() # Empty data frame
  
  # Mock functions
  mock_contains_r_folder <- mockery::mock(TRUE)
  mock_get_exports <- mockery::mock(data.frame(exported_function = character()))
  mock_get_function_details <- mockery::mock(data.frame())
  mock_get_suggested_exp_funcs <- mockery::mock(data.frame())
  mock_create_items_matched <- mockery::mock(data.frame())
  mock_process_items_matched <- mockery::mock(data.frame())
  
  # Replace the actual functions with mocks
  mockery::stub(check_suggested_exp_funcs, "risk.assessr::contains_r_folder", mock_contains_r_folder)
  mockery::stub(check_suggested_exp_funcs, "risk.assessr::get_exports", mock_get_exports)
  # mockery::stub(check_suggested_exp_funcs, "get_function_details", mock_get_function_details)
  mockery::stub(check_suggested_exp_funcs, "risk.assessr::get_suggested_exp_funcs", mock_get_suggested_exp_funcs)
  mockery::stub(check_suggested_exp_funcs, "create_items_matched", mock_create_items_matched)
  mockery::stub(check_suggested_exp_funcs, "process_items_matched", mock_process_items_matched)
  
  # Call the function
  result <- check_suggested_exp_funcs(pkg_name, pkg_source_path, suggested_deps)
  
  # Check the result
  expect_equal(result$message, "No Imports or Suggested packages in the DESCRIPTION file")
  expect_true(is.na(result$suggested_function))
  expect_true(is.na(result$targeted_package))
})


test_that("check_suggested_exp_funcs handles empty exported functions from source package", {
  # Mock inputs
  pkg_name <- "mockpkg"
  pkg_source_path <- "mock/path"
  suggested_deps <- data.frame(package = "dplyr", stringsAsFactors = FALSE)
  
  # Stub functions to simulate conditions
  mockery::stub(check_suggested_exp_funcs, "contains_r_folder", function(...) TRUE)
  
  # Simulate get_exports returning a data frame with only NA or blank values
  mockery::stub(check_suggested_exp_funcs, "get_exports", function(...) {
    data.frame(exported_function = c(NA, " "), function_body = c(NA, " "), stringsAsFactors = FALSE)
  })
  
  # Simulate get_suggested_exp_funcs returning a valid data frame
  mockery::stub(check_suggested_exp_funcs, "get_suggested_exp_funcs", function(...) {
    data.frame(suggested_function = c("filter"), targeted_package = c("dplyr"), stringsAsFactors = FALSE)
  })
  
  # Run the function
  result <- check_suggested_exp_funcs(pkg_name, pkg_source_path, suggested_deps)
  
  # Check the result
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 1)
  expect_equal(result$source, pkg_name)
  expect_true(is.na(result$suggested_function))
  expect_true(is.na(result$targeted_package))
  expect_match(result$message, "No exported functions from source package")
})

#' Convert input to ISO 8601 date (YYYY-MM-DD)
#'
#' @description
#' Formats inputs as ISO 8601 calendar dates (`"YYYY-MM-DD"`).
#'
#' @param x A vector coercible to `Date` (e.g., character, `Date`, or `POSIXct`).
#'   If `NULL`, the function returns `NULL`.
#'
#' @return
#' A character vector the same length as `x` (or `NULL` if `x` is `NULL`)
#' with dates formatted as `"YYYY-MM-DD"`. Unparseable elements or `NA`s
#' become `NA_character_`.
#'
#' @details
#' Coercion uses [base::as.Date()], so time-of-day and timezone information are
#' dropped. For `POSIXct` inputs, the date is computed by `as.Date.POSIXct`
#' (which uses `tz = "UTC"` by default). Be aware that ambiguous character
#' dates (e.g., `"01/02/2024"`) depend on your R locale/format unless you
#' specify a format before calling this function.
#'
#'
#' @keywords internal
as_iso_date <- function(x) {
  
  if (is.null(x)) return(NULL)
  
  tryCatch({
    d <- as.Date(x)
    format(d, "%Y-%m-%d")
  }, error = function(e) {
    NULL
  })
}

#' Get Internal Package URL
#'
#' This function retrieves the URL of an internal package on Sanofi Mirror, its latest version, 
#' and a list of all available versions.
#'
#' @param package_name A character string specifying the name of the package.
#' @param version An optional character string specifying the version of the package.
#' Defaults to `NULL`, in which case the latest version will be used.
#' @param base_url A character string specifying the base URL of the internal package manager.
#' 
#' @return A list containing:
#'   - `url`: A character string of the package URL (or `NULL` if not found).
#'   - `last_version`: A list with `version` and `date` of the latest version (or `NULL`).
#'   - `version`: The version used to generate the URL (or `NULL`).
#'   - `all_versions`: A list of all available versions, each as a list with `version` and `date`.
#'
#' @examples
#' \dontrun{
#' result <- get_internal_package_url("internalpackage", version = "1.0.1")
#' print(result)
#' }
#'
#' @importFrom curl curl_fetch_memory
#' @importFrom jsonlite fromJSON
#' @export
get_internal_package_url <- function(package_name, version = NULL,
                                     base_url = NULL) {
  
  if (is.null(base_url)) {
    repos <- getOption("repos")
    
    if (is.null(repos) || !("INTERNAL_RSPM" %in% names(repos))) {
        message("NO INTERNAL_RSPM FOUND")
        return(list(url = NULL, last_version = NULL, all_versions = list(), repo_id = NULL, repo_name = NULL))
    }
    base_url <- repos["INTERNAL_RSPM"]
  }
  
  # Function to fetch JSON data from a URL
  fetch_json <- function(url) {
    response <- tryCatch(curl::curl_fetch_memory(url), error = function(e) NULL)
    if (is.null(response) || response$status_code != 200) return(NULL)
    tryCatch({
      json_content <- rawToChar(response$content)
      jsonlite::fromJSON(json_content)
    }, error = function(e) {
      message("Failed to parse JSON:", conditionMessage(e))
      NULL
    })
  }
  
  # Fetch the list of repositories
  repos_url <- paste0(base_url, "/__api__/repos/")
  repos_data <- fetch_json(repos_url)
  
  if (is.null(repos_data)) {
    return(list(url = NULL, last_version = NULL, all_versions = list(), repo_id = NULL, repo_name = NULL))
  }
  
  # Get all repository IDs 
  repo_ids <- repos_data$id
  repo_names <- repos_data$name
  names(repo_ids) <- repo_names
  
  found_data <- NULL
  found_repo_id <- NULL
  found_repo_name <- NULL
  
  for (i in seq_along(repo_ids)) {
    repo_id <- repo_ids[i]
    repo_name <- repo_names[i]
    
    package_url <- paste0(base_url, "/__api__/repos/", repo_id, "/packages/", package_name)
    data <- fetch_json(package_url)
    
    if (!is.null(data) && !is.null(data$version)) {
      found_data <- data
      found_repo_id <- repo_id
      found_repo_name <- repo_name
      break
    }
  }
  
  if (is.null(found_data)) {
    return(list(url = NULL, last_version = NULL, all_versions = list(), repo_id = NULL, repo_name = NULL))
  }
  
  # Process versions
  last_version <- NULL
  all_versions <- list()
  
  if (!is.null(found_data$version) && !is.null(found_data$date_publication)) {
    last_version <- list(
      version = as.character(found_data$version)[1L],  # JSON may return non-atomic
      date = as.Date(substr(found_data$date_publication, 1, 10))
    )
    all_versions[[length(all_versions) + 1]] <- last_version
  }
  
  if (!is.null(found_data$archived)) {
    if (is.data.frame(found_data$archived)) {
      for (i in seq_len(nrow(found_data$archived))) {
        ver <- as.character(found_data$archived$version[i])[1L]
        date <- as.Date(substr(found_data$archived$date[i], 1, 10))
        all_versions[[length(all_versions) + 1]] <- list(version = ver, date = date)
      }
    } else if (is.list(found_data$archived)) {
      for (av in found_data$archived) {
        if (!is.null(av$version) && !is.null(av$date)) {
          ver <- as.character(av$version)[1L]
          date <- as.Date(substr(av$date, 1, 10))
          all_versions[[length(all_versions) + 1]] <- list(version = ver, date = date)
        }
      }
    }
  }
  
  # Determine the URL for the chosen version (coerce to scalar character; JSON may return list/vector)
  chosen_version <- if (is.null(version)) last_version$version else version
  chosen_version <- as.character(chosen_version)[1L]
  last_ver_char <- if (!is.null(last_version)) as.character(last_version$version)[1L] else NA_character_
  url <- NULL

  if (!is.na(chosen_version) && nzchar(chosen_version)) {
    if (!is.na(last_ver_char) && chosen_version == last_ver_char) {
      # Latest version URL
      url <- paste0(base_url, "/", found_repo_name, "/latest/src/contrib/", 
                    package_name, "_", chosen_version, ".tar.gz")
    } else {
      # Archived version URL
      version_list <- vapply(all_versions, function(x) as.character(x$version)[1L], character(1L))
      if (length(version_list) > 0L && chosen_version %in% version_list) {
        url <- paste0(base_url, "/", found_repo_name, "/latest/src/contrib/Archive/", 
                      package_name, "/", package_name, "_", chosen_version, ".tar.gz")
      }
    }
  }
  
  return(list(
    url = url, 
    last_version = last_version, 
    all_versions = all_versions, 
    repo_id = found_repo_id,
    repo_name = found_repo_name
  ))
}


#' Check if a Package Exists on CRAN
#'
#' This function checks if a given package is available on CRAN. 
#'
#' @param package_name A character string specifying the name of the package to check.
#'
#' @return A logical value: `TRUE` if the package exists on CRAN, `FALSE` otherwise.
#'
#' @examples
#' \dontrun{
#' # Check if the package "ggplot2" exists on CRAN
#' check_cran_package("ggplot2")
#' }
#' @importFrom curl curl_fetch_memory
#' @export
check_cran_package <- function(package_name) {
  base_url <- "https://cran.r-project.org/web/packages/"
  package_url <- paste0(base_url, package_name, "/index.html")
  
  # Perform GET request
  response <- curl::curl_fetch_memory(package_url)
  
  # If package does not exist, return FALSE
  if (response$status_code != 200) {
    return(FALSE)
  }
  
  content <- rawToChar(response$content)
  html_doc <- xml2::read_html(content)
  clean_content <- xml2::xml_text(html_doc)
  clean_content <- tolower(clean_content)
  
  clean_content <- gsub("[[:punct:]]", "", clean_content)
  clean_content <- gsub("\\s+", " ", clean_content)
  
  escaped_package_name <- gsub("\\.", "\\\\.", package_name)
  escaped_package_name <- gsub("[[:punct:]]", "", escaped_package_name)  
  
  removed_pattern <- paste0("package ", escaped_package_name, " was removed from the cran repository")
  
  # Check if the package has been removed
  if (grepl(removed_pattern, clean_content, ignore.case = TRUE, perl = TRUE)) {
    return(FALSE)
  }
  return(TRUE)
}


#' Parse Package Information from CRAN Archive
#'
#' This function retrieves the package archive information from the CRAN Archive.
#'
#' @param name A character string specifying the name of the package to fetch information for.
#'
#' @return A character string containing the raw HTML content of the package archive page, or `NULL`
#' if the request fails or the package is not found.
#'
#' @examples
#' \dontrun{
#' # Fetch package archive information for "dplyr"
#' parse_package_info("dplyr")
#'
#'}
#' @importFrom curl curl_fetch_memory
#' @export
parse_package_info <- function(name) {
  url <- paste0("https://cran.r-project.org/src/contrib/Archive/", name, "/")
  
  # Perform GET request
  response <- curl::curl_fetch_memory(url)
  
  if (response$status_code == 200) {
    return(rawToChar(response$content))
  } else {
    return(NULL)
  }
}

#' Parse HTML Version Information for a Package
#'
#' This function extracts version information from the HTML content of a CRAN archive page.
#'
#' @param html_content A character string containing the HTML content of a CRAN package archive page.
#' @param package_name A character string specifying the name of the package to extract version information for.
#'
#' @return A list of lists, where each sublist contains:
#'   - `package_name`: package name.
#'   - `package_version`: package version.
#'   - `link`: link to the package tarball.
#'   - `date`: The date associated with the package version.
#'   - `size`: The size of the package tarball.
#'
#' @examples
#' \dontrun{
#' html_content <- parse_package_info("dplyr")
#' parse_html_version(html_content, "dplyr")
#'}
#' @importFrom xml2 read_html xml_find_all xml_find_first xml_text xml_attr
#' @keywords internal
parse_html_version <- function(html_content, package_name) {
  
  data <- list()
  
  if (!is.null(html_content)) {
    # Parse the HTML content
    doc <- xml2::read_html(html_content)
    rows <- xml2::xml_find_all(doc, "//tr")
    
    for (row in rows) {
      cols <- xml2::xml_find_all(row, "td")
      if (length(cols) >= 4) {  # Ensure it's not a divider row
        link_tag <- xml2::xml_find_first(cols[2], "a")
        
        if (!is.na(xml2::xml_text(link_tag))) {
          href <- xml2::xml_attr(link_tag, "href")
          text <- xml2::xml_text(link_tag)
          date <- xml2::xml_text(cols[3])
          size <- xml2::xml_text(cols[4])
          version <- ""
          
          if (grepl(paste0("^", package_name, "(?:_|$)"), text)) {
            version <- gsub(paste0("^", package_name, "(?:_|$)|\\.tar\\.gz$"), "", text)
          } else {
            version <- ""  
          }
          
          if (version == "Parent Directory" | version == "") {
            next
          }
          
          data <- append(data, list(
            list(
              package_name = package_name,
              package_version = version,
              link = href,
              date = date,
              size = size
            )
          ))
        }
      }
    }
  }
  
  return(data)
}


#' Get Package Versions
#'
#' This function retrieves all available versions of a package, including the latest version,
#' by parsing the provided version table and querying the RStudio Package Manager.
#'
#' @param table A list of parsed package data (e.g., from `parse_html_version()`), where each element contains package details such as `package_version` and `date`.
#' @param package_name A character string specifying the name of the package to fetch versions for.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{all_versions}{A list of named lists, each containing:
#'     \describe{
#'       \item{version}{The version number of the package.}
#'       \item{date}{The associated publication date as a string.}
#'     }
#'   }
#'   \item{last_version}{A named list containing the latest version of the package with:
#'     \describe{
#'       \item{version}{The latest version number.}
#'       \item{date}{The publication date of the latest version.}
#'     }
#'     May be \code{NULL} if the API call fails.
#'   }
#' }
#'
#' @examples
#' \dontrun{
#' # Define the input table
#' table <- list(
#'   list(
#'     package_name = "here",
#'     package_version = "0.1",
#'     link = "here_0.1.tar.gz",
#'     date = "2017-05-28 08:13",
#'     size = "3.5K"
#'   ),
#'   list(
#'     package_name = "here",
#'     package_version = "1.0.0",
#'     link = "here_1.0.0.tar.gz",
#'     date = "2020-11-15 18:10",
#'     size = "32K"
#'   )
#' )
#'
#' result <- get_versions(table, "here")
#' print(result)
#' }
#'
#' @importFrom curl curl_fetch_memory
#' @importFrom jsonlite fromJSON
#' @export
get_versions <- function(table, package_name) {
  
  # From the CRAN Archive table
  all_versions <- lapply(table, function(item) {
    if (!is.null(item$package_version)) {
      list(
        version = item$package_version,
        date = as_iso_date(item$date)  # handles "YYYY-MM-DD hh:mm"
      )
    } else {
      NULL
    }
  })
  
  # From Posit Package Manager (latest)
  url_latest_version <- paste0("https://packagemanager.posit.co/__api__/repos/1/packages/", package_name)
  response <- curl::curl_fetch_memory(url_latest_version)
  
  if (response$status_code == 200) {
    data <- jsonlite::fromJSON(rawToChar(response$content))
    if (!is.null(data$version) && !is.null(data$date_publication)) {
      last_version <- list(
        version = as.character(data$version),  # JSON may return numeric
        date = as_iso_date(substr(data$date_publication, 1, 10))
      )
      existing_versions <- sapply(all_versions, function(x) x$version)
      if (!(last_version$version %in% existing_versions)) {
        all_versions <- append(all_versions, list(last_version))
      }
    } else {
      last_version <- NULL
    }
  } else {
    last_version <- NULL
  }
  
  if (is.null(table)) {
    return(list(
      all_versions = if (!is.null(last_version)) list(last_version) else NULL,
      last_version = last_version
    ))
  }
  
  list(all_versions = all_versions, last_version = last_version)
}


#' Check and Fetch CRAN Package
#'
#' This function checks if a package exists on CRAN and retrieves the corresponding package URL and version details.
#' If a specific version is not provided, the latest version is used.
#'
#' @param package_name A character string specifying the name of the package to check and fetch.
#' @param package_version An optional character string specifying the version of the package to fetch. Defaults to `NULL`.
#'
#' @return A list with one of the following structures:
#' 
#' \describe{
#'   \item{package_url}{Character string; the URL to download the package tarball.}
#'   \item{last_version}{A named list with \code{version} and \code{date} for the latest available version.}
#'   \item{version}{Character string; the requested version (or \code{NULL} if not specified).}
#'   \item{all_versions}{A list of named lists, each with \code{version} and \code{date}, representing all available versions.}
#' }
#' 
#'
#' @examples
#' \dontrun{
#' # Check and fetch a specific version of "ggplot2"
#' result <- check_and_fetch_cran_package("ggplot2", package_version = "3.3.5")
#' print(result)
#' }
#'
#' @importFrom curl curl_fetch_memory
#' @importFrom jsonlite fromJSON
#' @export
check_and_fetch_cran_package <- function(package_name, package_version = NULL) {
  if (!check_cran_package(package_name)) {
    return(list(package_url = NULL, last_version = NULL, version = NULL, all_versions = NULL))
  }
  
  html <- parse_package_info(package_name)
  table <- if (!is.null(html)) parse_html_version(html, package_name) else NULL
  versions <- get_versions(table, package_name)

  url <- get_cran_package_url(package_name, package_version, versions$last_version, versions$all_versions)
  
  # Ensure all dates are ISO strings
  all_versions_list <- lapply(versions$all_versions, function(x) {
    list(version = x$version, date = as_iso_date(x$date))
  })
  
  if (is.null(url) && !is.null(package_version) &&
      !(package_version %in% sapply(versions$all_versions, function(x) x$version))) {
    return(list(
      error = paste("Version", package_version, "for", package_name, "not found"),
      versions_available = all_versions_list
    ))
  }
  
  list(
    package_url = url,
    last_version = if (is.null(versions$last_version)) NULL else list(
      version = versions$last_version$version,
      date = as_iso_date(versions$last_version$date)
    ),
    version = package_version,
    all_versions = all_versions_list
  )
}



#' Get CRAN Package URL
#'
#' This function constructs the CRAN package URL for a specified package and version.
#'
#' @param package_name A character string specifying the name of the package.
#' @param version An optional character string specifying the version of the package.
#' If `NULL`, the latest version is assumed.
#' @param last_version A named list with elements version and date, 
#' representing the latest available version and its publication date.
#' @param all_versions A list of named lists, each with elements version and date, 
#' representing all known versions of the package and their publication dates.
#'
#' @return A character string containing the URL to download the package tarball,
#' or "No valid URL found" if the version is not found in the list of available versions.
#'
#' @examples
#' \dontrun{
#' # Example data structure
#' last_version <- list(version = "1.0.10", date = "2024-01-01")
#' all_versions <- list(
#'   list(version = "1.0.0", date = "2023-01-01"),
#'   list(version = "1.0.10", date = "2024-01-01")
#' )
#'
#' # Get the URL for the latest version of "dplyr"
#' url <- get_cran_package_url("dplyr", NULL, last_version, all_versions)
#' print(url)
#' }
#' @export
get_cran_package_url <- function(package_name, version, last_version, all_versions) {
  if (is.null(all_versions)) {
    all_versions <- list()
  } else {
    all_versions <- all_versions[!vapply(all_versions, is.null, logical(1L))]
  }
  if (length(all_versions) == 0L && is.null(last_version)) {
    return("No valid URL found")
  }
  # Extract version strings and coerce to character for consistent comparison
  # (JSON/Posit API may return numeric, e.g. 1.02 instead of "1.0.2")
  version_list <- if (length(all_versions) > 0L) {
    vapply(all_versions, function(x) as.character(x$version), character(1L))
  } else {
    character(0L)
  }

  # Set version to the latest if not provided
  if (is.null(version)) {
    if (!is.null(last_version)) {
      version <- as.character(last_version$version)
    } else if (length(version_list) > 0L) {
      # Posit API unreachable: use most recent version from Archive
      version <- version_list[which.max(numeric_version(version_list, strict = FALSE))]
    } else {
      return("No valid URL found")
    }
  } else {
    version <- as.character(version)
  }

  # Check if the version exists in version_list
  if (version %in% version_list) {
    # If it's the latest version (main contrib vs Archive)
    last_ver_char <- if (!is.null(last_version)) as.character(last_version$version) else NA_character_
    if (!is.na(last_ver_char) && version == last_ver_char) {
      return(paste0("https://cran.r-project.org/src/contrib/", package_name, "_", version, ".tar.gz"))
    }
    
    # Otherwise, it's an archived version
    return(paste0("https://cran.r-project.org/src/contrib/Archive/", package_name, "/", package_name, "_", version, ".tar.gz"))
  }
  
  # Return fallback message
  return("No valid URL found")
}



#' Extract and Validate Package Hosting Information
#' 
#' This function retrieves hosting links for an R package from various sources such as GitHub, CRAN, internal repositories, or Bioconductor.
#' 
#' @param pkg_name Character. The name of the package.
#' @param pkg_version Character. The version of the package.
#' @param pkg_source_path Character. The file path to the package source directory containing the DESCRIPTION file.
#' 
#' @return A list containing the following elements:
#' 
#' - `github_links`: GitHub links related to the package.
#' - `cran_links`: CRAN links
#' - `internal_links`: internal repository links.
#' - `bioconductor_links`: Bioconductor links 
#' 
#' If links are found, return empty or NULL.
#' 
#' @details
#' The function extracts hosting links by:
#' 1. Parsing the `DESCRIPTION` file for GitHub and BugReports URLs.
#' 2. Checking if the package is valid on CRAN and others host repository
#' 
#' If no links are found in the `DESCRIPTION` file, returns `NULL`
#' 
#' @examples
#' \dontrun{
#' result <- get_host_package(pkg_name, pkg_version, pkg_source_path)
#' print(result)
#' }
#' @export
get_host_package <- function(pkg_name, pkg_version, pkg_source_path) {
  
  message(glue::glue("Checking host package"))
  
  # Path to the DESCRIPTION file
  pkg_desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  
  # Load the DESCRIPTION file
  d <- description$new(file = pkg_desc_path)
  description_text <- d$str()
  
  # Extract URLs from $get_urls() and $get_list("BugReports")
  urls <- d$get_urls()
  
  if (d$has_fields("BugReports")) {
    bug_reports <- d$get_list("BugReports")
  } else {
    bug_reports <- NULL  
  }  
  
  all_links <- c(urls, bug_reports)
  github_pattern <- "https://github.com/([^/]+)/([^/]+).*"
  matching_links <- grep(github_pattern, all_links, value = TRUE)
  owner_names <- sub(github_pattern, "\\1", matching_links)
  package_names_github <- sub(github_pattern, "\\2", matching_links)
  
  valid <- which(owner_names != "" & package_names_github != "")
  
  if (length(valid) > 0) {
    github_links <- unique(paste0("https://github.com/", owner_names[valid], "/", package_names_github[valid]))
  } else {
    github_links <- NULL
  }
  
  is_package_valid <- check_cran_package(pkg_name)
  
  if (is_package_valid) {
    # Fetch the archive content
    result_cran <- check_and_fetch_cran_package(pkg_name, pkg_version)  
    
    # get CRAN links
    cran_links <- get_cran_package_url(pkg_name, pkg_version, result_cran$last_version, result_cran$all_versions)
    
  } else {
    cran_links <- NULL
    
  }
  
  # get internal package link
  result_internal <- get_internal_package_url(pkg_name, pkg_version)
  internal_links <- result_internal$url
  
  bioconductor_links <-  NULL
  
  # Define the Bioconductor pattern
  bioconductor_pattern <- paste0("https://(git\\.)?bioconductor\\.org/packages/", pkg_name, "/?(?=[\\s,]|$)")
  
  # Find the first match in the text for Bioconductor
  bioconductor_match <- regexpr(bioconductor_pattern, description_text, perl = TRUE)
  
  if (bioconductor_match != -1) {
    bioconductor_links <- regmatches(description_text, bioconductor_match)
  }
  
  result <- list(
    github_links = github_links,
    cran_links = cran_links,
    internal_links = internal_links,
    bioconductor_links = bioconductor_links
  )
  
  return(result)
}


#' Download R Package Source Tarball
#'
#' @description
#' Downloads the source tarball (.tar.gz) for a specified R package from CRAN, 
#' Bioconductor, or an internal repository fallback.
#'
#' @param package_name Character string specifying the name of the package to download.
#' @param version Optional character string specifying the version of the package.
#'        If NULL (default), the latest available version will be downloaded.
#' @param repos Optional character vector of repository URLs to use.
#'        If NULL (default), the current repositories set in options() will be used.
#'
#' @return Character string with the path to the downloaded tarball file, or NULL if
#'         the download was unsuccessful.
#'
#'
#' @examples
#' \dontrun{
#' # Download the latest version of dplyr
#' tarball_path <- get_package_tarfile("dplyr")
#'
#' # Download a specific version
#' tarball_path <- get_package_tarfile("dplyr", version = "1.0.0")
#'
#' # Use a specific repository
#' tarball_path <- get_package_tarfile("dplyr", repos = "https://cloud.r-project.org")
#' }
#'
#' @importFrom remotes download_version
#' @export
get_package_tarfile <- function(package_name, version = NULL, repos = getOption("repos")) {

  download_successful <- FALSE
  temp_file <- NULL

  # Try configured repos first (RSPM, CRAN mirror, etc.)
  repos_usable <- !is.null(repos) && length(repos) > 0L &&
    !all(repos %in% c("@CRAN@", NA_character_))
  if (repos_usable) {
    message(paste("Checking", package_name, "in configured repositories..."))
    tryCatch({
      temp_file <- remotes::download_version(
        package = package_name,
        version = version,
        repos = repos,
        type = "source"
      )
      download_successful <- TRUE
    }, error = function(e) {
      message(paste("Download from configured repos failed:", e$message))
    })
  }

  # Fallback: try CRAN (public cran.r-project.org)
  if (!download_successful) {
    message(paste("Checking", package_name, "on CRAN..."))
    if (check_cran_package(package_name)) {
      result_cran <- check_and_fetch_cran_package(package_name = package_name, package_version = version)
      package_url <- result_cran$package_url
      if (!is.null(package_url) && is.character(package_url) && startsWith(package_url, "http")) {
        temp_file <- tempfile(fileext = ".tar.gz")
        tryCatch({
          download.file(url = package_url, destfile = temp_file, mode = "wb")
          download_successful <- TRUE
        }, error = function(e) {
          message(paste("CRAN download failed:", e$message))
        })
      } else {
        message("CRAN package URL could not be resolved.")
      }
    }
  }
  
  # If not successful on CRAN, try Bioconductor
  if (!download_successful) {
    message(paste("Checking", package_name, "on Bioconductor..."))
    
    html_content <- fetch_bioconductor_releases()
    release_data <- parse_bioconductor_releases(html_content)
    result_bio <- get_bioconductor_package_url(package_name, version, release_data)
    
    if (!is.null(result_bio$url)) {
      temp_file <- tempfile(fileext = ".tar.gz")
      tryCatch({
        download.file(url = result_bio$url, destfile = temp_file, mode = "wb")
        download_successful <- TRUE
      }, error = function(e) {
        message(paste("Bioconductor download failed:", e$message))
      })
    } else {
      message(paste("No", package_name, "package found on Bioconductor"))
    }
  }
  
  # Final fallback: internal mirror
  if (!download_successful) {
    message(paste("Attempting internal fallback download for", package_name))
    package_info <- get_internal_package_url(package_name, version)
    
    tryCatch({
      if (!is.null(package_info$url)) {
        temp_file <- tempfile(fileext = ".tar.gz")
        download.file(package_info$url, temp_file, mode = "wb", quiet = TRUE)
        download_successful <- TRUE
      } else {
        stop("Failed to find the package URL from the internal repository.")
      }
    }, error = function(e) {
      stop(
        paste("Failed to download the package from the internal repository:", e$message),
        call. = FALSE
      )
    })
  }
  
  if (download_successful && !is.null(temp_file)) {
    message(paste("Package tarball saved at:", temp_file))
    return(temp_file)
  } else {
    stop("Failed to obtain package tarball.")
  }
}

test_that("falls back to Bioconductor if CRAN download fails", {
  # Counter to simulate first download (CRAN) failing, second (Bioc) succeeding
  call_i <- 0
  
  mockery::stub(get_package_tarfile, "check_cran_package", TRUE)
  mockery::stub(get_package_tarfile, "check_and_fetch_cran_package", function(package_name, package_version = NULL) {
    list(package_url = sprintf("https://cran.r-project.org/src/contrib/%s_%s.tar.gz",
                               package_name, ifelse(is.null(package_version), "0.1", package_version)))
  })
  
  # download.file: first call throws, second call creates tarball
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, ...) {
    call_i <<- call_i + 1
    if (call_i == 1) stop("CRAN download failed due to timeout")
    make_tarball(destfile, pkg_name = "fallbackpkg", version = "0.1")
    0
  })
  
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html_content")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) {
    list(url = "http://dummybioc.org/fallback.tar")
  })
  
  path <- get_package_tarfile("SomePkg")
  expect_true(file.exists(path))
  expect_match(normalizePath(path), "tar\\.gz$")
})

test_that("downloads a package from Bioconductor if not on CRAN", {
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html_content")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) {
    list(url = "http://dummybioc.org/pkg.tar")
  })
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, ...) {
    make_tarball(destfile, pkg_name = "dummybiocpkg", version = "0.1")
    0
  })
  
  path <- get_package_tarfile("Biobase")
  expect_true(file.exists(path))
})

test_that("downloads a package from internal mirror if not on CRAN or Bioconductor", {
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) list(url = NULL))
  
  # NOTE: your function must call: get_internal_package_url(package_name, version)
  mockery::stub(get_package_tarfile, "get_internal_package_url", function(package_name, version = NULL, base_url = NULL) {
    list(
      url = "https://internal.example/repo/src/contrib/InternalPkg_0.1.tar.gz",
      last_version = list(version = "0.1", date = as.Date("2023-01-01")),
      all_versions = list(list(version = "0.1", date = as.Date("2023-01-01"))),
      repo_id = "123", repo_name = "internal-repo"
    )
  })
  
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, mode = "wb", quiet = TRUE) {
    make_tarball(destfile, pkg_name = "internalpkg", version = "0.1")
    0
  })
  
  path <- get_package_tarfile("InternalPkg")
  expect_true(file.exists(path))
})

test_that("errors if package not found on CRAN, Bioconductor, or internal", {
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) list(url = NULL))
  
  # Internal returns nothing useful
  mockery::stub(get_package_tarfile, "get_internal_package_url", function(package_name, version = NULL, base_url = NULL) {
    list(url = NULL)
  })
  
  # Now we expect an error, not a message + NULL
  expect_error(
    get_package_tarfile("GhostPkg"),
    regexp = "Failed to obtain package tarball|Failed to find the package URL|Failed to download"
  )
})



test_that("downloads a package successfully from CRAN", {
  mockery::stub(get_package_tarfile, "check_cran_package", TRUE)
  mockery::stub(get_package_tarfile, "check_and_fetch_cran_package", function(package_name, package_version = NULL) {
    list(package_url = sprintf("https://cran.r-project.org/src/contrib/%s_%s.tar.gz",
                               package_name, ifelse(is.null(package_version), "0.1", package_version)))
  })
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, ...) {
    make_tarball(destfile, pkg_name = "cranpkg", version = "0.1")
    0
  })
  
  path <- get_package_tarfile("ggplot2")
  expect_true(file.exists(path))
})

test_that("uses manually specified repos when provided (and restores them afterward)", {
  
  # Save original repos to check restoration
  old_repos <- getOption("repos")
  
  # Dummy repo to set during the call
  dummy_repo <- c(CRAN = "https://my.custom.repo")
  
  # Force a failure path so that get_package_tarfile errors
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) NULL)
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) list(url = NULL))
  mockery::stub(get_package_tarfile, "get_internal_package_url", function(...) list(url = NULL))
  
  expect_error(
    get_package_tarfile("nonexistentpkg", repos = dummy_repo),
    regexp = "Failed to obtain package tarball|Failed to find the package URL|Failed to download"
  )
  
  # Optional: ensure repos are restored after error
  expect_identical(getOption("repos"), old_repos)
})

test_that("NULL input returns NULL", {
  expect_null(as_iso_date(NULL))
  expect_null(as_iso_date("not-a-date"))
})

test_that("Date inputs are formatted correctly", {
  d <- as.Date(c("2025-01-01", "2025-12-31"))
  out <- as_iso_date(d)
  expect_type(out, "character")
  expect_identical(out, c("2025-01-01", "2025-12-31"))
})

test_that("Character inputs coerce via as.Date and format as ISO", {
  x <- c("2024-02-28", "2024/02/29", "not-a-date")
  out <- as_iso_date(x)
  # as.Date("2024/02/29") parses under some locales; ensure expected positions
  expect_true(out[1] == "2024-02-28")
  expect_true(is.na(out[3]))
})

test_that("POSIXct inputs drop time and timezone (UTC by as.Date default)", {
  ts <- as.POSIXct(c("2025-03-14 00:00:00", "2025-03-14 23:59:59"), tz = "UTC")
  out <- as_iso_date(ts)
  expect_identical(out, c("2025-03-14", "2025-03-14"))
})

test_that("NA values are preserved as NA_character_", {
  x <- c(NA, "2020-01-02")
  out <- as_iso_date(x)
  expect_true(is.na(out[1]))
  expect_identical(out[2], "2020-01-02")
})

test_that("Vectorization: length and element-wise behavior", {
  x <- c("2021-01-01", "bad", "2021-01-03")
  out <- as_iso_date(x)
  expect_length(out, length(x))
  expect_identical(out[c(1,3)], c("2021-01-01", "2021-01-03"))
  expect_true(is.na(out[2]))
})


test_that("Empty HTML content", {
  empty_html <- NULL
  result <- parse_html_version(empty_html, "dummy_package")
  expect_equal(result, list())
})

test_that("HTML content without versions", {
  html_with_no_version <- '<table>
   <tr><td valign="top"></td><td><a href="https://example.com/package.tar.gz">package.tar.gz</a></td><td align="right">2023-01-01 12:00</td><td align="right">1.0M</td></tr>
  </table>'
  
  result <- parse_html_version(html_with_no_version, "package")
  expect_equal(result, list())
})

test_that("Valid HTML with one version", {
  valid_html <- '<table>
    <tr><td valign="top"></td><td><a href="https://example.com/package_1.0.tar.gz">package_1.0.tar.gz</a></td><td align="right">2023-01-01 12:00</td><td align="right">1.0M</td></tr>
  </table>'
  
  result <- parse_html_version(valid_html, "package")
  
  expected <- list(
    list(
      package_name = "package",
      package_version = "1.0",
      link = "https://example.com/package_1.0.tar.gz",
      date = "2023-01-01 12:00",
      size = "1.0M"
    )
  )
  expect_equal(result, expected)
})

test_that("Valid HTML with multiple versions", {
  valid_html <- '<table>
    <tr><td valign="top"></td><td><a href="https://example.com/package_1.0.tar.gz">package_1.0.tar.gz</a></td><td align="right">2023-01-01 12:00</td><td align="right">1.0M</td></tr>
    <tr><td valign="top"></td><td><a href="https://example.com/package_2.0.tar.gz">package_2.0.tar.gz</a></td><td align="right">2023-02-01 12:00</td><td align="right">1.5M</td></tr>
  </table>'
  
  result <- parse_html_version(valid_html, "package")
  expected <- list(
    list(
      package_name = "package",
      package_version = "1.0",
      link = "https://example.com/package_1.0.tar.gz",
      date = "2023-01-01 12:00",
      size = "1.0M"
    ),
    list(
      package_name = "package",
      package_version = "2.0",
      link = "https://example.com/package_2.0.tar.gz",
      date = "2023-02-01 12:00",
      size = "1.5M"
    )
  )
  expect_equal(result, expected)
})

test_that("HTML with missing version in table", {
  html_missing_version <- '<table>
    <tr><td valign="top"></td><td><a href="https://example.com/package.tar.gz"></a></td><td align="right">2023-01-01 12:00</td><td align="right">1.0M</td></tr>
  </table>'
  
  result <- parse_html_version(html_missing_version, "package")
  expect_equal(result, list())
})

test_that("HTML with corrupted table structure", {
  html_corrupted <- '<table>
    <tr><td valign="top"></td><td></td><td align="right">2023-01-01 12:00</td></tr>
  </table>'
  
  result <- parse_html_version(html_corrupted, "package")
  expect_equal(result, list())
})


# Test cases: parse_package_info

test_that("parse_package_info works correctly with mock response", {
  # Create a mock response object
  mock_response <- list(
    status_code = 200,
    content = charToRaw("<html>Mock Content</html>")
  )
  
  # Mock curl::curl_fetch_memory
  mock_fetch <- mockery::mock(mock_response)
  
  # Use with_mocked_bindings and explicitly bind curl::curl_fetch_memory
  with_mocked_bindings(
    `curl_fetch_memory` = mock_fetch,
    {
      result <- parse_package_info("mockpackage")
      
      # Check that the function returns the mocked content
      expect_type(result, "character")
      expect_match(result, "Mock Content")
    },
    .package = "curl"
  )
})

test_that("parse_package_info handles non-successful response", {
  # Create a mock response object for a 404 status
  mock_response <- list(
    status_code = 404,
    content = charToRaw("")
  )
  
  # Mock curl::curl_fetch_memory
  mock_fetch <- mockery::mock(mock_response)
  
  # Use with_mocked_bindings and explicitly bind curl::curl_fetch_memory
  with_mocked_bindings(
    `curl_fetch_memory` = mock_fetch,
    {
      result <- parse_package_info("nonexistentpackage")
      
      # Check that the function returns NULL for non-successful response
      expect_null(result)
    },
    .package = "curl"
  )
})


# check_cran_package


test_that("check_cran_package returns TRUE for a valid package", {
  # Create a mock response object for a valid package
  mock_response <- list(
    status_code = 200,
    content = charToRaw("<html><body>Package ‘mockpackage’ is available.</body></html>")
  )
  
  # Mock curl::curl_fetch_memory
  mock_fetch <- mockery::mock(mock_response)
  
  # Use with_mocked_bindings
  with_mocked_bindings(
    `curl_fetch_memory` = mock_fetch,
    {
      result <- check_cran_package("mockpackage")
      
      # Check that the function returns TRUE for a valid package
      expect_true(result)
    },
    .package = "curl"
  )
})

test_that("check_cran_package returns FALSE for a removed package", {
  # Create a mock response object for a removed package
  mock_response <- list(
    status_code = 200,
    content = charToRaw("<html><body>Package ‘mockpackage’ was removed from the CRAN repository.</body></html>")
  )
  
  # Mock curl::curl_fetch_memory
  mock_fetch <- mockery::mock(mock_response)
  
  # Use local_mocked_bindings instead of with_mocked_bindings (newer approach)
  local_mocked_bindings(curl_fetch_memory = mock_fetch, .package = "curl")
  
  # Call function
  result <- check_cran_package("mockpackage")
  
  # Expect FALSE because the package was removed
  expect_false(result)
})


test_that("check_cran_package returns FALSE for a package containing a point ", {
  # Create a mock response object for a removed package
  mock_response <- list(
    status_code = 200,
    content = charToRaw("<!DOCTYPE html>\n<html>\n<head>\n<title>CRAN: Package assertive.base</title>\n<link rel=\"canonical\" href=\"https://CRAN.R-project.org/package=assertive.base\"/>\n<link rel=\"stylesheet\" type=\"text/css\" href=\"../../CRAN_web.css\"/>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>\n<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=yes\"/>\n</head>\n<body>\n<div class=\"container\">\n<p>Package &lsquo;assertive.base&rsquo; was removed from the CRAN repository.</p>\n<p>\nFormerly available versions can be obtained from the\n<a href=\"https://CRAN.R-project.org/src/contrib/Archive/assertive.base\"><span class=\"CRAN\">archive</span></a>.\n</p>\n<p>\nArchived on 2024-04-12 as issues were not corrected despite reminders.\n</p>\n<p>A summary of the most recent check results can be obtained from the <a href=\"https://cran-archive.R-project.org/web/checks/2024/2024-04-12_check_results_assertive.base.html\"><span class=\"CRAN\">check results archive</span></a>.</p>\n<p>Please use the canonical form\n<a href=\"https://CRAN.R-project.org/package=assertive.base\"><span class=\"CRAN\"><samp>https://CRAN.R-project.org/package=assertive.base</samp></span></a>\nto link to this page.</p>\n</div>\n</body>\n</html>\n")
  )
  
  # Mock curl::curl_fetch_memory
  mock_fetch <- mockery::mock(mock_response)
  local_mocked_bindings(curl_fetch_memory = mock_fetch, .package = "curl")
  result <- check_cran_package("assertive.base")
  
  # Expect FALSE because the package was removed
  expect_false(result)
})




test_that("check_cran_package returns FALSE for a non-existent package", {
  # Create a mock response object for a 404 status
  mock_response <- list(
    status_code = 404,
    content = charToRaw("")
  )
  
  # Mock curl::curl_fetch_memory
  mock_fetch <- mockery::mock(mock_response)
  
  # Use with_mocked_bindings
  with_mocked_bindings(
    `curl_fetch_memory` = mock_fetch,
    {
      result <- check_cran_package("nonexistentpackage")
      
      # Check that the function returns FALSE for a non-existent package
      expect_false(result)
    },
    .package = "curl"
  )
})


# get_cran_package_url

test_that("get_cran_package_url returns URL for the latest version when version is NULL", {
  last_version <- list(version = "1.1.0", date = "2024-05-01")
  all_versions <- list(
    list(version = "1.0.0", date = "2023-01-01"),
    list(version = "1.1.0", date = "2024-05-01")
  )
  result <- get_cran_package_url("mockpackage", NULL, last_version, all_versions)
  expect_equal(result, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
})

test_that("get_cran_package_url returns URL for the latest version when explicitly provided", {
  last_version <- list(version = "1.1.0", date = "2024-05-01")
  all_versions <- list(
    list(version = "1.0.0", date = "2023-01-01"),
    list(version = "1.1.0", date = "2024-05-01")
  )
  result <- get_cran_package_url("mockpackage", "1.1.0", last_version, all_versions)
  expect_equal(result, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
})

test_that("get_cran_package_url returns URL for a specific older version", {
  last_version <- list(version = "1.1.0", date = "2024-05-01")
  all_versions <- list(
    list(version = "1.0.0", date = "2023-01-01"),
    list(version = "1.1.0", date = "2024-05-01")
  )
  result <- get_cran_package_url("mockpackage", "1.0.0", last_version, all_versions)
  expect_equal(result, "https://cran.r-project.org/src/contrib/Archive/mockpackage/mockpackage_1.0.0.tar.gz")
})

test_that("get_cran_package_url returns fallback message for a version that doesn't exist", {
  last_version <- list(version = "1.1.0", date = "2024-05-01")
  all_versions <- list(
    list(version = "1.0.0", date = "2023-01-01"),
    list(version = "1.1.0", date = "2024-05-01")
  )
  result <- get_cran_package_url("mockpackage", "2.0.0", last_version, all_versions)
  expect_equal(result, "No valid URL found")
})



# Test cases: get_versions

test_that("get_versions with empty table returns last version correctly", {
  # Mock input table
  mock_table <- NULL
  
  # Mock package name
  package_name <- "mockpackage"
  
  # Mock curl_fetch_memory response with version and date_publication
  mock_response <- list(
    status_code = 200,
    content = charToRaw('{"version": "1.2.0", "date_publication": "2024-05-01T00:00:00Z"}')
  )
  
  # Create a mock function for curl_fetch_memory
  mock_curl_fetch <- mockery::mock(mock_response)
  
  # Use with_mocked_bindings to override curl_fetch_memory in the curl package
  with_mocked_bindings(
    curl_fetch_memory = mock_curl_fetch,
    {
      result <- get_versions(mock_table, package_name)
      
      expect_equal(result$all_versions, list(list(version = "1.2.0", date = "2024-05-01")))
      expect_equal(result$last_version, list(version = "1.2.0", date = "2024-05-01"))
    },
    .package = "curl"
  )
})


test_that("get_versions extracts and combines versions correctly", {
  mock_table <- list(
    list(package_version = "1.0.0", date = "2022-01-01"),
    list(package_version = "1.1.0", date = "2023-01-01")
  )
  
  package_name <- "mockpackage"
  
  mock_response <- list(
    status_code = 200,
    content = charToRaw('{"version": "1.2.0", "date_publication": "2024-05-01T00:00:00Z"}')
  )
  mock_curl_fetch <- mockery::mock(mock_response)
  
  with_mocked_bindings(
    curl_fetch_memory = mock_curl_fetch,
    {
      result <- get_versions(mock_table, package_name)
      
      expect_equal(result$all_versions, list(
        list(version = "1.0.0", date = "2022-01-01"),
        list(version = "1.1.0", date = "2023-01-01"),
        list(version = "1.2.0", date = "2024-05-01")
      ))
      
      expect_equal(result$last_version, list(
        version = "1.2.0",
        date = "2024-05-01"
      ))
    },
    .package = "curl"
  )
})


test_that("get_versions handles invalid API response", {
  mock_table <- list(
    list(package_version = "1.0.0", date = "2022-01-01"),
    list(package_version = "1.1.0", date = "2023-01-01")
  )
  
  package_name <- "mockpackage"
  
  mock_response <- list(
    status_code = 500,
    content = charToRaw("")
  )
  mock_curl_fetch <- mockery::mock(mock_response)
  
  with_mocked_bindings(
    curl_fetch_memory = mock_curl_fetch,
    {
      result <- get_versions(mock_table, package_name)
      
      expect_equal(result$all_versions, list(
        list(version = "1.0.0", date = "2022-01-01"),
        list(version = "1.1.0", date = "2023-01-01")
      ))
      expect_null(result$last_version)
    },
    .package = "curl"
  )
})




test_that("get_versions removes duplicate versions", {
  mock_table <- list(
    list(package_version = "1.0.0", date = "2022-01-01"),
    list(package_version = "1.0.0", date = "2022-01-01"),
    list(package_version = "1.1.0", date = "2023-01-01")
  )
  
  package_name <- "mockpackage"
  
  mock_response <- list(
    status_code = 200,
    content = charToRaw('{"version": "1.1.0", "date_publication": "2023-01-01T00:00:00Z"}')
  )
  mock_curl_fetch <- mockery::mock(mock_response)
  
  with_mocked_bindings(
    curl_fetch_memory = mock_curl_fetch,
    {
      result <- get_versions(mock_table, package_name)
      
      expect_equal(result$all_versions, list(
        list(version = "1.0.0", date = "2022-01-01"),
        list(version = "1.0.0", date = "2022-01-01"),
        list(version = "1.1.0", date = "2023-01-01")
      ))
      
      expect_equal(result$last_version, list(
        version = "1.1.0",
        date = "2023-01-01"
      ))
    },
    .package = "curl"
  )
})

test_that("get_versions handles missing API version", {
  mock_table <- list(
    list(package_version = "1.0.0", date = "2022-01-01"),
    list(package_version = "1.1.0", date = "2023-01-01")
  )
  
  package_name <- "mockpackage"
  
  mock_response <- list(
    status_code = 200,
    content = charToRaw('{}')  # No version or date_publication
  )
  mock_curl_fetch <- mockery::mock(mock_response)
  
  with_mocked_bindings(
    curl_fetch_memory = mock_curl_fetch,
    {
      result <- get_versions(mock_table, package_name)
      
      expect_equal(result$all_versions, list(
        list(version = "1.0.0", date = "2022-01-01"),
        list(version = "1.1.0", date = "2023-01-01")
      ))
      expect_null(result$last_version$version)
    },
    .package = "curl"
  )
})



# Test cases: check_and_fetch_cran_package


test_that("check_and_fetch_cran_package fetches the correct URL for the latest version", {
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock HTML Content</html>")
  mock_parse_html_version <- mockery::mock(
    list(
      list(package_version = "1.0.0", date = "2022-01-01"),
      list(package_version = "1.1.0", date = "2023-01-01")
    )
  )
  mock_get_versions <- mockery::mock(list(
    all_versions = list(
      list(version = "1.0.0", date = "2022-01-01"),
      list(version = "1.1.0", date = "2023-01-01"),
      list(version = "1.2.0", date = "2024-01-01")
    ),
    last_version = list(version = "1.2.0", date = "2024-01-01")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.2.0.tar.gz")
  
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    get_versions = mock_get_versions,
    get_cran_package_url = mock_get_cran_package_url,
    {
      result <- check_and_fetch_cran_package("mockpackage")
      
      expect_equal(result$package_url, "https://cran.r-project.org/src/contrib/mockpackage_1.2.0.tar.gz")
      expect_equal(result$last_version$version, "1.2.0")
      expect_equal(result$last_version$date, "2024-01-01")
      expect_null(result$version)
      expect_equal(
        result$all_versions,
        list(
          list(version = "1.0.0", date = "2022-01-01"),
          list(version = "1.1.0", date = "2023-01-01"),
          list(version = "1.2.0", date = "2024-01-01")
        )
      )
    }
  )
})

test_that("check_and_fetch_cran_package handles specific version correctly", {
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock HTML Content</html>")
  mock_parse_html_version <- mockery::mock(
    list(
      list(package_version = "1.0.0", date = "2022-01-01"),
      list(package_version = "1.1.0", date = "2023-01-01")
    )
  )
  mock_get_versions <- mockery::mock(list(
    all_versions = list(
      list(version = "1.0.0", date = "2022-01-01"),
      list(version = "1.1.0", date = "2023-01-01")
    ),
    last_version = list(version = "1.1.0", date = "2023-01-01")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/Archive/mockpackage/mockpackage_1.0.0.tar.gz")
  
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    get_versions = mock_get_versions,
    get_cran_package_url = mock_get_cran_package_url,
    {
      result <- check_and_fetch_cran_package("mockpackage", "1.0.0")
      
      expect_equal(result$package_url, "https://cran.r-project.org/src/contrib/Archive/mockpackage/mockpackage_1.0.0.tar.gz")
      expect_equal(result$version, "1.0.0")
      expect_equal(result$last_version$version, "1.1.0")
      expect_equal(result$last_version$date, "2023-01-01")
      expect_equal(
        result$all_versions,
        list(
          list(version = "1.0.0", date = "2022-01-01"),
          list(version = "1.1.0", date = "2023-01-01")
        )
      )
    }
  )
})

test_that("check_and_fetch_cran_package handles missing version", {
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock HTML Content</html>")
  mock_parse_html_version <- mockery::mock(
    list(
      list(package_version = "1.0.0", date = "2022-01-01"),
      list(package_version = "1.1.0", date = "2023-01-01")
    )
  )
  mock_get_versions <- mockery::mock(list(
    all_versions = list(
      list(version = "1.0.0", date = "2022-01-01"),
      list(version = "1.1.0", date = "2023-01-01")
    ),
    last_version = list(version = "1.1.0", date = "2023-01-01")
  ))
  mock_get_cran_package_url <- mockery::mock(NULL)
  
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    get_versions = mock_get_versions,
    get_cran_package_url = mock_get_cran_package_url,
    {
      result <- check_and_fetch_cran_package("mockpackage", "2.0.0")
      
      expect_equal(result$error, "Version 2.0.0 for mockpackage not found")
      expect_equal(
        result$versions_available,
        list(
          list(version = "1.0.0", date = "2022-01-01"),
          list(version = "1.1.0", date = "2023-01-01")
        )
      )
    }
  )
})

test_that("check_and_fetch_cran_package with version table null", {
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock(NULL)
  mock_parse_html_version <- mockery::mock(NULL)
  
  mock_get_versions <- mockery::mock(list(
    all_versions = list(
      list(version = "1.2.0", date = "2024-01-01")
    ),
    last_version = list(version = "1.2.0", date = "2024-01-01")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.2.0.tar.gz")
  
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    get_versions = mock_get_versions,
    get_cran_package_url = mock_get_cran_package_url,
    {
      result <- check_and_fetch_cran_package("mockpackage")
      
      expect_equal(result$package_url, "https://cran.r-project.org/src/contrib/mockpackage_1.2.0.tar.gz")
      expect_equal(result$last_version$version, "1.2.0")
      expect_equal(result$last_version$date, "2024-01-01")
      expect_null(result$version)
      expect_equal(
        result$all_versions,
        list(
          list(version = "1.2.0", date = "2024-01-01")
        )
      )
    }
  )
})


# get_internal_package_url


# test-get_internal_package_url.R

test_that("check_and_fetch_cran_package with version table null", {
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock(NULL)
  mock_parse_html_version <- mockery::mock(NULL, cycle = TRUE)
  mock_get_versions <- mockery::mock(list(
    all_versions = list(list(version = "1.2.0", date = "2024-01-01")),
    last_version = list(version = "1.2.0", date = "2024-01-01")
  ), cycle = TRUE)
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.2.0.tar.gz")
  
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    get_versions = mock_get_versions,
    get_cran_package_url = mock_get_cran_package_url,
    {
      result <- check_and_fetch_cran_package("mockpackage")
      
      expect_equal(result$package_url, "https://cran.r-project.org/src/contrib/mockpackage_1.2.0.tar.gz")
      expect_equal(result$last_version$version, "1.2.0")
      expect_equal(result$last_version$date, "2024-01-01")
      expect_null(result$version)
      expect_equal(result$all_versions, list(list(version = "1.2.0", date = "2024-01-01")))
    }
  )
})

# get_internal_package_url

test_that("get_internal_package_url works correctly for latest version", {
  # Mock the repos API response
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  # Mock the package API response
  mock_package_response <- list(
    content = charToRaw(enc2utf8('{
      "version": "1.2.0",
      "date_publication": "2024-01-01T12:00:00+01:00",
      "archived": [{"version": "1.0.0", "date": "2023-01-01T00:00:00+01:00"}, {"version": "1.1.0", "date": "2023-06-01T00:00:00+01:00"}]
    }')),
    status_code = 200
  )
  
  # Create a mock that returns different responses for different URLs
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,  # First call - for repos list
    mock_package_response # Second call - for package info
  )
  
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  
  result <- get_internal_package_url("mockpackage", base_url="https://rstudio-pm.com")
  
  # Verify the mock was called with the correct URLs
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  # Verify the result
  expect_equal(result$url, "https://rstudio-pm.com/art-git/latest/src/contrib/mockpackage_1.2.0.tar.gz")
  expect_equal(result$last_version$version, "1.2.0")
  expect_equal(result$all_versions, list(
    list(version = "1.2.0", date = as.Date("2024-01-01")),
    list(version = "1.0.0", date = as.Date("2023-01-01")),
    list(version = "1.1.0", date = as.Date("2023-06-01"))
  ))
  expect_equal(result$repo_id, c("art-git" = 6))
  expect_equal(result$repo_name, "art-git")
})

test_that("get_internal_package_url works correctly for latest version", {

  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  mock_package_response <- list(
    content = charToRaw(enc2utf8('{
      "version": "1.2.0",
      "date_publication": "2024-01-01T12:00:00+01:00",
      "archived": [{"version": "1.0.0", "date": "2023-01-01T00:00:00+01:00"}, {"version": "1.1.0", "date": "2023-06-01T00:00:00+01:00"}]
    }')),
    status_code = 200
  )
  
   mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,  
    mock_package_response 
  )
  
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  
  result <- get_internal_package_url("mockpackage", base_url="https://rstudio-pm.com")
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  expect_equal(result$url, "https://rstudio-pm.com/art-git/latest/src/contrib/mockpackage_1.2.0.tar.gz")
  expect_equal(result$last_version$version, "1.2.0")
  expect_equal(result$all_versions, list(
    list(version = "1.2.0", date = as.Date("2024-01-01")),
    list(version = "1.0.0", date = as.Date("2023-01-01")),
    list(version = "1.1.0", date = as.Date("2023-06-01"))
  ))
  expect_equal(result$repo_id, c("art-git" = 6))
  expect_equal(result$repo_name, "art-git")
})

test_that("get_internal_package_url works correctly for archived version", {
  
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  mock_package_response <- list(
    content = charToRaw(enc2utf8('{
      "version": "1.2.0",
      "date_publication": "2024-01-01T12:00:00+01:00",
      "archived": [{"version": "1.0.0", "date": "2023-01-01T00:00:00+01:00"}, {"version": "1.1.0", "date": "2023-06-01T00:00:00+01:00"}]
    }')),
    status_code = 200
  )
  
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,  
    mock_package_response 
  )
  
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  
  result <- get_internal_package_url("mockpackage", "1.0.0", base_url="https://rstudio-pm.com")
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  expect_equal(result$url, "https://rstudio-pm.com/art-git/latest/src/contrib/Archive/mockpackage/mockpackage_1.0.0.tar.gz")
  expect_equal(result$last_version$version, "1.2.0")
  expect_equal(result$all_versions, list(
    list(version = "1.2.0", date = as.Date("2024-01-01")),
    list(version = "1.0.0", date = as.Date("2023-01-01")),
    list(version = "1.1.0", date = as.Date("2023-06-01"))
  ))
  expect_equal(result$repo_id, c("art-git" = 6))
  expect_equal(result$repo_name, "art-git")
})


test_that("get_internal_package_url returns NULLs when no last version exists", {
  
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  # Mock the package API response with no current version
  mock_package_response <- list(
    content = charToRaw(enc2utf8('{
      "archived": [{"version": "1.0.0", "date": "2023-01-01"}, {"version": "1.1.0", "date": "2023-06-01"}]
    }')),
    status_code = 200
  )
  
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,  
    mock_package_response 
  )
  
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  result <- get_internal_package_url("mockpackage", "2.0.0", base_url="https://rstudio-pm.com")
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
  expect_null(result$repo_id)
  expect_null(result$repo_name)
})

test_that("get_internal_package_url returns NULLs when no last version exists and no version params", {
  mock_response <- list(
    content = charToRaw(enc2utf8('{
      "archived": [{"version": "1.0.0", "date": "2023-01-01"}, {"version": "1.1.0", "date": "2023-06-01"}]
    }')),
    status_code = 200
  )

  mock_curl_fetch_memory <- mockery::mock(mock_response)
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  result <- get_internal_package_url("mockpackage", base_url="https://rstudio-pm.com")
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
})

test_that("get_internal_package_url works correctly when no archived versions exist", {
  
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  mock_package_response <- list(
    content = charToRaw(enc2utf8('{
      "version": "1.2.0",
      "date_publication": "2024-01-01T12:00:00+01:00"
    }')),
    status_code = 200
  )
  
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,  
    mock_package_response 
  )
  
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  result <- get_internal_package_url("mockpackage", "1.2.0", base_url="https://rstudio-pm.com")
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  # Verify the result
  expect_equal(result$url, "https://rstudio-pm.com/art-git/latest/src/contrib/mockpackage_1.2.0.tar.gz")
  expect_equal(result$last_version$version, "1.2.0")
  expect_equal(result$all_versions, list(
    list(version = "1.2.0", date = as.Date("2024-01-01"))
  ))
  expect_equal(result$repo_id, c("art-git" = 6))
  expect_equal(result$repo_name, "art-git")
})


test_that("get_internal_package_url handles empty API response", {
  mock_response <- list(
    content = raw(0),
    status_code = 200
  )

  mock_curl_fetch_memory <- mockery::mock(mock_response)
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")

  result <- get_internal_package_url("mockpackage")

  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
})

test_that("get_internal_package_url handles invalid JSON response", {
  mock_response <- list(
    content = charToRaw("Invalid JSON"),
    status_code = 200
  )

  mock_curl_fetch_memory <- mockery::mock(mock_response)
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")

  result <- get_internal_package_url("mockpackage")

  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
})

test_that("get_internal_package_url handles 404 (package not found)", {
  mock_response <- list(
    content = charToRaw("Not Found"),
    status_code = 404
  )

  mock_curl_fetch_memory <- mockery::mock(mock_response)
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")

  result <- get_internal_package_url("nonexistentpackage")

  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
})


test_that("get_internal_package_url handles NULL base_url and missing INTERNAL_RSPM option", {
  
  old_repos <- options("repos")
  on.exit(options(repos = old_repos$repos))
  
  options(repos = NULL)
  
  expect_message(
    result <- get_internal_package_url("mockpackage"),
    "NO INTERNAL_RSPM FOUND"
  )
  
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
  expect_null(result$repo_id)
  expect_null(result$repo_name)
})

test_that("get_internal_package_url handles repos without INTERNAL_RSPM", {
  
  old_repos <- options("repos")
  on.exit(options(repos = old_repos$repos))
  
  # Set repos without INTERNAL_RSPM
  options(repos = c(CRAN = "https://cloud.r-project.org"))
  
  expect_message(
    result <- get_internal_package_url("mockpackage"),
    "NO INTERNAL_RSPM FOUND"
  )
  
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
  expect_null(result$repo_id)
  expect_null(result$repo_name)
})

test_that("get_internal_package_url handles failure in first API call", {

  mock_curl_fetch_memory <- mockery::mock(NULL)
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  
  result <- get_internal_package_url("mockpackage", base_url="https://rstudio-pm.com")
  
  # Verify the mock was called
  expect_equal(length(mockery::mock_args(mock_curl_fetch_memory)), 1)
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  
  # Verify the result
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
  expect_null(result$repo_id)
  expect_null(result$repo_name)
})

test_that("get_internal_package_url handles failure in second API call", {
  # Mock the repos API response
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,  
    NULL                  
  )
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  
  result <- get_internal_package_url("mockpackage", base_url="https://rstudio-pm.com")
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
  expect_null(result$repo_id)
  expect_null(result$repo_name)
})


test_that("get_internal_package_url handles failure in all package API calls", {
  # Mock the repos API response
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  # Mock package API responses (all fail)
  mock_package_response_fail <- list(
    content = charToRaw("Not Found"),
    status_code = 404
  )
  
  # Mock curl_fetch_memory to succeed on first call but return 404 on subsequent calls
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,        # First call succeeds
    mock_package_response_fail, # First package call fails
    mock_package_response_fail  # Second package call fails
  )
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  
  result <- get_internal_package_url("mockpackage", base_url="https://rstudio-pm.com")
  
  # Verify the mock was called three times
  expect_equal(length(mockery::mock_args(mock_curl_fetch_memory)), 3)
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]], 
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]], 
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[3]], 
    list("https://rstudio-pm.com/__api__/repos/1/packages/mockpackage")
  )
  
  # Verify the result
  expect_null(result$url)
  expect_null(result$last_version)
  expect_equal(result$all_versions, list())
  expect_null(result$repo_id)
  expect_null(result$repo_name)
})

test_that("get_internal_package_url handles archived as list (not data.frame)", {
  # JSON with inconsistent archived structure so jsonlite parses as list, not data.frame
  # Covers: } else if (is.list(found_data$archived)) { for (av in found_data$archived) ...
  mock_repos_response <- list(
    content = charToRaw(enc2utf8('[
      {"id": 6, "name": "art-git"},
      {"id": 1, "name": "prod-cran"}
    ]')),
    status_code = 200
  )
  
  # Archived array: first element has date, second has notes - different keys => list, not data.frame
  mock_package_response <- list(
    content = charToRaw(enc2utf8('{
      "version": "1.2.0",
      "date_publication": "2024-01-01T12:00:00+01:00",
      "archived": [
        {"version": "1.0.0", "date": "2023-01-01"},
        {"version": "1.1.0", "date": "2023-06-01"}
      ]
    }')),
    status_code = 200
  )
  
  # Use simplifyDataFrame=FALSE for package response so archived is a list
  original_fromJSON <- jsonlite::fromJSON
  fromJSON_calls <- 0L
  mock_fromJSON <- function(txt, ...) {
    fromJSON_calls <<- fromJSON_calls + 1L
    if (fromJSON_calls == 2L && inherits(txt, "character") && grepl("archived", txt, fixed = TRUE)) {
      return(original_fromJSON(txt, simplifyDataFrame = FALSE, ...))
    }
    return(original_fromJSON(txt, ...))
  }
  
  mock_curl_fetch_memory <- mockery::mock(
    mock_repos_response,
    mock_package_response
  )
  
  local_mocked_bindings(curl_fetch_memory = mock_curl_fetch_memory, .package = "curl")
  local_mocked_bindings(fromJSON = mock_fromJSON, .package = "jsonlite")
  
  result <- get_internal_package_url("mockpackage", "1.0.0", base_url = "https://rstudio-pm.com")
  
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[1]],
    list("https://rstudio-pm.com/__api__/repos/")
  )
  expect_equal(
    mockery::mock_args(mock_curl_fetch_memory)[[2]],
    list("https://rstudio-pm.com/__api__/repos/6/packages/mockpackage")
  )
  
  expect_equal(result$url, "https://rstudio-pm.com/art-git/latest/src/contrib/Archive/mockpackage/mockpackage_1.0.0.tar.gz")
  expect_equal(result$last_version$version, "1.2.0")
  expect_equal(result$all_versions, list(
    list(version = "1.2.0", date = as.Date("2024-01-01")),
    list(version = "1.0.0", date = as.Date("2023-01-01")),
    list(version = "1.1.0", date = as.Date("2023-06-01"))
  ))
  expect_equal(result$repo_id, c("art-git" = 6))
  expect_equal(result$repo_name, "art-git")
})

# get_host_package

test_that("get_host_package returns correct links for valid inputs", {
  # Mock description$new
  mock_description <- mockery::mock(list(
    str = function() "Mocked DESCRIPTION content",
    get_urls = function() c("https://github.com/owner1/mockpackage"),
    has_fields = function(field) field == "BugReports",
    get_list = function(field) {
      if (field == "BugReports") "https://github.com/owner1/mockpackage/issues"
    }
  ))

  # Stub description$new
  mockery::stub(get_host_package, "description$new", mock_description)

  # Mock other dependencies
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock Archive Content</html>")
  mock_parse_html_version <- mockery::mock(list(
    list(package_version = "1.0.0"),
    list(package_version = "1.1.0")
  ),
  cycle = TRUE
  )

  mock_check_and_fetch_cran_package <- mockery::mock(list(
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
  mock_get_internal_package_url <- mockery::mock(list(
    url = "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz",
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))

  # Use local_mocked_bindings for other dependencies
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    check_and_fetch_cran_package = mock_check_and_fetch_cran_package,
    get_cran_package_url = mock_get_cran_package_url,
    get_internal_package_url = mock_get_internal_package_url,
    {
      # Call the function with mocked dependencies
      result <- get_host_package("mockpackage", "1.1.0", "/path/to/source")

      # Validate the result
      expect_equal(result$github_links, "https://github.com/owner1/mockpackage")
      expect_equal(result$cran_links, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
      expect_equal(result$internal_links, "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz")
      expect_null(result$bioconductor_links)
    }
  )
})

test_that("get_host_package with github name and CRAN name different", {
  # Mock description$new
  mock_description <- mockery::mock(list(
    str = function() "Mocked DESCRIPTION content",
    get_urls = function() c("https://github.com/ow-ner1/mockpackage_no_cran"),
    has_fields = function(field) field == "BugReports",
    get_list = function(field) {
      if (field == "BugReports") "https://github.com/ow-ner1/mockpackage_no_cran/issues"
    }
  ))

  # Stub description$new
  mockery::stub(get_host_package, "description$new", mock_description)

  # Mock other dependencies
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock Archive Content</html>")
  mock_parse_html_version <- mockery::mock(list(
    list(package_version = "1.0.0"),
    list(package_version = "1.1.0")
  ),
  cycle = TRUE
  )

  mock_check_and_fetch_cran_package <- mockery::mock(list(
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
  mock_get_internal_package_url <- mockery::mock(list(
    url = "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz",
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))

  # Use local_mocked_bindings for other dependencies
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    check_and_fetch_cran_package = mock_check_and_fetch_cran_package,
    get_cran_package_url = mock_get_cran_package_url,
    get_internal_package_url = mock_get_internal_package_url,
    {
      # Call the function with mocked dependencies
      result <- get_host_package("mockpackage", "1.1.0", "/path/to/source")

      # Validate the result
      expect_equal(result$github_links, "https://github.com/ow-ner1/mockpackage_no_cran")
      expect_equal(result$cran_links, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
      expect_equal(result$internal_links, "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz")
      expect_null(result$bioconductor_links)
    }
  )
})


test_that("get_host_package with no github link, no bug report", {
  # Mock description$new
  mock_description <- mockery::mock(list(
    str = function() "Mocked DESCRIPTION content",
    get_urls = function() c(""),
    has_fields = function(field) field == "BugReports",
    get_list = function(field) {
      if (field == "BugReports") ""
    }
  ))

  # Stub description$new
  mockery::stub(get_host_package, "description$new", mock_description)

  # Mock other dependencies
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock Archive Content</html>")
  mock_parse_html_version <- mockery::mock(list(
    list(package_version = "1.0.0"),
    list(package_version = "1.1.0")
  ),
  cycle = TRUE
  )
  mock_check_and_fetch_cran_package <- mockery::mock(list(
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
  mock_get_internal_package_url <- mockery::mock(list(
    url = "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz",
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))

  # Use local_mocked_bindings for other dependencies
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    check_and_fetch_cran_package = mock_check_and_fetch_cran_package,
    get_cran_package_url = mock_get_cran_package_url,
    get_internal_package_url = mock_get_internal_package_url,
    {
      # Call the function with mocked dependencies
      result <- get_host_package("mockpackage", "1.1.0", "/path/to/source")

      # Validate the result
      expect_null(result$github_links)
      expect_equal(result$cran_links, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
      expect_equal(result$internal_links, "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz")
      expect_null(result$bioconductor_links)
    }
  )
})


test_that("get_host_package with github name, but not bug report", {
  # Mock description$new
  mock_description <- mockery::mock(list(
    str = function() "Mocked DESCRIPTION content",
    get_urls = function() c(""),
    has_fields = function(field) field == "BugReports",
    get_list = function(field) {
      if (field == "BugReports") "https://github.com/ow-ner1/mockpackage_no_cran/issues"
    }
  ))

  # Stub description$new
  mockery::stub(get_host_package, "description$new", mock_description)

  # Mock other dependencies
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock Archive Content</html>")
  mock_parse_html_version <- mockery::mock(list(
    list(package_version = "1.0.0"),
    list(package_version = "1.1.0")
  ),
  cycle = TRUE
  )

  mock_check_and_fetch_cran_package <- mockery::mock(list(
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))

  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
  mock_get_internal_package_url <- mockery::mock(list(
    url = "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz",
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))

  # Use local_mocked_bindings for other dependencies
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    check_and_fetch_cran_package = mock_check_and_fetch_cran_package,
    get_cran_package_url = mock_get_cran_package_url,
    get_internal_package_url = mock_get_internal_package_url,
    {
      # Call the function with mocked dependencies
      result <- get_host_package("mockpackage", "1.1.0", "/path/to/source")

      # Validate the result
      expect_equal(result$github_links, "https://github.com/ow-ner1/mockpackage_no_cran")
      expect_equal(result$cran_links, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
      expect_equal(result$internal_links, "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz")
      expect_null(result$bioconductor_links)
    }
  )
})


test_that("get_host_package with no github name, but bug report", {
  # Mock description$new
  mock_description <- mockery::mock(list(
    str = function() "Mocked DESCRIPTION content",
    get_urls = function() c("https://github.com/ow-ner1/mockpackage_no_cran"),
    has_fields = function(field) field == "BugReports",
    get_list = function(field) {
      if (field == "BugReports") ""
    }
  ))

  # Stub description$new
  mockery::stub(get_host_package, "description$new", mock_description)

  # Mock other dependencies
  mock_check_cran_package <- mockery::mock(TRUE)
  mock_parse_package_info <- mockery::mock("<html>Mock Archive Content</html>")
  mock_parse_html_version <- mockery::mock(list(
    list(package_version = "1.0.0"),
    list(package_version = "1.1.0")
  ),
  cycle = TRUE
  )
  mock_check_and_fetch_cran_package <- mockery::mock(list(
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))
  mock_get_cran_package_url <- mockery::mock("https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
  mock_get_internal_package_url <- mockery::mock(list(
    url = "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz",
    last_version = "1.1.0",
    all_versions = c("1.0.0", "1.1.0")
  ))

  # Use local_mocked_bindings for other dependencies
  with_mocked_bindings(
    check_cran_package = mock_check_cran_package,
    parse_package_info = mock_parse_package_info,
    parse_html_version = mock_parse_html_version,
    check_and_fetch_cran_package = mock_check_and_fetch_cran_package,
    get_cran_package_url = mock_get_cran_package_url,
    get_internal_package_url = mock_get_internal_package_url,
    {
      # Call the function with mocked dependencies
      result <- get_host_package("mockpackage", "1.1.0", "/path/to/source")

      # Validate the result
      expect_equal(result$github_links, "https://github.com/ow-ner1/mockpackage_no_cran")
      expect_equal(result$cran_links, "https://cran.r-project.org/src/contrib/mockpackage_1.1.0.tar.gz")
      expect_equal(result$internal_links, "https://rstudio-pm.prod.example.com/mockpackage_1.1.0.tar.gz")
      expect_null(result$bioconductor_links)
    }
  )
})

test_that("get_package_tarfile works with system test package via mocked download", {
  # Get system file - same pattern as test-install_package_local
  dp_orig <- system.file("test-data",
                         "test.package.0001_0.1.0.tar.gz",
                         package = "risk.assessr")
  
  testthat::skip_if_not(
    file.exists(dp_orig),
    "test.package.0001_0.1.0.tar.gz not found in test-data"
  )
  
  # Copy test package to a temp file
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Mock download.file to copy from our test package instead of downloading
  mockery::stub(get_package_tarfile, "check_cran_package", TRUE)
  mockery::stub(get_package_tarfile, "check_and_fetch_cran_package", function(package_name, package_version = NULL) {
    list(package_url = "https://cran.r-project.org/src/contrib/test.package.0001_0.1.0.tar.gz")
  })
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, ...) {
    file.copy(dp, destfile, overwrite = TRUE)
    0
  })
  
  path <- get_package_tarfile("test.package.0001", version = "0.1.0")
  
  # Defer cleanup of returned tarball
  withr::defer(unlink(path), envir = parent.frame())
  
  expect_true(file.exists(path))
  expect_match(path, "tar\\.gz$")
  # Verify content matches our test package
  expect_equal(file.size(path), file.size(dp))
})

test_that("get_package_tarfile passes version to internal fallback", {
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) list(url = NULL))
  
  captured_version <- NULL
  mockery::stub(get_package_tarfile, "get_internal_package_url", function(package_name, version = NULL, base_url = NULL) {
    captured_version <<- version
    list(
      url = "https://internal.example/repo/src/contrib/VersionedPkg_1.2.3.tar.gz",
      last_version = list(version = "1.2.3", date = as.Date("2024-01-01")),
      all_versions = list(list(version = "1.2.3", date = as.Date("2024-01-01"))),
      repo_id = "1", repo_name = "internal-repo"
    )
  })
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, mode = "wb", quiet = TRUE) {
    make_tarball(destfile, pkg_name = "versionedpkg", version = "1.2.3")
    0
  })
  
  path <- get_package_tarfile("VersionedPkg", version = "1.2.3")
  withr::defer(unlink(path), envir = parent.frame())
  
  expect_true(file.exists(path))
  expect_identical(captured_version, "1.2.3")
})

test_that("Bioconductor branch uses system test package when result_bio$url is not null", {
  # Get system file - same pattern as test-install_package_local
  dp_orig <- system.file("test-data",
                         "test.package.0001_0.1.0.tar.gz",
                         package = "risk.assessr")
  
  testthat::skip_if_not(
    file.exists(dp_orig),
    "test.package.0001_0.1.0.tar.gz not found in test-data"
  )
  
  # Copy test package to a temp file
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Exercise Bioconductor branch: check_cran_package FALSE, result_bio$url not null
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html_content")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) {
    list(url = "http://dummybioc.org/test.package.0001_0.1.0.tar.gz")
  })
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, ...) {
    file.copy(dp, destfile, overwrite = TRUE)
    0
  })
  
  path <- get_package_tarfile("test.package.0001", version = "0.1.0")
  
  # Defer cleanup of returned tarball
  withr::defer(unlink(path), envir = parent.frame())
  
  expect_true(file.exists(path))
  expect_match(path, "tar\\.gz$")
  expect_equal(file.size(path), file.size(dp))
})

test_that("Bioconductor download error handler runs and falls through to internal mirror", {
  # result_bio$url is not null, but download.file throws
  # Covers: tryCatch error handler with message("Bioconductor download failed:", e$message)
  mockery::stub(get_package_tarfile, "check_cran_package", FALSE)
  mockery::stub(get_package_tarfile, "fetch_bioconductor_releases", function() "html_content")
  mockery::stub(get_package_tarfile, "parse_bioconductor_releases", function(html) "release_data")
  mockery::stub(get_package_tarfile, "get_bioconductor_package_url", function(...) {
    list(url = "http://dummybioc.org/pkg.tar")  # url not null - enters the if branch
  })
  
  # download.file throws - exercises error handler
  mockery::stub(get_package_tarfile, "download.file", function(url, destfile, ...) {
    if (grepl("dummybioc", url)) {
      stop("Bioconductor network timeout")
    }
    make_tarball(destfile, pkg_name = "internalpkg", version = "0.1")
    0
  })
  mockery::stub(get_package_tarfile, "get_internal_package_url", function(package_name, version = NULL, base_url = NULL) {
    list(
      url = "https://internal.example/repo/src/contrib/FallbackPkg_0.1.tar.gz",
      last_version = list(version = "0.1", date = as.Date("2023-01-01")),
      all_versions = list(list(version = "0.1", date = as.Date("2023-01-01"))),
      repo_id = "1", repo_name = "internal-repo"
    )
  })
  
  expect_message(
    path <- get_package_tarfile("BiocPkg"),
    "Bioconductor download failed"
  )
  
  withr::defer(unlink(path), envir = parent.frame())
  
  expect_true(file.exists(path))
})

#' Assess an R package by name (deprecated)
#'
#' Use [risk_assess_pkg()] with `package` instead.
#'
#' @param package_name Package name.
#' @param version Package version. Default `NA` uses latest.
#' @param repos Repository URLs. Default `getOption("repos")`.
#'
#' @return Assessment results or `NULL`.
#'
#' @export
assess_pkg_r_package <- function(package_name, version = NA, repos = getOption("repos")) {
  .Deprecated(
    "risk_assess_pkg",
    msg = "assess_pkg_r_package() is deprecated. Use risk_assess_pkg(package = ..., version = ..., repos = ...) instead."
  )
  risk_assess_pkg(package = package_name, version = version, repos = repos)
}

# assess_pkg_r_package is a deprecated wrapper around risk_assess_pkg.
# Tests verify it forwards arguments correctly and shows deprecation.

test_that("assess_pkg_r_package forwards to risk_assess_pkg and shows deprecation", {
  captured <- new.env(parent = emptyenv())
  mockery::stub(assess_pkg_r_package, "risk_assess_pkg", function(path = NULL, package = NULL, version = NA, repos = getOption("repos")) {
    captured$path <- path
    captured$package <- package
    captured$version <- version
    captured$repos <- repos
    "mock-result"
  })

  expect_warning(res <- assess_pkg_r_package("somepkg", version = "1.2.3", repos = c(CRAN = "https://cloud.r-project.org")), "deprecated")
  expect_equal(res, "mock-result")
  expect_null(captured$path)
  expect_equal(captured$package, "somepkg")
  expect_equal(captured$version, "1.2.3")
  expect_equal(captured$repos, c(CRAN = "https://cloud.r-project.org"))
})

test_that("assess_pkg_r_package passes defaults correctly", {
  captured <- new.env(parent = emptyenv())
  mockery::stub(assess_pkg_r_package, "risk_assess_pkg", function(path = NULL, package = NULL, version = NA, repos = getOption("repos")) {
    captured$package <- package
    captured$version <- version
    captured$repos <- repos
    NULL
  })

  expect_warning(assess_pkg_r_package("mypkg"), "deprecated")
  expect_equal(captured$package, "mypkg")
  expect_identical(captured$version, NA)
  expect_equal(captured$repos, getOption("repos"))
})

#' Assess Rd files for examples (exported functions only)
#'
#' Scans the package Rd database at `pkg_source_path` and reports whether each
#' **exported function** has examples. Resolution supports shared topics via
#' roxygen `@rdname` (multiple functions documented on the same Rd page) and
#' multiple `\\alias{}` entries.
#'
#' @param pkg_name Character scalar; package name (must be loadable so exports can be read).
#' @param pkg_source_path Directory path where Rd files can be found by `tools::Rd_db()`.
#'
#' @return A list with:
#' \describe{
#'   \item{data}{A data.frame with columns `function_name`, `documentation_name`,
#'               `documentation_location`, and `example`.}
#'   \item{example_score}{Numeric; percentage of exported functions with examples.}
#' }
#' @examples
#' \dontrun{
#'   res <- assess_examples("stringr", "<path-to-stringr-source>")
#'   head(res$data)
#'   res$example_score
#' }
#' @keywords internal
assess_examples <- function(pkg_name, pkg_source_path) {
  # Load Rd database for the source path
  db <- tools::Rd_db(dir = pkg_source_path)
  
  # Remove only the package-level Rd file (pkgname-package.Rd)
  db <- db[!names(db) %in% paste0(pkg_name, "-package.Rd")]
  
  # Namespace and exported objects
  ns <- asNamespace(pkg_name)
  exported <- getNamespaceExports(ns)
  
  # Keep only exported functions
  is_fun <- function(name) {
    obj <- try(getExportedValue(ns, name), silent = TRUE)
    is.function(obj)
  }
  exported_funs <- Filter(is_fun, exported)
  
  if (length(exported_funs) == 0) {
    df <- data.frame(
      function_name = character(0),
      documentation_name = character(0),
      documentation_location = character(0),
      example = character(0),
      stringsAsFactors = FALSE
    )
    example_score <- 0
    message(sprintf("%s: no exported functions found; example score = %.2f%%",
                    pkg_name, example_score))
    return(list(data = df, example_score = example_score))
  }
  
  # Build Rd index once for performance and correctness
  idx <- build_rd_index(db)
  
  rows <- lapply(exported_funs, function(fun) {
    hit <- find_rd_for_fun(fun, db, idx)  # returns list(rd, filename) or NULL
    if (is.null(hit)) {
      return(data.frame(
        function_name = fun,
        documentation_name = "No documentation found",
        documentation_location = NA_character_,
        example = "no Rd file",
        stringsAsFactors = FALSE
      ))
    }
    
    ex_txt <- extract_examples_text(hit$rd)
    data.frame(
      function_name = fun,
      documentation_name = rd_name(hit$rd),
      documentation_location = file.path("man", hit$filename),
      example = if (is.null(ex_txt)) "no example" else ex_txt,
      stringsAsFactors = FALSE
    )
  })
  
  df <- do.call(rbind, rows)
  
  # Example score: % of exported functions that have examples
  has_example <- df$example != "no example" & df$example != "no Rd file"
  total <- nrow(df)
  example_score <- if (total > 0) round(sum(has_example) / total, 2) else 0
  
  
  
  message(sprintf("%s: %.2f%% of exported functions have examples", 
                  pkg_name, 
                  example_score * 100))
  return(list(data = df, 
              example_score = example_score))
}


#' Get all Rd sections by tag
#'
#' Recursively collects all nodes whose `Rd_tag` matches a given section
#' name, e.g., "alias", "examples", "example", "name".
#'
#' @param rd An Rd object (a nested list with "Rd_tag" attributes on nodes).
#' @param section Section name without leading backslash (e.g., "alias", "examples").
#' @return A list of Rd nodes matching the section (possibly empty).
#' @keywords internal
get_rd_sections <- function(rd, section) {
  target <- paste0("\\", section)
  out <- list()
  dfs <- function(node) {
    tag <- attr(node, "Rd_tag")
    if (!is.null(tag) && identical(tag, target)) {
      out[[length(out) + 1L]] <<- node
    }
    if (is.list(node)) {
      for (child in node) dfs(child)
    }
    NULL
  }
  dfs(rd)
  out
}

#' Get the first Rd section by tag
#'
#' Depth-first search returning the first node whose `Rd_tag` equals
#' `paste0("\\\\", section)`.
#'
#' @inheritParams get_rd_sections
#' @return The first matching Rd node or `NULL` if not found.
#' @keywords internal
get_rd_section <- function(rd, section) {
  target <- paste0("\\", section)
  dfs <- function(node) {
    tag <- attr(node, "Rd_tag")
    if (!is.null(tag) && identical(tag, target)) {
      return(node)
    }
    if (is.list(node)) {
      for (child in node) {
        found <- dfs(child)
        if (!is.null(found)) return(found)
      }
    }
    NULL
  }
  dfs(rd)
}

#' Flatten an Rd node to plain text
#'
#' Recursively flattens a node (or list of nodes) into a character string.
#'
#' @param node An Rd node or list of nodes.
#' @return Character scalar containing the concatenated text.
#' @keywords internal
rd_flatten_text <- function(node) {
  if (is.list(node)) {
    paste(vapply(node, rd_flatten_text, "", USE.NAMES = FALSE), collapse = "")
  } else {
    as.character(node)
  }
}

#' Extract all aliases from an Rd document
#'
#' Collects text from **all** `\\alias{}` nodes, trimming whitespace and
#' de-duplicating.
#'
#' @param rd An Rd object.
#' @return A character vector of aliases (possibly empty).
#' @keywords internal
rd_aliases <- function(rd) {
  alias_nodes <- get_rd_sections(rd, "alias")
  if (length(alias_nodes) == 0) return(character(0))
  vals <- trimws(vapply(alias_nodes, rd_flatten_text, "", USE.NAMES = FALSE))
  unique(vals[nzchar(vals)])
}

#' Extract the topic name from an Rd document
#'
#' Returns the `\\name{}` (roxygen `@rdname`) of the Rd document.
#'
#' @param rd An Rd object.
#' @return A length-1 character scalar (topic name) or `NA_character_` if missing.
#' @keywords internal
rd_name <- function(rd) {
  nm_node <- get_rd_section(rd, "name")
  if (is.null(nm_node)) return(NA_character_)
  trimws(rd_flatten_text(nm_node))
}

#' Build a fast lookup index for an Rd database
#'
#' Provides (1) an alias index, mapping `alias -> filename`, and
#' (2) a topic index, mapping `filename -> \\name` (topic).
#'
#' @param db An Rd database list as returned by `tools::Rd_db()`.
#' @return A list with elements `alias_index` (environment) and `topic_by_file` (named character vector).
#' @keywords internal
build_rd_index <- function(db) {
  alias_index <- new.env(parent = emptyenv())
  topic_by_file <- setNames(
    vapply(names(db), function(nm) rd_name(db[[nm]]), "", USE.NAMES = FALSE),
    names(db)
  )
  for (nm in names(db)) {
    als <- rd_aliases(db[[nm]])
    for (a in als) {
      if (nzchar(a) && is.null(alias_index[[a]])) {
        alias_index[[a]] <- nm
      }
    }
  }
  list(alias_index = alias_index, topic_by_file = topic_by_file)
}

#' Find an Rd document for a function using an index
#'
#' Tries `fun.Rd`, then alias-based resolution via the index, and as a last
#' resort matches the topic name if it equals the function name.
#'
#' @param fun Function name (character scalar).
#' @param db An Rd database list as returned by `tools::Rd_db()`.
#' @param idx Optional index from `build_rd_index()`. If `NULL`, it is built on demand.
#' @return `NULL` if not found, otherwise a list with `rd` (the Rd object) and `filename` (the Rd filename).
#' @keywords internal
find_rd_for_fun <- function(fun, db, idx = NULL) {
  if (is.null(idx)) idx <- build_rd_index(db)
  # 1) conventional filename
  direct <- paste0(fun, ".Rd")
  if (!is.null(db[[direct]])) {
    return(list(rd = db[[direct]], filename = direct))
  }
  # 2) alias-based via index
  nm <- idx$alias_index[[fun]]
  if (!is.null(nm)) {
    return(list(rd = db[[nm]], filename = nm))
  }
  # 3) topic name equals function name (rare fallback)
  topic_match <- names(idx$topic_by_file)[idx$topic_by_file == fun]
  if (length(topic_match) > 0) {
    nm <- topic_match[[1]]
    return(list(rd = db[[nm]], filename = nm))
  }
  NULL
}


#' Flatten example nodes to a text block
#'
#' Recursively collects example text from nodes that carry R code or text,
#' unwrapping containers such as `\\dontrun{}`, `\\donttest{}`, and `\\dontshow{}`.
#'
#' @param node An Rd node.
#' @return Character scalar containing example text or `""` if none.
#' @keywords internal
rd_examples_to_text <- function(node) {
  tag <- attr(node, "Rd_tag")
  if (is.list(node)) {
    paste(vapply(node, rd_examples_to_text, "", USE.NAMES = FALSE), collapse = "")
  } else {
    # Keep code + text comments in examples. If you only want code, drop "TEXT".
    if (!is.null(tag) && tag %in% c("RCODE", "VERB", "TEXT")) {
      as.character(node)
    } else {
      ""
    }
  }
}

#' Extract examples text from an Rd document
#'
#' Supports both `\\examples{}` and `\\example{}` sections. If multiple
#' sections exist, their contents are concatenated with blank lines.
#'
#' @param rd_doc An Rd document.
#' @return Character scalar of combined examples text, or `NULL` if none/non‑informative.
#' @keywords internal
extract_examples_text <- function(rd_doc) {
  nodes <- c(get_rd_sections(rd_doc, "examples"),
             get_rd_sections(rd_doc, "example"))
  if (length(nodes) == 0) return(NULL)
  chunks <- vapply(nodes, rd_examples_to_text, "", USE.NAMES = FALSE)
  txt <- paste(chunks[nzchar(chunks)], collapse = "\n\n")
  # Trim outer whitespace
  txt <- gsub("^[ \t\r\n]+|[ \t\r\n]+$", "", txt)
  if (nzchar(txt)) txt else NULL
}


#' Assess documentation coverage for exported functions
#'
#' Resolves each **exported function** in a package to its Rd page and reports:
#' the function name, the Rd topic name (roxygen `@rdname`), and the Rd location.
#' Resolution uses the same alias/topic index as `assess_examples()`, so it
#' correctly handles functions documented under a shared topic (via `@rdname`)
#' and multiple `\\alias{}` entries (e.g., `stringr::str_to_upper` under
#' `man/case.Rd`).
#'
#' @param pkg_name Character scalar; package name (must be loadable so exports can be read).
#' @param pkg_source_path Directory path where Rd files can be found by `tools::Rd_db()`.
#'
#' @return A list with:
#' \describe{
#'   \item{data}{A `data.frame` with columns:
#'     \describe{
#'       \item{function_name}{Exported function name.}
#'       \item{documentation_name}{The Rd topic name (`\\name{}`), or `"No documentation found"`.}
#'       \item{documentation_location}{`man/<file>.Rd` if found, otherwise `NA_character_`.}
#'     }
#'   }
#'   \item{documentation_score}{Numeric; percentage of exported functions with an Rd page.}
#' }
#'
#' @keywords internal
assess_exported_functions_docs <- function(pkg_name, pkg_source_path) {
  
  # Load Rd database for the source path
  db <- tools::Rd_db(dir = pkg_source_path)
  
  # Remove only the package-level Rd file (pkgname-package.Rd).
  # Keep files like "<pkg_name>.Rd" as they may document a function with the same name.
  db <- db[!names(db) %in% paste0(pkg_name, "-package.Rd")]
  
  # Namespace and exported objects
  ns <- asNamespace(pkg_name)
  exported <- getNamespaceExports(ns)
  
  # Keep only exported functions
  is_fun <- function(name) {
    obj <- try(getExportedValue(ns, name), silent = TRUE)
    is.function(obj)
  }
  exported_funs <- Filter(is_fun, exported)
  
  # --- Handle packages with NO exported functions ---------------------------
  if (length(exported_funs) == 0) {
    df <- data.frame(
      function_name = character(0),
      documentation_name = character(0),
      documentation_location = character(0),
      stringsAsFactors = FALSE
    )
    doc_score <- 0
    message(sprintf("%s: no exported functions found; documentation score = %.2f%%",
                    pkg_name, doc_score))
    return(list(data = df, has_docs_score = doc_score))
  }
  
  # Build Rd index once (reuses helpers from assess_examples)
  idx <- build_rd_index(db)
  
  # Resolve each exported function to its Rd page
  rows <- lapply(exported_funs, function(fun) {
    
    hit <- find_rd_for_fun(fun, db, idx)  # -> list(rd, filename) or NULL
    if (is.null(hit)) {
      return(data.frame(
        function_name = fun,
        documentation_name = "No documentation found",
        documentation_location = NA_character_,
        stringsAsFactors = FALSE
      ))
    }
    
    topic <- rd_name(hit$rd)
    if (is.na(topic) || !nzchar(topic)) topic <- fun
    
    data.frame(
      function_name = fun,
      documentation_name = topic,
      documentation_location = file.path("man", hit$filename),
      stringsAsFactors = FALSE
    )
  })
  
  df <- do.call(rbind, rows)
  
  # Documentation score: % of exported functions that have any Rd doc
  has_doc <- !is.na(df$documentation_location) &
    df$documentation_name != "No documentation found"
  total <- nrow(df)
  doc_score <- if (total > 0) round(sum(has_doc) / total, 2) else 0
  
  message(sprintf("%s: %.2f%% of exported functions have documentation", 
                  pkg_name, 
                  doc_score * 100))
  
  return(list(data = df, 
              has_docs_score = doc_score))
  
}  

#' Assess exported functions to namespace
#'
#' @param data pkg source path
#' @keywords internal
assess_exports <- function(data) {
  
  exports <- parseNamespaceFile(basename(data), dirname(data), mustExist = FALSE)$exports
  if (length(exports) > 0) {
    export_calc <- 1
  } else {
    export_calc <- 0
  }
} 

#' assess_export_help
#'
#' @param pkg_name - name of the package 
#' @param pkg_source_path - pkg_source_path - source path for install local
#'
#' @return - export_help - variable with score
#' @keywords internal
assess_export_help <- function(pkg_name, pkg_source_path) {
  
  exported_functions <- getNamespaceExports(pkg_name)
  if (length(exported_functions) > 0) {
    
    # Use get_func_descriptions to retrieve documentation
    func_descriptions <- get_func_descriptions(pkg_name)
    
    documented_names <- names(func_descriptions)
    missing_docs <- setdiff(exported_functions, documented_names)
    
    if (length(missing_docs) == 0) {
      message(glue::glue("All exported functions have corresponding help files in {pkg_name}"))
      export_help <- 1
    } else {
      message(glue::glue("Some exported functions are missing help files in {pkg_name}"))
      # message(glue::glue("The following exported functions are missing help files in {pkg_name}"))
      # print(missing_docs) # Uncomment if needed
      export_help <- 0
    }
  } else {
    message(glue::glue("No exported functions in {pkg_name}"))
    export_help <- 0
  }
  
  return(export_help)
}


#' Combine example/docs scores into a single metric (sum).
#'
#' @param example_score Numeric (scalar or vector) in [0, 1] or NA.
#' @param has_docs_score Numeric (scalar or vector) in [0, 1] or NA.
#' @return Numeric vector: example_score + has_docs_score in [0, 2],
#'   with NA where both inputs are NA at that position.
#' @keywords internal
create_has_ex_docs_score <- function(example_score, has_docs_score) {
  
  vals <- c(example_score, has_docs_score)
  if (all(is.na(vals))) {
    has_ex_docs_score <- NA_real_
  } else {
    has_ex_docs_score <- mean(vals, na.rm = TRUE)  
  }
  
  return(has_ex_docs_score)
}


#' assess_description_file_elements
#'
#' @param pkg_name - name of the package 
#' @param pkg_source_path - pkg_source_path - source path for install local
#'
#' @return - list - list with scores for description file elements
#' @keywords internal
assess_description_file_elements <- function(pkg_name, pkg_source_path) {
  
  desc_elements <- get_pkg_desc(pkg_source_path, fields = c(
    "Package", 
    "BugReports",
    "Maintainer",
    "URL"
  ))
  
  if (is.null(desc_elements$BugReports) | (is.na(desc_elements$BugReports))) {
    message(glue::glue("{pkg_name} does not have bug reports URL"))
    has_bug_reports_url <- NULL
  } else {
    message(glue::glue("{pkg_name} has bug reports URL"))
    has_bug_reports_url <- desc_elements$BugReports
  }
  
  has_maintainer <- desc::desc_get_author(role = "cre",
                                          file = pkg_source_path)
  if (length(has_maintainer) == 0) {
    has_maintainer <-  NULL
    message(glue::glue("{pkg_name} does not have a maintainer"))
  } else {
    message(glue::glue("{pkg_name} has a maintainer"))
  }  
  
  if (is.null(desc_elements$URL) | (is.na(desc_elements$URL))) {
    message(glue::glue("{pkg_name} does not have a website"))
    has_website <- NULL
  } else {
    message(glue::glue("{pkg_name} has a website"))
    has_website <- desc_elements$URL
  }
  
  # Check for source control URL
  # Pattern includes GitHub Pages URLs (pages.github.io, github.io) and other common source control hosts
  patterns <- "github\\.com|pages\\.github\\.io|github\\.io|bitbucket\\.org|gitlab\\.com|\\.ac\\.uk|\\.edu\\.au|bioconductor\\.org"
  
  # First check URL field
  url_to_check <- desc_elements$URL
  url_matches <- character(0)
  
  if (!is.null(url_to_check) && !is.na(url_to_check)) {
    url_matches <- grep(patterns, url_to_check, value = TRUE)
  }
  
  # If URL field doesn't match, check BugReports as fallback
  if (length(url_matches) == 0 && !is.null(desc_elements$BugReports) && !is.na(desc_elements$BugReports)) {
    bugreports_matches <- grep(patterns, desc_elements$BugReports, value = TRUE)
    if (length(bugreports_matches) > 0) {
      url_matches <- bugreports_matches
      message(glue::glue("{pkg_name} source control detected via BugReports URL"))
    }
  }
  
  if (length(url_matches) == 0) {
    message(glue::glue("{pkg_name} does not have a source control"))
    has_source_control <- NULL
  } else {
    message(glue::glue("{pkg_name} has a source control"))
    has_source_control <- url_matches
  }
  
  desc_elements_scores <- list(
    has_bug_reports_url = has_bug_reports_url,
    has_maintainer = has_maintainer,
    has_website = has_website,
    has_source_control = has_source_control
  )
  
  return(desc_elements_scores)
}

#' Assess Rd files for news
#'
#' @param pkg_name - name of the package 
#' @param pkg_source_path - source path for install local
#'
#' @return has_news - variable with score
#' @keywords internal
assess_news <- function(pkg_name, pkg_source_path) {
  
  news <- list.files(pkg_source_path, 
                     pattern = "^NEWS\\.", 
                     full.names = TRUE)
  
  if (length(news) > 0) {
    message(glue::glue("{pkg_name} has news"))
    has_news <- 1
  } else {
    message(glue::glue("{pkg_name} has no news"))
    has_news <- 0
  }
  return(has_news)
}

#' Assess Rd files for news
#'
#' @param pkg_name - name of the package
#' @param pkg_ver - package version 
#' @param pkg_source_path - source path for install local
#'
#' @return news_current - variable with score
#' @keywords internal
assess_news_current <- function(pkg_name, pkg_ver, pkg_source_path) {
  
  # get NEWS.md filepath
  news_path <- file.path(pkg_source_path, "NEWS.md")
  
  # check news for latest package version
  if (file.exists(news_path)) {
    news_content <- readLines(news_path)
    version_pattern <- paste0("^# ",
                              pkg_name,
                              " ",
                              pkg_ver)
    version_lines <- grep(version_pattern, 
                          news_content, 
                          value = TRUE)
  } else {
    message(glue::glue("{pkg_name} has no news path"))
    version_lines <- character(0)
  }
  
  if (length(version_lines) == 0) {
    message(glue::glue("{pkg_name} has no current news"))
    news_current <- 0
  } else {
    message(glue::glue("{pkg_name} has current news"))
    news_current <- 1
  }
  return(news_current)
}


#' assess codebase size
#'
#' @description Scores packages based on its codebase size, 
#' as determined by number of lines of code.
#'
#' @param pkg_source_path - source path for install local 
#'
#' @return - size_codebase - numeric value between \code{0} (for small codebase) and \code{1} (for large codebase)
#' @keywords internal
assess_size_codebase <- function(pkg_source_path) {
  
  # create character vector of function files
  files <- list.files(path = file.path(pkg_source_path, "R"), full.names = T)
  
  # define the function for counting code base
  count_lines <- function(x){
    # read the lines of code into a character vector
    code_base <- readLines(x)
    
    # count all the lines
    n_tot <- length(code_base)
    
    # count lines for roxygen headers starting with #
    n_head <- length(grep("^#+", code_base))
    
    # count the comment lines with leading spaces
    n_comment <- length(grep("^\\s+#+", code_base))
    
    # count the line breaks or only white space lines
    n_break <- length(grep("^\\s*$", code_base))
    
    # compute the line of code base
    n_tot - (n_head + n_comment + n_break)
  }
  
  # count number of lines for all functions
  nloc <- sapply(files, count_lines)
  
  # sum the number of lines
  size_codebase <- sum(nloc)
  
  return(size_codebase)
}

#' Assess vignettes
#'
#' @param pkg_name - name of the package 
#' @param pkg_source_path - source path for install local 
#' 
#' @return - has_vignettes - variable with score
#' @keywords internal
assess_vignettes <- function(pkg_name, pkg_source_path) {
  
  folder <- c(source = "/vignettes", bundle = "/inst/doc", binary = "/doc")
  files <- unlist(lapply(paste0(pkg_source_path, folder), list.files, full.names = TRUE))
  
  file_path = unique(tools::file_path_sans_ext(files))
  filename = basename(file_path)
  names(file_path) <- filename
  
  if (length(filename) == 0) {
    message(glue::glue("{pkg_name} has no vignettes"))
    has_vignettes <- 0
  } else {
    message(glue::glue("{pkg_name} has vignettes"))
    has_vignettes <- 1
  }
  
  return(has_vignettes)
}

#' Run all relevant documentation riskmetric checks
#'
#' @param pkg_name name of the package
#' @param pkg_ver version of the package
#' @param pkg_source_path path to package source code (untarred)
#'
#' @returns raw riskmetric outputs
#'@keywords internal
doc_riskmetric <- function(pkg_name, pkg_ver, pkg_source_path) {
  
  
  export_help <- assess_export_help(pkg_name, pkg_source_path)
  desc_elements <- assess_description_file_elements(pkg_name, pkg_source_path)
  
  if (fs::dir_exists(fs::path(pkg_source_path, "R"))) {
    size_codebase <- assess_size_codebase(pkg_source_path)
  } else {
    size_codebase <- 0
    message(glue::glue("{pkg_name} has no R folder to assess codebase size"))
  }
  has_vignettes <- assess_vignettes(pkg_name, pkg_source_path)
  has_examples <- assess_examples(pkg_name, pkg_source_path)
  has_docs <- assess_exported_functions_docs(pkg_name, pkg_source_path)
  has_ex_docs_score <- create_has_ex_docs_score(has_examples$example_score, 
                                                has_docs$has_docs_score)
  has_news <- assess_news(pkg_name, pkg_source_path)
  news_current <- assess_news_current(pkg_name, pkg_ver, pkg_source_path)
  
  doc_scores <- list(
    export_help = export_help,
    has_bug_reports_url = desc_elements$has_bug_reports_url,
    has_source_control = desc_elements$has_source_control,
    has_maintainer = desc_elements$has_maintainer,
    has_website = desc_elements$has_website,
    size_codebase = size_codebase,
    has_vignettes = has_vignettes,
    has_examples = has_examples,
    has_docs = has_docs,
    has_ex_docs_score = has_ex_docs_score, 
    has_news = has_news,
    news_current = news_current
  )
  
  return(doc_scores)
}


#' Assess Authors
#'
#' @param pkg_name - name of the package 
#' @param pkg_source_path - source path for install local 
#' 
#' @return - Authors related variable
#' @importFrom desc description desc_coerce_authors_at_r
#' @keywords internal
get_pkg_author <- function(pkg_name, pkg_source_path) {
  message(glue::glue("Checking for author in {pkg_name}"))
  
  # Path to the DESCRIPTION file
  pkg_desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  
  # Load the DESCRIPTION file
  d <- description$new(file = pkg_desc_path)
  
  # Check if Authors@R exists, and if not, convert from Author and Maintainer
  if (!d$has_fields("Authors@R")) {
    if (d$has_fields("Author") || d$has_fields("Maintainer")) {
      desc_coerce_authors_at_r(file = pkg_desc_path)
    }
  }
  
  # check for package creator
  pkg_creator <- desc::desc_get_author(role = "cre",
                                       file = pkg_desc_path)
  
  if (length(pkg_creator) == 0) pkg_creator <-  NULL
  
  # check for package funder
  pkg_fnd <- desc::desc_get_author(role = "fnd",
                                   file = pkg_desc_path)
  if (length(pkg_fnd) == 0) pkg_fnd <- NULL
  
  # check for package authors
  pkg_author <- desc::desc_get_authors(file = pkg_desc_path)
  if (length(pkg_author) == 0) pkg_author <- NULL
  
  # Combine all elements into a list for return
  result <- list(
    maintainer = pkg_creator,
    funder = pkg_fnd,
    authors = pkg_author
  )
  
  return(result)
}


#' Clean and normalize license names
#'
#' Splits a license string on commas, plus signs, or pipes,
#' trims whitespace, removes trailing noise like "file LICENSE",
#' and normalizes each part to uppercase letters only.
#'
#' @param x A character string containing license information.
#'
#' @return A character vector of normalized license names.
#'
#' @keywords internal
clean_license <- function(x) {
  # Handle NA, NULL, or non-character inputs
  if (is.null(x) || (length(x) == 1 && is.na(x))) {
    return(NULL)
  }
  
  # Convert to character if needed
  if (!is.character(x)) {
    x <- as.character(x)
  }
  
  # Handle empty strings
  if (length(x) == 0 || nchar(x) == 0) {
    return(NULL)
  }
  
  parts <- unlist(strsplit(x, "[,+|]"))
  base_license_names <- c()
  
  for (part in parts) {
    part <- trimws(part)
    part <- gsub("\\b(file\\s+)?license\\b.*", "", part, ignore.case = TRUE)
    # FIXED: do toupper AFTER keeping A-Za-z
    normalized <- toupper(gsub("[^A-Za-z]", "", part))
    if (nchar(normalized) > 0) {
      base_license_names <- c(base_license_names, normalized)
    }
  }
  
  # Return NULL for empty result to match existing test expectations
  if (length(base_license_names) == 0) {
    return(NULL)
  }
  
  return(base_license_names)
}

test_that("test doc_riskmetrics", {
  
  # set CRAN repo 
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "stringr-1.5.1.tar.gz",
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # Defer cleanup: remove test package from temp dirs
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    pkg_desc <- get_pkg_desc(pkg_source_path, 
                             fields = c("Package", 
                                        "Version"))
    
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    doc_riskmetric_test <- 
      doc_riskmetric(pkg_name, pkg_ver, pkg_source_path)
    
    expect_identical(length(doc_riskmetric_test), 12L)
    expect_true(checkmate::check_list(doc_riskmetric_test, all.missing = FALSE))
    expect_true(checkmate::check_list(doc_riskmetric_test, any.missing = TRUE))
  }
})


test_that("doc_riskmetric handles missing R folder", {
  
  # Stub fs::dir_exists to simulate missing R folder
  mockery::stub(doc_riskmetric, "fs::dir_exists", function(path) FALSE)
  
  # Stub fs::path to return a dummy path
  mockery::stub(doc_riskmetric, "fs::path", function(...) "mock/path/R")
  
  # Stub assess_description_file_elements to return expected structure
  mockery::stub(doc_riskmetric, "assess_description_file_elements", function(pkg_name, pkg_source_path) {
    list(
      has_bug_reports_url = TRUE,
      has_source_control = TRUE,
      has_maintainer = TRUE,
      has_website = TRUE
    )
  })
  
  # Stub other assess_* functions to return dummy values
  mockery::stub(doc_riskmetric, "assess_export_help", function(...) TRUE)
  mockery::stub(doc_riskmetric, "assess_vignettes", function(...) TRUE)
  mockery::stub(doc_riskmetric, "assess_examples", function(...) TRUE)
  mockery::stub(doc_riskmetric, "assess_news", function(...) TRUE)
  mockery::stub(doc_riskmetric, "assess_news_current", function(...) TRUE)
  
  
  # Stub assess_examples to avoid tools::Rd_db
  mockery::stub(doc_riskmetric, "assess_examples", function(...) {
    list(data = data.frame(function_name = "fun", example = "no example"), example_score = 50)
  })
  
  
  # Stub assess_exported_functions_docs to avoid tools::Rd_db
  mockery::stub(doc_riskmetric, "assess_exported_functions_docs", function(...) {
    list(data = data.frame(function_name = "fun", documentation_name = "topic", documentation_location = "man/topic.Rd"),
         documentation_score = 75)
  })
  
  
  pkg_name <- "mockpkg"
  pkg_ver <- "0.1.0"
  pkg_source_path <- "mock/path"
  
  expect_message(
    doc_scores <- doc_riskmetric(pkg_name, pkg_ver, pkg_source_path),
    glue::glue("{pkg_name} has no R folder to assess codebase size"),
    fixed = TRUE
  )
  
  expect_equal(doc_scores$size_codebase, 0)
})

test_that("get_pkg_author returns correct structure without funder", {
  # Mocked return values
  mock_creator <- list(list(email = "krlmlr+r@mailbox.org"))
  mock_authors <- list(
    list(email = "krlmlr+r@mailbox.org"),
    list(email = "jenny@rstudio.com")
  )
  
  # Create a fake description object with the required methods
  fake_desc <- list(
    has_fields = function(field) TRUE
  )
  class(fake_desc) <- "description"
  
  # Stub all necessary functions
  mockery::stub(get_pkg_author, "description$new", function(file) fake_desc)
  mockery::stub(get_pkg_author, "desc::desc_get_author", function(role, file) {
    if (role == "cre") return(mock_creator)
    if (role == "fnd") return(NULL)
  })
  mockery::stub(get_pkg_author, "desc::desc_get_authors", function(file) mock_authors)
  
  result <- get_pkg_author("test", "fake/path")
  
  expect_equal(result$maintainer[[1]]$email, "krlmlr+r@mailbox.org")
  expect_null(result$funder)
  expect_equal(length(result$authors), 2)
  expect_equal(result$authors[[1]]$email, "krlmlr+r@mailbox.org")
  expect_equal(result$authors[[2]]$email, "jenny@rstudio.com")
  
})


test_that("get_pkg_author returns correct structure with funder", {
  # Mocked return values
  mock_creator <- list(list(email = "hadley@posit.co"))
  mock_funder <- "Posit Software, PBC [cph, fnd]"
  mock_authors <- list(
    list(email = "hadley@posit.co"),
    list(email = "another@posit.co")
  )
  
  # Create a fake description object with the required methods
  fake_desc <- list(
    has_fields = function(field) TRUE
  )
  class(fake_desc) <- "description"
  
  # Stub all necessary functions
  mockery::stub(get_pkg_author, "description$new", function(file) fake_desc)
  mockery::stub(get_pkg_author, "desc::desc_get_author", function(role, file) {
    if (role == "cre") return(mock_creator)
    if (role == "fnd") return(mock_funder)
  })
  mockery::stub(get_pkg_author, "desc::desc_get_authors", function(file) mock_authors)
  
  result <- get_pkg_author("test", "fake/path")
  
  
  expect_equal(result$maintainer[[1]]$email, "hadley@posit.co")
  expect_equal(as.character(result$funder), mock_funder)
  expect_equal(length(result$authors), 2)
  expect_equal(result$authors[[1]]$email, "hadley@posit.co")
})


test_that("get_pkg_author handles missing creator and author", {
  
  # Stub description$new to return an object with has_fields method
  mock_desc_obj <- list(
    has_fields = function(field) FALSE
  )
  class(mock_desc_obj) <- "description"
  
  mockery::stub(get_pkg_author, "description$new", function(file) mock_desc_obj)
  
  # Stub desc_coerce_authors_at_r to do nothing
  mockery::stub(get_pkg_author, "desc_coerce_authors_at_r", function(file) NULL)
  
  # Stub desc::desc_get_author to return empty vector for both roles
  mockery::stub(get_pkg_author, "desc::desc_get_author", function(role, file) character(0))
  
  # Stub desc::desc_get_authors to return empty vector
  mockery::stub(get_pkg_author, "desc::desc_get_authors", function(file) character(0))
  
  pkg_name <- "mockpkg"
  pkg_source_path <- "mock/path"
  
  result <- get_pkg_author(pkg_name, pkg_source_path)
  
  expect_null(result$maintainer)
  expect_null(result$authors)
  expect_null(result$funder)
})



test_that("parse authors for mocked package folder works correctly", {
  mock_dir <- withr::local_tempdir()
  dir.create(file.path(mock_dir, "R"), showWarnings = FALSE)
  
  desc_text <- "Package: here
Title: A Simpler Way to Find Your Files
Version: 1.0.2.9000
Date: 2025-09-15
Authors@R:
    c(person(given = \"Kirill\",
             family = \"M\\u00fcller\",
             role = c(\"aut\", \"cre\"),
             email = \"kirill@cynkra.com\",
             comment = c(ORCID = \"0000-0002-1416-3412\")),
      person(given = \"Jennifer\",
             family = \"Bryan\",
             role = \"ctb\",
             email = \"jenny@rstudio.com\",
             comment = c(ORCID = \"0000-0002-6983-2759\")))
Description: Constructs paths to your project's files.
    Declare the relative path of a file within your project with 'i_am()'.
    Use the 'here()' function as a drop-in replacement for 'file.path()',
    it will always locate the files relative to your project root.
License: MIT + file LICENSE
URL: https://here.r-lib.org/, https://github.com/r-lib/here
BugReports: https://github.com/r-lib/here/issues
Imports:
    rprojroot (>= 2.1.0)
Suggests:
    conflicted,
    covr,
    fs,
    knitr,
    palmerpenguins,
    plyr,
    readr,
    rlang,
    rmarkdown,
    testthat,
    uuid,
    withr
VignetteBuilder:
    knitr
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3.9000
Config/testthat/edition: 3
Config/Needs/website: tidyverse/tidytemplate
"
  
  writeLines(desc_text, file.path(mock_dir, "DESCRIPTION"), useBytes = TRUE)
  writeLines("exportPattern(\"^[[:alpha:]]+\")", file.path(mock_dir, "NAMESPACE"))
  
  result <- get_pkg_author("test", mock_dir)
  
  expect_equal(result$maintainer[[1]]$email, "kirill@cynkra.com")
  expect_null(result$funder)
  
  expect_true(!is.null(result$authors))
  expect_equal(length(result$authors), 2)
  expect_equal(result$authors[[1]]$email, "kirill@cynkra.com")
  expect_equal(result$authors[[2]]$email, "jenny@rstudio.com")
})



test_that("parse authors for mocked package folder works correctly 2", {
  mock_dir <- withr::local_tempdir()
  dir.create(file.path(mock_dir, "R"), showWarnings = FALSE)
  
  desc_text <- "Package: stringr
Title: Simple, Consistent Wrappers for Common String Operations
Version: 1.5.2.9000
Authors@R: c(
    person(\"Hadley\", \"Wickham\", , \"hadley@posit.co\", role = c(\"aut\", \"cre\", \"cph\")),
    person(\"Posit Software, PBC\", role = c(\"cph\", \"fnd\"))
  )
Description: A consistent, simple and easy to use set of wrappers around
    the fantastic 'stringi' package. All function and argument names (and
    positions) are consistent, all functions deal with \"NA\"'s and zero
    length vectors in the same way, and the output from one function is
    easy to feed into the input of another.
License: MIT + file LICENSE
URL: https://stringr.tidyverse.org, https://github.com/tidyverse/stringr
BugReports: https://github.com/tidyverse/stringr/issues
Depends:
    R (>= 3.6)
Imports:
    cli,
    glue (>= 1.6.1),
    lifecycle (>= 1.0.3),
    magrittr,
    rlang (>= 1.0.0),
    stringi (>= 1.5.3),
    vctrs (>= 0.4.0)
Suggests:
    covr,
    dplyr,
    gt,
    htmltools,
    htmlwidgets,
    knitr,
    rmarkdown,
    testthat (>= 3.0.0),
    tibble
VignetteBuilder:
    knitr
Config/Needs/website: tidyverse/tidytemplate
Config/potools/style: explicit
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
"
  
  writeLines(desc_text, file.path(mock_dir, "DESCRIPTION"), useBytes = TRUE)
  writeLines("exportPattern(\"^[[:alpha:]]+\")", file.path(mock_dir, "NAMESPACE"))
  
  result <- get_pkg_author("test", mock_dir)
  
  expect_equal(result$maintainer[[1]]$email, "hadley@posit.co")
  expect_equal(as.character(result$funder), "Posit Software, PBC [cph, fnd]")
  
  expect_equal(length(result$authors), 2)
  expect_equal(as.character(result$maintainer[[1]]$email), "hadley@posit.co")
})

test_that("parse license for tar file MIT", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # Defer cleanup: remove test package from temp dirs
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    result <- extract_license_from_description(pkg_source_path)
    expect_equal(result, "MIT + file LICENSE")
  }
})


test_that("parse authors for tar file Apache License", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    result <- extract_license_from_description(pkg_source_path)
    expect_equal(result, "Apache License (>= 2)")
  }
})


test_that("parse license not present", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0007_0.1.0.tar.gz", 
                         package = "risk.assessr")
  
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    suppressWarnings(result <- extract_license_from_description(pkg_source_path))
    expect_true(is.na(result))
  }
})


# clean_license

test_that("clean_license splits and normalizes single license", {
  expect_equal(clean_license("MIT"), c("MIT"))
  expect_equal(clean_license("MIT + file LICENSE"), "MIT")
  expect_equal(clean_license("MIT file LICENSE"), "MIT")
  expect_equal(clean_license("Apache-2.0"), c("APACHE"))
  expect_equal(clean_license("NOT AVAILABLE"), c("NOTAVAILABLE"))
})

test_that("clean_license removes trailing file LICENSE", {
  expect_equal(clean_license("MIT file LICENSE"), c("MIT"))
  expect_equal(clean_license("Apache License"), c("APACHE"))
  expect_equal(clean_license("Apache file LICENSE and stuff"), c("APACHE"))
})

test_that("clean_license splits on commas, plus, and pipes", {
  expect_equal(clean_license("MIT, GPL"), c("MIT", "GPL"))
  expect_equal(clean_license("MIT + GPL"), c("MIT", "GPL"))
  expect_equal(clean_license("MIT | GPL"), c("MIT", "GPL"))
})

test_that("clean_license trims whitespace and normalizes to uppercase letters only", {
  expect_equal(clean_license(" mit "), c("MIT"))
  expect_equal(clean_license("Apache License 2.0"), c("APACHE"))
  expect_equal(clean_license("GPL-3.0"), c("GPL"))
})

test_that("clean_license ignores empty parts", {
  expect_equal(clean_license(", , MIT"), c("MIT"))
  expect_equal(clean_license("| | GPL |"), c("GPL"))
})

test_that("clean_license handles empty string input", {
  expect_equal(clean_license(""), NULL)
})

test_that("test assess_description_file_elements for all elements present", {
# check source control
  test_source_control_elements <- function() {
  # Define toy data for testing
    toy_data <- list(
      no_source_control = list(
        pkg_name = "pkg_no_source_control",
        desc_elements = list(URL = "http://example.com"),
        expected_message = "pkg_no_source_control does not have a source control",
        expected_has_source_control = 0
      ),
      has_github = list(
        pkg_name = "pkg_has_github",
        desc_elements = list(URL = "https://github.com/user/repo"),
        expected_message = "pkg_has_github has a source control",
        expected_has_source_control = 1
      ),
      has_ac_uk = list(
        pkg_name = "pkg_has_ac_uk",
        desc_elements = list(URL = "http://www.stats.ox.ac.uk/pub/MASS4/"),
        expected_message = "pkg_has_ac_uk has a source control",
        expected_has_source_control = 1
      ),
      has_bitbucket = list(
        pkg_name = "pkg_has_bitbucket",
        desc_elements = list(URL = "https://bitbucket.org/user/repo"),
        expected_message = "pkg_has_bitbucket has a source control",
        expected_has_source_control = 1
      ),
      has_gitlab = list(
        pkg_name = "pkg_has_gitlab",
        desc_elements = list(URL = "https://gitlab.com/user/repo"),
        expected_message = "pkg_has_gitlab has a source control",
        expected_has_source_control = 1
      ),
      has_cambridge_repo = list(
        pkg_name = "pkg_has_cambridge_repo",
        desc_elements = list(URL = "https://www.repository.cam.ac.uk/items/da5b9b21-ef5f-4ac8-80e4-553d99014aaf/full"),
        expected_message = "pkg_has_cambridge_repo has a source control",
        expected_has_source_control = 1
      ),
      has_wehi = list(
        pkg_name = "pkg_has_wehi",
        desc_elements = list(URL = "http://bioinf.wehi.edu.au/limma"),
        expected_message = "pkg_has_wehi has a source control",
        expected_has_source_control = 1
      ),
      has_bioconductor = list(
        pkg_name = "pkg_has_bioconductor",
        desc_elements = list(URL = "https://bioconductor.org/packages/IRanges"),
        expected_message = "pkg_has_bioconductor has a source control",
        expected_has_source_control = 1
      )
    )
    
    # Define the patterns
    patterns <- "github\\.com|bitbucket\\.org|gitlab\\.com|\\.ac\\.uk|\\.edu\\.au|bioconductor\\.org"
  
  # Test each scenario
    for (test_case in toy_data) {
      pkg_name <- test_case$pkg_name
      desc_elements <- test_case$desc_elements
      expected_message <- test_case$expected_message
      expected_has_source_control <- test_case$expected_has_source_control
      
      expect_message({
        if (is.null(desc_elements$URL) | (is.na(desc_elements$URL))) {
          message(glue::glue("{pkg_name} does not have a source control"))
          has_source_control <- 0
        } else {
          source_matches <- grep(patterns, desc_elements$URL, value = TRUE)
          if (length(source_matches) == 0) {
            message(glue::glue("{pkg_name} does not have a source control"))
            has_source_control <- 0
          } else {
            message(glue::glue("{pkg_name} has a source control"))
            has_source_control <- 1
          }
        }
      }, expected_message)
      
      expect_equal(has_source_control, expected_has_source_control)
    }
  }  

  test_source_control_elements()  
})

test_that("GitHub Pages URLs are detected as source control", {
  # Create a temporary package directory with DESCRIPTION file
  pkg_source_path <- tempfile(pattern = "testpkg")
  dir.create(pkg_source_path)
  on.exit(unlink(pkg_source_path, recursive = TRUE), add = TRUE)
  
  # org/user pages: *.github/* ; legacy project pages: *.pages.github.io
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  description_text <- paste0(
    "Package: testpkg\n",
    "Title: Test Package\n",
    "Version: 1.0.0\n",
    "URL: https://sanofi-public.github.io/risk.assessr/\n",
    "BugReports: https://github.com/Sanofi-Public/risk.assessr/issues\n"
  )
  writeLines(description_text, desc_path)
  
  # Test that GitHub Pages URL is detected
  result <- assess_description_file_elements("testpkg", pkg_source_path)
  expect_false(is.null(result$has_source_control))
  expect_true(length(result$has_source_control) > 0)
  expect_true(grepl("github\\.io", result$has_source_control))
})

test_that("BugReports URL is used as fallback when URL field doesn't match", {
  # Create a temporary package directory with DESCRIPTION file
  pkg_source_path <- tempfile(pattern = "testpkg")
  dir.create(pkg_source_path)
  on.exit(unlink(pkg_source_path, recursive = TRUE), add = TRUE)
  
  # Create DESCRIPTION with non-matching URL but GitHub BugReports
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  description_text <- paste0(
    "Package: testpkg\n",
    "Title: Test Package\n",
    "Version: 1.0.0\n",
    "URL: https://example.com/package\n",
    "BugReports: https://github.com/user/repo/issues\n"
  )
  writeLines(description_text, desc_path)
  
  # Test that BugReports URL is used as fallback
  result <- assess_description_file_elements("testpkg", pkg_source_path)
  expect_false(is.null(result$has_source_control))
  expect_true(length(result$has_source_control) > 0)
  expect_true(grepl("github\\.com", result$has_source_control))
})

test_that("test assess_description_file_elements for all elements present", {
  
  library(risk.assessr)
  # set CRAN repo 
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "stringr-1.5.1.tar.gz",
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # Defer cleanup: remove test package from temp dirs
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
  
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has bug reports URL"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a source control"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a maintainer"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a website"),
      fixed = TRUE
    )
    desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path)
    expect_identical(length(desc_elements_test), 4L)
    expect_true(checkmate::check_list(desc_elements_test, all.missing = FALSE))
    expect_true(checkmate::check_list(desc_elements_test, any.missing = TRUE))
  }
})

test_that("test assess_description_file_elements for all elements present", {
  
  library(risk.assessr)
  # set CRAN repo 
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0007_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} does not have bug reports URL"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} does not have a source control"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a maintainer"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} does not have a website"),
      fixed = TRUE
    )
    
    desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path)
    expect_identical(length(desc_elements_test), 4L)
    expect_true(checkmate::check_list(desc_elements_test, all.missing = FALSE))
    expect_true(checkmate::check_list(desc_elements_test, any.missing = TRUE))
  }
})


test_that("assess_description_file_elements handles missing maintainer", {
  
  
  # Stub desc_get_author to simulate no maintainer
  mockery::stub(assess_description_file_elements, "desc::desc_get_author", function(role, file) {
    character(0)
  })
  
  # Stub get_pkg_desc to return all required fields
  mockery::stub(assess_description_file_elements, "get_pkg_desc", function(path, fields) {
    list(
      Package = "mockpkg",
      BugReports = "https://example.com/bugs",
      Maintainer = NULL,
      URL = "https://example.com"
    )
  })
  
  pkg_name <- "mockpkg"
  pkg_source_path <- "mock/path"
  
  testthat::expect_message(
    desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
    glue::glue("{pkg_name} does not have a maintainer"),
    fixed = TRUE
  )
  
  # Optional: check output structure
  expect_true(checkmate::check_list(desc_elements_test, any.missing = TRUE))
})



# Authors

test_that("get_pkg_author retrieves all authors correctly", {
  # Create a temporary directory to simulate the package folder
  temp_dir <- tempdir()
  pkg_source_path <- file.path(temp_dir, "limma")
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  dir.create(pkg_source_path, recursive = TRUE, showWarnings = FALSE)
  
  # Provided DESCRIPTION text
  description_text <- "Package: here
Title: A Simpler Way to Find Your Files
Version: 1.0.1
Date: 2020-12-13
Authors@R: 
    c(person(given = 'Kirill',
             family = 'M\u00fcller',
             role = c('aut', 'cre'),
             email = 'krlmlr+r@mailbox.org',
             comment = c(ORCID = '0000-0002-1416-3412')),
      person(given = 'Jennifer',
             family = 'Bryan',
             role = 'ctb',
             email = 'jenny@rstudio.com',
             comment = c(ORCID = '0000-0002-6983-2759')))
Description: Constructs paths to your project's files.
    Declare the relative path of a file within your project with 'i_am()'.
    Use the 'here()' function as a drop-in replacement for 'file.path()',
    it will always locate the files relative to your project root.
License: MIT + file LICENSE
URL: https://here.r-lib.org/, https://github.com/r-lib/here
BugReports: https://github.com/r-lib/here/issues
Imports: rprojroot (>= 2.0.2)
Suggests: conflicted, covr, fs, knitr, palmerpenguins, plyr, readr,
        rlang, rmarkdown, testthat, uuid, withr
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1.9000
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2020-12-13 06:59:33 UTC; kirill
Author: Kirill Müller [aut, cre] (<https://orcid.org/0000-0002-1416-3412>),
  Jennifer Bryan [ctb] (<https://orcid.org/0000-0002-6983-2759>)
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2020-12-13 07:30:02 UTC
"
  
  # Write the DESCRIPTION text to a file
  writeLines(description_text, desc_path)
  
  # Run the function with the mocked package path
  result <- get_pkg_author("limma", pkg_source_path)
  
  # Assertions
  expect_type(result, "list")
  expect_true(!is.null(result$maintainer))
  expect_true(!is.null(result$authors))
  expect_equal(result$maintainer[[1]]$given, "Kirill")
  expect_equal(result$maintainer[[1]]$family, "Müller")
  
  # Clean up the temporary directory
  unlink(pkg_source_path, recursive = TRUE)
})

test_that("get_pkg_author retrieves all authors correctly without @Author", {
  # Create a temporary directory to simulate the package folder
  temp_dir <- tempdir()
  pkg_source_path <- file.path(temp_dir, "limma")
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  dir.create(pkg_source_path, recursive = TRUE, showWarnings = FALSE)
  
  # Provided DESCRIPTION text
  description_text <- "Package: limma
Version: 2.10.7
Date: 2007/09/24
Title: Linear Models for Microarray Data
Author: Gordon Smyth with contributions from Matthew Ritchie, Jeremy Silver, James Wettenhall, Natalie Thorne, Mette Langaas, Egil Ferkingstad, Marcus Davy, Francois Pepin and Dongseok Choi.
Maintainer: Gordon Smyth <smyth@wehi.edu.au>
Depends: R (<= 2.5.1), methods
Suggests: affy, marray, MASS, splines, sma, statmod (>= 1.2.2), vsn
LazyLoad: yes
Description:  Data analysis, linear models and differential expression for microarray data.
License: LGPL
URL: http://bioinf.wehi.edu.au/limma
biocViews: Microarray, OneChannel, TwoChannel, DataImport, QualityControl, Preprocessing, Statistics, DifferentialExpression, MultipleComparisons, TimeCourse
Packaged: Mon Sep 24 13:01:39 2007; smyth
"
  
  # Write the DESCRIPTION text to a file
  writeLines(description_text, desc_path)
  
  # Run the function with the mocked package path
  result <- get_pkg_author("limma", pkg_source_path)
  
  # Assertions
  expect_type(result, "list")
  expect_true(!is.null(result$maintainer))
  expect_true(!is.null(result$authors))
  expect_equal(result$maintainer[[1]]$given, "Gordon")
  expect_equal(result$maintainer[[1]]$family, "Smyth")
  
  # Clean up the temporary directory
  unlink(pkg_source_path, recursive = TRUE)
})

test_that("test assess_description_file_elements for all elements present", {
  # check source control
  test_source_control_elements <- function() {
    # Define toy data for testing
    toy_data <- list(
      no_source_control = list(
        pkg_name = "pkg_no_source_control",
        desc_elements = list(URL = "http://example.com"),
        expected_message = "pkg_no_source_control does not have a source control",
        expected_has_source_control = 0
      ),
      has_github = list(
        pkg_name = "pkg_has_github",
        desc_elements = list(URL = "https://github.com/user/repo"),
        expected_message = "pkg_has_github has a source control",
        expected_has_source_control = 1
      ),
      has_ac_uk = list(
        pkg_name = "pkg_has_ac_uk",
        desc_elements = list(URL = "http://www.stats.ox.ac.uk/pub/MASS4/"),
        expected_message = "pkg_has_ac_uk has a source control",
        expected_has_source_control = 1
      ),
      has_bitbucket = list(
        pkg_name = "pkg_has_bitbucket",
        desc_elements = list(URL = "https://bitbucket.org/user/repo"),
        expected_message = "pkg_has_bitbucket has a source control",
        expected_has_source_control = 1
      ),
      has_gitlab = list(
        pkg_name = "pkg_has_gitlab",
        desc_elements = list(URL = "https://gitlab.com/user/repo"),
        expected_message = "pkg_has_gitlab has a source control",
        expected_has_source_control = 1
      ),
      has_cambridge_repo = list(
        pkg_name = "pkg_has_cambridge_repo",
        desc_elements = list(URL = "https://www.repository.cam.ac.uk/items/da5b9b21-ef5f-4ac8-80e4-553d99014aaf/full"),
        expected_message = "pkg_has_cambridge_repo has a source control",
        expected_has_source_control = 1
      ),
      has_wehi = list(
        pkg_name = "pkg_has_wehi",
        desc_elements = list(URL = "http://bioinf.wehi.edu.au/limma"),
        expected_message = "pkg_has_wehi has a source control",
        expected_has_source_control = 1
      ),
      has_bioconductor = list(
        pkg_name = "pkg_has_bioconductor",
        desc_elements = list(URL = "https://bioconductor.org/packages/IRanges"),
        expected_message = "pkg_has_bioconductor has a source control",
        expected_has_source_control = 1
      )
    )
    
    # Define the patterns
    patterns <- "github\\.com|bitbucket\\.org|gitlab\\.com|\\.ac\\.uk|\\.edu\\.au|bioconductor\\.org"
    
    # Test each scenario
    for (test_case in toy_data) {
      pkg_name <- test_case$pkg_name
      desc_elements <- test_case$desc_elements
      expected_message <- test_case$expected_message
      expected_has_source_control <- test_case$expected_has_source_control
      
      expect_message({
        if (is.null(desc_elements$URL) | (is.na(desc_elements$URL))) {
          message(glue::glue("{pkg_name} does not have a source control"))
          has_source_control <- 0
        } else {
          source_matches <- grep(patterns, desc_elements$URL, value = TRUE)
          if (length(source_matches) == 0) {
            message(glue::glue("{pkg_name} does not have a source control"))
            has_source_control <- 0
          } else {
            message(glue::glue("{pkg_name} has a source control"))
            has_source_control <- 1
          }
        }
      }, expected_message)
      
      expect_equal(has_source_control, expected_has_source_control)
    }
  }  
  
  test_source_control_elements()  
})

test_that("test assess_description_file_elements for all elements present", {
  
  library(risk.assessr)
  # set CRAN repo 
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "stringr-1.5.1.tar.gz",
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
  
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has bug reports URL"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a source control"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a maintainer"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a website"),
      fixed = TRUE
    )
    desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path)
    expect_identical(length(desc_elements_test), 4L)
    expect_true(checkmate::check_list(desc_elements_test, all.missing = FALSE))
    expect_true(checkmate::check_list(desc_elements_test, any.missing = TRUE))
  }
})

test_that("test assess_description_file_elements for all elements present", {
  
  library(risk.assessr)
  # set CRAN repo 
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0007_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} does not have bug reports URL"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} does not have a source control"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} has a maintainer"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path),
      glue::glue("{pkg_name} does not have a website"),
      fixed = TRUE
    )
    
    desc_elements_test <- assess_description_file_elements(pkg_name, pkg_source_path)
    expect_identical(length(desc_elements_test), 4L)
    expect_true(checkmate::check_list(desc_elements_test, all.missing = FALSE))
    expect_true(checkmate::check_list(desc_elements_test, any.missing = TRUE))
  }
})

# Authors

test_that("get_pkg_author retrieves all authors correctly", {
  # Create a temporary directory to simulate the package folder
  temp_dir <- tempdir()
  pkg_source_path <- file.path(temp_dir, "limma")
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  dir.create(pkg_source_path, recursive = TRUE, showWarnings = FALSE)
  
  # Provided DESCRIPTION text
  description_text <- "Package: here
Title: A Simpler Way to Find Your Files
Version: 1.0.1
Date: 2020-12-13
Authors@R: 
    c(person(given = 'Kirill',
             family = 'M\u00fcller',
             role = c('aut', 'cre'),
             email = 'krlmlr+r@mailbox.org',
             comment = c(ORCID = '0000-0002-1416-3412')),
      person(given = 'Jennifer',
             family = 'Bryan',
             role = 'ctb',
             email = 'jenny@rstudio.com',
             comment = c(ORCID = '0000-0002-6983-2759')))
Description: Constructs paths to your project's files.
    Declare the relative path of a file within your project with 'i_am()'.
    Use the 'here()' function as a drop-in replacement for 'file.path()',
    it will always locate the files relative to your project root.
License: MIT + file LICENSE
URL: https://here.r-lib.org/, https://github.com/r-lib/here
BugReports: https://github.com/r-lib/here/issues
Imports: rprojroot (>= 2.0.2)
Suggests: conflicted, covr, fs, knitr, palmerpenguins, plyr, readr,
        rlang, rmarkdown, testthat, uuid, withr
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1.9000
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2020-12-13 06:59:33 UTC; kirill
Author: Kirill Müller [aut, cre] (<https://orcid.org/0000-0002-1416-3412>),
  Jennifer Bryan [ctb] (<https://orcid.org/0000-0002-6983-2759>)
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2020-12-13 07:30:02 UTC
"
  
  # Write the DESCRIPTION text to a file
  writeLines(description_text, desc_path)
  
  # Run the function with the mocked package path
  result <- get_pkg_author("limma", pkg_source_path)
  
  # Assertions
  expect_type(result, "list")
  expect_true(!is.null(result$maintainer))
  expect_true(!is.null(result$authors))
  expect_equal(result$maintainer[[1]]$given, "Kirill")
  expect_equal(result$maintainer[[1]]$family, "Müller")
  
  # Clean up the temporary directory
  unlink(pkg_source_path, recursive = TRUE)
})

test_that("get_pkg_author retrieves all authors correctly without @Author", {
  # Create a temporary directory to simulate the package folder
  temp_dir <- tempdir()
  pkg_source_path <- file.path(temp_dir, "limma")
  desc_path <- file.path(pkg_source_path, "DESCRIPTION")
  dir.create(pkg_source_path, recursive = TRUE, showWarnings = FALSE)
  
  # Provided DESCRIPTION text
  description_text <- "Package: limma
Version: 2.10.7
Date: 2007/09/24
Title: Linear Models for Microarray Data
Author: Gordon Smyth with contributions from Matthew Ritchie, Jeremy Silver, James Wettenhall, Natalie Thorne, Mette Langaas, Egil Ferkingstad, Marcus Davy, Francois Pepin and Dongseok Choi.
Maintainer: Gordon Smyth <smyth@wehi.edu.au>
Depends: R (<= 2.5.1), methods
Suggests: affy, marray, MASS, splines, sma, statmod (>= 1.2.2), vsn
LazyLoad: yes
Description:  Data analysis, linear models and differential expression for microarray data.
License: LGPL
URL: http://bioinf.wehi.edu.au/limma
biocViews: Microarray, OneChannel, TwoChannel, DataImport, QualityControl, Preprocessing, Statistics, DifferentialExpression, MultipleComparisons, TimeCourse
Packaged: Mon Sep 24 13:01:39 2007; smyth
"
  
  # Write the DESCRIPTION text to a file
  writeLines(description_text, desc_path)
  
  # Run the function with the mocked package path
  result <- get_pkg_author("limma", pkg_source_path)
  
  # Assertions
  expect_type(result, "list")
  expect_true(!is.null(result$maintainer))
  expect_true(!is.null(result$authors))
  expect_equal(result$maintainer[[1]]$given, "Gordon")
  expect_equal(result$maintainer[[1]]$family, "Smyth")
  
  # Clean up the temporary directory
  unlink(pkg_source_path, recursive = TRUE)
})

test_that("assess exports for tar file works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    export_calc <- assess_exports(pkg_source_path)
    
    expect_identical(length(export_calc), 1L)
    
    expect_vector(export_calc)
    
    expect_true(checkmate::check_numeric(export_calc, 
                                         any.missing = FALSE)
    )
    testthat::expect_equal(export_calc, 1)
  }
})

test_that("assess exports for tar file works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0006_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    export_calc <- assess_exports(pkg_source_path)
    
    expect_identical(length(export_calc), 1L)
    
    expect_vector(export_calc)
    
    expect_true(checkmate::check_numeric(export_calc, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(export_calc, 0) 
  }
  
})

test_that("assess exports for examples works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    
    testthat::expect_message(
      has_examples <- assess_examples(pkg_name, pkg_source_path),
      glue::glue("{(pkg_name)}: 75.00% of exported functions have examples"),
      fixed = TRUE
    )
    
    has_examples <- assess_examples(pkg_name, pkg_source_path)
    
    expect_identical(length(has_examples), 2L)
    
    expect_vector(has_examples)
    
    expect_true(checkmate::check_list(has_examples, 
                                         any.missing = TRUE)
    )
    
    testthat::expect_length(has_examples, 2L) 
  }
  
})


test_that("assess_examples returns 'No documentation found' and 'no Rd file' when find_rd_for_fun() returns NULL", {
  # Bind the function object
  fn <- assess_examples
  
  # Stub tools::Rd_db to return an empty Rd database (or minimal)
  mockery::stub(fn, "tools::Rd_db", function(dir) list())
  
  # Stub build_rd_index to return a dummy index
  mockery::stub(fn, "build_rd_index", function(db) list(alias_index = new.env(), topic_by_file = character(0)))
  
  # Stub find_rd_for_fun to always return NULL (simulate missing Rd doc)
  mockery::stub(fn, "find_rd_for_fun", function(fun, db, idx) NULL)
  
  # Stub extract_examples_text (won't be called because hit is NULL)
  mockery::stub(fn, "extract_examples_text", function(rd_doc) stop("Should not be called"))
  
  # Stub rd_name (won't be called because hit is NULL)
  mockery::stub(fn, "rd_name", function(rd) stop("Should not be called"))
  
  # Stub asNamespace and getNamespaceExports to simulate exported functions
  mockery::stub(fn, "asNamespace", function(pkg) pkg)
  mockery::stub(fn, "getNamespaceExports", function(ns) c("funA", "funB"))
  
  # Stub getExportedValue to return dummy functions
  mockery::stub(fn, "getExportedValue", function(ns, name) function() NULL)
  
  # Run the function
  result <- fn(pkg_name = "mockpkg", pkg_source_path = "mock/path")
  
  # Assertions
  expect_s3_class(result$data, "data.frame")
  expect_equal(nrow(result$data), 2) # two exported functions
  expect_true(all(result$data$documentation_name == "No documentation found"))
  expect_true(all(result$data$example == "no Rd file"))
  expect_true(all(is.na(result$data$documentation_location)))
  expect_equal(result$example_score, 0) # no examples found
})


test_that("assess_exported_functions_docs returns empty data frame when no exported functions", {
  fn <- assess_exported_functions_docs
  
  # Stub tools::Rd_db to return an empty Rd database
  mockery::stub(fn, "tools::Rd_db", function(dir) list())
  
  # Stub asNamespace and getNamespaceExports to simulate no exports
  mockery::stub(fn, "asNamespace", function(pkg) pkg)
  mockery::stub(fn, "getNamespaceExports", function(ns) character(0))
  
  # Run the function
  expect_message(
    result <- fn(pkg_name = "mockpkg", pkg_source_path = "mock/path"),
    regexp = "mockpkg: no exported functions found; documentation score = 0.00%"
  )
  
  # Assertions
  expect_s3_class(result$data, "data.frame")
  expect_equal(nrow(result$data), 0)
  expect_equal(result$has_docs_score, 0)
})


test_that("assess_exported_functions_docs returns 'No documentation found' when find_rd_for_fun returns NULL", {
  fn <- assess_exported_functions_docs
  
  # Stub tools::Rd_db to return a dummy Rd database
  mockery::stub(fn, "tools::Rd_db", function(dir) list())
  
  # Stub build_rd_index to return a dummy index
  mockery::stub(fn, "build_rd_index", function(db) list(alias_index = new.env(), topic_by_file = character(0)))
  
  # Stub find_rd_for_fun to always return NULL
  mockery::stub(fn, "find_rd_for_fun", function(fun, db, idx) NULL)
  
  # Stub asNamespace and getNamespaceExports to simulate exported functions
  mockery::stub(fn, "asNamespace", function(pkg) pkg)
  mockery::stub(fn, "getNamespaceExports", function(ns) c("funA", "funB"))
  
  # Stub getExportedValue to return dummy functions
  mockery::stub(fn, "getExportedValue", function(ns, name) function() NULL)
  
  # Run the function
  expect_message(
    result <- fn(pkg_name = "mockpkg", pkg_source_path = "mock/path"),
    regexp = "mockpkg: 0.00% of exported functions have documentation"
  )
  
  # Assertions
  expect_s3_class(result$data, "data.frame")
  expect_equal(nrow(result$data), 2)
  expect_true(all(result$data$documentation_name == "No documentation found"))
  expect_true(all(is.na(result$data$documentation_location)))
  expect_equal(result$has_docs_score, 0)
})


test_that("assess exports for missing examples works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  
  # Ensure installs during this test go to a temp lib that stays on .libPaths()
  temp_lib <- file.path(tempdir(), "testthat-lib")
  dir.create(temp_lib, recursive = TRUE, showWarnings = FALSE)
  withr::local_libpaths(new = temp_lib, action = "prefix")
  
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  testthat::skip_if(identical(dp_orig, ""), "Bundled test tarball not found")
  
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    
    
    # First attempt: should be found on .libPaths() because we prefixed temp_lib
    available <- requireNamespace(pkg_name, quietly = TRUE)
    
    # Fallback: load from source if not found (use pkgload if available)
    if (!available && dir.exists(pkg_source_path)) {
      testthat::skip_if_not_installed("pkgload")
      pkgload::load_all(pkg_source_path, quiet = TRUE, helpers = FALSE)
      available <- requireNamespace(pkg_name, quietly = TRUE)
    }
    
    testthat::skip_if_not(
      available,
      message = paste("Package", pkg_name, "not available in .libPaths() in CI")
    )
    
    
    testthat::expect_message(
      has_examples <- assess_examples(pkg_name, pkg_source_path),
      glue::glue("{(pkg_name)}: 0.00% of exported functions have examples"),
      fixed = TRUE
    )
    
    has_examples <- assess_examples(pkg_name, pkg_source_path)
    
    expect_identical(length(has_examples), 2L)
    
    expect_vector(has_examples)
    
    expect_true(checkmate::check_list(has_examples, 
                                      any.missing = TRUE)
    )
    
    testthat::expect_length(has_examples, 2L) 
  }
  
})

test_that("assess exports for packages - no exported functions works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0005_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    
    testthat::expect_message(
      has_examples <- assess_examples(pkg_name, pkg_source_path),
      glue::glue("{(pkg_name)}: no exported functions found; example score = 0.00%"),
      fixed = TRUE
    )
    
    has_examples <- assess_examples(pkg_name, pkg_source_path)
    
    expect_identical(length(has_examples), 2L)
    
    expect_vector(has_examples)
    
    expect_true(checkmate::check_list(has_examples, 
                                      any.missing = TRUE)
    )
    
    testthat::expect_length(has_examples, 2L) 
  }
  
})

test_that("assess exports for help files works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      export_help <- assess_export_help(pkg_name, pkg_source_path),
      glue::glue("All exported functions have corresponding help files in {(pkg_name)}"),
      fixed = TRUE
    )
    
    export_help <- assess_export_help(pkg_name, pkg_source_path)
    
    expect_identical(length(export_help), 1L)
    expect_vector(export_help)
    expect_true(checkmate::check_numeric(export_help, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(export_help, 1L) 
  }
  
})

test_that("assess exports for missing help files works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "stringr-1.5.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      export_help <- assess_export_help(pkg_name, pkg_source_path),
      glue::glue("Some exported functions are missing help files in {(pkg_name)}"),
      fixed = TRUE
    )
    
    export_help <- assess_export_help(pkg_name, pkg_source_path)
    
    expect_identical(length(export_help), 1L)
    expect_vector(export_help)
    expect_true(checkmate::check_numeric(export_help, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(export_help, 0L) 
  }
  
})

test_that("assess exports for no help files works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0006_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # install package locally to ensure test works
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {	
    
    # ensure path is set to package source path
    rcmdcheck_args$path <- pkg_source_path
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    testthat::expect_message(
      export_help <- assess_export_help(pkg_name, pkg_source_path),
      glue::glue("No exported functions in {(pkg_name)}"),
      fixed = TRUE
    )
    
    export_help <- assess_export_help(pkg_name, pkg_source_path)
    
    expect_identical(length(export_help), 1L)
    expect_vector(export_help)
    expect_true(checkmate::check_numeric(export_help, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(export_help, 0L) 
  }
  
})

test_that("assess exports for news works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    
    testthat::expect_message(
      has_news <- assess_news(pkg_name, pkg_source_path),
      glue::glue("{(pkg_name)} has news"),
      fixed = TRUE
    )
    
    has_news <- assess_news(pkg_name, pkg_source_path)
    
    expect_identical(length(has_news), 1L)
    
    expect_vector(has_news)
    
    expect_true(checkmate::check_numeric(has_news, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(has_news, 1L) 
  }
  
})

test_that("assess exports for missing news works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, 
                             fields = c("Package", 
                                        "Version"))
    pkg_name <- pkg_desc$Package
    
    testthat::expect_message(
      has_news <- assess_news(pkg_name, pkg_source_path),
      glue::glue("{(pkg_name)} has no news"),
      fixed = TRUE
    )
    
    has_news <- assess_news(pkg_name, pkg_source_path)
    
    expect_identical(length(has_news), 1L)
    
    expect_vector(has_news)
    
    expect_true(checkmate::check_numeric(has_news, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(has_news, 0L) 
  }
  
})

test_that("assess exports for current news works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <-get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    
    testthat::expect_message(
      news_current <- assess_news_current(pkg_name, pkg_ver,pkg_source_path),
      glue::glue("{(pkg_name)} has current news"),
      fixed = TRUE
    )
    
    news_current <- assess_news_current(pkg_name, pkg_ver, pkg_source_path)
    
    expect_identical(length(news_current), 1L)
    
    expect_vector(news_current)
    
    expect_true(checkmate::check_numeric(news_current, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(news_current, 1L) 
  }
  
})

test_that("assess exports for missing news works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
 
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    
    # Get package name and version
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    
    testthat::expect_message(
      news_current <- assess_news_current(pkg_name, pkg_ver, pkg_source_path),
      glue::glue("{(pkg_name)} has no current news"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      news_current <- assess_news_current(pkg_name, pkg_ver, pkg_source_path),
      glue::glue("{(pkg_name)} has no news"),
      fixed = TRUE
    )
    
    news_current <- assess_news_current(pkg_name, pkg_ver, pkg_source_path)
    
    expect_identical(length(news_current), 1L)
    
    expect_vector(news_current)
    
    expect_true(checkmate::check_numeric(news_current, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(news_current, 0L) 
  }
})

test_that("assess vignettes for tar file with vignettes works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    has_vignettes <- assess_vignettes(pkg_name, pkg_source_path)
    
    expect_identical(length(has_vignettes), 1L)
    
    expect_vector(has_vignettes)
    
    expect_true(checkmate::check_numeric(has_vignettes, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(has_vignettes, 1L) 
  }
  
})

test_that("assess vignettes for tar file with no vignettes works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  
  if (package_installed == TRUE ) {	
    
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    has_vignettes <- assess_vignettes(pkg_name, pkg_source_path)
    
    expect_identical(length(has_vignettes), 1L)
    
    expect_vector(has_vignettes)
    
    expect_true(checkmate::check_numeric(has_vignettes, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_equal(has_vignettes, 0L) 
  }
  
})

test_that("assess code base size for small package works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  if (package_installed == TRUE ) {	
    size_codebase <- assess_size_codebase(pkg_source_path)
    
    expect_identical(length(size_codebase), 1L)
    
    expect_vector(size_codebase)
    
    expect_true(checkmate::check_numeric(size_codebase, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_gt(size_codebase, 0.010) 
  }
  
})

test_that("assess code base size for large package works correctly", {
  
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "stringr-1.5.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {	
    size_codebase <- assess_size_codebase(pkg_source_path)
    
    expect_identical(length(size_codebase), 1L)
    
    expect_vector(size_codebase)
    
    expect_true(checkmate::check_numeric(size_codebase, 
                                         any.missing = FALSE)
    )
    
    testthat::expect_gt(size_codebase, 0.85) 
  }
  
})


test_that("returns NA when both inputs are NA", {
  expect_true(is.na(create_has_ex_docs_score(NA_real_, NA_real_)))
})

test_that("returns the non-NA input when the other is NA", {
  expect_equal(create_has_ex_docs_score(0.75, NA_real_), 0.75)
  expect_equal(create_has_ex_docs_score(NA_real_, 1), 1)
})

test_that("computes the mean for numeric scalars", {
  expect_equal(create_has_ex_docs_score(0, 0), 0)
  expect_equal(create_has_ex_docs_score(1, 1), 1)
  expect_equal(create_has_ex_docs_score(0, 1), 0.5)
  expect_equal(create_has_ex_docs_score(0.75, 1), 0.875)
})

test_that("works with integer inputs", {
  expect_equal(create_has_ex_docs_score(0L, 1L), 0.5)
  expect_type(create_has_ex_docs_score(0L, 1L), "double")  # result should be double
})

test_that("vector inputs are concatenated and averaged (not elementwise)", {
  e <- c(0.2, 0.8)
  d <- c(0.4, 0.6)
  expected <- mean(c(e, d), na.rm = TRUE)  # = 0.5
  expect_equal(create_has_ex_docs_score(e, d), expected)
})

test_that("handles NA within vectors using na.rm=TRUE", {
  e <- c(0.2, NA_real_)
  d <- c(NA_real_, 0.8)
  # concatenated -> c(0.2, NA, NA, 0.8) -> mean of c(0.2, 0.8) = 0.5
  expect_equal(create_has_ex_docs_score(e, d), 0.5)
})

test_that("treats NaN like NA (removed when na.rm=TRUE)", {
  e <- c(0.2, NaN)
  d <- c(NA_real_, 0.8)
  expected <- mean(c(0.2, NaN, NA_real_, 0.8), na.rm = TRUE)  # = 0.5
  expect_equal(create_has_ex_docs_score(e, d), expected)
})

test_that("mixed vector + scalar works (scalar just joins the pool)", {
  e <- c(0.2, 0.8)
  d <- 0.5
  expected <- mean(c(e, d), na.rm = TRUE)  # = 0.5
  expect_equal(create_has_ex_docs_score(e, d), expected)
})

test_that("symmetry: order of arguments does not change the result", {
  e <- c(0.1, 0.5, 0.9)
  d <- c(0.3, NA_real_)
  expect_equal(create_has_ex_docs_score(e, d),
               create_has_ex_docs_score(d, e))
})

test_that("always returns a length-1 double", {
  res <- create_has_ex_docs_score(c(0.2, 0.3), c(0.4, 0.5))
  expect_equal(length(res), 1L)
  expect_type(res, "double")
})

#' Fetch GitHub Repository Data
#'
#' This function retrieves metadata about a GitHub repository, including creation date,
#' stars, forks, and the number of recent commits within the last 30 days.
#'
#' @param owner A character string specifying the owner of the repository.
#' @param repo A character string specifying the name of the repository.
#'
#' A GitHub Personal Access Token (PAT) will be needed for some requests or to help with the rate limit.
#' Use Sys.setenv(GITHUB_TOKEN = "personal_access_token") or store your token in a .Renviron file.
#' The token is passed via the `Authorization` header only if `Sys.getenv("GITHUB_TOKEN")` is non-empty.
#'
#' @return A list containing:
#'   - `created_at`: Creation date of the repository.
#'   - `stars`: Number of stars.
#'   - `forks`: Number of forks.
#'   - `date`: Acquisition date (YYYY-MM-DD).
#'   - `recent_commits_count`: Number of commits in the last 30 days.
#'   - `open_issues`: Number of open issues.
#'
#' @details
#' Repository data is fetched using the GitHub API via `curl` and `jsonlite`.
#'
#' @examples
#' \dontrun{
#' result <- get_github_data("tidyverse", "ggplot2")
#' print(result)
#' }
#'
#' @importFrom curl curl_fetch_memory new_handle
#' @importFrom jsonlite fromJSON
#' @export


get_github_data <- function(owner, repo) {
  
  message(glue::glue("Checking GitHub data for {owner}/{repo}..."))
  
  empty_response <- list(
    created_at = NULL,
    stars = NULL,
    forks = NULL,
    date = NULL,
    recent_commits_count = NULL,
    open_issues = NULL
  )
  
  if (is.na(owner) || owner == "") {
    message("Owner is NA or empty. Returning empty response.")
    return(empty_response)
  }
  
  todays_date <- Sys.Date()
  headers <- c(Accept = "application/vnd.github.v3+json")
  handle <- curl::new_handle()
  curl::handle_setheaders(handle, .list = headers)
  base_url <- "https://api.github.com"
  repo_url <- paste0(base_url, "/repos/", owner, "/", repo)
  
  # Fetch repo details
  repo_resp <- tryCatch({
    resp <- curl::curl_fetch_memory(repo_url, handle = curl::new_handle(httpheader = headers))
    jsonlite::fromJSON(rawToChar(resp$content))
  }, error = function(e) {
    message("Failed to fetch repository details: ", conditionMessage(e))
    return(NULL)
  })
  
  if (is.null(repo_resp)) return(empty_response)
  
  # Parse repo data
  created_at <- repo_resp$created_at
  stars <- repo_resp$stargazers_count
  forks <- repo_resp$forks_count
  open_issues <- repo_resp$open_issues_count
  
  # Fetch commits in last 30 days
  since <- format(todays_date - 30, "%Y-%m-%dT%H:%M:%SZ")
  commits_url <- paste0(base_url, "/repos/", owner, "/", repo, "/commits?since=", since)
  
  commits_resp <- tryCatch({
    resp <- curl::curl_fetch_memory(commits_url, handle = curl::new_handle(httpheader = headers))
    commits_data <- jsonlite::fromJSON(rawToChar(resp$content))
    commits_data
  }, error = function(e) {
    message("Failed to fetch commits: ", conditionMessage(e))
    return(NULL)
  })
  
  if (!is.null(commits_resp)) {
    recent_commits_count <- tryCatch({
      if (is.data.frame(commits_resp)) {
        nrow(commits_resp)
      } else if (!is.null(commits_resp$sha)) {
        length(commits_resp$sha)
      } else {
        0
      }
    }, error = function(e) {
      message("Failed to count recent commits: ", conditionMessage(e))
      0
    })
  } else {
    recent_commits_count <- 0
  }
  
  
  result <- list(
    created_at = format(as.Date(created_at), "%Y-%m-%d"),
    stars = stars,
    forks = forks,
    date = format(todays_date, "%Y-%m-%d"),
    recent_commits_count = recent_commits_count,
    open_issues = open_issues
  )
  
  return(result)
}


#' Retrieve GitHub Commits as Weekly Counts (using curl)
#'
#' This function fetches all commits from a specified GitHub repository
#' using the GitHub API and returns a data frame with weekly commit counts.
#'
#' @param owner Character. The GitHub username or organization that owns the repository.
#' @param repo Character. The name of the GitHub repository.
#' @param years Numeric. Number of years to look back from today's date (default is 1).
#'
#' @return A data frame with four columns:
#' \describe{
#'   \item{week_start}{Start date of the week (class \code{Date}).}
#'   \item{year}{Year of the commits.}
#'   \item{month}{Month of the commits.}
#'   \item{n_commits}{Number of commits during the week.}
#' }
#'
#' @examples
#' \dontrun{
#' # Get commit counts for the past year
#' get_commits_since(owner = "tidyverse", repo = "ggplot2")
#' }
#'
#' @importFrom curl curl_fetch_memory new_handle
#' @importFrom jsonlite fromJSON
#' @importFrom dplyr group_by summarise n arrange mutate select %>%
#' @export
get_commits_since <- function(owner, repo, years = 1) {
  base_url <- "https://api.github.com"
  since <- format(Sys.Date() - years * 365, "%Y-%m-%dT%H:%M:%SZ")
  until <- format(Sys.Date(), "%Y-%m-%dT%H:%M:%SZ")
  
  headers <- c(Accept = "application/vnd.github.v3+json")
  handle <- curl::new_handle()
  curl::handle_setheaders(handle, .list = headers)
  all_commits <- data.frame()
  page <- 1
  more_pages <- TRUE
  
  while (more_pages) {
    url <- paste0(base_url, "/repos/", owner, "/", repo,
                  "/commits?since=", since,
                  "&until=", until,
                  "&per_page=100&page=", page)
    
    resp <- tryCatch({
      curl::curl_fetch_memory(url, handle = curl::new_handle(httpheader = headers))
    }, error = function(e) {
      message("Failed to fetch page ", page, ": ", conditionMessage(e))
      return(NULL)
    })
    
    if (is.null(resp)) break
    
    commits_data <- tryCatch({
      raw <- rawToChar(resp$content)
      parsed <- jsonlite::fromJSON(raw, flatten = TRUE)
      parsed
    }, error = function(e) {
      message("Error parsing JSON on page ", page, ": ", conditionMessage(e))
      return(NULL)
    })

    if (is.null(commits_data) || (is.list(commits_data) && length(commits_data) == 0)) {
      more_pages <- FALSE
    } else {
      all_commits <- dplyr::bind_rows(all_commits, commits_data)
      page <- page + 1
    }
  }
  

  
  if (nrow(all_commits) == 0) {
    return(data.frame(week_start = as.Date(character()), n_commits = integer()))
  }
  
 
  # Extract commit dates safely
  commit_dates <- as.Date(all_commits$commit.author.date)
  
  # Safeguard
  if (length(commit_dates) == 0 || all(is.na(commit_dates))) {
    return(data.frame(week_start = as.Date(character()), n_commits = integer()))
  }
  
  df <- data.frame(date = commit_dates)
  df$week_start <- as.Date(cut(df$date, breaks = "week", start.on.monday = TRUE))
  
  weekly_df <- df %>%
    group_by(week_start) %>%
    summarise(n_commits = n(), .groups = "drop") %>%
    arrange(desc(week_start)) %>%
    mutate(
      year = as.integer(format(week_start, "%Y")),
      month = as.integer(format(week_start, "%m"))
    ) %>%
    select(week_start, year, month, n_commits)
  
  return(weekly_df)
}

#' Count Commits in the Last Months 
#' 
#' Counts the total number of commits from the last `months` months using a 
#' data frame with weekly commit data (column `week_start`).
#'
#' @param df A data frame with `week_start` (Date) and `n_commits` (int) columns.
#' @param months Integer. Number of months to look back (default is 1).
#' @param today today's date.default is Sys.Date()
#'
#' @return Integer. Total number of commits in the time window.
#'
#' @examples
#' \dontrun{
#' df <- get_commits_since("tidyverse", "ggplot2", years = 1)
#' count_commits_last_months(df, months = 3)
#' }
#' @export
count_commits_last_months <- function(df, months = 1, today = Sys.Date()) {
  if (!all(c("year", "month", "n_commits") %in% names(df))) {
    stop("Data frame must have 'year', 'month', and 'n_commits' columns.")
  }
  
  current_year <- as.integer(format(today, "%Y"))
  current_month <- as.integer(format(today, "%m"))
  
  # Compute the earliest allowed year-month
  start_month <- current_month - months
  start_year <- current_year
  
  while (start_month <= 0) {
    start_month <- start_month + 12
    start_year <- start_year - 1
  }
  
  # Filter rows within range
  within_range <- (df$year > start_year) |
    (df$year == start_year & df$month >= start_month)
  
  within_range <- within_range & (
    (df$year < current_year) |
      (df$year == current_year & df$month <= current_month)
  )
  
  filtered_df <- df[within_range, , drop = FALSE]
  
  if (nrow(filtered_df) == 0) {
    return(0)
  }
  
  sum(filtered_df$n_commits, na.rm = TRUE)
}

#' Extract GitHub repository owner from links
#'
#' Given a vector of GitHub links and a package name, this function finds the link
#' corresponding to the package and extracts the GitHub repository owner.
#'
#' @param links A character vector of GitHub URLs.
#' @param pkg_name A string representing the name of the package.
#'
#' @return A string containing the GitHub repository owner, or NA if not found.
#' @keywords internal
get_repo_owner <- function(links, pkg_name) {
  if (!is.null(links) && all(links != "No GitHub link found")) {
    # Remove trailing slashes
    links <- sub("/+$", "", links)
    
    matching_link <- links[grepl(paste0("/", pkg_name, "$"), links)]
    
    if (length(matching_link) > 0) {
      return(sub("https://github.com/([^/]+)/.*", "\\1", matching_link[1]))
    } else {
      return(NA)
    }
  } else {
    return(NA)
  }
}

#' Calculate Average Time to Close GitHub Issues
#'
#' This function uses the GitHub API to retrieve closed issues from a repository and calculates
#' the average time (in hours) it took to close them, excluding pull requests.
#'
#' @param owner Character. GitHub organization or username.
#' @param repo Character. Repository name.
#' @param max_pages Integer. Maximum number of API result pages to query. Each page can contain up to `per_page` issues. Default is 10.
#' @param per_page Integer. Number of issues to request per page (max 100). Default is 100.
#' @param headers Named character vector of HTTP headers to send with the API request. Default sets GitHub API version.
#'
#' @return Numeric. Average time in hours to close issues, or `NA` if no closed issues were found.
#'
#' @examples
#' \dontrun{
#' average_issue_close_time("tidyverse", "ggplot2")
#' }
#' @export
average_issue_close_time <- function(owner, repo, max_pages = 10, per_page = 100,
                                     headers = c(Accept = "application/vnd.github.v3+json")) {
  
  


  #headers <- c(Accept = "application/vnd.github.v3+json")
  handle <- curl::new_handle()
  curl::handle_setheaders(handle, .list = headers)
  durations <- numeric(0)
  page <- 1

  repeat {
    if (page > max_pages) break
    
    url <- paste0("https://api.github.com/repos/", owner, "/", repo,
                  "/issues?state=closed&per_page=", per_page, "&page=", page)
    

    handle <- curl::new_handle()
    curl::handle_setheaders(handle, .list = headers)
    
    resp <- tryCatch({
      curl::curl_fetch_memory(url, handle = handle)
    }, error = function(e) {
      message("Failed to fetch page ", page, ": ", conditionMessage(e))
      return(NULL)
    })
    
    
  
    if (is.null(resp)) break
    
    issues <- tryCatch({
      jsonlite::fromJSON(rawToChar(resp$content), flatten = TRUE)
    }, error = function(e) {
      message("Failed to parse page ", page, ": ", conditionMessage(e))
      return(NULL)
    })
   
    # ✅ Fix: check that 'issues' is a *data.frame* before looping


    if (!is.null(issues$message)) {
      msg <- tolower(issues$message)
      
      if (grepl("abuse detection", msg)) {
        message("GitHub API rate-limited us: ", issues$message)
      } else if (grepl("not found", msg)) {
        message("Repository not found: ", issues$message)
      } else {
        message("GitHub returned a message: ", issues$message)
      }
      
      return(NA)  # <- Clean exit
    }
    
    
    if (!is.data.frame(issues)) {
      message("Issues data is not a data frame. Skipping.")
      return(NA)
    }
    
    
    # Loop through rows safely
    for (i in seq_len(nrow(issues))) {
      issue <- issues[i, ]
      # Safely check for pull_request column
      if (!is.na(issue[["pull_request.url"]]) && nzchar(issue[["pull_request.url"]])) next
      if (is.na(issue$created_at) || is.na(issue$closed_at)) next

      created <- as.POSIXct(issue$created_at, format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
      closed  <- as.POSIXct(issue$closed_at,  format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
      
      if (is.na(created) || is.na(closed)) next

      durations <- c(durations, as.numeric(difftime(closed, created, units = "hours")))
    }
    
    page <- page + 1
  }
 

  
  if (length(durations) == 0) {
    message("No closed issues found.")
    return(NA)
  }

  return(mean(durations, na.rm = TRUE))
}

mock_curl_success <- function(url, handle) {
  if (grepl("/commits", url)) {
    commits <- replicate(10, list(sha = "mocksha"), simplify = FALSE)
    content <- jsonlite::toJSON(commits, auto_unbox = TRUE)
  } else if (grepl("/repos/", url)) {
    repo <- list(
      created_at = "2015-06-17T09:29:49Z",
      stargazers_count = 5674,
      forks_count = 2040,
      open_issues_count = 42
    )
    content <- jsonlite::toJSON(repo, auto_unbox = TRUE)
  } else {
    stop("Unknown URL")
  }
  list(content = charToRaw(content))
}

mock_curl_failure <- function(url, handle) {
  stop("Mocked API call failure")
}

mock_curl_commits_failure <- function(url, handle) {
  if (grepl("/commits", url)) {
    stop("Mocked commits API failure")
  } else if (grepl("/repos/", url)) {
    repo <- list(
      created_at = "2015-06-17T09:29:49Z",
      stargazers_count = 5674,
      forks_count = 2040,
      open_issues_count = 42
    )
    content <- jsonlite::toJSON(repo, auto_unbox = TRUE)
    return(list(content = charToRaw(content)))
  }
}

# get_github_data

test_that("Valid repository returns correct data", {
  mockery::stub(get_github_data, "curl::curl_fetch_memory", mock_curl_success)
  result <- get_github_data("tidyverse", "ggplot2")
  
  expect_type(result, "list")
  expect_named(result, c('created_at', 'stars', 'forks', 'date', 'recent_commits_count', 'open_issues'))
  expect_equal(result$created_at, "2015-06-17")
  expect_equal(result$stars, 5674)
  expect_equal(result$forks, 2040)
  expect_equal(result$open_issues, 42)
  expect_equal(result$recent_commits_count, 10) 
})

test_that("Invalid owner returns empty response", {
  result <- get_github_data("", "ggplot2")
  expect_type(result, "list")
  expect_named(result, c('created_at', 'stars', 'forks', 'date', 'recent_commits_count', 'open_issues'))
  expect_null(result$created_at)
  expect_null(result$stars)
  expect_null(result$forks)
  expect_null(result$recent_commits_count)
  expect_null(result$open_issues)
})

test_that("Non-existent repository returns empty response on API failure", {
  mockery::stub(get_github_data, "curl::curl_fetch_memory", mock_curl_failure)
  result <- get_github_data("tidyverse", "non_existent_repo")
  
  expect_null(result$created_at)
  expect_null(result$stars)
  expect_null(result$forks)
  expect_null(result$recent_commits_count)
  expect_null(result$open_issues)
})

test_that("Commits endpoint failure returns zero recent commits", {
  mockery::stub(get_github_data, "curl::curl_fetch_memory", mock_curl_commits_failure)
  result <- get_github_data("tidyverse", "ggplot2")
  
  expect_equal(result$created_at, "2015-06-17")
  expect_equal(result$stars, 5674)
  expect_equal(result$forks, 2040)
  expect_equal(result$open_issues, 42)
  expect_equal(result$recent_commits_count, 0)
})

test_that("Handles both repo and commits failure gracefully", {
  mockery::stub(get_github_data, "curl::curl_fetch_memory", mock_curl_failure)
  result <- get_github_data("tidyverse", "non_existent_repo")
  
  expect_null(result$created_at)
  expect_null(result$stars)
  expect_null(result$forks)
  expect_null(result$recent_commits_count)
  expect_null(result$open_issues)
})


# average_issue_close_time

mock_curl_closed_issues_success <- function(url, handle) {
  issues <- list(
    list(created_at = "2024-01-01T10:00:00Z", closed_at = "2024-01-02T10:00:00Z"),
    list(created_at = "2024-01-01T08:00:00Z", closed_at = "2024-01-03T08:00:00Z"),
    list(created_at = "2024-01-01T00:00:00Z", closed_at = "2024-01-04T00:00:00Z", `pull_request.url` = "some-url") # excluded
  )
  content <- jsonlite::toJSON(issues, auto_unbox = TRUE)
  list(content = charToRaw(content))
}

mock_curl_issues_empty <- function(url, handle) {
  content <- jsonlite::toJSON(list(), auto_unbox = TRUE)
  list(content = charToRaw(content))
}

mock_curl_issues_not_found <- function(url, handle) {
  issues <- list(
    message = "Not Found",
    documentation_url = "https://docs.github.com/rest/issues/issues#list-repository-issues",
    status = "404"
  )
  content <- jsonlite::toJSON(issues, auto_unbox = TRUE)
  list(content = charToRaw(content))
}


test_that("average_issue_close_time calculates correctly with curl mock", {
  mockery::stub(average_issue_close_time, "curl::curl_fetch_memory", mock_curl_closed_issues_success)
  
  result <- average_issue_close_time("owner", "repo", max_pages = 1)
  
  expect_equal(result, 36)  # (24h + 48h) / 2
})


test_that("average_issue_close_time calculates correctly with mocked API", {
  mock_gh <- function(endpoint, ..., owner, repo, state, per_page, page) {
    if (grepl("/repos/.+/.+/issues", endpoint)) {
      return(list(
        list(
          created_at = "2024-01-01T10:00:00Z",
          closed_at =  "2024-01-02T10:00:00Z"
        ),
        list(
          created_at = "2024-01-01T08:00:00Z",
          closed_at  = "2024-01-03T08:00:00Z"
        ),
        list(
          created_at = "2024-01-01T00:00:00Z",
          closed_at  = "2024-01-04T00:00:00Z",
          pull_request = list(url = "https://api.github.com/repos/org/repo/pulls/123")
        )
      ))
    }
    stop("Unknown endpoint")
  }
  
  mock_curl <- function(url, handle) {
    issues <- list(
      list(
        created_at = "2024-01-01T10:00:00Z",
        closed_at  = "2024-01-02T10:00:00Z"
      ),
      list(
        created_at = "2024-01-01T08:00:00Z",
        closed_at  = "2024-01-03T08:00:00Z"
      ),
      list(
        created_at = "2024-01-01T00:00:00Z",
        closed_at  = "2024-01-04T00:00:00Z",
        `pull_request.url` = "https://api.github.com/repos/org/repo/pulls/123"
      )
    )
    content <- jsonlite::toJSON(issues, auto_unbox = TRUE)
    list(content = charToRaw(content))
  }
  
  mockery::stub(average_issue_close_time, "curl::curl_fetch_memory", mock_curl)
  
  result <- average_issue_close_time("someowner", "somerepo", max_pages = 1)
  
  # (24h + 48h) / 2 = 36h
  expect_equal(result, 36)
})


test_that("average_issue_close_time returns NA when only pull requests returned", {
  mock_curl_only_pull_requests <- function(url, handle) {
    if (grepl("/issues", url)) {
      issues <- list(
        list(
          created_at = "2024-01-01T00:00:00Z",
          closed_at = "2024-01-02T00:00:00Z",
          `pull_request.url` = "https://api.github.com/repos/org/repo/pulls/123"
        )
      )
      content <- jsonlite::toJSON(issues, auto_unbox = TRUE)
      return(list(content = charToRaw(content)))
    } else {
      stop("Unexpected URL in mock")
    }
  }
  
  mockery::stub(average_issue_close_time, "curl::curl_fetch_memory", mock_curl_only_pull_requests)
  
  expect_message(
    result <- average_issue_close_time("owner", "repo", max_pages = 1),
    "No closed issues found",
    fixed = TRUE
  )
  
  expect_true(is.na(result))
})




test_that("average_issue_close_time returns NA when API returns no issues", {
  result <- average_issue_close_time("owner", "repo", max_pages = 1)
  expect_true(is.na(result))
})


# commit history get_commits_since

test_that("get_commits_since handles pagination and aggregates weekly commits", {
  # Simulated JSON responses
  mock_commits_page1 <- jsonlite::toJSON(list(
    list(commit = list(author = list(date = "2024-04-01T12:00:00Z"))),
    list(commit = list(author = list(date = "2024-04-02T12:00:00Z")))
  ), auto_unbox = TRUE)
  
  mock_commits_page2 <- jsonlite::toJSON(list(
    list(commit = list(author = list(date = "2024-04-03T12:00:00Z")))
  ), auto_unbox = TRUE)
  
  mock_commits_empty <- jsonlite::toJSON(list(), auto_unbox = TRUE)
  
  mock_curl <- mockery::mock(
    list(content = charToRaw(mock_commits_page1)),
    list(content = charToRaw(mock_commits_page2)),
    list(content = charToRaw(mock_commits_empty))
  )
  
  mockery::stub(get_commits_since, "curl::curl_fetch_memory", mock_curl)
  
  result <- get_commits_since("someowner", "somerepo", years = 1)
  
  expect_s3_class(result, "data.frame")
  expect_true(all(c("week_start", "n_commits") %in% colnames(result)))
  expect_equal(sum(result$n_commits), 3)  # 3 commits across 2 pages
})



test_that("get_commits_since returns empty data frame when no commits", {
  # Simulate GitHub API returning no commits (empty list)
  mock_empty_commits_json <- jsonlite::toJSON(list(), auto_unbox = TRUE)
  
  mock_curl <- mockery::mock(
    list(content = charToRaw(mock_empty_commits_json))  # First page
  )
  
  mockery::stub(get_commits_since, "curl::curl_fetch_memory", mock_curl)
  
  result <- get_commits_since("someowner", "somerepo", years = 1)
  
  expect_equal(nrow(result), 0)
  expect_true(all(c("week_start", "n_commits") %in% colnames(result)))
})


test_that("get_commits_since correctly groups commits by week", {
  # Simulate API response with commits across different weeks
  mock_commits <- list(
    list(commit = list(author = list(date = "2024-03-31T23:59:59Z"))),  # Sunday
    list(commit = list(author = list(date = "2024-04-01T00:00:00Z")))   # Monday
  )

  mock_json <- jsonlite::toJSON(mock_commits, auto_unbox = TRUE)

  # First call returns data, second returns empty list to stop
  mock_curl <- mockery::mock(
    list(content = charToRaw(mock_json)),
    list(content = charToRaw(jsonlite::toJSON(list(), auto_unbox = TRUE)))
  )

  mockery::stub(get_commits_since, "curl::curl_fetch_memory", mock_curl)

  result <- get_commits_since("someowner", "somerepo", years = 1)

  expect_equal(nrow(result), 2)  # One commit in each week
  expect_equal(sum(result$n_commits), 2)
})


# count commit

df <- data.frame(
  year = c(2025, 2025, 2025, 2025, 2025, 2025, 2025),
  month = c(4, 4, 3, 3, 3, 2, 2),
  n_commits = c(2, 18, 4, 27, 1, 2, 1)
)

mock_today <- as.Date("2025-04-21")

test_that("counts commits for 1 month correctly", {
  result <- count_commits_last_months(df, months = 1, today = mock_today)
  # Should include March + April (4+27+1+2+18) = 52
  expect_equal(result, 52)
})

test_that("counts commits for 2 months correctly", {
  result <- count_commits_last_months(df, months = 2, today = mock_today)
  # Should include February + March + April (all rows) = 55
  expect_equal(result, 55)
})

test_that("returns 0 when no matching months", {
  result <- count_commits_last_months(df, months = 0, today = mock_today)
  # Only April matches exactly
  expect_equal(result, 20)
})


test_that("returns 0 for an empty data frame", {
  empty_df <- data.frame(year = integer(), month = integer(), n_commits = integer())
  result <- count_commits_last_months(empty_df, months = 1, today = mock_today)
  expect_equal(result, 0)
})

test_that("throws an error if required columns are missing", {
  invalid_df <- data.frame(date = as.Date("2025-04-01"), commits = 5)
  expect_error(
    count_commits_last_months(invalid_df, months = 1, today = mock_today),
    "Data frame must have 'year', 'month', and 'n_commits' columns."
  )
})


# get_repo_owner

test_that("extracts correct owner when link matches package name", {
  links <- c("https://github.com/tidyverse/haven", "https://github.com/other/repo")
  pkg_name <- "haven"
  expect_equal(get_repo_owner(links, pkg_name), "tidyverse")
})

test_that("returns NA when no link matches the package name", {
  links <- c("https://github.com/tidyverse/haven")
  pkg_name <- "dplyr"
  expect_true(is.na(get_repo_owner(links, pkg_name)))
})

test_that("returns NA when links are 'No GitHub link found'", {
  links <- "No GitHub link found"
  pkg_name <- "haven"
  expect_true(is.na(get_repo_owner(links, pkg_name)))
})

test_that("returns NA when links are NULL", {
  links <- NULL
  pkg_name <- "haven"
  expect_true(is.na(get_repo_owner(links, pkg_name)))
})

test_that("handles trailing slash in GitHub link", {
  links <- c("https://github.com/tidyverse/haven/")
  pkg_name <- "haven"
  expect_equal(get_repo_owner(links, pkg_name), "tidyverse")
})

test_that("handles multiple trailing slashes", {
  links <- c("https://github.com/tidyverse/haven///")
  pkg_name <- "haven"
  expect_equal(get_repo_owner(links, pkg_name), "tidyverse")
})

#' Install package locally
#'
#' @param pkg_source_path - source path for install local
#'
#' @return logical - package_installed 
#' @examples
#' \dontrun{
#' results <- install_package_local("pkg_source_path")
#' print(results)
#' }
#' @export
install_package_local <- function(pkg_source_path) {
  
  pkg_disp <- basename(pkg_source_path)
  message(glue::glue("installing {pkg_disp} locally"))
  
  # Check if the package source path exists
  if (!dir.exists(pkg_source_path)) {
    message(glue::glue("No such file or directory: {pkg_source_path}"))
    package_installed <- FALSE
  } else if (requireNamespace(pkg_disp, quietly = TRUE)) {
    message(glue::glue("{pkg_disp} is already installed"))
    package_installed <- TRUE
  } else {
    tryCatch(
      {
        remotes::install_local(
          pkg_source_path,
          dependencies = TRUE,
          upgrade = "never",
          force = TRUE,
          quiet = TRUE,
          INSTALL_opts = "--with-keep.source"
        )
        message(glue::glue("{pkg_disp} installed locally"))
        package_installed <- TRUE
      },  
      error = function(cond) {
        message(glue::glue("Local installation issue is: {cond}"))
        message(glue::glue("{pkg_disp} not installed locally"))
        package_installed <- FALSE
      }
    )
  }
  
  return(package_installed)
}

#' Run Coverage Analysis on Package Source and Tests
#'
#' This internal function maps source files to test files, checks for skipped/problematic tests,
#' runs the remaining test files manually, calculates coverage, and optionally generates a report.
#'
#' @param pkg_source_path Character string. Path to the root of the package source directory.
#' @param test_pkg_data List. Output from `check_pkg_tests_and_snaps()` indicating test framework presence.
#'
#'
#' @return returns a list with coverage object and source files with no tests.
#'
#' @examples
#' \dontrun{
#' r = getOption("repos")
#' r["CRAN"] = "http://cran.us.r-project.org"
#' options(repos = r)
#' 
#' dp <- system.file("test-data", 
#'   "MASS_7.3-65.tar.gz", 
#'   package = "risk.assessr")
#' 
#' # set up package
#' install_list <- set_up_pkg(dp)
#'
#' build_vignettes <- install_list$build_vignettes
#' package_installed <- install_list$package_installed
#' pkg_source_path <- install_list$pkg_source_path
#' rcmdcheck_args <- install_list$rcmdcheck_args
#'
#' # install package locally to ensure test works
#' package_installed <- install_package_local(pkg_source_path)
#' package_installed <- TRUE
#'
#' rcmdcheck_args$path <- pkg_source_path
#' test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
#' test_covr_list <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
#'
#' options(old)
#' }
#' @export
run_covr_skip_nstf <- function(pkg_source_path, test_pkg_data) {
  
  cov_env <- new.env(parent = globalenv())
  
  pkg_name <- extract_package_name(pkg_source_path)
  
  # set up path to test directory 
  # convert fs_path to character string for test_dir
  path <- as.character(pkg_source_path)
  testdir <- file.path(path, "tests")
  
  test_path <- get_test_path(test_pkg_data, testdir)
  
  test_path <- normalizePath(test_path)
  
  message(glue::glue("performing source test mapping for {pkg_name}"))
  mapping <- tryCatch({
    get_source_test_mapping_nstf(pkg_source_path, test_path)
  }, error = function(e) {
    cleanup_and_return_null(glue::glue("Error in source-test mapping for {pkg_name}: {e$message}"))
  })
  if (is.null(mapping)) return(NULL)
  
  message(glue::glue("identifying problem tests for {pkg_name}"))
  
  # check for testit testing framework
  if (isTRUE(test_pkg_data$has_testit)) {
    problems <- tryCatch({
      check_covr_skip_testit(pkg_name, mapping, test_path)
    }, error = function(e) {
      cleanup_and_return_null(glue::glue("Error identifying skipped/problematic tests for {pkg_name}: {e$message}"))
    })
  } else {
    problems <- tryCatch({
      check_covr_skip_nstf(pkg_source_path, mapping)
    }, error = function(e) {
      cleanup_and_return_null(glue::glue("Error identifying skipped/problematic tests for {pkg_name}: {e$message}"))
    })
  }
  if (is.null(problems)) return(NULL)
  
  # check for tests to skip
  skip_tests <- problems$test_file[
    !is.na(problems$test_file) &
      !startsWith(problems$issue_type, "warning:") &
      problems$issue_type != "No tests skipped"
  ]
  
  # create df with source files with no tests
  no_tests_df <- get_function_no_tests(mapping) 
  
  # remove source files with no tests
  clean_mapping <- if (anyNA(mapping)) tidyr::drop_na(mapping) else mapping
  
  # create test files clean with testit testing framework and tests testing framework
  if (isTRUE(test_pkg_data$has_testit)) {
    test_files_clean <- file.path(pkg_source_path, "tests/testit", clean_mapping$test_file)
    if (!is.null(skip_tests)| (length(skip_tests) > 0)) {
      test_files_clean <- setdiff(test_files_clean, file.path(pkg_source_path, "tests/testit", skip_tests))
    }
  } else {
    test_files_clean <- file.path(pkg_source_path, "tests", clean_mapping$test_file)
    if (!is.null(skip_tests)| (length(skip_tests) > 0)) {
      test_files_clean <- setdiff(test_files_clean, file.path(pkg_source_path, "tests", skip_tests))
    }
  }
  
  message(glue::glue("loading package and datasets for {pkg_name}"))
  tryCatch({
    pkgload::load_all(pkg_source_path)
    
    pkg_all <- ls(envir = asNamespace(pkg_name), all.names = TRUE)
    
    for (obj in pkg_all) {
      tryCatch({
        assign(obj, get(obj, envir = asNamespace(pkg_name)), envir = cov_env)
      }, error = function(e) {
        message(glue::glue("Failed to assign object {obj}: {e$message}"))
      })
    }
    
    
    datasets <- load_package_datasets(pkg_name, env = cov_env)
    message("Datasets loaded: ", paste(datasets, collapse = ", "))
  }, error = function(e) {
    cleanup_and_return_null(glue::glue("Error loading package or datasets for {pkg_name}: {e$message}"))
  })
  
  message(glue::glue("running coverage manually for {pkg_name}"))
  
  coverage <- tryCatch({
    covr::environment_coverage(
      env = cov_env,
      test_files = test_files_clean
    )
  }, error = function(e) {
    cleanup_and_return_null(glue::glue("Error running coverage for {pkg_name}: {e$message}"), env = cov_env)
  })
  
  
  if (!is.null(coverage)) {
    
    # create coverage list
    res_cov <- tryCatch({
      coverage_list <- covr::coverage_to_list(coverage)
      list(name = pkg_name, coverage = coverage_list, errors = NA, notes = NA)
    },
    error = function(cond){
      coverage_list <- list(filecoverage = NA, totalcoverage = NA_integer_)
      list(
        name = pkg_name, coverage = coverage_list,
        errors = cond,
        notes = NA
      )
    })
    
    total_cov <- as.numeric(res_cov$coverage$totalcoverage/100)
    percent_cov <- as.numeric(res_cov$coverage$totalcoverage)
    
    percent_cov_round <- round(percent_cov, 2)
    message(glue::glue("Total Coverage for {pkg_name} is: {percent_cov_round} %"))
    
    cleanup_and_return_null(env = cov_env)
    
    # structure the return list
    covr_list <- list(
      total_cov = total_cov,
      res_cov = list(
        name = res_cov$name,
        coverage = res_cov$coverage,
        errors = res_cov$errors,
        notes = res_cov$notes
      ),
      functions_no_tests = no_tests_df,
      tests_skipped = skip_tests,
      tests_passing = test_files_clean
    )
    
  } else {
    message("No test coverrage for this configuration")
    covr_list <- list(
      total_cov = 0,
      res_cov = list(
        name = pkg_name,
        coverage = list(
          filecoverage = matrix(0, nrow = 1, dimnames = list("No functions tested")),
          totalcoverage = 0
        ),
        errors = "No testthat or testit configuration",
        notes = NA
      )
    )
    
  } 
  return(covr_list)
}

#' Check for skipped or errored test files (non-testit)
#'
#' This internal function attempts to run test files mapped to source files and
#' identifies any that produce warnings or errors. It does not handle `testit`.
#'
#' @param pkg_source_path Path to the root of the package source.
#' @param mapping A data frame with a column `test_file` listing test file names.
#'
#' @return A data frame listing test files and any issues encountered.
#' @keywords internal
check_covr_skip_nstf <- function(pkg_source_path, mapping) {
  
  test_dir <- file.path(pkg_source_path, "tests")
  
  clean_mapping <- if (anyNA(mapping)) tidyr::drop_na(mapping) else mapping
  
  problems <- data.frame(test_file = character(), issue_type = character(), stringsAsFactors = FALSE)
  
  problem_list <- lapply(clean_mapping$test_file, function(test) {
    test_path <- file.path(test_dir, test)
    
    result <- tryCatch(
      {
        suppressMessages(
          suppressWarnings(
            capture.output(
              source(test_path, local = new.env()),
              file = NULL
            )
          )
        )
        NULL
      },
      warning = function(w) "warning",
      error = function(e) "error"
    )
    
    if (!is.null(result)) {
      return(data.frame(test_file = test, issue_type = result, stringsAsFactors = FALSE))
    } else {
      return(NULL)
    }
  })
  
  problems <- do.call(rbind, problem_list)
  
  if (is.null(problems) || nrow(problems) == 0) {
    problems <- data.frame(
      test_file = NA_character_,
      issue_type = "No tests skipped",
      stringsAsFactors = FALSE
    )
  }
  
  return(problems)
}

#' Run testit tests individually, capturing all warnings and errors, with dynamic package root detection
#'
#' @param pkg_name The package name (not used in this function, but kept for compatibility)
#' @param mapping A data frame with a column `test_file` listing test file names
#' @param test_path The path to the test directory (e.g., tests/testit)
#'
#' @return A data frame with test_file and issue_type columns
#' @keywords internal
#' @noRd
check_covr_skip_testit <- function(pkg_name, mapping, test_path) {
  
  # Helper to find the package root by searching for DESCRIPTION upward
  find_pkg_root <- function(start_path) {
    current <- normalizePath(start_path, mustWork = TRUE)
    while (!file.exists(file.path(current, "DESCRIPTION"))) {
      parent <- dirname(current)
      if (parent == current) stop("Package root (with DESCRIPTION) not found.")
      current <- parent
    }
    current
  }
  
  pkg_root <- find_pkg_root(test_path)
  
  # Prepare mapping
  clean_mapping <- if (anyNA(mapping)) tidyr::drop_na(mapping) else mapping
  
  problems <- data.frame(test_file = character(), issue_type = character(), stringsAsFactors = FALSE)
  
  problem_list <- lapply(clean_mapping$test_file, function(test) {
    test_file_path <- file.path(test_path, test)
    # Set working directory to package root for each test
    original_wd <- getwd()
    setwd(pkg_root)
    on.exit(setwd(original_wd), add = TRUE)
    
    result <- tryCatch(
      {
        capture.output(
          source(test_file_path, local = new.env()),
          file = NULL
        )
        NULL
      },
      warning = function(w) paste("warning:", conditionMessage(w)),
      error = function(e) paste("error:", conditionMessage(e))
    )
    
    if (!is.null(result)) {
      data.frame(test_file = test, issue_type = result, stringsAsFactors = FALSE)
    } else {
      NULL
    }
  })
  
  problems <- do.call(rbind, problem_list)
  
  if (is.null(problems) || nrow(problems) == 0) {
    problems <- data.frame(
      test_file = NA_character_,
      issue_type = "No tests skipped",
      stringsAsFactors = FALSE
    )
  }
  
  return(problems)
}


#' Map R Source Files to Corresponding Test Files
#'
#' This internal helper function scans the `R/` and `tests/` directories of a package
#' and attempts to match each source file with a corresponding test file based on filename similarity.
#'
#' @param pkg_source_path Character string. Path to the root of the package source directory.
#' @param test_dir character string. directory where tests are
#'
#' @return A data frame with two columns:
#' \describe{
#'   \item{source_file}{Name of the source file in the `R/` directory.}
#'   \item{test_file}{Name of the matched test file in the `tests/` directory, or \code{NA} if no match is found.}
#' }
#'
#' @keywords internal
get_source_test_mapping_nstf <- function(pkg_source_path, test_dir) {
  source_dir <- file.path(pkg_source_path, "R")
  # test_dir <- file.path(pkg_source_path, "tests")
  
  src_files <- list.files(source_dir, pattern = "\\.R$", full.names = FALSE)
  test_files <- list.files(test_dir, pattern = "\\.R$", full.names = FALSE)
  
  mapping <- do.call(rbind, lapply(tools::file_path_sans_ext(src_files), function(src) {
    match <- test_files[grepl(src, tools::file_path_sans_ext(test_files), ignore.case = TRUE)]
    data.frame(
      source_file = paste0(src, ".R"),
      test_file = ifelse(length(match) == 0, NA, match[1]),
      stringsAsFactors = FALSE
    )
  }))
  
  return(mapping)
}

test_that("run_covr_skip_nstf returns correct structure on success", {
  # Mock data
  pkg_source_path <- "mock/path"
  test_pkg_data <- list(has_testit = FALSE)
  mapping <- data.frame(
    source_file = "file1.R",
    test_file = "test-file1.R",
    stringsAsFactors = FALSE
  )
  problems <- data.frame(
    test_file = "test-file1.R",
    issue_type = "skipped",
    stringsAsFactors = FALSE
  )
  no_tests_df <- data.frame(
    source_file = "file2.R",
    stringsAsFactors = FALSE
  )
  coverage_obj <- structure(list(), class = "coverage")
  coverage_list <- list(filecoverage = matrix(1), totalcoverage = 80)
  
  # Stubbing dependencies
  mockery::stub(run_covr_skip_nstf, "extract_package_name", "mockpkg")
  mockery::stub(run_covr_skip_nstf, "get_test_path", "mock/tests/testthat")
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", mapping)
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_nstf", problems)
  mockery::stub(run_covr_skip_nstf, "get_function_no_tests", no_tests_df)
  mockery::stub(run_covr_skip_nstf, "pkgload::load_all", NULL)
  mockery::stub(run_covr_skip_nstf, "ls", c("fun1", "fun2"))
  mockery::stub(run_covr_skip_nstf, "get", function(obj, envir) obj)
  mockery::stub(run_covr_skip_nstf, "load_package_datasets", c("dataset1"))
  mockery::stub(run_covr_skip_nstf, "covr::environment_coverage", coverage_obj)
  mockery::stub(run_covr_skip_nstf, "covr::coverage_to_list", coverage_list)
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  # Run function
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  
  # Check structure
  expect_type(result, "list")
  expect_true("total_cov" %in% names(result))
  expect_true("res_cov" %in% names(result))
  expect_true("functions_no_tests" %in% names(result))
  expect_true("tests_skipped" %in% names(result))
  expect_true("tests_passing" %in% names(result))
  expect_equal(result$total_cov, 0.8)
})


test_that("run_covr_skip_nstf returns testit nonzero coverage when at least one test passes", {
  pkg_source_path <- "mockpkg"
  test_pkg_data <- list(has_testit = TRUE)
  pkg_name <- "mockpkg"
  mapping <- data.frame(source_file = "foo.R", test_file = "test-foo.R", stringsAsFactors = FALSE)
  test_path <- "mockpkg/tests/testit"
  
  # Stubs
  mockery::stub(run_covr_skip_nstf, "extract_package_name", pkg_name)
  mockery::stub(run_covr_skip_nstf, "get_test_path", test_path)
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", mapping)
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  
  # Simulate all tests passing (no problems)
  fake_problems <- data.frame(test_file = NA_character_, issue_type = "No tests skipped", stringsAsFactors = FALSE)
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_testit", fake_problems)
  
  mockery::stub(run_covr_skip_nstf, "get_function_no_tests", data.frame())
  mockery::stub(run_covr_skip_nstf, "pkgload::load_all", NULL)
  mockery::stub(run_covr_skip_nstf, "ls", c("fun1"))
  mockery::stub(run_covr_skip_nstf, "get", function(obj, envir) obj)
  mockery::stub(run_covr_skip_nstf, "load_package_datasets", character(0))
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  # Simulate coverage object and coverage list with nonzero coverage
  fake_coverage <- structure(list(), class = "coverage")
  fake_coverage_list <- list(
    filecoverage = matrix(1, nrow = 1, dimnames = list("foo.R")),
    totalcoverage = 100
  )
  mockery::stub(run_covr_skip_nstf, "covr::environment_coverage", fake_coverage)
  mockery::stub(run_covr_skip_nstf, "covr::coverage_to_list", fake_coverage_list)
  
  # Run
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  
  # Check
  expect_type(result, "list")
  expect_true(result$total_cov > 0)
  expect_equal(result$res_cov$coverage$totalcoverage, 100)
  expect_equal(rownames(result$res_cov$coverage$filecoverage), "foo.R")
  expect_equal(result$tests_skipped, character(0)) # No skipped tests
  expect_equal(result$tests_passing, file.path(pkg_source_path, "tests/testit", "test-foo.R"))
})


test_that("run_covr_skip_nstf handles multiple problems from check_covr_skip_testit", {
  pkg_source_path <- "mockpkg"
  test_pkg_data <- list(has_testit = TRUE)
  pkg_name <- "mockpkg"
  mapping <- data.frame(source_file = "foo.R", test_file = "test1.R", stringsAsFactors = FALSE)
  test_path <- "mockpkg/tests/testit"
  
  # Mock dependencies
  mockery::stub(run_covr_skip_nstf, "extract_package_name", pkg_name)
  mockery::stub(run_covr_skip_nstf, "get_test_path", test_path)
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", mapping)
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  # This is the key stub for this test
  fake_problems <- data.frame(
    test_file = c("test1.R", "test2.R"),
    issue_type = c("error: Test error!", "warning: Test warning!"),
    stringsAsFactors = FALSE
  )
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_testit", fake_problems)
  
  # Stub the rest to avoid side effects
  mockery::stub(run_covr_skip_nstf, "get_function_no_tests", data.frame())
  mockery::stub(run_covr_skip_nstf, "pkgload::load_all", NULL)
  mockery::stub(run_covr_skip_nstf, "ls", character(0))
  mockery::stub(run_covr_skip_nstf, "get", function(obj, envir) obj)
  mockery::stub(run_covr_skip_nstf, "load_package_datasets", character(0))
  mockery::stub(run_covr_skip_nstf, "covr::environment_coverage", NULL)
  
  # Run
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  
 
  expect_type(result, "list")
  expect_equal(result$total_cov, 0)
  expect_equal(result$res_cov$name, "mockpkg")
  expect_equal(result$res_cov$coverage$totalcoverage, 0)
  expect_equal(rownames(result$res_cov$coverage$filecoverage), "No functions tested")
  expect_equal(as.numeric(result$res_cov$coverage$filecoverage[1, 1]), 0)
  expect_equal(result$res_cov$errors, "No testthat or testit configuration")
  expect_true(is.na(result$res_cov$notes))
  
})


test_that("run_covr_skip_nstf handles error in coverage_to_list", {
  pkg_source_path <- "mockpkg"
  test_pkg_data <- list(has_testit = TRUE)
  pkg_name <- "mockpkg"
  mapping <- data.frame(source_file = "foo.R", test_file = "test-foo.R", stringsAsFactors = FALSE)
  test_path <- "mockpkg/tests/testit"
  
  # Stubs for setup
  mockery::stub(run_covr_skip_nstf, "extract_package_name", pkg_name)
  mockery::stub(run_covr_skip_nstf, "get_test_path", test_path)
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", mapping)
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_testit", data.frame(test_file = NA_character_, issue_type = "No tests skipped", stringsAsFactors = FALSE))
  mockery::stub(run_covr_skip_nstf, "get_function_no_tests", data.frame())
  mockery::stub(run_covr_skip_nstf, "pkgload::load_all", NULL)
  mockery::stub(run_covr_skip_nstf, "ls", c("fun1"))
  mockery::stub(run_covr_skip_nstf, "get", function(obj, envir) obj)
  mockery::stub(run_covr_skip_nstf, "load_package_datasets", character(0))
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  # Simulate a valid coverage object
  fake_coverage <- structure(list(), class = "coverage")
  mockery::stub(run_covr_skip_nstf, "covr::environment_coverage", fake_coverage)
  
  # Simulate coverage_to_list throwing an error
  mockery::stub(run_covr_skip_nstf, "covr::coverage_to_list", function(x) stop("Simulated coverage_to_list error"))
  
  # Run
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  
  # Check error handling in res_cov
  expect_type(result, "list")
  expect_true("res_cov" %in% names(result))
  expect_equal(result$res_cov$name, pkg_name)
  expect_true(is.na(result$res_cov$coverage$filecoverage))
  expect_true(is.na(result$res_cov$coverage$totalcoverage))
  expect_s3_class(result$res_cov$errors, "simpleError")
})


test_that("find_pkg_root finds DESCRIPTION after traversing up", {
  # Simulate directory structure
  test_path <- "/home/user/project/tests/testit"
  pkg_root <- "/home/user/project"
  
  # Mock normalizePath to just return the input
  mockery::stub(check_covr_skip_testit, "normalizePath", function(x, mustWork = TRUE) x)
  
  # Mock file.exists: first call FALSE, second call TRUE
  file_exists_calls <- 0
  mockery::stub(check_covr_skip_testit, "file.exists", function(path) {
    file_exists_calls <<- file_exists_calls + 1
    if (file_exists_calls == 1) return(FALSE) # Not found in test_path
    if (file_exists_calls == 2) return(TRUE)  # Found in pkg_root
    FALSE
  })
  
  # Mock dirname: first call returns parent, then returns itself
  mockery::stub(check_covr_skip_testit, "dirname", function(path) {
    if (path == test_path) return(pkg_root)
    return(pkg_root)
  })
  
  # Minimal mapping and test file
  mapping <- data.frame(test_file = "test-foo.R", stringsAsFactors = FALSE)
  
  # Mock getwd/setwd/source to avoid side effects
  mockery::stub(check_covr_skip_testit, "getwd", function() pkg_root)
  mockery::stub(check_covr_skip_testit, "setwd", function(x) NULL)
  mockery::stub(check_covr_skip_testit, "source", function(file, local) NULL)
  
  # Should not error and should return "No tests skipped"
  result <- check_covr_skip_testit("mockpkg", mapping, test_path)
  expect_s3_class(result, "data.frame")
  expect_equal(result$issue_type[1], "No tests skipped")
})

test_that("find_pkg_root errors if DESCRIPTION never found", {
  test_path <- "/home/user/project/tests/testit"
  
  # Mock normalizePath to just return the input
  mockery::stub(check_covr_skip_testit, "normalizePath", function(x, mustWork = TRUE) x)
  
  # Mock file.exists: always FALSE
  mockery::stub(check_covr_skip_testit, "file.exists", function(path) FALSE)
  
  # Mock dirname: returns parent until root
  mockery::stub(check_covr_skip_testit, "dirname", function(path) {
    if (path == "/") return("/")
    return("/")
  })
  
  mapping <- data.frame(test_file = "test-foo.R", stringsAsFactors = FALSE)
  # Mock getwd/setwd/source to avoid side effects
  mockery::stub(check_covr_skip_testit, "getwd", function() "/")
  mockery::stub(check_covr_skip_testit, "setwd", function(x) NULL)
  mockery::stub(check_covr_skip_testit, "source", function(file, local) NULL)
  
  # Should error with "Package root (with DESCRIPTION) not found."
  expect_error(
    check_covr_skip_testit("mockpkg", mapping, test_path),
    "Package root \\(with DESCRIPTION\\) not found."
  )
})


test_that("returns NULL if get_source_test_mapping_nstf errors", {
  pkg_source_path <- "mockpkg"
  test_pkg_data <- list(has_testit = FALSE)
  pkg_name <- "mockpkg"
  test_path <- "mockpkg/tests"
  
  mockery::stub(run_covr_skip_nstf, "extract_package_name", pkg_name)
  mockery::stub(run_covr_skip_nstf, "get_test_path", test_path)
  # Simulate error in get_source_test_mapping_nstf
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", function(...) stop("Simulated mapping error"))
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  expect_null(result)
})


test_that("returns NULL if check_covr_skip_testit errors", {
  pkg_source_path <- "mockpkg"
  test_pkg_data <- list(has_testit = TRUE)
  pkg_name <- "mockpkg"
  mapping <- data.frame(source_file = "foo.R", test_file = "test-foo.R", stringsAsFactors = FALSE)
  test_path <- "mockpkg/tests/testit"
  
  mockery::stub(run_covr_skip_nstf, "extract_package_name", pkg_name)
  mockery::stub(run_covr_skip_nstf, "get_test_path", test_path)
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", mapping)
  # Simulate error in check_covr_skip_testit
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_testit", function(...) stop("Simulated testit error"))
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  expect_null(result)
})


test_that("returns NULL if check_covr_skip_nstf errors", {
  pkg_source_path <- "mockpkg"
  test_pkg_data <- list(has_testit = FALSE)
  pkg_name <- "mockpkg"
  mapping <- data.frame(source_file = "foo.R", test_file = "test-foo.R", stringsAsFactors = FALSE)
  test_path <- "mockpkg/tests"
  
  mockery::stub(run_covr_skip_nstf, "extract_package_name", pkg_name)
  mockery::stub(run_covr_skip_nstf, "get_test_path", test_path)
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", mapping)
  # Simulate error in check_covr_skip_nstf
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_nstf", function(...) stop("Simulated nstf error"))
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  result <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)
  expect_null(result)
})

test_that("run_covr_skip_nstf returns NULL if problems fails", {
  mockery::stub(run_covr_skip_nstf, "extract_package_name", "mockpkg")
  mockery::stub(run_covr_skip_nstf, "get_test_path", "mock/tests/testthat")
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", data.frame(source_file = "file1.R", test_file = "test-file1.R"))
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_nstf", NULL)
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  result <- run_covr_skip_nstf("mock/path", list(has_testit = FALSE))
  expect_null(result)
})

test_that("run_covr_skip_nstf handles no coverage", {
  mockery::stub(run_covr_skip_nstf, "extract_package_name", "mockpkg")
  mockery::stub(run_covr_skip_nstf, "get_test_path", "mock/tests/testthat")
  mockery::stub(run_covr_skip_nstf, "get_source_test_mapping_nstf", data.frame(source_file = "file1.R", test_file = "test-file1.R"))
  mockery::stub(run_covr_skip_nstf, "check_covr_skip_nstf", data.frame(test_file = "test-file1.R", issue_type = "skipped"))
  mockery::stub(run_covr_skip_nstf, "get_function_no_tests", data.frame(source_file = "file2.R"))
  mockery::stub(run_covr_skip_nstf, "pkgload::load_all", NULL)
  mockery::stub(run_covr_skip_nstf, "ls", c("fun1", "fun2"))
  mockery::stub(run_covr_skip_nstf, "get", function(obj, envir) obj)
  mockery::stub(run_covr_skip_nstf, "load_package_datasets", c("dataset1"))
  mockery::stub(run_covr_skip_nstf, "covr::environment_coverage", NULL)
  mockery::stub(run_covr_skip_nstf, "cleanup_and_return_null", NULL)
  mockery::stub(run_covr_skip_nstf, "normalizePath", function(path, ...) path)
  
  result <- run_covr_skip_nstf("mock/path", list(has_testit = FALSE))
  expect_type(result, "list")
  expect_equal(result$total_cov, 0)
})


test_that("returns problems for test files with errors and warnings", {
  dummy_mapping <- data.frame(
    source_file = c("modA.R", "modB.R"),
    test_file = c("test-modA.R", "test-modB.R"),
    stringsAsFactors = FALSE
  )
  
  # Stub file.path to return predictable paths
  mockery::stub(check_covr_skip_nstf, "file.path", function(...) paste(..., sep = "/"))
  
  # Stub source to simulate error and warning
  mockery::stub(check_covr_skip_nstf, "source", function(file, local) {
    if (grepl("test-modA", file)) stop("error in test-modA")
    if (grepl("test-modB", file)) warning("warning in test-modB")
  })
  
  result <- check_covr_skip_nstf("/mock/pkg", dummy_mapping)
  
  expect_equal(nrow(result), 1)
  expect_equal(result$test_file, c("test-modA.R"))
  expect_equal(sort(result$issue_type), c("error"))
})

test_that("returns default message when no problems found", {
  dummy_mapping <- data.frame(
    source_file = c("modA.R"),
    test_file = c("test-modA.R"),
    stringsAsFactors = FALSE
  )
  
  # Stub file.path
  mockery::stub(check_covr_skip_nstf, "file.path", function(...) paste(..., sep = "/"))
  
  # Stub source to simulate successful execution
  mockery::stub(check_covr_skip_nstf, "source", function(file, local) NULL)
  
  result <- check_covr_skip_nstf("/mock/pkg", dummy_mapping)
  
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$test_file))
  expect_equal(result$issue_type, "No tests skipped")
})

test_that("handles NA in mapping and drops them", {
  dummy_mapping <- data.frame(
    source_file = c("modA.R", "modB.R"),
    test_file = c(NA, "test-modB.R"),
    stringsAsFactors = FALSE
  )
  
  # Stub file.path
  mockery::stub(check_covr_skip_nstf, "file.path", function(...) paste(..., sep = "/"))
  
  # Stub source to simulate successful execution
  mockery::stub(check_covr_skip_nstf, "source", function(file, local) NULL)
  
  result <- check_covr_skip_nstf("/mock/pkg", dummy_mapping)
  
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$test_file))
  expect_equal(result$issue_type, "No tests skipped")
})


test_that("returns 'No tests skipped' if all tests source fine", {
  mapping <- data.frame(test_file = c("test1.R", "test2.R"), stringsAsFactors = FALSE)
  pkg_source_path <- "mockpkg"
  
  # stub source to always succeed (returns NULL)
  mockery::stub(check_covr_skip_nstf, "source", NULL)
  
  result <- check_covr_skip_nstf(pkg_source_path, mapping)
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$test_file[1]))
  expect_equal(result$issue_type[1], "No tests skipped")
})

test_that("returns problems for tests that error or warn", {
  mapping <- data.frame(test_file = c("test1.R", "test2.R", "test3.R"), stringsAsFactors = FALSE)
  pkg_source_path <- "mockpkg"
  
  # stub source: first is fine, second errors, third warns
  mockery::stub(check_covr_skip_nstf, "source", function(path, local) {
    if (grepl("test2", path)) stop("fail")
    if (grepl("test3", path)) warning("warn")
    NULL
  })
  
  result <- check_covr_skip_nstf(pkg_source_path, mapping)
  expect_equal(nrow(result), 1)
  expect_setequal(result$test_file, c("test2.R"))
  expect_setequal(result$issue_type, c("error"))
})

test_that("drops NA rows in mapping", {
  mapping <- data.frame(test_file = c("test1.R", NA), stringsAsFactors = FALSE)
  pkg_source_path <- "mockpkg"
  
  mockery::stub(check_covr_skip_nstf, "source", NULL)
  mockery::stub(check_covr_skip_nstf, "tidyr::drop_na", data.frame(test_file = "test1.R", stringsAsFactors = FALSE))
  
  result <- check_covr_skip_nstf(pkg_source_path, mapping)
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$test_file[1]))
  expect_equal(result$issue_type[1], "No tests skipped")
})

test_that("handles empty mapping", {
  mapping <- data.frame(test_file = character(0), stringsAsFactors = FALSE)
  pkg_source_path <- "mockpkg"
  
  mockery::stub(check_covr_skip_nstf, "source", NULL)
  
  result <- check_covr_skip_nstf(pkg_source_path, mapping)
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$test_file[1]))
  expect_equal(result$issue_type[1], "No tests skipped")
})


test_that("returns problems for test files with errors", {
  # Mock mapping
  mapping <- data.frame(test_file = c("test1.R", "test2.R"), stringsAsFactors = FALSE)
  test_path <- "mock/tests/testit"
  pkg_name <- "mockpkg"
  
  # Mock find_pkg_root to just return a dummy path
  mockery::stub(check_covr_skip_testit, "find_pkg_root", "mockpkgroot")
  
  # Mock normalizePath to just return the input
  mockery::stub(check_covr_skip_testit, "normalizePath", function(x, mustWork = TRUE) x)
  
  # Mock file.exists to always return TRUE for DESCRIPTION
  mockery::stub(check_covr_skip_testit, "file.exists", function(x) TRUE)
  
  # Mock dirname to just return the parent directory
  mockery::stub(check_covr_skip_testit, "dirname", function(x) "mockpkgroot")
  
  # Mock getwd/setwd to do nothing
  mockery::stub(check_covr_skip_testit, "getwd", function() "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "setwd", function(x) NULL)
  
  # Mock source to throw error for test1.R, warning for test2.R
  mockery::stub(check_covr_skip_testit, "source", function(file, local) {
    if (grepl("test1.R", file)) stop("Test error!")
    if (grepl("test2.R", file)) warning("Test warning!")
    NULL
  })
  
  # Run function
  result <- check_covr_skip_testit(pkg_name, mapping, test_path)
  
  # Check output
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_equal(result$test_file, c("test1.R", "test2.R"))
  expect_true(any(grepl("error:", result$issue_type)))
  expect_true(any(grepl("warning:", result$issue_type)))
})

test_that("returns 'No tests skipped' if all tests pass", {
  mapping <- data.frame(test_file = c("test1.R"), stringsAsFactors = FALSE)
  test_path <- "mock/tests/testit"
  pkg_name <- "mockpkg"
  
  mockery::stub(check_covr_skip_testit, "find_pkg_root", "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "normalizePath", function(x, mustWork = TRUE) x)
  mockery::stub(check_covr_skip_testit, "file.exists", function(x) TRUE)
  mockery::stub(check_covr_skip_testit, "dirname", function(x) "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "getwd", function() "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "setwd", function(x) NULL)
  mockery::stub(check_covr_skip_testit, "source", function(file, local) NULL)
  
  result <- check_covr_skip_testit(pkg_name, mapping, test_path)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 1)
  expect_true(is.na(result$test_file[1]))
  expect_equal(result$issue_type[1], "No tests skipped")
})

test_that("handles mapping with NA values", {
  mapping <- data.frame(test_file = c(NA, "test2.R"), stringsAsFactors = FALSE)
  test_path <- "mock/tests/testit"
  pkg_name <- "mockpkg"
  
  mockery::stub(check_covr_skip_testit, "find_pkg_root", "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "normalizePath", function(x, mustWork = TRUE) x)
  mockery::stub(check_covr_skip_testit, "file.exists", function(x) TRUE)
  mockery::stub(check_covr_skip_testit, "dirname", function(x) "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "getwd", function() "mockpkgroot")
  mockery::stub(check_covr_skip_testit, "setwd", function(x) NULL)
  mockery::stub(check_covr_skip_testit, "source", function(file, local) warning("Test warning!"))
  
  result <- check_covr_skip_testit(pkg_name, mapping, test_path)
  
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 1)
  expect_equal(result$test_file[1], "test2.R")
  expect_true(grepl("warning:", result$issue_type[1]))
})


test_that("mapping matches test files to source files", {
  pkg_source_path <- "mockpkg"
  test_dir <- "mockpkg/tests"
  
  # Mock source and test files
  src_files <- c("foo.R", "bar.R", "baz.R")
  test_files <- c("test-foo.R", "test-bar.R")
  
  # Mock list.files for source_dir and test_dir
  mockery::stub(get_source_test_mapping_nstf, "list.files", function(path, pattern, full.names) {
    if (grepl("R$", path)) return(src_files)
    if (grepl("tests", path)) return(test_files)
    character(0)
  })
  
  mapping <- get_source_test_mapping_nstf(pkg_source_path, test_dir)
  
  expect_s3_class(mapping, "data.frame")
  expect_equal(nrow(mapping), 3)
  expect_equal(mapping$source_file, c("foo.R", "bar.R", "baz.R"))
  expect_equal(mapping$test_file, c("test-foo.R", "test-bar.R", NA))
})

test_that("returns empty mapping if no source files", {
  pkg_source_path <- "mockpkg"
  test_dir <- "mockpkg/tests"
  
  mockery::stub(get_source_test_mapping_nstf, "list.files", function(path, pattern, full.names) {
    if (grepl("R$", path)) return(character(0))
    if (grepl("tests", path)) return(c("test-foo.R"))
    character(0)
  })
  
  mapping <- get_source_test_mapping_nstf(pkg_source_path, test_dir)
  
  expect_equal(nrow(mapping), NULL)
})

test_that("returns NA for test_file if no match", {
  pkg_source_path <- "mockpkg"
  test_dir <- "mockpkg/tests"
  
  mockery::stub(get_source_test_mapping_nstf, "list.files", function(path, pattern, full.names) {
    if (grepl("R$", path)) return(c("foo.R"))
    if (grepl("tests", path)) return(c("test-bar.R"))
    character(0)
  })
  
  mapping <- get_source_test_mapping_nstf(pkg_source_path, test_dir)
  
  expect_equal(mapping$source_file, "foo.R")
  expect_true(is.na(mapping$test_file))
})

test_that("case-insensitive matching works", {
  pkg_source_path <- "mockpkg"
  test_dir <- "mockpkg/tests"
  
  mockery::stub(get_source_test_mapping_nstf, "list.files", function(path, pattern, full.names) {
    if (grepl("R$", path)) return(c("Foo.R"))
    if (grepl("tests", path)) return(c("test-foo.R"))
    character(0)
  })
  
  mapping <- get_source_test_mapping_nstf(pkg_source_path, test_dir)
  
  expect_equal(mapping$source_file, "Foo.R")
  expect_equal(mapping$test_file, "test-foo.R")
})

#' Capture CRAN URL warnings and errors
#'
#' This internal function attempts to open a CRAN URL and read its contents.
#' It captures and classifies warnings and errors related to network issues,
#' such as timeouts or inaccessible URLs.
#'
#' @param cran_url Character. Base URL of the CRAN mirror.
#' @param path Character. Path to the resource on the CRAN server.
#'
#' @return A list with two elements:
#' \describe{
#'   \item{status}{Character string indicating the result type: "success", "url_error", "timeout", "error", "url_warning", "timeout_warning", or "warning".}
#'   \item{message}{Character string with the captured warning or error message, or NULL if successful.}
#' }
#'
#' @keywords internal
#' @examples
#' \dontrun{
#'   capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
#' }
capture_cran_warning <- function(cran_url, path) {
  full_url <- sprintf("%s/%s", cran_url, path)
  result <- tryCatch({
    con <- url(full_url, open = "rb")
    on.exit(close(con), add = TRUE)
    readBin(con, "raw", n = 1e6)
    list(status = "success", message = NULL)
  }, error = function(e) {
    msg <- conditionMessage(e)
    if (grepl("cannot open URL", msg)) {
      return(list(status = "url_error", message = msg))
    } else if (grepl("504 Gateway Timeout", msg)) {
      return(list(status = "timeout", message = msg))
    } else {
      return(list(status = "error", message = msg))
    }
  }, warning = function(w) {
    msg <- conditionMessage(w)
    if (grepl("cannot open URL", msg)) {
      return(list(status = "url_warning", message = msg))
    } else if (grepl("504 Gateway Timeout", msg)) {
      return(list(status = "timeout_warning", message = msg))
    } else {
      return(list(status = "warning", message = msg))
    }
  })
  return(result)
}

# risk metrics for package 1
risk_results_1 <- list(
  pkg_name = "synapser",
  pkg_version = "0.2.1",
  pkg_source_path = "/tmp/RtmpNpDlUz/temp_file_1fe56774aacc/synapser",
  has_bug_reports_url = 1, 
  has_examples = 1,
  has_maintainer = 1,
  size_codebase = 0.06702413,
  has_news = 0,
  has_source_control = 0,
  has_vignettes = 1,
  has_website = 1,
  news_current = 0,
  export_help = 1,
  export_calc = 0.586281,
  check = .7,
  covr = .1084,
  dep_score = .9706878,
  revdep_score = .1260338
)

# create test data for testing update_pscore_results
update_risk_results <- list(
  pkg_name = "synapser",
  pkg_version = "0.2.1",
  pkg_source_path = "/tmp/RtmpNpDlUz/temp_file_1fe56774aacc/synapser",
  has_bug_reports_url = "",
  has_examples = "",
  has_maintainer = "",
  size_codebase = "",
  has_news = "",
  has_source_control = "",
  has_vignettes = "",
  has_website = "",
  news_current = "",
  export_help = "",
  export_calc = "",
  check = "",
  covr = "",
  dep_score = "",
  revdep_score = ""
)

# create test data for testing update_pscore_results
update_pscore <- list(
  export_help = 1,
  has_bug_reports_url = NA,
  size_codebase = .532,
  has_maintainer = 1,
  has_examples = 1,
  has_news = 1,
  has_source_control = 1,
  has_vignettes = 1,
  has_website = 1,
  has_license = NA,
  news_current = 1
)

#create data for testing create_empty_results
sys <- list(
  sysname = "Linux",
  version = "#1 SMP Tue Aug 18 14:50:17 EDT 2020",
  release = "3.10.0-1160.el7.x86_64",
  machine = "x86_64"
)

info <- list(
  sys = sys
)

test_metadata <- list(
  datetime = "2024-06-20 15:47:46",
  executor = "test",
  info = info
)

test_pkg_name <- "there"
test_pkg_ver <- "1.0.1"
test_pkg_source_path <- "/tmp/Rtmp0syfE3/temp_file_4644567f965d/there-1.0.1"
test_comments <- "test run"

# create test data to test score dependencies

package <- c(
  "rprojroot (>= 2.0.2)", "conflicted",
  "covr", "fs", "knitr", "palmerpenguins",      
  "plyr", "readr", "rlang",               
  "rmarkdown", "testthat", "uuid",                
  "withr"       
)

type <- c(
  "Imports",  "Suggests", "Suggests",
  "Suggests", "Suggests", "Suggests",
  "Suggests", "Suggests", "Suggests",
  "Suggests", "Suggests", "Suggests",
  "Suggests"
)

test_deps <- data.frame(package, type)

r = getOption("repos")
r["CRAN"] = "http://cran.us.r-project.org"
options(repos = r)

#create results list for testing
results <- list(
  check = "",
  covr = ""
)

# create deps df for testing
# 
package <- c("rprojroot (>= 2.0.2)", "conflicted", "covr",
             "fs", "knitr", "palmerpenguins", "plyr",
             "readr", "rlang", "rmarkdown", "testthat",
             "uuid", "withr"
             )
type <- c("Imports", "Suggests", "Suggests",
          "Suggests", "Suggests", "Suggests",
          "Suggests", "Suggests", "Suggests",
          "Suggests", "Suggests", "Suggests",
          "Suggests"
          ) 
deps_df <- data.frame(package, type)


fake_tar <- function() {
  tf <- tempfile(fileext = ".tar.gz")
  file.create(tf)
  tf
}

make_tarball <- function(destfile, pkg_name = "pkg", version = "0.1") {
  pkg_dir <- file.path(tempdir(), pkg_name)
  dir.create(pkg_dir, showWarnings = FALSE, recursive = TRUE)
  desc <- sprintf(
    "Package: %s\nVersion: %s\nAuthors@R: person('First', 'Last', email = 'first@example.com', role = 'aut')\n",
    pkg_name, version
  )
  writeLines(desc, file.path(pkg_dir, "DESCRIPTION"))
  old_wd <- getwd()
  on.exit(setwd(old_wd), add = TRUE)
  setwd(tempdir())
  utils::tar(destfile, files = pkg_name, tar = "internal")
  invisible(0)
}

# Define the generic function
s3_function_no_body <- function(data, ...) {
  UseMethod("s3_function_no_body")
}

#' Filter the first three rows of column y for dataframe_with_y class
#' but body commented out
#'
#' @param data A dataframe of class dataframe_with_y
#' @return A dataframe with filtered values
#' @export
#' @importFrom magrittr %>%
#' @examples
#' \donttest{
#' library(dplyr)
#' library(magrittr)
#'
#' df <- data.frame(x = 1:4, y = 1:4)
#' class(df) <- "dataframe_with_y"
#'
#' filtered_y <- filter_first_three_rows(df)
#' print(filtered_y)
#' }
s3_function_no_body.dataframe_with_y <- function(data) {
  # library(dplyr)
  # y <- NULL
  # class(data) <- "data.frame"  # Remove the custom class
  # dplyr::filter(data, row_number() <= 3) %>% select(y)
}

options(repos = c(CRAN = "https://cran.r-project.org"))

# Minimal results object required by write_summary_report()
fake_results <- list(
  results = list(
    pkg_name    = "mockpkg",
    pkg_version = "0.1.0"
  ),
  extra = "anything"
)

#' Create empty results
#' 
#' @description This creates an empty results list
#' for all risk metrics
#'
#' @param pkg_name package name
#' @param pkg_ver package version
#' @param pkg_source_path package source path
#' @param metadata list with metadata
#' 
#' @return - list with empty risk profile values
#' 
#' @keywords internal
create_empty_results <- function(pkg_name,
                                 pkg_ver,
                                 pkg_source_path,
                                 metadata) {

results <- list(
  pkg_name = pkg_name,
  pkg_version = pkg_ver,
  pkg_source_path = pkg_source_path,
  date_time = metadata$datetime,
  executor = metadata$executor,
  sysname = metadata$info$sys$sysname,
  version = metadata$info$sys$version,
  release = metadata$info$sys$release,
  machine = metadata$info$sys$machine,
  comments = " ",
  has_bug_reports_url = "",
  license = "",
  license_name = "",
  has_examples = "",
  has_docs = "",
  has_ex_docs_score = " ",
  has_maintainer = "",
  size_codebase = "",
  has_news = "",
  has_source_control= "",
  has_vignettes = "",
  has_website = "",
  news_current = "",
  export_help = "",
  export_calc = "",
  check = "",
  covr = "",
  dependencies = "",
  suggested_deps = "",
  rev_deps = "",
  author = "",
  host = "",
  github_data = "",
  version_info = "",
  download = ""
)
return(results)
}

test_that("test create_empty_results", {

  results <- create_empty_results(test_pkg_name,
                                                      test_pkg_ver,
                                                      test_pkg_source_path,
                                                      test_metadata)

  expect_identical(length(results), 35L)
  
  expect_true(checkmate::check_list(results, all.missing = FALSE))
  
  expect_true(checkmate::check_list(results, any.missing = TRUE))
})

#' Check for Vignette Folder and .Rmd Files in a .tar File
#'
#' This function checks if a given .tar file contains a 'vignettes' folder
#' and if there is at least one .Rmd file within that folder. If both 'vignettes'
#' and 'inst/doc' folders exist, the function will return \code{FALSE}.
#'
#' @param package_dir A character string specifying the path to the package to be checked.
#' 
#' @return A logical value: \code{TRUE} if the 'R' folder exists, \code{FALSE} otherwise.
#'
#' @details The function checks if the specified folder exists. If  'R'
#' folder does not exist, the function returns \code{FALSE}. 
#' If the R folder exists, the function returns \code{TRUE}. 
#' @keywords internal
contains_r_folder <- function(package_dir) {
  
  if (fs::dir_exists(fs::path(package_dir, "R"))) {
    r_folder <- TRUE
  } else {
    r_folder <- FALSE
  }
  
  return(r_folder)
}

test_that("contains_r_folder works correctly", {

  # Create a temporary directory for testing
  temp_dir <- tempdir()

  # Create a toy dataset
  toy_dataset <- function(dir_name) {
    fs::dir_create(fs::path(temp_dir, dir_name))
  }

  # Case 1: Directory with 'R' folder
  toy_dataset("package_with_R")
  fs::dir_create(fs::path(temp_dir, "package_with_R", "R"))
  expect_true(contains_r_folder(fs::path(temp_dir, "package_with_R")))
  
  # Case 2: Directory without 'R' folder
  toy_dataset("package_without_R")
  expect_false(contains_r_folder(fs::path(temp_dir, "package_without_R")))
  
  # Clean up
  fs::dir_delete(fs::path(temp_dir, "package_with_R"))
  fs::dir_delete(fs::path(temp_dir, "package_without_R"))
})

#' Run R CMD CHECK
#'
#' @param pkg_source_path directory path to R project
#' @param rcmdcheck_args list of arguments to pass to `rcmdcheck::rcmdcheck`
#'
#' @details
#' rcmdcheck takes either a tarball or an installation directory.
#'
#' The basename of `pkg_source_path` should be the package name and version pasted together
#'
#' The returned score is calculated via a weighted sum of notes (0.10), warnings (0.25), and errors (1.0). 
#' It has a maximum score of 1 (indicating no errors, notes or warnings)
#' and a minimum score of 0 (equal to 1 error, 4 warnings, or 10 notes). 
#' This scoring methodology is expanded from [riskmetric::metric_score.pkg_metric_r_cmd_check()] by including
#' forbidden notes, 
#' @keywords internal
run_rcmdcheck <- function(pkg_source_path, rcmdcheck_args) {
  
  # We never want the rcmdcheck to fail
  rcmdcheck_args$error_on <- "never"
  
  # run rcmdcheck
  pkg_name <- basename(pkg_source_path)
  
  message(glue::glue("running rcmdcheck for {pkg_name}"))
  
  # Use tryCatch to handle potential errors during rcmdcheck
  res_check <- tryCatch({
    do.call(rcmdcheck::rcmdcheck, rcmdcheck_args)
  }, error = function(e) {
    message(glue::glue("rcmdcheck for {pkg_name} failed to run: {e$message}"))
    return(list(notes = character(0), warnings = character(0), errors = e$message))
  })
  
  
  # Apply forbidden notes check
  res_check <- check_forbidden_notes(res_check, pkg_name)
  
  # Initialize check_score to 0 in case res_check is NULL or calculation fails
  check_score <- 0
  
  # If res_check is not NULL, attempt to calculate the check score
  if (!is.null(res_check)) {
    
    sum_vars <- c(notes = length(res_check$notes), warnings = length(res_check$warnings), errors = length(res_check$errors))
    score_weightings <- c(notes = 0.1, warnings = 0.25, errors = 1)
    check_score <- 1 - min(c(sum(score_weightings * sum_vars), 1))
  }
  
  if(check_score == 1){
    message(glue::glue("rcmdcheck for {pkg_name} passed"))
  }else if(check_score < 1 && check_score > 0){
    message(glue::glue("rcmdcheck for {pkg_name} passed with warnings and/or notes"))
  }else if(check_score == 0){
    message(glue::glue("rcmdcheck for {pkg_name} failed. Read in the rcmdcheck output to see what went wrong: "))
  }
  
  check_list <- list(
    res_check = res_check,
    check_score = check_score
  )
  
  return(check_list)
}

#' Reclassify Forbidden Notes as Errors in rcmdcheck Results
#'
#' This internal helper function scans the `notes` field of an `rcmdcheck` result object
#' for specific patterns that indicate more serious issues. If any of these patterns are found,
#' the corresponding notes are reclassified as errors by moving them from the `notes` field
#' to the `errors` field.
#'
#' @param res_check A list representing the result of an `rcmdcheck` run. It should contain
#'   at least the elements `notes`, `warnings`, and `errors`, each being a character vector.
#' @param pkg_name name of the package   
#'
#' @return A modified version of `res_check` where certain notes matching forbidden patterns
#'   are moved to the `errors` field.
#'
#' @export
#'
#' @examples
#' \donttest{
#' pkg_name <- "foo"
#' res_check <- list(
#'   notes = c("'foo' import not declared from: 'bar'",
#'             "Namespace in Imports field not imported from: 'baz'",
#'             "no visible global function definition for 'qux'",
#'             "some harmless note"),
#'   warnings = character(0),
#'   errors = character(0)
#' )
#' updated <- check_forbidden_notes(res_check, pkg_name)
#' print(updated$errors)  # Should include the first three notes
#' print(updated$notes)   # Should include only the harmless note
#' }
check_forbidden_notes <- function(res_check, pkg_name) {
  if (is.null(res_check$notes) || length(res_check$notes) == 0) {
    message(glue::glue("No forbidden notes for {pkg_name}"))
  } else {
    forbidden_patterns <- c(
      "'.*' import not declared from",
      "Namespace in Imports field not imported from",
      "no visible global .+ definition"
    )
    
    forbidden_matches <- sapply(forbidden_patterns, function(pattern) {
      grepl(pattern, res_check$notes, perl = TRUE)
    })
    
    if (length(res_check$notes) == 1) {
      is_forbidden <- as.logical(forbidden_matches)
    } else {
      is_forbidden <- apply(forbidden_matches, 1, any)
    }
    
    if (any(is_forbidden)) {
      res_check$errors <- c(res_check$errors, res_check$notes[is_forbidden])
      res_check$notes <- res_check$notes[!is_forbidden]
    } else {
      message(glue::glue("No forbidden notes for {pkg_name}"))
    }
  }
  
  return(res_check)
}

#' Run covr and potentially save results to disk
#'
#' @param pkg_source_path package installation directory
#' @param timeout Timeout to pass to [callr::r_safe()] when running covr.
#'
#'
#' @return list with total coverage and function coverage
#' @keywords internal
run_coverage <- function(pkg_source_path, timeout = Inf) {
  
  pkg_name <- basename(pkg_source_path)
  
  message(glue::glue("running code coverage for {pkg_name}"))
  
  # run covr
  res_cov <- tryCatch({
    coverage_list <- run_covr(pkg_source_path, timeout)
    
    # If no testable functions are found in the package, `filecoverage` and `totalcoverage`
    # will yield logical(0) and NaN respectively. Coerce to usable format
    if(is.na(coverage_list$totalcoverage)){
      if(rlang::is_empty(coverage_list$filecoverage) && is.logical(coverage_list$filecoverage)){
        coverage_list$totalcoverage <- 0
        notes <- "no testable functions found"
      }else{
        message(glue::glue("Total coverage returned NaN. This likely means the package had non-standard characteristics."))
        notes <- NA
      }
    }else{
      notes <- NA
    }
    
    list(name = pkg_name, coverage = coverage_list, errors = NA, notes = notes)
  },
  error = function(cond){
    coverage_list <- list(filecoverage = NA, totalcoverage = NA_integer_)
    list(
      name = pkg_name, coverage = coverage_list,
      errors = cond,
      notes = NA
    )
  })
  
  if(is.na(res_cov$coverage$totalcoverage)) {
    message(glue::glue("code coverage for {pkg_name} unsuccessful"))
  } else {
    message(glue::glue("code coverage for {pkg_name} successful"))
  }  
  
  # return total coverage as fraction
  total_cov <- as.numeric(res_cov$coverage$totalcoverage/100)
  
  if(is.na(total_cov)){
    message(glue::glue("R coverage for {pkg_name} failed. Read in the covr output to see what went wrong: "))
  }
  
  if(!is.na(res_cov$notes)){
    message(glue::glue("R coverage for {pkg_name} had notes: {res_cov$notes}"))
  }
  
  covr_list <- list(
    total_cov = total_cov,
    res_cov = res_cov
  )
  return(covr_list)
}

#' Run covr in subprocess with timeout
#'
#' @param path - path to source file
#' @param timeout - length of timeout - set to Inf
#' @keywords internal
run_covr <- function(path, timeout) {
  callr::r_safe(
    function(p) {
      covr::coverage_to_list(covr::package_coverage(p, type = "tests"))
    },
    args = list(path),
    libpath = .libPaths(),
    repos = NULL,
    package = FALSE,
    user_profile = FALSE,
    error = "error",
    timeout = timeout
  )
}

test_that("forbidden notes are reclassified as errors", {
  res_check <- list(
    notes = c(
      "'foo' import not declared from: 'bar'",
      "Namespace in Imports field not imported from: 'baz'",
      "no visible global function definition for 'qux'",
      "some harmless note"
    ),
    warnings = character(0),
    errors = character(0)
  )
  
  updated <- check_forbidden_notes(res_check, pkg_name = "mockpkg")
  
  expect_length(updated$notes, 1)
  expect_match(updated$notes, "some harmless note")
  expect_length(updated$errors, 3)
  expect_true(any(grepl("import not declared", updated$errors)))
  expect_true(any(grepl("Namespace in Imports", updated$errors)))
  expect_true(any(grepl("no visible global", updated$errors)))
})

test_that("no forbidden notes leaves notes and errors unchanged", {
  res_check <- list(
    notes = c("this is a harmless note", "another safe note"),
    warnings = character(0),
    errors = character(0)
  )
  
  updated <- check_forbidden_notes(res_check, pkg_name = "mockpkg")
  
  expect_equal(updated$notes, res_check$notes)
  expect_equal(updated$errors, character(0))
})

test_that("empty notes does not cause errors", {
  res_check <- list(
    notes = character(0),
    warnings = character(0),
    errors = character(0)
  )
  
  updated <- check_forbidden_notes(res_check, pkg_name = "mockpkg")
  
  expect_equal(updated$notes, character(0))
  expect_equal(updated$errors, character(0))
})

test_that("run_coverage returns correct fraction on success", {
  pkg_path <- "path/to/test.pkg"
  
  # Mock run_covr to return 75% coverage
  mock_cov_list <- list(totalcoverage = 75, filecoverage = c(a = 75))
  mockery::stub(run_coverage, "run_covr", mock_cov_list)
  
  testthat::expect_message(
    res <- run_coverage(pkg_path),
    "code coverage for test.pkg successful"
  )
  
  # Check fraction conversion (75/100 = 0.75)
  testthat::expect_equal(res$total_cov, 0.75)
  testthat::expect_equal(res$res_cov$name, "test.pkg")
})

test_that("run_coverage handles packages with no testable functions", {
  pkg_path <- "path/to/empty.pkg"
  
  # Mock the specific state: NaN total and logical(0) file coverage
  mock_cov_list <- list(totalcoverage = NaN, filecoverage = logical(0))
  mockery::stub(run_coverage, "run_covr", mock_cov_list)
  
  testthat::expect_message(
    res <- run_coverage(pkg_path),
    "R coverage for empty.pkg had notes: no testable functions found"
  )
  
  # Code coerces NaN to 0 in this specific case
  testthat::expect_equal(res$total_cov, 0)
  testthat::expect_equal(res$res_cov$notes, "no testable functions found")
})

test_that("run_coverage catches errors from run_covr", {
  pkg_path <- "path/to/failing.pkg"
  
  # Force run_covr to throw an error
  mockery::stub(run_coverage, "run_covr", function(...) stop("Process Timeout"))
  
  testthat::expect_message(
    res <- run_coverage(pkg_path),
    "code coverage for failing.pkg unsuccessful"
  )
  
  # Verify error capture
  testthat::expect_true(is.na(res$total_cov))
  testthat::expect_s3_class(res$res_cov$errors, "error")
  testthat::expect_match(res$res_cov$errors$message, "Process Timeout")
})

test_that("run_coverage identifies non-standard package failures such as NaN coverage with non-empty filecoverage", {
  pkg_path <- "path/to/weird.pkg"
  
  # NaN total but filecoverage is NOT an empty logical
  mock_cov_list <- list(totalcoverage = NaN, filecoverage = list(complex = 1))
  mockery::stub(run_coverage, "run_covr", mock_cov_list)
  
  testthat::expect_message(
    res <- run_coverage(pkg_path),
    "Total coverage returned NaN"
  )
  
  testthat::expect_true(is.na(res$total_cov))
  testthat::expect_true(is.na(res$res_cov$notes))
})

test_that("run_rcmdcheck returns score of 1 when no issues found", {
  pkg_path <- "path/to/test.pkg"
  args <- list(quiet = TRUE)
  
  # Mock rcmdcheck to return empty results
  mock_res <- list(notes = character(0), warnings = character(0), errors = character(0))
  mockery::stub(run_rcmdcheck, "rcmdcheck::rcmdcheck", mock_res)
  
  # Mock forbidden notes to return input as-is
  mockery::stub(run_rcmdcheck, "check_forbidden_notes", function(x, y) x)
  
  testthat::expect_message(
    res <- run_rcmdcheck(pkg_path, args),
    "rcmdcheck for test.pkg passed"
  )
  
  testthat::expect_equal(res$check_score, 1)
  testthat::expect_identical(res$res_check, mock_res)
})

test_that("run_rcmdcheck handles errors and calculates score of 0", {
  pkg_path <- "path/to/test.pkg"
  
  # Mock rcmdcheck to return one error
  mock_res <- list(notes = character(0), warnings = character(0), errors = "One critical error")
  mockery::stub(run_rcmdcheck, "rcmdcheck::rcmdcheck", mock_res)
  mockery::stub(run_rcmdcheck, "check_forbidden_notes", function(x, y) x)
  
  testthat::expect_message(
    res <- run_rcmdcheck(pkg_path, list()),
    "rcmdcheck for test.pkg failed"
  )
  
  # Error weighting is 1.0, so 1 - 1.0 = 0
  testthat::expect_equal(res$check_score, 0)
})

test_that("run_rcmdcheck accounts for forbidden notes escalated to errors", {
  pkg_path <- "path/to/test.pkg"
  
  # Initial check returns 1 note
  mock_res <- list(notes = "no visible global function definition", warnings = character(0), errors = character(0))
  mockery::stub(run_rcmdcheck, "rcmdcheck::rcmdcheck", mock_res)
  
  # The real check_forbidden_notes should move that note to errors
  # (No stub for check_forbidden_notes here so we test the integration)
  
  res <- run_rcmdcheck(pkg_path, list())
  
  # Note becomes error (weight 1.0), so score should be 0, not 0.9
  testthat::expect_equal(res$check_score, 0)
  testthat::expect_length(res$res_check$errors, 1)
})

#' Parse DCF of description file
#'
#' @param path pkg_ref path
#' 
#' @keywords internal
parse_dcf_dependencies <- function(path){
  
  deps <- desc::desc_get_deps(file.path(path, "DESCRIPTION"))
  deps <- deps[, c("type", "package"), drop = FALSE]
  
  # Check CRAN availability before filtering base/recommended packages
  cran_check <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  if (is.null(cran_check$message)) {
    deps <- remove_base_packages(deps)
  } else {
    message("CRAN not available, skipping base/recommended package filtering: ", cran_check$message)
  }
  
  rownames(deps) <- NULL
  return(deps)
}

#' Helper function to remove base and recommended packages
#'
#' @param df Data frame of dependencies of a package.
#' 
#' @keywords internal
remove_base_packages <- function(df) {
  # Create a temporary file to store the downloaded RDS
  local <- tempfile(fileext = ".rds")
  on.exit(unlink(local), add = TRUE)
  
  # Try downloading the CRAN package metadata
  cp <- tryCatch({
    utils::download.file(
      "https://cran.r-project.org/web/packages/packages.rds",
      local,
      mode = "wb",
      quiet = TRUE
    )
    readRDS(local)
  }, error = function(e) {
    message("Failed to download or read CRAN package metadata: ", e$message)
  })
  
  # Identify base and recommended packages from the metadata
  inst_priority <- cp[, "Priority"]
  inst_is_base_rec <- !is.na(inst_priority) & inst_priority %in% c("base", "recommended")
  base_rec_pkgs <- cp[inst_is_base_rec, "Package"]
  
  # Filter out "R" and base/recommended packages from the input df
  deps <- df[!(df$package == "R" | df$package %in% base_rec_pkgs), ]
  
  return(deps)
}


#' Get dependencies
#'
#' @param pkg_source_path package source path
#' 
#' @keywords internal
get_dependencies <- function(pkg_source_path) {
  
  pkg_name <- basename(pkg_source_path)
  message(glue::glue("getting package dependencies for {pkg_name}"))
  deps <- parse_dcf_dependencies(pkg_source_path)
  message(glue::glue("package dependencies successful for {pkg_name}"))
  return(deps)
}


#' Find Reverse Dependencies of a CRAN Package
#'
#' This function finds the reverse dependencies of a CRAN package, i.e., packages that depend on the specified package.
#'
#' @param pkg A character string representing the name of the package.
#' @param dependencies A character vector specifying the types of dependencies to consider. Default is c("Depends", "Imports", "Suggests", "LinkingTo").
#' @param recursive A logical value indicating whether to find dependencies recursively. Default is FALSE.
#' @param ignore A character vector of package names to ignore. Default is NULL.
#' @param installed A matrix of installed packages.
#'
#' @return deps - A sorted character vector of package names that depend on the specified package.
#'
#' @examples
#' \dontrun{
#' cran_revdep("ggplot2")
#' }
#'
#' @export
cran_revdep <- function(pkg,
                   dependencies = c("Depends", "Imports", "Suggests", "LinkingTo"),
                   recursive = FALSE, ignore = NULL,
                   installed = NULL) {
  if (missing(pkg)) {
    message("Package name is required")
    deps <- NULL
  } else {
    if (is.null(installed)) {
      installed <- cran_packages()
    }
    
    deps <- dependsOnPkgs(pkg, dependencies = dependencies,
                          recursive = recursive,
                          installed = installed)
    deps <- setdiff(deps, ignore)
    sort(deps, method = "radix")
  }
  return(deps)
}



#' Determine Packages that Depend on Given Packages
#'
#' This function identifies packages that depend on the specified packages,
#' considering various types of dependencies (e.g., strong, most, all).
#'
#' @param pkgs A character vector of package names to check dependencies for.
#' @param dependencies A character string specifying the types of dependencies
#'   to consider. Can be "strong", "most", "all", or a custom vector of dependency types.
#' @param recursive A logical value indicating whether to recursively check dependencies.
#' @param lib.loc A character vector of library locations to search for installed packages.
#' @param installed A matrix of installed packages, obtained from cran_packages function.
#'   .
#'
#' @return A character vector of package names that depend on the specified packages.
#'
#' @examples
#' \dontrun{
#' installed <- cran_packages()
#' dependsOnPkgs("here", installed = installed)
#' }
#'
#' @export
dependsOnPkgs <- function(pkgs, dependencies = "most",
                          recursive = TRUE, lib.loc = NULL,
                          installed = NULL) {
  dependencies <- expand_dependency_type_spec(dependencies)
  
  av <- installed[, dependencies, drop = FALSE]
  rn <- as.character(installed[, "Package"])
  
  need <- apply(av, 1L, function(x)
    any(pkgs %in% clean_up_dependencies(x)))
  
  uses <- rn[need]
  
  if (recursive) {
    p <- pkgs
    repeat {
      p <- unique(c(p, uses))
      need <- apply(av, 1L, function(x)
        any(p %in% clean_up_dependencies(x)))
      uses <- unique(c(p, rn[need]))
      if (length(uses) <= length(p)) break
    }
  }
  
  setdiff(uses, pkgs)
}


#' Expand Dependency Type Specification
#'
#' Expands a dependency type specification into a character vector of dependency types.
#'
#' @param x A character string specifying the dependency type ("strong", "most", "all", or custom).
#'
#' @return A character vector of dependency types.
#'
#' @keywords internal
expand_dependency_type_spec <- function(x) {
  if (identical(x, "strong"))
    c("Depends", "Imports", "LinkingTo")
  else if (identical(x, "most"))
    c("Depends", "Imports", "LinkingTo", "Suggests")
  else if (identical(x, "all"))
    c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")
  else
    x
}

#' Clean Up Dependencies
#'
#' Cleans up a list of dependency strings by extracting unique package names.
#'
#' @param x A character vector of dependency strings.
#'
#' @return A character vector of unique package names.
#'
#' @keywords internal
clean_up_dependencies <- function(x) {
  if (length(x) == 0) return(character())
  unique.default(unlist(lapply(x, extract_dependency_package_names)))
}


#' Extract Package Names from a Dependency String
#'
#' Parses a single package dependency string and extracts valid package names,
#' excluding "R" and ignoring version constraints or comments.
#'
#' @param x A character string representing a package dependency specification,
#' such as `"pkgA (>= 1.0), pkgB, R (>= 3.5.0)"`.
#'
#' @return A character vector of package names extracted from the input string.
#' The result excludes "R" and any version information.
#'
#' @keywords internal
extract_dependency_package_names <- function(x) {
  if (is.na(x)) return(character())
  
  # Split by commas
  parts <- strsplit(x, ",", fixed = TRUE)[[1L]]
  
  # Extract the first alphanumeric token from each part
  pkgs <- sub("[[:space:]]*([[:alnum:].]+).*", "\\1", parts)
  
  # Trim whitespace from each package name
  pkgs <- trimws(pkgs) 
  
  # Filter out empty strings and "R"
  pkgs[nzchar(pkgs) & pkgs != "R"]
}

#' Retrieve the List of CRAN Packages (Internal)
#'
#' Downloads and caches the current list of packages available on CRAN.
#' 
#' @param cache Logical. If \code{TRUE} (default), the result is cached using \code{memoise::memoise()} 
#' with a timeout of 30 minutes to avoid repeated downloads. If \code{FALSE}, the metadata is fetched 
#' directly from CRAN each time the function is called.
#'
#' @return A matrix of package metadata similar to the output of \code{utils::available.packages()},
#' with package names as row names.
#'
#' @details The function downloads the \code{packages.rds} file from the CRAN website,
#' which contains metadata about all available packages. The result is cached using
#' \code{memoise::memoise()} with a timeout of 30 minutes.
#'
#' @examples
#' \donttest{
#' pkgs <- cran_packages()
#' head(rownames(pkgs))
#' }
#'
#' @importFrom memoise memoise timeout
#' @importFrom utils download.file
#' @export
cran_packages <- function(cache = TRUE) {
  get_cran_packages <- function() {
    local <- tempfile(fileext = ".rds")
    on.exit(unlink(local), add = TRUE)
    
    tryCatch({
      utils::download.file("https://cran.r-project.org/web/packages/packages.rds", local,
                           mode = "wb", quiet = TRUE)
      cp <- readRDS(local)
      rownames(cp) <- unname(cp[, 1])
      cp
    }, error = function(e) {
      stop("Failed to download or read CRAN package metadata: ", e$message)
    })
  }
  
  if (cache && requireNamespace("memoise", quietly = TRUE)) {
    memoise::memoise(get_cran_packages, ~memoise::timeout(30 * 60))()
  } else {
    get_cran_packages()
  }
}

test_that("extracts package names correctly from typical dependency strings", {
  expect_equal(
    extract_dependency_package_names("ggplot2 (>= 3.3.0), dplyr, tidyr"),
    c("ggplot2", "dplyr", "tidyr")
  )
})

test_that("removes 'R' from the list", {
  expect_equal(
    extract_dependency_package_names("R (>= 3.5.0), stringr"),
    "stringr"
  )
})

test_that("returns empty character vector for NA input", {
  expect_equal(
    extract_dependency_package_names(NA),
    character()
  )
})

test_that("handles whitespace and version constraints", {
  expect_equal(
    extract_dependency_package_names("  purrr (>= 0.3.0) , tibble "),
    c("purrr", "tibble")
  )
})

test_that("returns empty vector for empty string", {
  expect_equal(
    extract_dependency_package_names(""),
    character()
  )
})

test_that("ignores malformed entries gracefully", {
  test <- 
  expect_equal(
    extract_dependency_package_names(" , , , "),
    character()
  )
})

test_that("expand_dependency_type_spec returns correct values for named types", {
  expect_equal(
    expand_dependency_type_spec("strong"),
    c("Depends", "Imports", "LinkingTo")
  )
  
  expect_equal(
    expand_dependency_type_spec("most"),
    c("Depends", "Imports", "LinkingTo", "Suggests")
  )
  
  expect_equal(
    expand_dependency_type_spec("all"),
    c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")
  )
})

test_that("expand_dependency_type_spec returns custom input unchanged", {
  expect_equal(
    expand_dependency_type_spec(c("Depends", "Suggests")),
    c("Depends", "Suggests")
  )
})

test_that("parse deps for tar file works correctly", {
  skip_on_cran()
  
  # Keep the existing CRAN repo setup (harmless here)
  r <- getOption("repos")
  r["CRAN"] <- "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  install_list <- set_up_pkg(dp)
  package_installed <- install_list$package_installed
  pkg_source_path   <- install_list$pkg_source_path
  
  if (package_installed == TRUE) {
    # --- derive the package name from the tarball path (no stringr needed)
    pkg_tar   <- basename(pkg_source_path)             # e.g. "here-1.0.1.tar.gz"
    pkg_name0 <- sub("\\.tar\\.gz$", "", pkg_tar)      # "here-1.0.1"
    pkg_name  <- sub("[_|-].*$", "", pkg_name0)        # "here"
    
    # --- fake CRAN index (like utils::available.packages())
    # columns must include dependency fields used by dependsOnPkgs()
    fake <- matrix(
      c(
        "pkgA","here", "",    "",      "",
        "pkgB","",     "here","",      "",
        "pkgC","",     "",    "here",  "",
        "here","R",    "",    "",      ""
      ),
      ncol = 5, byrow = TRUE,
      dimnames = list(NULL, c("Package","Depends","Imports","Suggests","LinkingTo"))
    )
    rownames(fake) <- fake[, "Package"]
    
    # --- call the new API directly; no network, no stubs
    revdeps_list <- cran_revdep(
      pkg_name,
      dependencies = c("Depends","Imports","Suggests","LinkingTo"),
      recursive = FALSE,
      installed = fake
    )
    
    expect_identical(length(revdeps_list), 3L)
    checkmate::assert_character(revdeps_list, any.missing = FALSE)
    expect_setequal(revdeps_list, c("pkgA","pkgB","pkgC"))
  }
})

# Mock function for remove_base_packages to avoid network calls
mock_remove_base <- function(df) {
  base_rec_pkgs <- c("base", "compiler", "datasets", "graphics", "grDevices", 
                     "grid", "methods", "parallel", "splines", "stats", 
                     "stats4", "tcltk", "tools", "utils")
  df[!(df$package == "R" | df$package %in% base_rec_pkgs), ]
}

test_that("parse deps for tar file works correctly", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  mockery::stub(parse_dcf_dependencies, "remove_base_packages", mock_remove_base)
  
  dp <- system.file("test-data", "here-1.0.1.tar.gz", 
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # Defer cleanup: remove test package from temp dirs
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  # install package locally to ensure test works
  package_installed <- install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {	
    deps <- parse_dcf_dependencies(pkg_source_path)
    
    expect_identical(length(deps), 2L)
    
    expect_true(checkmate::check_data_frame(deps, 
                                            any.missing = FALSE))
    
    expect_true(checkmate::check_data_frame(deps, 
                                            col.names = "named")
    )
  }
  
})

test_that("parse deps for tar file works correctly", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  mockery::stub(parse_dcf_dependencies, "remove_base_packages", mock_remove_base)
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0001_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # install package locally to ensure test works
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {	
    deps <- parse_dcf_dependencies(pkg_source_path)
    
    expect_identical(length(deps), 2L)
    
    expect_true(checkmate::check_data_frame(deps, 
                                            any.missing = FALSE))
    
    expect_true(checkmate::check_data_frame(deps, 
                                            col.names = "named")
    )
  }
})

test_that("parse_dcf_dependencies extracts dependencies when CRAN is available", {
  # Check CRAN availability
  cran_status <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  skip_if(
    !is.null(cran_status$message),
    message = paste("Skipping test because CRAN is not available:", cran_status$message)
  )
  
  mockery::stub(parse_dcf_dependencies, "remove_base_packages", mock_remove_base)
  
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  writeLines(c(
    "Package: testpackage",
    "Title: A Sample Package",
    "Version: 0.1.0",
    "Depends: R (>= 3.5.0)",
    "Imports: dplyr, tidyr, ggplot2",
    "Suggests: testthat",
    "License: MIT"
  ), desc_path)
  
  deps <- parse_dcf_dependencies(temp_dir)
  
  expected_deps <- data.frame(
    type = c("Imports", "Imports", "Imports", "Suggests"),
    package = c("dplyr", "tidyr", "ggplot2", "testthat"),
    stringsAsFactors = FALSE
  )
  
  expect_equal(deps, expected_deps)
})

test_that("parse_dcf_dependencies extracts dependencies when CRAN is not available", {
  # Check CRAN availability
  cran_status <- capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
  
  skip_if(
    is.null(cran_status$message),
    message = "Skipping test because CRAN is available"
  )
  
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  writeLines(c(
    "Package: testpackage",
    "Title: A Sample Package",
    "Version: 0.1.0",
    "Depends: R (>= 3.5.0)",
    "Imports: dplyr, tidyr, ggplot2",
    "Suggests: testthat",
    "License: MIT"
  ), desc_path)
  
  deps <- parse_dcf_dependencies(temp_dir)
  
  expected_deps <- data.frame(
    type = c("Depends", "Imports", "Imports", "Imports", "Suggests"),
    package = c("R", "dplyr", "tidyr", "ggplot2", "testthat"),
    stringsAsFactors = FALSE
  )
  
  expect_equal(deps, expected_deps)
})


test_that("parse_dcf_dependencies extracts dependencies different format", {
  # Mock remove_base_packages to ensure the test works regardless of CRAN availability
  mockery::stub(parse_dcf_dependencies, "remove_base_packages", mock_remove_base)
  
  temp_dir <- tempdir()
  desc_path <- file.path(temp_dir, "DESCRIPTION")
  
  writeLines(c(
    "Package: testpackage",
    "Title: A Sample Package",
    "Version: 0.1.0",
    "Imports: ",
    "    rprojroot (>= 2.0.2)",
    "Suggests: ",
    "    conflicted,",
    "    covr,",
    "    fs,",
    "    knitr,",
    "    palmerpenguins,",
    "    plyr,",
    "    readr,",
    "    rlang,",
    "    rmarkdown,",
    "    testthat,",
    "    uuid,",
    "    withr"
  ), desc_path)
  
  deps <- parse_dcf_dependencies(temp_dir)
  
  expected_deps <- data.frame(
    type = c("Imports", "Suggests", "Suggests", "Suggests", "Suggests", "Suggests", "Suggests",
             "Suggests", "Suggests", "Suggests", "Suggests", "Suggests", "Suggests"),
    package = c("rprojroot", "conflicted", "covr", "fs", "knitr", "palmerpenguins",
                "plyr", "readr", "rlang", "rmarkdown", "testthat", "uuid", "withr"),
    stringsAsFactors = FALSE
  )
  
  expect_equal(deps, expected_deps)
})


test_that("remove_base_packages removes base and recommended packages", {
  # Fake CRAN metadata
  fake_cp <- data.frame(
    Package = c("stats", "utils", "dplyr", "ggplot2"),
    Priority = c("base", "recommended", NA, NA),
    stringsAsFactors = FALSE
  )
  
  # Input df
  df <- data.frame(
    package = c("stats", "utils", "dplyr", "R"),
    stringsAsFactors = FALSE
  )
  
  # Stub download.file to do nothing
  mockery::stub(remove_base_packages, "utils::download.file", NULL)
  
  # Stub readRDS to return our fake metadata
  mockery::stub(remove_base_packages, "readRDS", fake_cp)
  
  result <- remove_base_packages(df)
  
  expect_equal(result, "dplyr")
})


test_that("remove_base_packages keeps non-base packages", {
  fake_cp <- data.frame(
    Package = c("basepkg", "recpkg"),
    Priority = c("base", "recommended"),
    stringsAsFactors = FALSE
  )
  
  df <- data.frame(
    package = c("nonbase1", "nonbase2"),
    stringsAsFactors = FALSE
  )
  
  mockery::stub(remove_base_packages, "utils::download.file", NULL)
  mockery::stub(remove_base_packages, "readRDS", fake_cp)
  
  result <- remove_base_packages(df)
  
  expect_equal(result, c("nonbase1", "nonbase2"))
})


test_that("remove_base_packages removes 'R' even if not in CRAN metadata", {
  fake_cp <- data.frame(
    Package = c("foo"),
    Priority = c(NA),
    stringsAsFactors = FALSE
  )
  
  df <- data.frame(
    package = c("R", "foo"),
    stringsAsFactors = FALSE
  )
  
  mockery::stub(remove_base_packages, "utils::download.file", NULL)
  mockery::stub(remove_base_packages, "readRDS", fake_cp)
  
  result <- remove_base_packages(df)
  
  expect_equal(result, "foo")
})


test_that("remove_base_packages errors if download fails", {
  # Stub download.file to error
  mockery::stub(
    remove_base_packages,
    "utils::download.file",
    function(...) stop("network failure")
  )
  
  df <- data.frame(package = "dplyr", stringsAsFactors = FALSE)
  
  expect_message(
    remove_base_packages(df),
    "Failed to download or read CRAN package metadata"
  )
})


test_that("get_dependencies calls parse_dcf_dependencies and returns its value", {
  fake_path <- "/some/path/mypkg"
  fake_deps <- data.frame(package = c("dplyr", "rlang"), stringsAsFactors = FALSE)
  
  # Stub out parse_dcf_dependencies
  mockery::stub(get_dependencies, "parse_dcf_dependencies", fake_deps)
  
  # Stub basename to control the printed name
  mockery::stub(get_dependencies, "basename", "mypkg")
  
  # Capture messages
  msgs <- capture.output({
    result <- get_dependencies(fake_path)
  }, type = "message")
  
  # Expectations
  expect_equal(result, fake_deps)
  expect_true(any(grepl("getting package dependencies for mypkg", msgs)))
  expect_true(any(grepl("package dependencies successful for mypkg", msgs)))
})


test_that("get_dependencies passes pkg_source_path correctly to parse_dcf_dependencies", {
  fake_path <- "/tmp/testpkg"
  called_with <- NULL
  
  # stub parse_dcf_dependencies but record its argument
  mock_parse <- function(x) {
    called_with <<- x
    data.frame(package = "foo", stringsAsFactors = FALSE)
  }
  
  mockery::stub(get_dependencies, "parse_dcf_dependencies", mock_parse)
  mockery::stub(get_dependencies, "basename", "testpkg")
  
  get_dependencies(fake_path)
  
  expect_equal(called_with, fake_path)
})


test_that("get_dependencies prints messages with correct package name", {
  fake_path <- "/path/to/anotherpkg"
  
  mockery::stub(get_dependencies, "parse_dcf_dependencies",
                data.frame(package = "bar", stringsAsFactors = FALSE))
  mockery::stub(get_dependencies, "basename", "anotherpkg")
  
  msgs <- capture.output({
    get_dependencies(fake_path)
  }, type = "message")
  
  expect_true(any(grepl("getting package dependencies for anotherpkg", msgs)))
  expect_true(any(grepl("package dependencies successful for anotherpkg", msgs)))
})


test_that("get_dependencies returns what parse_dcf_dependencies provides", {
  fake_return <- data.frame(package = "test", stringsAsFactors = FALSE)
  
  mockery::stub(get_dependencies, "parse_dcf_dependencies", fake_return)
  mockery::stub(get_dependencies, "basename", "pkgname")
  
  result <- get_dependencies("whatever/path")
  
  expect_identical(result, fake_return)
})

test_that("cran_packages returns a matrix with expected structure", {
  skip_on_cran()
  pkgs <- cran_packages()
  
  expect_true(is.matrix(pkgs))
  expect_true("Package" %in% colnames(pkgs))
  expect_true(all(rownames(pkgs) %in% pkgs[, "Package"]))
})

test_that("cran_packages returns consistent results (memoised)", {
  skip_on_cran()
  pkgs1 <- cran_packages()
  Sys.sleep(1) # simulate time passing
  pkgs2 <- cran_packages()
  
  expect_identical(pkgs1, pkgs2)
})

test_that("dependsOnPkgs returns direct reverse dependencies", {
  installed <- matrix(
    c("pkgA", "pkgB", "pkgC", "pkgD",  # Package
      "",     "pkgA", "",     "pkgB",  # Depends
      "",     "",     "pkgA", "",      # Imports
      "",     "",     "",     "pkgC",  # Suggests
      "",     "",     "",     ""),     # LinkingTo
    nrow = 4,
    dimnames = list(NULL, c("Package", "Depends", "Imports", "Suggests", "LinkingTo"))
  )
  rownames(installed) <- installed[, "Package"]
  
  result <- dependsOnPkgs("pkgA", dependencies = "strong", recursive = FALSE, installed = installed)
  expect_equal(sort(result), sort(c("pkgB", "pkgC")))
})


test_that("dependsOnPkgs returns recursive reverse dependencies", {
  installed <- matrix(
    c("pkgA", "pkgB", "pkgC", "pkgD",  # Package
      "pkgB", "",     "",     "pkgA",  # Depends
      "",     "pkgA", "",     "pkgB",  # Imports
      "",     "",     "",     "",      # Suggests
      "",     "",     "",     ""),     # LinkingTo
    nrow = 4,
    dimnames = list(NULL, c("Package", "Depends", "Imports", "Suggests", "LinkingTo"))
  )
  rownames(installed) <- installed[, "Package"]
  
  result <- dependsOnPkgs("pkgA", dependencies = "most", recursive = TRUE, installed = installed)
  expect_equal(sort(result), sort(c("pkgB", "pkgD")))
})

test_that("dependsOnPkgs handles no dependencies", {
  installed <- matrix(
    c("pkgX", "pkgY",  # Package
      "",     "",      # Depends
      "",     "",      # Imports
      "",     ""),     # LinkingTo
    nrow = 2,
    dimnames = list(NULL, c("Package", "Depends", "Imports", "LinkingTo"))
  )
  
  rownames(installed) <- installed[, "Package"]
  
  result <- dependsOnPkgs("pkgX", dependencies = "strong", recursive = TRUE, installed = installed)
  expect_equal(result, character(0))
})

test_that("dependsOnPkgs handles multiple input packages", {
  installed <- matrix(
    c("pkgA", "pkgB", "pkgC",  # Package
      "pkgB", "",     "pkgA",  # Depends
      "",     "pkgA", "",      # Imports
      "",     "",     ""),     # LinkingTo
    nrow = 3,
    dimnames = list(NULL, c("Package", "Depends", "Imports", "LinkingTo"))
  )
  
  rownames(installed) <- installed[, "Package"]
  
  result <- dependsOnPkgs(c("pkgA", "pkgB"), dependencies = "strong", recursive = TRUE, installed = installed)
  expect_equal(sort(result), "pkgC")
})

test_that("clean_up_dependencies extracts and cleans multiple dependency strings", {
  input <- c("pkgA (>= 1.0), pkgB", "R (>= 3.5.0), pkgC")
  expected <- c("pkgA", "pkgB", "pkgC")
  
  result <- clean_up_dependencies(input)
  
  expect_equal(sort(result), sort(expected))
})

test_that("clean_up_dependencies handles empty input", {
  expect_equal(clean_up_dependencies(character()), character())
})

test_that("clean_up_dependencies removes duplicates", {
  input <- c("pkgA, pkgB", "pkgB, pkgC")
  expected <- c("pkgA", "pkgB", "pkgC")
  
  result <- clean_up_dependencies(input)
  
  expect_equal(sort(result), sort(expected))
})

test_that("clean_up_dependencies handles NA values", {
  input <- c(NA, "pkgX (>= 1.0)")
  expected <- "pkgX"
  
  result <- clean_up_dependencies(input)
  
  expect_equal(result, expected)
})

test_that("revdep returns reverse dependencies correctly", {
  installed <- matrix(
    c("pkgA", "pkgB", "pkgC", "pkgD",
      "",     "pkgA", "",     "pkgB",
      "",     "",     "pkgA", "",
      "",     "",     "",     "",
      "",     "",     "",     ""),
    nrow = 4,
    dimnames = list(NULL, c("Package", "Depends", "Imports", "Suggests", "LinkingTo"))
  )
  rownames(installed) <- installed[, "Package"]
  
  result <- cran_revdep("pkgA", recursive = TRUE, installed = installed)
  expect_equal(sort(result), sort(c("pkgB", "pkgC", "pkgD")))
})

test_that("cran_revdep respects ignore argument", {
  installed <- matrix(
    c("pkgA", "pkgB", "pkgC",
      "",     "pkgA", "pkgA",
      "",     "",     "",
      "",     "",     "",
      "",     "",     ""),
    nrow = 3,
    dimnames = list(NULL, c("Package", "Depends", "Imports", "Suggests", "LinkingTo"))
  )
  rownames(installed) <- installed[, "Package"]
  
  result <- cran_revdep("pkgA", ignore = "pkgC", installed = installed)
  expect_equal(result, "pkgB")
})

test_that("cran_revdep returns NULL and prints message if pkg is missing", {
  expect_message(result <- cran_revdep(), "Package name is required")
  expect_null(result)
})

#' Process lock files 
#'
#' @description This function processes `renv.lock` and `pak.lock` files 
#' to produce risk metric data
#' 
#' @param input_data - path to a lock file
#'
#' @return assessment_results - nested list containing risk metric data
#'
#' @importFrom jsonlite fromJSON
#' @importFrom stats setNames
#'
#' @examples
#' \dontrun{
#'   input_data <- ("path/to/mypak.lock")
#'   pak_results <- risk_assess_pkg_lock_files(input_data) 
#'   print(pak_results)
#'  } 
#' @export
risk_assess_pkg_lock_files <- function(input_data) {
  
  message("Starting lock file processing")
  
  # Input checking
  checkmate::assert_string(input_data)
  checkmate::assert_file_exists(input_data,
                                extension = ".lock",
                                access = "r")
  
  # Read the lock file
  lockfile <- fromJSON(input_data)
  
  # Determine the type of lock file and extract package information
  if ("Packages" %in% names(lockfile)) {
    # Process a renv.lock file
    packages <- lockfile$Packages
    is_pak_lock <- FALSE
  } else if ("packages" %in% names(lockfile)) {
    # Process a pak.lock file
    packages <- lockfile$packages
    is_pak_lock <- TRUE
  } else {
    stop("Unsupported lock file format.")
  }
  
  # Function to process each package
  process_package <- function(pkg_info) {
    
    if (is_pak_lock == TRUE) {    
      pkg_name <- pkg_info$package
      pkg_version <- pkg_info$version
    } else {
      pkg_name <- pkg_info$Package
      pkg_version <- pkg_info$Version
    }  
    
    risk_assess_package <- risk.assessr::risk_assess_pkg(
      package = pkg_name,
      version = pkg_version
    )
    return(list(name = pkg_name, version = pkg_version, result = risk_assess_package))
  }
  

  # Apply the process_package function to each package
  if (is.data.frame(packages)) {
    results <- lapply(seq_len(nrow(packages)), function(i) process_package(packages[i, ]))
  } else {
    results <- lapply(packages, process_package)
  }
  
  # Filter out NULL results and create a named list
  assessment_results <- setNames(lapply(results, `[[`, "result"), 
                                 sapply(results, function(x) paste(x$name, x$version, sep = "_")))
  
  message("Finishing lock file processing")
  
  return(assessment_results)
}

test_that("running risk_assess_pkg_lock_files with tar file", {
  
  lock_file <- system.file("test-data", "here-1.0.1.tar.gz",
                           package = "risk.assessr")
  
  expect_error(
    pak_results <-
      risk.assessr::risk_assess_pkg_lock_files(lock_file),
    regexp = "Assertion on 'input_data' failed.*\\.lock"
  )
  
})

# Mock the risk_assess_pkg function (used when processing lock file packages)
mock_risk_assess_pkg <- function(path = NULL, package = NULL, version = NA, repos = getOption("repos")) {
  list(
    pkg_name = package,
    pkg_version = version,
    risk_profile = "Medium"
  )
}

test_that("risk_assess_pkg_lock_files processes renv.lock correctly", {
  mockery::stub(risk_assess_pkg_lock_files, "risk.assessr::risk_assess_pkg", mock_risk_assess_pkg)
  
  # Create a temporary .lock file
  temp_lock_file <- tempfile(fileext = ".lock")
  writeLines('{
    "R": {
      "Version": "4.0.2",
      "Repositories": [
        {
          "Name": "CRAN",
          "URL": "https://cloud.r-project.org"
        }
      ]
    },
    "Packages": {
      "askpass": {
        "Package": "markdown",
        "Version": "1.0",
        "Source": "Repository",
        "Repository": "CRAN",
        "Hash": "4584a57f565dd7987d59dda3a02cfb41"
      },
      "mime": {
        "Package": "mime",
        "Version": "0.12",
        "Source": "GitHub",
        "RemoteType": "github",
        "RemoteHost": "api.github.com",
        "RemoteUsername": "yihui",
        "RemoteRepo": "mime",
        "RemoteRef": "main",
        "RemoteSha": "1763e0dcb72fb58d97bab97bb834fc71f1e012bc",
        "Requirements": ["tools"],
        "Hash": "c2772b6269924dad6784aaa1d99dbb86"
      }
    }
  }', temp_lock_file)
  
  result <- risk_assess_pkg_lock_files(temp_lock_file)
  
  testthat::expect_true("markdown_1.0" %in% names(result))
  testthat::expect_true("mime_0.12" %in% names(result))
  testthat::expect_identical(length(result), 2L)
  testthat::expect_true(checkmate::check_class(result, "list"))
})

test_that("risk_assess_pkg_lock_files processes pak.lock correctly", {
  mockery::stub(risk_assess_pkg_lock_files, "risk.assessr::risk_assess_pkg", mock_risk_assess_pkg)
  
  # Create a temporary .lock file with pak.lock format
  temp_lock_file <- tempfile(fileext = ".lock")
  writeLines('{
    "pak_version": "0.1.0",
    "r_version": "4.0.2",
    "repos": [
      {
        "name": "CRAN",
        "url": "https://cloud.r-project.org"
      }
    ],
    "packages": [
      {
        "package": "markdown",
        "version": "1.0",
        "source": "Repository",
        "repository": "CRAN",
        "hash": "4584a57f565dd7987d59dda3a02cfb41"
      }
    ]
  }', temp_lock_file)

  result <- risk_assess_pkg_lock_files(temp_lock_file)
  
  testthat::expect_true("markdown_1.0" %in% names(result))
  testthat::expect_identical(length(result), 1L)
  testthat::expect_true(checkmate::check_class(result, "list"))
})

#' Create a Traceability Matrix
#'
#' Returns a table that links all exported functions and their aliases to their documentation (`man` files),
#' the R scripts containing them, and the test scripts that reference them.
#' 
#' @param pkg_name name of package
#' @param pkg_source_path path to a source package
#' @param func_covr function coverage
#' @param verbose Logical (`TRUE`/`FALSE`). If `TRUE`, show any warnings/messages per function.
#' @param execute_coverage Logical (`TRUE`/`FALSE`). If `TRUE`, execute test coverage.
#'
#' @returns a nested list with fine grained traceability matrices
#' 
#' @examples
#' \dontrun{
#' # set CRAN repo to enable running of reverse dependencies
#' r = getOption("repos")
#' r["CRAN"] = "http://cran.us.r-project.org"
#' old <- options(repos = r)
#' 
#' dp <- system.file("test-data", "here-1.0.1.tar.gz", 
#'    package = "risk.assessr")
#' 
#' # Set up the package using the temporary file
#' install_list <- set_up_pkg(dp)
#' 
#' # Extract information from the installation list
#' build_vignettes <- install_list$build_vignettes
#' package_installed <- install_list$package_installed
#' pkg_source_path <- install_list$pkg_source_path
#' rcmdcheck_args <- install_list$rcmdcheck_args
#' 
#' # check if the package needs to be installed locally
#' package_installed <- install_package_local(pkg_source_path)
#' 
#' # Get package name and version
#' pkg_desc <- get_pkg_desc(pkg_source_path, 
#'                          fields = c("Package", 
#'                                     "Version"))
#' pkg_name <- pkg_desc$Package
#' test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
#' 
#' covr_timeout <- Inf
#' 
#' if (test_pkg_data$has_testthat || test_pkg_data$has_testit) {
#'   
#'   covr_list <- run_coverage(
#'     pkg_source_path,  
#'     covr_timeout
#'   )    
#' } else {
#'   message("No testthat or testit configuration")
#'   covr_list <- list(
#'     total_cov = 0,
#'     res_cov = list(
#'       name = pkg_name,
#'       coverage = list(
#'         filecoverage = matrix(0, nrow = 1, dimnames = list("No functions tested")),
#'         totalcoverage = 0
#'       ),
#'       errors = "No testthat or testit configuration",
#'       notes = NA
#'     )
#'   )
#' }
#' # run without test coverage   
#' tm_list_no_covr <- create_traceability_matrix(pkg_name, 
#'                                       pkg_source_path,
#'                                       execute_coverage = FALSE)   
#'                                       
#' # run with test coverage                                         
#' tm_list_covr <- create_traceability_matrix(pkg_name, 
#'                                       pkg_source_path,
#'                                       covr_list$res_cov)
#'                                       
#' # run with test coverage parameter but no test coverage list                                        
#' tm_list_no_covr_list <- create_traceability_matrix(pkg_name, 
#'                                       pkg_source_path)   
#'                                          
#' options(old)
#' }
#' 
#' @export
create_traceability_matrix <- function(pkg_name, 
                                       pkg_source_path, 
                                       func_covr = NULL,
                                       verbose = FALSE,
                                       execute_coverage = TRUE){
  
 
  message(glue::glue("creating traceability matrix for {pkg_name}"))
  
  # Check that package has an R folder
  tm_possible <- contains_r_folder(pkg_source_path) 
  # 
  if (tm_possible == FALSE) {
    tm_list <- create_empty_tm(pkg_name)
    message(glue::glue("no R folder to create traceability matrix for {pkg_name}"))
  } else {  
    
    # Get data.frame of exported functions
    exports_df <- get_exports(pkg_source_path)
    
    # Locate script for each exported function
    exports_df <- map_functions_to_scripts(exports_df, pkg_source_path, verbose)
    
    # Remove duplicates from exports for function and class
    exports_df <- exports_df %>%
      dplyr::distinct(exported_function, class, .keep_all = TRUE)
    
    # Map all Rd files to functions, then join back to exports
    exports_df <- map_functions_to_docs(exports_df, pkg_source_path, verbose)
    
    descrip <- get_func_descriptions(pkg_name)
    
    # Check if descrip is an empty named list and create descript_df accordingly
    if (length(descrip) == 0) {
      descript_df <- data.frame(
        exported_function = "No function found",
        description = "No description found"
      )
    } else {
      # Create df
      descript_df <- data.frame(
        exported_function = names(descrip),
        description = unlist(descrip, use.names = FALSE)
      )
    }
    
    exports_df <- dplyr::left_join(exports_df, 
                                   descript_df, 
                                   by = "exported_function")
    
   
    # Filter out rows with specific keywords in descriptions
    exports_df <- exports_df %>%
      dplyr::filter(!grepl("%>%", exported_function, ignore.case = TRUE)) %>%
      # filter out these functions from the methods package 
      # as they have no documentation
      dplyr::filter(!grepl("^\\.__C__|^\\.__T__", exported_function))
    
    if (execute_coverage == TRUE) {
      if (is.null(func_covr) || is.null(func_covr$coverage) || 
          is.null(func_covr$coverage$filecoverage)) {
        message(glue::glue("the execute_coverage parameter = TRUE but the 'function_coverage' object for {pkg_name} is NULL or non compliant"))
        
        # create dummy func_coverage to allow for 
        # tm creation without test coverage
        func_coverage <- data.frame(
          code_script = factor(exports_df$code_script),
          coverage_percent = rep(0, length(exports_df$code_script)),
          stringsAsFactors = FALSE
        )
        
        # create dummy filecoverage matrix
        filecoverage <- array(
          rep(0, length(func_coverage$code_script)),
          dim = length(func_coverage$code_script),
          dimnames = list(as.character(func_coverage$code_script))
        )
        
        # Set total coverage to 0
        totalcoverage <- 0
        
        # create empty function coverage list
        func_covr <- list(
          name = pkg_name,  
          coverage = list(
            filecoverage = filecoverage,
            totalcoverage = totalcoverage
          ),
          errors = "function coverage' is NULL or non compliant",
          notes = NA
        )
      }
      message(glue::glue("creating traceability matrices for {pkg_name} with test coverage"))
      # convert array to df
      func_coverage <- as.data.frame.table(func_covr$coverage$filecoverage)
      
      # check for column existence
      column_exists <- function(df, column_name) {
        return(column_name %in% names(df))
      }
      cl_exists <- column_exists(func_coverage, "Var1")
      
      if (cl_exists == TRUE) {  
        
        #Step 1: Strip temp path prefix if Var1 contains full paths
        if (any(grepl("^(/|[A-Za-z]:)", func_coverage$Var1))) {
          func_coverage$Var1 <- sub(glue::glue("^/tmp/.+?/{pkg_name}/"), "", func_coverage$Var1)
        }
        
        # Step 2: Rename columns
        func_coverage <- func_coverage %>%
          rename(code_script = Var1, coverage_percent = Freq)
        
        
        # create total traceability matrix
        tm <- dplyr::left_join(exports_df, 
                               func_coverage, 
                               by = "code_script") 
        
        # create fine grained traceability matrices
        tm_list <- fine_grained_tms(tm, pkg_name)
        
        message(glue::glue("traceability matrices for {pkg_name} successful"))
      } else {
        tm_list <- create_empty_tm(pkg_name)
        message(glue::glue("traceability matrix for {pkg_name} unsuccessful"))
      }
    } else {
      message(glue::glue("creating traceability matrices for {pkg_name} without test coverage"))
      # initalise tm 
      tm <- exports_df
      
      # create dummy func_coverage to allow for 
      # tm creation without test coverage
      func_coverage <- data.frame(
        code_script = factor(exports_df$code_script),
        coverage_percent = rep(0, length(exports_df$code_script)),
        stringsAsFactors = FALSE
      )
      
      # create total traceability matrix
      tm <- dplyr::left_join(exports_df, 
                             func_coverage, 
                             by = "code_script") 
      
      # Remove duplicates from tm for function and class
      tm <- tm %>%
        dplyr::distinct(exported_function, class, .keep_all = TRUE)
      
      # create fine grained traceability matrices
      tm_list <- fine_grained_tms(tm, pkg_name)
      
    }
  }
  return(tm_list)
}

#' Get all exported functions and map them to R script where they are defined
#'
#' adapted from mpn.scorecard
#'
#' @param exports_df data.frame with a column, named `exported_function`,
#'   containing the names of all exported functions. Can also have other columns
#'   (which will be returned unmodified).
#' @param pkg_source_path a file path pointing to an unpacked/untarred package directory
#' @param verbose check for extra information
#'
#' @return A data.frame with the columns `exported_function` and `code_script`.
#' @keywords internal
map_functions_to_scripts <- function(exports_df, pkg_source_path, verbose){
 
  # Search for scripts functions are defined in
  funcs_df <- get_toplevel_assignments(pkg_source_path)
  
  if(nrow(funcs_df) == 0){
    # Triggering this means an R/ directory exists, but no assignments were found.
    exports_df <- data.frame()
    message("No top level assignments found in R folder for ", basename(pkg_source_path))
  } else {
  
    #Create join keys 
    exports_df <- exports_df %>% 
      dplyr::mutate(join_key = ifelse(is.na(class), 
                                      exported_function, 
                                      paste0(exported_function, ".", class)))
    # Perform the join
    exports_df <- dplyr::left_join(exports_df, funcs_df, by = c("join_key" = "func"))
    
    if (any(is.na(exports_df$code_script))) {
      if(isTRUE(verbose)) {
        missing_from_files <- exports_df$exported_function[is.na(exports_df$code_script)] %>%
          paste(collapse = "\n")
        message(glue::glue("The following exports were not found in R/ for {basename(pkg_source_path)}:\n{missing_from_files}\n\n"))
      }
    }
  }
  return(exports_df)
}

#' Remove specific symbols from vector of functions
#'
#' adapted from mpn.scorecard
#' 
#' @param funcs vector of functions to filter
#'
#' @keywords internal
filter_symbol_functions <- function(funcs){
  ignore_patterns <- c("\\%>\\%", "\\$", "\\[\\[", "\\[", "\\+", "\\%", "<-")
  pattern <- paste0("(", paste(ignore_patterns, collapse = "|"), ")")
  funcs_return <- grep(pattern, funcs, value = TRUE, invert = TRUE)
  return(funcs_return)
}

#' list all top-level objects defined in the package code
#'
#' adapted from mpn.scorecard
#' 
#' This is primarily for getting all _functions_, but it also returns top-level
#' declarations, regardless of type. This is intentional, because we also want
#' to capture any global variables or anything else that could be potentially
#' exported by the package.
#'
#' @param pkg_source_path a file path pointing to an unpacked/untarred package directory
#'
#' @return A data.frame with the columns `function` and `code_script` with a row for
#'   every top-level object defined in the package.
#'
#' @keywords internal
get_toplevel_assignments <- function(pkg_source_path){
  
  r_files <- tools::list_files_with_type(file.path(pkg_source_path, "R"), "code")
  
  # Triggering this means an R/ directory exists, but no R/Q/S files were found.
  if(rlang::is_empty(r_files)){
    # This shouldn't be triggered, and either indicates a bug in `get_toplevel_assignments`,
    # or an unexpected package setup that we may want to support.
    message(glue::glue("No sourceable R scripts were found in the R/ directory for package {basename(pkg_source_path)}. Make sure this was expected."))
    return(dplyr::tibble(func = character(), code_script = character()))
  }
  
  pkg_functions <- purrr::map_dfr(r_files, function(r_file_i) {
    
    exprs <- tryCatch(parse(r_file_i), error = identity)
    if (inherits(exprs, "error")) {
      warning("Failed to parse ", r_file_i, ": ", conditionMessage(exprs))
      return(dplyr::tibble(func = character(), code_script = character()))
    }
    
    calls <- purrr::keep(as.list(exprs), function(e) {
      if (is.call(e)) {
        op <- as.character(e[[1]])
        return(length(op) == 1 && op %in% c("<-", "=", "setGeneric", "setMethod"))
      }
      return(FALSE)
    })
    lhs <- purrr::map(calls, function(e) {
      name <- as.character(e[[2]])
      if (length(name) == 1) {
        return(name)
      }
    })
    
    function_names <- unlist(lhs) %||% character()
    if (length(function_names) == 0 ) {
      return(dplyr::tibble(func = character(), code_script = character()))
    }
    return(dplyr::tibble(
      func = function_names,
      code_script = rep(fs::path_rel(r_file_i, pkg_source_path), length(function_names))
    ))
  })
  # TODO: do we need to check if there are any funcs defined in multiple files?
  
  return(pkg_functions)
}

#' Map all Rd files to the functions they describe
#'
#' adapted from mpn.scorecard
#'
#' @return Returns the data.frame passed to `exports_df`, with a `documentation`
#'   column appended. This column will contain the path to the `.Rd` files in
#'   `man/` that document the associated exported functions.
#'
#' @keywords internal
map_functions_to_docs <- function(exports_df, pkg_source_path, verbose) {
  
  rd_files <- list.files(file.path(pkg_source_path, "man"), full.names = TRUE)
  rd_files <- rd_files[grep("\\.Rd$", rd_files)]
  if (length(rd_files) == 0) {
    if(isTRUE(verbose)){
      message("No top level assignments found in R folder for ", basename(pkg_source_path))
    }
    return(dplyr::mutate(exports_df, "documentation" = NA))
  }
  
  docs_df <- purrr::map_dfr(rd_files, function(rd_file.i) {
    rd_lines <- readLines(rd_file.i) %>% suppressWarnings()
    
    # Get Rd file and aliases for exported functions
    function_names_lines <- rd_lines[grep("(^\\\\alias)|(^\\\\name)", rd_lines)]
    function_names <- unique(gsub("\\}", "", gsub("((\\\\alias)|(\\\\name))\\{", "", function_names_lines)))
    
    man_name <- paste0(basename(rd_file.i))
    
    data.frame(
      pkg_function = function_names,
      documentation = rep(man_name, length(function_names))
    )
  })
  
  # if any functions are aliased in more than 1 Rd file, collapse those Rd files to a single row
  # TODO: is this necessary? is it even possible to have this scenario without R CMD CHECK failing?
  docs_df <- docs_df %>%
    dplyr::group_by(.data$pkg_function) %>%
    dplyr::summarize(documentation = paste(unique(.data$documentation), collapse = ", ")) %>%
    dplyr::ungroup()
  
  # join back to filter to only exported functions
  exports_df <- dplyr::left_join(exports_df, 
                                 docs_df, 
                                 by = c("join_key" = "pkg_function"))
  
  # Clean up the join_key column 
  exports_df <- exports_df %>% dplyr::select(-join_key)
  
  # message if any exported functions aren't documented
  if (any(is.na(exports_df$documentation)) && isTRUE(verbose)) {
    docs_missing <- exports_df %>% dplyr::filter(is.na(.data$documentation))
    exports_missing <- unique(docs_missing$exported_function) %>% paste(collapse = "\n")
    code_files_missing <- unique(docs_missing$documentation) %>% paste(collapse = ", ")
    message(glue::glue("In package `{basename(pkg_source_path)}`, the R scripts ({code_files_missing}) are missing documentation for the following exports: \n{exports_missing}"))
  }
  
  return(exports_df)
}

#' Create fine grained traceability matrices
#'
#' @param tm - general traceability matrix with no filtered information
#' @param pkg_name - name of the package
#'
#' @return - nested list with fine grained tms
#' @keywords internal
fine_grained_tms <- function(tm, pkg_name) {
    
    # create high risk coverage tm 
    high_risk_coverage_tm <- tm |> 
      dplyr::filter(coverage_percent <= 60 | is.na(coverage_percent)) 
    if (nrow(high_risk_coverage_tm) == 0) {
      high_risk_coverage_tm <- create_empty_tm(pkg_name)
      message(glue::glue("high risk coverage traceability matrix for {pkg_name} not generated"))
    }
    
    # create medium risk coverage tm 
    medium_risk_coverage_tm <- tm |>
      dplyr::filter(coverage_percent > 60, coverage_percent < 80)
    if (nrow(medium_risk_coverage_tm) == 0) {
      medium_risk_coverage_tm <- create_empty_tm(pkg_name)
      message(glue::glue("medium risk coverage traceability matrix for {pkg_name} not generated"))
    }
    
    # create low risk coverage tm 
    low_risk_coverage_tm <- tm |>
      dplyr::filter(coverage_percent >= 80)  
    if (nrow(low_risk_coverage_tm) == 0) {
      low_risk_coverage_tm <- create_empty_tm(pkg_name)
      message(glue::glue("low risk coverage traceability matrix for {pkg_name} not generated"))
    }
   
    # create defunct function tm
    defunct_function_tm <- tm %>% 
      dplyr::filter(
        grepl("defunct|deprecated|deprec|superseded", function_body, ignore.case = TRUE) |
        grepl("defunct|deprecated|deprec|superseded", code_script, ignore.case = TRUE) |
        grepl("defunct|deprecated|deprec|superseded", description, ignore.case = TRUE) |
        grepl("defunct|deprecated|deprec|superseded", documentation, ignore.case = TRUE)
      )
    if (nrow(defunct_function_tm) == 0) {
      defunct_function_tm <- create_empty_tm(pkg_name)
      message(glue::glue("defunct functions traceability matrix for {pkg_name} not generated"))
    }
    
    # create imported function tm
    imported_function_tm <- tm %>% 
      dplyr::filter(
        grepl("imported", code_script, ignore.case = TRUE) | 
        grepl("imported", description, ignore.case = TRUE) |
        grepl("imported", documentation, ignore.case = TRUE)
      )  
    if (nrow(imported_function_tm) == 0) {
      imported_function_tm <- create_empty_tm(pkg_name)
      message(glue::glue("imported functions traceability matrix for {pkg_name} not generated"))
    }
    
    # create reexported function tm
    rexported_function_tm <- tm %>% 
      dplyr::filter(
        grepl("re-exported|reexports", code_script, ignore.case = TRUE) |
        grepl("re-exported|reexports", description, ignore.case = TRUE) |
        grepl("re-exported|reexports", documentation, ignore.case = TRUE)
      )
    
    if (nrow(rexported_function_tm) == 0) {
      rexported_function_tm <- create_empty_tm(pkg_name)
      message(glue::glue("rexported functions traceability matrix for {pkg_name} not generated"))
    }
  
    # create experimental function tm
    experimental_function_tm <- tm %>% 
      dplyr::filter(
        grepl("experimental", code_script, ignore.case = TRUE) |
          grepl("experimental", description, ignore.case = TRUE) |
          grepl("experimental", documentation, ignore.case = TRUE)
      )
    
    if (nrow(experimental_function_tm) == 0) {
      experimental_function_tm <- create_empty_tm(pkg_name)
      message(glue::glue("experimental functions traceability matrix for {pkg_name} not generated"))
    }
    
    # create a nested list
    tm_list <- list(
      tm = tm,
      coverage = list(
        high_risk = high_risk_coverage_tm,
        medium_risk = medium_risk_coverage_tm,
        low_risk = low_risk_coverage_tm
      ),
      function_type = list(
        defunct = defunct_function_tm,
        imported = imported_function_tm,
        rexported = rexported_function_tm,
        experimental = experimental_function_tm
      )
    )
  
  return(tm_list)
}

# Mock function to simulate get_toplevel_assignments
mock_get_toplevel_assignments <- function(pkg_source_path) {
  data.frame(
    func = c("func1", "func2"),
    code_script = c("script1.R", NA),
    stringsAsFactors = FALSE
  )
}

test_that("map_functions_to_scripts handles missing code_script correctly", {
  # Mock the get_toplevel_assignments function
  with_mocked_bindings(
    get_toplevel_assignments = mock_get_toplevel_assignments,
    {
      # Create a sample exports_df
      exports_df <- data.frame(
        exported_function = c("func1", "func2"),
        class = c(NA, NA),
        stringsAsFactors = FALSE
      )
      
      # Capture the messages
      messages <- capture_messages(
        result <- map_functions_to_scripts(exports_df, "dummy_path", verbose = TRUE)
      )
      
      # Check that the expected message is present
      expect_true(any(grepl("The following exports were not found in R/ for dummy_path", messages)))
      
      # Check that the result has NA in code_script for func2
      expect_true(any(is.na(result$code_script)))
    }
  )
})

test_that("get_toplevel_assignments works correctly", {
  # Mock the tools::list_files_with_type function
  mock_list_files_with_type <- function(path, type) {
    if (path == "dummy_path/R") {
      return(c("file1.R", "file2.R"))
    }
    return(character(0))
  }
  
  # Mock the parse function
  mock_parse <- function(file) {
    if (file == "file1.R") {
      return(parse(text = "func1 <- function() {}"))
    } else if (file == "file2.R") {
      return(parse(text = "func2 <- function() {}"))
    }
    stop("Unexpected file")
  }
  
  # Mock the fs::path_rel function
  mock_path_rel <- function(path, start) {
    return(basename(path))
  }
  
  pkg_name <- "dummy_path"
  
  # Use mockery to stub the functions
  mockery::stub(get_toplevel_assignments, "tools::list_files_with_type", mock_list_files_with_type)
  mockery::stub(get_toplevel_assignments, "parse", mock_parse)
  mockery::stub(get_toplevel_assignments, "fs::path_rel", mock_path_rel)
  
  # Test case: Normal scenario
  result <- get_toplevel_assignments(pkg_name)
  expect_equal(result$func, c("func1", "func2"))
  expect_equal(result$code_script, c("file1.R", "file2.R"))
  
  # Test case: No R files found
  mock_list_files_with_type_empty <- function(path, type) {
    return(character(0))
  }
  
  mockery::stub(get_toplevel_assignments, "tools::list_files_with_type", mock_list_files_with_type_empty)
  
  messages <- capture_messages(result <- get_toplevel_assignments(pkg_name))
  expect_true(any(grepl(glue::glue("No sourceable R scripts were found in the R/ directory for package {pkg_name}. Make sure this was expected."), messages)))
  expect_equal(nrow(result), 0)
})

test_that("running tm for created package in tar file with no notes", {
  skip_on_cran()
  dp <- system.file("test-data", "stringr-1.5.1.tar.gz",
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args

  # Defer cleanup: remove test package from temp dirs
  withr::defer({
    unlink(pkg_source_path, recursive = TRUE, force = TRUE)
  })
  
  if (package_installed == TRUE ) {

    # setup parameters for running covr
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)

    covr_timeout <- Inf

    covr_list <- run_coverage(
      pkg_source_path,  # must use untarred package dir
      covr_timeout
    )

    tm_list <- create_traceability_matrix(pkg_name,
                                     pkg_source_path,
                                     covr_list$res_cov
                                     )
    
    testthat::expect_identical(length(tm_list), 3L)
    
    testthat::expect_identical(length(tm_list$tm), 9L)

    # Column to check
    column_name <- "exported_function"
    
    # Test to check if there are no missing values in the specified column
    testthat::expect_true(all(!is.na(tm_list$tm[[column_name]])))

    testthat::expect_true(checkmate::check_data_frame(tm_list$tm,
                                            col.names = "named")
    )

  } else {
    message(glue::glue("cannot run traceability matrix for {basename(pkg_source_path)}"))
  }

})


# The following test is commented out temporarily
# the test with devtools::test() but the problem is caused by
#  https://github.com/hadley/testthat/issues/144
# and https://github.com/r-lib/testthat/issues/86
test_that("running tm for created package in tar file with no tests", {
  skip_on_cran()
  
  # Copy test package to a temp file
  dp_orig <- system.file("test-data", 
                         "test.package.0004_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())

  # set up package
  install_list <- set_up_pkg(dp)

  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args

  # install package locally to ensure test works
  package_installed <- 
    install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) {

    # ensure path is set to package source path
    rcmdcheck_args$path <- pkg_source_path
    
    # setup parameters for running covr
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)

    covr_timeout <- Inf

    covr_list <- suppressMessages(run_coverage(
      pkg_source_path,  # must use untarred package dir
      covr_timeout)
    )

    tm_list <- create_traceability_matrix(pkg_name,
                                       pkg_source_path,
                                       covr_list$res_cov)

    testthat::expect_identical(length(tm_list), 3L)
    
    testthat::expect_identical(length(tm_list$tm), 9L)

    testthat::expect_true(checkmate::check_data_frame(tm_list$tm,
                                                      any.missing = TRUE))

    testthat::expect_true(checkmate::check_data_frame(tm_list$tm,
                                                      col.names = "named")
    )

    testthat::expect_equal(covr_list$total_cov, 0)

    testthat::expect_equal(covr_list$res_cov$coverage$totalcoverage, 0)

  } else {
    message(glue::glue("cannot run traceability matrix for {basename(pkg_source_path)}"))
  }

})

test_that("running tm for created package in tar file with no R directory", {
  skip_on_cran()
  
  dp_orig <- system.file("test-data", 
                         "test.package.0006_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args

  if (package_installed == TRUE ) {
    
    # setup parameters for running covr
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)
    
    covr_timeout <- Inf
    
    covr_list <- suppressMessages(run_coverage(
      pkg_source_path,  # must use untarred package dir
      covr_timeout)
    )
    
    testthat::expect_message(
      tm_list <- create_traceability_matrix(pkg_name, 
                                       pkg_source_path, 
                                       covr_list$res_cov),
      glue::glue("creating traceability matrix for {basename(pkg_source_path)}"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      tm_list <- create_traceability_matrix(pkg_name, 
                                       pkg_source_path, 
                                       covr_list$res_cov),
      glue::glue("traceability matrix for {basename(pkg_source_path)} unsuccessful"),
      fixed = TRUE
    )
    
    testthat::expect_message(
      tm_list <- create_traceability_matrix(pkg_name, 
                                       pkg_source_path, 
                                       covr_list$res_cov),
      glue::glue("no R folder to create traceability matrix for {basename(pkg_source_path)}"),
      fixed = TRUE
    )
    
  } else {
    message(glue::glue("cannot run traceability matrix for {basename(pkg_source_path)}"))
  } 
  
})

# The following test will be reactivated when 
# https://github.com/Sanofi-Public/risk.assessr/issues/78 is fixed

test_that("running tm for created package in tar file with empty R directory", {
  skip_on_cran()
  
  dp_orig <- system.file("test-data", 
                         "test.package.0005_0.1.0.tar.gz", 
                         package = "risk.assessr")
  dp <- tempfile(fileext = ".tar.gz")
  file.copy(dp_orig, dp)
  
  # Defer cleanup of copied tarball
  withr::defer(unlink(dp), envir = parent.frame())
  
  # Defer cleanup of unpacked source directory
  withr::defer(unlink(pkg_source_path, recursive = TRUE, force = TRUE),
               envir = parent.frame())
  
  # set up package
  install_list <- set_up_pkg(dp)

  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args

  if (package_installed == TRUE ) {

    # setup parameters for running covr
    pkg_desc <- get_pkg_desc(pkg_source_path, fields = c("Package", "Version"))
    pkg_name <- pkg_desc$Package
    pkg_ver <- pkg_desc$Version
    pkg_name_ver <- paste0(pkg_name, "_", pkg_ver)

    covr_timeout <- Inf

    covr_list <- suppressMessages(run_coverage(
      pkg_source_path,  # must use untarred package dir
      covr_timeout)
    )

    testthat::expect_message(
      {
        tm <- try(
          create_traceability_matrix(pkg_name, pkg_source_path, covr_list$res_cov),
          silent = TRUE
        )
      },
      "creating traceability matrix for test.package.0005",
      fixed = TRUE
    )
    
    testthat::expect_message(
      {
        tm <- try(
          create_traceability_matrix(pkg_name, pkg_source_path, covr_list$res_cov),
          silent = TRUE
        )
      },
      "No top level assignments found in R folder for test.package.0005",
      fixed = TRUE
    )
    } else {
    message(glue::glue("cannot run traceability matrix for {basename(pkg_source_path)}"))
  }

})

test_that("get_toplevel_assignments works correctly", {
  # Mock the tools::list_files_with_type function
  mock_list_files_with_type <- function(path, type) {
    if (path == "dummy_path/R") {
      return(c("file1.R", "file2.R"))
    }
    return(character(0))
  }
  
  # Mock the parse function
  mock_parse <- function(file) {
    if (file == "file1.R") {
      return(parse(text = "func1 <- function() {}"))
    } else if (file == "file2.R") {
      return(parse(text = "func2 <- function() {}"))
    }
    stop("Unexpected file")
  }
  
  # Mock the fs::path_rel function
  mock_path_rel <- function(path, start) {
    return(basename(path))
  }
  
  pkg_name <- "dummy_path"
  
  # Use mockery to stub the functions
  mockery::stub(get_toplevel_assignments, "tools::list_files_with_type", mock_list_files_with_type)
  mockery::stub(get_toplevel_assignments, "parse", mock_parse)
  mockery::stub(get_toplevel_assignments, "fs::path_rel", mock_path_rel)
  
  # Test case: Normal scenario
  result <- get_toplevel_assignments(pkg_name)
  expect_equal(result$func, c("func1", "func2"))
  expect_equal(result$code_script, c("file1.R", "file2.R"))
  
  # Test case: No R files found
  mock_list_files_with_type_empty <- function(path, type) {
    return(character(0))
  }
  
  mockery::stub(get_toplevel_assignments, "tools::list_files_with_type", mock_list_files_with_type_empty)
  
  messages <- capture_messages(result <- get_toplevel_assignments(pkg_name))
  print(messages)
  expect_true(any(grepl(glue::glue("No sourceable R scripts were found in the R/ directory for package {pkg_name}. Make sure this was expected."), messages)))
  expect_equal(nrow(result), 0)
})

test_that("map_functions_to_scripts returns correct mapping", {
  # Mock the exports_df
  exports_df <- dplyr::tibble(exported_function = c("function1", "function2", "generic1"),
                              class = c(NA, NA, "class1"))
  
  # Mock the funcs_df
  funcs_df <- dplyr::tibble(func = c("function1", "function2", "generic1.class1"), code_script = c("script1.R", "script2.R", "script3.R"))
  
  # Mock the get_toplevel_assignments function
  mock_get_toplevel_assignments <- mockery::mock(funcs_df)
  mockery::stub(map_functions_to_scripts, "get_toplevel_assignments", mock_get_toplevel_assignments)
  
  # Call the function
  result <- map_functions_to_scripts(exports_df, "mock/path", verbose = FALSE)
  
  # Expected result
  expected <- dplyr::tibble(
    exported_function = c("function1", "function2", "generic1"),
    class = c(NA, NA, "class1"),
    join_key = c("function1", "function2", "generic1.class1"),
    code_script = c("script1.R", "script2.R", "script3.R")
  )
  
  # Check the result
  expect_equal(result, expected)
})

test_that("map_functions_to_scripts handles no top level assignments found", {
  # Mock the exports_df
  exports_df <- dplyr::tibble(exported_function = c("function1", "function2", "generic1"),
                              class = c(NA, NA, "class1"))
  
  # Mock the funcs_df with no assignments
  funcs_df <- dplyr::tibble()
  
  # Mock the get_toplevel_assignments function
  mock_get_toplevel_assignments <- mockery::mock(funcs_df)
  mockery::stub(map_functions_to_scripts, "get_toplevel_assignments", mock_get_toplevel_assignments)
  
  # Call the function
  result <- map_functions_to_scripts(exports_df, "mock/path", verbose = FALSE)
  
  # Expected result
  expected <- data.frame()
  
  # Check the result
  expect_equal(result, expected)
})

test_that("map_functions_to_scripts handles missing functions in scripts", {
  # Mock the exports_df
  exports_df <- dplyr::tibble(exported_function = c("function1", "function2", "generic1"),
                              class = c(NA, NA, "class1"))
  
  # Mock the funcs_df with missing functions
  funcs_df <- dplyr::tibble(func = c("function1"), code_script = c("script1.R"))
  
  # Mock the get_toplevel_assignments function
  mock_get_toplevel_assignments <- mockery::mock(funcs_df)
  mockery::stub(map_functions_to_scripts, "get_toplevel_assignments", mock_get_toplevel_assignments)
  
  # Call the function
  result <- map_functions_to_scripts(exports_df, "mock/path", verbose = TRUE)
  
  # Expected result
  expected <- dplyr::tibble(
    exported_function = c("function1", "function2", "generic1"),
    class = c(NA, NA, "class1"),
    join_key = c("function1", "function2", "generic1.class1"),
    code_script = c("script1.R", NA, NA)
  )
  
  # Check the result
  expect_equal(result, expected)
})

test_that("map_functions_to_docs returns correct mapping", {
  # Mock the exports_df
  exports_df <- dplyr::tibble(exported_function = c("function1", "function2", "generic1"),
                              class = c(NA, NA, "class1"),
                              join_key = c("function1", "function2", "generic1.class1"))
  
  # Mock the Rd files
  rd_files <- c("man/function1.Rd", "man/function2.Rd", "man/generic1.class1.Rd")
  
  # Mock the list.files function
  mock_list_files <- mockery::mock(rd_files)
  mockery::stub(map_functions_to_docs, "list.files", mock_list_files)
  
  # Mock the readLines function
  mock_read_lines <- mockery::mock(
    c("\\name{function1}", "\\alias{function1}"),
    c("\\name{function2}", "\\alias{function2}"),
    c("\\name{generic1.class1}", "\\alias{generic1.class1}")
  )
  mockery::stub(map_functions_to_docs, "readLines", mock_read_lines)
  
  # Call the function
  result <- map_functions_to_docs(exports_df, "mock/path", verbose = FALSE)
  
  # Expected result
  expected <- dplyr::tibble(
    exported_function = c("function1", "function2", "generic1"),
    class = c(NA, NA, "class1"),
    documentation = c("function1.Rd", "function2.Rd", "generic1.class1.Rd")
  )
  
  # Check the result
  expect_equal(result, expected)
})

test_that("map_functions_to_docs handles no documentation found", {
  # Mock the exports_df
  exports_df <- dplyr::tibble(exported_function = c("function1", "function2", "generic1"),
                              class = c(NA, NA, "class1"),
                              join_key = c("function1", "function2", "generic1.class1"))
  
  # Mock the Rd files with no documentation
  rd_files <- character(0)
  
  # Mock the list.files function
  mock_list_files <- mockery::mock(rd_files)
  mockery::stub(map_functions_to_docs, "list.files", mock_list_files)
  
  # Call the function
  result <- map_functions_to_docs(exports_df, "mock/path", verbose = FALSE)
  
  # Expected result
  expected <- dplyr::mutate(exports_df, documentation = NA)
  
  # Check the result
  expect_equal(result, expected)
})

test_that("map_functions_to_docs handles missing functions in documentation", {
  # Mock the exports_df
  exports_df <- dplyr::tibble(exported_function = c("function1", "function2", "generic1"),
                              class = c(NA, NA, "class1"),
                              join_key = c("function1", "function2", "generic1.class1"))
  
  # Mock the Rd files with missing functions
  rd_files <- c("man/function1.Rd")
  
  # Mock the list.files function
  mock_list_files <- mockery::mock(rd_files)
  mockery::stub(map_functions_to_docs, "list.files", mock_list_files)
  
  # Mock the readLines function
  mock_read_lines <- mockery::mock(c("\\name{function1}", "\\alias{function1}"))
  mockery::stub(map_functions_to_docs, "readLines", mock_read_lines)
  
  # Call the function
  result <- map_functions_to_docs(exports_df, "mock/path", verbose = TRUE)
  
  # Expected result
  expected <- dplyr::tibble(
    exported_function = c("function1", "function2", "generic1"),
    class = c(NA, NA, "class1"),
    documentation = c("function1.Rd", NA, NA)
  )
  
  # Check the result
  expect_equal(result, expected)
})

test_that("filter_symbol_functions filters out symbols correctly", {
  # Mocked data
  funcs <- c("%>%", "$", "[[", "[", "+", "%", "<-", "function1", "function2")
  
  # Call the function
  result <- filter_symbol_functions(funcs)
  
  # Expected result
  expected <- c("function1", "function2")
  
  # Check the result
  expect_equal(result, expected)
})

test_that("filter_symbol_functions handles empty input", {
  # Mocked data
  funcs <- character(0)
  
  # Call the function
  result <- filter_symbol_functions(funcs)
  
  # Expected result
  expected <- character(0)
  
  # Check the result
  expect_equal(result, expected)
})

test_that("filter_symbol_functions handles input with no symbols", {
  # Mocked data
  funcs <- c("function1", "function2", "function3")
  
  # Call the function
  result <- filter_symbol_functions(funcs)
  
  # Expected result
  expected <- c("function1", "function2", "function3")
  
  # Check the result
  expect_equal(result, expected)
})

test_that("filter_symbol_functions handles input with only symbols", {
  # Mocked data
  funcs <- c("%>%", "$", "[[", "[", "+", "%", "<-")
  
  # Call the function
  result <- filter_symbol_functions(funcs)
  
  # Expected result
  expected <- character(0)
  
  # Check the result
  expect_equal(result, expected)
})

test_that("fine_grained_tms correctly categorizes coverage levels", {
  tm <- data.frame(
    function_name = c("f1", "f2", "f3"),
    coverage_percent = c(50, 70, 90),
    function_body = c("", "", ""),
    code_script = c("", "", ""),
    description = c("", "", ""),
    documentation = c("", "", ""),
    stringsAsFactors = FALSE
  )
  
  result <- fine_grained_tms(tm, "testpkg")
  
  expect_equal(nrow(result$coverage$high_risk), 1)
  expect_equal(nrow(result$coverage$medium_risk), 1)
  expect_equal(nrow(result$coverage$low_risk), 1)
})

test_that("fine_grained_tms handles defunct functions", {
  tm <- data.frame(
    function_name = "f_defunct",
    coverage_percent = 100,
    function_body = "This function is defunct",
    code_script = "",
    description = "",
    documentation = "",
    stringsAsFactors = FALSE
  )
  
  result <- fine_grained_tms(tm, "testpkg")
  
  expect_equal(nrow(result$function_type$defunct), 1)
})

test_that("fine_grained_tms handles imported functions", {
  tm <- data.frame(
    function_name = "f_imported",
    coverage_percent = 100,
    function_body = "",
    code_script = "This function is imported",
    description = "",
    documentation = "",
    stringsAsFactors = FALSE
  )
  
  result <- fine_grained_tms(tm, "testpkg")
  
  expect_equal(nrow(result$function_type$imported), 1)
})

test_that("fine_grained_tms handles reexported functions", {
  tm <- data.frame(
    function_name = "f_reexported",
    coverage_percent = 100,
    function_body = "",
    code_script = "This function is re-exported",
    description = "",
    documentation = "",
    stringsAsFactors = FALSE
  )
  
  result <- fine_grained_tms(tm, "testpkg")
  
  expect_equal(nrow(result$function_type$rexported), 1)
})

test_that("fine_grained_tms handles experimental functions", {
  tm <- data.frame(
    function_name = "f_experimental",
    coverage_percent = 100,
    function_body = "",
    code_script = "This function is experimental",
    description = "",
    documentation = "",
    stringsAsFactors = FALSE
  )
  
  result <- fine_grained_tms(tm, "testpkg")
  
  expect_equal(nrow(result$function_type$experimental), 1)
})


test_that("fine_grained_tms correctly categorizes coverage levels", {
  tm <- data.frame(
    function_name = c("f1", "f2", "f3"),
    coverage_percent = c(50, 70, 90),
    function_body = c("", "", ""),
    code_script = c("", "", ""),
    description = c("", "", ""),
    documentation = c("", "", ""),
    stringsAsFactors = FALSE
  )
  
  result <- fine_grained_tms(tm, "testpkg")
  
  expect_equal(nrow(result$coverage$high_risk), 1)
  expect_equal(nrow(result$coverage$medium_risk), 1)
  expect_equal(nrow(result$coverage$low_risk), 1)
})


test_that("create_traceability_matrix works without coverage", {
  pkg_name <- "mockpkg"
  pkg_source_path <- "mock/path"
  func_covr <- NULL  # not used when execute_coverage = FALSE
  
  # Mocked data for exports_df
  mock_exports_df <- data.frame(
    exported_function = c("func1", "func2"),
    class = c("S3", "S4"),
    code_script = c("script1.R", "script2.R"),
    stringsAsFactors = FALSE
  )
  
  # Stub all dependencies
  mockery::stub(create_traceability_matrix, "contains_r_folder", TRUE)
  mockery::stub(create_traceability_matrix, "get_exports", mock_exports_df)
  mockery::stub(create_traceability_matrix, "map_functions_to_scripts", mock_exports_df)
  mockery::stub(create_traceability_matrix, "map_functions_to_docs", mock_exports_df)
  mockery::stub(create_traceability_matrix, "get_func_descriptions", list(func1 = "desc1", func2 = "desc2"))
  mockery::stub(create_traceability_matrix, "fine_grained_tms", function(tm, pkg_name) {
    list(total_tm = tm)
  })
  
  # Run the function with execute_coverage = FALSE
  result <- create_traceability_matrix(pkg_name, pkg_source_path, func_covr, execute_coverage = FALSE)
  
  # Assertions
  expect_type(result, "list")
  expect_true("total_tm" %in% names(result))
  expect_equal(nrow(result$total_tm), 2)
  expect_equal(result$total_tm$coverage_percent, c(0, 0))  # joined with dummy coverage
})

test_that("create_traceability_matrix creates dummy func_covr when input is NULL", {
  pkg_name <- "mockPkg"
  pkg_source_path <- "mock/path"
  
  # Mocked exports_df
  mock_exports_df <- data.frame(
    exported_function = c("fun1", "fun2"),
    class = c("S3", "S4"),
    code_script = c("R/fun1.R", "R/fun2.R"),
    stringsAsFactors = FALSE
  )
  
  # Stub all dependencies
  mockery::stub(create_traceability_matrix, "contains_r_folder", function(path) TRUE)
  mockery::stub(create_traceability_matrix, "get_exports", function(path) mock_exports_df)
  mockery::stub(create_traceability_matrix, "map_functions_to_scripts", function(df, path, verbose) df)
  mockery::stub(create_traceability_matrix, "map_functions_to_docs", function(df, path, verbose) df)
  mockery::stub(create_traceability_matrix, "get_func_descriptions", function(pkg) list(fun1 = "desc1", fun2 = "desc2"))
  mockery::stub(create_traceability_matrix, "fine_grained_tms", function(tm, pkg) list(tm = tm))
  
  # Run the function with func_covr = NULL
  result <- create_traceability_matrix(pkg_name, pkg_source_path, func_covr = NULL, execute_coverage = TRUE)
  
  # Assertions
  expect_type(result, "list")
  expect_true("tm" %in% names(result))
  expect_equal(result$tm$coverage_percent, c(0, 0))
  expect_equal(result$tm$code_script, (c("R/fun1.R", "R/fun2.R")))
})

test_that("create_traceability_matrix creates dummy func_covr when coverage is missing", {
  pkg_name <- "mockPkg"
  pkg_source_path <- "mock/path"
  
  mock_exports_df <- data.frame(
    exported_function = c("fun1", "fun2"),
    class = c("S3", "S4"),
    code_script = c("R/fun1.R", "R/fun2.R"),
    stringsAsFactors = FALSE
  )
  
  bad_func_covr <- list(
    name = pkg_name,
    coverage = NULL,
    errors = NA,
    notes = NA
  )
  
  mockery::stub(create_traceability_matrix, "contains_r_folder", function(path) TRUE)
  mockery::stub(create_traceability_matrix, "get_exports", function(path) mock_exports_df)
  mockery::stub(create_traceability_matrix, "map_functions_to_scripts", function(df, path, verbose) df)
  mockery::stub(create_traceability_matrix, "map_functions_to_docs", function(df, path, verbose) df)
  mockery::stub(create_traceability_matrix, "get_func_descriptions", function(pkg) list(fun1 = "desc1", fun2 = "desc2"))
  mockery::stub(create_traceability_matrix, "fine_grained_tms", function(tm, pkg) list(tm = tm))
  
  result <- create_traceability_matrix(pkg_name, pkg_source_path, func_covr = bad_func_covr, execute_coverage = TRUE)
  
  expect_type(result, "list")
  expect_true("tm" %in% names(result))
  expect_equal(result$tm$coverage_percent, c(0, 0))
})

#' Run covr with failing tests
#' #' 
#' This function runs test coverage and skip analysis for a package using testthat or other supported frameworks.
#'
#' @param pkg_source_path Character. Path to the root of the package source.
#' @param test_pkg_data List. Output from `check_pkg_tests_and_snaps()` indicating test framework presence.
#'
#' @return skip_results - test coverage with PASS tests passing or FAIL for tests failing
#' 
#' @examples
#' \dontrun{
#' r = getOption("repos")
#' r["CRAN"] = "http://cran.us.r-project.org"
#' old <- options(repos = r)
#'
#' dp <- system.file("test-data", 
#'   "test.package.0014_0.1.0.tar.gz", 
#'   package = "risk.assessr")
#' 
#' # set up package
#' install_list <- set_up_pkg(dp)
#'
#' build_vignettes <- install_list$build_vignettes
#' package_installed <- install_list$package_installed
#' pkg_source_path <- install_list$pkg_source_path
#' rcmdcheck_args <- install_list$rcmdcheck_args
#'
#' # install package locally to ensure test works
#' package_installed <- install_package_local(pkg_source_path)
#' package_installed <- TRUE
#'
#' rcmdcheck_args$path <- pkg_source_path
#' test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
#' test_covr_list <- run_covr_skip_stf(pkg_source_path, test_pkg_data)
#' 
#' options(old)
#' }  
#' @export
run_covr_skip_stf <- function(pkg_source_path, test_pkg_data) {
  
  # Input checking
  checkmate::assert_string(pkg_source_path)
  checkmate::assert_directory_exists(pkg_source_path)
  
  # set up path to test directory 
  # convert fs_path to character string for test_dir
  path <- as.character(pkg_source_path)
  testdir <- file.path(path, "tests")
  
  pkg <- get_pkg_name(pkg_source_path)
  
  test_path <- get_test_path(test_pkg_data, testdir)
  
  if (is.null(test_path))  {
    message(glue::glue("No test directory found for {pkg}"))
    skip_results <- list(
      res_cov = NA,
      long_summary = NA,
      test_skip = NA
    )
    return(skip_results)
  }
  
  # set path as working directory in the test env
  setwd(path)
  
  message(glue::glue("Creating Default test reporter for {pkg}"))
  
  # set max fails to infinite
  testthat::set_max_fails(Inf)
  
  # default reporter
  sink(tempfile())
  test_results <- testthat::test_dir(
    path = test_path,
    reporter = testthat::default_reporter(),
    stop_on_failure = FALSE,
    stop_on_warning = FALSE,
    load_package = "source"
  )
  sink()  # Reset output
  message(glue::glue("Mapping package test structure for {pkg}"))
  
  # create path to test folder for test map
  map_path <- test_path
  
  # Map the hierarchy structure of a testthat test file
  # Query a test file to see what expectations are defined in it.
  test_map <- map_tests_stf(map_path)
  
  # create long summary
  test_results_long <- 
    get_tests_long_sum_stf(test_results)
  
  #rename columns for aligning in join
  lookup <- c(code_script = "context", context = "test")
  
  test_results_long <- dplyr::rename(test_results_long, all_of(lookup))
  
  # Extract failed/skipped expectations only
  test_results_filtered <- test_results_long[test_results_long$status != 'PASS', ]
 
  # Clean up file names
  test_results_filtered$file <- gsub('#(.*?)$', '', basename(test_results_filtered$file))
  
  # align column names in test map
  test_map <- test_map |> 
    dplyr::rename(context = "test")
  
  
  # Filter tests with status FAILED, ERROR, or SKIPPED
  status_filter <- c("FAILED", "ERROR", "SKIPPED")
  problem_tests <- subset(test_results_filtered, status %in% status_filter)
  
  # create empty df with 1 row if problem tests is empty
  if (nrow(problem_tests) == 0) {
    problem_tests <- data.frame(
      file = "",
      code_script = "",
      context = "",
      status = "",
      n = NA_integer_,
      time = NA_real_,
      stringsAsFactors = FALSE
    )
    
  }
  
  # check for failed tests
  failed <- any(test_results_long$status%in%c('ERROR','FAILED'))
  
  if (failed == TRUE) {
  
    test_skip <- get_tests_skip_stf(problem_tests, test_map)
    
    message(glue::glue("Creating skip test map for {pkg}"))
    
    cvr <- run_covr_safe_stf(pkg, map_path, test_skip, test_map)
    
    message(glue::glue("Creating test coverage for {pkg}"))
    
    res_sum <- create_results_summary(cvr)
    
    # structure the return list
    covr_list <- list(
      total_cov = res_sum$total_cov,
      res_cov = list(
        name = res_sum$res_cov$name,
        coverage = res_sum$res_cov$coverage,
        errors = res_sum$res_cov$errors,
        notes = res_sum$res_cov$notes
      ),
      long_summary = test_results_long,
      test_skip = test_skip
    )
  } else {
    message(glue::glue("Creating test results for no skipped tests for {pkg}"))
    covr_list <- create_covr_list_no_skip(test_map, 
                                          test_results, 
                                          pkg_source_path,
                                          pkg)
  }
  
  return(covr_list)
}

#' Run skip coverage safely in a separate R session
#'
#' This internal helper function uses `callr::r_safe()` to execute the
#' `coverage_skip()` function from the `risk.assessr` package in a clean R session.
#' This avoids issues with loaded namespaces (e.g., `dplyr`) that may interfere with coverage analysis.
#'
#' It includes error handling and a configurable timeout to prevent long-running or stuck processes.
#'
#' @param pkg Character. Name of the package being tested.
#' @param map_path Character. Path to the `testthat` directory containing mapped test structure.
#' @param test_skip Data frame. Skip map generated from problematic tests.
#' @param test_map Data frame. Mapped structure of test files and expectations.
#' @param timeout Numeric. Maximum time (in seconds) allowed for the coverage process to run. Default is 120 seconds.
#'
#' @return A coverage object returned by `coverage_skip()`, or `NULL` if an error occurs or timeout is reached.
#' @keywords internal
run_covr_safe_stf <- function(pkg, map_path, test_skip, test_map, timeout = 120) {
  tryCatch({
    callr::r_safe(
      function(pkg, map_path, test_skip, test_map) {
        # Load the required package inside the safe environment
        library(risk.assessr)
        
        # Run coverage
        create_coverage_skip_stf(
          test_path = map_path,
          test_skip = test_skip,
          test_map = test_map
        )
      },
      args = list(pkg = pkg, map_path = map_path, test_skip = test_skip, test_map = test_map),
      timeout = timeout
    )
  }, error = function(e) {
    message(glue::glue("Error running coverage for {pkg}: {e$message}"))
    return(NULL)
  })
}

#' @title Run covr with failing tests
#' @description Run \code{\link[covr]{package_coverage}} ignoring failing tests from \code{\link[testthat]{test_dir}}.
#' @param test_path character, path to test directory, Default: 'tests/testthat'
#' @param test_skip data.frame, mapping of failing tests, PARAM_DESCRIPTION,
#' @param test_map data.frame, mapping of all the tests in the package
#' @return NULL
#' @details If test_skip is NULL then a new test_dir will be run internally checking
#' for failing tests.
#' @seealso
#'  \code{\link[testthat]{test_dir}}
#'  \code{\link[covr]{package_coverage}}
#' @rdname coverage_skip
#' @family utility
#' @export
#' @importFrom testthat test_dir
#' @importFrom covr package_coverage
create_coverage_skip_stf <- function(
    test_path = "tests/testthat",
    test_skip,
    test_map) {
  
  # Identify lines to skip based on test_skip
  
  test_skip_lines <- lapply(
    split(test_skip, test_skip$file),
    function(x) {
      unlist(mapply(seq, from = x$line1, to = x$line2, SIMPLIFY = FALSE))
    }
  )
  
  # If there are lines to skip, comment them out
  if (length(test_skip_lines) > 0) {
    on.exit({
      # Restore original test files by uncommenting lines
      for (nx in names(test_skip_lines)) {
        file_path <- file.path(test_path, nx)
        test_lines <- readLines(file_path, warn = FALSE)
        lines_to_uncomment <- test_skip_lines[[nx]]
        test_lines[lines_to_uncomment] <- gsub("^#", "", test_lines[lines_to_uncomment])
        cat(test_lines, file = file_path, sep = "\n")
      }
    }, add = TRUE)
    
    # Comment out failing test lines
    for (nx in names(test_skip_lines)) {
      file_path <- file.path(test_path, nx)
      test_lines <- readLines(file_path, warn = FALSE)
      lines_to_comment <- test_skip_lines[[nx]]
      test_lines[lines_to_comment] <- sprintf("#%s", test_lines[lines_to_comment])
      cat(test_lines, file = file_path, sep = "\n")
    }
  }
  
  # Run coverage
  covr::package_coverage()
}

#' Create a results summary for easier access to scores
#' 
#' @description - retrieve overall percentage and file coverage percentages
#' 
#' @param cvr - covr object with full results of skip coverage
#'
#' @return - dataframe with total scores
#'@keywords internal
create_results_summary <- function(cvr) {
  # check for empty cvr object
  if (is.null(cvr)) {
    # create filecoverage object
    filecoverage <- structure(
      c(NA_real_),
      dim = c(1),
      dimnames = list("no_coverage.R")
    )
    # create covr_list object
    covr_list <- list(
      total_cov = NA,
      res_cov = list(
        name = NA,
        coverage = list(
          filecoverage = filecoverage,
          totalcoverage = NA
        ),
        errors = NA,
        notes = NA
      )
    )
    
  } else {
    # Extract package name
    pkg_name <- attr(cvr, "package")$package
    # Extract overall coverage percentage
    total_cov <- covr::percent_coverage(cvr)
    
    # convert percentage into a score between 0 and 1
    total_cov <- total_cov / 100
    
    
    # Aggregate coverage by base filename
    file_coverage <- lapply(names(cvr), function(srcref_name) {
      entries <- cvr[[srcref_name]]
      values <- entries$value
      base_file <- sub(":.*", "", srcref_name)  # Extract base filename before first colon
      data.frame(filename = base_file, covered = values > 0)
    })
    
    file_coverage_df <- do.call(rbind, file_coverage)
    
    # Summarize coverage per base file
    file_summary <- file_coverage_df %>%
      dplyr::group_by(filename) %>%
      dplyr::summarise(
        percent = round(100 * sum(covered) / n(), 2),
        .groups = "drop"
      )
    
    # Create 1D numeric vector with dimnames (correct structure)
    filecoverage <- setNames(file_summary$percent, file_summary$filename)
    filecoverage <- structure(
      filecoverage,
      dim = c(length(filecoverage)),
      dimnames = list(names(filecoverage))
    )
    
    # Build result structure
    covr_list <- list(
      total_cov = total_cov,
      res_cov = list(
        name = pkg_name,
        coverage = list(
          filecoverage = filecoverage,
          totalcoverage = total_cov
        ),
        errors = NA,
        notes = NA
      )
    )
  }
  return(covr_list)
}


#' create covr_list when no tests are skipped
#'
#' @param test_map - R object with mapped tests
#' @param test_results_long - cvr object with no tests skipped
#' @param pkg_source_path - path to the package
#' @param pkg - name of the package
#'
#' @returns - summary cvr object
#' @keywords internal
create_covr_list_no_skip <- function(test_map, 
                                     test_results_long, 
                                     pkg_source_path, 
                                     pkg) {
  
  # Attempt to unload package safely
  tryCatch({
    if (pkg %in% loadedNamespaces()) {
      message(glue::glue("Unloading {pkg} package to avoid coverage conflict..."))
      unloadNamespace(pkg)
    }
  }, error = function(e) {
    message(glue::glue("Warning: Failed to unload {pkg} package. Proceeding anyway."))
    message("Details: ", e$message)
  })
  
  # Run coverage on the package
  message(glue::glue("Creating package coverage for {pkg}"))
  
  coverage <- covr::package_coverage(path = pkg_source_path, type = "tests")
  
  # Enrich each test entry with passed, failed, and skipped counts
  test_results_long <- lapply(test_results_long, function(entry) {
    entry$passed <- sum(sapply(entry$results, function(res) {
      inherits(res, "expectation_success")
    }))
    entry$failed <- sum(sapply(entry$results, function(res) {
      inherits(res, "expectation_failure")
    }))
    entry$skipped <- sum(sapply(entry$results, function(res) {
      inherits(res, "expectation_skip")
    }))
    return(entry)
  })
  
  # Extract coverage list from covr::coverage_to_list
  coverage_list <- covr::coverage_to_list(coverage)
  
  # Extract filecoverage and totalcoverage
  filecoverage <- coverage_list$filecoverage
  totalcoverage <- coverage_list$totalcoverage
  
  # Aggregate passed and failed counts
  total_passed <- sum(sapply(test_results_long, function(entry) entry$passed))
  total_failed <- sum(sapply(test_results_long, function(entry) entry$failed))
  
  # Collect skipped test entries
  tests_skipped <- Filter(function(entry) entry$skipped > 0, test_results_long)
  
  # Construct res_cov
  res_cov <- list(
    name = pkg,
    coverage = list(
      filecoverage = filecoverage,
      totalcoverage = totalcoverage
    ),
    errors = NA,
    notes = NA,
    passed = total_passed,
    failed = total_failed
  )
  
  # Compute total_cov
  total_cov <- if (!is.na(totalcoverage)) totalcoverage / 100 else NA_real_
  
  # Final covr_list
  covr_list <- list(
    total_cov = total_cov,
    res_cov = res_cov,
    tests_skipped = tests_skipped
  )
  
  # remove yarn datset created by pls package
  if (exists("yarn", envir = .GlobalEnv)) {
    rm(yarn, envir = .GlobalEnv)
  }
  
  return(covr_list)
}

test_that("running run_covr_skip for test package in tar file", {
  skip_on_cran()
  r = getOption("repos")
  r["CRAN"] = "http://cran.us.r-project.org"
  options(repos = r)
  
  dp <- system.file("test-data", "test.package.0014_0.1.0.tar.gz",
                    package = "risk.assessr")
  
  # set up package
  install_list <- set_up_pkg(dp)
  
  build_vignettes <- install_list$build_vignettes
  package_installed <- install_list$package_installed
  pkg_source_path <- install_list$pkg_source_path
  rcmdcheck_args <- install_list$rcmdcheck_args
  
  # install package locally to ensure test works
  package_installed <- install_package_local(pkg_source_path)
  package_installed <- TRUE
  
  if (package_installed == TRUE ) { 
    rcmdcheck_args$path <- pkg_source_path
    
    test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
    
    results <- run_covr_skip_stf(pkg_source_path, test_pkg_data)
    
    testthat::expect_identical(length(results), 4L)
    
    testthat::expect_true(checkmate::check_class(results, "list"))
    
    testthat::expect_identical(length(results$res_cov), 4L)
    
    testthat::expect_type(results$res_cov, "list")
  
    testthat::expect_true(length(results$res_cov) > 0)
    
    testthat::expect_true(checkmate::check_class(results$long_summary, "data.frame"))
    testthat::expect_true(all(!is.na(results$long_summary$file)))
    testthat::expect_true(all(!is.na(results$long_summary$code_script)))
    testthat::expect_true(all(!is.na(results$long_summary$context)))
    testthat::expect_true(all(!is.na(results$long_summary$status)))
    testthat::expect_true(all(checkmate::test_numeric(results$long_summary$n)))
    testthat::expect_true(all(checkmate::test_numeric(results$long_summary$time)))
    
    testthat::expect_true(checkmate::check_class(results$test_skip, "data.frame"))
    testthat::expect_true(all(!is.na(results$test_skip$file)))
    testthat::expect_true(all(checkmate::test_numeric(results$test_skip$line1)))
    testthat::expect_true(all(checkmate::test_numeric(results$test_skip$line2)))
  } 
})


test_that("run_covr_skip_stf filters test results and generates skip map", {
  # Mock testthat_long_summary output
  mock_test_results_long <- data.frame(
    file = c("test-a.R", "test-b.R", "test-c.R"),
    context = c("script1", "script2", "script3"),
    test = c("test 1", "test 2", "test 3"),
    status = c("FAILED", "PASS", "SKIPPED"),
    n = c(1, 1, 1),
    time = c(0.01, 0.02, 0.03),
    stringsAsFactors = FALSE
  )
  
  # Mock test map
  mock_test_map <- data.frame(
    test = c("test 1", "test 2", "test 3"),
    expectation = c("expect_equal", "expect_true", "expect_error"),
    line1 = c(2, 4, 6),
    line2 = c(2, 4, 6),
    file = c("test-a.R", "test-b.R", "test-c.R"),
    stringsAsFactors = FALSE
  )
  
  # Mock test_skip output
  mock_test_skip <- data.frame(
    file = c("test-a.R", "test-c.R"),
    line1 = c(2, 6),
    line2 = c(2, 6),
    stringsAsFactors = FALSE
  )
  
  # Mock test_pkg_data
  mock_test_pkg_data <- list(
    has_testthat = TRUE,
    has_testit = FALSE,
    has_tests_base = FALSE
  )
  
  # Stub all dependencies
  mockery::stub(run_covr_skip_stf, "checkmate::assert_string", function(...) TRUE)
  mockery::stub(run_covr_skip_stf, "checkmate::assert_directory_exists", function(...) TRUE)
  mockery::stub(run_covr_skip_stf, "get_pkg_name", function(...) "mockpkg")
  mockery::stub(run_covr_skip_stf, "setwd", function(...) NULL)
  mockery::stub(run_covr_skip_stf, "message", function(...) NULL)
  mockery::stub(run_covr_skip_stf, "testthat::set_max_fails", function(...) NULL)
  mockery::stub(run_covr_skip_stf, "testthat::test_dir", function(...) list())
  mockery::stub(run_covr_skip_stf, "map_tests_stf", function(...) mock_test_map)
  mockery::stub(run_covr_skip_stf, "get_tests_long_sum_stf", function(...) mock_test_results_long)
  mockery::stub(run_covr_skip_stf, "get_tests_skip_stf", function(...) mock_test_skip)
  mockery::stub(run_covr_skip_stf, "run_covr_safe_stf", function(pkg, map_path, test_skip, test_map) {
    return("mock_coverage")
  })
  mockery::stub(run_covr_skip_stf, "create_results_summary", function(cvr) {
    list(
      total_cov = 85.5,
      res_cov = list(
        name = "mockpkg",
        coverage = 85.5,
        errors = NULL,
        notes = "Coverage summary generated"
      )
    )
  })
  mockery::stub(run_covr_skip_stf, "package_name", function(...) "mockpkg")
  mockery::stub(run_covr_skip_stf, "unloadNamespace", function(...) NULL)
  
  # Run the function with test_pkg_data passed explicitly
  result <- run_covr_skip_stf(pkg_source_path = tempdir(), test_pkg_data = mock_test_pkg_data)
  
  # Check result
  expect_type(result, "list")
  expect_equal(result$total_cov, 85.5)
  expect_equal(result$res_cov$name, "mockpkg")
  expect_equal(result$res_cov$coverage, 85.5)
  expect_equal(result$res_cov$notes, "Coverage summary generated")
  expect_equal(result$test_skip, mock_test_skip)
  expect_equal(result$long_summary$context, mock_test_results_long$test)
})


test_that("run_covr_skip_stf processes no test dir and generates message", {
  # Simulate a package with no recognizable test directories
  mock_test_pkg_data <- list(
    has_testthat = FALSE,
    has_testit = FALSE,
    has_tests_base = FALSE
  )
  
  # Stub only the necessary dependencies
  mockery::stub(run_covr_skip_stf, "checkmate::assert_string", function(...) TRUE)
  mockery::stub(run_covr_skip_stf, "checkmate::assert_directory_exists", function(...) TRUE)
  mockery::stub(run_covr_skip_stf, "get_pkg_name", function(...) "mockpkg")
  mockery::stub(run_covr_skip_stf, "setwd", function(...) NULL)
  mockery::stub(run_covr_skip_stf, "message", function(...) NULL)
  
  # Run the function with no test directory
  result <- run_covr_skip_stf(pkg_source_path = tempdir(), test_pkg_data = mock_test_pkg_data)
  
  # Assertions
  testthat::expect_type(result, "list")
  testthat::expect_named(result, c("res_cov", "long_summary", "test_skip"))
  testthat::expect_true(all(vapply(result, function(x) all(is.na(x)), logical(1))))
})


test_that("run_covr_skip_stf handles missing test directory correctly", {
  # Create a temporary directory to simulate a package source path
  temp_pkg_path <- tempfile("pkg")
  dir.create(temp_pkg_path)
  
  # Mock test_pkg_data to simulate no test directories
  mock_test_pkg_data <- list(
    has_testthat = FALSE,
    has_testit = FALSE,
    has_tests_base = FALSE
  )
  
  # Stub required dependencies
  mockery::stub(run_covr_skip_stf, "checkmate::assert_string", function(...) TRUE)
  mockery::stub(run_covr_skip_stf, "checkmate::assert_directory_exists", function(...) TRUE)
  mockery::stub(run_covr_skip_stf, "get_pkg_name", function(...) "dummyPkg")
  mockery::stub(run_covr_skip_stf, "setwd", function(...) NULL)
  mockery::stub(run_covr_skip_stf, "message", function(...) NULL)
  
  # Run the function with the mocked test_pkg_data
  result <- run_covr_skip_stf(pkg_source_path = temp_pkg_path, 
                          test_pkg_data = mock_test_pkg_data)
  
  # Check that the result contains NAs as expected
  expect_type(result, "list")
  expect_named(result, c("res_cov", "long_summary", "test_skip"))
  expect_true(is.na(result$res_cov))
  expect_true(is.na(result$long_summary))
  expect_true(is.na(result$test_skip))
})

test_that("fallback problem_tests is created when no failed/skipped/error tests", {
  # Mocked test_map structure
  fake_test_map <- data.frame(
    test = rep(c("two working tests", "three working tests"), each = 2),
    expectation = rep("expect_equal", 4),
    line1 = c(2, 4, 8, 10),
    line2 = c(2, 4, 8, 10),
    file = rep("test-myscript1.R", 4),
    stringsAsFactors = FALSE
  )
  
  # Mocked test_results_long structure with only PASS statuses
  fake_test_results_long <- data.frame(
    file = rep("test-myscript1.R#L", 2),
    context = rep("myscript1", 2),
    test = c("two working tests", "three working tests"),
    status = rep("PASS", 2),
    n = c(2, 3),
    time = c(0.01, 0.01),
    stringsAsFactors = FALSE
  )
  
  # Stub dependencies
  mockery::stub(run_covr_skip_stf, "get_test_path", function(...) "tests")
  mockery::stub(run_covr_skip_stf, "testthat::test_dir", function(...) list())
  mockery::stub(run_covr_skip_stf, "map_tests_stf", function(...) fake_test_map)
  mockery::stub(run_covr_skip_stf, "get_tests_long_sum_stf", function(...) fake_test_results_long)
  mockery::stub(run_covr_skip_stf, "create_covr_list_no_skip", function(test_map, test_results, pkg_source_path, pkg) {
    list(
      total_cov = 0.5,
      res_cov = list(
        name = pkg,
        coverage = list(filecoverage = fake_test_map, totalcoverage = 50),
        errors = NA,
        notes = NA,
        passed = 5,
        failed = 0
      ),
      tests_skipped = list()
    )
  })
  
  # Run the function
  result <- run_covr_skip_stf(pkg_source_path = tempdir(), test_pkg_data = NULL)
  
  # Check that fallback logic was triggered and result is structured correctly
  expect_true(is.list(result))
  expect_named(result, c("total_cov", "res_cov", "tests_skipped"))
  expect_equal(result$res_cov$passed, 5)
  expect_equal(result$res_cov$failed, 0)
})

test_that("run_covr_safe_stf returns coverage object on success", {
  # Mock coverage object
  mock_coverage <- list(coverage = TRUE)
  
  # Stub callr::r_safe to return mock coverage
  mockery::stub(run_covr_safe_stf, "callr::r_safe", function(...) mock_coverage)
  
  result <- run_covr_safe_stf(
    pkg = "mockpkg",
    map_path = "mock/path",
    test_skip = data.frame(),
    test_map = data.frame(),
    timeout = 60
  )
  
  expect_equal(result, mock_coverage)
})

test_that("run_covr_safe_stf returns NULL on error", {
  # Stub callr::r_safe to throw an error
  mockery::stub(run_covr_safe_stf, "callr::r_safe", function(...) stop("Simulated error"))
  
  result <- run_covr_safe_stf(
    pkg = "mockpkg",
    map_path = "mock/path",
    test_skip = data.frame(),
    test_map = data.frame(),
    timeout = 60
  )
  
  expect_null(result)
})

test_that("create_coverage_skip_stf comments and uncomments lines correctly", {
  # Sample mock data
  mock_test_skip <- data.frame(
    file = c("test-script.R", "test-script.R"),
    line1 = c(2, 5),
    line2 = c(3, 6)
  )
  
  mock_test_map <- data.frame(
    context = c("test context"),
    expectation = c("expect_equal"),
    line1 = c(2),
    line2 = c(2),
    file = c("test-script.R")
  )
  
  # Mock lines in a test file
  mock_lines <- c(
    "test_that(\"test 1\", { expect_equal(1, 1) })",
    "test_that(\"test 2\", { expect_equal(2, 2) })",
    "test_that(\"test 3\", { expect_equal(3, 3) })",
    "test_that(\"test 4\", { expect_equal(4, 4) })",
    "test_that(\"test 5\", { expect_equal(5, 5) })",
    "test_that(\"test 6\", { expect_equal(6, 6) })"
  )
  
  # Stub readLines to return mock lines
  mockery::stub(create_coverage_skip_stf, "readLines", function(...) mock_lines)
  
  
  commented_lines <- c(2, 3, 5, 6)
  
  # Stub cat to capture output instead of writing to file
  mockery::stub(create_coverage_skip_stf, "cat", function(lines, file, sep) {
    # expect_true(all(grepl("^#", lines[commented_lines])))
    # expect_true(all(grepl("^#", lines[2])))
    # expect_true(all(grepl("^#", lines[5:6])))
  })
  
  # Stub covr::package_coverage to avoid running actual coverage
  mockery::stub(create_coverage_skip_stf, "covr::package_coverage", function(...) "mock_coverage")
  
  result <- create_coverage_skip_stf(
    test_path = "tests/testthat",
    test_skip = mock_test_skip,
    test_map = mock_test_map
  )
  
  expect_equal(result, "mock_coverage")
})


test_that("create_results_summary returns correct structure and values", {
  # Mock coverage object
  mock_cvr <- list(
    "file1.R:1" = list(value = c(1, 0, 1)),
    "file2.R:2" = list(value = c(0, 0, 1)),
    "file1.R:3" = list(value = c(1, 1))
  )
  
  # Add package attribute
  attr(mock_cvr, "package") <- list(package = "mockpkg")
  
  # Stub covr::percent_coverage to return a fixed value
  mockery::stub(create_results_summary, "covr::percent_coverage", function(x) 75.0)
  
  # Run the function
  result <- create_results_summary(mock_cvr)
  
  # Check top-level structure
  expect_type(result, "list")
  expect_named(result, c("total_cov", "res_cov"))
  
  # Check total coverage
  expect_equal(result$total_cov, 0.75)
  
  # Check res_cov structure
  expect_type(result$res_cov, "list")
  expect_equal(result$res_cov$name, "mockpkg")
  expect_type(result$res_cov$coverage, "list")
  
  # Check file coverage vector
  filecoverage <- result$res_cov$coverage$filecoverage
  expect_true(is.numeric(filecoverage))
  expect_true(!is.null(dimnames(filecoverage)))
  
  # Check that coverage values are correct
  expect_equal(names(filecoverage), c("file1.R", "file2.R"))
  
  expect_equal(filecoverage["file1.R"], round(100 * 4/5, 2), ignore_attr = TRUE)
  expect_equal(filecoverage["file2.R"], round(100 * 1/3, 2), ignore_attr = TRUE)
  
  
  # Check notes and errors
  expect_true(is.na(result$res_cov$errors))
  expect_true(is.na(result$res_cov$notes))
})


test_that("create_results_summary handles NULL input correctly", {
  # Run the function with NULL input
  result <- create_results_summary(NULL)
  
  # Check top-level structure
  expect_type(result, "list")
  expect_named(result, c("total_cov", "res_cov"))
  
  # Check that total_cov is NA
  expect_true(is.na(result$total_cov))
  
  # Check res_cov structure
  expect_type(result$res_cov, "list")
  expect_true(is.na(result$res_cov$name))
  expect_type(result$res_cov$coverage, "list")
  
  # Check filecoverage is an empty numeric vector with dimnames
  filecoverage <- result$res_cov$coverage$filecoverage
  expect_true(is.numeric(filecoverage))
  expect_equal(length(filecoverage), 1)
  expect_equal(dim(filecoverage), c(1))
  expect_true(is.list(dimnames(filecoverage)))
  expect_equal(length(dimnames(filecoverage)), 1)
  expect_equal(dimnames(filecoverage)[[1]], "no_coverage.R")
  
  
  # Check totalcoverage is NA
  expect_true(is.na(result$res_cov$coverage$totalcoverage))
  
  # Check notes and errors
  expect_true(is.na(result$res_cov$errors))
  expect_true(is.na(result$res_cov$notes))
})


test_that("create_covr_list_no_skip returns correct structure with mocked coverage", {
  # Mock test_map
  test_map <- data.frame(
    context = rep("Some test", 3),
    expectation = rep("expect_equal", 3),
    line1 = c(10, 20, 30),
    line2 = c(11, 21, 31),
    file = rep("test_file.R", 3),
    stringsAsFactors = FALSE
  )
  
  # Mock test_results_long
  test_results_long <- list(
    list(
      file = "test_file.R",
      context = "Some test",
      test = "Test description",
      user = 0.01,
      system = 0.00,
      real = 0.01,
      results = list(
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition"))
      )
    )
  )
  
  # Mock coverage object
  mock_coverage <- list(dummy = TRUE)
  
  # Mock coverage_to_list output
  mock_coverage_list <- list(
    filecoverage = list("R/test_file.R" = 75.0),
    totalcoverage = 75.0
  )
  
  # Stub covr::package_coverage and covr::coverage_to_list
  mockery::stub(create_covr_list_no_skip, "covr::package_coverage", mock_coverage)
  mockery::stub(create_covr_list_no_skip, "covr::coverage_to_list", mock_coverage_list)
  
  # Run the function
  result <- create_covr_list_no_skip(
    test_map = test_map,
    test_results_long = test_results_long,
    pkg_source_path = "path/to/pkg",
    pkg = "mockpkg"
  )
  
  # Assertions
  expect_type(result, "list")
  expect_equal(result$total_cov, 0.75)
  expect_equal(result$res_cov$name, "mockpkg")
  expect_equal(result$res_cov$coverage$totalcoverage, 75.0)
  expect_equal(result$res_cov$passed, 3)
  expect_equal(result$res_cov$failed, 0)
  expect_equal(length(result$tests_skipped), 0)
})


test_that("create_covr_list_no_skip unloads package when loaded", {
  # Mock inputs
  # Mock test_map
  test_map <- data.frame(
    context = rep("Some test", 3),
    expectation = rep("expect_equal", 3),
    line1 = c(10, 20, 30),
    line2 = c(11, 21, 31),
    file = rep("test_file.R", 3),
    stringsAsFactors = FALSE
  )
  
  # Mock test_results_long
  test_results_long <- list(
    list(
      file = "test_file.R",
      context = "Some test",
      test = "Test description",
      user = 0.01,
      system = 0.00,
      real = 0.01,
      results = list(
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition"))
      )
    )
  )
  pkg_source_path <- "mock/path"
  pkg <- "mockpkg"
  
  # Mock unloadNamespace to simulate successful unload
  mock_unloadNamespace <- function(pkg) {
    message(glue::glue("Mock unloadNamespace called for {pkg}"))
  }
  
  # Stub loadedNamespaces to simulate that the package is loaded
  mock_loadedNamespaces <- function() {
    c("mockpkg", "stats", "utils")
  }
  
  # Stub covr::package_coverage to return dummy coverage
  mock_package_coverage <- function(path, type) {
    structure(list(), class = "coverage")
  }
  
  # Stub covr::coverage_to_list to return dummy coverage list
  
  mock_coverage_to_list <- function(x) {
    list(
      filecoverage = matrix(1),
      totalcoverage = 100  
    )
  }
  
  # Apply stubs
  mockery::stub(create_covr_list_no_skip, "unloadNamespace", mock_unloadNamespace)
  mockery::stub(create_covr_list_no_skip, "loadedNamespaces", mock_loadedNamespaces)
  mockery::stub(create_covr_list_no_skip, "covr::package_coverage", mock_package_coverage)
  mockery::stub(create_covr_list_no_skip, "covr::coverage_to_list", mock_coverage_to_list)
  
  # Run the function
  result <- create_covr_list_no_skip(test_map, test_results_long, pkg_source_path, pkg)
  
  # Assertions
  expect_equal(result$total_cov, 1)
  expect_true(is.list(result$res_cov))
})



test_that("create_covr_list_no_skip handles error when unloading package", {
  # Mock inputs
  test_map <- data.frame(
    context = rep("Some test", 3),
    expectation = rep("expect_equal", 3),
    line1 = c(10, 20, 30),
    line2 = c(11, 21, 31),
    file = rep("test_file.R", 3),
    stringsAsFactors = FALSE
  )
  
  # Mock test_results_long
  test_results_long <- list(
    list(
      file = "test_file.R",
      context = "Some test",
      test = "Test description",
      user = 0.01,
      system = 0.00,
      real = 0.01,
      results = list(
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition"))
      )
    )
  )
  pkg_source_path <- "mock/path"
  pkg <- "mockpkg"
  
  # Mock unloadNamespace to simulate failure
  mock_unloadNamespace <- function(pkg) {
    stop("Simulated unload failure")
  }
  
  # Stub loadedNamespaces to simulate that the package is loaded
  mock_loadedNamespaces <- function() {
    c("mockpkg", "stats", "utils")
  }
  
  # Stub covr::package_coverage to return dummy coverage
  mock_package_coverage <- function(path, type) {
    structure(list(), class = "coverage")
  }
  
  mock_coverage_to_list <- function(x) {
    list(
      filecoverage = matrix(1),
      totalcoverage = 100  # top-level element
    )
  }
  
  # Apply stubs
  mockery::stub(create_covr_list_no_skip, "unloadNamespace", mock_unloadNamespace)
  mockery::stub(create_covr_list_no_skip, "loadedNamespaces", mock_loadedNamespaces)
  mockery::stub(create_covr_list_no_skip, "covr::package_coverage", mock_package_coverage)
  mockery::stub(create_covr_list_no_skip, "covr::coverage_to_list", mock_coverage_to_list)
  
  # Run the function
  result <- create_covr_list_no_skip(test_map, test_results_long, pkg_source_path, pkg)
  
  expect_equal(result$total_cov, 1)
  expect_true(is.list(result$res_cov))
  expect_equal(result$res_cov$coverage$totalcoverage, 100)
})


test_that("create_covr_list_no_skip removes yarn from global environment", {
  # Create a mock yarn object in the global environment
  assign("yarn", "mock_data", envir = .GlobalEnv)
  expect_true(exists("yarn", envir = .GlobalEnv))  # Confirm it's there
  
  # Minimal mocks to allow function to run
  test_map <- data.frame(
    context = rep("Some test", 3),
    expectation = rep("expect_equal", 3),
    line1 = c(10, 20, 30),
    line2 = c(11, 21, 31),
    file = rep("test_file.R", 3),
    stringsAsFactors = FALSE
  )
  
  # Mock test_results_long
  test_results_long <- list(
    list(
      file = "test_file.R",
      context = "Some test",
      test = "Test description",
      user = 0.01,
      system = 0.00,
      real = 0.01,
      results = list(
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition")),
        structure("As expected", class = c("expectation_success", "expectation", "condition"))
      )
    )
  )
  pkg_source_path <- "mock/path"
  pkg <- "mockpkg"
  
  mock_unloadNamespace <- function(pkg) {}
  mock_loadedNamespaces <- function() character(0)
  mock_package_coverage <- function(path, type) structure(list(), class = "coverage")
  mock_coverage_to_list <- function(x) {
    list(
      filecoverage = matrix(1),
      totalcoverage = 100
    )
  }
  
  # Apply stubs
  mockery::stub(create_covr_list_no_skip, "unloadNamespace", mock_unloadNamespace)
  mockery::stub(create_covr_list_no_skip, "loadedNamespaces", mock_loadedNamespaces)
  mockery::stub(create_covr_list_no_skip, "covr::package_coverage", mock_package_coverage)
  mockery::stub(create_covr_list_no_skip, "covr::coverage_to_list", mock_coverage_to_list)
  
  # Run the function
  result <- create_covr_list_no_skip(test_map, test_results_long, pkg_source_path, pkg)
  
  # Check that yarn was removed
  expect_false(exists("yarn", envir = .GlobalEnv))
})

#' Check for Vignette Folder and .Rmd Files in a .tar File
#'
#' This function checks if a given .tar file contains a 'vignettes' folder
#' and if there is at least one .Rmd file within that folder. If both 'vignettes'
#' and 'inst/doc' folders exist, the function will return \code{FALSE}.
#'
#' @param tar_file A character string specifying the path to the .tar file to be checked.
#' 
#' @return A logical value: \code{TRUE} if the 'vignettes' folder exists and contains at least one .Rmd file,
#' and neither 'vignettes' nor 'inst/doc' folders are present, \code{FALSE} otherwise.
#'
#' @details The function checks if the specified file exists and has a valid .tar extension using \code{utils::untar}. If the file is empty or
#' any error occurs during the extraction, the function stops and returns an error message. If both 'vignettes'
#' and 'inst/doc' folders exist, the function returns \code{FALSE}. If the 'vignettes' folder exists and contains 
#' at least one .Rmd file, the function returns \code{TRUE}. Otherwise, it returns \code{FALSE}.
#'
#' @examples
#' \dontrun{
#'   tar_file <- system.file("test-data", "here-1.0.1.tar.gz", 
#'    package = "risk.assessr")
#'   result <- contains_vignette_folder(tar_file)
#'   print(result)
#' }
#' 
#' @import utils
#' @export
contains_vignette_folder <- function(tar_file) {
  
  if (!file.exists(tar_file)) {
    stop("File does not exist. Please provide a valid .tar file.")
  }
  
  # Check if the file has a .tar extension
  if (!grepl("\\.tar$", tar_file) && !grepl("\\.tar\\.gz$", tar_file) && 
      !grepl("\\.gz$", tar_file) && !grepl("\\.tgz$", tar_file) &&
      !grepl("\\.tar\\.bz2$", tar_file) && !grepl("\\.tbz2$", tar_file)) {
    stop("Unsupported file type. Please provide a .tar file.")
  }
  
  # Try to list the contents of the .tar file, handling any errors
  file_list <- tryCatch(
    {
      suppressWarnings(utils::untar(tar_file, list = TRUE))
    },
    error = function(e) {
      stop("Error in untar: ", conditionMessage(e))
    }
  )
  
  # If the tar file is empty, return FALSE
  if (length(file_list) == 0) {
    stop("Error in untar: file is empty")
  }
  
  # Normalize file paths to use forward slashes for consistency
  normalized_paths <- gsub("\\\\", "/", file_list)
  
  # Extract paths that contain the vignettes folder
  vignette_folder <- grep("/vignettes(/|$)", normalized_paths, value = TRUE)
  
  # Extract paths that contain the inst/doc folder
  inst_doc_folder <- grep("/inst/doc(/|$)", normalized_paths, value = TRUE)
  
  # Check if both folders exist
  if (length(vignette_folder) > 0 && length(inst_doc_folder) > 0) {
    return(FALSE)
  }
  
  # Check if the vignettes folder exists
  if (length(vignette_folder) == 0) {
    return(FALSE)
  }
  
  # Check for .Rmd files specifically within the vignette folder
  vignette_rmd_files <- grep("/vignettes/[^/]+\\.Rmd$", normalized_paths, value = TRUE)
  
  # Determine if there are any .Rmd files in the vignette folder
  has_rmd_in_vignette <- length(vignette_rmd_files) > 0
  
  return(has_rmd_in_vignette)
}

test_that("contains_vignette_folder handles non-existent file", {
  expect_error(contains_vignette_folder("nonexistent_file.tar.gz"), "File does not exist")
})

test_that("contains_vignette_folder handles non-existent file", {
  expect_error(contains_vignette_folder("eezrfrfrr"), "File does not exist")
})

test_that("empty folder", {
  
  # Temporary directory for testing
  temp_dir <- tempdir()
  
  # Define the folder structure using the temporary directory
  main_dir <- file.path(temp_dir, "old_structure")
  
  # Step 1: Create the Folder Structure and Multiple .Rmd Files
  dir.create(main_dir, showWarnings = FALSE)
  
  # Step 2: Create the .tar Archive
  tar_file <- file.path(temp_dir, "old_structure.tar.gz")
  tar(tar_file, files = main_dir)
  
  # Ensure cleanup happens even if the test fails
  withr::defer(unlink(main_dir, recursive = TRUE))
  withr::defer(unlink(tar_file))
  
  # Check that the tar file contains .Rmd files
  expect_error(contains_vignette_folder(tar_file), "Error in untar: file is empty")
})

test_that("correct repo with Non-tar file throws an error", {
  tar_file <- system.file("test-data/check_vignettes", "got_vignette.teeeeeeear", package = "risk.assessr")
  expect_error(contains_vignette_folder(tar_file), "Unsupported file type. Please provide a .tar file.")
})

test_that("test on package with vignette folder", {
  tar_file <- system.file("test-data/check_vignettes", "got_vignette.tar.gz", package = "risk.assessr")
  expect_true(contains_vignette_folder(tar_file))
  expect_silent(contains_vignette_folder(tar_file))
})

test_that("test on package without vignette folder", {
  
  tar_file <- system.file("test-data/check_vignettes", "no_vignette.tar.gz", package = "risk.assessr")
  expect_false(contains_vignette_folder(tar_file))
  expect_silent(contains_vignette_folder(tar_file))
})

test_that("test on package with vignette folder but no .rmd file", {
  
  tar_file <- system.file("test-data/check_vignettes", "just_vignette_folder.tar.gz", package = "risk.assessr")
  expect_false(contains_vignette_folder(tar_file))
  expect_silent(contains_vignette_folder(tar_file))
})

test_that("Unpacking a tar file works correctly", {
  pkg <- system.file("test-data", "here-1.0.1.tar.gz", 
                     package = "risk.assessr")
  
  extract_files <-  
    suppressWarnings(unpack_tarball(pkg))
  
  expect_true(checkmate::checkVector(extract_files))
  
  expect_true(checkmate::check_class(extract_files, "fs_path"))
  
  expect_true(checkmate::check_string(extract_files))
  
  expect_true(checkmate::check_directory_exists(extract_files))
})

test_that("Unpacking an empty tar file works correctly", {
  pkg <- system.file("test-data", "empty.tar.gz", 
                     package = "risk.assessr")
  
  extract_files <-  
    suppressWarnings(unpack_tarball(pkg))
  
  expect_true(checkmate::checkVector(extract_files))
  
  expect_true(checkmate::check_class(extract_files, "fs_path"))
  
  expect_identical(checkmate::check_string(extract_files), 
                   "Must have length 1"
  )
  
  expect_identical(checkmate::check_directory_exists(extract_files), 
                   "No directory provided"
  )
})  

---
title: "Get pubmed data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get pubmed data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

This vignette demonstrates how to retrieve citation data from PubMed using:

- `get_pubmed_count()` — to retrieve the total count of articles matching a term.
- `get_pubmed_by_year()` — to get article counts by publication year.

These functions are part of the `risk.assessr` package.

DISCLAIMER: This feature only works if you choose a good R package name.

---

## Setting Your PubMed API Key

To increase request limits and improve stability, you can use an NCBI API key.

To add a PubMed API key (token), please follow the steps [Here](https://www.ncbi.nlm.nih.gov/books/NBK25497/#chapter2.API_Keys)



```{r api-key-example, eval = FALSE}
# Replace with your actual API key
options(pubmed.api_key = "your_actual_ncbi_api_key")

```

---

## Load Required Libraries

```{r setup}
library(risk.assessr)
library(ggplot2)

package_name <- "limma"
```

---

## Total Number of Articles Matching a Term

```{r, message=FALSE, warning=FALSE, eval=FALSE}
pubmed_count <- get_pubmed_count(package_name)
pubmed_count
```

---

## Number of Articles by Year

```{r, message=FALSE, warning=FALSE, eval=FALSE}
citation_data <- get_pubmed_by_year(package_name, years_back = 20)
citation_data
```

---

## Plotting the Article Trend Over Time

```{r, message=FALSE, warning=FALSE, echo=FALSE, eval=FALSE}
ggplot(citation_data, aes(x = Year, y = Count)) +
  geom_col(fill = "steelblue") +
  labs(
    title = "PubMed Articles per Year",
    subtitle = paste("Search term:", package_name),
    x = "Publication Year",
    y = "Number of Articles"
  ) +
  theme_minimal()
```

---
title: "Examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Run risk analysis on the `admiral` package:

```bash
Rscript inst/examples/assessr.R '{"name": "admiral", "version": "1.1.1"}'
```	

---
title: "risk.assessr metrics"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{risk.assessr metrics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(risk.assessr)
```

```{r acquisition, eval=FALSE}
here <- risk.assessr::assess_pkg_r_package("here")
```

# Package Risk Assessment Summary

This report presents a risk assessment of the R package: **here**.


## Metadata

```{r, eval=FALSE}
here$results[c("pkg_name", "pkg_version", "sysname", "release", "date_time")]
```


## Documentation Metrics

```{r, eval=FALSE}
here$results[c("license_name", "has_examples", "has_vignettes", "has_news", 
               "has_website", "has_source_control", "export_help", 
               "export_calc", "covr", "check")]
```

## Dependencies

```{r, eval=FALSE}
here$results$dependencies
```

## Test Coverage

```{r, eval=FALSE}
here$covr_list$res_cov$coverage$totalcoverage
```

## CRAN Check Results

```{r, eval=FALSE}
here$check_list$res_check$errors
```


## Popularity Metrics

### CRAN Downloads

```{r, eval=FALSE}
here$results$download
```

### GitHub Statistics

```{r, eval=FALSE}
here$results$github_data
```

### Reverse Dependencies (Extended)

```{r, eval=FALSE}
head(here$results$rev_deps, 10)  
```

## Authorship

```{r, eval=FALSE}
here$results$author
```

## Hosting Information

```{r, eval=FALSE}
here$results$host
```

## Suggested Imports Review

```{r, eval=FALSE}
here$risk_analysis
```

[More info Here](https://github.com/pharmaverse/risk.assessr/blob/gh-pages/articles/define_custom_risk_rules.html)




## Advanced feature

### Traceability Matrix

```{r, eval=FALSE}
here$tm_list
```

[More info Here](https://github.com/pharmaverse/risk.assessr/blob/gh-pages/articles/Traceability_matrix.html)


## Suggested Imports Review

```{r, eval=FALSE}
here$results$suggested_deps
```

---
title: "&nbsp;"
output:
  rmarkdown::html_document:
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/",
  out.width = "100%"
)

# Suppress package loading messages
suppressMessages({
  library(dplyr)
  library(kableExtra)
  library(knitr)
})

suppressWarnings({
  library(htmltools)
  library(htmlwidgets)
})
```

# Package Assessment Report for `r get('pkg_name', envir = report_env)` version `r get('pkg_version', envir = report_env)` <a><img src="`r system.file('man/figures/logo.png', package = 'risk.assessr')`" align="right" height="100" /><br></a>



```{r risk_summary, echo=FALSE}
library(knitr)
library(kableExtra)

risk_summary_output <- get("risk_summary_output", envir = report_env)
license_maxes <- get("license_maxes", envir = report_env)

# Render the table using knitr and kableExtra
risk_summary_output_html <- knitr::kable(risk_summary_output,
                                         col.names = c("Type", "Values"),
                                         format = "html") %>%
  kableExtra::kable_styling("striped", 
                            position = "left", 
                            font_size = 12) %>%
  kableExtra::row_spec(0, 
                       bold = TRUE,
                       italic = TRUE,
                       background = "green") %>%
  kableExtra::add_header_above(c("Risk Assessment Summary" = 2), 
                               bold = TRUE, 
                               background = "green", 
                               color = "black")

# Apply conditional formatting based on Risk Profile values
risk_summary_output_html <- risk_summary_output_html %>%
  kableExtra::row_spec(which(risk_summary_output$Value == "Low"), background = "green") %>%
  kableExtra::row_spec(which(risk_summary_output$Value == "Medium"), background = "yellow") %>%
  kableExtra::row_spec(which(risk_summary_output$Value == "High"), background = "red")

# Apply conditional formatting for License metric
for (i in seq_len(nrow(risk_summary_output))) {
  if (risk_summary_output$Metric[i] == "License") {
    license <- risk_summary_output$Value[i]
    
    if (any(grepl(paste(license_maxes$high, collapse = "|"), license, ignore.case = TRUE))) {
      risk_summary_output_html <- risk_summary_output_html %>%
        kableExtra::row_spec(i, background = "red")
    } else if (any(grepl(paste(license_maxes$medium, collapse = "|"), license, ignore.case = TRUE))) {
      risk_summary_output_html <- risk_summary_output_html %>%
        kableExtra::row_spec(i, background = "yellow")
    } else if (any(grepl(paste(license_maxes$low, collapse = "|"), license, ignore.case = TRUE))) {
      risk_summary_output_html <- risk_summary_output_html %>%
        kableExtra::row_spec(i, background = "green")
    }

  }
}

# Display the table
risk_summary_output_html
```   

```{r risk_details, echo=FALSE, message=FALSE, warning=FALSE}
library(knitr)
library(kableExtra)
library(dplyr)
risk_details_output <- get("risk_details_output", envir = report_env)
code_covr_maxes <- get("code_covr_maxes", envir = report_env)
cmd_check_maxes <- get("cmd_check_maxes", envir = report_env)

# Convert percentage strings to numeric values for conditional formatting
risk_details_output <- risk_details_output %>%
  mutate(NumericValue = suppressWarnings(as.numeric(gsub("%", "", Value)) / 100))

# Select only the columns to display in the table
risk_details_output_display <- risk_details_output %>%
  select(Metric, Value)

# Display df
risk_details_output_html <- knitr::kable((risk_details_output_display),
                               col.names = c(
                                 "Type",
                                 "Details"),
                          "html")
kableExtra::kable_styling(risk_details_output_html, 
                          "striped", 
                          position = "left", 
                          font_size = 12) %>%
  kableExtra::row_spec(0, 
                       bold = TRUE,
                       italic = TRUE,
                       background = "green"
                               ) %>%
  kableExtra::row_spec(which(risk_details_output$Metric == "R CMD Check Score" & risk_details_output$NumericValue >= cmd_check_maxes$medium_max), 
                       background = "green") %>%
  kableExtra::row_spec(which(risk_details_output$Metric == "R CMD Check Score" & risk_details_output$NumericValue < cmd_check_maxes$medium_max), 
                       background = "red") %>%
  kableExtra::row_spec(which(risk_details_output$Metric == "R CMD Check Score" & risk_details_output$NumericValue > cmd_check_maxes$high_max & risk_details_output$NumericValue < cmd_check_maxes$medium_max), 
                       background = "yellow") %>%
  kableExtra::row_spec(which(risk_details_output$Metric == "Test Coverage Score" & risk_details_output$NumericValue > code_covr_maxes$medium_max), 
                       background = "green") %>%
  kableExtra::row_spec(which(risk_details_output$Metric == "Test Coverage Score" & risk_details_output$NumericValue < code_covr_maxes$high_max), 
                       background = "red") %>%
  kableExtra::row_spec(which(risk_details_output$Metric == "Test Coverage Score" & risk_details_output$NumericValue > code_covr_maxes$high_max & risk_details_output$NumericValue < code_covr_maxes$medium_max), 
                       background = "yellow") %>%
  kableExtra::add_header_above(c("Core Metrics and Assessment Run Details" = 2), bold = TRUE, background = "green", color = "black")
```   

```{r rcmdcheck, echo=FALSE}
library(knitr)
library(kableExtra)
rcmd_check_output <- get("rcmd_check_output", envir = report_env)

# Get the column indices based on the column names
# score_col <- which(names(rcmd_check_output) == "Score")
errors_col <- which(names(rcmd_check_output) == "Errors")
warnings_col <- which(names(rcmd_check_output) == "Warnings")
notes_col <- which(names(rcmd_check_output) == "Notes")

# Render the table using knitr and kableExtra
# Display df
rcmd_check_output_html <- knitr::kable((rcmd_check_output),
                               col.names = c(
                                 " ",
                                 "Score",
                                 "Errors",
                                 "Warnings",
                                 "Notes"
                                 ),
                          "html")
kableExtra::kable_styling(rcmd_check_output_html, 
                          "striped", 
                          position = "left", 
                          font_size = 12) %>%
  kableExtra::row_spec(0, 
                       bold = TRUE,
                       italic = TRUE,
                       background = "green" 
                               ) %>%
  kableExtra::column_spec(errors_col, 
                          background = ifelse(is.na(rcmd_check_output$Errors) | rcmd_check_output$Errors == "No errors", "green", "red")) %>%
  kableExtra::column_spec(warnings_col, 
                          background = ifelse(is.na(rcmd_check_output$Warnings) | rcmd_check_output$Warnings == "No warnings", "green", "yellow")) %>%
  kableExtra::column_spec(notes_col, 
                           background = ifelse(is.na(rcmd_check_output$Notes) | rcmd_check_output$Notes == "No notes", "green", "yellow")) %>%
  kableExtra::add_header_above(c("R CMD check" = 5), bold = TRUE, background = "green", color = "black") 
```

```{r test_coverage, echo=FALSE}
library(knitr)
library(kableExtra)

coverage_output <- get("coverage_output", envir = report_env)

render_coverage_dt <- function(df, title = "Test Coverage") {
  colnames(df) <- c("Function", "Coverage", "Errors", "Notes")
  tbl <- DT::datatable(df,
                       rownames = FALSE,
                       colnames = c("Function", "Coverage", "Errors", "Notes"),
                       options = list(
                         pageLength = 10,
                         lengthMenu = c(10, 20, 50),
                         search = list(search = ""),
                         dom = 'Bfrtip',
                         buttons = c('copy', 'csv', 'excel', 'pdf', 'print')
                       )) %>%
    DT::formatStyle(
      columns = 'Errors',
      backgroundColor = DT::styleEqual(
        levels = c("No testthat or testit configuration", "No test coverage errors", "Other Value"),
        values = c("red", "green", "red")
      )
    ) %>%
    DT::formatStyle(
      columns = 'Notes',
      backgroundColor = DT::styleEqual(
        levels = c("No test coverage notes", "Other Value"),
        values = c("green", "yellow")
      )
    )
  htmltools::tagList(
    htmltools::tags$h2(title, style = "font-weight: bold; background-color: green; color: black; text-align: center;"),
    tbl
  )
}

if (is.data.frame(coverage_output)) {
  render_coverage_dt(coverage_output)
} else {
  htmltools::tagList(lapply(names(coverage_output), function(fw) {
    render_coverage_dt(coverage_output[[fw]], title = paste("Test Coverage -", fw))
  }))
}
```

```{r vulnerabilities, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmltools)

vulnerabilities_output <- get("vulnerabilities_output", envir = report_env)

if (identical(names(vulnerabilities_output), "Message")) {

  # No known vulnerabilities: green banner reassuring the reader
  vulnerabilities_no_html <- knitr::kable(
    vulnerabilities_output,
    col.names = " ",
    format = "html"
  ) %>%
    kableExtra::kable_styling("striped", position = "left", font_size = 12) %>%
    kableExtra::row_spec(0, bold = TRUE, italic = TRUE, background = "green") %>%
    kableExtra::row_spec(1, background = "green", color = "black") %>%
    kableExtra::add_header_above(
      c("Security Vulnerabilities" = 1),
      bold = TRUE, background = "green", color = "black"
    )

  vulnerabilities_no_html

} else {

  # Known vulnerabilities present: red header + interactive table
  vulnerabilities_dt <- DT::datatable(
    vulnerabilities_output,
    rownames = FALSE,
    options = list(
      pageLength = 10,
      lengthMenu = c(10, 20, 50),
      search = list(search = ""),
      dom = 'Bfrtip'
    )
  )

  htmltools::tagList(
    htmltools::tags$h2(
      "Security Vulnerabilities",
      style = "font-weight: bold; background-color: red; color: black; text-align: center;"
    ),
    vulnerabilities_dt
  )
}
```

```{r doc_metrics, echo=FALSE}
library(knitr)
library(kableExtra)
doc_metrics_output <- get("doc_metrics_output", envir = report_env)


# --- Extract the doc metrics section ---
doc_metrics_df    <- doc_metrics_output$doc_metrics


# -------------------------
# 1) Documentation Metrics (keep existing styling)
# -------------------------
doc_metrics_output_html <- knitr::kable(
  doc_metrics_df,
  col.names = c("Metric Type", "Value"),
  format = "html"
)

doc_metrics_kbl <- kableExtra::kable_styling(
  doc_metrics_output_html,
  "striped",
  position = "left",
  font_size = 12
) %>%
  kableExtra::row_spec(0, bold = TRUE, italic = TRUE, background = "yellow") %>%
  kableExtra::row_spec(which(doc_metrics_df$Value != "Not Included"), background = "green") %>%
  kableExtra::row_spec(which(doc_metrics_df$Value == "Not Included"), background = "red") %>%
  # Highlight links present for URL-like metrics
  kableExtra::row_spec(
    which(
      doc_metrics_df$Metric %in% c("Has Bug Reports URL", "Has Source Control", "Has Website") &
        grepl("^https?://", doc_metrics_df$Value)
    ),
    background = "green"
  ) %>%
  # Maintainer present vs missing (treat NA/empty as missing)
  kableExtra::row_spec(
    which(doc_metrics_df$Metric == "Has Maintainer" & !is.na(doc_metrics_df$Value) & nzchar(doc_metrics_df$Value)),
    background = "green"
  ) %>%
  kableExtra::row_spec(
    which(doc_metrics_df$Metric == "Has Maintainer" & (is.na(doc_metrics_df$Value) | !nzchar(doc_metrics_df$Value))),
    background = "red"
  ) %>%
  # Treat "N/A" as not present
  kableExtra::row_spec(which(doc_metrics_df$Value == "N/A"), background = "red") %>%
  kableExtra::add_header_above(
    c("Documentation Metrics" = 2),
    bold = TRUE, background = "yellow", color = "black"
  )


  # render doc metrics
  doc_metrics_kbl

```

```{r doc_examples, echo=FALSE}
library(knitr)
library(kableExtra)
doc_metrics_output <- get("doc_metrics_output", envir = report_env)

# Fetch thresholds
example_maxes <- get("example_maxes", envir = report_env)


# --- Extract the example section ---

has_examples_df   <- doc_metrics_output$has_examples


# Scores are stored as attributes on the data.frames
has_examples_score <- attr(has_examples_df, "score")

has_examples_score <- has_examples_score * 100

# Build text for the sub-header
sub_header <- sprintf(
  "Percentage of functions with examples (score: %s%%)",
  ifelse(is.na(has_examples_score), "NA", has_examples_score)
)


# Determine sub-header color
header_color <- if (is.na(has_examples_score)) {
  "red"
} else if (has_examples_score > example_maxes$medium_max) {
  "green"
} else if (has_examples_score < example_maxes$high_max) {
  "red"
} else {
  "yellow"
}


# 1) Ensure no row names 
rownames(has_examples_df) <- NULL


# Ensure the data frame has exactly 4 columns in this order
colnames(has_examples_df) <- c(
  "Function Name",
  "Documentation Name",
  "Documentation Location",
  "Example"
)

examples_kbl <- has_examples_df %>%
  knitr::kable("html", escape = TRUE, row.names = FALSE) %>%
  kableExtra::kable_styling(
    bootstrap_options = c("striped"),
    position = "left",
    font_size = 12
  ) %>%
  # Sub-header BELOW main header (but ABOVE column names)
  kableExtra::add_header_above(
    setNames(ncol(has_examples_df), sub_header),
    bold = TRUE,
    background = header_color,
    color = "black"
  ) %>%
  # Main header ABOVE everything
  kableExtra::add_header_above(
    c("Documentation of Examples" = ncol(has_examples_df)),
    bold = TRUE,
    background = "yellow",
    color = "black"
  ) %>%
  # Style the column header row (row 0)
  kableExtra::row_spec(0, bold = TRUE, italic = TRUE, background = "yellow") %>%
  # Conditionally color data rows red where Example == "no example"
   kableExtra::row_spec(
    which(has_examples_df$Example == "no example" | has_examples_df$Example == "no Rd file"),
    background = "red")

examples_kbl

```

```{r func_docs_summary, echo=FALSE}
library(knitr)
library(kableExtra)


# --- Fetch documentation metrics and thresholds ---
doc_metrics_output <- get("doc_metrics_output", envir = report_env)
func_doc_maxes     <- get("func_doc_maxes", envir = report_env)

# Extract the has_docs dataframe
has_docs_df   <- doc_metrics_output$has_docs
has_docs_score <- attr(has_docs_df, "score")

has_docs_score <- has_docs_score * 100

# --- Build the summary dataframe (1 row) ---
num_doc_functions <- if (has_docs_score == 0) 0 else nrow(has_docs_df)

func_docs_summary <- data.frame(
  `Number of Documented Functions` = num_doc_functions,
  check.names = FALSE
)

# --- Build subheader text (same style as detailed table) ---
sub_header_docs <- sprintf(
  "Percentage of functions with documentation (score: %s%%)",
  ifelse(is.na(has_docs_score), "NA", has_docs_score)
)

# --- Determine header color using same logic as details display ---
header_color <- if (is.na(has_docs_score)) {
  "red"
} else if (has_docs_score > func_doc_maxes$medium_max) {
  "green"
} else if (has_docs_score < func_doc_maxes$high_max) {
  "red"
} else {
  "yellow"
}

# --- Build summary table using identical kableExtra styling ---
kbl_cols <- ncol(func_docs_summary)

func_docs_summary_html <- func_docs_summary %>%
  knitr::kable("html", escape = TRUE, row.names = FALSE) %>%
  kableExtra::kable_styling(
    bootstrap_options = c("striped"),
    position = "left",
    font_size = 12
  ) %>%
  
  # Sub-header BELOW main header
  kableExtra::add_header_above(
    setNames(kbl_cols, sub_header_docs),
    bold = TRUE,
    background = header_color,
    color = "black"
  ) %>%
  
  # Main header ABOVE everything
  kableExtra::add_header_above(
    setNames(kbl_cols, "Function Documentation Summary"),
    bold = TRUE,
    background = "yellow",
    color = "black"
  ) %>%
  
  # Style the column header row
  kableExtra::row_spec(
    0,
    bold = TRUE,
    italic = TRUE,
    background = "yellow"
  ) %>%
  
  
# Apply row coloring based on thresholds (mirrors reverse‑deps logic)
{
  val <- func_docs_summary$`Number of Documented Functions`

  if (!is.null(func_doc_maxes)) {

    if (val < func_doc_maxes$high_max) {
      kableExtra::row_spec(., 1, background = "red")

    } else if (val > func_doc_maxes$medium_max) {
      kableExtra::row_spec(., 1, background = "green")

    } else {
      kableExtra::row_spec(., 1, background = "yellow")
    }

  } else {
    .   # ← return table unchanged if no thresholds available
  }
}


func_docs_summary_html
```

```{r func_docs_details, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)

# Fetch the documentation dataframe
doc_metrics_output <- get("doc_metrics_output", envir = report_env)

has_docs_df  <- doc_metrics_output$has_docs

colnames(has_docs_df) <- c(
  "Function Name",
  "Documentation Name",
  "Documentation Location"
)

# Final ordering

rownames(has_docs_df) <- NULL

# Build the DT table
func_docs_details_html_DT <- DT::datatable(
  has_docs_df,
  rownames = FALSE,
  escape = TRUE,
  options = list(
    pageLength = 10,
    lengthMenu = c(10, 20, 50),
    search = list(search = "")
  )
) %>%
  htmlwidgets::onRender("function(el, x) {
    $(el).find('thead').hide();
  }")

# Display details table
func_docs_details_html_DT
```


```{r pop_metrics, echo=FALSE}
library(knitr)
library(kableExtra)
pop_metrics_output <- get("pop_metrics_output", envir = report_env)
pop_maxes <- get("pop_maxes", envir = report_env)

# Select only the columns to display in the table
pop_metrics_output_display <- pop_metrics_output %>%
  select(Metric, Value)

# Create the HTML table
pop_metrics_output_html <- knitr::kable(
  pop_metrics_output_display,
  col.names = c("Metric Type", "Value"),
  format = "html"
)

# Identify rows for conditional formatting
red_rows <- which(pop_metrics_output$Value %in% c("N/A", "0"))
green_rows <- which(
  pop_metrics_output$Metric %in% c("Date created", 
                                   "Data Extract Date", 
                                   "Stars", 
                                   "Forks", 
                                   "Recent Commits", 
                                   "Open Issues",
                                   "Downloads Last Month"
                                   ) &
    !(pop_metrics_output$Value %in% c("N/A", "0"))
)

# Apply styling
pop_metrics_output_html %>%
  kable_styling("striped", position = "left", font_size = 12) %>%
  row_spec(0, bold = TRUE, italic = TRUE, background = "yellow") %>%
  row_spec(red_rows, background = "red") %>%
  row_spec(green_rows, background = "green") %>%
  kableExtra::row_spec(which(pop_metrics_output$Metric == "Downloads Total" & pop_metrics_output$NumericValue >= pop_maxes$medium_max), 
                       background = "green") %>%
  kableExtra::row_spec(which(pop_metrics_output$Metric == "Downloads Total" & pop_metrics_output$NumericValue < pop_maxes$medium_max), 
                       background = "red") %>%
  kableExtra::row_spec(which(pop_metrics_output$Metric == "Downloads Total" & pop_metrics_output$NumericValue > pop_maxes$high_max & risk_details_output$NumericValue < pop_maxes$medium_max)) %>% 
  add_header_above(c("Popularity Metrics" = 2), bold = TRUE, background = "yellow", color = "black")
```

```{r deps_score, echo=FALSE}
library(knitr)
library(kableExtra)

deps_output <- get("deps_df", envir = report_env)
import_count <- get("import_count", envir = report_env)
suggest_count <- get("suggest_count", envir = report_env)
dep_maxes <- get("dep_maxes", envir = report_env)

# Create a summary data frame
deps_summary <- data.frame(
  No_of_Dependencies = nrow(deps_output),
  No_of_Imports = import_count,
  No_of_Suggests = suggest_count
)

# Render the summary table using knitr and kableExtra
deps_summary_html <- knitr::kable(deps_summary,
                                  col.names = c( 
                                                "Total Number of Dependencies",
                                                "Number of Imports",
                                                "Number of Suggests"
                                                ),
                                  "html") %>%
  kableExtra::kable_styling("striped", 
                            position = "left", 
                            font_size = 12) %>%
  kableExtra::row_spec(0, 
                       bold = TRUE,
                       italic = TRUE,
                       background = "#AA4A44") %>%
  kableExtra::row_spec(which(deps_summary$No_of_Imports < dep_maxes$low_max), 
                       background = "green") %>%
  kableExtra::row_spec(which(deps_summary$No_of_Imports > dep_maxes$medium_max), 
                       background = "red") %>%
  kableExtra::row_spec(which(deps_summary$No_of_Imports > dep_maxes$low_max & deps_summary$No_of_Dependencies < dep_maxes$medium_max), 
                       background = "yellow") %>%
  kableExtra::add_header_above(c("Dependencies" = 3), bold = TRUE, background = "#AA4A44", color = "black")

# Display the summary table
deps_summary_html
```

```{r deps_details, echo=FALSE}
library(knitr)
library(DT)
# Get the dependency score from report_env
deps_output <- get("deps_df", envir = report_env)

# Render the detailed table using DT
deps_output_html_DT <- DT::datatable(deps_output, 
                                     options = list(
                                       pageLength = 10,  # Number of rows to display per page
                                       lengthMenu = c(10, 20, 50),  # Options for number of rows per page
                                       search = list(search = "")  # Enable search functionality
                                     ))

# Display the interactive DT table
deps_output_html_DT
```

```{r rev_deps_sum, echo=FALSE}
library(knitr)
library(kableExtra)

# Get the rev deps data 
rev_deps_output <- get("rev_deps_df", envir = report_env)
rev_deps_summary_list <- get("rev_deps_summary", envir = report_env)
revdep_maxes <- get("revdep_maxes", envir = report_env)

# Convert the summary list to a data frame
rev_deps_summary <- data.frame(
  No_of_Dependencies = rev_deps_summary_list$rev_deps_no
)

# Render the summary table using knitr and kableExtra
rev_deps_summary_html <- knitr::kable(rev_deps_summary,
                                      col.names = c("Number of Reverse Dependencies"),
                                      "html") %>%
  kableExtra::kable_styling("striped", 
                            position = "left", 
                            font_size = 12) %>%
  kableExtra::row_spec(0, 
                       bold = TRUE,
                       italic = TRUE,
                       background = "#AA4A44") %>%
  kableExtra::row_spec(which(rev_deps_summary$No_of_Dependencies < revdep_maxes$high_max), 
                       background = "red") %>%
  kableExtra::row_spec(which(rev_deps_summary$No_of_Dependencies > revdep_maxes$medium_max), 
                       background = "green") %>%
  kableExtra::row_spec(which(rev_deps_summary$No_of_Dependencies > revdep_maxes$high_max & rev_deps_summary$No_of_Dependencies < revdep_maxes$medium_max), 
                       background = "yellow") %>%
  kableExtra::add_header_above(c("Reverse Dependencies" = 1), bold = TRUE, background = "#AA4A44", color = "black")

# Display the summary table
rev_deps_summary_html
```

```{r rev_deps_details, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)


# Get the rev deps detailed data 
rev_deps_output <- get("rev_deps_df", envir = report_env)

# Render the detailed table using DT
rev_deps_output_html_DT <- DT::datatable(rev_deps_output, 
                                     options = list(
                                       pageLength = 10,  # Number of rows to display per page
                                       lengthMenu = c(10, 20, 50),  # Options for number of rows per page
                                       search = list(search = "")
                                     ))  %>%
  htmlwidgets::onRender("function(el, x) {
    $(el).find('thead').hide();
  }")

# Display the interactive DT table
rev_deps_output_html_DT
```


```{r, echo=FALSE, results='asis'}
cat('
<div style="margin-bottom: 20px;">
  <label for="matrixSelector" style="
    background-color: purple;
    color: white;
    font-size: 20px;
    font-weight: bold;
    padding: 10px;
    border-radius: 5px;
    display: inline-block;
    margin-bottom: 10px;
  ">
    Select Traceability Matrix:
  </label>
  <select id="matrixSelector" onchange="toggleMatrix(this.value)" style="
    margin-left: 10px;
    padding: 6px;
    font-size: 18px;
    border-radius: 4px;
  ">
    <option value="all">All Functions</option>
    <option value="high">High Risk Functions</option>
    <option value="medium">Medium Risk Functions</option>
    <option value="low">Low Risk Functions</option>
    <option value="defunct">Defunct Functions</option>
    <option value="imported">Imported Functions</option>
    <option value="reexported">Re-exported Functions</option>
    <option value="experimental">Experimental Functions</option>
  </select>
</div>

<script>
  function toggleMatrix(value) {
    document.getElementById("trace_matrix_all").style.display = (value === "all") ? "block" : "none";
    document.getElementById("trace_matrix_high").style.display = (value === "high") ? "block" : "none";
    document.getElementById("trace_matrix_medium").style.display = (value === "medium") ? "block" : "none";
    document.getElementById("trace_matrix_low").style.display = (value === "low") ? "block" : "none";
    document.getElementById("trace_matrix_defunct").style.display = (value === "defunct") ? "block" : "none";
    document.getElementById("trace_matrix_imported").style.display = (value === "imported") ? "block" : "none";
    document.getElementById("trace_matrix_reexported").style.display = (value === "reexported") ? "block" : "none";
    document.getElementById("trace_matrix_experimental").style.display = (value === "experimental") ? "block" : "none";
  }

  document.addEventListener("DOMContentLoaded", function() {
    toggleMatrix("all");
  });
</script>
')
```



```{r trace_matrix_high_risk, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

high_trace_matrix_output <- get("high_risk_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
high_trace_matrix_output$Description <- sapply(high_trace_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
high_tm_datatable_html <- DT::datatable(high_trace_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(high_trace_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
high_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - High Risk", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  high_tm_datatable_html
)


# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_high",  
  high_tm_datatable_html
)

```

```{r trace_matrix_medium_risk, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

medium_trace_matrix_output <- get("medium_risk_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
medium_trace_matrix_output$Description <- sapply(medium_trace_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
medium_tm_datatable_html <- DT::datatable(medium_trace_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(medium_trace_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
medium_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - Medium Risk", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  medium_tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_medium",  
  medium_tm_datatable_html
)

```

```{r trace_matrix_low_risk, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

low_trace_matrix_output <- get("low_risk_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
low_trace_matrix_output$Description <- sapply(low_trace_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
low_tm_datatable_html <- DT::datatable(low_trace_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(low_trace_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
low_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - Low Risk", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  low_tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_low",  
  low_tm_datatable_html
)

```

```{r trace_matrix_defunct_func, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

defunct_func_matrix_output <- get("defunct_func_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
defunct_func_matrix_output$Description <- sapply(defunct_func_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
defunct_tm_datatable_html <- DT::datatable(defunct_func_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(defunct_func_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
defunct_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - Defunct Functions", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  defunct_tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_defunct",  
  defunct_tm_datatable_html
)
```

```{r trace_matrix_imported_func, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

imported_func_matrix_output <- get("imported_func_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
imported_func_matrix_output$Description <- sapply(imported_func_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
imported_tm_datatable_html <- DT::datatable(imported_func_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(imported_func_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
imported_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - Imported Functions", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  imported_tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_imported",  
  imported_tm_datatable_html
)
```

```{r trace_matrix_reexported_func, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

reexported_func_matrix_output <- get("reexported_func_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
reexported_func_matrix_output$Description <- sapply(reexported_func_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
reexported_tm_datatable_html <- DT::datatable(reexported_func_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(reexported_func_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
reexported_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - Re-exported Functions", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  reexported_tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_reexported",  
  reexported_tm_datatable_html
)
```

```{r trace_matrix_experimental_func, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

experimental_func_matrix_output <- get("experimental_func_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
experimental_func_matrix_output$Description <- sapply(experimental_func_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
experimental_tm_datatable_html <- DT::datatable(experimental_func_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(experimental_func_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
experimental_tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - Experimental Functions", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  experimental_tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_experimental",  
  experimental_tm_datatable_html
)
```


```{r trace_matrix_all, echo=FALSE}
library(knitr)
library(kableExtra)
library(DT)
library(htmlwidgets)
library(htmltools)
library(stringr)

trace_matrix_output <- get("trace_matrix_output", envir = report_env)

# Wrap the Description column at 90 characters
trace_matrix_output$Description <- sapply(trace_matrix_output$Description, function(x) {
  paste(stringr::str_wrap(x, width = 90), collapse = "\n")
})

brks <- c(60, 80)  # Breakpoints for 0-60, 61-80, and >80
clrs <- c('red', 'yellow', 'green')  # Colors for the ranges

# Create the datatable with additional options and custom CSS for styling
tm_datatable_html <- DT::datatable(trace_matrix_output, options = list(
  pageLength = 10,  # Number of rows to display per page
  lengthMenu = c(10, 20, 50),  # Options for number of rows per page
  search = list(search = ""),  # Enable search functionality
  dom = 'Bfrtip'  # Include buttons, filter, and pagination
)) %>%
  DT::formatStyle(
    columns = names(trace_matrix_output),
    backgroundColor = DT::styleInterval(brks, clrs),
    valueColumns = 'Test_Coverage'
  ) %>%
  DT::formatStyle(
    columns = 1:5,
    fontSize = '12px',
    fontWeight = 'bold'
  ) %>%
  DT::formatStyle(
    columns = 1,
    color = 'white',
    backgroundColor = 'purple'
  )

# header with purple background and text "Traceability Matrix"
tm_datatable_html <- htmltools::tagList(
  htmltools::tags$h2("Traceability Matrix - All functions", 
                     style = "background-color: purple; 
                     color: black; 
                     padding: 10px; 
                     text-align: center;"),
  tm_datatable_html
)

# Print the datatable inside a div for toggling
htmltools::tags$div(
  id = "trace_matrix_all",
  tm_datatable_html
)
```

---
title: "Create your weights profile"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Create your weights profile}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(risk.assessr)
```

## Create your own weights profile.

The `risk.assessr` package has a default weighting profile which assigns a weighting profile of 1 to each riskmetric
giving equal weighting to each risk metric.

If you wish to create your own weighting profile, 

1 - go to the R script 
`create_weights_profile()` in the `R` folder.

2 - Open the script and you will see the following code:

```
```{r, eval=FALSE}
library(risk.assessr)

create_weights_profile <- function() {
  weights <- c(
    has_bug_reports_url = .2,
    license = .1,
    has_examples = .4,
    has_maintainer = .2,
    has_news = .2,
    size_codebase = .2,
    has_source_control = .1,
    has_vignettes = .4,
    has_website = .4,
    news_current = .2,
    export_help = .2,
    export_calc = .2,
    check = .75,
    covr = .6,
    dep_score = .5,
    revdep_score = .1
  )
  return(weights)
}
```

3 - set your own weights for each risk metric

4 - save the file

5 - Go to the `Build` Tab

6 - Click the `More` Tab

7 - Click `Document`, `Clean and Install`, and `Load All` 

8 - Run the riskmetric functions `assess_pkg()` or `assess_pkg_r_package()` to
    calculate riskmetrics using your weights profile. 

---
title: "Using Risk Assessment Functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Risk Assessment Functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

This vignette demonstrates how to use the `risk_assess_pkg()` and `assess_pkg_r_package()` functions from the `risk.assessr` package to assess the risk of R packages.

You can assess:

- A local `.tar.gz` source package (by upload and using path)
- A package from CRAN or Bioconductor
- A package defined by an `renv.lock` file

## 1. Assessing a Local Source Package

To assess your own package:

### Step 1: Build your package
Use RStudio: `Build > More > Build Source Package` to generate a `.tar.gz` file.

### Step 2: Upload the package (optional)
If using a web interface or app built on top of the risk engine, upload the `.tar.gz` file through the UI.

### Step 3: Run the assessment

```
# Assess a local .tar.gz R package by tar file upload

# risk_result <- risk_assess_pkg()

# OR by providing a path

risk_result <- risk_assess_pkg(path/to/my/package)

```

## 2. Assessing via renv.lock

You can assess risks for all packages defined in an `renv.lock` or `pak.lock` file. This is helpful for auditing projects.

```
# Assess based on renv.lock
risk_result <- risk_assess_pkg("path/to/project/with/renv.lock") # or pak.lock
```

**Note:** This can be slow and is it better to run as a batch job or in CI (e.g., GitHub Actions).


## 3. Assessing a CRAN or Bioconductor Package

Use this method to check a remote package and version directly from public repositories.

# Assess the latest version from CRAN

```
risk_result <- assess_pkg_r_package("stringr")
```
# Or a specific version

```
risk_result <- assess_pkg_r_package("stringr", version = "1.5.0")
```

## Summary

These functions provide a consistent interface to assess risk for:

- Locally built packages
- renv or pak-based project dependencies
- Published open source packages from CRAN or Bioconductor

You can extend functionality by passing a custom `risk_config` to override default.

See more [Here](define_custom_risk_rules.html)

---
title: "Get available versions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get available versions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(risk.assessr)
```

## Get package version from CRAN

**risk.assessr** can fetch the version available for an R package along with the release date.

```{r check and fetch, eval=FALSE}
result_cran <- check_and_fetch_cran_package("admiral", "1.0.0")
```

```{r url, eval=FALSE}
result_cran$package_url
```

```{r last_version, eval=FALSE}
result_cran$last_version
```


```{r all_versions, eval=FALSE}
head(result_cran$all_versions, n = 2) 
```


```{r version_history, echo=FALSE, eval=FALSE}
# Example: flatten result$all_versions
version_history <- do.call(rbind, lapply(result_cran$all_versions, function(entry) {
  data.frame(
    Version = entry$version,
    Date = as.Date(entry$date),
    stringsAsFactors = FALSE
  )
}))

# Show nicely
knitr::kable(version_history, caption = "CRAN Version History")
```


## Get package version from Bioconductor

**risk.assessr** can fetch the version available for an R package stored on [bioconductor](https://www.bioconductor.org/) 


```{r bio_fetch, eval=FALSE}
html_content <- fetch_bioconductor_releases()
release_data <- parse_bioconductor_releases(html_content)
```


```{r bio_release, echo=FALSE, eval=FALSE}
# Example: flatten result$all_versions
release <- do.call(rbind, lapply(release_data, function(entry) {
  data.frame(
    Version = entry$release,
    Date = entry$date,
    software_packages = entry$software_packages,
    r_version = entry$r_version,
    stringsAsFactors = FALSE
  )
}))

# Show nicely
knitr::kable(release, caption = "release Bioconductor versions")
```

The function below will retrieve the versions of bioconductor package corresponding to bioconductor version. In the example below, we find the versions **2.20.0** of **flowCore** package for the bioconductor version **3.21**

Note: This function is not able to find all the archived versions for a Bioconductor package 

```{r flowCore, eval=FALSE}
fetch_bioconductor_package_info("3.21", "flowCore")
```

This function below gets the **flowCore** package version for all version of Bioconductor

```{r result_flowCore, eval=FALSE}
html_content <- fetch_bioconductor_releases()
release_data <- parse_bioconductor_releases(html_content)
result_bio <- get_bioconductor_package_url("flowCore", "2.18.0", release_data)
```

```{r bio_all_versions, eval=FALSE}
head(result_bio$all_versions, n=2)
```

```{r bio_last_version, eval=FALSE}
result_bio$last_version
```

```{r bio_package_versions, eval=FALSE}
result_bio$bioconductor_version_package
```


```{r bio_url, eval=FALSE}
result_bio$url
```


```{r bio_archived, eval=FALSE}
result_bio$archived
```

---
title: "Risk Assessment with Custom Configurations"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Risk Assessment with Custom Configurations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The `assess_pkg_r_package()` function in the `risk.assessr` package allows users to evaluate the risk of an R package. You can pass a custom risk configuration to control how risk levels are interpreted.

This vignette demonstrates:

- Running the function with the default risk configuration
- Running it with two different custom configurations

## Load the Package

```{r, message=FALSE, warning=FALSE, eval=FALSE}
library(risk.assessr)
options(repos = c(CRAN = "http://cran.us.r-project.org"))
```

## Example 1: Use Default Configuration

```{r, message=FALSE, warning=FALSE, eval=FALSE}
result_default <- risk.assessr::assess_pkg_r_package("stringr")
```

```{r, message=FALSE, eval=FALSE}
str(result_default$risk_analysis)
```

## Example 2: Use Custom Configuration (Strict Code Coverage)

```{r, message=FALSE, warning=FALSE, eval=FALSE}

strict_coverage_config <- list(
  list(
    label = "code coverage",
    id = "code_coverage",
    key = "code_coverage",
    thresholds = list(
      list(level = "high", max = 0.9999),
      list(level = "low", max = NULL)
    )
  ),
  list(
    label = "popularity",
    id = "popularity",
    key = "last_month_download",
    thresholds = list(
      list(level = "high", max = 21200000),          
      list(level = "medium", max = 11200000),      
      list(level = "low", max = NULL)       
    )
  )
)

# Set the option
options(risk.assessr.risk_definition = strict_coverage_config)
result_strict <- risk.assessr::assess_pkg_r_package("stringr")

```

```{r, message=FALSE, warning=FALSE, eval=FALSE}
str(result_strict$risk_analysis)
```

## Summary

The `risk_config` parameter allows you to tailor the risk scoring logic to your organization’s policies. You can use it to enforce stricter standards, accommodate internal tooling priorities, or meet compliance requirements.

---
title: "Traceability matrix"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Traceability matrix}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Setup environment

Load the necessary package and set CRAN repository.

```{r setup}
library(risk.assessr)
options(repos = "http://cran.us.r-project.org")
```

## Run traceability matrices and risk assessment

Analyze the **stringr** package to create a traceability matrix.

There are a number of ways to do this:

Firstly, if you just want a traceability matrix with no test coverage, you can run the following code:

```{r run_tm_no_test, message=FALSE, warning=FALSE, eval=FALSE}
stringr_tm_no_test <- risk.assessr::generate_traceability_matrix("stringr", version = "1.5.1" )
```

```{r tm_list_no_test, eval=FALSE}
str(stringr_tm_no_test$tm)
```

Secondly, if you want a traceability matrix with test coverage, you can run the following code:

```{r run_tm_test, message=FALSE, warning=FALSE, eval=FALSE}
stringr_tm_test <- risk.assessr::generate_traceability_matrix("stringr", version = "1.5.1", execute_coverage = TRUE)
```

```{r tm_list_test, eval=FALSE}
str(stringr_tm_test$tm)
```

If you want a full analysis of the package containing a traceability matrix and ither risk data, you can run the following:


```{r run_full_analysis, message=FALSE, warning=FALSE, eval=FALSE}
stringr <- risk.assessr::assess_pkg_r_package("stringr", version = "1.5.1" )
```
This returns a structured list with traceability and function risk information.

## Structure of traceability matrices

```{r tm_list, eval=FALSE}
str(stringr$tm_list)
```

## Get coverage summary by risk level

See how functions are mapped to risk and test coverage.
This shows all the exported functions.

```{r tm_list-tm, eval=FALSE}
stringr$tm_list$tm
```

This shows functions categorized as high, medium, or low risk test coverage.

```{r tm_list-coverage, eval=FALSE}
stringr$tm_list$coverage
```

### High risk test coverage

```{r high-risk, eval=FALSE}
stringr$tm_list$coverage$high_risk
```

### Medium risk test coverage

```{r medium-risk, eval=FALSE}
stringr$tm_list$coverage$medium_risk
```

### Low risk test coverage

```{r low-risk, eval=FALSE}
low_risk <- stringr$tm_list$coverage$low_risk
low_risk
```

## View function type summary

This shows functions categorized by higher risk function types.

### Defunct Functions

```{r defunct-functions, eval=FALSE}
stringr$tm_list$function_type
```

### Imported functions

```{r imported-summary, eval=FALSE}
stringr$tm_list$function_type$imported
```

### Re-exported functions

```{r rexported-summary, eval=FALSE}
stringr$tm_list$function_type$rexported
```

### Experimental functions

```{r experimental-summary, eval=FALSE}
stringr$tm_list$function_type$experimental
```

---
title: "Dependency tree"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Dependency tree}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r}
library(risk.assessr)


tryCatch({
  options(repos = c(CRAN = "http://cran.us.r-project.org"))
}, error = function(e) {
  message("No CRAN available")
})
```

## Dependencies Tree 

The `risk.assessr` package provides a set of functions to get dependencies and visualize dependency trees, allowing users to analyze package dependencies and identify potential conflicts. 

The package also supports license extraction and customizable depth control for exploring dependency layers.

### Basic Usage

Fetch your primary dependencies from your package:

```{r}
download_and_parse_dependencies("stringr")
```

Create and visualize your dependency tree:

```{r}
dep_data <- fetch_all_dependencies("stringr")
```

```{r}
dep_data
```

```{r}
print_tree(dep_data)
```

### License Extraction

You can extract license information from package DESCRIPTION files by setting `get_license = TRUE`. 

```{r}
dep_data_with_license <- fetch_all_dependencies("stringr", get_license = TRUE)
```

```{r}
print_tree(dep_data_with_license)
```

By default, the dependency tree explores up to 3 dependency levels deep. You can control this using the `max_level` parameter:

```{r}
# Shallow exploration (only 2 levels)
dep_data_shallow <- fetch_all_dependencies("stringr", get_license = TRUE, max_level = 2)
print_tree(dep_data_shallow)
```

```{r}
# Deeper exploration (5 levels)
dep_data_deep <- fetch_all_dependencies("stringr", get_license = TRUE, max_level = 5)
print_tree(dep_data_deep)
```

### Check for Conflicting Dependency Versions

Detect version conflicts in your dependency tree:

```{r}
detect_version_conflicts(dep_data)
```

---
title: "Generate html risk report"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Generate html risk report}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(risk.assessr)
```


```{r, eval=FALSE}
result <- risk.assessr::assess_pkg_r_package("ggplot2")
generate_html_report(result)
```

# Generate the risk report in a specific path

```{r, eval=FALSE}
generate_html_report(result, output_dir= system.file("examples/", package = "risk.assessr"))
```

# See the `ggplot2` package risk report in the web browser

## The report includes different types of traceability matrices: 

 - All Exported Functions
 - High Risk Functions
 - Medium Risk Functions
 - Low Risk Functions
 - Defunct Functions
 - Imported Functions
 - Re-exported Functions
 - Experimental Functions


```{r, echo=FALSE, eval=FALSE}
file_path <- system.file("examples", "risk_report_ggplot2_3.5.2.html", package = "risk.assessr")


if (interactive()) browseURL(file_path)
```

---
title: "Get package host"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get package host}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Overview

This vignette demonstrates how to use a set of R functions to programmatically retrieve the URL and code host package hosted on CRAN, Bioconductor and Github.

We will walk through each step, from checking if a package exists on CRAN to fetching its version history and constructing an appropriate download URL.

## CRAN

```{r setup}
library(DT)
library(risk.assessr)
```

### Check if the Package Exists on CRAN

```{r, eval=FALSE}
risk.assessr:::check_cran_package("here")
```

### Parse Package Archive HTML from CRAN database

fetch information from "https://cran.r-project.org/src/contrib/Archive/here/"

```{r, eval=FALSE}
html <- risk.assessr:::parse_package_info("here")
html
```

![](assets/Picture1.png)

### Extract Version Information from the Archive Page

Create table with `package_name`, `package_version`, `link`, `date`, `size` from CRAN website

```{r, echo=FALSE, eval=FALSE}
table <- risk.assessr:::parse_html_version(html, "here")
pkg_df <- do.call(rbind, lapply(table, as.data.frame))
datatable(pkg_df)
```


### Get All Versions and the Latest Version

```{r, eval=FALSE}
version_info <- risk.assessr:::get_versions(table, "here")
version_info$last_version
```

```{r, echo=FALSE, eval=FALSE}
pkg_df <- do.call(rbind, lapply(version_info$all_versions, as.data.frame))
datatable(pkg_df)
```

### Get CRAN package URL source code

```{r, eval=FALSE}
url <- risk.assessr:::get_cran_package_url(
  package_name = "here",
  version = NULL,
  last_version = version_info$last_version,
  all_versions = version_info$all_versions
)
url
```

## Internal R package

`risk.assessr` can also provide similar information from Internal mirror

```{r, eval=FALSE}
result_intern <- risk.assessr:::get_internal_package_url("herald")
```

```{r, eval=FALSE}
result_intern$url
```

```{r, eval=FALSE}
result_intern$last_version
```

```{r, echo=FALSE, eval=FALSE}
pkg_df <- do.call(rbind, lapply(result_intern$all_versions, as.data.frame))
datatable(pkg_df)
```

## Bioconductor package

Steps to get an R package stored on Bioconductor

```{r, message=FALSE, warning=FALSE, eval=FALSE}
html_content <- fetch_bioconductor_releases()
release_data <- parse_bioconductor_releases(html_content)
result_bio <- get_bioconductor_package_url("flowCore", "2.18.0", release_data)
```

```{r, eval=FALSE}
result_bio$url
```

## Github repository

R packages stored on Github are assess by looking at BugReports or URL in DESCRIPTION file to find a owner. github link are then created such as below and used to request Github API.

```{r, eval=FALSE}
urls <- c(
  "https://github.com/tidyverse/ggplot2"
)
bug_reports <- c(
  "https://github.com/tidyverse/ggplot2/issues"
)

all_links <- c(urls, bug_reports)

github_pattern <- "https://github.com/([^/]+)/([^/]+).*"
matching_links <- grep(github_pattern, all_links, value = TRUE)
owner_names <- sub(github_pattern, "\\1", matching_links)
package_names_github <- sub(github_pattern, "\\2", matching_links)

valid <- which(owner_names != "" & package_names_github != "")

  if (length(valid) > 0) {
    github_links <- unique(paste0("https://github.com/", owner_names[valid], "/", package_names_github[valid]))
  } else {
    github_links <- NULL
  }

github_links
```



## Get R package source code

```{r, eval=FALSE}
tarball_path <- get_package_tarfile("dplyr", version = "1.0.0")
```

---
title: "Get popularity metrics from CRAN or Bioconductor package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get popularity metrics from CRAN or Bioconductor package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

knitr::opts_chunk$set(fig.width = 12, fig.height = 5, out.width = "100%" ) 
```

```{r setup, include = FALSE}
library(risk.assessr)
library(ggplot2)
library(dplyr)

suppressMessages({
  library(kableExtra)
  library(knitr)
})

suppressWarnings({
  library(htmltools)
  library(htmlwidgets)
})
```

## CRAN download

```{r cran-data, eval=FALSE}
download_cran_df <- get_package_download_cran("admiral", years = 5)
head(download_cran_df)
```

```{r, echo=FALSE, cran-data-viz, eval=FALSE}
monthly_downloads <- download_cran_df %>%
  mutate(month = format(date, "%Y-%m")) %>%
  group_by(month) %>%
  summarise(
    downloads = sum(count),
    .groups = "drop"
  ) %>%
  mutate(month = as.Date(paste0(month, "-01")))

head(monthly_downloads)
```

```{r, eval=FALSE}
version_infos <- check_and_fetch_cran_package("admiral")
```

```{r, echo=FALSE, eval=FALSE}
version_dates <- bind_rows(lapply(version_infos$all_versions, as.data.frame)) %>%
  mutate(date = as.Date(date))

ggplot(monthly_downloads, aes(x = month, y = downloads)) +
  geom_line(color = "#2C3E50") +
  geom_point(color = "#E74C3C", size = 1) +
  geom_vline(
    data = version_dates,
    aes(xintercept = as.numeric(date)),
    color = "blue",
    linetype = "dashed",
    linewidth = 0.5
  ) +
  labs(
    title = "Monthly Downloads of 'admiral' Package",
    x = "Month",
    y = "Number of Downloads"
  ) +
  theme_minimal(base_size = 14) +
  theme(
    plot.title = element_text(face = "bold", hjust = 0.5, margin = margin(b = 10, t = 20)),
    axis.text.x = element_text(angle = 45, hjust = 1),
    plot.margin = margin(t = 30, r = 20, b = 20, l = 20)
  )
```

## GitHub commit activity

```{r github-meta, eval=FALSE}
repo_info <- get_github_data("tidyverse", "ggplot2")
repo_info
```

```{r github-data, eval=FALSE}
df_commit <- get_commits_since("tidyverse", "ggplot2", years = 1)
head(df_commit)
```

```{r, echo=FALSE, github-data-viz, eval=FALSE}
df_commit <- df_commit %>%
  arrange(week_start) %>%
  mutate(cumulative_commits = cumsum(n_commits))

ggplot(df_commit, aes(x = week_start)) +
  geom_col(aes(y = n_commits), fill = "#1ABC9C") +
  geom_line(aes(y = cumulative_commits), color = "#34495E", linewidth = 1, linetype = "dashed") +
  labs(
    title = "Weekly Commits on GitHub (ggplot2)",
    x = "Week",
    y = "Number of Commits"
  ) +
  theme_minimal(base_size = 14) +
  theme(
    plot.title = element_text(face = "bold", hjust = 0.5),
    axis.text.x = element_text(angle = 45, hjust = 1)
  )
```

## Bioconductor download

```{r}
download_bio_df <- get_package_download_bioconductor("limma")
head(download_bio_df$all_data)
```

```{r, echo=FALSE, message=FALSE, eval=FALSE}
html_content <- fetch_bioconductor_releases()
release_data <- parse_bioconductor_releases(html_content)
info <- get_bioconductor_package_url("limma", release_data = release_data)
```

```{r bioconductor-data, eval=FALSE}
download_bio_df <- get_package_download_bioconductor("DESeq2")
head(download_bio_df$all_data)
```

```{r, echo=FALSE, bioconductor-data-viz, eval=FALSE}
ggplot(download_bio_df$all_data, aes(x = Date, y = Nb_of_downloads)) +
  geom_line(color = "#9B59B6") +
  geom_point(color = "#2980B9") +
  labs(
    title = "Bioconductor Downloads",
    x = "Month",
    y = "Number of Downloads"
  ) +
  theme_minimal(base_size = 14) +
  theme(
    plot.title = element_text(
      face = "bold",
      hjust = 0.5,
      margin = margin(b = 10, t = 20)  
    ),
    plot.margin = margin(t = 30, r = 20, b = 20, l = 20), 
    axis.text.x = element_text(angle = 45, hjust = 1)
  )
```

---
title: "Security vulnerabilities"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Security vulnerabilities}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The security vulnerabilities feature checks for known, publicly disclosed
vulnerabilities in the R package being assessed. It is accessed through:

- `get_security_vulnerabilities()` — the exported function that retrieves
  advisories for a package.

It also runs automatically as part of a full assessment via
`risk_assess_pkg()` / `assess_pkg()`.

---

## Where the data comes from

Advisories are retrieved from the **Open Source Vulnerabilities (OSV)**
database (<https://osv.dev>) using a `POST` request to the query endpoint
`https://api.osv.dev/v1/query`. For R packages, OSV serves advisories from the
[R Consortium r-advisory-database](https://github.com/RConsortium/r-advisory-database),
which carry `RSEC-*` identifiers.

The package is queried by name only within the `"CRAN"` ecosystem. All
advisories for the package are fetched, then filtered locally against the
installed version. Querying by name (rather than name + version) avoids missing
range-based advisories that a version-scoped OSV query can silently omit.

---

## Load the package

```{r setup}
library(risk.assessr)
```

---

## Retrieving vulnerabilities directly

```{r, message=FALSE, warning=FALSE, eval=FALSE}
vulns <- get_security_vulnerabilities("commonmark", "1.7")
vulns
```

The function returns a data frame with one row per advisory and the following
columns:

- `id` — advisory identifier (e.g. `RSEC-2023-6`)
- `summary` — short vulnerability type
- `details` — full description
- `introduced` — version the issue was introduced
- `fixed` — version the issue was fixed
- `modified`, `published` — advisory timestamps

If no advisories apply, a zero-row data frame with the same columns is returned.
Omitting the version returns every advisory known for the package.

---

## Within a full assessment

When you assess a package, the vulnerabilities are stored in the results object:

```{r, message=FALSE, warning=FALSE, eval=FALSE}
result <- risk_assess_pkg(package = "commonmark", version = "1.7")
result$results$vulnerabilities
```

---

## How results are reported

**Summary report** — vulnerabilities appear as a single metric,
"Security Vulnerabilities", showing the count of advisories found. Any count of
one or more is scored **High** risk; zero is scored **Low**. This contributes to
the overall recommendation alongside the other risk metrics.

**HTML report** — when advisories are found, the report renders a red-headed,
searchable table listing every advisory (`ID`, `Summary`, `Details`,
`Introduced`, `Fixed`, `Modified`, `Published`). When none are found, a green
"No security vulnerabilities" banner is shown instead.

---

## Limitations

- Only the `"CRAN"` ecosystem is queried; Bioconductor is not auto-detected.
- The feature requires network access to the OSV API.
- API failures return an empty result, so absence of rows is not by itself a
  guarantee that a package is free of vulnerabilities — check the console
  messages.
- Vulnerability scoring is not configurable through 
  `risk-definition.json` at the moment.

---
title: Summary
params:
  name: test
  version: 1
  analysis: 1
  tool_version: 1
  data: ""
---

```{r, include = FALSE, warning=FALSE}
library(dplyr)
library(kableExtra)
library(knitr)
library(htmltools)
library(htmlwidgets)
library(risk.assessr)
```


## Info

```{r, echo=FALSE}
df <- data.frame(
  v = c("Name", "Version", "Date", "Risk.assessr version"),
  k = c(params$name, params$version, format(Sys.Date(), "%d-%b-%Y", tz = "CET"), params$tool_version)
)
names(df) <- NULL
knitr::kable(df)
```


## Results

```{r echo=FALSE}
# Define the desired order of metrics
ordered_metrics <- c(
  "CMD Check",
  "Code Coverage",
  "Dependencies Count",
  "License Risk",
  "URL Documentation Score",
  "Percentage of functions with examples",
  "Percentage of functions with documentation",
  "Combined Examples Documentation Score",
  "Reverse Dependencies Count",
  "Later Version Available",
  "Total Downloads",
  "Bug Reports URL",
  "Source Control",
  "Security Vulnerabilities"
)

df <- params$analysis[match(ordered_metrics, params$analysis$Metric), , drop = FALSE]

# Apply conditional formatting for each row based on Risk Level
for (i in seq_len(nrow(df))) {
  risk_level <- tolower(df$Risk_Level[i])
  cell_color <- switch(risk_level,
    low = "green",
    medium = "yellow",
    high = "red"
  )
  df[i, 2] <- kableExtra::cell_spec(df[i, 2], background = cell_color)
}

knitr::kable(df, col.names = c("Metric", "Risk Level", "Risk Value"))
```

## Recommendation

`r evaluate_risk_highlighting(params$analysis)`

```{r approver, echo=FALSE}

approvers_df <- data.frame(
  Approvers = "",
  stringsAsFactors = FALSE
)

# Render as a pandoc table (portable across md/docx/pdf/html)
knitr::kable(
  approvers_df,
  format = "pandoc",
  col.names = c("Approvers"),
  align = "l")

```

```{r signature_table, echo=FALSE}
signature_df <- data.frame(
  `Electronic Signature` = "",
  `Date` = "",
  stringsAsFactors = FALSE
)

knitr::kable(
  signature_df,
  format = "pandoc",
  col.names = c("Electronic Signature", "Date"),
  align = c("l", "l")
)
```

## Raw data

```{r results='asis', echo=FALSE}
json <- jsonlite::toJSON(params$data, pretty = TRUE, force = TRUE)

cat('<details><summary><strong>Show / Hide raw data</strong></summary>\n\n')
cat("```json\n", json, "\n```\n")
```

---
title: "Get package host"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get package host}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Overview

This vignette demonstrates how to use a set of R functions to programmatically retrieve the URL and code host package hosted on CRAN, Bioconductor and Github.

We will walk through each step, from checking if a package exists on CRAN to fetching its version history and constructing an appropriate download URL.

## CRAN

```{r setup}
library(DT)
library(risk.assessr)
```

### Check if the Package Exists on CRAN

```{r, eval=FALSE}
risk.assessr:::check_cran_package("here")
```

### Parse Package Archive HTML from CRAN database

fetch information from "https://cran.r-project.org/src/contrib/Archive/here/"

```{r, eval=FALSE}
html <- risk.assessr:::parse_package_info("here")
html
```

![](assets/Picture1.png)

### Extract Version Information from the Archive Page

Create table with `package_name`, `package_version`, `link`, `date`, `size` from CRAN website

```{r, echo=FALSE, eval=FALSE}
table <- risk.assessr:::parse_html_version(html, "here")
pkg_df <- do.call(rbind, lapply(table, as.data.frame))
datatable(pkg_df)
```


### Get All Versions and the Latest Version

```{r, eval=FALSE}
version_info <- risk.assessr:::get_versions(table, "here")
version_info$last_version
```

```{r, echo=FALSE, eval=FALSE}
pkg_df <- do.call(rbind, lapply(version_info$all_versions, as.data.frame))
datatable(pkg_df)
```

### Get CRAN package URL source code

```{r, eval=FALSE}
url <- risk.assessr:::get_cran_package_url(
  package_name = "here",
  version = NULL,
  last_version = version_info$last_version,
  all_versions = version_info$all_versions
)
url
```

## Internal R package

`risk.assessr` can also provide similar information from Internal mirror

```{r, eval=FALSE}
result_intern <- risk.assessr:::get_internal_package_url("herald")
```

```{r, eval=FALSE}
result_intern$url
```

```{r, eval=FALSE}
result_intern$last_version
```

```{r, echo=FALSE, eval=FALSE}
pkg_df <- do.call(rbind, lapply(result_intern$all_versions, as.data.frame))
datatable(pkg_df)
```

## Bioconductor package

Steps to get an R package stored on Bioconductor

```{r, message=FALSE, warning=FALSE, eval=FALSE}
html_content <- fetch_bioconductor_releases()
release_data <- parse_bioconductor_releases(html_content)
result_bio <- get_bioconductor_package_url("flowCore", "2.18.0", release_data)
```

```{r, eval=FALSE}
result_bio$url
```

## Github repository

R packages stored on Github are assess by looking at BugReports or URL in DESCRIPTION file to find a owner. github link are then created such as below and used to request Github API.

```{r, eval=FALSE}
urls <- c(
  "https://github.com/tidyverse/ggplot2"
)
bug_reports <- c(
  "https://github.com/tidyverse/ggplot2/issues"
)

all_links <- c(urls, bug_reports)

github_pattern <- "https://github.com/([^/]+)/([^/]+).*"
matching_links <- grep(github_pattern, all_links, value = TRUE)
owner_names <- sub(github_pattern, "\\1", matching_links)
package_names_github <- sub(github_pattern, "\\2", matching_links)

valid <- which(owner_names != "" & package_names_github != "")

  if (length(valid) > 0) {
    github_links <- unique(paste0("https://github.com/", owner_names[valid], "/", package_names_github[valid]))
  } else {
    github_links <- NULL
  }

github_links
```



## Get R package source code

```{r}
tarball_path <- get_package_tarfile("dplyr", version = "1.0.0")
```

# here package

    Code
      print_tree(here, show_version = TRUE)
    Output
      └── rprojroot (v1.0.1)

# here package - no versions

    Code
      print_tree(here, show_version = FALSE)
    Output
      └── rprojroot

# stringr package

    Code
      print_tree(stringr, show_version = TRUE)
    Output
      ├── cli (v1.5.1)
      │   └── utils (base)
      ├── glue (v1.7.0)
      │   └── methods (base)
      ├── lifecycle (v1.0.4)
      │   ├── cli (v3.6.2)
      │   │   └── utils (base)
      │   ├── glue (v1.7.0)
      │   │   └── methods (base)
      │   └── rlang (v1.1.3)
      │       └── utils (base)
      ├── magrittr (v2.0.3)
      ├── rlang (v1.1.3)
      │   └── utils (base)
      ├── stringi (v1.8.3)
      │   ├── tools (base)
      │   ├── utils (base)
      │   └── stats (base)
      └── vctrs (v0.6.5)
          ├── cli (v3.6.2)
          │   └── utils (base)
          ├── glue (v1.7.0)
          │   └── methods (base)
          ├── lifecycle (v1.0.4)
          └── rlang (v1.1.3)
              └── utils (base)

# stringr package - no versions

    Code
      print_tree(stringr, show_version = FALSE)
    Output
      ├── cli
      │   └── utils (base)
      ├── glue
      │   └── methods (base)
      ├── lifecycle
      │   ├── cli
      │   │   └── utils (base)
      │   ├── glue
      │   │   └── methods (base)
      │   └── rlang
      │       └── utils (base)
      ├── magrittr
      ├── rlang
      │   └── utils (base)
      ├── stringi
      │   ├── tools (base)
      │   ├── utils (base)
      │   └── stats (base)
      └── vctrs
          ├── cli
          │   └── utils (base)
          ├── glue
          │   └── methods (base)
          ├── lifecycle
          └── rlang
              └── utils (base)

# print_tree with licenses matches snapshot

    Code
      print_tree(test_tree, show_version = TRUE)
    Output
      └── stringr (v1.5.1) MIT + file LICENSE license
          └── cli (v3.6.2) MIT + file LICENSE
              └── utils (base)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{parse_package_info}
\alias{parse_package_info}
\title{Parse Package Information from CRAN Archive}
\usage{
parse_package_info(name)
}
\arguments{
\item{name}{A character string specifying the name of the package to fetch information for.}
}
\value{
A character string containing the raw HTML content of the package archive page, or `NULL`
if the request fails or the package is not found.
}
\description{
This function retrieves the package archive information from the CRAN Archive.
}
\examples{
\dontrun{
# Fetch package archive information for "dplyr"
parse_package_info("dplyr")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{rd_flatten_text}
\alias{rd_flatten_text}
\title{Flatten an Rd node to plain text}
\usage{
rd_flatten_text(node)
}
\arguments{
\item{node}{An Rd node or list of nodes.}
}
\value{
Character scalar containing the concatenated text.
}
\description{
Recursively flattens a node (or list of nodes) into a character string.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{function_is_ggproto}
\alias{function_is_ggproto}
\title{function to check value of ggproto}
\usage{
function_is_ggproto(x)
}
\arguments{
\item{x}{- ggproto functions from the target package}
}
\description{
function to check if first argument inherits from any of the classes specified
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_pkg.R
\name{assess_pkg}
\alias{assess_pkg}
\title{Assess package}
\usage{
assess_pkg(pkg_source_path, rcmdcheck_args, covr_timeout = Inf)
}
\arguments{
\item{pkg_source_path}{- source path for install local}

\item{rcmdcheck_args}{- arguments for R Cmd check - these come from setup_rcmdcheck_args}

\item{covr_timeout}{- setting for covr time out}
}
\value{
list containing results - list containing metrics, covr, tm - trace matrix, and R CMD check
}
\description{
assess package for risk metrics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_metrics.R
\name{run_coverage}
\alias{run_coverage}
\title{Run covr and potentially save results to disk}
\usage{
run_coverage(pkg_source_path, timeout = Inf)
}
\arguments{
\item{pkg_source_path}{package installation directory}

\item{timeout}{Timeout to pass to [callr::r_safe()] when running covr.}
}
\value{
list with total coverage and function coverage
}
\description{
Run covr and potentially save results to disk
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{fetch_all_dependencies}
\alias{fetch_all_dependencies}
\title{Fetch All Dependencies for a Package}
\usage{
fetch_all_dependencies(package_name, get_license = FALSE, max_level = 3)
}
\arguments{
\item{package_name}{A character string specifying the name of the package.}

\item{get_license}{A logical value indicating whether to extract and include license information from DESCRIPTION files (default: `FALSE`).}

\item{max_level}{An integer specifying the maximum depth level for dependency recursion (default: 3).
Set to a higher value to explore deeper dependency layers.}
}
\value{
A nested list representing the dependency tree of the package. If `get_license` is `TRUE`, 
        each package in the tree (including the root package and all dependencies) will have a 
        `license` field containing that package's license extracted from its DESCRIPTION file.
}
\description{
This function builds and retrieves the full dependency tree for a given R package.
}
\examples{
\dontrun{
fetch_all_dependencies("ggplot2")
fetch_all_dependencies("ggplot2", get_license = TRUE)
fetch_all_dependencies("ggplot2", max_level = 5)  # Deeper dependency exploration
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_tests_stf.R
\name{get_expect}
\alias{get_expect}
\title{Extract expectation calls from parsed test code}
\usage{
get_expect(x, token_text = "^expect_")
}
\arguments{
\item{x}{data.frame, parse data from utils::getParseData}

\item{token_text}{character, regex pattern to match expectation function names, Default: '^expect_'}
}
\value{
data.frame with expectation names and their line ranges
}
\description{
Internal helper function to identify and extract expectation function calls (e.g., expect_equal) from parsed testthat code.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_traceability_matrix.R
\name{map_functions_to_docs}
\alias{map_functions_to_docs}
\title{Map all Rd files to the functions they describe}
\usage{
map_functions_to_docs(exports_df, pkg_source_path, verbose)
}
\value{
Returns the data.frame passed to `exports_df`, with a `documentation`
  column appended. This column will contain the path to the `.Rd` files in
  `man/` that document the associated exported functions.
}
\description{
adapted from mpn.scorecard
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_modes.R
\name{run_covr_modes}
\alias{run_covr_modes}
\title{Run Coverage Analysis with Test Detection}
\usage{
run_covr_modes(pkg_source_path, covr_timeout = 60)
}
\arguments{
\item{pkg_source_path}{Character. Path to the root directory of the R package source.}

\item{covr_timeout}{Numeric. Timeout in seconds for running coverage analysis. Default is 60.}
}
\value{
A list with two elements:
\describe{
  \item{covr_list}{A list containing coverage results. If no supported tests are found, returns a default structure with zero coverage.}
  \item{test_pkg_data}{A list summarizing the test configuration, including presence of `testthat`, `testit`, snapshots, and base R test scripts.}
}
}
\description{
This function checks the test configuration of an R package and runs code coverage analysis
using `testthat`, `testit` or `test base` frameworks if available. If no supported test framework is found,
it returns a default coverage result indicating no functions were tested.
}
\examples{
\dontrun{
r = getOption("repos")
r["CRAN"] = "http://cran.us.r-project.org"
old <- options(repos = r)

dp <- system.file("test-data", 
  "test.package.0001_0.1.0.tar.gz", 
  package = "risk.assessr")

# set up package
install_list <- set_up_pkg(dp)

build_vignettes <- install_list$build_vignettes
package_installed <- install_list$package_installed
pkg_source_path <- install_list$pkg_source_path
rcmdcheck_args <- install_list$rcmdcheck_args

# install package locally to ensure test works
package_installed <- install_package_local(pkg_source_path)
package_installed <- TRUE

rcmdcheck_args$path <- pkg_source_path
covr_mode_list <- run_covr_modes(pkg_source_path)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_metrics.R
\name{run_rcmdcheck}
\alias{run_rcmdcheck}
\title{Run R CMD CHECK}
\usage{
run_rcmdcheck(pkg_source_path, rcmdcheck_args)
}
\arguments{
\item{pkg_source_path}{directory path to R project}

\item{rcmdcheck_args}{list of arguments to pass to `rcmdcheck::rcmdcheck`}
}
\description{
Run R CMD CHECK
}
\details{
rcmdcheck takes either a tarball or an installation directory.

The basename of `pkg_source_path` should be the package name and version pasted together

The returned score is calculated via a weighted sum of notes (0.10), warnings (0.25), and errors (1.0). 
It has a maximum score of 1 (indicating no errors, notes or warnings)
and a minimum score of 0 (equal to 1 error, 4 warnings, or 10 notes). 
This scoring methodology is expanded from [riskmetric::metric_score.pkg_metric_r_cmd_check()] by including
forbidden notes,
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{get_internal_package_url}
\alias{get_internal_package_url}
\title{Get Internal Package URL}
\usage{
get_internal_package_url(package_name, version = NULL, base_url = NULL)
}
\arguments{
\item{package_name}{A character string specifying the name of the package.}

\item{version}{An optional character string specifying the version of the package.
Defaults to `NULL`, in which case the latest version will be used.}

\item{base_url}{A character string specifying the base URL of the internal package manager.}
}
\value{
A list containing:
  - `url`: A character string of the package URL (or `NULL` if not found).
  - `last_version`: A list with `version` and `date` of the latest version (or `NULL`).
  - `version`: The version used to generate the URL (or `NULL`).
  - `all_versions`: A list of all available versions, each as a list with `version` and `date`.
}
\description{
This function retrieves the URL of an internal package on Sanofi Mirror, its latest version, 
and a list of all available versions.
}
\examples{
\dontrun{
result <- get_internal_package_url("internalpackage", version = "1.0.1")
print(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_empty_tm.R
\name{create_empty_tm}
\alias{create_empty_tm}
\title{create empty tm}
\usage{
create_empty_tm(pkg_name)
}
\arguments{
\item{pkg_name}{- name of package}
}
\value{
empty_tm - list with no tm
}
\description{
create empty tm
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{rd_name}
\alias{rd_name}
\title{Extract the topic name from an Rd document}
\usage{
rd_name(rd)
}
\arguments{
\item{rd}{An Rd object.}
}
\value{
A length-1 character scalar (topic name) or `NA_character_` if missing.
}
\description{
Returns the `\\name{}` (roxygen `@rdname`) of the Rd document.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_risk_analysis.R
\name{get_risk_definition}
\alias{get_risk_definition}
\title{Get Risk Definition}
\usage{
get_risk_definition()
}
\value{
A list of risk configuration items.
}
\description{
Load risk definition config from options, JSON file, or fallback to bundled file.
}
\examples{
risk_rule <- get_risk_definition()
print(risk_rule)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_empty_results.R
\name{create_empty_results}
\alias{create_empty_results}
\title{Create empty results}
\usage{
create_empty_results(pkg_name, pkg_ver, pkg_source_path, metadata)
}
\arguments{
\item{pkg_name}{package name}

\item{pkg_ver}{package version}

\item{pkg_source_path}{package source path}

\item{metadata}{list with metadata}
}
\value{
- list with empty risk profile values
}
\description{
This creates an empty results list
for all risk metrics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/risk_assess_pkg.R
\name{risk_assess_pkg}
\alias{risk_assess_pkg}
\title{Assess package for risk metrics}
\usage{
risk_assess_pkg(
  path = NULL,
  package = NULL,
  version = NA,
  repos = getOption("repos")
)
}
\arguments{
\item{path}{Path to a local .tar.gz package archive. Use only when not
providing `package`.}

\item{package}{Package name to fetch from CRAN, Bioconductor, or internal
repos. Use only when not providing `path`.}

\item{version}{(optional) Package version when using `package`. Default `NA`
uses the latest available.}

\item{repos}{Repository URLs when fetching by `package`. Default is
`getOption("repos")`.}
}
\value{
List containing results (metrics, covr, trace matrix, R CMD check),
  or `NULL` if the package cannot be assessed.
}
\description{
Assess an R package for risk metrics. Use exactly one of:
* `path` – path to a local .tar.gz archive, or
* `package` – package name to fetch (optionally with `version`), or
* neither – opens an interactive file chooser.
}
\examples{
\dontrun{
# Local package (path or file chooser)
results <- risk_assess_pkg()
results <- risk_assess_pkg(path = "path/to/package.tar.gz")

# Package by name from repositories
results <- risk_assess_pkg(package = "here")
results <- risk_assess_pkg(package = "here", version = "1.0.1")
results <- risk_assess_pkg(package = "here", repos = "https://cloud.r-project.org")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_up_pkg.R
\name{set_up_pkg}
\alias{set_up_pkg}
\title{Creates information on package installation}
\usage{
set_up_pkg(dp, check_type = "1")
}
\arguments{
\item{dp}{data path and name for the package.}

\item{check_type}{basic R CMD check type - "1" CRAN R CMD check_type - "2"}
}
\value{
list with local package install
}
\description{
Creates information on package installation
}
\examples{
\dontrun{
set_up_pkg(path/to/package, "mypackage")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{cran_revdep}
\alias{cran_revdep}
\title{Find Reverse Dependencies of a CRAN Package}
\usage{
cran_revdep(
  pkg,
  dependencies = c("Depends", "Imports", "Suggests", "LinkingTo"),
  recursive = FALSE,
  ignore = NULL,
  installed = NULL
)
}
\arguments{
\item{pkg}{A character string representing the name of the package.}

\item{dependencies}{A character vector specifying the types of dependencies to consider. Default is c("Depends", "Imports", "Suggests", "LinkingTo").}

\item{recursive}{A logical value indicating whether to find dependencies recursively. Default is FALSE.}

\item{ignore}{A character vector of package names to ignore. Default is NULL.}

\item{installed}{A matrix of installed packages.}
}
\value{
deps - A sorted character vector of package names that depend on the specified package.
}
\description{
This function finds the reverse dependencies of a CRAN package, i.e., packages that depend on the specified package.
}
\examples{
\dontrun{
cran_revdep("ggplot2")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{doc_riskmetric}
\alias{doc_riskmetric}
\title{Run all relevant documentation riskmetric checks}
\usage{
doc_riskmetric(pkg_name, pkg_ver, pkg_source_path)
}
\arguments{
\item{pkg_name}{name of the package}

\item{pkg_ver}{version of the package}

\item{pkg_source_path}{path to package source code (untarred)}
}
\value{
raw riskmetric outputs
}
\description{
Run all relevant documentation riskmetric checks
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R
\name{extract_version}
\alias{extract_version}
\title{Extract Package Version from File Path}
\usage{
extract_version(path, package_name)
}
\arguments{
\item{path}{A character string specifying the file path or URL.}

\item{package_name}{A character string specifying the name of the package.}
}
\value{
A character string representing the extracted version number, or `NULL` if no match is found.
}
\description{
This function extracts the version number from a package source file name
based on the package name and expected file pattern.
}
\examples{
\dontrun{
link <- "https://bioconductor.org/packages/3.14/bioc/src/contrib/GenomicRanges_1.42.0.tar.gz"
extract_version(link, "GenomicRanges")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{classify_function_body}
\alias{classify_function_body}
\title{classify_function_body}
\usage{
classify_function_body(row, package_name)
}
\arguments{
\item{row}{- row with exported functions}

\item{package_name}{- name of the package}
}
\value{
row - row with the function details plus function body
}
\description{
This function gets the function bodies for exported functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{extract_examples_text}
\alias{extract_examples_text}
\title{Extract examples text from an Rd document}
\usage{
extract_examples_text(rd_doc)
}
\arguments{
\item{rd_doc}{An Rd document.}
}
\value{
Character scalar of combined examples text, or `NULL` if none/non‑informative.
}
\description{
Supports both `\\examples{}` and `\\example{}` sections. If multiple
sections exist, their contents are concatenated with blank lines.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/risk_assess_pkg_lock_files.R
\name{risk_assess_pkg_lock_files}
\alias{risk_assess_pkg_lock_files}
\title{Process lock files}
\usage{
risk_assess_pkg_lock_files(input_data)
}
\arguments{
\item{input_data}{- path to a lock file}
}
\value{
assessment_results - nested list containing risk metric data
}
\description{
This function processes `renv.lock` and `pak.lock` files 
to produce risk metric data
}
\examples{
\dontrun{
  input_data <- ("path/to/mypak.lock")
  pak_results <- risk_assess_pkg_lock_files(input_data) 
  print(pak_results)
 } 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{generate_fg_trace_matrix_section}
\alias{generate_fg_trace_matrix_section}
\title{Generate Fine grained Trace Matrices Section}
\usage{
generate_fg_trace_matrix_section(tm_type, tm_df)
}
\arguments{
\item{tm_type}{- type of traceability matrix}

\item{tm_df}{- input data}
}
\description{
Generates the Trace Matrices section for the HTML report.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{parse_dcf_dependencies}
\alias{parse_dcf_dependencies}
\title{Parse DCF of description file}
\usage{
parse_dcf_dependencies(path)
}
\arguments{
\item{path}{pkg_ref path}
}
\description{
Parse DCF of description file
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_exports}
\alias{assess_exports}
\title{Assess exported functions to namespace}
\usage{
assess_exports(data)
}
\arguments{
\item{data}{pkg source path}
}
\description{
Assess exported functions to namespace
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_tests_stf.R
\name{map_tests_stf}
\alias{map_tests_stf}
\title{Map the hierarchy structure of tests directory in standard testing framework}
\usage{
map_tests_stf(path = "tests/testthat")
}
\arguments{
\item{path}{character, path to tests, Default: 'tests/testthat'}
}
\value{
data.frame
}
\description{
Query a test directory for the unit test structure.
}
\details{
Return data.frame containing which expectations are in the testthat directory
by file/context/description/test/expectation/linerange
}
\seealso{
\code{\link[base]{names}}, \code{\link[base]{lapply}}, \code{\link[base]{rbind}}

Other utility: 
\code{\link{create_coverage_skip_stf}()},
\code{\link{map_test}()}
}
\concept{utility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{generate_html_report}
\alias{generate_html_report}
\title{Generate HTML Report for Package Assessment}
\usage{
generate_html_report(assessment_results, output_dir = NULL)
}
\arguments{
\item{assessment_results}{List containing the results from risk_assess_pkg function.}

\item{output_dir}{Character string indicating the directory where the report will be saved.}
}
\value{
Path to the generated HTML report.
}
\description{
Generates an HTML report for the package assessment results using rmarkdown.
}
\examples{
\dontrun{
assessment_results <- risk_assess_pkg()
generate_html_report(assessment_results, output_dir = "path/to/save/report")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{is_base}
\alias{is_base}
\title{Check if a Package is a Base or Recommended R Package}
\usage{
is_base(pkg)
}
\arguments{
\item{pkg}{A character string representing the name of the package.}
}
\value{
A logical value: `TRUE` if the package is a base or recommended package, `FALSE` otherwise.
}
\description{
This function determines whether a given package is a base or recommended package
using CRAN metadata.
}
\examples{
\dontrun{
is_base("stats")     # TRUE
is_base("ggplot2")   # FALSE
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_stf.R
\name{create_coverage_skip_stf}
\alias{create_coverage_skip_stf}
\title{Run covr with failing tests}
\usage{
create_coverage_skip_stf(test_path = "tests/testthat", test_skip, test_map)
}
\arguments{
\item{test_path}{character, path to test directory, Default: 'tests/testthat'}

\item{test_skip}{data.frame, mapping of failing tests, PARAM_DESCRIPTION,}

\item{test_map}{data.frame, mapping of all the tests in the package}
}
\description{
Run \code{\link[covr]{package_coverage}} ignoring failing tests from \code{\link[testthat]{test_dir}}.
}
\details{
If test_skip is NULL then a new test_dir will be run internally checking
for failing tests.
}
\seealso{
\code{\link[testthat]{test_dir}}
 \code{\link[covr]{package_coverage}}

Other utility: 
\code{\link{map_test}()},
\code{\link{map_tests_stf}()}
}
\concept{utility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_suggested_exp_funcs.R
\name{check_suggested_exp_funcs}
\alias{check_suggested_exp_funcs}
\title{Function to check suggested exported functions}
\usage{
check_suggested_exp_funcs(pkg_name, pkg_source_path, suggested_deps)
}
\arguments{
\item{pkg_name}{- name of the target package}

\item{pkg_source_path}{- source of the target package}

\item{suggested_deps}{- dependencies in Suggests}
}
\value{
- data frame with results of Suggests check
}
\description{
This function checks the exported functions of target package
against the exported functions of Suggested dependencies to see if any 
Suggested packages should be in Imports in the DESCRIPTION file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_traceability_matrix.R
\name{map_functions_to_scripts}
\alias{map_functions_to_scripts}
\title{Get all exported functions and map them to R script where they are defined}
\usage{
map_functions_to_scripts(exports_df, pkg_source_path, verbose)
}
\arguments{
\item{exports_df}{data.frame with a column, named `exported_function`,
containing the names of all exported functions. Can also have other columns
(which will be returned unmodified).}

\item{pkg_source_path}{a file path pointing to an unpacked/untarred package directory}

\item{verbose}{check for extra information}
}
\value{
A data.frame with the columns `exported_function` and `code_script`.
}
\description{
adapted from mpn.scorecard
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_suggested_exp_funcs.R
\name{create_items_matched}
\alias{create_items_matched}
\title{Create items matched}
\usage{
create_items_matched(extracted_functions, suggested_exp_func)
}
\arguments{
\item{extracted_functions}{- exported functions from the target package}

\item{suggested_exp_func}{- exported functions from a suggested package}
}
\description{
create a df with functions that match exported functions from
a suggested package
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_tests_stf.R
\name{nest_test}
\alias{nest_test}
\title{Nest parsed testthat code into logical test blocks}
\usage{
nest_test(x, token_text = "^context$")
}
\arguments{
\item{x}{data.frame, parse data from utils::getParseData}

\item{token_text}{character, regex pattern to match test block function names, Default: '^context$'}
}
\value{
named list of data.frames, each representing a nested test block
}
\description{
Internal helper function to group parsed testthat code into blocks based on context, test_that, describe, or it calls.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{create_file_coverage_df}
\alias{create_file_coverage_df}
\title{Generate file coverage df}
\usage{
create_file_coverage_df(file_names, file_coverage, errors, notes)
}
\arguments{
\item{file_names}{- file names}

\item{file_coverage}{- test coverage for files}

\item{errors}{- test coverage errors}

\item{notes}{- test coverage notes}
}
\description{
Generates file coverage df when errors list created.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_github_data.R
\name{average_issue_close_time}
\alias{average_issue_close_time}
\title{Calculate Average Time to Close GitHub Issues}
\usage{
average_issue_close_time(
  owner,
  repo,
  max_pages = 10,
  per_page = 100,
  headers = c(Accept = "application/vnd.github.v3+json")
)
}
\arguments{
\item{owner}{Character. GitHub organization or username.}

\item{repo}{Character. Repository name.}

\item{max_pages}{Integer. Maximum number of API result pages to query. Each page can contain up to `per_page` issues. Default is 10.}

\item{per_page}{Integer. Number of issues to request per page (max 100). Default is 100.}

\item{headers}{Named character vector of HTTP headers to send with the API request. Default sets GitHub API version.}
}
\value{
Numeric. Average time in hours to close issues, or `NA` if no closed issues were found.
}
\description{
This function uses the GitHub API to retrieve closed issues from a repository and calculates
the average time (in hours) it took to close them, excluding pull requests.
}
\examples{
\dontrun{
average_issue_close_time("tidyverse", "ggplot2")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/badges.R
\name{list_badges}
\alias{list_badges}
\title{List badges image URLs from a local README}
\usage{
list_badges(path)
}
\arguments{
\item{path}{Character scalar; path to a local README file (e.g., "README.md").}
}
\value{
data.frame with badge info
}
\description{
Scans a local README (Markdown) and returns badge image URLs.
}
\examples{
\dontrun{
tmp <- tempfile(fileext = ".md")
writeLines(c(
  "# MyPkg",
  "![build](build-status.svg)",
  "![cov](coverage.svg)"
), tmp)
out <- list_badges(tmp)
print(out)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{get_exports}
\alias{get_exports}
\title{list all package exports}
\usage{
get_exports(pkg_source_path)
}
\arguments{
\item{pkg_source_path}{a file path pointing to an unpacked/untarred package directory}
}
\value{
data.frame, with one column `exported_function`, that can be passed
  to all downstream map_* helpers
}
\description{
list all package exports
}
\examples{
\dontrun{
tmpdir <- tempdir()

# Locate the package source tarball bundled with risk.assessr
dp <- system.file("test-data", "test.package.0001_0.1.0.tar.gz",
                  package = "risk.assessr")

# Extract the source package into the temporary directory
utils::untar(dp, exdir = tmpdir)

# Find the extracted package directory (it should contain DESCRIPTION)
pkg_dirs <- list.dirs(tmpdir, full.names = TRUE, recursive = FALSE)
desc_dir <- pkg_dirs[file.exists(file.path(pkg_dirs, "DESCRIPTION"))]
stopifnot(length(desc_dir) == 1L)
pkg_source_path <- desc_dir

# Now operate on the *source* (no installation)
exports_df <- get_exports(pkg_source_path)

# Optional: inspect result (kept simple for examples)
head(exports_df)

# Clean up the extracted sources
unlink(pkg_source_path, recursive = TRUE, force = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{extract_ggproto_methods}
\alias{extract_ggproto_methods}
\title{function to extract ggproto methods}
\usage{
extract_ggproto_methods(obj)
}
\arguments{
\item{obj}{- ggproto env from the target package}
}
\description{
function to get all the methods of a ggproto function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_github_data.R
\name{get_repo_owner}
\alias{get_repo_owner}
\title{Extract GitHub repository owner from links}
\usage{
get_repo_owner(links, pkg_name)
}
\arguments{
\item{links}{A character vector of GitHub URLs.}

\item{pkg_name}{A string representing the name of the package.}
}
\value{
A string containing the GitHub repository owner, or NA if not found.
}
\description{
Given a vector of GitHub links and a package name, this function finds the link
corresponding to the package and extracts the GitHub repository owner.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R
\name{get_bioconductor_package_url}
\alias{get_bioconductor_package_url}
\title{Retrieve Bioconductor Package URL}
\usage{
get_bioconductor_package_url(
  package_name,
  package_version = NULL,
  release_data
)
}
\arguments{
\item{package_name}{A character string specifying the name of the Bioconductor package.}

\item{package_version}{(Optional) A character string specifying the package version. Defaults to `NULL`, which retrieves the latest version.}

\item{release_data}{A list containing Bioconductor release information.}
}
\value{
A list containing:
 \item{url}{The URL of the source package (if available).}
 \item{version}{The specified or latest available package version.}
 \item{last_version}{A list with version, date, and Bioconductor version.}
 \item{all_versions}{A list of all discovered versions with version, date, and Bioconductor version.}
 \item{bioconductor_version_package}{The Bioconductor version matched to the selected version.}
 \item{archived}{A logical value indicating whether the package is archived.}
}
\description{
This function fetches the source package URL for a given Bioconductor package.
If no version is specified, it retrieves the latest available version.
}
\examples{
\dontrun{
release_data <- list(
  list(release = "3.17", date = "October 14, 2005"),
  list(release = "3.18", date = "October 4, 2006"),
  list(release = "3.19", date = "October 8, 2007")
)

get_bioconductor_package_url("GenomicRanges", release_data = release_data)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unpack_tarball.R
\name{unpack_tarball}
\alias{unpack_tarball}
\title{Untar package and return installation directory}
\usage{
unpack_tarball(pkg_tar, temp_file_name = "temp_file_")
}
\arguments{
\item{pkg_tar}{path to tarball package}

\item{temp_file_name}{name of `tempfile`}
}
\description{
Untar package and return installation directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{clean_license}
\alias{clean_license}
\title{Clean and normalize license names}
\usage{
clean_license(x)
}
\arguments{
\item{x}{A character string containing license information.}
}
\value{
A character vector of normalized license names.
}
\description{
Splits a license string on commas, plus signs, or pipes,
trims whitespace, removes trailing noise like "file LICENSE",
and normalizes each part to uppercase letters only.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{get_versions}
\alias{get_versions}
\title{Get Package Versions}
\usage{
get_versions(table, package_name)
}
\arguments{
\item{table}{A list of parsed package data (e.g., from `parse_html_version()`), where each element contains package details such as `package_version` and `date`.}

\item{package_name}{A character string specifying the name of the package to fetch versions for.}
}
\value{
A list with the following elements:
\describe{
  \item{all_versions}{A list of named lists, each containing:
    \describe{
      \item{version}{The version number of the package.}
      \item{date}{The associated publication date as a string.}
    }
  }
  \item{last_version}{A named list containing the latest version of the package with:
    \describe{
      \item{version}{The latest version number.}
      \item{date}{The publication date of the latest version.}
    }
    May be \code{NULL} if the API call fails.
  }
}
}
\description{
This function retrieves all available versions of a package, including the latest version,
by parsing the provided version table and querying the RStudio Package Manager.
}
\examples{
\dontrun{
# Define the input table
table <- list(
  list(
    package_name = "here",
    package_version = "0.1",
    link = "here_0.1.tar.gz",
    date = "2017-05-28 08:13",
    size = "3.5K"
  ),
  list(
    package_name = "here",
    package_version = "1.0.0",
    link = "here_1.0.0.tar.gz",
    date = "2020-11-15 18:10",
    size = "32K"
  )
)

result <- get_versions(table, "here")
print(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_pkg_desc}
\alias{get_pkg_desc}
\title{Read Description file and parse the package name and version}
\usage{
get_pkg_desc(pkg_source_path, fields = NULL)
}
\arguments{
\item{pkg_source_path}{path to package source code (untarred)}

\item{fields}{- select specified elements from description}
}
\value{
list with package description
}
\description{
Read Description file and parse the package name and version
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_modes.R
\name{cleanup_and_return_null}
\alias{cleanup_and_return_null}
\title{Clean up global environment and return NULL}
\usage{
cleanup_and_return_null(msg = NULL, env = cov_env)
}
\arguments{
\item{msg}{Optional character string. If provided, the message will be printed before cleanup.}

\item{env}{A character string specifying the covr env}
}
\value{
NULL (invisibly)
}
\description{
This internal utility function is used to clean up the global environment
by removing all objects and optionally displaying a message before returning `NULL`.
It is primarily used within `run_coverage_base()` to ensure consistent cleanup
in error-handling scenarios.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify_description_file.R
\name{modify_description_file}
\alias{modify_description_file}
\title{Modify the DESCRIPTION File in a R Package Tarball}
\usage{
modify_description_file(tar_file)
}
\arguments{
\item{tar_file}{A string representing the path to the `.tar.gz` file that contains the R package.}
}
\value{
A string containing the path to the newly created modified `.tar.gz` file.
}
\description{
This function recreate a `.tar.gz` R package file after modifying its `DESCRIPTION` file
by appending Config/build/clean-inst-doc: false parameter.
}
\examples{
\dontrun{
  modified_tar <- modify_description_file("path/to/mypackage.tar.gz")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{get_license_thresholds}
\alias{get_license_thresholds}
\title{Get License Levels from Thresholds}
\usage{
get_license_thresholds(thresholds)
}
\arguments{
\item{thresholds}{A list where each element contains a `level` (character) and `values` (list of character vectors).}
}
\value{
A named list with elements `"high"`, `"medium"`, and `"low"`, each containing a character vector of license names.
}
\description{
This internal utility function processes a list of license thresholds and returns
a named list of license values grouped by their risk level (e.g., "high", "medium", "low").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{get_all_s4_methods}
\alias{get_all_s4_methods}
\title{Extract all S4 methods}
\usage{
get_all_s4_methods(generic_name)
}
\arguments{
\item{generic_name}{name of function}
}
\value{
list with method_strings, generic, and methods
}
\description{
Extract all S4 methods
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/risk.assessr-package.R
\docType{package}
\name{risk.assessr-package}
\alias{risk.assessr}
\alias{risk.assessr-package}
\title{risk.assessr: Assessing Package Risk Metrics}
\description{
A reliable and validated tool that captures detailed risk metrics such as R 'CMD' check, test coverage, traceability matrix, documentation, dependencies, reverse dependencies, suggested dependency analysis, repository data, and enhanced reporting for R packages that are local or stored on remote repositories such as GitHub, CRAN, and Bioconductor.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://sanofi-public.github.io/risk.assessr/}
  \item Report bugs at \url{https://github.com/Sanofi-Public/risk.assessr/issues}
}

}
\author{
\strong{Maintainer}: Edward Gillian \email{edward.gillian-ext@sanofi.com} (\href{https://orcid.org/0000-0003-2732-5107}{ORCID})

Authors:
\itemize{
  \item Hugo Bottois \email{hugo.bottois@sanofi.com} (\href{https://orcid.org/0000-0003-4674-0875}{ORCID})
  \item Paulin Charliquart \email{paulin.charliquart@sanofi.com}
  \item Andre Couturier \email{andre.couturier@sanofi.com}
}

Other contributors:
\itemize{
  \item Sanofi [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_risk_analysis.R
\name{extract_risk_inputs}
\alias{extract_risk_inputs}
\title{Extract Risk Inputs}
\usage{
extract_risk_inputs(flat_data)
}
\arguments{
\item{flat_data}{Flat package data as returned by \code{normalize_data()}.}
}
\value{
A list of computed values required for risk scoring.
}
\description{
Compute intermediate risk input values from flattened package data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_func_descriptions.R
\name{get_func_descriptions}
\alias{get_func_descriptions}
\title{Get function descriptions}
\usage{
get_func_descriptions(pkg_name)
}
\arguments{
\item{pkg_name}{- name of the package}
}
\description{
get descriptions of exported functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contains_R_folder.R
\name{contains_r_folder}
\alias{contains_r_folder}
\title{Check for Vignette Folder and .Rmd Files in a .tar File}
\usage{
contains_r_folder(package_dir)
}
\arguments{
\item{package_dir}{A character string specifying the path to the package to be checked.}
}
\value{
A logical value: \code{TRUE} if the 'R' folder exists, \code{FALSE} otherwise.
}
\description{
This function checks if a given .tar file contains a 'vignettes' folder
and if there is at least one .Rmd file within that folder. If both 'vignettes'
and 'inst/doc' folders exist, the function will return \code{FALSE}.
}
\details{
The function checks if the specified folder exists. If  'R'
folder does not exist, the function returns \code{FALSE}. 
If the R folder exists, the function returns \code{TRUE}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{rd_aliases}
\alias{rd_aliases}
\title{Extract all aliases from an Rd document}
\usage{
rd_aliases(rd)
}
\arguments{
\item{rd}{An Rd object.}
}
\value{
A character vector of aliases (possibly empty).
}
\description{
Collects text from **all** `\\alias{}` nodes, trimming whitespace and
de-duplicating.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_news_current}
\alias{assess_news_current}
\title{Assess Rd files for news}
\usage{
assess_news_current(pkg_name, pkg_ver, pkg_source_path)
}
\arguments{
\item{pkg_name}{- name of the package}

\item{pkg_ver}{- package version}

\item{pkg_source_path}{- source path for install local}
}
\value{
news_current - variable with score
}
\description{
Assess Rd files for news
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{safe_value}
\alias{safe_value}
\title{Helper to conditionally apply handle_null or abbreviation}
\usage{
safe_value(x)
}
\arguments{
\item{x}{- value}
}
\description{
Helper to conditionally apply handle_null or abbreviation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{get_package_tarfile}
\alias{get_package_tarfile}
\title{Download R Package Source Tarball}
\usage{
get_package_tarfile(package_name, version = NULL, repos = getOption("repos"))
}
\arguments{
\item{package_name}{Character string specifying the name of the package to download.}

\item{version}{Optional character string specifying the version of the package.
If NULL (default), the latest available version will be downloaded.}

\item{repos}{Optional character vector of repository URLs to use.
If NULL (default), the current repositories set in options() will be used.}
}
\value{
Character string with the path to the downloaded tarball file, or NULL if
        the download was unsuccessful.
}
\description{
Downloads the source tarball (.tar.gz) for a specified R package from CRAN, 
Bioconductor, or an internal repository fallback.
}
\examples{
\dontrun{
# Download the latest version of dplyr
tarball_path <- get_package_tarfile("dplyr")

# Download a specific version
tarball_path <- get_package_tarfile("dplyr", version = "1.0.0")

# Use a specific repository
tarball_path <- get_package_tarfile("dplyr", repos = "https://cloud.r-project.org")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_traceability_matrix.R
\name{generate_traceability_matrix}
\alias{generate_traceability_matrix}
\title{Assess an R Package traceability matrix from package name and version}
\usage{
generate_traceability_matrix(
  package_name,
  version = NA,
  repos = getOption("repos"),
  execute_coverage = FALSE
)
}
\arguments{
\item{package_name}{A character string specifying the name of the package to assess.}

\item{version}{A character string specifying the version of the package to assess. Default is `NA`, which assesses the latest version.}

\item{repos}{A character string specifying the repo directly. Default is getOption("repos")}

\item{execute_coverage}{Logical (`TRUE`/`FALSE`). If `TRUE`, execute test coverage.}
}
\value{
The function returns package traceability_matrix
If the package cannot be downloaded or installed, an error message is returned.
}
\description{
This function use `risk.assessr::create_traceability_matrix` function with only the package name and version
}
\examples{
\dontrun{

results_no_test_covr <- generate_traceability_matrix(
 "here", 
 version = "1.0.1", 
 execute_coverage = FALSE
)

results_test_covr <- generate_traceability_matrix(
 "here", 
 version = "1.0.1", 
 execute_coverage = TRUE
)


print(results_no_test_covr)

print(results_test_covr)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_download.R
\name{get_package_download_bioconductor}
\alias{get_package_download_bioconductor}
\title{Get Bioconductor Package Download Statistics}
\usage{
get_package_download_bioconductor(pkg)
}
\arguments{
\item{pkg}{Character string. The name of a Bioconductor package (e.g. "GenomicRanges").}
}
\value{
A list with three elements:
\describe{
  \item{all_data}{A data.frame of the full cleaned dataset with dates and cumulative counts.}
  \item{total_6_months}{Total downloads in the last 6 complete months.}
  \item{total_download}{Total all-time downloads.}
}
}
\description{
Downloads and processes monthly download statistics for a Bioconductor package
from the Bioconductor package statistics archive. Returns the full history,
the total downloads over the last 6 full months (current month excluded),
and the all-time total downloads.
}
\examples{
\dontrun{
  result <- get_package_download_bioconductor("bioconductorpackage")
  print(result$total_6_months)
  head(result$all_data)
  
#$all_data
#Date Year Month Nb_of_distinct_IPs Nb_of_downloads Cumulative_downloads
#1   2009-01-01 2009   Jan               3341            7053                 7053
#2   2009-02-01 2009   Feb               3229            6681                13734
#3   2009-03-01 2009   Mar               3753            8021                21755
  
  #$total_6_months
  #[1] 547560
  
  #$total_download
  #[1] 7318550
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pubmed_data.R
\name{get_pubmed_count}
\alias{get_pubmed_count}
\title{Get Total Number of PubMed Articles for a Search Term}
\usage{
get_pubmed_count(package_name, api_key = getOption("pubmed.api_key", NULL))
}
\arguments{
\item{package_name}{A character string representing the search term (e.g., an R package name).}

\item{api_key}{Optional. A character string with an NCBI API key. If not supplied,
it will attempt to use the option \code{getOption("pubmed.api_key")}.}
}
\value{
An integer representing the total number of PubMed articles matching the search term, 
or \code{NA} if the request fails.
}
\description{
This function queries the NCBI E-utilities API to retrieve the total number of 
PubMed articles that match a given search term or R package name.
}
\examples{
\dontrun{
get_pubmed_count("ggplot2")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_github_data.R
\name{get_github_data}
\alias{get_github_data}
\title{Fetch GitHub Repository Data}
\usage{
get_github_data(owner, repo)
}
\arguments{
\item{owner}{A character string specifying the owner of the repository.}

\item{repo}{A character string specifying the name of the repository.

A GitHub Personal Access Token (PAT) will be needed for some requests or to help with the rate limit.
Use Sys.setenv(GITHUB_TOKEN = "personal_access_token") or store your token in a .Renviron file.
The token is passed via the `Authorization` header only if `Sys.getenv("GITHUB_TOKEN")` is non-empty.}
}
\value{
A list containing:
  - `created_at`: Creation date of the repository.
  - `stars`: Number of stars.
  - `forks`: Number of forks.
  - `date`: Acquisition date (YYYY-MM-DD).
  - `recent_commits_count`: Number of commits in the last 30 days.
  - `open_issues`: Number of open issues.
}
\description{
This function retrieves metadata about a GitHub repository, including creation date,
stars, forks, and the number of recent commits within the last 30 days.
}
\details{
Repository data is fetched using the GitHub API via `curl` and `jsonlite`.
}
\examples{
\dontrun{
result <- get_github_data("tidyverse", "ggplot2")
print(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pubmed_data.R
\name{get_pubmed_by_year}
\alias{get_pubmed_by_year}
\title{Get Annual PubMed Article Counts for a Search Term}
\usage{
get_pubmed_by_year(
  package_name,
  years_back = 10,
  api_key = getOption("pubmed.api_key", NULL)
)
}
\arguments{
\item{package_name}{A character string representing the search term (e.g., an R package name).}

\item{years_back}{An integer specifying how many years back to query. Defaults to 10.}

\item{api_key}{Optional. A character string with an NCBI API key. If not supplied,
it will attempt to use the option \code{getOption("pubmed.api_key")}.}
}
\value{
A data frame with two columns:
\describe{
  \item{Year}{The publication year (integer).}
  \item{Count}{The number of articles published in that year (integer).}
}
Returns an empty data frame if no valid data is retrieved.
}
\description{
This function queries the NCBI E-utilities API to retrieve the number of PubMed articles
published each year over a specified number of past years for a given search term or package name.
}
\examples{
\dontrun{
get_pubmed_by_year("ggplot2", years_back = 5)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{generate_trace_matrix_section}
\alias{generate_trace_matrix_section}
\title{Generate Trace Matrix Section}
\usage{
generate_trace_matrix_section(total_coverage, file_coverage, tm_df)
}
\arguments{
\item{total_coverage}{- total coverage for the package}

\item{file_coverage}{- file coverage for the package}

\item{tm_df}{- input data}
}
\description{
Generates the Trace Matrix section for the HTML report.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_summary_report.R
\name{write_summary_report}
\alias{write_summary_report}
\title{Write the risk assessment summary report}
\usage{
write_summary_report(
  results,
  output_file = NULL,
  output_format = NULL,
  output_dir = NULL
)
}
\arguments{
\item{results}{A results object containing the risk assessment results. It is
expected to include `results$results$pkg_name` and `results$results$pkg_version`,
which are used to construct default file names and report parameters.}

\item{output_file}{`character(1)` or `NULL`. The output file path **or** a directory.
- If `NULL`, a default name is generated:
  `"summary_report_<pkg_name>_<pkg_version>.html"` in `output_dir`.
- If it is a directory, the default name is composed inside it.
- If it is a relative path, it is resolved against `output_dir`.}

\item{output_format}{`character(1)` or `NULL`. Explicit output format to render:
one of `"html"` or `"md"` (case-insensitive; do **not** include a leading dot).
If `NULL`, the format is inferred from the extension of `output_file`.
Unknown values trigger a warning and default to HTML.}

\item{output_dir}{`character(1)` or `NULL`. Base directory used when
`output_file` is `NULL` or a relative path. If `NULL`, defaults to the
current working directory (`getwd()`), with an informational message.}
}
\value{
(Invisibly) the absolute path to the written report file.
}
\description{
Renders the package risk assessment report (`summary.Rmd`) to a chosen format
and writes it to disk. The output file path can be inferred or explicitly set.
You can specify the output format directly via `output_format` or let it be
inferred from the extension of `output_file`. If `output_dir` is not provided,
the current working directory is used.
}
\details{
The function calls [rmarkdown::render()] on an internal template:
`system.file("report_templates", "summary.Rmd", package = "risk.assessr")`.

The format mapping is:
- `"html"` -> [rmarkdown::html_document()]
- `"md"`   -> [rmarkdown::md_document()]

If `output_format` is provided, it takes precedence for rendering even if
`output_file` has a different extension; in that case the filename is adjusted
to match the format. If `output_format` is not provided, the function infers the
format from `output_file`'s extension. For unknown extensions, it warns and
rewrites the extension to `.html`.
}
\section{Behavior}{

- If `output_dir` is `NULL`, it defaults to `getwd()` and a message is printed.
- If `output_file` is `NULL`, a file name is auto-generated as
  `"summary_report_<pkg>_<version>.html"` inside `output_dir`.
- If `output_file` points to an existing directory (or ends with a path separator),
  a file name is composed inside that directory as
  `"summary_report_<pkg>_<version>.html"`.
- If `output_file` is a relative path, it is resolved against `output_dir`.
- If `output_format` is provided, it determines the render format
  (and the `output_file` name is adjusted to match the format).
- If `output_format` is not provided, the render format is inferred from the
  extension of `output_file`. Unknown extensions trigger a warning and the
  extension is changed to `.html`.
}

\examples{
\dontrun{
# generate assessment_results
results <- risk_assess_pkg()

# Basic usage: auto-name in the working directory (HTML)
write_summary_report(results)

# Specify output directory (auto-name inside that directory)
write_summary_report(results, output_dir = "artifacts")

# Provide an explicit filename (relative to output_dir)
write_summary_report(results, output_file = "reports/summary.html")

# Provide a directory as output_file: auto-name inside that directory
write_summary_report(results, output_file = "reports/")

# Render to Markdown explicitly
write_summary_report(results, output_format = "md")

# Let the extension drive the format (md)
write_summary_report(results, output_file = "reports/summary.md")

# Case-insensitive formats work; avoid leading dots (".md" is not valid)
write_summary_report(results, output_format = "MD")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_stf.R
\name{run_covr_safe_stf}
\alias{run_covr_safe_stf}
\title{Run skip coverage safely in a separate R session}
\usage{
run_covr_safe_stf(pkg, map_path, test_skip, test_map, timeout = 120)
}
\arguments{
\item{pkg}{Character. Name of the package being tested.}

\item{map_path}{Character. Path to the `testthat` directory containing mapped test structure.}

\item{test_skip}{Data frame. Skip map generated from problematic tests.}

\item{test_map}{Data frame. Mapped structure of test files and expectations.}

\item{timeout}{Numeric. Maximum time (in seconds) allowed for the coverage process to run. Default is 120 seconds.}
}
\value{
A coverage object returned by `coverage_skip()`, or `NULL` if an error occurs or timeout is reached.
}
\description{
This internal helper function uses `callr::r_safe()` to execute the
`coverage_skip()` function from the `risk.assessr` package in a clean R session.
This avoids issues with loaded namespaces (e.g., `dplyr`) that may interfere with coverage analysis.
}
\details{
It includes error handling and a configurable timeout to prevent long-running or stuck processes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{generate_doc_metrics_section}
\alias{generate_doc_metrics_section}
\title{Generate Doc Metrics Section}
\usage{
generate_doc_metrics_section(assessment_results)
}
\arguments{
\item{assessment_results}{- input data}
}
\description{
Generates the documentation section for the HTML report.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_stf.R
\name{run_covr_skip_stf}
\alias{run_covr_skip_stf}
\title{Run covr with failing tests
#' 
This function runs test coverage and skip analysis for a package using testthat or other supported frameworks.}
\usage{
run_covr_skip_stf(pkg_source_path, test_pkg_data)
}
\arguments{
\item{pkg_source_path}{Character. Path to the root of the package source.}

\item{test_pkg_data}{List. Output from `check_pkg_tests_and_snaps()` indicating test framework presence.}
}
\value{
skip_results - test coverage with PASS tests passing or FAIL for tests failing
}
\description{
Run covr with failing tests
#' 
This function runs test coverage and skip analysis for a package using testthat or other supported frameworks.
}
\examples{
\dontrun{
r = getOption("repos")
r["CRAN"] = "http://cran.us.r-project.org"
old <- options(repos = r)

dp <- system.file("test-data", 
  "test.package.0014_0.1.0.tar.gz", 
  package = "risk.assessr")

# set up package
install_list <- set_up_pkg(dp)

build_vignettes <- install_list$build_vignettes
package_installed <- install_list$package_installed
pkg_source_path <- install_list$pkg_source_path
rcmdcheck_args <- install_list$rcmdcheck_args

# install package locally to ensure test works
package_installed <- install_package_local(pkg_source_path)
package_installed <- TRUE

rcmdcheck_args$path <- pkg_source_path
test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
test_covr_list <- run_covr_skip_stf(pkg_source_path, test_pkg_data)

options(old)
}  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{get_rd_section}
\alias{get_rd_section}
\title{Get the first Rd section by tag}
\usage{
get_rd_section(rd, section)
}
\arguments{
\item{rd}{An Rd object (a nested list with "Rd_tag" attributes on nodes).}

\item{section}{Section name without leading backslash (e.g., "alias", "examples").}
}
\value{
The first matching Rd node or `NULL` if not found.
}
\description{
Depth-first search returning the first node whose `Rd_tag` equals
`paste0("\\\\", section)`.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{get_pkg_author}
\alias{get_pkg_author}
\title{Assess Authors}
\usage{
get_pkg_author(pkg_name, pkg_source_path)
}
\arguments{
\item{pkg_name}{- name of the package}

\item{pkg_source_path}{- source path for install local}
}
\value{
- Authors related variable
}
\description{
Assess Authors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_pkg_r_package.R
\name{assess_pkg_r_package}
\alias{assess_pkg_r_package}
\title{Assess an R package by name (deprecated)}
\usage{
assess_pkg_r_package(package_name, version = NA, repos = getOption("repos"))
}
\arguments{
\item{package_name}{Package name.}

\item{version}{Package version. Default `NA` uses latest.}

\item{repos}{Repository URLs. Default `getOption("repos")`.}
}
\value{
Assessment results or `NULL`.
}
\description{
Use [risk_assess_pkg()] with `package` instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_thresholds.R
\name{extract_thresholds_by_key}
\alias{extract_thresholds_by_key}
\title{Extract risk thresholds by key}
\usage{
extract_thresholds_by_key(risk_rule, key_value)
}
\arguments{
\item{risk_rule}{- list containing all risk rules}

\item{key_value}{- character vector containing risk rule choice}
}
\value{
list containing specific risk rules
}
\description{
Extract risk thresholds by key
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{get_s3_method}
\alias{get_s3_method}
\title{function to get S3 method}
\usage{
get_s3_method(generic, class, package)
}
\arguments{
\item{generic}{- function generic}

\item{class}{- function class}

\item{package}{- package name}
}
\description{
function to get S3 method from generic
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_modes.R
\name{get_function_no_tests}
\alias{get_function_no_tests}
\title{get functions with no tests}
\usage{
get_function_no_tests(mapping_df)
}
\arguments{
\item{mapping_df}{- data frame with source files and test files}
}
\value{
no_tests_df - data frame with source files with no tests
}
\description{
get functions with no tests
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_suggested_exp_funcs.R
\name{get_suggested_exp_funcs}
\alias{get_suggested_exp_funcs}
\title{Function to get suggested exported functions}
\usage{
get_suggested_exp_funcs(data)
}
\arguments{
\item{data}{- all packages listed in the DESCRIPTION file}
}
\value{
- data with package names and exported functions
}
\description{
This function gets exported functions for all packages
in the Suggests section of the target package's DESCRIPTION file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_export_help}
\alias{assess_export_help}
\title{assess_export_help}
\usage{
assess_export_help(pkg_name, pkg_source_path)
}
\arguments{
\item{pkg_name}{- name of the package}

\item{pkg_source_path}{- pkg_source_path - source path for install local}
}
\value{
- export_help - variable with score
}
\description{
assess_export_help
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_traceability_matrix.R
\name{create_traceability_matrix}
\alias{create_traceability_matrix}
\title{Create a Traceability Matrix}
\usage{
create_traceability_matrix(
  pkg_name,
  pkg_source_path,
  func_covr = NULL,
  verbose = FALSE,
  execute_coverage = TRUE
)
}
\arguments{
\item{pkg_name}{name of package}

\item{pkg_source_path}{path to a source package}

\item{func_covr}{function coverage}

\item{verbose}{Logical (`TRUE`/`FALSE`). If `TRUE`, show any warnings/messages per function.}

\item{execute_coverage}{Logical (`TRUE`/`FALSE`). If `TRUE`, execute test coverage.}
}
\value{
a nested list with fine grained traceability matrices
}
\description{
Returns a table that links all exported functions and their aliases to their documentation (`man` files),
the R scripts containing them, and the test scripts that reference them.
}
\examples{
\dontrun{
# set CRAN repo to enable running of reverse dependencies
r = getOption("repos")
r["CRAN"] = "http://cran.us.r-project.org"
old <- options(repos = r)

dp <- system.file("test-data", "here-1.0.1.tar.gz", 
   package = "risk.assessr")

# Set up the package using the temporary file
install_list <- set_up_pkg(dp)

# Extract information from the installation list
build_vignettes <- install_list$build_vignettes
package_installed <- install_list$package_installed
pkg_source_path <- install_list$pkg_source_path
rcmdcheck_args <- install_list$rcmdcheck_args

# check if the package needs to be installed locally
package_installed <- install_package_local(pkg_source_path)

# Get package name and version
pkg_desc <- get_pkg_desc(pkg_source_path, 
                         fields = c("Package", 
                                    "Version"))
pkg_name <- pkg_desc$Package
test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)

covr_timeout <- Inf

if (test_pkg_data$has_testthat || test_pkg_data$has_testit) {
  
  covr_list <- run_coverage(
    pkg_source_path,  
    covr_timeout
  )    
} else {
  message("No testthat or testit configuration")
  covr_list <- list(
    total_cov = 0,
    res_cov = list(
      name = pkg_name,
      coverage = list(
        filecoverage = matrix(0, nrow = 1, dimnames = list("No functions tested")),
        totalcoverage = 0
      ),
      errors = "No testthat or testit configuration",
      notes = NA
    )
  )
}
# run without test coverage   
tm_list_no_covr <- create_traceability_matrix(pkg_name, 
                                      pkg_source_path,
                                      execute_coverage = FALSE)   
                                      
# run with test coverage                                         
tm_list_covr <- create_traceability_matrix(pkg_name, 
                                      pkg_source_path,
                                      covr_list$res_cov)
                                      
# run with test coverage parameter but no test coverage list                                        
tm_list_no_covr_list <- create_traceability_matrix(pkg_name, 
                                      pkg_source_path)   
                                         
options(old)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{detect_version_conflicts}
\alias{detect_version_conflicts}
\title{Detect Version Conflicts from dependency tree}
\usage{
detect_version_conflicts(pkg_list)
}
\arguments{
\item{pkg_list}{A nested list structure from fetch_all_dependencies function}
}
\value{
A list of strings describing package version conflicts such as "Conflict in package cli: versions found - 3.6.2, 3.6.1"
`NULL` if no conflicts are found.
}
\description{
This function identifies duplicate version packages and reports conflicts.
}
\examples{
pkg_list <- list(
  ggplot2 = list(
    version = "3.5.1",
    cli = list(version = "3.6.2"),
    gtable = list(
      version = "0.3.4",
      cli = list(version = "3.6.1")
    )
  )
)
detect_version_conflicts(pkg_list)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{extract_exported_function_info}
\alias{extract_exported_function_info}
\title{Extract Exported Function Metadata from an R Package}
\usage{
extract_exported_function_info(.pkg_source_path, package_name)
}
\arguments{
\item{.pkg_source_path}{Character. Path to the source directory of the R package.}

\item{package_name}{Character. Name of the package (typically the folder name or actual package name).}
}
\value{
A tibble with columns: `exported_function`, `class`, `function_type`, `function_body`, and `where`.
}
\description{
This internal utility function analyzes the namespace of an R package and extracts
metadata about its exported functions, including classification into S3, S4, S7, and regular functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_stf.R
\name{create_covr_list_no_skip}
\alias{create_covr_list_no_skip}
\title{create covr_list when no tests are skipped}
\usage{
create_covr_list_no_skip(test_map, test_results_long, pkg_source_path, pkg)
}
\arguments{
\item{test_map}{- R object with mapped tests}

\item{test_results_long}{- cvr object with no tests skipped}

\item{pkg_source_path}{- path to the package}

\item{pkg}{- name of the package}
}
\value{
- summary cvr object
}
\description{
create covr_list when no tests are skipped
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{check_and_fetch_cran_package}
\alias{check_and_fetch_cran_package}
\title{Check and Fetch CRAN Package}
\usage{
check_and_fetch_cran_package(package_name, package_version = NULL)
}
\arguments{
\item{package_name}{A character string specifying the name of the package to check and fetch.}

\item{package_version}{An optional character string specifying the version of the package to fetch. Defaults to `NULL`.}
}
\value{
A list with one of the following structures:

\describe{
  \item{package_url}{Character string; the URL to download the package tarball.}
  \item{last_version}{A named list with \code{version} and \code{date} for the latest available version.}
  \item{version}{Character string; the requested version (or \code{NULL} if not specified).}
  \item{all_versions}{A list of named lists, each with \code{version} and \code{date}, representing all available versions.}
}
}
\description{
This function checks if a package exists on CRAN and retrieves the corresponding package URL and version details.
If a specific version is not provided, the latest version is used.
}
\examples{
\dontrun{
# Check and fetch a specific version of "ggplot2"
result <- check_and_fetch_cran_package("ggplot2", package_version = "3.3.5")
print(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_stf.R
\name{create_results_summary}
\alias{create_results_summary}
\title{Create a results summary for easier access to scores}
\usage{
create_results_summary(cvr)
}
\arguments{
\item{cvr}{- covr object with full results of skip coverage}
}
\value{
- dataframe with total scores
}
\description{
- retrieve overall percentage and file coverage percentages
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{get_host_package}
\alias{get_host_package}
\title{Extract and Validate Package Hosting Information}
\usage{
get_host_package(pkg_name, pkg_version, pkg_source_path)
}
\arguments{
\item{pkg_name}{Character. The name of the package.}

\item{pkg_version}{Character. The version of the package.}

\item{pkg_source_path}{Character. The file path to the package source directory containing the DESCRIPTION file.}
}
\value{
A list containing the following elements:

- `github_links`: GitHub links related to the package.
- `cran_links`: CRAN links
- `internal_links`: internal repository links.
- `bioconductor_links`: Bioconductor links 

If links are found, return empty or NULL.
}
\description{
This function retrieves hosting links for an R package from various sources such as GitHub, CRAN, internal repositories, or Bioconductor.
}
\details{
The function extracts hosting links by:
1. Parsing the `DESCRIPTION` file for GitHub and BugReports URLs.
2. Checking if the package is valid on CRAN and others host repository

If no links are found in the `DESCRIPTION` file, returns `NULL`
}
\examples{
\dontrun{
result <- get_host_package(pkg_name, pkg_version, pkg_source_path)
print(result)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_metrics.R
\name{run_covr}
\alias{run_covr}
\title{Run covr in subprocess with timeout}
\usage{
run_covr(path, timeout)
}
\arguments{
\item{path}{- path to source file}

\item{timeout}{- length of timeout - set to Inf}
}
\description{
Run covr in subprocess with timeout
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{extract_maintainer_info}
\alias{extract_maintainer_info}
\title{Helper to create maintainer}
\usage{
extract_maintainer_info(person_obj)
}
\arguments{
\item{person_obj}{- value}
}
\description{
Helper to create maintainer
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_tests_stf.R
\name{unrowname}
\alias{unrowname}
\title{Add identifier column and remove row names from a data.frame}
\usage{
unrowname(identifier, ret, label)
}
\arguments{
\item{identifier}{character, the name or label to assign to the new column}

\item{ret}{list of data.frames, typically the result of applying a function over multiple files}

\item{label}{character, name of the new column to store the identifier}
}
\value{
data.frame with identifier column prepended and row names removed
}
\description{
Internal helper function to restructure a data.frame by adding an identifier column and removing row names.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{parse_dcf_dependencies_version}
\alias{parse_dcf_dependencies_version}
\title{Parse Dependencies from a Package DESCRIPTION File}
\usage{
parse_dcf_dependencies_version(path)
}
\arguments{
\item{path}{A character string specifying the path to the package directory containing the DESCRIPTION file.}
}
\value{
A data frame with columns:
	- `package`: The name of the imported package.
	- `type`: The type of dependency (e.g., "Imports").
}
\description{
This function extracts and returns the dependencies from the DESCRIPTION file
of an R package, focusing on the `Imports` field.
}
\examples{

\dontrun{
parse_dcf_dependencies_version("/path/to/package")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unittest_methods.R
\name{get_test_path}
\alias{get_test_path}
\title{Determine the appropriate test path for a package}
\usage{
get_test_path(test_pkg_data, testdir)
}
\arguments{
\item{test_pkg_data}{A named list containing logical flags such as `has_testthat`, `has_testit`, `has_tests_base`, etc.}

\item{testdir}{A character string specifying the root directory where test folders are located.}
}
\value{
A character string representing the path to the test directory, or `NULL` if no known test framework is found.
}
\description{
This internal utility function inspects the `test_pkg_data` structure to identify
which testing framework is used in the package and returns the corresponding test path.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_download.R
\name{get_package_download_cran}
\alias{get_package_download_cran}
\title{Get CRAN Daily Downloads for a Package}
\usage{
get_package_download_cran(pkg, years = 1)
}
\arguments{
\item{pkg}{Character string, name of the CRAN package.}

\item{years}{integer, number of past years (default 1) for historical}
}
\value{
A data frame with columns: \code{date}, \code{count}, and \code{cumulative_downloads}.
}
\description{
Retrieves daily CRAN download counts over the past years for a given package,
including cumulative downloads.
}
\examples{
\dontrun{
  get_package_download_cran("ggplot2")
  # Example output:
  #          date  count cumulative_downloads
  # 1 2025-04-22  78379           20073615
  # 2 2025-04-21  63195           19995236
  # 3 2025-04-20  42119           19932041
  # 4 2025-04-19  40848           19889922
  # 5 2025-04-18  54914           19849074
  # 6 2025-04-17  86273           19794160
  # 7 2025-04-16  80201           19707887
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{expand_dependency_type_spec}
\alias{expand_dependency_type_spec}
\title{Expand Dependency Type Specification}
\usage{
expand_dependency_type_spec(x)
}
\arguments{
\item{x}{A character string specifying the dependency type ("strong", "most", "all", or custom).}
}
\value{
A character vector of dependency types.
}
\description{
Expands a dependency type specification into a character vector of dependency types.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_traceability_matrix.R
\name{get_toplevel_assignments}
\alias{get_toplevel_assignments}
\title{list all top-level objects defined in the package code}
\usage{
get_toplevel_assignments(pkg_source_path)
}
\arguments{
\item{pkg_source_path}{a file path pointing to an unpacked/untarred package directory}
}
\value{
A data.frame with the columns `function` and `code_script` with a row for
  every top-level object defined in the package.
}
\description{
adapted from mpn.scorecard
}
\details{
This is primarily for getting all _functions_, but it also returns top-level
declarations, regardless of type. This is intentional, because we also want
to capture any global variables or anything else that could be potentially
exported by the package.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{check_cran_package}
\alias{check_cran_package}
\title{Check if a Package Exists on CRAN}
\usage{
check_cran_package(package_name)
}
\arguments{
\item{package_name}{A character string specifying the name of the package to check.}
}
\value{
A logical value: `TRUE` if the package exists on CRAN, `FALSE` otherwise.
}
\description{
This function checks if a given package is available on CRAN.
}
\examples{
\dontrun{
# Check if the package "ggplot2" exists on CRAN
check_cran_package("ggplot2")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{build_dependency_tree}
\alias{build_dependency_tree}
\title{Build a Dependency Tree for an R Package}
\usage{
build_dependency_tree(
  package_name,
  level = 1,
  get_license = FALSE,
  max_level = 3
)
}
\arguments{
\item{package_name}{A character string specifying the name of the package.}

\item{level}{An integer indicating the current recursion depth (default: 1).}

\item{get_license}{A logical value indicating whether to extract and include license information from DESCRIPTION files (default: `FALSE`).}

\item{max_level}{An integer specifying the maximum depth level for dependency recursion (default: 3). 
Set to a higher value to explore deeper dependency layers.}
}
\value{
A nested list representing the dependency tree, where base packages are marked as "base".
        If `get_license` is `TRUE`, each package entry in the tree includes a `license` field containing 
        that specific package's license (extracted from its DESCRIPTION file). Each package's license is 
        independent - the license of a parent package does not imply the licenses of its dependencies.
}
\description{
This function constructs a nested list representing the dependency tree of a specified R package.
The recursion stops at the specified maximum depth level (default: 3) or until a base package is reached.
}
\examples{
\dontrun{
build_dependency_tree("ggplot2")
build_dependency_tree("ggplot2", get_license = TRUE)
build_dependency_tree("ggplot2", max_level = 5)  # Deeper dependency exploration
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{extract_dependency_package_names}
\alias{extract_dependency_package_names}
\title{Extract Package Names from a Dependency String}
\usage{
extract_dependency_package_names(x)
}
\arguments{
\item{x}{A character string representing a package dependency specification,
such as `"pkgA (>= 1.0), pkgB, R (>= 3.5.0)"`.}
}
\value{
A character vector of package names extracted from the input string.
The result excludes "R" and any version information.
}
\description{
Parses a single package dependency string and extracts valid package names,
excluding "R" and ignoring version constraints or comments.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{rd_examples_to_text}
\alias{rd_examples_to_text}
\title{Flatten example nodes to a text block}
\usage{
rd_examples_to_text(node)
}
\arguments{
\item{node}{An Rd node.}
}
\value{
Character scalar containing example text or `""` if none.
}
\description{
Recursively collects example text from nodes that carry R code or text,
unwrapping containers such as `\\dontrun{}`, `\\donttest{}`, and `\\dontshow{}`.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R,
%   R/get_package_host_info.R
\name{parse_html_version}
\alias{parse_html_version}
\title{Parse HTML Content for Package Versions}
\usage{
parse_html_version(html_content, package_name)

parse_html_version(html_content, package_name)
}
\arguments{
\item{html_content}{A character string containing the HTML content of a CRAN package archive page.}

\item{package_name}{A character string specifying the name of the package to extract version information for.}
}
\value{
A list of lists containing package details such as package name, version, link,
        date, and size. Returns an empty list if no versions are found.

A list of lists, where each sublist contains:
  - `package_name`: package name.
  - `package_version`: package version.
  - `link`: link to the package tarball.
  - `date`: The date associated with the package version.
  - `size`: The size of the package tarball.
}
\description{
This function extracts version information from an HTML page listing
available versions of a Bioconductor package.

This function extracts version information from the HTML content of a CRAN archive page.
}
\examples{
\dontrun{
parse_html_version(html_content, "GenomicRanges")
}
\dontrun{
html_content <- parse_package_info("dplyr")
parse_html_version(html_content, "dplyr")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{download_and_parse_dependencies}
\alias{download_and_parse_dependencies}
\title{Download and Parse Dependencies of an R Package}
\usage{
download_and_parse_dependencies(
  package_name,
  version = NA,
  get_license = FALSE
)
}
\arguments{
\item{package_name}{Name of the package to download.}

\item{version}{Package version to download (default: `NA` for latest).}

\item{get_license}{Whether to extract license information (default: `FALSE`).}
}
\value{
When `get_license = FALSE`, a data frame with columns `package`, `type`, and `parent_package`.
  When `get_license = TRUE`, a list with:
    - `dependencies`: Data frame of direct dependencies.
    - `license`: License of the specified package only (not dependencies). Returns `NA` if not found.
}
\description{
Downloads a package from a repository, extracts it, and parses dependencies from the DESCRIPTION file.
}
\details{
The `license` field refers only to the package specified by `package_name`, not its dependencies.
  Use `build_dependency_tree()` or `fetch_all_dependencies()` with `get_license = TRUE` for full license information.
}
\examples{
\dontrun{
download_and_parse_dependencies("dplyr")
download_and_parse_dependencies("dplyr", get_license = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_tests_skip_stf.R
\name{get_tests_skip_stf}
\alias{get_tests_skip_stf}
\title{get skipped tests with line locations}
\usage{
get_tests_skip_stf(problem_tests, test_map)
}
\arguments{
\item{problem_tests}{- problem tests}

\item{test_map}{- all tests with line locations}
}
\value{
test_skip
}
\description{
get skipped tests with line locations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_traceability_matrix.R
\name{filter_symbol_functions}
\alias{filter_symbol_functions}
\title{Remove specific symbols from vector of functions}
\usage{
filter_symbol_functions(funcs)
}
\arguments{
\item{funcs}{vector of functions to filter}
}
\description{
adapted from mpn.scorecard
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{check_ggproto}
\alias{check_ggproto}
\title{function to check value of ggproto}
\usage{
check_ggproto(value)
}
\arguments{
\item{value}{- ggproto functions from the target package}
}
\description{
function to check all classes of a ggproto function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R
\name{bioconductor_reverse_deps}
\alias{bioconductor_reverse_deps}
\title{Find Bioconductor Package Reverse Dependencies}
\usage{
bioconductor_reverse_deps(
  pkg,
  which = "Imports",
  only.bioc = FALSE,
  version = BiocManager::version(),
  db = NULL,
  biocdb = NULL
)
}
\arguments{
\item{pkg}{Character string. The name of the package for which to find reverse dependencies.}

\item{which}{Character vector. The dependency categories to check.
One or more of \code{"Depends"}, \code{"Imports"}, \code{"LinkingTo"},
\code{"Suggests"}, or \code{"Enhances"}. Defaults to \code{"Imports"}.}

\item{only.bioc}{Logical. If \code{TRUE} (default), only reverse dependencies
that are Bioconductor packages are returned.}

\item{version}{Bioconductor version to use. Defaults to the current version.}

\item{db}{Optional. A pre-loaded package database to use for lookups.}

\item{biocdb}{Optional. A pre-loaded Bioconductor package database.}
}
\value{
A named list of reverse dependency package names.
}
\description{
This function returns the reverse dependencies for a given Bioconductor package.
}
\examples{
\dontrun{
# Get reverse Imports dependencies as a list:
bioconductor_reverse_deps("limma")

# Get multiple categories:
bioconductor_reverse_deps("limma", which = c("Depends", "Suggests"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_risk_analysis.R
\name{compute_risk}
\alias{compute_risk}
\title{Compute Risk Level}
\usage{
compute_risk(value, risk)
}
\arguments{
\item{value}{The numeric or categorical value to evaluate.}

\item{risk}{A list describing the thresholds for the metric.}
}
\value{
The computed risk level (e.g. "low", "medium", "high").
}
\description{
Apply risk thresholds to compute risk level for a single metric.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{build_rd_index}
\alias{build_rd_index}
\title{Build a fast lookup index for an Rd database}
\usage{
build_rd_index(db)
}
\arguments{
\item{db}{An Rd database list as returned by `tools::Rd_db()`.}
}
\value{
A list with elements `alias_index` (environment) and `topic_by_file` (named character vector).
}
\description{
Provides (1) an alias index, mapping `alias -> filename`, and
(2) a topic index, mapping `filename -> \\name` (topic).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_traceability_matrix.R
\name{fine_grained_tms}
\alias{fine_grained_tms}
\title{Create fine grained traceability matrices}
\usage{
fine_grained_tms(tm, pkg_name)
}
\arguments{
\item{tm}{- general traceability matrix with no filtered information}

\item{pkg_name}{- name of the package}
}
\value{
- nested list with fine grained tms
}
\description{
Create fine grained traceability matrices
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_html_report.R
\name{get_max_thresholds}
\alias{get_max_thresholds}
\title{Extract Maximum Thresholds for Code Coverage Levels}
\usage{
get_max_thresholds(thresholds)
}
\arguments{
\item{thresholds}{A list of threshold objects, each containing a `level` and `max` field.}
}
\value{
A list with three elements:
\describe{
  \item{high_max}{The maximum threshold value for the `"high"` level.}
  \item{medium_max}{The maximum threshold value for the `"medium"` level.}
  \item{medium_max}{The maximum threshold value for the `"low"` level.}
}
}
\description{
This internal function retrieves the `max` values for `"high"`,`"medium"`, and `"low"` levels
from a list of code coverage thresholds.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_risk_analysis.R
\name{normalize_data}
\alias{normalize_data}
\title{Normalize Nested Package Data}
\usage{
normalize_data(results)
}
\arguments{
\item{results}{The nested package metadata (list structure as produced by your package).}
}
\value{
A list with flattened package information.
}
\description{
Flattens nested package metadata into a flat list for further processing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_nstf.R
\name{run_covr_skip_nstf}
\alias{run_covr_skip_nstf}
\title{Run Coverage Analysis on Package Source and Tests}
\usage{
run_covr_skip_nstf(pkg_source_path, test_pkg_data)
}
\arguments{
\item{pkg_source_path}{Character string. Path to the root of the package source directory.}

\item{test_pkg_data}{List. Output from `check_pkg_tests_and_snaps()` indicating test framework presence.}
}
\value{
returns a list with coverage object and source files with no tests.
}
\description{
This internal function maps source files to test files, checks for skipped/problematic tests,
runs the remaining test files manually, calculates coverage, and optionally generates a report.
}
\examples{
\dontrun{
r = getOption("repos")
r["CRAN"] = "http://cran.us.r-project.org"
options(repos = r)

dp <- system.file("test-data", 
  "MASS_7.3-65.tar.gz", 
  package = "risk.assessr")

# set up package
install_list <- set_up_pkg(dp)

build_vignettes <- install_list$build_vignettes
package_installed <- install_list$package_installed
pkg_source_path <- install_list$pkg_source_path
rcmdcheck_args <- install_list$rcmdcheck_args

# install package locally to ensure test works
package_installed <- install_package_local(pkg_source_path)
package_installed <- TRUE

rcmdcheck_args$path <- pkg_source_path
test_pkg_data <- check_pkg_tests_and_snaps(pkg_source_path)
test_covr_list <- run_covr_skip_nstf(pkg_source_path, test_pkg_data)

options(old)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_risk_analysis.R
\name{get_risk_analysis}
\alias{get_risk_analysis}
\title{Get Risk Analysis}
\usage{
get_risk_analysis(data)
}
\arguments{
\item{data}{The nested package metadata.}
}
\value{
A named list of computed risk levels.
}
\description{
Compute risk levels for the package metadata.
}
\examples{
# Minimal mock input for demonstration
mock_data <- list(
  dependencies_count = 5,
  later_version = 2,
  code_coverage = 0.75,
  last_month_download = 150000,
  license = "MIT",
  reverse_dependencies_count = 10,
  documentation_score = 2,
  has_examples = list(
    data = data.frame(
      function_name          = "somefunction",
      documentation_name     = "No documentation found",
      documentation_location = 0,
      example                = "no Rd file",
      stringsAsFactors       = FALSE
    ),
    example_score = 0.20   
  ),
  has_docs = list(
    data = data.frame(
      function_name    = "somefunction",
      stringsAsFactors = FALSE
    ),
    has_docs_score = 0.30  
  ),
  cmd_check = 0
)

get_risk_analysis(mock_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R
\name{fetch_bioconductor_package_info}
\alias{fetch_bioconductor_package_info}
\title{Fetch Bioconductor Package Information}
\usage{
fetch_bioconductor_package_info(bioconductor_version, package_name)
}
\arguments{
\item{bioconductor_version}{A character string specifying the Bioconductor version (e.g., "3.14").}

\item{package_name}{A character string specifying the name of the package.}
}
\value{
A list containing package details, including the latest version, package URL,
        source package link, and any archived versions if available.
        Returns `FALSE` if the package does not exist or cannot be retrieved.
}
\description{
This function retrieves information about a specific Bioconductor package
for a given Bioconductor version. It fetches the package details, such as
version, source package URL, and archived versions if available.
}
\examples{
\dontrun{
info <- fetch_bioconductor_package_info("3.14", "GenomicRanges")
print(info)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{print_tree}
\alias{print_tree}
\title{Print a Package Dependency Tree}
\usage{
print_tree(tree, prefix = "", last = TRUE, show_version = TRUE)
}
\arguments{
\item{tree}{A nested list representing the package dependency tree, where each package has a `version` field and potentially sub-dependencies.}

\item{prefix}{A character string used for formatting tree branches (default: "").}

\item{last}{A logical value indicating if the current package is the last in the tree level (default: `TRUE`).}

\item{show_version}{A logical value indicating whether to display package versions if available (default: `TRUE`).}
}
\value{
Prints the dependency tree to the console.
}
\description{
This function prints a hierarchical representation of a package's dependencies, including version and base status
}
\examples{
\dontrun{
db <- list(
  stringr = list(
    version = "1.5.1",
    cli = list(version = "3.6.2", utils = "base"),
    glue = list(version = "1.7.0", methods = "base"),
    lifecycle = list(
      version = "1.0.4",
      cli = list(version = "3.6.2", utils = "base"),
      glue = list(version = "1.7.0", methods = "base"),
      rlang = list(version = "1.1.3", utils = "base")
    ),
    magrittr = list(version = "2.0.3"),
    rlang = list(version = "1.1.3", utils = "base"),
    stringi = list(version = "1.8.3", tools = "base", utils = "base", stats = "base"),
    vctrs = list(
      version = "0.6.5",
      cli = list(version = "3.6.2", utils = "base"),
      glue = list(version = "1.7.0", methods = "base"),
      lifecycle = list(version = "1.0.4"),
      rlang = list(version = "1.1.3", utils = "base")
    )
  )
)
print_tree(db)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{extract_package_version}
\alias{extract_package_version}
\title{Extract the Installed Version of a Package}
\usage{
extract_package_version(package_name)
}
\arguments{
\item{package_name}{A character string specifying the name of the package.}
}
\value{
A character string representing the installed package version, or `NA` if the package is not found.
}
\description{
This function retrieves the installed version of a specified R package.
If the package is not installed, returns `NA`.
}
\examples{
\dontrun{
extract_package_version("dplyr")
# 1.1.4
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_github_data.R
\name{get_commits_since}
\alias{get_commits_since}
\title{Retrieve GitHub Commits as Weekly Counts (using curl)}
\usage{
get_commits_since(owner, repo, years = 1)
}
\arguments{
\item{owner}{Character. The GitHub username or organization that owns the repository.}

\item{repo}{Character. The name of the GitHub repository.}

\item{years}{Numeric. Number of years to look back from today's date (default is 1).}
}
\value{
A data frame with four columns:
\describe{
  \item{week_start}{Start date of the week (class \code{Date}).}
  \item{year}{Year of the commits.}
  \item{month}{Month of the commits.}
  \item{n_commits}{Number of commits during the week.}
}
}
\description{
This function fetches all commits from a specified GitHub repository
using the GitHub API and returns a data frame with weekly commit counts.
}
\examples{
\dontrun{
# Get commit counts for the past year
get_commits_since(owner = "tidyverse", repo = "ggplot2")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{as_iso_date}
\alias{as_iso_date}
\title{Convert input to ISO 8601 date (YYYY-MM-DD)}
\usage{
as_iso_date(x)
}
\arguments{
\item{x}{A vector coercible to `Date` (e.g., character, `Date`, or `POSIXct`).
If `NULL`, the function returns `NULL`.}
}
\value{
A character vector the same length as `x` (or `NULL` if `x` is `NULL`)
with dates formatted as `"YYYY-MM-DD"`. Unparseable elements or `NA`s
become `NA_character_`.
}
\description{
Formats inputs as ISO 8601 calendar dates (`"YYYY-MM-DD"`).
}
\details{
Coercion uses [base::as.Date()], so time-of-day and timezone information are
dropped. For `POSIXct` inputs, the date is computed by `as.Date.POSIXct`
(which uses `tz = "UTC"` by default). Be aware that ambiguous character
dates (e.g., `"01/02/2024"`) depend on your R locale/format unless you
specify a format before calling this function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{remove_base_packages}
\alias{remove_base_packages}
\title{Helper function to remove base and recommended packages}
\usage{
remove_base_packages(df)
}
\arguments{
\item{df}{Data frame of dependencies of a package.}
}
\description{
Helper function to remove base and recommended packages
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_tests_stf.R
\name{nest_expect}
\alias{nest_expect}
\title{Extract and structure expectations from test blocks}
\usage{
nest_expect(x)
}
\arguments{
\item{x}{list of parsed test blocks}
}
\value{
data.frame containing structured expectations with test labels
}
\description{
Internal helper function to extract expectations from a list of test blocks.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_tests_stf.R
\name{map_test}
\alias{map_test}
\title{Map the hierarchy structure of a testthat test file}
\usage{
map_test(path)
}
\arguments{
\item{path}{character, path to file}
}
\value{
data.frame
}
\description{
Query a test file to see what expectations are defined in it.
}
\details{
Return data.frame containing which expectations are in the test file
 by context/description/test/expectation/linerange
}
\seealso{
\code{\link[utils]{getParseData}}

Other utility: 
\code{\link{create_coverage_skip_stf}()},
\code{\link{map_tests_stf}()}
}
\concept{utility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{get_rd_sections}
\alias{get_rd_sections}
\title{Get all Rd sections by tag}
\usage{
get_rd_sections(rd, section)
}
\arguments{
\item{rd}{An Rd object (a nested list with "Rd_tag" attributes on nodes).}

\item{section}{Section name without leading backslash (e.g., "alias", "examples").}
}
\value{
A list of Rd nodes matching the section (possibly empty).
}
\description{
Recursively collects all nodes whose `Rd_tag` matches a given section
name, e.g., "alias", "examples", "example", "name".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_size_codebase}
\alias{assess_size_codebase}
\title{assess codebase size}
\usage{
assess_size_codebase(pkg_source_path)
}
\arguments{
\item{pkg_source_path}{- source path for install local}
}
\value{
- size_codebase - numeric value between \code{0} (for small codebase) and \code{1} (for large codebase)
}
\description{
Scores packages based on its codebase size, 
as determined by number of lines of code.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_examples}
\alias{assess_examples}
\title{Assess Rd files for examples (exported functions only)}
\usage{
assess_examples(pkg_name, pkg_source_path)
}
\arguments{
\item{pkg_name}{Character scalar; package name (must be loadable so exports can be read).}

\item{pkg_source_path}{Directory path where Rd files can be found by `tools::Rd_db()`.}
}
\value{
A list with:
\describe{
  \item{data}{A data.frame with columns `function_name`, `documentation_name`,
              `documentation_location`, and `example`.}
  \item{example_score}{Numeric; percentage of exported functions with examples.}
}
}
\description{
Scans the package Rd database at `pkg_source_path` and reports whether each
**exported function** has examples. Resolution supports shared topics via
roxygen `@rdname` (multiple functions documented on the same Rd page) and
multiple `\\alias{}` entries.
}
\examples{
\dontrun{
  res <- assess_examples("stringr", "<path-to-stringr-source>")
  head(res$data)
  res$example_score
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{clean_up_dependencies}
\alias{clean_up_dependencies}
\title{Clean Up Dependencies}
\usage{
clean_up_dependencies(x)
}
\arguments{
\item{x}{A character vector of dependency strings.}
}
\value{
A character vector of unique package names.
}
\description{
Cleans up a list of dependency strings by extracting unique package names.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_description_file_elements}
\alias{assess_description_file_elements}
\title{assess_description_file_elements}
\usage{
assess_description_file_elements(pkg_name, pkg_source_path)
}
\arguments{
\item{pkg_name}{- name of the package}

\item{pkg_source_path}{- pkg_source_path - source path for install local}
}
\value{
- list - list with scores for description file elements
}
\description{
assess_description_file_elements
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependency_tree.R
\name{extract_license_from_description}
\alias{extract_license_from_description}
\title{Extract License from Package DESCRIPTION File}
\usage{
extract_license_from_description(path)
}
\arguments{
\item{path}{A character string specifying the path to the package directory containing the DESCRIPTION file.}
}
\value{
A character string representing the package license, or `NA` if not found.
}
\description{
This function extracts the license information from a package's DESCRIPTION file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{get_dependencies}
\alias{get_dependencies}
\title{Get dependencies}
\usage{
get_dependencies(pkg_source_path)
}
\arguments{
\item{pkg_source_path}{package source path}
}
\description{
Get dependencies
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{preprocess_func_full_name}
\alias{preprocess_func_full_name}
\title{function to preprocess func_full_name}
\usage{
preprocess_func_full_name(func_full_name)
}
\arguments{
\item{func_full_name}{- full name of the function}
}
\description{
function to extract package, class, and generic
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{cran_packages}
\alias{cran_packages}
\title{Retrieve the List of CRAN Packages (Internal)}
\usage{
cran_packages(cache = TRUE)
}
\arguments{
\item{cache}{Logical. If \code{TRUE} (default), the result is cached using \code{memoise::memoise()} 
with a timeout of 30 minutes to avoid repeated downloads. If \code{FALSE}, the metadata is fetched 
directly from CRAN each time the function is called.}
}
\value{
A matrix of package metadata similar to the output of \code{utils::available.packages()},
with package names as row names.
}
\description{
Downloads and caches the current list of packages available on CRAN.
}
\details{
The function downloads the \code{packages.rds} file from the CRAN website,
which contains metadata about all available packages. The result is cached using
\code{memoise::memoise()} with a timeout of 30 minutes.
}
\examples{
\donttest{
pkgs <- cran_packages()
head(rownames(pkgs))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{get_r6_methods_details}
\alias{get_r6_methods_details}
\title{function to get R6 methods}
\usage{
get_r6_methods_details(class_name)
}
\arguments{
\item{class_name}{- function class}
}
\description{
function to get R6 methods from class
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_summary_report.R
\name{generate_risk_analysis}
\alias{generate_risk_analysis}
\title{Generate Risk Analysis}
\usage{
generate_risk_analysis(assessment_results)
}
\arguments{
\item{assessment_results}{- input data containing risk analysis metrics}
}
\description{
Generates the Risk Analysis table for the HTML report.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup_rcmdcheck_args.R
\name{setup_rcmdcheck_args}
\alias{setup_rcmdcheck_args}
\title{set up rcmdcheck arguments}
\usage{
setup_rcmdcheck_args(check_type = "1", build_vignettes)
}
\arguments{
\item{check_type}{basic R CMD check type - "1" CRAN R CMD check_type - "2"}

\item{build_vignettes}{Logical (T/F). Whether or not to build vignettes}
}
\value{
- list with rcmdcheck arguments
}
\description{
This sets up rcmdcheck arguments
}
\details{
{Some packages need to have build vignettes as a 
build argument as their vignettes structure is inst/doc or inst/docs}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R
\name{fetch_bioconductor_releases}
\alias{fetch_bioconductor_releases}
\title{Fetch Bioconductor Release Announcements}
\usage{
fetch_bioconductor_releases()
}
\value{
An XML document from bioconductor version page.
}
\description{
This function retrieves the Bioconductor release announcements page and returns
its HTML content for further processing.
}
\examples{
\dontrun{
html_content <- fetch_bioconductor_releases()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_result_path}
\alias{get_result_path}
\title{Assign output file path for various outputs during scorecard rendering}
\usage{
get_result_path(
  out_dir,
  ext = c("check.rds", "covr.rds", "tm_doc.rds", "tm_doc.xlsx")
)
}
\arguments{
\item{out_dir}{output directory for saving results}

\item{ext}{file name and extension}
}
\description{
Assign output file path for various outputs during scorecard rendering
}
\details{
The basename of `out_dir` should be the package name and version pasted together
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capture_cran_warning.R
\name{capture_cran_warning}
\alias{capture_cran_warning}
\title{Capture CRAN URL warnings and errors}
\usage{
capture_cran_warning(cran_url, path)
}
\arguments{
\item{cran_url}{Character. Base URL of the CRAN mirror.}

\item{path}{Character. Path to the resource on the CRAN server.}
}
\value{
A list with two elements:
\describe{
  \item{status}{Character string indicating the result type: "success", "url_error", "timeout", "error", "url_warning", "timeout_warning", or "warning".}
  \item{message}{Character string with the captured warning or error message, or NULL if successful.}
}
}
\description{
This internal function attempts to open a CRAN URL and read its contents.
It captures and classifies warnings and errors related to network issues,
such as timeouts or inaccessible URLs.
}
\examples{
\dontrun{
  capture_cran_warning("http://cran.us.r-project.org", "src/contrib/Meta/archive.rds")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_session_dependencies.R
\name{get_session_dependencies}
\alias{get_session_dependencies}
\title{Get Dependencies}
\usage{
get_session_dependencies(deps_list)
}
\arguments{
\item{deps_list}{A data frame containing the dependency information of the package (provided by calc_dependencies function)}
}
\value{
A list with two elements:
\item{imports}{A named list of packages in the "Imports" section along with their corresponding versions }
\item{suggests}{A named list of packages in the "Suggests" section along with their corresponding versions }
}
\description{
This function extracts the version information of imported and suggested packages for a given package from the current R session.
}
\examples{
\donttest{
deps_list <- data.frame(
  package = c("dplyr", "ggplot2", "testthat", "knitr"),
  type = c("Imports", "Imports", "Suggests", "Suggests")
)
get_session_dependencies(deps_list)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_tests_long_sum_stf.R
\name{get_tests_long_sum_stf}
\alias{get_tests_long_sum_stf}
\title{Summarize detailed testthat results}
\usage{
get_tests_long_sum_stf(x)
}
\arguments{
\item{x}{list of test result objects}
}
\value{
data.frame with detailed test results including status and source line links
}
\description{
Internal helper function to generate a long-format summary of testthat results, including status and line references.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_package_local.R
\name{install_package_local}
\alias{install_package_local}
\title{Install package locally}
\usage{
install_package_local(pkg_source_path)
}
\arguments{
\item{pkg_source_path}{- source path for install local}
}
\value{
logical - package_installed
}
\description{
Install package locally
}
\examples{
\dontrun{
results <- install_package_local("pkg_source_path")
print(results)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exports.R
\name{extract_package_name}
\alias{extract_package_name}
\title{Extract package name from package source path}
\usage{
extract_package_name(pkg_source_path)
}
\arguments{
\item{pkg_source_path}{a file path pointing to an unpacked/untarred package directory}
}
\value{
package_name
}
\description{
Extract package name from package source path
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{create_has_ex_docs_score}
\alias{create_has_ex_docs_score}
\title{Combine example/docs scores into a single metric (sum).}
\usage{
create_has_ex_docs_score(example_score, has_docs_score)
}
\arguments{
\item{example_score}{Numeric (scalar or vector) in [0, 1] or NA.}

\item{has_docs_score}{Numeric (scalar or vector) in [0, 1] or NA.}
}
\value{
Numeric vector: example_score + has_docs_score in [0, 2],
  with NA where both inputs are NA at that position.
}
\description{
Combine example/docs scores into a single metric (sum).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dependencies.R
\name{dependsOnPkgs}
\alias{dependsOnPkgs}
\title{Determine Packages that Depend on Given Packages}
\usage{
dependsOnPkgs(
  pkgs,
  dependencies = "most",
  recursive = TRUE,
  lib.loc = NULL,
  installed = NULL
)
}
\arguments{
\item{pkgs}{A character vector of package names to check dependencies for.}

\item{dependencies}{A character string specifying the types of dependencies
to consider. Can be "strong", "most", "all", or a custom vector of dependency types.}

\item{recursive}{A logical value indicating whether to recursively check dependencies.}

\item{lib.loc}{A character vector of library locations to search for installed packages.}

\item{installed}{A matrix of installed packages, obtained from cran_packages function.
.}
}
\value{
A character vector of package names that depend on the specified packages.
}
\description{
This function identifies packages that depend on the specified packages,
considering various types of dependencies (e.g., strong, most, all).
}
\examples{
\dontrun{
installed <- cran_packages()
dependsOnPkgs("here", installed = installed)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{find_rd_for_fun}
\alias{find_rd_for_fun}
\title{Find an Rd document for a function using an index}
\usage{
find_rd_for_fun(fun, db, idx = NULL)
}
\arguments{
\item{fun}{Function name (character scalar).}

\item{db}{An Rd database list as returned by `tools::Rd_db()`.}

\item{idx}{Optional index from `build_rd_index()`. If `NULL`, it is built on demand.}
}
\value{
`NULL` if not found, otherwise a list with `rd` (the Rd object) and `filename` (the Rd filename).
}
\description{
Tries `fun.Rd`, then alias-based resolution via the index, and as a last
resort matches the topic name if it equals the function name.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_download.R
\name{get_cran_total_downloads}
\alias{get_cran_total_downloads}
\title{Get CRAN Total or Recent Downloads for a Package}
\usage{
get_cran_total_downloads(pkg, months = NULL)
}
\arguments{
\item{pkg}{Character string, the name of the CRAN package.}

\item{months}{Integer. If NULL, returns all-time total; otherwise, returns total from the last `months` full months (excluding current).}
}
\value{
An integer: total number of downloads.
}
\description{
Retrieves either all-time total downloads or the total from the last `n` full months
for a given CRAN package.
}
\details{
Similar data can be obtained via:

https://cranlogs.r-pkg.org/badges/grand-total/ggplot2
https://cranlogs.r-pkg.org/badges/last-month/ggplot2
}
\examples{
\dontrun{
  get_cran_total_downloads("ggplot2")        # all-time total
  # 160776616

  get_cran_total_downloads("ggplot2", 3)     # last 3 full months
  # 4741707

  get_cran_total_downloads("ggplot2", 12)    # last 12 full months
  # 19878885
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_host_info.R
\name{get_cran_package_url}
\alias{get_cran_package_url}
\title{Get CRAN Package URL}
\usage{
get_cran_package_url(package_name, version, last_version, all_versions)
}
\arguments{
\item{package_name}{A character string specifying the name of the package.}

\item{version}{An optional character string specifying the version of the package.
If `NULL`, the latest version is assumed.}

\item{last_version}{A named list with elements version and date, 
representing the latest available version and its publication date.}

\item{all_versions}{A list of named lists, each with elements version and date, 
representing all known versions of the package and their publication dates.}
}
\value{
A character string containing the URL to download the package tarball,
or "No valid URL found" if the version is not found in the list of available versions.
}
\description{
This function constructs the CRAN package URL for a specified package and version.
}
\examples{
\dontrun{
# Example data structure
last_version <- list(version = "1.0.10", date = "2024-01-01")
all_versions <- list(
  list(version = "1.0.0", date = "2023-01-01"),
  list(version = "1.0.10", date = "2024-01-01")
)

# Get the URL for the latest version of "dplyr"
url <- get_cran_package_url("dplyr", NULL, last_version, all_versions)
print(url)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bioconductor_data.R
\name{parse_bioconductor_releases}
\alias{parse_bioconductor_releases}
\title{Parse Bioconductor Release Announcements}
\usage{
parse_bioconductor_releases(html_content)
}
\arguments{
\item{html_content}{The parsed HTML document from `fetch_bioconductor_releases`.}
}
\value{
A list of lists containing Bioconductor release details: release version, date,
        number of software packages, and corresponding R version.
}
\description{
This function extracts Bioconductor release details such as version number,
release date, number of software packages, and required R version from the 
release announcements HTML page.
}
\examples{
\dontrun{
html_content <- fetch_bioconductor_releases()
release_data <- parse_bioconductor_releases(html_content)
print(release_data)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_github_data.R
\name{count_commits_last_months}
\alias{count_commits_last_months}
\title{Count Commits in the Last Months}
\usage{
count_commits_last_months(df, months = 1, today = Sys.Date())
}
\arguments{
\item{df}{A data frame with `week_start` (Date) and `n_commits` (int) columns.}

\item{months}{Integer. Number of months to look back (default is 1).}

\item{today}{today's date.default is Sys.Date()}
}
\value{
Integer. Total number of commits in the time window.
}
\description{
Counts the total number of commits from the last `months` months using a 
data frame with weekly commit data (column `week_start`).
}
\examples{
\dontrun{
df <- get_commits_since("tidyverse", "ggplot2", years = 1)
count_commits_last_months(df, months = 3)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_suggested_exp_funcs.R
\name{process_items_matched}
\alias{process_items_matched}
\title{process items matched}
\usage{
process_items_matched(items_matched, suggested_exp_func)
}
\arguments{
\item{items_matched}{- exported functions match the suggested package function}

\item{suggested_exp_func}{- exported functions from a suggested package}
}
\description{
create a df with functions that match exported functions from
a suggested package
}
\details{
This function extracts all the functions from the function body 
and filters them to keep only function calls
It also extracts all the valid function names, matches with the source package
and writes a message
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contains_vignette_folder.R
\name{contains_vignette_folder}
\alias{contains_vignette_folder}
\title{Check for Vignette Folder and .Rmd Files in a .tar File}
\usage{
contains_vignette_folder(tar_file)
}
\arguments{
\item{tar_file}{A character string specifying the path to the .tar file to be checked.}
}
\value{
A logical value: \code{TRUE} if the 'vignettes' folder exists and contains at least one .Rmd file,
and neither 'vignettes' nor 'inst/doc' folders are present, \code{FALSE} otherwise.
}
\description{
This function checks if a given .tar file contains a 'vignettes' folder
and if there is at least one .Rmd file within that folder. If both 'vignettes'
and 'inst/doc' folders exist, the function will return \code{FALSE}.
}
\details{
The function checks if the specified file exists and has a valid .tar extension using \code{utils::untar}. If the file is empty or
any error occurs during the extraction, the function stops and returns an error message. If both 'vignettes'
and 'inst/doc' folders exist, the function returns \code{FALSE}. If the 'vignettes' folder exists and contains 
at least one .Rmd file, the function returns \code{TRUE}. Otherwise, it returns \code{FALSE}.
}
\examples{
\dontrun{
  tar_file <- system.file("test-data", "here-1.0.1.tar.gz", 
   package = "risk.assessr")
  result <- contains_vignette_folder(tar_file)
  print(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_metrics.R
\name{check_forbidden_notes}
\alias{check_forbidden_notes}
\title{Reclassify Forbidden Notes as Errors in rcmdcheck Results}
\usage{
check_forbidden_notes(res_check, pkg_name)
}
\arguments{
\item{res_check}{A list representing the result of an `rcmdcheck` run. It should contain
at least the elements `notes`, `warnings`, and `errors`, each being a character vector.}

\item{pkg_name}{name of the package}
}
\value{
A modified version of `res_check` where certain notes matching forbidden patterns
  are moved to the `errors` field.
}
\description{
This internal helper function scans the `notes` field of an `rcmdcheck` result object
for specific patterns that indicate more serious issues. If any of these patterns are found,
the corresponding notes are reclassified as errors by moving them from the `notes` field
to the `errors` field.
}
\examples{
\donttest{
pkg_name <- "foo"
res_check <- list(
  notes = c("'foo' import not declared from: 'bar'",
            "Namespace in Imports field not imported from: 'baz'",
            "no visible global function definition for 'qux'",
            "some harmless note"),
  warnings = character(0),
  errors = character(0)
)
updated <- check_forbidden_notes(res_check, pkg_name)
print(updated$errors)  # Should include the first three notes
print(updated$notes)   # Should include only the harmless note
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_summary_report.R
\name{evaluate_risk_highlighting}
\alias{evaluate_risk_highlighting}
\title{Evaluate Risk Highlighting for Rejection and Mitigation}
\usage{
evaluate_risk_highlighting(risk_analysis_output)
}
\arguments{
\item{risk_analysis_output}{A data frame containing at least two columns:
\describe{
  \item{Metric}{Character vector of metric names (e.g., "CMD Check", "Code Coverage").}
  \item{Risk_Level}{Character vector of risk levels (e.g., "low", "medium", "high").}
}}
}
\value{
status - 3 possible values - "Accepted", "Mitigation Needed", "Rejected"
}
\description{
This function analyzes risk levels for key metrics ("CMD Check" and "Code Coverage")
and other metrics to determine whether a report should be highlighted for rejection
or mitigation based on predefined rules.
}
\details{
**Rejection Logic:**
- Reject if BOTH key metrics ("CMD Check" and "Code Coverage") are medium/high.
- OR if there are severe other risks (greater equal to 2 high OR greater equal to 3 medium)
- AND at least one key metric is not low.

**Mitigation Logic:**
- Mitigate if only one key metric is medium/high OR other risks are significant
  (greater equal to 1 high OR greater equal to 2 medium), provided the report is not rejected.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_metrics.R
\name{assess_exported_functions_docs}
\alias{assess_exported_functions_docs}
\title{Assess documentation coverage for exported functions}
\usage{
assess_exported_functions_docs(pkg_name, pkg_source_path)
}
\arguments{
\item{pkg_name}{Character scalar; package name (must be loadable so exports can be read).}

\item{pkg_source_path}{Directory path where Rd files can be found by `tools::Rd_db()`.}
}
\value{
A list with:
\describe{
  \item{data}{A `data.frame` with columns:
    \describe{
      \item{function_name}{Exported function name.}
      \item{documentation_name}{The Rd topic name (`\\name{}`), or `"No documentation found"`.}
      \item{documentation_location}{`man/<file>.Rd` if found, otherwise `NA_character_`.}
    }
  }
  \item{documentation_score}{Numeric; percentage of exported functions with an Rd page.}
}
}
\description{
Resolves each **exported function** in a package to its Rd page and reports:
the function name, the Rd topic name (roxygen `@rdname`), and the Rd location.
Resolution uses the same alias/topic index as `assess_examples()`, so it
correctly handles functions documented under a shared topic (via `@rdname`)
and multiple `\\alias{}` entries (e.g., `stringr::str_to_upper` under
`man/case.Rd`).
}
\keyword{internal}
